c-----------------------------------------------------------------------
c                  VENUS 5.21 / 01. Mar. 1996 22:35
c-----------------------------------------------------------------------
c     
c
c  Authors: Klaus WERNER
c           Eugen FURLER (upgrade 5.04)
c           Michael HLADIK (upgrades 5.05, 5.13, 5.15, 5.17)
c
c  Laboratoire Subatech, Universite de Nantes - IN2P3/CNRS - EMN
c  4 rue Alfred Kastler, 44070 Nantes Cedex 03, France
c
c  Email: <last name>@nanhp3.in2p3.fr
c
c  Before running VENUS, it is advised to read the following documentation, 
c  which contains in particular many examples of how to use the code.
c
c
c  What is VENUS?        
c  --------------
c
c  VENUS is a Monte Carlo procedure to simulate hadronic interactions at
c  ultrarelativistic energies (hadron-hadron, hadron-nucleus, nucleus-
c  nucleus scattering).
c
c  VENUS is based on Gribov-Regge theory (of multiple Pomeron exchange)
c  and classical relativistic string dynamics. A detailed description can 
c  be found in a review article, published in Physics Reports 232 (1993)  
c  pp. 87-299.
c
c  New features (since version 4.12):
c
c  (1) New input/output format.
c
c  (2) Documented analysis facilities.
c
c  (3) Consideration of extended quark matter droplets
c      n-body decay of droplets 
c
c  (4) New parton distribution funcions (GRV and MRS) available
c
c
c  Copyright Notice
c  ----------------
c
c  VENUS -- a program to simulate Very Energetic NUclear Scattering.
c  Copyright K. Werner, Nantes 1995.
c  Copyright reserved in all countries of the world.
c  The program may not be reproduced by any method without prior 
c  consent of the author (K. Werner). 
c
c
c  Important Features of VENUS
c  ---------------------------
c
c  Sophisticated fragmentation procedure (area law method).
c
c  Very large resonance table, including for example all nucleon
c  resonances up to 2 gev. the decay table includes also kshort
c  and lambda decays. 
c
c  Hadronic interactions strictly based on gribov-regge theory.
c  
c  Participation of antiquarks (in addition to quarks) in the
c  colour exchange mechanism to form strings.
c
c  Possibility of diquark breakup, leading to multi-strings, like
c  a forward quark linked via two (!) strings to two backward
c  quarks (double-string). 
c
c  Covariant treatment of secondary interactions.
c
c  Consideration of quark matter droplets.
c
c
c  Running VENUS
c  -------------
c
c  Input is requested from `standard input'.
c
c  When VENUS is executed, first of all a header is printed, having the 
c  form
c
c    ###################################################################
c    #                VENUS N.NN       Klaus WERNER                    #
c    ###################################################################
c    ?
c
c  in case of a preliminary version also a warning may be printed, which 
c  should be taken seriously.
c
c  The question mark '?' indicates that the program is expecting input.
c
c  The program may run in different modes for different applications.
c  So the first and most important input amounts to specifying the mode.
c  In the following, a brief discussion of the different modes is provided.
c
c  The 'hadron' mode is the most important one. This mode has to be 
c                    selected in order to simulate hadronic interactions,
c                    i.e. hadron-hadron, hadron-nucleus, nucleus-nucleus 
c                    scattering.
c
c  The 'geometry' mode serves also to simulate hadronic interactions, 
c                    however, the events are not fully simulated, only
c                    the geometry aspects are considered.
c
c  The 'lepton' mode is needed in order to simulate e+e- annihilation,
c                    lepton nucleon, or lepton nucleus scattering.
c
c  The 'droplet' mode allows to simulate the phase space decay of 
c                    individual quark matter droplets.
c
c  The 'analysis' mode provides a tool to analyse the simulated data,
c                    i.e. one may with little effort obtain histograms
c                    with respect to different variables.
c
c  To specify the mode, the following input command is required:
c
c     application <mode>
c
c  where <mode> is any of the following options:
c
c         hadron, geometry, lepton, droplet, analysis
c
c  corresponding obviously to the above five different modes.
c
c  Example:
c
c     ?
c     application hadron
c
c  this is needed in order to simulate hadronic interactions (including 
c  nucleus-nucleus).
c
c  After having chosen the mode, one needs to specify the system to be
c  simulated, to choose certain options, and perhaps to modify parameters.
c  This can be achieved by changing certain parameter settings, using the
c  'set' command. The general form is:
c
c     set <param> <pvalue>
c
c  where <param> may be (almost) any of the parameters set in subr. aaset, 
c  and where <pvalue> represents the corresponding value.
c  The set command may be used an arbitrary number of times.
c
c  Examples:
c
c  in order to simulate 5000 events:
c
c     ?
c     set nevent 5000     
c
c  to define an incident energy of 200 GeV:
c
c     ?
c     set pnll 200       
c
c  to define the projectile to be a sulfur nucleus:
c
c     ?
c     set laproj 16      
c     set maproj 32      
c
c  to define the target to be a gold nucleus:
c
c     ?
c     set latarg 79     
c     set matarg 197     
c
c  to change the parameter 'ptf' (average transverse momentum for string
c  breakup) to 0.450:
c
c     ?
c     set ptf 0.450    
c
c  The order of the 'set' commands is completely arbitrary, if the same
c  parameter is set several times, the last setting counts.
c
c  In order to get a complete printout of all parameter settings, one may 
c  use the show command,
c
c     ?
c     show
c
c  or, for a somewhat reduced output,
c
c     ?
c     showa
c
c  The action of the command 'showa' depends on the chosen mode, it 
c  provides a printout of the most relevant parameter settings for the 
c  chosen mode.
c
c  After having set the parameters, in particular having chosen the system
c  which has to simulated, one may start the simulation via
c
c     ?
c     run
c
c  The program performs the simulation as requested, when the task is 
c  performed, the prompt '?' will appear again.
c  One may start another run starting with 'application <mode>' etc,
c  or one may stop the execution via
c
c     ?
c     stop
c
c
c  Input from a File
c  -----------------
c
c  Usually it is more convenient to have the sequence of input commands
c  (as discussed above) in a file rather that typing them in interactively.
c  This can clearly be done using redirected input (see below: running
c  VENUS under UNIX). We call a file containing a sequence of VENUS
c  input commands `optns-file' and propose the convention to name these
c  files <name>.optns, with same arbitrary <name>.
c
c  Two things need to be discussed, when input is taken from a file. First,
c  there is always a prompt `?' printed to indicate that input is expected.
c  This is not necessary in case of input from a file, even more, it's
c  kind of a nuisance. So one should first of all turn off the prompt, via
c
c       prompt off
c
c  which should be the first command in the file.
c
c  The second important topic in connention with input from a file refers
c  to the `echo'. Using the switch
c     
c     echo on
c
c  anything which is typed in (or read from a file) is repeated. The switch
c
c     echo off
c
c  suppresses this option. Whereas for interactive input one does not want
c  the echo, it is quite useful in the case of input from a file.
c  So the second statement after the `prompt off' should be `echo on'.
c
c  An optns-file to simulate nucleus-nucleus scattering may thus have
c  the following structure (anything following `!' is considered  to be 
c  a comment and is ignored by the VENUS program):
c
c     prompt off          !turns off the prompt 
c     echo on             !turns on the echo 
c     application hadron  !indicates that hadron-hadron, hadron-nucleus, 
c                         !  or nucleus-nucleus is simulated
c     set nevent <nevent> !causes <nevent> events to be simulated
c     set pnll   <pnll>   !sets incident momentum (per nucleon)
c     set laproj <laproj> !sets projectile charge
c     set maproj <maproj> !sets projectile mass
c     set latarg <latarg> !sets target charge
c     set matarg <matarg> !sets target mass
c     run                 !performs the simulation
c     stop                !stops program
c
c  this is kind of a minimum input, further possibilities are discussed
c  below.
c
c
c  An Example Input File
c  ---------------------
c
c  In the following, we provide an explicit example optns-file, to simulate
c  5000 events of sulfur on gold at 200 GeV:
c
c     prompt off
c     echo on
c     !----------------------------- 
c     ! S + Au at 200 GeV, 5000 evts
c     !-----------------------------
c     application hadron  !hadronic interaction
c     set nevent 5000     !5000 events
c     set pnll 200        !200 gev
c     set laproj 16       !charge sulfur
c     set maproj 32       !mass sulfur
c     set latarg 79       !charge gold
c     set matarg 197      !mass gold
c     run                 
c     stop                
c  
c  It is not necessary to start a new line for each command, so one may write
c  instead of the above example:
c
c     prompt off echo on
c     application hadron  
c     set nevent 5000  set pnll 200
c     set laproj 16 set maproj 32 set latarg 79 set matarg 197
c     run stop                
c
c  The lengthy method, one line for each command, is, however, usually better 
c  readable and also allows to include a comment, so for the following
c  we prefer this method.
c
c
c  Controlling the Impact Parameter
c  --------------------------------
c
c  Per default, the impact parameter, (b,phi) in polar coordinates,
c  is chosen randomly. There is the option to chose b randomly within
c  a user defined range <b1> and <b2> via
c
c     set bminim <b1>  !length unit for <b1>: fm
c     set bmaxim <b2>  !length unit for <b2>: fm 
c
c  and correspondingly one may define the phi range <phi1> and <phi2> via 
c
c     set phimax <phi1>
c     set phimin <phi2>
c
c  in particular one may fix b by using the same value for <b1> and <b2>,
c  and correspondingly one may fix phi by using the same value for <phi1> 
c  and <phi2>.
c
c  The easiest way to run only central collisions amounts to selecting b only
c  between zero and some upper limit <b2> for bmaxim. Since bminim is set
c  to zero by default, one needs only to write
c
c     set bmaxim <b2>
c
c  with some small value for <b2>. the extreme case, central collisions with 
c  b=0, are obviously chosen with
c
c     set bmaxim 0
c
c  Example:
c
c     prompt off
c     echo on
c     !------------------------------------------------
c     ! central S + Au at 200 GeV, 5000 evts, phi fixed
c     !------------------------------------------------
c     application hadron  !hadronic interaction
c     set nevent 5000     !5000 events
c     set pnll 200        !200 gev
c     set laproj 16       !charge sulfur
c     set maproj 32       !mass sulfur
c     set latarg 79       !charge gold
c     set matarg 197      !mass gold
c     set bmaxim 1        !only central collisions with b < 1.0 fm
c     set phimin 1.5708   ! \  
c     set phimax 1.5708   !  \phi fixed to 1.5708
c     run
c     stop
c     
c
c  Arbitrary Hadrons as Projectile or Target
c  -----------------------------------------
c
c  Arbitrary hadrons (as projectile or target) may be considered via
c
c     set idproj <id> 
c
c  or
c
c     set idtarg <id>
c
c  where <id> is the particle id of the hadron (id-description later).
c  for meson-nucleon scattering the diffractive probability 'wproj'
c  has to be changed (and maybe also 'wtarg').
c  warning: meson-nucleon and (even more) meson-nucleus
c  scattering has been tested much less than purely nucleonic reactions,
c  so the parameters are not neccessarily optimized, and the author
c  would appreciate comments and suggestions!
c
c  Example: 
c
c     prompt off
c     echo on
c     !------------------------------------------------
c     ! pi+ + Pb at 200 GeV, 5000 evts, 'wproj' changed
c     !------------------------------------------------
c     application hadron  !hadronic interaction
c     set nevent 5000     !5000 events
c     set pnll 200        !200 gev
c     set idproj 120      !120 is the id for a pi+
c     set latarg 82       !charge lead
c     set matarg 207      !mass lead
c     set wproj 0.20      !wproj changed 
c     run
c     stop
c
c
c  Decay Suppression
c  -----------------
c
c  There are two ways to suppress certain decays: changing the variable 'ndecay'
c  via the 'set ndecay' command or using the 'nodecay' command.
c  the latter is more flexible and easier to explain and will be discussed
c  in the following. The syntax is
c
c     nodecay <id>
c
c  where <id> is the venus particle id code (see list below). 
c  the nodecay command may be used several times, to suppress several decays.
c
c  Example:
c
c     prompt off 
c     echo on
c     !----------------------------------------------------
c     !central s + au at 200 gev, 5000 evts 
c     !pi0 decay suppressed, kshort decay suppressed 
c     !lambda decay suppressed, antilambda decay suppressed
c     !---------------------------------------------------- 
c     application hadron  !hadronic interaction
c     set nevent 5000     !5000 events
c     set pnll 200        !200 gev
c     set laproj 16       !charge sulfur
c     set maproj 32       !mass sulfur
c     set latarg 79       !charge gold
c     set matarg 197      !mass gold
c     set bmaxim 1        !central (b<1fm)
c     nodecay 110         !pi0 decay suppressed
c     nodecay 20          !kshort decay suppressed 
c     nodecay 2130        !lambda decay suppressed
c     nodecay -2130       !antilambda decay suppressed
c     run
c     stop
c
c
c  The Command `modsho'
c  --------------------
c
c  Per default, after each event, a short message is printed (to standard
c  output). To modify this, one may change the `modsho' parameter:
c
c     set modsho <N>
c
c  with <N> being some integer. Now the printout occurs only every <N>th 
c  event.
c
c
c  The Command `setm'
c  ------------------
c
c  instead of 
c
c     set nevent <nevent> set modsho <modsho> set pnll <pnll> 
c     set laproj <laproj> set maproj <maproj> 
c     set latarg <latarg> set matarg <matarg> 
c
c  one may use the setm command
c
c     setm <nevent> <modsho> <pnll> <laproj> <maproj> <latarg> <matarg>  
c
c  as a `short cut'. So a complete input would then be:
c
c     prompt off
c     echo on
c     application hadron 
c     setm <nevent> <modsho> <pnll> <laproj> <maproj> <latarg> <matarg>  
c     run 
c     stop
c
c  Example:
c
c     prompt off
c     echo on
c     !---------------------------------------------------------    
c     ! S + Au at 200 GeV, 5000 events, message after 100 events
c     !---------------------------------------------------------    
c     application hadron
c     setm 5000 100 200.0 16 32 79 197    
c     run 
c     stop
c
c
c  The Command `input'
c  -------------------
c
c  Apart of redirected input, there is another possibility to read 
c  commands from a file, namely via
c     
c     input <filename>
c
c  with <filename> being the name of the input file.
c
c  Example:
c
c     input S+Au
c
c  where the file S+Au should contain commands to run venus.
c
c
c  Output
c  ------
c
c  Output is written to:
c
c  standard output  (most important messages
c                    and  a prompt in case of expected input)
c
c  zzz.check  (error messages, warnings, and, upon request, a more 
c              or less complete listing of intermediate results)
c
c  zzz.histo  (used for application analysis, see below)
c
c  zzz.data   (simulation data)
c
c  the default file names zzz.check, zzz.histo, zzz.data
c  may be changed via the commands
c
c     fname check <fncheck>
c     fname histo <fnhisto>
c     fname data  <fndata>
c
c  with <fncheck>, <fnhisto>, <fndata> being any valid file names.
c
c  Example:
c
c     prompt off
c     echo on
c     fname check S+Au.check       !error messages written to S+Au.check
c     fname histo S+Au.histo       !results from the analysis part written to 
c                                  !S+Au.histo 
c     fname data /d3/tmp/S+Au.data !simulation data (particles) written to
c                                  !/d3/tmp/S+Au.data
c     application hadron           !hadronic interaction
c     set nevent 5000              !5000 events
c     set laproj 16                !charge sulfur
c     set maproj 32                !mass sulfur
c     set latarg 79                !charge gold
c     set matarg 197               !mass gold
c     run
c     stop
c
c
c  Controlling Output of Simulation Data
c  -------------------------------------
c
c  As already discussed, the simulation data are written to the
c  data-file.
c
c  The general structure of the output is as follows: for each event,
c  there is one event record written, followed by one record per particle 
c  for each particle of the event.
c
c  There are several possibilities to control this output,
c  to be discussed in the following.
c
c  The parameter `istore', to be set via
c
c     set istore <istore>
c
c  allows to select among several possibilities of writing the event records
c  and the particle records.  
c
c  <istore> = 0: nothing is written.
c
c  <istore> = 1: an event record has the following structure:
c
c                nrevt,nptevt,bimevt,phievt,kolevt,colevt,
c                pmxevt,egyevt,npjevt,ntgevt
c
c                and a particle record:
c
c                i,idptl(i),pptl(1,i),pptl(2,i),pptl(3,i),pptl(4,i),pptl(5,i),
c                iorptl(i),jorptl(i),istptl(i),
c                xorptl(1,i),xorptl(2,i),xorptl(3,i),xorptl(4,i),
c                tivptl(1,i),tivptl(2,i)
c
c  <istore> = 2: an event record has the following structure:
c
c                nrevt,nptevt,bimevt,phievt
c                npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt 
c
c                and a particle record:
c
c                idptl(i),pptl(1,i),pptl(2,i),pptl(3,i),pptl(4,i),
c                xorptl(1,i),xorptl(2,i),xorptl(3,i),xorptl(4,i),
c                idiptl(i),idjptl(i)
c
c  with
c
c     nrevt.......... event number
c     nptevt ........ number of (stored!) particles per event
c     bimevt ........ absolute value of impact parameter
c     phievt ........ angle of impact parameter
c     kolevt ........ number of collisions
c     colevt ........ effective number of collisions
c     pmxevt ........ reference momentum
c     egyevt ........ pp cm energy (hadron) or string energy (lepton)
c     npjevt ........ number of primary projectile participants
c     ntgevt ........ number of primary target participants
c     npnevt ........ number of primary projectile neutron spectators
c     nppevt ........ number of primary projectile proton spectators
c     ntnevt ........ number of primary target neutron spectators
c     ntpevt ........ number of primary target proton spectators
c     jpnevt ........ number of absolute projectile neutron spectators
c     jppevt ........ number of absolute projectile proton spectators
c     jtnevt ........ number of absolute target neutron spectators
c     jtpevt ........ number of absolute target proton spectators
c
c  and
c
c     i ............. particle number
c     idptl(i) ...... particle id
c     pptl(1,i) ..... x-component of particle momentum 
c     pptl(2,i) ..... y-component of particle momentum 
c     pptl(3,i) ..... z-component of particle momentum 
c     pptl(4,i) ..... particle energy 
c     pptl(5,i) ..... particle mass 
c     iorptl(i) ..... particle number of father (if .le. 0 : no father) 
c     jorptl(i) ..... particle number of mother (if .le. 0 : no mother)
c     istptl(i) ..... generation flag: last gen. (0) or not (1) or ghost (2)
c     xorptl(1,i) ... x-component of formation point
c     xorptl(2,i) ... y-component of formation point
c     xorptl(3,i) ... z-component of formation point
c     xorptl(4,i) ... formation time
c     tivptl(1,i) ... formation time (always in the pp-cms!)
c     tivptl(2,i) ... destruction time (always in the pp-cms!)
c     idiptl(i) ..... id of father (999 if no father)
c     idjptl(i) ..... id of mother (999 if no mother)
c
c  The generation flag istptl(i) provides information about whether the
c  particle is last generation or not.
c  For istptl(i)=0, the particle is last generation, for istptl(i)=1 it is
c  not. There is another possibility, istptl(i)=2, but in this 
c  case the particle is a ghost, a particle which never existed, which has
c  been created for some technical reasons, and should never be considered
c  in the analysis.
c
c  The parameter `istmax' controls, which generations are written. As usual,
c  `istmax' is set via
c
c     set istmax <istmax>
c
c  with <istmax> being 0 or 1. For <istmax> = 0, only the last generation 
c  particles (with istptl(i)=0) are written, for <istmax> = 1 all generations
c  are written (i.e. particles with istptl(i)=0 or 1).
c
c  Obviously, <istmax>=1 provides a complete information about the event.
c  With iorptl(i) and jorptl(i) pointing back to the particle number of
c  father and mother, the event structure may be reconstructed.  Father 
c  and mother have the following meaning: a particle originating from a decay,
c  has only a father (the decaying object) and no mother, which means that
c  jorptl(i) is zero, and iorptl(i) is a positive integer, refering to the
c  particle number of the decaying object. In case of a particle (in a general
c  sense, including droplets) being produced via fusing two objects
c  (droplet formation procedure), this particle has father and mother, i.e.
c  iorptl(i) and jorptl(i) are both positive integers pointing to the 
c  particle numbers of the two fusing objects. In case of elasstic scattering, 
c  the two outgoing particles have both father and mother, namely the two 
c  incoming particles.
c
c  For <istmax>=0, only the last generation particles are stored. In this case,
c  the particle numbers of the parents, iorptl(i) and jorptl(i), provide a 
c  useless information, because the particles with these numbers are not stored.
c  For <istmax>=0 it is therefore more appropriate to use the option
c  <istore>=2, where i, iorptl(i) and jorptl(i) are not written (being anyhow
c  useless in case of <istmax>=0), BUT where instead the particle id codes
c  of the parents are written, idiptl(i) and idjptl(i), for the ids of father
c  and mother. If there is no father or mother, idiptl(i) or idjptl(i)
c  assume 999 (this really means there IS no father or mother, and not, that
c  father or mother are not stored!).
c
c  So when one only needs to know the id of the parents, and there is no need 
c  to reconstruct the whole event, it is advised to run VENUS with <istmax>=0
c  and <istore>=2, i.e. one should use the input commands
c
c     set istmax 0
c     set istore 2
c
c
c  How to Extract Information about Spectators
c  -------------------------------------------
c
c  Knowing the particle number `i' as well as the values of iorptl(i) and 
c  istptl(i), one can unambiguously determine the participant/spectator  
c  property of a particle.
c
c  One first needs to know that projectile and target nucleons have well 
c  defined positions in the particle list. Here, one finds first the projectile 
c  nucleons and then  the target nucleons. So, with <maproj> being the 
c  projectile mass number and <matarg> the target mass number, we have:
c
c     for i between 1 and <maproj> : projectile nucleons
c     for i between <maproj>+1 and  <maproj>+<matarg> : target nucleons
c  
c  So, in order to study spectators, it is enough to consider only the first
c  <maproj>+<matarg> particles in the list.
c
c  For these particles, iorptl(i) defines, whether a particle is a primary
c  participant (suffering a primary interaction = colour exchange) or
c  not. In the latter case, the particle is called a primary spectator. 
c  We have:
c
c     iorptl(i)=-1  <=>  particle is primary participant
c     iorptl(i)=0   <=>  particle is primary spectator
c
c  The primary spectators may undergo a secondary interaction, or not.
c  In the latter case, when the particles survives the whole collision
c  without any interaction, we call such a particle `absolute spectator'
c  or sometimes simply `spectator'. An absolute spectator can be 
c  characterized as projectile or target nucleon (thus with `i' less or equal
c  <maproj>+<matarg>) with iorptl(i)=0 (because it must be also a primary
c  spectator) and in addition with istptl(i)=0 (because it must be a last
c  generation particle). So we have:
c
c     iorptl(i)=0 and istptl(i)=0  <=>  particle is absolute spectator
c
c  To differentiate between spectator neutrons and protons, one simply needs 
c  to consider in addition the variable idptl(i). As can be seen from the
c  list of particle id codes below, we use
c
c     idptl(i) = 1120 <=> proton
c     idptl(i) = 1220 <=> neutron
c
c  A primary projectile neutron spectator is characterized by:
c
c     `i' between 1 and <maproj>
c     iorptl(i)=0, idptl(i)=1220
c
c  A primary target neutron spectator is characterized by:
c
c     `i' between <maproj>+1 and <maproj>+<matarg>
c     iorptl(i)=0, idptl(i)=1220
c
c  A absolute projectile neutron spectator is characterized by:
c
c     `i' between 1 and <maproj>
c     iorptl(i)=0, istptl(i)=0, idptl(i)=1220
c
c  A absolute target neutron spectator is characterized by:
c
c     `i' between <maproj>+1 and <maproj>+<matarg>
c     iorptl(i)=0, istptl(i)=0, idptl(i)=1220
c
c  For proton spectators one has idptl(i)=1120 instead of idptl(i)=1220
c
c  Based on these considerations, the numbers of primary/absolute 
c  neutron/proton projectile/target spectators have been calculated, the
c  corresponding variables are: 
c
c     npnevt ........ number of primary projectile neutron spectators
c     nppevt ........ number of primary projectile proton spectators
c     ntnevt ........ number of primary target neutron spectators
c     ntpevt ........ number of primary target proton spectators
c     jpnevt ........ number of absolute projectile neutron spectators
c     jppevt ........ number of absolute projectile proton spectators
c     jtnevt ........ number of absolute target neutron spectators
c     jtpevt ........ number of absolute target proton spectators
c
c  Using the option <istore>=2, the event record contains these variables.
c
c  The command
c
c     print afinal 6
c
c  provides for each event a printout of information about spectators
c  into the check-file.
c 
c
c  The Reference Frame
c  -------------------
c
c  The natural reference frame is the nucleon--nucleon centre-of-mass frame
c  (NN CM frame). All internal calculations are performed in this frame, and 
c  therefore any output into the check-file during the run refers to this
c  frame.
c
c  There is the possibilty to perform a Lorentz boost of the particle
c  momenta as well as the formation points into the lab frame.
c  This can be controlled via 
c
c     set labsys <labsys>
c
c  where <labsys> = 1 causes a transformation into the lab frame, whereas
c  for <labsys> = 0 no transformation is done; the final results in the
c  latter case refer to the NN CM frame.
c
c  The default setting corresponds to <labsys> = 1, so per default the final
c  results refer to the lab frame.
c
c 
c  A Complete Input for HBT Studies
c  --------------------------------
c
c  In the following, a complete input (optns-file) is provided, 
c  which should meet many of the needs to do HBT studies, as fixing the 
c  impact parameter angle, controlling decays, providing information about 
c  parents.
c
c     prompt off              !prompt turned off 
c     echo on                 !echo turned on
c     fname check Pb+Pb.check !error messages written to Pb+Pb.check
c     fname data Pb+Pb.data   !simulation data (particles) written to Pb+Pb.data
c     application hadron      !hadronic interaction
c     set nevent 5000         !5000 events
c     set laproj 82           !charge lead (projectile)
c     set maproj 207          !mass lead (projectile)
c     set latarg 82           !charge lead (target)
c     set matarg 207          !mass lead (target)
c     set bmaxim 1            !central (b<1fm)
c     set labsys 0            !nucleon-nucleon centre-of-mass frame
c     nodecay 110             !pi0 decay suppressed
c     nodecay 20              !kshort decay suppressed 
c     nodecay 2130            !lambda decay suppressed
c     nodecay -2130           !antilambda decay suppressed
c     set phimin 1.5708       ! \  
c     set phimax 1.5708       !  \phi fixed to 1.5708
c     set istmax 0            !only last generations particles stored
c     set istore 2            !defines option to write simulation data 
c     run
c     stop
c
c
c  Running VENUS under UNIX
c  ------------------------
c
c  Be 'venus.exe' the name of the venus executable, and be 'S+Au.optns',
c  'Pb+Pb.optns',... a couple of optns-files for different systems 
c  (and/or different parameter choices or whatsoever). One may now 
c  very conveniently use the following script (called for example venus):
c            
c      # unix script `venus' to run venus
c      venus.exe -n << END
c      prompt off
c      echo on
c      fname check $1.check
c      fname data  $1.data
c      input $1.optns
c      END
c      exit
c
c  Typing 
c
c     venus Pb+Pb
c
c  will then execute the venus script with the argument Pb+Pb. This means the
c  following:
c
c    * venus starts executing
c    * the prompt is turned off 
c           (this is useful because the input is read from files and there
c            is therefore no need for a prompt)
c    * the echo is turned on
c           (this is useful to control what is read in from input files,
c            the echo should only be turned off in case of interactive use)
c    * the check-file is defined as Pb+Pb.check
c    * the data-file is defined as Pb+Pb.data
c    * the commands from the file Pb+Pb.optns are read 
c
c  In this way one can run several jobs parallel, like for example
c
c     venus Pb+Pb; venus S+Au; ...
c
c  where the input is taken first from Pb+Pb.optns, then from  or S+Au.optns ...
c  and the output is correspondingly written first to Pb+Pb... , then to
c  S+Au... and so on.
c
c
c  The Command `print'
c  -------------------
c
c  During the execution of VENUS it is possible to print out information
c  (into the check-file). Each print statement has a level assigned, an 
c  integer beween 0 and 9. The higher the level, the more detailed is the
c  printed information.
c  
c  One may controll the printout by using the print command, via
c
c     print <subroutine> <level> 
c
c  where <subroutine> is a name of a VENUS subroutine and <level>
c  an integer between 0 and 9. This command causes printout of levels
c  0 to <level> to be active in subroutine <subroutine>. For <level>=1,
c  only very basic things like error messages and serious warnings are
c  printed, <level>=9 causes a complete printout, in the sense that the
c  program flow may be reconstructed. 
c  
c  The choice <subroutine> = * affects all subroutines, this means
c
c     print * <level>
c
c  activated printout of level <level> in all routines. Since <level>=0
c  means no printout, one may deactivate printout via
c
c     print * 0
c
c  The print command works only for selected subroutines, in case this
c  possibility is not forseen for a given subroutine, the effect of the
c  print command is zero, it is ignored.
c
c  Being newly installed, the print command works presently only for 
c  few subroutines: 
c
c     afinal, jfrade, racpro
c
c  For insiders: there is the following correspondence between <level>
c  and the `ish' parameter:
c
c      <level>:   0   1   2   3   4   5   6   7   8   9
c        ish:     0  90  91  92  93  94  95  96  97  98
c
c  Using the print command is, however, much more flexible than setting `ish'.
c
c  
c  Useful Print Requests
c  --------------------- 
c
c  In the following, we supply a couple of examples, providing useful
c  print requests:
c
c     print afinal 6  !prints info about spectators for each event
c
c     print jfrade 2  !prints listing of strings and particles, 
c                     !which is useful to reconstruct the event
c
c     print racpro 6  !prints pp cross sections and cut Pomeron weights 
c                     !calculated in subr racpro. 
c
c      
c  Analysis
c  -------- 
c
c  The analysis facilities in venus, although quite powerful and flexible,
c  are described only in a very schematic way, more meant to be a summary
c  for insiders. It is assumed that the 'average user' has his or her
c  own analysis programs, so there seems to be no need to make the
c  venus analysis facilities public.
c
c  `application analysis' starts the analysis facilities, 
c  which amounts to filling a histogram `y versus x' (for example
c  y = number of pions, x = rapidity). "x" and "y" are specified
c  via setting `xvaria' and `yvaria' (see table 1 in the beginning
c  of subr aread). the appropriate normalization is specified via 
c  setting `normal' (see table 2 in the beginning of subr aread).
c  xvaria, yvaria, normal (and more) may be set via setm:
c     setm <xvaria> <yvaria> <normal> <xminim> <xmaxim> <nrbins>
c  where <xvaria>, <yvaria>, and <normal> are the desired settings for
c  xvaria, yvaria, and normal, <xminim> and <xmaxim> specify the
c  desired x-range, and <nrbins> the number of bins.
c  trigger condition are specified via the trigger command:
c    trigger <trvari> <trmin> <trmax>
c  which forces variable <trvari> to  be considered only between
c  <trmin> and <trmax> (possible choices for <trvari> as for
c  `xvaria' or `yvaria').
c  particle id's are specified via tha idcode command:
c    idcode <id> 
c  with <id> being a venus id.
c  the command writearray writes the histogram to the histo-file.
c
c  Popular choices for <xvaria> are: 
c     numptl ... number of particles
c     numevt ... number of events
c  Popular choices for <yvaria> and <trvari>:
c     ycmptl ... rapidity (cms)
c     ylbptl ... rapidity (lab)
c     ptrptl ... transverse momentum
c     mulevt ... multiplicity 
c     kolevt ... number of collisions
c  Popular choices for <normal>:
c     0  ... no normalisation
c     1  ... / # of events
c     11 ... / # of events / binsize
c
c  Example:
c
c     prompt off
c     echo on
c     application hadron
c     ...
c     run
c     application analysis             
c     setm ycmptl numptl 11 -3 3 100   !to generate a rapidity distribution
c     trigger ptrptl 0.2 2.0           !only pt between .2 and 2 considered
c     idcode 120                       !pi+ considered
c     idcode -120                      !and pi- considered
c     run
c     write "rapidity ditribution:"    !writes text into histo-file
c     writearray                       !writes histogram into histo-file
c     stop
c
c
c  Notes
c  -----
c
c     *** the initial seed for the random number generator can be
c         changed via 'set seedi <seedi>' (seed at start)
c            or   via 'set seedj <seedj>' (seed before first event)
c
c     *** the following dimensions are used (def with parameter statemt)
c            max # of strings per event:      mxstr=20000
c            max # of particles per event:    mxptl=65000
c            max # of big droplets per event: myptl=1000
c
c
c  Upgrades
c  --------
c
c  VENUS 2.00  string flip involving antiquarks.
c
c  VENUS 3.00  fragmentation according to classical string theory.
c              extended resonance table.
c              rescattering of produced particles among themselves.
c              rescattering of produced particles with spectators.
c
c  VENUS 3.01  removing a slight impact-parameter trigger.
c
c  VENUS 3.02  adding analysis routines 'wimi', 'wimino'.
c
c  VENUS 3.04  adding phase space factor for qq-qqbar production.
c              removing an impact parameter trigger at ags energies.
c              no rescattering for iorptl=-1.
c              removing 'tmefrz' parameter, adding 'ndecay' instead.
c              adding parameter 'jorptl'. changing dimension mxptl.
c
c  VENUS 3.05  also rescattering if iorptl=0.
c              removing 'tmeiac' parameter, adding 'tmefor' instead.
c              separating stmt numbers and if stmts.
c
c  VENUS 3.06  lorentz contraction of rescattering distance.
c              radiac+radias as interaction radius.
c              meaning of amsiac modified (amsiac->m+amsiac).
c              multiple rescattering per time step included.
c              cluster decay acc. to phase space.
c              option 'bminim' added.
c
c  VENUS 3.07  option 'labsys' added.
c              analysis routines 'wspa' and 'wspafi' added.
c              nucleon hard core properly treated.
c              string origin as average collision location.
c
c  VENUS 3.10  rho0 -> rhophi*rho0 + (1-rhophi)*omega .
c              new sr idquac, wclu, wclufi.
c              changes to allow lhc energies (26m gev),
c                 new sr utlob2, utrot2, utchm,
c                 new sr utclea to clean /cptl/ .
c              covariant rescattering:
c                 taus (hyperbola and tangent) replaces t.
c                 new sr/fu uttain, uttaus.
c              covariant treatment in sr jamfra. new sr uttaix.
c              iaaptl=0 for parents of interact. ptls.
c              wspa: iacptl=0 for parents and childr of counted ptls.
c              new sr wspa, wspafi: defs acc kinetic theory.
c              no mass limit'n for cluster i.a. any more.
c              new parameter pcomp.
c              sr jclude considerably modified.
c              nrptla and nstptl removed from /cptl/ .
c              ibptl considered in elastic scattering.
c              structure functions (st...) reorganized.
c              new features regarding muon-nucleus scattering.
c                 new sr/fu: lep..., stx... (from g. jancso).
c              call idtr4 only within idquac.
c              idquac: special treatment for !id!<100.
c              hmucex changed and renamed racpro.
c              hard component for quark-pt (fctn sptq).
c
c  VENUS 3.12  ish redefined.
c              no string mass requirement any more at string formatn.
c              no en adjustment any more in sr hasi, only pt is chgd.
c              const sea probability possible (parameter prosea).
c              ish=16 prints calculated sea prob.
c              problems involving >10 quarks of the same flavour fixed.
c              update of subr. pvjpsf.
c              sr jamfra: split string: split not poss for ns=1 -> bl
c              sr wspa(fi): new option iwcent.
c              seedc determined in sr astore (so far in sr avenus).
c              qmust and ptq log energy dependent as default.
c              exact proton and neutron number (not average any more)
c              new option ioctet for octet-octet fragmentation.
c                 ioctet=2 default.
c              change in idtr4 to make ic=1,1 and 2,2 equally probabl
c
c  VENUS 3.14  new cluster decay acc to phase space (new sr jclude).
c
c  VENUS 3.15  leading strings treated as multistrings.
c                 changes in sr's hahabs, hastfs, hastfl, hastpr.
c                 new sr jsplit.
c                 new fn utamnu (formerly part of fn sjcgam).
c
c  VENUS 4.01  sr racpro: gribovs cut-pomeron probabilities.
c
c  VENUS 4.02  sr wjpsi, wjpsif for jpsi analysis added.
c              sr wspa: some storage-saving fixes (also in jfrade).
c              sr ainitl: energy dependent phard (as default)
c
c  VENUS 4.10  sr jamfra: split into jamfra and jambr1
c              alternative to jambr1: jambr2 = SAMBA string breaking proc
c              low mass q-qbar str: fusion, pt transfer, only sea
c              new value w=.32
c              diffractive val + sea
c              phard=.30+12(lg s - lg 30^2)
c              new value pth=1.0
c              hard scatt: e > pt
c              new value ptq1=.26
c              new value iopt=2
c              new value cutmss=0.001
c              new parameter cutmsq=2.0
c              new valence quark count
c              all legs absorbed: random excitation
c              x>1-.85/s: redo h-h
c              x>1-10/s and only qqbar and no q-qbar: cluster
c              rescale strings from multistring
c              absorption of small mass legs
c              covariant string formation
c              new value rstras=0
c              sr idmimj removed
c              random formation time
c              arbitrary large entropy possible
c              high energy rescattering: colour exchange
c                 new parameter delrap=1.5
c              new value iopent=5
c              new parameter thaged=.23
c              new parameter kentro=100000 (repl kzentr)
c              new parameter uentro=4
c              new parameter ebamin=1.5
c              split (covariant) of leading particle
c              epart=elead/nsccx
c              ssplit(x)=z^3-z^5, z=(x+1)/2
c              remnant (after cutting legs): 33% excited, 33% s=3/2 acc
c              small mass str dec
c              new value pdiqua=.12
c              pdiq=0
c              new value raphdi=0
c              sr idresi: rho0=omega, rho0=rho+, no a,f, rho -> .950
c              new value parea=.60
c              pudx=pud*0.945
c              sr hastfr and sr jsplit more accurate (double precision)
c              sr jamfra: changes reg small mass str and backw +jets
c              sr veanly: selection of nondiffractive events possible
c              sr jclude: changes to avoid redo (infinite loops possible)
c
c  VENUS 4.11  sr jclude: modifications concerning redo cluster decay     
c              sr jfrade: split into jfrade and jintac   
c              sr jintac: split into jintac, jintcc, jintfs, jintce,
c                  jintch, jintpa ,jintel, jintfu
c              sr jintce: no iaaptl()=2
c
c  VENUS 4.12  fctn uttaus -> sr uttaus
c              rename sr jintac  to  sr jinta1
c              new sr jinta2 as alternative to jinta1 (option iojint)
c                default: iojint=2 --> call jinta2
c                iojint=1 --> call jinta1 --> equiv 4.10+ 
c                      (4.10+ = 4.10 + redo improvements) 
c              new sr jintfp
c              new sr jintcl
c
c  VENUS 4.15  new value radiac (old: 0.65)
c              new value radias (old: 0.35)
c                 ( if iojint=1, radiac=0.65, radias=0.35
c                   --> equiv 4.10+ )
c              sr jinta2: introduce percolation method
c              remove sr sjcen0 and fctn sjcen4
c              new sr jintov
c              new sr uttaui
c              new sr uttaug
c              xaver() new defined
c              new parameters volnuc, volmes (repl radiac, radias)
c              sr jinta2: expanding droplets
c              rename sr jdeca --> jdecax
c              new sr jdecas to split droplets
c              new sr idenct
c              new sr uttaux
c              nonuniform droplets: new /c4ptl/, new sr uttauh
c              new sr utcley
c
c  VENUS 5.00  unused local variables removed
c              (found by sun f77 compiler)
c
c              sr jinta2, sr jdecas: introduce iflptl(,,) 
c              to account for local flavour conservation 
c              during fusion and droplet decay
c
c              new parameter epscri (repl volmes) (and newly set)
c              new parameter epsbar (repl volbar) (and newly set)
c              new parameter amuseg (repl amumin) (and newly set)
c              new global parameter corlen (and newly set)
c              new local parameter amusef
c
c              function ssplix modified
c              new value parea=0.20 (old:0.60)
c              new value ptf=0.45 (old:0.40)
c              new value pdiqua=0.08 
c              new value pudx=0.47
c              new value pud=0.445
c              prob for qs-anti_qs string break reduced (in jambr1)
c
c              smeared string origins
c
c              new sr's jchinv, jchprt, jcheps
c
c              time evolution followed up to 50 fm/c
c              exponentially distributed  life times
c
c  VENUS 5.03  completely new input/output format
c              i.e. many changes for a-, v-, and w-routines
c
c  VENUS 5.04  nbody decay of droplets (qnb-routines)
c
c  VENUS 5.05  GRV and MRS parton distribution functions
c
c  VENUS 5.06  grand canonical treatment of droplet decay (qgc-routines)
c
c  VENUS 5.07  acceleration of n-body decay procedure (qnb-routines)
c
c  VENUS 5.08  new sr xdensi
c
c  VENUS 5.09  minor technical improvements
c              debugging utclea, utcley, utrepl
c
c  VENUS 5.10  new variable istore (to suppress writing on data-file)
c              renamed: radiac->radbar, radias->radmes
c              new variables rinmes, rinbar (repl epscri, epsbar)
c              second. hadron-hadron: use radmes, radbar; else: rinmes, rinbar
c              new variables sigmes, sigbar, epscri
c              dead code removed (in racpro) and dead variables (qmust*)
c              new value amuseg=10 
c              acceleration of sr qnbpaj (suggestion heck)
c              acceleration of sr jinta2 (suggestion heck)
c              sr jinta1 removed
c              removed: sr xqnbme
c              renamed & changed: sr xqnbmu->xqnbmz
c              new: sr's xqnben, xqnbit, xqnbmu
c              changed: sr's qnbzen, qnbzmu (parts moved to xqnb??)
c
c  VENUS 5.11  unused local variables removed
c              (found by hp f77 compiler)
c
c  VENUS 5.12  new option istore=2
c              minor modifications in xqnb?? routines
c
c  VENUS 5.13  metropolis: better start configuration (from grand can)
c                new and replaced qgc-routines
c              facilities to display grand canonical results
c                new and replaced qgc- and xqgc-routines
c              facilities for statistical analysis of metropolis method              
c                new xtau-routines
c
c  VENUS 5.14  better initial documentation
c              renamed: aset->aaset, astore->aastor
c              debugging for istore=2 option
c              some minor changes to avoid hp compiler warnings
c              new variable deltau, factau
c
c  VENUS 5.15  metropolis: improvements concerning start configurations
c              new sr qgcpen
c              sr's xtauex, xtauin improved
c              new value factau=1.05
c              new variable seedj
c              new variable iterma
c              new sr qnbcor
c              sr qnbodz: new method for "complete specification of event"
c                     (random rotations and then deformations)
c                     new sr's qnbro?
c              sr's utpr?? removed
c              new commons /parof/, /parog/
c              new sr wtime
c
c  VENUS 5.16  new global variable delrex
c              new values pud=0.465, pdiqua=0.095
c              sr qnbini: additional check for nlattc
c
c  VENUS 5.17  qgc-routines: poisson distr rather than gauss
c                  modifications in qgcaaa, xchic, xqgcen 
c              new sr xchamt
c
c  VENUS 5.18  exponential distribution for decay time of small droplets
c              modification of wtime
c
c  VENUS 5.19  technical improvements to avoid warnings
c              redo for initial conf with log(weight).le.1e-35
c
c  VENUS 5.20  sr jinta2: shad correction
c              sr uttain modified
c              srs jdecas, jfrade: minor debugging
c              sr xdensi renamed xdense, minor debugging
c              new parameters phimax, phimin
c
c  VENUS 5.21  ndecax, ndecaw removed
c              new input command nodecay  
c              new input command print  
c              new documentation utlob2
c              modifications in sr aastor
c              new sr afinal
c                includes trafo for space-time points for labsys=1 
c              improved documentation
c              new variables npnevt,nppevt,ntnevt,ntpevt,
c                jpnevt,jppevt,jtnevt,jtpevt
c
c  Particle ID Codes (ISAJET Codes)
c  --------------------------------
c
c  Quarks and leptons are numbered in order of mass:
c        up=1 dn=2 st=3 ch=4 bt=5 tp=6
c        nue=11 e-=12 num=13 mu-=14 nut=15 tau-=16
c  with a negative sign for antiparticles.
c  Arbitrary conventions are:
c        gl=9 gm=10 ks=20 kl=-20 w+=80 z0=90 .
c  The code for a meson is an integer +-jkl, where j.le.k are the
c  quarks and l is the spin. The sign is for the j quark.
c  Flavor singlet mesons are ordered by mass:
c        pi0=110 eta=220 etap=330 etac=440 .
c  Similarly, the code for a baryon is a compound integer +-ijkl
c  formed from the three quarks i,j,k and a spin label l=0,1.
c  The code for a diquark is +-ij00.
c
c  List of ident codes:
c
c     ident     label           mass    charge
c         1     UP        .30000E+00       .67
c        -1     UB        .30000E+00      -.67
c         2     DN        .30000E+00      -.33
c        -2     DB        .30000E+00       .33
c         3     ST        .50000E+00      -.33
c        -3     SB        .50000E+00       .33
c         4     CH        .16000E+01       .67
c        -4     CB        .16000E+01      -.67
c         5     BT        .49000E+01      -.33
c        -5     BB        .49000E+01       .33
c         6     TP        .30000E+02       .67
c        -6     TB        .30000E+02      -.67
c
c         9     GL       0.               0.00
c
c        10     GM       0.               0.00
c
c        11     NUE      0.               0.00
c       -11     ANUE     0.               0.00
c        12     E-        .51100E-03     -1.00
c       -12     E+        .51100E-03      1.00
c        13     NUM      0.               0.00
c       -13     ANUM     0.               0.00
c        14     MU-       .10566E+00     -1.00
c       -14     MU+       .10566E+00      1.00
c        15     NUT      0.               0.00
c       -15     ANUT     0.               0.00
c        16     TAU-      .18070E+01     -1.00
c       -16     TAU+      .18070E+01      1.00
c
c        20     KS        .49767E+00      0.00
c       -20     KL        .49767E+00      0.00
c
c        80     W+        SIN2W=.23       1.00
c       -80     W-        SIN2W=.23      -1.00
c        90     Z0        SIN2W=.23       0.00
c
c       110     PI0       .13496E+00      0.00
c       120     PI+       .13957E+00      1.00
c      -120     PI-       .13957E+00     -1.00
c       220     ETA       .54880E+00      0.00
c       130     K+        .49367E+00      1.00
c      -130     K-        .49367E+00     -1.00
c       230     K0        .49767E+00      0.00
c      -230     AK0       .49767E+00      0.00
c       330     ETAP      .95760E+00      0.00
c       140     AD0       .18633E+01      0.00
c      -140     D0        .18633E+01      0.00
c       240     D-        .18683E+01     -1.00
c      -240     D+        .18683E+01      1.00
c       340     F-        .20300E+01     -1.00
c      -340     F+        .20300E+01      1.00
c       440     ETAC      .29760E+01      0.00
c
c       111     RHO0      .77000E+00      0.00
c       121     RHO+      .77000E+00      1.00
c      -121     RHO-      .77000E+00     -1.00
c       221     OMEG      .78260E+00      0.00
c       131     K*+       .88810E+00      1.00
c      -131     K*-       .88810E+00     -1.00
c       231     K*0       .89220E+00      0.00
c      -231     AK*0      .89220E+00      0.00
c       331     PHI       .10196E+01      0.00
c       141     AD*0      .20060E+01      0.00
c      -141     D*0       .20060E+01      0.00
c       241     D*-       .20086E+01     -1.00
c      -241     D*+       .20086E+01      1.00
c       341     F*-       .21400E+01     -1.00
c      -341     F*+       .21400E+01      1.00
c       441     JPSI      .30970E+01      0.00
c
c      1120     P         .93828E+00      1.00
c     -1120     AP        .93828E+00     -1.00
c      1220     N         .93957E+00      0.00
c     -1220     AN        .93957E+00      0.00
c      1130     S+        .11894E+01      1.00
c     -1130     AS-       .11894E+01     -1.00
c      1230     S0        .11925E+01      0.00
c     -1230     AS0       .11925E+01      0.00
c      2130     L         .11156E+01      0.00
c     -2130     AL        .11156E+01      0.00
c      2230     S-        .11974E+01     -1.00
c     -2230     AS+       .11974E+01      1.00
c      1330     XI0       .13149E+01      0.00
c     -1330     AXI0      .13149E+01      0.00
c      2330     XI-       .13213E+01     -1.00
c     -2330     AXI+      .13213E+01      1.00
c      1140     SC++      .24300E+01      2.00
c     -1140     ASC--     .24300E+01     -2.00
c      1240     SC+       .24300E+01      1.00
c     -1240     ASC-      .24300E+01     -1.00
c      2140     LC+       .22600E+01      1.00
c     -2140     ALC-      .22600E+01     -1.00
c      2240     SC0       .24300E+01      0.00
c     -2240     ASC0      .24300E+01      0.00
c      1340     USC.      .25000E+01      1.00
c     -1340     AUSC.     .25000E+01     -1.00
c      3140     SUC.      .24000E+01      1.00
c     -3140     ASUC.     .24000E+01     -1.00
c      2340     DSC.      .25000E+01      0.00
c     -2340     ADSC.     .25000E+01      0.00
c      3240     SDC.      .24000E+01      0.00
c     -3240     ASDC.     .24000E+01      0.00
c      3340     SSC.      .26000E+01      0.00
c     -3340     ASSC.     .26000E+01      0.00
c      1440     UCC.      .35500E+01      2.00
c     -1440     AUCC.     .35500E+01     -2.00
c      2440     DCC.      .35500E+01      1.00
c     -2440     ADCC.     .35500E+01     -1.00
c      3440     SCC.      .37000E+01      1.00
c     -3440     ASCC.     .37000E+01     -1.00
c
c      1111     DL++      .12320E+01      2.00
c     -1111     ADL--     .12320E+01     -2.00
c      1121     DL+       .12320E+01      1.00
c     -1121     ADL-      .12320E+01     -1.00
c      1221     DL0       .12320E+01      0.00
c     -1221     ADL0      .12320E+01      0.00
c      2221     DL-       .12320E+01     -1.00
c     -2221     ADL+      .12320E+01      1.00
c      1131     S*+       .13823E+01      1.00
c     -1131     AS*-      .13823E+01     -1.00
c      1231     S*0       .13820E+01      0.00
c     -1231     AS*0      .13820E+01      0.00
c      2231     S*-       .13875E+01     -1.00
c     -2231     AS*+      .13875E+01      1.00
c      1331     XI*0      .15318E+01      0.00
c     -1331     AXI*0     .15318E+01      0.00
c      2331     XI*-      .15350E+01     -1.00
c     -2331     AXI*+     .15350E+01      1.00
c      3331     OM-       .16722E+01     -1.00
c     -3331     AOM+      .16722E+01      1.00
c      1141     UUC*      .26300E+01      2.00
c     -1141     AUUC*     .26300E+01     -2.00
c      1241     UDC*      .26300E+01      1.00
c     -1241     AUDC*     .26300E+01     -1.00
c      2241     DDC*      .26300E+01      0.00
c     -2241     ADDC*     .26300E+01      0.00
c      1341     USC*      .27000E+01      1.00
c     -1341     AUSC*     .27000E+01     -1.00
c      2341     DSC*      .27000E+01      0.00
c     -2341     ADSC*     .27000E+01      0.00
c      3341     SSC*      .28000E+01      0.00
c     -3341     ASSC*     .28000E+01      0.00
c      1441     UCC*      .37500E+01      2.00
c     -1441     AUCC*     .37500E+01     -2.00
c      2441     DCC*      .37500E+01      1.00
c     -2441     ADCC*     .37500E+01     -1.00
c      3441     SCC*      .39000E+01      1.00
c     -3441     ASCC*     .39000E+01     -1.00
c      4441     CCC*      .48000E+01      2.00
c     -4441     ACCC*     .48000E+01     -2.00
c
c
c  More Particle ID Codes (beyond ISAJET)
c  --------------------------------------
c 
c  (antiparticles are not explicitely listed)
c
c      1112     dl++(1620)               
c      1113     dl++(1700)               
c      1114     dl++(1925)               
c      2222     dl-(1620)     
c      2223     dl-(1700)     
c      2224     dl-(1925)     
c      1122     n*+(1440)     
c      1123     n*+(1530)     
c      1124     dl+(1620)     
c      1125     n*+(1665)     
c      1126     dl+(1700)     
c      1127     n*+(1710)     
c      1128     dl+(1925)     
c      1222     n*0(1440)          
c      1223     n*0(1530)          
c      1224     dl0(1620)          
c      1225     n*0(1665)          
c      1226     dl0(1700)          
c      1227     n*0(1710)          
c      1228     dl0(1925)          
c      1233     lambda(1405)       
c      1234     lambda(1520)       
c      1235     lambda(1645)       
c      1236     sigma0(1665)       
c      1237     sigma0(1776)       
c      1238     lambda(1845)       
c      1239     sigma0(1930)       
c      1132     sigma+(1665)       
c      1133     sigma+(1776)       
c      1134     sigma+(1930)       
c      2232     sigma-(1665)       
c      2233     sigma-(1776)       
c      2234     sigma-(1930)       
c      332      f0(975)          
c      112      a0(980)          
c      122      a+(980)          
c
c
c  ID Codes of Exotic Particles
c  ----------------------------
c
c  Exotic particles refers to objects outside the resonance table
c  (strings or droplets) 
c
c  The id for these objects is only meant to characterize the quark 
c  content, and not, as for the above particles, also the mass (or spin).
c
c  The general code consists of a 9 digit integer, the first one being
c  the number '8'. the next 4 digits represent the numbers of u quarks,
c  d quarks, s quarks, and c quarks. the following 4 digits represent
c  the corresponding numbers of antiquarks.
c
c  Examples:
c 
c  833000000 represents an object with 3 u quarks and 3 d quarks;
c  800303000 represents an object with 3 s quarks and 3 u antiquarks. 
c
c  There is clearly a problem, when for (at least) one flavour there are 
c  more than 9 (anti)quarks. In this case one uses a code, again consisting 
c  of 9 digits, the first one now being the number '7'. the following 4 
c  digits have to be considered as a four digit number reprenting the number 
c  of quarks, and the last four digits represent correspondingly the number
c  of antiquarks.
c
c  Examples:
c
c  700270000 represents an object with 27 quarks;
c  701210013 represents an object with 121 quarks and 13 antiquarks. 

 
c-----------------------------------------------------------------------
      program aamain
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea       
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub      
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/parob/istore,iprmpt,iecho
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt

      call aaset(0)
      call atitle
9999  continue
      call aread
         if(iappl.le.0)then
      call veanly
      goto9999
         endif
      call afiles
      call ainit
      if(iappl.gt.4)stop'invalid application'
      do  n=1,nevent
      if(irewch.eq.1)rewind(ifch)
      if(iappl.le.3)call avenus
      if(iappl.eq.4)call qnbaaa
      if(nevt.eq.1)call afinal
      if(nevt.eq.1.and.istore.ne.0)call aastor
      call aseed
      enddo
      call astat
      goto9999
      end

c-----------------------------------------------------------------------
      subroutine aaset(iop)
c-----------------------------------------------------------------------
c     sets parameters and options, initializes counters ...
c-----------------------------------------------------------------------
c    ***** aaset commons
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      character*80 fnch,fnhi,fndt
      common/cfname/fnch,nfnch,fnhi,nfnhi,fndt,nfndt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paroi/bmaxim,bminim,phimax,phimin
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      common/paro3/uentro,sigppe,sigppd,asuhax(7),asuhay(7),omega
      common/paro4/grigam,grirsq,gridel,grislo,gricel
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paro8/smas,uumas,usmas,ssmas,ptq1,ptq2,ptq3
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/paroa/iopadi,q2soft
      common/parob/istore,iprmpt,iecho
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/parod/ioinfl,ioinct,iowidn,epsgc
      common/paroe/prob(99),icbac(99,2),icfor(99,2)
      common/cnsta/pi,pii,hquer,prom,piom,ainfin /cvsn/iversn,iverso 
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas /cptlu/nptlu /cnrclu/nrclu
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cmetro/keepr /cepsr/epsr,nepsr
      common/cinfla/keu,ked,kes,kec,keb,ket
      double precision seedi,seedj,seedc
      common /cseed/seedi,seedj,seedc  
      parameter (mxtau=4,mxvol=10,mxeps=16)
      common/cjintc/clust(mxtau,mxvol,mxeps)
      common/cjintd/volsum(mxtau),vo2sum(mxtau),nclsum(mxtau)
      common/ciutot/iutotc,iutote
      common/copen/nopen
      parameter (mxtrig=99,mxidco=99)
      character*6 xvaria,yvaria,trvari
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/p11/nrtrig,trmin(mxtrig),trmax(mxtrig),trvari(mxtrig)
      common/p12/nridco,idcode(mxidco)
      parameter(mxnody=200)
      common/p13/nrnody,nody(mxnody),ndecay 
      parameter(mxpri=200) 
      character*6 subpri
      common/p14/nrpri,subpri(mxpri),ishpri(mxpri) 
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      common/ckhist/khisto
      common/cctcor/nctcor/ccttim/ncttim
      parameter (mxnev=20,matau=6,nzeta=60,mxcoox=20)
      common/cdensi/kdensi(mxnev,matau,nzeta,mxcoox),tauv(matau)
c    ***** end aaset commons
 
      if(iop.eq.1)write(ifmt,'(a)')'default settings ...'
      if(iop.eq.2)goto1002

c  general

      iversn=5.21 *100        !version number
      iverso=4.12 *100        !last official version
      if(iop.eq.0) iappl=1    !choice for application (0,1,2,3,4)

c  number of events

      nevent=1                !number of events

c  file names and units

           if(iop.eq.0)then

      fnch='zzz.check'
      nfnch=9
      fnhi='zzz.histo'
      nfnhi=9
      fndt='zzz.data'
      nfndt=8

      ifop=5
      ifmt=6
      ifch=31
      ifhi=35
      ifdt=51

           endif

c  printout options

      if(iop.eq.0)iprmpt=1   !prompt (>0) or not (<0)
      ish=0      !option for printout
                 !14: call uttima
                 !16: prints sea prob.
                 !18: sr jclude, no-phase-space droplets
                 !19: sr ainitl, call smassp
                 !21: creates histogram for sea distribution
                 !22: sr jfrade, msg after call utclea
                 !23: call jintfp
                 !24: call jintcl
                 !25: call jchprt
                 !26: call qgcpen (plot of energy and flavor density)
                 !27: call qnbcor
                 !29: call qgcpfl (plot of dispersion)
                 !90,91,92,93,94,95: more and more detailed messages.
      ishsub=0   !option for printout
                 !ishsub=ijmn, ij specifies location where ish=mn.
                 !ij=01: sr jcludc
                 !ij=02: sr jetgen
                 !ij=03: sr jfrade, starting before fragmentation
                 !ij=04: sr jdecay
                 !ij=05: sr jdecax
                 !ij=06: sr nucoll
                 !ij=07: sr nucoge+-
                 !ij=08: sr aastor
                 !ij=09: sr jfrade, starting after fragmentation
                 !ij=10: sr jfrade, starting before decay
                 !ij=11: sr jfrade, starting after interactions
                 !ij=12: sr jcentr, entro() in data format
                 !ij=13: sr jcentp
                 !ij=14: sr jdecax if droplet decay
                 !ij=15: sr jsplit
                 !ij=16: sr jfrade
                 !ij=17: sr racpro
                 !ij=18: sr utclea
                 !ij=19: sr jinta1, jinta2, after call utclea
                 !ij=20: sr jdecas 
                 !ij=21: sr jdecas (without jdecax)
                 !ij=22: sr utcley
                 !ij=50: sr qgcnbi
      irandm=0   !prints all random numbers (1)
      irewch=0   !rewinds check file before each event (1)
      ishevt=0   !if ishevt.ne.0: for evt#.ne.ishevt ish is set to 0
      ipagi=0    !print marks between whom ish is set to ish(init)
      if(iop.eq.0)iecho=0    !verify option for input reading
      modsho=1   !message all modsho events
      idensi=0   !must be 1 when subr xdense is used 

c  fragmentation and decay

      pud=0.465  !prob. for u or d quark production 
      pdiqua=.095 !qq-qqbar probability
      pspinl=.50 !spin probability (for light flavours)
      pspinh=.75 !spin probability (for heavy flavours)
      pispn=.50  !isospin probability
      ioptf=1    !option for p_t distribution (1=exponential,2=Gaussian)
      ptf=.45    !average p_transverse
      ptmx=6     !range for pt distribution
      tensn=1.0  !string tension
      parea=.20  !string decay parameter
      delrem=1.  !threshold resonance-string
      delrex=.05 !absolute threshold resonance-string
      kutdiq=4   !cutoff for kmaxor beyond which pdiq=0 in sr jsplit
      iopbrk=1   !option for breaking procedure (1=amor,2=samba)
      smas=0.    !s quark mass
      uumas=0.   !uu diquark mass
      usmas=0.   !us diquark mass
      ssmas=0.   !ss diquark mass
      ndecay=0   !ndecay suppresses certain resonance decays
                 !0000001: all resonance decays suppressed
                 !0000010: k_short/long (+-20) decays suppressed
                 !0000100: lambda (+-2130) decays suppressed
                 !0001000: sigma (+-1130,+-2230) decays suppressed
                 !0010000: cascade (+-2330,+-1330) decays suppressed
                 !0100000: omega (+-3331) decays suppressed
                 !1000000: pi0 (110) decays suppressed
                 !also several "1"s may be used with obvious meaning
      maxres=99999 !maximum resonance spin
      rhophi=.5  !rho/rho+phi ratio

c  lepton-nucleon and e+e-

      engy=10      !string energy
      elepti=43.   !nue energy
      elepto=26.24 !mue  energy
      angmue=3.9645/360.*2*3.14159 !mue angle
 
c  hadron-hadron

      pnll=200    !incident hadron momentum
      idproj=1120 !projectile particle id
      idtarg=1120 !target particle id
      ioptq=2     !option for quark p_t distr (1=exp, 2=Gauss, 3=power)
      ptq1=0.260  !energy dependence of mean transverse momentum of quarks:
      ptq2=0.     !  ptq1+ptq2*alog(e)+ptq3*alog(e)**2
      ptq3=0.     !  with e=sqrt(s)
      phard=-1    !probability for semihard interaction (not used if negative)
      pth=1.0     !cutoff parameter for p_t distr for semihard interactions
      rstras=0.   !effective ratio of strange sea over u sea
      wproj=.32   !diffractive probability (proj)
      wtarg=.32   !diffractive probability (targ)
      cutmsq=2.0  ! eff cutoff mass in structure fcts for sea-valence ratio
      cutmss=.001 !
      pvalen=0.30 !valence quark fraction in case of diffractive interaction
      delmss=.300 !phase space parameter
      neqmn=-5    !minimum number of valence quarks
      neqmx=5     !maximum number of valence quarks
      iaqu=1      !antiquark color exchange (1) or not (0)
      prosea=-1.  !sea prob (if < 0 then calculated from structure fncts)
      idpm=0      !dual parton model (1) or not (else)
      iopadi=0    !choice for parton distribution functions
                  !0: duke and owens
                  !1: glueck,reya,vogt
                  !   (glueck, reya and vogt, z.phys.53 (1992) 127)
                  !2: martin, stirling, roberts (dis)
                  !3: martin, stirling, roberts (msbar)
      q2soft=0.   !q**2 for soft processes
 
c  gribov-regge-theory

      grigam=3.64*0.04 !gamma (in fm**2)
      grirsq=3.56*0.04 !r**2(in fm**2)
      gridel=0.07      !delta=intercept of regge trajectory-1
      grislo=0.25*0.04 !slope of regge trajectory (in fm**2)
      gricel=1.5       !c (determ rel weight of elastic and diffr cross sctn)
      sigppi=-1        !inelastic pp cross section [fm**2]
                       !  if negative: calculated from gribov-regge-theory
 
c  nucleus-nucleus

      laproj=1      !projectile charge number
      maproj=1      !projectile mass number
      latarg=1      !target charge number
      matarg=1      !target mass number
      core=0.8      !hard core distance
      ncolmx=100000 !maximum number of collisions
      fctrmx=10     !parameter determining range for density distribution
      ko1ko2=00009999 !coll trigger (only coll between ko1 and ko2 are used)
      bmaxim=10000  !maximum impact parameter 
      bminim=0.     !minimum impact parameter
      phimax=2*3.1415927 !maximum phi
      phimin=0      !minimum phi 

c  droplet formation

      taurea=1.5    !reaction time
      sigbar=5.00   !baryon-baryon cross section (fm**2)
      sigmes=3.33   !meson-baryon cross section (fm**2)
      rinmes=0.3    !inner meson  radius (<=0: no secondary ia's)
      rinbar=0.4    !inner baryon radius (<=0: no secondary ia's)
      epscri=0.5    !critical energy density
      amsiac=.8     !interaction mass
      iojint=2      !option to call jinta1 (1) or jinta2 (2)
      amprif=0      !print option
      delvol=1      !print option
      deleps=1      !print option
      taumin=1      !min tau for space-time evolution
      deltau=0.2    !initial delta_tau for space-time evolution
      factau=1.05   !factor for delta_tau for space-time evolution
      numtau=80     !number of tau steps for space-time evolution
      dlzeta=0.5    !delta_zeta for longitudinal droplet binning
      etafac=1.2    !factor determining inner range

c  droplet decay 

      ioclud=2      !option for decay of quark matter droplets
                    !1: sequential decay (naive method)
                    !2: nbody decay; hadron gas, microcanonical
      corlen=1.0    !correlation length
                    !   --->corlen>1.0: first fix short droplet breaking<---
      dezzer=1e-4   !abs(dezptl())<dezzer treated as no-droplet
      amuseg=10.0    !segment mass
      bag4rt=0.200  !bag constant ^1/4
      iopent=5      !option for entropy calculation
      uentro=4.     !option for entropy calculation
      kentro=100000 !option for entropy calculation
                    !iopent=0: zero entropy
                    !iopent=1: oscillator model (0 for k.le.uentro)
                    !iopent=2: fermi gas w const volume (0 for k.le.uentro)
                    !iopent=3: fermi gas w const density (0 for k.le.uentro)
                    !iopent=4: fermi gas w const vol - new (0 for k.le.uentro)
                    !iopent=5: resonance gas (hagedorn) (0 for u.le.uentro)
      taunll=1.0    !decay time (comoving frame)
      omega=.500    !oscillator quantum
      ientro=2      !entro() calculated (1) or from data (2)

c  droplet decay initializations

         asuhax(1)=1.134  !two lowest multiplets
         asuhax(2)=1.301  !two lowest multiplets
         asuhax(3)=1.461  !two lowest multiplets
         asuhax(4)=1.673  !two lowest multiplets
         asuhax(5)=0.6125 !two lowest multiplets
         asuhax(6)=0.7915 !two lowest multiplets
         asuhax(7)=1.2320 !two lowest multiplets
         asuhay(1)=0.940  !lowest multiplet
         asuhay(2)=1.200  !lowest multiplet
         asuhay(3)=1.322  !lowest multiplet
         asuhay(4)=1.673  !lowest multiplet
         asuhay(5)=0.1400 !lowest multiplet
         asuhay(6)=0.4977 !lowest multiplet
         asuhay(7)=1.2320 !lowest multiplet

c  droplet specification

      keu=0     !u flavour of droplet
      ked=0     !d flavour of droplet
      kes=0     !s flavour of droplet
      kec=0     !c flavour of droplet
      keb=0     !b flavour of droplet
      ket=0     !t flavour of droplet
      tecm=10   !energy of droplet
      volu=70   !volume of droplet

c  metropolis and grand canonical

      iospec=10 !option for particle species
      iocova=2  !covariant (1) or noncovariant (2) phase space integral
      iopair=2  !single pair (1) or double pair (2) method
      iozero=65 !relative weight of zeros (compared to hadrons)
                ! (-1) nspecs
                ! (-2) nspecs/sqrt(tecm/volu)
      ioflac=1  !test multipl distr without (1) or with (2) flavour conserv
                !  (2 only good for nspecs=3,7)
      iostat=1  !use boltzmann (1) or quantum (0) statistics in qgc-routines
      ioinco=1  !call qnbmin for initial configuration (0)
                !call qgcnbi for initial configuration to generate better
                !initial configuration (1)
                !call qgcnbi for initial configuration to generate optimal
                !initial configuration (2)
      iograc=1  !call qgcaaa in case of ioinco=0 (1)
      epsgc=2.  !required accuracy in qgcaaa 10**(-epsgc)
      iocite=0  !perform counting at metropolis iterations (1) or not (else)
      ioceau=0  !perform counting for exp. autocorrel. time (1) or not (else)
      iociau=0  !perform counting for int. autocorrel. time (1) or not (else)
      ioinct=0  !test initial configuration by chi^2-method (1)
                !test grand canonical metropolis sampling (2)
                !initial configuration without flavor coservation (3)
      ioinfl=1  !conserve flavor in initial configuration in qgcnbi
      iowidn=2  !width of total multiplicity distribution in qgcnbi
                ! sigma_tot -> sigma_tot/iowidn
                ! >0 unnormalized
                ! <0 normalized
      ionlat=2  !determine nlattc ,old method (0)
                !or determine nlattc in qgcnbi as:
                ! (1) max(1.3*<N>,<N>+2*sigma,6)
                ! (2) max(1.5*<N>,<N>+3*sigma,6)
                ! (3) max(2.0*<N>,<N>+4*sigma,6)
      iomom=1   !number of momenta to be changed in qnbodz
      ioobsv=0  !observable for autocorrelation time calculation
                !0: total multiplicity
                !else: particle id for particle species
      iosngl=0  !event # for which counting at metropolis iterations is done
      iorejz=0  !reject pair exchange with only zeros (1) or not (else)
      iompar=4  !parameter for windowing algorithm
      iozinc=0  !if iozevt>0: modifies iozero for testing (in sr qgcnbi)
      iozevt=0  !if >0: modifies iozero for testing (in sr qgcnbi)
      nadd=0    !number of pi0s added to minimum initial configuration
      iterma=-6 !>0: maximum number of iterations 
                !<0: - number of iterations per corr time  
      iterpr=10000 !iter-increment for printout 
      iterpl=1  !iter-increment for plot
      iternc=50 !iterations not counted for analysis
      epsr=1e-4 !required accuracy in sr qnbraw
      keepr=1   !keep most random numbers rnoz in qnbodz (1) 
                !  or update all (0)
 
c  strangelets

      iopenu=1      !option for minimum energy in sjcgam
                    !1: sum of hadron masses
                    !2: bag model curve with minimum at nonzero strangen
      themas=.51225 !parameter theta in berger/jaffe mass formula

c  jpsi 

      jpsi=0     !jpsi to be produced (1) or not (0)
      jpsifi=0   !jpsi final state interaction (1) or not (0)
      sigj=0.2   !jpsi nucleon cross section [fm**2]
      taumx=20   !max time for jpsi evolution
      nsttau=100 !time steps for jpsi evolution
      ijphis=0   !fill jpsi histograms (1) or not (0)

c  analysis: intermittency, space-time, droplets, formation time

      ymximi=2   !upper limit for rapidity interval for intermittency analysis
      imihis=0   !fill intermittency histograms (1) or not (0)
      isphis=0   !fill space-time histograms (1) or not (0)
      ispall=1   !wspa: all ptls (1) or only interacting ptls (else)
      wtmini=-3  !tmin in wspa
      wtstep=1   !t-step in wspa
      iwcent=0   !only central point (1) or longitudinal distr (else) in wspa
      iclhis=0   !fill droplet histograms (1) or not (0)
      iwtime=0   !fill formation time histogram (1) or not (else)
      wtimet=100 !max time in wtime
      wtimei=0   !max mass in wtime
      wtimea=1000 !max mass in wtime

c  other
 
      gaumx=8    !range for gauss distribution
      nclean=0   !clean /cptl/ if nclean > 0 (every nclean_th time step)
      istore=1   !0: no storage to data-file
                 !1: full storage 
                 !2: reduced storage 
      labsys=1   !trafo from pp-cm into lab-system (1) or not (else)
      irescl=1   !momentum rescaling (1) or not (else)
      ifrade=1   !suppression of calling jfrade (0). jfrade=fragm+decay+rescatt
      ntrymx=10  !try-again parameter
      istmax=0   !store only stable ptl (0) or also parents (1)
      seedi=0.   !.ne.0.: seed for random number generator: at start program
      seedj=0.   !.ne.0.: seed for random number generator: for first event

c  constants

      pi=3.1415927
      pii=1./(2*pi**2)
      hquer=0.197327
      prom=.94
      piom=.14
      ainfin=1e30
 
c  zero initializations

      do 6 i=1,99
      prob(i)=0
      do 6 j=1,2
      icbac(i,j)=0
6     icfor(i,j)=0
      imsg=0
      jerr=0
      ntevt=0
      nrevt=0
      naevt=0
      nrstr=0
      nrptl=0
      inoiac=0
      ilamas=0
      nptlu=0
      do itau=1,mxtau
      volsum(itau)=0
      vo2sum(itau)=0
      nclsum(itau)=0
      do ivol=1,mxvol
      do ieps=1,mxeps
      clust(itau,ivol,ieps)=0
      enddo
      enddo
      enddo
      iutotc=0
      iutote=0
      nopen=0
      xpar1=0
      xpar2=0
      xpar3=0
      xpar4=0
      xpar5=0
      xpar6=0
      xpar7=0
      xpar8=0
      if(iop.eq.0)khisto=0
      nrclu=0
      nrnody=0
      do n=1,mxnody
      nody(n)=0
      enddo
      nrpri=0
      do n=1,mxpri
      subpri(n)='      '
      ishpri(n)=0
      enddo
      nctcor=0
      ncttim=0
      do n=1,matau
      tauv(n)=0
      enddo

c  analysis

1002  continue
      xvaria='numptl'
      yvaria='ycmptl'
      normal=11
      xminim=-100
      xmaxim=100
      nrbins=100
      hisfac=1
      nrtrig=0
      nridco=0
      do nr=1,mxbins
      do l=1,5
      ar(nr,l)=0
      enddo
      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine aastor
c-----------------------------------------------------------------------
c     writes the results of a simulation into the file with unit ifdt
c     contains a description of the stored variables.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/parob/istore,iprmpt,iecho
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      integer idiptl(mxptl),idjptl(mxptl)
      
c  calculate id of the father (--> idiptl) and mother (--> idjptl)
c  (set to 999 if the corresponding parent does not exist)    

      do i=1,nptl
      if(iorptl(i).le.0)idiptl(i)=999      
      if(iorptl(i).gt.0)idiptl(i)=idptl(iorptl(i))      
      if(jorptl(i).le.0)idjptl(i)=999      
      if(jorptl(i).gt.0)idjptl(i)=idptl(jorptl(i))      
      enddo
 
c  count the number of particles to be stored (--> nptevt)

      nptevt=0
      do i=1,nptl
      if(istptl(i).le.istmax)nptevt=nptevt+1
      enddo
       
c  store event variables:

      if(istore.eq.1)write(ifdt)nrevt,nptevt,bimevt,phievt,kolevt
     *,colevt,pmxevt,egyevt,npjevt,ntgevt

      if(istore.eq.2)write(ifdt)nrevt,nptevt,bimevt,phievt
     *,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt 
 
c     nrevt.......... event number
c     nptevt ........ number of (stored!) particles per event
c     bimevt ........ absolute value of impact parameter
c     phievt ........ angle of impact parameter
c     kolevt ........ number of collisions
c     colevt ........ effective number of collisions
c     pmxevt ........ reference momentum
c     egyevt ........ pp cm energy (hadron) or string energy (lepton)
c     npjevt ........ number of primary projectile participants
c     ntgevt ........ number of primary target participants
c     npnevt ........ number of primary projectile neutron spectators
c     nppevt ........ number of primary projectile proton spectators
c     ntnevt ........ number of primary target neutron spectators
c     ntpevt ........ number of primary target proton spectators
c     jpnevt ........ number of absolute projectile neutron spectators
c     jppevt ........ number of absolute projectile proton spectators
c     jtnevt ........ number of absolute target neutron spectators
c     jtpevt ........ number of absolute target proton spectators
 
      if(iappl.eq.2)return
 
      do i=1,nptl
      if(istptl(i).le.istmax)then
      
c  store particle variables:
 
      if(istore.eq.1)write(ifdt)
     *i,idptl(i),pptl(1,i),pptl(2,i),pptl(3,i),pptl(4,i),pptl(5,i)
     *,iorptl(i),jorptl(i),istptl(i)
     *,xorptl(1,i),xorptl(2,i),xorptl(3,i),xorptl(4,i)
     *,tivptl(1,i),tivptl(2,i)

      if(istore.eq.2)write(ifdt)
     *idptl(i),pptl(1,i),pptl(2,i),pptl(3,i),pptl(4,i)
     *,xorptl(1,i),xorptl(2,i),xorptl(3,i),xorptl(4,i)
     *,idiptl(i),idjptl(i)

c     i ............. particle number
c     idptl(i) ...... particle id
c     pptl(1,i) ..... x-component of particle momentum 
c     pptl(2,i) ..... y-component of particle momentum 
c     pptl(3,i) ..... z-component of particle momentum 
c     pptl(4,i) ..... particle energy 
c     pptl(5,i) ..... particle mass 
c     iorptl(i) ..... particle number of father (if .le. 0 : no father) 
c     jorptl(i) ..... particle number of mother (if .le. 0 : no mother)
c     istptl(i) ..... generation flag: last gen. (0) or not (1) or ghost (2)
c     xorptl(1,i) ... x-component of formation point
c     xorptl(2,i) ... y-component of formation point
c     xorptl(3,i) ... z-component of formation point
c     xorptl(4,i) ... formation time
c     tivptl(1,i) ... formation time (always in the pp-cms!)
c     tivptl(2,i) ... destruction time (always in the pp-cms!)
c     idiptl(i) ..... id of father (999 if no father)
c     idjptl(i) ..... id of mother (999 if no mother)

      endif 
      enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine afiles
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      character*80 fnch,fnhi,fndt
      common/cfname/fnch,nfnch,fnhi,nfnhi,fndt,nfndt
      open(unit=ifch,file=fnch(1:nfnch),status='unknown')
      open(unit=ifhi,file=fnhi(1:nfnhi),status='unknown')
      open(unit=ifdt,file=fndt(1:nfndt),status='unknown'
     *,form='unformatted')
      return
      end

c-----------------------------------------------------------------------
      subroutine afinal
c-----------------------------------------------------------------------
c  does some final calculations, to be called before call aasto.
c  * calculates nptlu, the maximum nptl for all events.
c  * calculates xorptl(j,i), representing formation points.
c    (xorptl(j,i),j=1,4) is the 4-vector representing the space-time of
c    creation of particle i.
c  * in case of labsys=1, these vectors are transformed into the lab system
c    (being originally in the pp-cms),
c    but only for points in the forward lightcone
c    (otherwise all components of a vector are set to -ainfin)
c  * calculates numbers of spectators:
c    npnevt (number of primary proj neutron spectators)
c    nppevt (number of primary proj proton spectators)
c    ntnevt (number of primary targ neutron spectators)
c    ntpevt (number of primary targ proton spectators)
c    jpnevt (number of absolute proj neutron spectators)
c    jppevt (number of absolute proj proton spectators)
c    jtnevt (number of absolute targ neutron spectators)
c    jtpevt (number of absolute targ proton spectators)
c-----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/parob/istore,iprmpt,iecho
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      parameter(mxpri=200) 
      character*6 subpri
      common/p14/nrpri,subpri(mxpri),ishpri(mxpri) 
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas /cptlu/nptlu  
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      double precision p1,p2,p3,p4,p5,x1,x2,x3,x4
      
      ishini=ish
      if(nrpri.gt.0)then
      do nr=1,nrpri
      if(subpri(nr).eq.'afinal')then
      ish=ishpri(nr)
      endif       
      enddo
      endif

      nptlu=max0(nptl,nptlu)

      do i=1,nptl
      t=tivptl(1,i)
      xorptl(1,i)=xorptl(1,i)+(t-xorptl(4,i))*pptl(1,i)/pptl(4,i)
      xorptl(2,i)=xorptl(2,i)+(t-xorptl(4,i))*pptl(2,i)/pptl(4,i)
      xorptl(3,i)=xorptl(3,i)+(t-xorptl(4,i))*pptl(3,i)/pptl(4,i)
      xorptl(4,i)=t
      enddo

      if(labsys.eq.1)then
      p1=0
      p2=0
      p3=sinh(yhaha)
      p4=cosh(yhaha)
      p5=1
      do i=1,nptl
      x1=xorptl(1,i)
      x2=xorptl(2,i)
      x3=xorptl(3,i)
      x4=xorptl(4,i)
      call utlob2(-1,p1,p2,p3,p4,p5,x1,x2,x3,x4)
      xorptl(1,i)=x1
      xorptl(2,i)=x2
      xorptl(3,i)=x3
      xorptl(4,i)=x4
      enddo
      endif
      
      npnevt=0
      nppevt=0
      ntnevt=0
      ntpevt=0    
      jpnevt=0
      jppevt=0
      jtnevt=0
      jtpevt=0    
      if(ish.ge.95)write(ifch,'(/31a1/a/31a1)')('-',l=1,31)
     *,'primary and absolute spectators',('-',l=1,31)
      if(ish.ge.95)write(ifch,'(/a//a/)')'projectile nucleons:'
     *,'     i    id   ior   ist'        
      do i=1,maproj
      if(ish.ge.95)write(ifch,'(4i6)')i,idptl(i),iorptl(i),istptl(i)
      io=iorptl(i)
      id=idptl(i)
      is=istptl(i)
      if(io.eq.0.and.id.eq.1220)npnevt=npnevt+1
      if(io.eq.0.and.id.eq.1120)nppevt=nppevt+1
      if(io.eq.0.and.is.eq.0.and.id.eq.1220)jpnevt=jpnevt+1
      if(io.eq.0.and.is.eq.0.and.id.eq.1120)jppevt=jppevt+1
      enddo
      if(ish.ge.95)write(ifch,'(/a//a/)')'target nucleons:'
     *,'     i    id   ior   ist'        
      do i=maproj+1,maproj+matarg
      if(ish.ge.95)write(ifch,'(4i6)')i,idptl(i),iorptl(i),istptl(i)
      io=iorptl(i)
      id=idptl(i)
      is=istptl(i)
      if(io.eq.0.and.id.eq.1220)ntnevt=ntnevt+1
      if(io.eq.0.and.id.eq.1120)ntpevt=ntpevt+1 
      if(io.eq.0.and.is.eq.0.and.id.eq.1220)jtnevt=jtnevt+1
      if(io.eq.0.and.is.eq.0.and.id.eq.1120)jtpevt=jtpevt+1 
      enddo
      if(ish.ge.95)then
      write(ifch,'(/a/)')'numbers of participants and spectators:'
      write(ifch,'(a,i4,a,i4)')'primary participants:   projectile:'
     *,npjevt,'   target:',ntgevt
      write(ifch,'(a,i4,a,i4)')'primary spectators:     projectile:'
     *,npnevt+nppevt,'   target:',ntnevt+ntpevt
      write(ifch,'(a,i4,a,i4)')
     *'primary spectator neutrons:   projectile:',npnevt
     *,'   target:',ntnevt
      write(ifch,'(a,i4,a,i4)')
     *'primary spectator protons:    projectile:',nppevt
     *,'   target:',ntpevt
      write(ifch,'(a,i4,a,i4)')'absolute spectators:    projectile:'
     *,jpnevt+jppevt,'   target:',jtnevt+jtpevt
      endif

      if(istore.eq.0)then
      if(ish.eq.27)call qnbcor(2)
      if(imihis.eq.1)call wimi
      if(imihis.eq.1.and.nrevt.eq.nevent)call wimino
      if(isphis.eq.1)call wspa
      if(isphis.eq.1.and.nrevt.eq.nevent)call wspafi
      if(iclhis.eq.1)call wclu
      if(iclhis.eq.1.and.nrevt.eq.nevent)call wclufi
      if(ijphis.eq.1)call wjpsi
      if(ijphis.eq.1.and.nrevt.eq.nevent)call wjpsif
      if(iwtime.eq.1)call wtime(1)
      if(iwtime.eq.1.and.nrevt.eq.nevent)call wtime(2)
      endif
            
      ish=ishini       
      return
      end

c-----------------------------------------------------------------------
      subroutine ainit
c-----------------------------------------------------------------------
c    ***** aaset commons
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      character*80 fnch,fnhi,fndt
      common/cfname/fnch,nfnch,fnhi,nfnhi,fndt,nfndt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paroi/bmaxim,bminim,phimax,phimin
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      common/paro3/uentro,sigppe,sigppd,asuhax(7),asuhay(7),omega
      common/paro4/grigam,grirsq,gridel,grislo,gricel
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau  
      common/paro8/smas,uumas,usmas,ssmas,ptq1,ptq2,ptq3
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/paroa/iopadi,q2soft
      common/parob/istore,iprmpt,iecho
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/parod/ioinfl,ioinct,iowidn,epsgc
      common/paroe/prob(99),icbac(99,2),icfor(99,2)
      common/cnsta/pi,pii,hquer,prom,piom,ainfin /cvsn/iversn,iverso 
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas /cptlu/nptlu  /cnrclu/nrclu
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cmetro/keepr /cepsr/epsr,nepsr
      common/cinfla/keu,ked,kes,kec,keb,ket
      double precision seedi,seedj,seedc
      common /cseed/seedi,seedj,seedc  
      parameter (mxtau=4,mxvol=10,mxeps=16)
      common/cjintc/clust(mxtau,mxvol,mxeps)
      common/cjintd/volsum(mxtau),vo2sum(mxtau),nclsum(mxtau)
      common/ciutot/iutotc,iutote
      common/copen/nopen
      parameter (mxtrig=99,mxidco=99)
      character*6 xvaria,yvaria,trvari
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/p11/nrtrig,trmin(mxtrig),trmax(mxtrig),trvari(mxtrig)
      common/p12/nridco,idcode(mxidco)
      parameter(mxnody=200)
      common/p13/nrnody,nody(mxnody),ndecay 
      parameter(mxpri=200) 
      character*6 subpri
      common/p14/nrpri,subpri(mxpri),ishpri(mxpri) 
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      common/ckhist/khisto
      common/cctcor/nctcor/ccttim/ncttim
      parameter (mxnev=20,matau=6,nzeta=60,mxcoox=20)
      common/cdensi/kdensi(mxnev,matau,nzeta,mxcoox),tauv(matau)
c    ***** end aaset commons
      common/cquama/quama /cipio/ipio
      parameter (nptf=129)
      common /cptf/xptf(nptf),wptf(nptf)
     *,qptfu(nptf),qptfs(nptf),qptfuu(nptf),qptfus(nptf),qptfss(nptf)
      parameter (ngau=129)
      common /cgau/xgau(ngau),qgau(ngau),wgau(ngau)
      parameter (nsplit=129)
      common /csplit/xsplit(nsplit),qsplit(nsplit),qsplix(nsplit)
     *,wsplit(nsplit)
      parameter (nstru=2049)
      common/stru/xstru(nstru),wstru(nstru)
     *,qvap(nstru),qvat(nstru),qsep(nstru),qset(nstru)
      parameter (nptj=129)
      common /cptj/xptj(nptj),qptj(nptj),wptj(nptj)
      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      parameter(nprbms=20)
      common /cprbms/prbms(nprbms)
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      common /cden/massnr
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common /clatt/nlattc,npmax
      external sdensi,stzzvl,stzzse
     *,sptj,spth,sptq,sptf,sgau,ssplit,ssplix

      write(ifmt,'(a)')'initializations ...'

      if(iabs(idproj).lt.1000.or.iabs(idtarg).lt.1000)then
      sigppi=2
      write(ifmt,'(//a,f6.2,a//)')'hadron-nucleon cross section set to'
     *,sigppi*10,' mb  (independent of energy!!!!)'      
      endif

      if(seedi.ne.0.)call ranfst(seedi)
      seedc=seedi

      if(ish.eq.25)then
      call jchprt
      stop'ish=25 causes stop after call jchprt'
      endif

      if(isphis.eq.1)labsys=0
      if(icinpu.ge.1)elepti=engy
      if(idpm.eq.1)iaqu=0
      if(idpm.eq.1)neqmn=2
      if(idpm.eq.1)neqmx=3
      if(iopenu.eq.2)call smassi(themas)
      if(iopenu.eq.2.and.ish.eq.19)call smassp
      if(iopenu.eq.2.and.ish.eq.19)stop

      if(iojint.eq.2)then
      if(rinmes.le.0..or.rinbar.le.0.)then
      radbar=0.
      radmes=0.
      rinbar=0.
      rinmes=0.
      else
      radbar=sqrt(sigbar/4./pi)
      radmes=amax1(0., sqrt(sigmes/pi)-radbar )
      endif
      endif

      if(iappl.eq.3)then     
      yhaha=0
      ypjtl=0
      endif
 
           if(iappl.eq.1.or.iappl.eq.2)then

      call idmass(idproj,amproj)
      call idmass(idtarg,amtarg)
         if(pnll.lt.0)then
      engy=-pnll
      pnll=sqrt( ((engy**2-amproj**2-amtarg**2)/2/amtarg)**2-amproj**2 )
         else
      engy=sqrt( 2*sqrt(pnll**2+amproj**2)*amtarg+amtarg**2+amproj**2 )
         endif
      if(engy.lt.12.)
     *call utstop('ainitl: incident energy too small       ')
      engyi=engy
      pnlli=pnll
         if(pnll/amproj.lt.1e2)then
      trm=sqrt(pnll**2+amproj**2)
      engy=sqrt((trm+amtarg-pnll)*(trm+amtarg+pnll))
         else
      trm=amproj**2/2/pnll+amtarg
      engy=sqrt(trm*(2*pnll+trm))
         endif
      d1=abs(pnlli-pnll)/pnll
      d2=abs(engyi-engy)/engy
         if(d1.gt.1e-3.or.d2.gt.1e-3)then
      if(ish.ge.0)then
      call utmsg('ainitl')
      write(ifch,*)'*****  pnll,pnlli:',pnll,pnlli
      write(ifch,*)'*****  engy,engyi:',engy,engyi
      call utmsgf
      endif
         endif
      s=engy**2
      pnllx=utpcm(engy,amproj,amtarg)
      yhaha=alog((sqrt(pnll**2+s)+pnll)/sqrt(s))
      ypjtl=alog((sqrt(pnll**2+amproj**2)+pnll)/amproj)
      ptq=ptq1+ptq2*alog(engy)+ptq3*alog(engy)**2
      if(phard.lt.0.)then
c-c   phard=1-exp(-0.023329-0.000351*sqrt(s))
c-c   phard=0
c-c   if(sqrt(s).gt.100.)
c-c  *phard=1-exp(-0.000275*(sqrt(s)-100.))
      phard=0.030+0.12*(alog10(s)-alog10(30.**2))
      phard=amin1(1.,phard)
      phard=amax1(0.030,phard)
      endif

           elseif(iappl.eq.3)then

      s=12.**2
      ptq=ptq1+ptq2*alog(engy)+ptq3*alog(engy)**2

           endif

           if(iappl.eq.1.or.iappl.eq.3.or.iappl.eq.4)then

      call jdecin(.false.)

           endif
           
           if(iappl.eq.1.or.iappl.eq.3)then

      c=ptmx
      quama=0.
      call utquaf(sptf,nptf,xptf,qptfu,wptf,0.,.33*c,.66*c,c)
      quama=smas
      call utquaf(sptf,nptf,xptf,qptfs,wptf,0.,.33*c,.66*c,c)
      quama=uumas
      call utquaf(sptf,nptf,xptf,qptfuu,wptf,0.,.33*c,.66*c,c)
      quama=usmas
      call utquaf(sptf,nptf,xptf,qptfus,wptf,0.,.33*c,.66*c,c)
      quama=ssmas
      call utquaf(sptf,nptf,xptf,qptfss,wptf,0.,.33*c,.66*c,c)
      c=gaumx
      call utquaf(sgau,ngau,xgau,qgau,wgau,0.,.33*c,.66*c,c)
      call utquaf(ssplit,nsplit,xsplit,qsplit,wsplit,0.,.33,.66,1.)
      call utquaf(ssplix,nsplit,xsplit,qsplix,wsplit,0.,.33,.66,1.)
 
      xcut=cutmsq/sqrt(s)
      b=amin1(.05,xcut*500)
      a=amin1(.2*b,xcut*100)
      ipio=0
      if(iabs(idproj).lt.1000)ipio=1
      call utquaf(stzzvl,nstru,xstru,qvap,wstru,0.,a,b,1.)
      qvapc=qvap(nstru)
      call utquaf(stzzse,nstru,xstru,qsep,wstru,0.,a,b,1.)
      qsepc=qsep(nstru)
      ipio=0
      if(iabs(idtarg).lt.1000)ipio=1
      call utquaf(stzzvl,nstru,xstru,qvat,wstru,0.,a,b,1.)
      qvatc=qvat(nstru)
      call utquaf(stzzse,nstru,xstru,qset,wstru,0.,a,b,1.)
      qsetc=qset(nstru)
      if(ish.eq.16)write(ifch,*)'sr anitl: qvapc,qsepc,qvatc,qsetc:'
      if(ish.eq.16)write(ifch,*)qvapc,qsepc,qvatc,qsetc
 
      if(prosea.ge.0.)then
      qvapc=1.
      qsepc=prosea
      qvatc=1.
      qsetc=prosea
      endif
 
      xcut=cutmss/sqrt(s)
      b=amin1(.05,xcut*500)
      a=amin1(.2*b,xcut*100)
      ipio=0
      if(iabs(idproj).lt.1000)ipio=1
      call utquaf(stzzvl,nstru,xstru,qvap,wstru,0.,a,b,1.)
      call utquaf(stzzse,nstru,xstru,qsep,wstru,0.,a,b,1.)
      ipio=0
      if(iabs(idtarg).lt.1000)ipio=1
      call utquaf(stzzvl,nstru,xstru,qvat,wstru,0.,a,b,1.)
      call utquaf(stzzse,nstru,xstru,qset,wstru,0.,a,b,1.)
 
      if(ish.eq.21)call uthsea
      if(ish.eq.21)stop
 
      c=ptmx
      call utquaf(sptj,nptj,xptj,qptj,wptj,0.,.33*c,.66*c,c)
      c=ptmx
      call utquaf(spth,nptq,xptq,qpth,wptq,0.,.33*c,.66*c,c)
      call utquaf(sptq,nptq,xptq,qptq,wptq,0.,.33*c,.66*c,c)

      call racpro(nprbms,prbms)
 
           endif
 
           if(iappl.le.3)then

      massnr=maproj
      rmproj=0
      if(massnr.gt.1)then
      c=1.19*massnr**(1./3.)-1.61*massnr**(-1./3.)+fctrmx*.54
      rmproj=c
      call utquaf(sdensi,ndep,xdep,qdep,wdep,0.,.33*c,.66*c,c)
      endif
      massnr=matarg
      rmtarg=0
      if(massnr.gt.1)then
      c=1.19*massnr**(1./3.)-1.61*massnr**(-1./3.)+fctrmx*.54
      rmtarg=c
      call utquaf(sdensi,ndet,xdet,qdet,wdet,0.,.33*c,.66*c,c)
      endif
      bmax=rmproj+rmtarg

           endif
 
           if(iappl.eq.1)then

      if(idpm.eq.1)qsepc=0
      if(idpm.eq.1)qsetc=0

           endif
 

           if(iappl.eq.1.or.iappl.eq.3.or.iappl.eq.4)then

      call qnbspd(iospec)

           endif

      rewind ifdt

      if(seedj.ne.0.)call ranfst(seedj)
      seedc=seedj

      write(ifmt,'(a)')'generate events ...'
 
      return
      end
 
c---------------------------------------------------------------------
      subroutine aread
c---------------------------------------------------------------------
c  reads and interprets input commands
c---------------------------------------------------------------------     
c
c  commands (to be read from standard input)
c
c  !:     indicates comment
c         text between '!' and the end of line is ignored
c
c  application: defines application
c         syntax: application <appl>
c            with <appl> being one of the following:
c            hadron     (hadron-hadron, hadron-nucleus and nucleus-nucleus)
c            geometry   (hadron-nucleus, nucleus-nucleus, but only geometry)
c            lepton     (e+e-, lepton-nucleon, lepton-nucleus)
c            droplet    (decay of a given quark matter droplet)
c            analysis   (analysis of events stored in data file)
c
c  call:  calls a subroutine
c         syntax: call <subr> 
c            with <subr> being the name of the subroutine
c         choices for <subr>:
c            xdense 
c            xqgcen xqgcmt xqgcmu xqgcpt xqgcra 
c            xqnben xqnbit xqnbmu xqnbmz
c            xtauex xtauin
c
c  decayall:undoes the dacay suppression specified via nodecay
c         syntax: decayall
c
c  echo:  determines whether input is echoed
c         syntax: echo <ec>
c            with <ec>: on off
c
c  fname: defines file names
c         syntax: fname <type> <name>
c             with <type> referring to the type of output file
c             (check, histo, data)
c             and with <name> being an arbitrary character string 
c         default file names: zzz.check, zzz.histo, zzz.data
c
c  help:  provides list of valid commands
c         syntax: help
c
c  idcode:specifies ptl id's for particles to be considered in analysis
c         syntax: idcode <id>
c            with <id> being venus particle id (120 for pi+ ...)
c
c  input: reads commands from file
c         syntax: input <filename>
c            with <filename> being the name of a file with input commands
c         commands are read untill a 'return' command is found
c         without 'return', the whole file is read
c
c  nodecay:suppresses decay
c         syntax: nodecay <id>
c            with <id> being venus particle id (110 for pi0 ...)
c         examples:  
c            nodecay 110    !suppresses pi0 decay
c            nodecay 2130   !suppresses lambda decay
c            nodecay -2130  !suppresses antilambda decay
c            nodecay 20     !suppresses k-short decay
c
c  print:  controlls printout
c          syntax: print <subroutine> <level>
c             with <subroutine> being the name of a venus subroutine
c             and <level> an integer between 0 and 9 
c          printout of level 0 to <level> is activated in subr <subroutine>
c
c  prompt:determines whether a prompt is printed in case of expected input
c         syntax: prompt <pr>
c            with <pr>: on off auto
c         the option auto suppresses the prompt in case of input from a
c         file via the input command, otherwise the prompt is printed
c
c  return:terminates reading commands from file (see 'input' command)
c         syntax: return
c
c  run:   starts simulation
c         syntax: run
c
c  set:   sets parameter or option (p/o)
c         syntax: set <par> <val>
c            with <par> and <val> being the name (p/o-name) and the 
c            desired value (p/o-value) of a parameter or option 
c         a list of possible p/o-names <par> may be obtained
c         by using the show command
c
c  setm:  sets multiple parameters and options in one step
c         syntax: setm <p1> <p2> ...
c            with <p1> <p2> ... being values of certain parameters/options
c            depending on the current application, as shown in the foll table:
c            -------------------------------------------------------------
c            application  !  <p1> <p2> ...          
c            -------------------------------------------------------------
c            hadron       ! nevent modsho pnll laproj maproj latarg matarg
c            geometry     ! nevent modsho pnll laproj maproj latarg matarg
c            lepton       ! nevent modsho engy latarg matarg
c            droplet      ! tecm volu iterma iterpr iterpl iternc ish
c            analysis     ! xvaria yvaria normal xminim xmaxim nrbins
c            -------------------------------------------------------------
c         the setm command provides a shortcut to set frequently 
c         used parameters and options 
c
c  show:  shows parameter and option settings
c         syntax: show
c         the show command in particular provides a list of
c         parameters and options which may be changed by the set command
c
c  showa: shows only a selection of parameter and option settings relevant for
c         the current application
c         syntax: showa
c
c  stop:  stops execution
c         syntax: stop
c
c  string:defines a string for the lepton application
c         syntax: string <prob> <icbac1> <icbac2> <icfor1> <icfor2>
c            with <prob> being the relative weight of the string 
c            and <icbac1> <icbac2> being the id of the backward string end
c            and <icfor1> <icfor2> being the id of the forward string end
c         up to 99 strings may be defined this way
c         if no strings are defined in the lepton application, strings are
c         determined from the parton model, assuming muon-nucleon scattering            
c         (in this case the parameters elepti, elepto, angmue must be set)
c
c  trigger: defines trigger condition for application analysis
c         syntax: trigger <trvari> <trmin> <trmax>
c            with <trvari> chracterizing the variable (for possible
c            choices see table 1)
c            and with <trmin> and <trmax> specifying the minimum and
c            the maximum value of the variable
c
c  write: writes some text to histo-file
c         syntax: write "<text>" 
c         or:     write ""<text>""
c            with some arbitrary text <text>
c            (if <text> contains quotes ("), the double quotes
c            should be used: write "" ..."..."..."")
c
c  writearray: writes histogram (from sequence "application analysis ... run"
c         to histo-file
c         syntax: writearray <ncolumn>
c            with <ncolumn> being 2 or 3, specifying whether 
c            "x y"  or "x y delta_y" is written
c            ...
c  only the run command and the stop command are obligatory,
c  all other commands are optional, also set and setm
c  (all parameters are set initially to some default values, so setm
c  and/or set commands are only needed to modify these values)
c
c  example for a complete input sequence (without analysis):
c
c     application droplet
c     showa
c     setm 10 70   20000 100 100 1000   93
c     set iozero 27
c     showa
c     run 
c     stop
c
c
c  "application analysis" starts the analysis facilities, 
c  which amounts to filling a histogram "y versus x" (for example
c  y = number of pions, x = rapidity). "x" and "y" are specified
c  via setting "xvaria" and "yvaria" (see table 1 in the beginning
c  of subr aread). the appropriate normalization is specified via 
c  setting "normal" (see table 2 in the beginning of subr aread).
c  xvaria, yvaria, normal (and more) may be set via setm:
c     setm <xvaria> <yvaria> <normal> <xminim> <xmaxim> <nrbins>
c  where <xvaria>, <yvaria>, and <normal> are the desired settings for
c  xvaria, yvaria, and normal, <xminim> and <xmaxim> specify the
c  desired x-range, and <nrbins> the number of bins.
c  trigger condition are specified via the trigger command:
c    trigger <trvari> <trmin> <trmax>
c  which forces variable <trvari> to  be considered only between
c  <trmin> and <trmax> (possible choices for <trvari> as for
c  "xvaria" or "yvaria").
c  particle id's are specified via tha idcode command:
c    idcode <id> 
c  with <id> being a venus id.
c  the command writearray writes the histogram to the histo-file.
c  
c  example:
c
c     application hadron
c     ...
c     run
c     application analysis             
c     setm ycmptl numptl 11 -3 3 100   !to generate a rapidity distribution
c     trigger ptrptl 0.2 2.0           !only pt between .2 and 2 considered
c     idcode 120                       !pi+ considered
c     idcode -120                      !and pi- considered
c     run
c     write "rapidity ditribution:"    !writes text into histo-file
c     writearray                       !writes histogram into histo-file
c     stop
c
c
c  table 1: possible values for xvaria and yvaria
c
c     !----------------------------------------------------------!
c     !   particle variables:                                    !
c     !----------------------------------------------------------!
c     !   numptl ... number of ptls  (=1)                        !
c     !   nufptl ... number of ptls in forw hemisph              !
c     !   nubptl ... number of ptls in backw hemisph             !
c     !   nldptl ... number of lambdas                           !
c     !   nalptl ... number of alambdas                          !
c     !   nksptl ... number of k_shorts                          !
c     !   nngptl ... number of negatives                         !
c     !   npmptl ... number of ptls * 0+1-1 (zero/pos/neg chrg)  !
c     !   npaptl ... number of ptls * +1-1 (ptl/antiptl)         !
c     !   r.cptl ... hadronization point in pp cm (x,y,z,0)      !
c     !   yinptl ... space-time rapidity at hadr. point          !
c     !   tinptl ... proper time at hadr. point                  !
c     !   tmeptl ... time                                        !
c     !   tauptl ... eigentime                                   !
c     !   r.tptl ... space time (x,y,z,0)                        !
c     !   rttptl ... transverse space time                       !
c     !   xcmptl ... longitud. momentum fraction of ptl in pp cm !
c     !   xenptl ... energy fraction of ptl in pp cm             !
c     !   xelptl ... energy fraction of ptl in lab sys           !
c     !   xpcptl ... momentum fraction in cm                     !
c     !   xpaptl ... momentum fraction absolute in cm            !
c     !   xpeptl ... z-momentum (cm) / engy/2                    !
c     !   sobptl ... s / beta of ptl in pp cm                    !
c     !   xlbptl ... longitud. momentum fraction of ptl in lab   !
c     !   ycmptl ... rapdity of ptl in pp cm                     !
c     !   ycnptl ... -rapdity of ptl in pp cm                    !
c     !   yacptl ... /rapdity/ of ptl in pp cm                   !
c     !   ylbptl ... rapdity of ptl in lab                       !
c     !   gcmptl ... pseudo rapdity of ptl in pp cm              !
c     !   glbptl ... pseudo rapdity of ptl in lab                !
c     !   ptrptl ... transverse momentum of ptl                  !
c     !   plbptl ... momentum of ptl (lab)                       !
c     !   etlptl ... transverse energy of ptl                    !
c     !   pt2ptl ... transverse momentum squared of ptl          !
c     !   amtptl ... transverse mass of ptl                      !
c     !   amsptl ... mass of ptl                                 !
c     !   amlptl ... log10(mass) of ptl                          !
c     !   phlptl ... azimuthal angle (lab)                       !
c     !   alsptl ... log s of evt                                !
c     !   iorptl ... origin of ptl                               !
c     !   e0lptl ... energy of particle (lab)                    !
c     !----------------------------------------------------------!
c     !   event variables:                                       !
c     !----------------------------------------------------------!
c     !   numevt ... number of evts  (=1)                        !
c     !   xlbevt ... momentum fraction of evt in pp cm           !
c     !   enlevt ... total energy of evt                         !
c     !   ev4evt ... zero degree energy of evt, eta>4            !
c     !   ev6evt ... zero degree energy of evt, eta>6            !
c     !   eplevt ... total eff energy of evt + proj.spect.energy !
c     !   alsevt ... log s                                       !
c     !   etlevt ... transverve energy of evt                    !
c     !   ezlevt ... neutral energy of evt                       !
c     !   mulevt ... multiplicity of evt                         !
c     !   mufevt ... forw multiplicity of evt (rin1<y<rin2)      !
c     !   mubevt ... backw multiplicity of evt (-rin2<y<-rin1)   !
c     !   mbfevt ... mubevt*mufevt                               !
c     !   mldevt ... lambda multiplicity of evt                  !
c     !   malevt ... alambda multiplicity of evt                 !
c     !   mksevt ... k_short multiplicity of evt                 !
c     !   mngevt ... neg ptl multiplicity of evt (full phas sp)  !
c     !   rlnevt ... lambda/neg multiplicity of evt              !
c     !   ranevt ... alambda/neg multiplicity of evt             !
c     !   rknevt ... k_short/neg multiplicity of evt             !
c     !   bimevt ... impact parameter                            !
c     !   kolevt ... number of collisions                        !
c     !   colevt ... effective number of collisions              !
c     !   egyevt ... string energy (for choic=str)               !
c     !   npjevt ... number of involved projectile nucleons      !
c     !   ntgevt ... number of involved target nucleons          !
c     !----------------------------------------------------------!
c     !   parameters:                                            !
c     !----------------------------------------------------------!
c     !   parp12 ... parameter par1,par2                         !
c     !   parp34 ... parameter par3,par4                         !
c     !   parpbi ... individual (pp) probability                 !
c     !   parrin ... rapidity interval                           !
c     !   parish ... ish                                         !
c     !----------------------------------------------------------!
c
c
c  table 2: possible values for normal
c
c     !----------------------------------------------------------!
c     ! normal should be a three digit integer ijk               !
c     ! where i,j,k provide the following normalization          !
c     !----------------------------------------------------------!
c     !   k=0   *1                                               !
c     !   k=1   / # of events                                    !
c     !   k=2   / # of valid events                              !
c     !   k=3   / # of counts (normalized to one)                !
c     !   k=4   / # of counts per bin                            !
c     !   k=5   / # of all events (incl trigger exclusions)      !
c     !   k=6   / integral(y) (normalized to one)                !
c     !----------------------------------------------------------!
c     !   j=0   *1                                               !
c     !   j=1   / bin size                                       !
c     !   j=2   / bin size * sigppi(prt-ncl)                     !
c     !----------------------------------------------------------!
c     !   i=0   *1                                               !
c     !   i=1   * x                                              !
c     !   i=2   / (2pi*x)                                        !
c     !   i=3   * <x> ,  x/<x>  (kno-representation)             !
c     !   i=4   scatter plot                                     !
c     !   i=5   / (4pi*x*sqrt(m**2+x**2))                        !
c     !----------------------------------------------------------!
c
c
c---------------------------------------------------------------------
c    ***** aaset commons
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      character*80 fnch,fnhi,fndt
      common/cfname/fnch,nfnch,fnhi,nfnhi,fndt,nfndt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paroi/bmaxim,bminim,phimax,phimin
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      common/paro3/uentro,sigppe,sigppd,asuhax(7),asuhay(7),omega
      common/paro4/grigam,grirsq,gridel,grislo,gricel
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paro8/smas,uumas,usmas,ssmas,ptq1,ptq2,ptq3
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/paroa/iopadi,q2soft
      common/parob/istore,iprmpt,iecho
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/parod/ioinfl,ioinct,iowidn,epsgc
      common/paroe/prob(99),icbac(99,2),icfor(99,2)
      common/cnsta/pi,pii,hquer,prom,piom,ainfin /cvsn/iversn,iverso 
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas /cptlu/nptlu /cnrclu/nrclu
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cmetro/keepr /cepsr/epsr,nepsr
      common/cinfla/keu,ked,kes,kec,keb,ket
      double precision seedi,seedj,seedc
      common /cseed/seedi,seedj,seedc  
      parameter (mxtau=4,mxvol=10,mxeps=16)
      common/cjintc/clust(mxtau,mxvol,mxeps)
      common/cjintd/volsum(mxtau),vo2sum(mxtau),nclsum(mxtau)
      common/ciutot/iutotc,iutote
      common/copen/nopen
      parameter (mxtrig=99,mxidco=99)
      character*6 xvaria,yvaria,trvari
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/p11/nrtrig,trmin(mxtrig),trmax(mxtrig),trvari(mxtrig)
      common/p12/nridco,idcode(mxidco)
      parameter(mxnody=200)
      common/p13/nrnody,nody(mxnody),ndecay 
      parameter(mxpri=200) 
      character*6 subpri
      common/p14/nrpri,subpri(mxpri),ishpri(mxpri) 
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      common/ckhist/khisto
      common/cctcor/nctcor/ccttim/ncttim
      parameter (mxnev=20,matau=6,nzeta=60,mxcoox=20)
      common/cdensi/kdensi(mxnev,matau,nzeta,mxcoox),tauv(matau)
c    ***** end aaset commons
      common/cyield/yield
      double precision val
      character*80 line,linex
      data nappl /0/
      j=-1

    1 call utword(line,i,j,1)

           if(line(i:j).eq.'application')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'application?'
      call utword(line,i,j,0)
      if(line(i:j).eq.'analysis')  iappl=0
      if(line(i:j).eq.'hadron')    iappl=1
      if(line(i:j).eq.'geometry')  iappl=2
      if(line(i:j).eq.'lepton')    iappl=3
      if(line(i:j).eq.'droplet')iappl=4
      nappl=nappl+1
      if(iappl.gt.0.and.nappl.gt.1)call aaset(1)
      if(iappl.eq.0.and.nappl.gt.1)call aaset(2)

           elseif(line(i:j).eq.'call')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'subroutine?'
      call utword(line,i,j,0)
      if(line(i:j).eq.'xchamt')call xchamt(0.,-1)
      if(line(i:j).eq.'xchfla')call xchfla(0,0,0,-1)
      if(line(i:j).eq.'xchico')call xchico(-1.)
      if(line(i:j).eq.'xdense')call xdense(2,jd,idu,xdu,ydu,rdu,odu,udu)       
      if(line(i:j).eq.'xqgcen')call xqgcen
      if(line(i:j).eq.'xqgcmt')call xqgcmt
      if(line(i:j).eq.'xqgcmu')call xqgcmu
      if(line(i:j).eq.'xqgcmx')call xqgcmx
      if(line(i:j).eq.'xqgcpt')call xqgcpt
      if(line(i:j).eq.'xqgcra')call xqgcra
      if(line(i:j).eq.'xqnben')call xqnben
      if(line(i:j).eq.'xqnbit')call xqnbit
      if(line(i:j).eq.'xqnbmu')call xqnbmu
      if(line(i:j).eq.'xqnbmz')call xqnbmz
      if(line(i:j).eq.'xtauex')call xtauex(-1)
      if(line(i:j).eq.'xtauin')call xtauin(-1)

           elseif(line(i:j).eq.'decayall')then
           
      nrnody=0    

           elseif(line(i:j).eq.'echo')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'on or off?'
      call utword(line,i,j,0)
      if(line(i:j).eq.'on')iecho=1
      if(line(i:j).eq.'off')iecho=0
      if(line(i:j).ne.'on'.and.line(i:j).ne.'off')
     *write(ifmt,'(a)')'invalid option; command ignored'

           elseif(line(i:j).eq.'fname')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'file-type file-name?' 
      call utword(line,i,j,0)
      linex=line
      ix=i
      jx=j
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'file-name?'
      call utword(line,i,j,0)
      if(linex(ix:jx).eq.'check')fnch(1:j-i+1)=line(i:j)
      if(linex(ix:jx).eq.'histo')fnhi(1:j-i+1)=line(i:j)
      if(linex(ix:jx).eq.'data') fndt(1:j-i+1)=line(i:j)
      if(linex(ix:jx).eq.'check')nfnch=j-i+1
      if(linex(ix:jx).eq.'histo')nfnhi=j-i+1
      if(linex(ix:jx).eq.'data') nfndt=j-i+1

           elseif(line(i:j).eq.'help')then

      write(ifmt,'(77a1)')('-',l=1,77)
      write(ifmt,'(a)')'possible commands:'
      write(ifmt,'(77a1)')('-',l=1,77)
      write(ifmt,'(a,a/a,a)')
     *'application call decayall echo fname help idcode input nodecay '
     *,'print prompt'
     *,'return run set setm show showa stop string trigger write '
     *,' writearray' 
      write(ifmt,'(77a1)')('-',l=1,77)

           elseif(line(i:j).eq.'idcode')then

      if(nridco+1.gt.mxidco)then
      write(ifmt,'(a)')'too many ids; command ignored'
      j=80
      i=j+1
      goto1
      endif
      nridco=nridco+1
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'idcode?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      idcode(nridco)=val

           elseif(line(i:j).eq.'input')then

      if(nopen.ne.0)close(ifop)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'input file?'
      call utword(line,i,j,0)
      open(unit=ifop,file=line(i:j),status='old')
      nopen=1
      if(iprmpt.eq.1)iprmpt=-1

           elseif(line(i:j).eq.'nodecay')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'id?'
      call utword(line,i,j,0)
      if(nrnody.ge.mxnody)then
      write(ifmt,'(a)')'too many nodecay commands; command ignored'
      j=80
      i=j+1
      goto1
      endif
      nrnody=nrnody+1 
      read(line(i:j),*)val
      nody(nrnody)=nint(val)

           elseif(line(i:j).eq.'print')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'subroutine?'
      call utword(line,i,j,0)
       if(line(i:j).eq.'*')then
      nrpri=0
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'level?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      ishx=nint(val)
      if(ishx.eq.0)ish=0
      if(ishx.gt.0)ish=ishx+89
       else
      nrpri=nrpri+1    
      subpri(nrpri)='      '  
      subpri(nrpri)(1:j-i+1)=line(i:j)  
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'level?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      ishx=nint(val)
      if(ishx.eq.0)ishpri(nrpri)=0
      if(ishx.gt.0)ishpri(nrpri)=ishx+89
       endif
     
           elseif(line(i:j).eq.'prompt')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'on or off or auto?'
      call utword(line,i,j,0)
      if(line(i:j).eq.'on')iprmpt=2
      if(line(i:j).eq.'off')iprmpt=-2
      if(line(i:j).eq.'auto'.and.nopen.eq.0)iprmpt=1
      if(line(i:j).eq.'auto'.and.nopen.eq.1)iprmpt=-1
      if(line(i:j).ne.'on'.and.line(i:j).ne.'off'
     *.and.line(i:j).ne.'auto')
     *write(ifmt,'(a)')'invalid option; command ignored'

           elseif(line(i:j).eq.'return')then

      close(ifop)
      nopen=0
      if(iprmpt.eq.-1)iprmpt=1

           elseif(line(i:j).eq.'run')then

      return

           elseif(line(i:j).eq.'set')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'p/o-name p/o-value?'
      call utword(line,i,j,0)
      linex=line
      ix=i
      jx=j
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'p/o-value?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
c       general
      if(linex(ix:jx).eq.'iversn')iversn=nint(val)
      if(linex(ix:jx).eq.'iappl' )iappl=nint(val)
      if(linex(ix:jx).eq.'nevent')nevent=nint(val)
c       printout options
      if(linex(ix:jx).eq.'iprmpt')iprmpt=nint(val)
      if(linex(ix:jx).eq.  'ish' )ish=nint(val)
      if(linex(ix:jx).eq.'ishsub')ishsub=nint(val)
      if(linex(ix:jx).eq.'irandm')irandm=nint(val)
      if(linex(ix:jx).eq.'irewch')irewch=nint(val)
      if(linex(ix:jx).eq.'ishevt')ishevt=nint(val)
      if(linex(ix:jx).eq.'ipagi ')ipagi =nint(val)
      if(linex(ix:jx).eq.'iecho ')iecho =nint(val)
      if(linex(ix:jx).eq.'modsho')modsho=nint(val)
      if(linex(ix:jx).eq.'idensi')idensi=nint(val)
c       fragmentation and decay
      if(linex(ix:jx).eq.'pud   ')pud   =val
      if(linex(ix:jx).eq.'pdiqua')pdiqua=val
      if(linex(ix:jx).eq.'pspinl')pspinl=val
      if(linex(ix:jx).eq.'pspinh')pspinh=val
      if(linex(ix:jx).eq.'pispn ')pispn =val
      if(linex(ix:jx).eq.'ioptf ')ioptf =nint(val)
      if(linex(ix:jx).eq.'ptf   ')ptf   =val
      if(linex(ix:jx).eq.'ptmx  ')ptmx  =val
      if(linex(ix:jx).eq.'tensn ')tensn =val
      if(linex(ix:jx).eq.'parea ')parea =val
      if(linex(ix:jx).eq.'delrem')delrem=val
      if(linex(ix:jx).eq.'delrex')delrex=val
      if(linex(ix:jx).eq.'kutdiq')kutdiq=nint(val)
      if(linex(ix:jx).eq.'iopbrk')iopbrk=nint(val)
      if(linex(ix:jx).eq.'smas  ')smas  =val
      if(linex(ix:jx).eq.'uumas ')uumas =val
      if(linex(ix:jx).eq.'usmas ')usmas =val
      if(linex(ix:jx).eq.'ssmas ')ssmas =val
      if(linex(ix:jx).eq.'ndecay')ndecay=nint(val)
      if(linex(ix:jx).eq.'maxres')maxres=nint(val)
      if(linex(ix:jx).eq.'rhophi')rhophi=val
c       lepton-nucleon and e+e-
      if(linex(ix:jx).eq. 'engy' )engy=nint(val)
      if(linex(ix:jx).eq.'elepti')elepti=val
      if(linex(ix:jx).eq.'elepto')elepto=val
      if(linex(ix:jx).eq.'angmue')angmue=val
c       hadron-hadron
      if(linex(ix:jx).eq. 'pnll' )pnll=val
      if(linex(ix:jx).eq.'idproj')idproj=nint(val)
      if(linex(ix:jx).eq.'idtarg')idtarg=nint(val)
      if(linex(ix:jx).eq.'ioptq ')ioptq =nint(val)
      if(linex(ix:jx).eq.'ptq1  ')ptq1  =val
      if(linex(ix:jx).eq.'ptq2  ')ptq2  =val
      if(linex(ix:jx).eq.'ptq3  ')ptq3  =val
      if(linex(ix:jx).eq.'phard ')phard =val
      if(linex(ix:jx).eq.'pth   ')pth   =val
      if(linex(ix:jx).eq.'rstras')rstras=val
      if(linex(ix:jx).eq.'wproj ')wproj =val
      if(linex(ix:jx).eq.'wtarg ')wtarg =val
      if(linex(ix:jx).eq.'cutmsq')cutmsq=val
      if(linex(ix:jx).eq.'cutmss')cutmss=val
      if(linex(ix:jx).eq.'pvalen')pvalen=val
      if(linex(ix:jx).eq.'delmss')delmss=val
      if(linex(ix:jx).eq.'neqmn ')neqmn =nint(val)
      if(linex(ix:jx).eq.'neqmx ')neqmx =nint(val)
      if(linex(ix:jx).eq.'iaqu  ')iaqu  =nint(val)
      if(linex(ix:jx).eq.'prosea')prosea=val
      if(linex(ix:jx).eq.'idpm  ')idpm  =nint(val)
      if(linex(ix:jx).eq.'iopadi')iopadi=nint(val)
      if(linex(ix:jx).eq.'q2soft')q2soft=nint(val)
c       gribov-regge
      if(linex(ix:jx).eq.'grigam')grigam=val
      if(linex(ix:jx).eq.'grirsq')grirsq=val
      if(linex(ix:jx).eq.'gridel')gridel=val
      if(linex(ix:jx).eq.'grislo')grislo=val
      if(linex(ix:jx).eq.'gricel')gricel=val
      if(linex(ix:jx).eq.'sigppi')sigppi=val
c       nucleus-nucleus
      if(linex(ix:jx).eq.'laproj')laproj=nint(val)
      if(linex(ix:jx).eq.'maproj')maproj=nint(val)
      if(linex(ix:jx).eq.'latarg')latarg=nint(val)
      if(linex(ix:jx).eq.'matarg')matarg=nint(val)
      if(linex(ix:jx).eq.'core  ')core  =val
      if(linex(ix:jx).eq.'ncolmx')ncolmx=nint(val)
      if(linex(ix:jx).eq.'fctrmx')fctrmx=val
      if(linex(ix:jx).eq.'ko1ko2')ko1ko2=nint(val)
      if(linex(ix:jx).eq.'bmaxim')bmaxim=val
      if(linex(ix:jx).eq.'bminim')bminim=val
      if(linex(ix:jx).eq.'phimax')phimax=val
      if(linex(ix:jx).eq.'phimin')phimin=val
c       droplet formation
      if(linex(ix:jx).eq.'taurea')taurea=val
      if(linex(ix:jx).eq.'sigmes')sigmes=val
      if(linex(ix:jx).eq.'sigbar')sigbar=val
      if(linex(ix:jx).eq.'rinmes')rinmes=val
      if(linex(ix:jx).eq.'rinbar')rinbar=val
      if(linex(ix:jx).eq.'epscri')epscri=val
      if(linex(ix:jx).eq.'amsiac')amsiac=val
      if(linex(ix:jx).eq.'iojint')iojint=nint(val)
      if(linex(ix:jx).eq.'amprif')amprif=val
      if(linex(ix:jx).eq.'delvol')delvol=val
      if(linex(ix:jx).eq.'deleps')deleps=val
      if(linex(ix:jx).eq.'taumin')taumin=val
      if(linex(ix:jx).eq.'deltau')deltau=val
      if(linex(ix:jx).eq.'factau')factau=val
      if(linex(ix:jx).eq.'numtau')numtau=nint(val)
      if(linex(ix:jx).eq.'dlzeta')dlzeta=val
      if(linex(ix:jx).eq.'etafac')etafac=val
c       droplet decay
      if(linex(ix:jx).eq.'ioclud')ioclud=val
      if(linex(ix:jx).eq.'corlen')corlen=val
      if(linex(ix:jx).eq.'dezzer')dezzer=val
      if(linex(ix:jx).eq.'amuseg')amuseg=val
      if(linex(ix:jx).eq.'bag4rt')bag4rt=val
      if(linex(ix:jx).eq.'iopent')iopent=nint(val)
      if(linex(ix:jx).eq.'uentro')uentro=val
      if(linex(ix:jx).eq.'kentro')kentro=nint(val)
      if(linex(ix:jx).eq.'taunll')taunll=val
      if(linex(ix:jx).eq.'omega' )omega=val
      if(linex(ix:jx).eq.'ientro')ientro=nint(val)
c       droplet specification
      if(linex(ix:jx).eq. 'keu'  )keu=nint(val)
      if(linex(ix:jx).eq. 'ked'  )ked=nint(val)
      if(linex(ix:jx).eq. 'kes'  )kes=nint(val)
      if(linex(ix:jx).eq. 'kec'  )kec=nint(val)
      if(linex(ix:jx).eq. 'keb'  )keb=nint(val)
      if(linex(ix:jx).eq. 'ket'  )ket=nint(val)
      if(linex(ix:jx).eq. 'tecm' )tecm=val
      if(linex(ix:jx).eq. 'volu' )volu=val
c       metropolis
      if(linex(ix:jx).eq.'iospec')iospec=nint(val)
      if(linex(ix:jx).eq.'iocova')iocova=nint(val)
      if(linex(ix:jx).eq.'iopair')iopair=nint(val)
      if(linex(ix:jx).eq.'iozero')iozero=nint(val)
      if(linex(ix:jx).eq.'ioflac')ioflac=nint(val)
      if(linex(ix:jx).eq.'iostat')iostat=nint(val)
      if(linex(ix:jx).eq.'ioinco')ioinco=nint(val)
      if(linex(ix:jx).eq.'iograc')iograc=nint(val)
      if(linex(ix:jx).eq.'epsgc' )epsgc=nint(val)
      if(linex(ix:jx).eq.'iocite')iocite=nint(val)
      if(linex(ix:jx).eq.'ioceau')ioceau=nint(val)
      if(linex(ix:jx).eq.'iociau')iociau=nint(val)
      if(linex(ix:jx).eq.'ioinct')ioinct=nint(val)
      if(linex(ix:jx).eq.'ioinfl')ioinfl=nint(val)
      if(linex(ix:jx).eq.'iowidn')iowidn=nint(val)
      if(linex(ix:jx).eq.'ionlat')ionlat=nint(val)
      if(linex(ix:jx).eq.'iomom')iomom=nint(val)
      if(linex(ix:jx).eq.'ioobsv')ioobsv=nint(val)
      if(linex(ix:jx).eq.'iosngl')iosngl=nint(val)
      if(linex(ix:jx).eq.'iorejz')iorejz=nint(val)
      if(linex(ix:jx).eq.'iompar')iompar=nint(val)
      if(linex(ix:jx).eq.'iozinc')iozinc=nint(val)
      if(linex(ix:jx).eq.'iozevt')iozevt=nint(val)
      if(linex(ix:jx).eq. 'nadd' )nadd=nint(val)
      if(linex(ix:jx).eq.'iterma')iterma=nint(val)
      if(linex(ix:jx).eq.'itermx')stop'STOP: set iterma, not itermx'
      if(linex(ix:jx).eq.'iterpr')iterpr=nint(val)
      if(linex(ix:jx).eq.'iterpl')iterpl=nint(val)
      if(linex(ix:jx).eq.'iternc')iternc=nint(val)
      if(linex(ix:jx).eq.'epsr'  )epsr=val
      if(linex(ix:jx).eq.'keepr' )keepr=nint(val)
c       strangelets
      if(linex(ix:jx).eq.'iopenu')iopenu=nint(val)
      if(linex(ix:jx).eq.'themas')themas=val
c       jpsi
      if(linex(ix:jx).eq.'jpsi  ')jpsi  =nint(val)
      if(linex(ix:jx).eq.'jpsifi')jpsifi=nint(val)
      if(linex(ix:jx).eq.'sigj  ')sigj  =val
      if(linex(ix:jx).eq.'taumx ')taumx =val
      if(linex(ix:jx).eq.'nsttau')nsttau=nint(val)
      if(linex(ix:jx).eq.'ijphis')ijphis=nint(val)
c       analysis: intermittency, space-time, droplets, formation time
      if(linex(ix:jx).eq.'ymximi')ymximi=val
      if(linex(ix:jx).eq.'imihis')imihis=nint(val)
      if(linex(ix:jx).eq.'isphis')isphis=nint(val)
      if(linex(ix:jx).eq.'ispall')ispall=nint(val)
      if(linex(ix:jx).eq.'wtmini')wtmini=val
      if(linex(ix:jx).eq.'wtstep')wtstep=val
      if(linex(ix:jx).eq.'iwcent')iwcent=nint(val)
      if(linex(ix:jx).eq.'iclhis')iclhis=nint(val)
      if(linex(ix:jx).eq.'iwtime')iwtime=nint(val)
      if(linex(ix:jx).eq.'wtimet')wtimet=val
      if(linex(ix:jx).eq.'wtimei')wtimei=val
      if(linex(ix:jx).eq.'wtimea')wtimea=val
c       other
      if(linex(ix:jx).eq.'gaumx ')gaumx =val
      if(linex(ix:jx).eq.'nclean')nclean=nint(val)
      if(linex(ix:jx).eq.'istore')istore=nint(val)
      if(linex(ix:jx).eq.'labsys')labsys=nint(val)
      if(linex(ix:jx).eq.'irescl')irescl=nint(val)
      if(linex(ix:jx).eq.'ifrade')ifrade=nint(val)
      if(linex(ix:jx).eq.'ntrymx')ntrymx=nint(val)
      if(linex(ix:jx).eq.'istmax')istmax=nint(val)
      if(linex(ix:jx).eq.'seedi ')seedi =val
      if(linex(ix:jx).eq.'seedj ')seedj =val
      if(linex(ix:jx).eq.'xvaria')xvaria=line(i:j)
      if(linex(ix:jx).eq.'yvaria')yvaria=line(i:j)
      if(linex(ix:jx).eq.'normal')normal=nint(val)
      if(linex(ix:jx).eq.'xminim')xminim=val
      if(linex(ix:jx).eq.'xmaxim')xmaxim=val
      if(linex(ix:jx).eq.'nrbins')nrbins=nint(val)
      if(linex(ix:jx).eq.'hisfac')hisfac=val
      if(linex(ix:jx).eq.'xpar1' )xpar1=val
      if(linex(ix:jx).eq.'xpar2' )xpar2=val
      if(linex(ix:jx).eq.'xpar3' )xpar3=val
      if(linex(ix:jx).eq.'xpar4' )xpar4=val
      if(linex(ix:jx).eq.'xpar5' )xpar5=val
      if(linex(ix:jx).eq.'xpar6' )xpar6=val
      if(linex(ix:jx).eq.'xpar7' )xpar7=val
      if(linex(ix:jx).eq.'xpar8' )xpar8=val

      if(corlen.gt.1)then
      write(ifmt,*)'for large "corlen",  many "short fusion droplets"'
      write(ifmt,*)'show up. these droplets are presently not treated'
      write(ifmt,*)'properly; there is an artificial  quark-antiquark' 
      write(ifmt,*)'production.' 
      write(ifmt,*)'so: before using corlen>1, this must be fixed!!!!'
      stop
      endif

           elseif(line(i:j).eq.'setm')then       

           if(iappl.eq.1.or.iappl.eq.2)then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'nevent modsho pnll laproj maproj latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      nevent=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'modsho pnll laproj maproj latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      modsho=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'pnll laproj maproj latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      pnll=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'laproj maproj latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      laproj=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'maproj latarg matarg?'      
      call utword(line,i,j,0)
      read(line(i:j),*)val
      maproj=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      latarg=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      matarg=nint(val)

           elseif(iappl.eq.3)then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'nevent modsho engy latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      nevent=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'modsho engy latarg matarg?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      modsho=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'engy latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      engy=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'latarg matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      latarg=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'matarg?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      matarg=nint(val)
      ns=0
      icinpu=1

           elseif(iappl.eq.4)then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'tecm volu iterma iterpr iterpl iternc ish?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      tecm=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'volu iterma iterpr iterpl iternc ish?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      volu=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'iterma iterpr iterpl iternc ish?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      iterma=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'iterpr iterpl iternc ish?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      iterpr=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'iterpl iternc ish?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      iterpl=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'iternc ish?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      iternc=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'ish?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      ish=nint(val)

           elseif(iappl.eq.0)then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'xvaria yvaria normal xminim xmaxim nrbins?'
      call utword(line,i,j,0)
      xvaria=line(i:i+5)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'yvaria normal xminim xmaxim nrbins?'       
      call utword(line,i,j,0)
      yvaria=line(i:i+5)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'normal xminim xmaxim nrbins?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      normal=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'xminim xmaxim nrbins?'       
      call utword(line,i,j,0)
      read(line(i:j),*)val
      xminim=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'xmaxim nrbins?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      xmaxim=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)write(ifmt,'(a)')'nrbins?' 
      call utword(line,i,j,0)
      read(line(i:j),*)val
      nrbins=nint(val)

           endif

           elseif(line(i:j).eq.'show')then

      write(ifmt,'(77a1)')('-',l=1,77)
      write(ifmt,'(a)')
     *' iversn  iappl nevent'
      write(ifmt,'(3i7)')
     *iversn  ,iappl ,nevent
      write(ifmt,'(a,a)')
     *' iprmpt    ish ishsub irandm irewch',
     *' ishevt  ipagi  iecho modsho idensi'
      write(ifmt,'(10i7)')
     *iprmpt  ,ish ,ishsub ,irandm, irewch,
     *ishevt  ,ipagi    ,iecho ,modsho,idensi
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a,a)')
     *'    pud pdiqua pspinl pspinh  pispn',
     *'  ioptf    ptf   ptmx  tensn  parea delrem' 
      write(ifmt,'(5f7.3,i7,f7.3,2f7.1,2f7.3)')
     *    pud ,pdiqua ,pspinl ,pspinh  ,pispn,
     *  ioptf    ,ptf   ,ptmx  ,tensn  ,parea  ,delrem
      write(ifmt,'(a,a)')
     *' delrex kutdiq iopbrk   smas  uumas  usmas',
     *'  ssmas ndecay maxres'
      write(ifmt,'(f7.3,2i7,4f7.3,2i7)')
     * delrex ,kutdiq ,iopbrk   ,smas  ,uumas  ,usmas,
     *  ssmas ,ndecay ,maxres 
      write(ifmt,'(a)')' rhophi'
      write(ifmt,'(f7.3)')rhophi
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a)')
     *'   engy elepti elepto angmue'
      write(ifmt,'(4f7.2)')
     *   engy ,elepti ,elepto ,angmue
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a,a)')
     *'   pnll idproj idtarg  ioptq   ptq1',
     *'   ptq2   ptq3  phard    pth rstras  wproj'
      write(ifmt,'(f7.1,3i7,7f7.3)')
     *   pnll ,idproj ,idtarg  ,ioptq   ,ptq1,
     *   ptq2   ,ptq3  ,phard    ,pth ,rstras  ,wproj
      write(ifmt,'(a,a)')
     *'  wtarg cutmsq cutmss pvalen delmss',
     *'  neqmn  neqmx   iaqu prosea   idpm'
      write(ifmt,'(5f7.3,3i7,f7.3,i7)')
     *  wtarg ,cutmsq ,cutmss ,pvalen ,delmss,
     *  neqmn ,neqmx   ,iaqu ,prosea   ,idpm
      write(ifmt,'(a)')
     *' iopadi q2soft'
      write(ifmt,'(i7,f7.3)')
     * iopadi, q2soft
      write(ifmt,'(a,a)')
     *' grigam grirsq gridel grislo gricel',
     *' sigppi'
      write(ifmt,'(6f7.3,i7)')
     * grigam ,grirsq ,gridel ,grislo ,gricel,
     * sigppi
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a,a)')
     *' laproj maproj latarg matarg   core',
     *' ncolmx fctrmx ko1ko2 bmaxim bminim'
      write(ifmt,'(4i7,f7.3,i7,f7.1,i7,2f7.1)')
     * laproj ,maproj ,latarg ,matarg    ,core,
     * ncolmx ,fctrmx ,ko1ko2 ,bmaxim ,bminim
      write(ifmt,'(a)')' phimax phimin'
      write(ifmt,'(2f7.3)')phimax phimin
      write(ifmt,'(a,a)')
     *' taurea sigmes sigbar rinmes',
     *' rinbar epscri amsiac iojint amprif delvol'
      write(ifmt,'(7f7.3,i7,2f7.2)')
     * taurea ,sigmes ,sigbar ,rinmes,
     * rinbar ,epscri ,amsiac ,iojint ,amprif ,delvol 
      write(ifmt,'(a)')
     *' deleps taumin deltau factau numtau etafac dlzeta'
      write(ifmt,'(4f7.3,i7,2f7.3)')
     * deleps ,taumin ,deltau ,factau ,numtau ,etafac ,dlzeta
      write(ifmt,'(a,a)')
     *' ioclud corlen dezzer amuseg bag4rt iopent uentro',
     *' kentro taunll  omega ientro'
      write(ifmt,'(i7,f7.3,f7.5,2f7.3,i7,f7.3,i7,2f7.3,i7)')
     * ioclud, corlen ,dezzer, amuseg ,bag4rt ,iopent ,uentro,
     * kentro ,taunll  ,omega ,ientro
      write(ifmt,'(a)')
     *'   tecm   volu    keu    ked    kes    kec    keb    ket'
      write(ifmt,'(2f7.1,6i7)')
     *tecm,volu,keu,ked,kes,kec,keb,ket
      write(ifmt,'(a,a)')
     *' iterma iterpr iterpl iternc iospec iocova iopair iozero ioflac' 
     *,'  iomom'
      write(ifmt,'(10i7)')
     *iterma,iterpr,iterpl,iternc,iospec,iocova,iopair,iozero,ioflac
     *,iomom
      write(ifmt,'(a)')
     *' iograc iocite ioceau iociau   nadd   epsr  keepr'       
      write(ifmt,'(5i7,f7.1,i7)')
     *iograc,iocite,ioceau,iociau,nadd,epsr,keepr
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a)')
     *' iopenu themas   jpsi jpsifi   sigj  taumx nsttau ijphis'      
      write(ifmt,'(i7,1x,f6.5,2i7,f7.3,f7.1,2i7)')
     * iopenu ,themas ,jpsi ,jpsifi  ,sigj  ,taumx ,nsttau ,ijphis
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a,a)')
     *' ymximi imihis isphis ispall wtmini wtstep ',
     *'iwcent iclhis'
      write(ifmt,'(f7.1,3i7,2f7.2,3i7)')
     * ymximi ,imihis ,isphis ,ispall ,wtmini ,wtstep ,iwcent ,iclhis
      write(ifmt,'(a)')
     *'iwtime wtimet wtimei wtimea'
      write(ifmt,'(i7,3f7.1)')
     *iwtime,wtimet,wtimei,wtimea
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a,a)')
     *'  gaumx nclean istore labsys irescl ifrade',
     *' ntrymx istmax                seedi               seedj'
      write(ifmt,'(f7.2,7i7)')
     * gaumx,nclean,istore,labsys,irescl,ifrade,ntrymx,istmax
      write(ifmt,'(f21.4,f21.4)')
     *seedi,seedj
      write(ifmt,'(77a1)')('-',l=1,77)

      write(ifmt,'(a,a)')' xvaria yvaria normal xminim xmaxim',
     *' nrbins hisfac'       
      write(ifmt,'(1x,a6,1x,a6,i7,2f7.1,i7,f7.3)')
     *xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      if(nrtrig.gt.0.or.nridco.gt.0)then
      write(ifmt,'(a,a)')'     nr trvari  trmin  trmax',
     *'            nr idcode'
      do nr=1,max(nrtrig,nridco)
      if(nr.le.min(nrtrig,nridco))then
      write(ifmt,'(i7,1x,a6,f7.3,f7.3,7x,i7,i7)')
     *nr,trvari(nr),trmin(nr),trmax(nr),nr,idcode(nr)
      elseif(nr.le.nrtrig)then
      write(ifmt,'(i7,1x,a6,f7.3,f7.3)')
     *nr,trvari(nr),trmin(nr),trmax(nr)
      elseif(nr.le.nridco)then
      write(ifmt,'(28x,7x,i7,i7)')
     *nr,idcode(nr)
      endif
      enddo
      endif
      write(ifmt,'(77a1)')('-',l=1,77)

           elseif(line(i:j).eq.'showa')then

      if(iappl.eq.1)then
      write(ifmt,'(77a1)')('-',l=1,77)
      write(ifmt,'(a)')
     *' nevent modsho   pnll laproj maproj latarg matarg idproj idtarg'
      write(ifmt,'(2i7,f7.1,6i7)')
     *nevent,modsho,pnll,laproj,maproj,latarg,matarg,idproj,idtarg
      write(ifmt,'(77a1)')('-',l=1,77)
      endif

      if(iappl.eq.3)then
      write(ifmt,'(77a1)')('-',l=1,77)
      write(ifmt,'(a)')
     *' nevent modsho   engy latarg matarg'
      write(ifmt,'(2i7,f7.1,2i7)')
     *nevent,modsho,engy,latarg,matarg
      if(ns.gt.0)then
      write(ifmt,'(a)')
     *'  prob icbac1 icbac2 icfor1 icfor2'
      do n=1,ns
      write(ifmt,'(f7.2,4i7)')
     *prob(n) ,icbac(n,1) ,icbac(n,2) ,icfor(n,1) ,icfor(n,2)
      enddo
      endif
      write(ifmt,'(77a1)')('-',l=1,77)
      endif

      if(iappl.eq.4)then
      write(ifmt,'(77a1)')('-',l=1,77)
      write(ifmt,'(a)')
     *'   tecm   volu    keu    ked    kes    kec    keb    ket'
      write(ifmt,'(2f7.1,6i7)')
     *tecm,volu,keu,ked,kes,kec,keb,ket
      if(ioclud.eq.2)then
      write(ifmt,'(a,a)')
     *' iterma iterpr iterpl iternc iospec iocova iopair iozero ioflac' 
     *,'  iomom'
      write(ifmt,'(10i7)')
     *iterma,iterpr,iterpl,iternc,iospec,iocova,iopair,iozero,ioflac
     *,iomom
      endif
      write(ifmt,'(77a1)')('-',l=1,77)
      endif

      if(iappl.eq.0)then
      write(ifmt,'(77a1)')('-',l=1,77)
      write(ifmt,'(a,a)')' xvaria yvaria normal xminim xmaxim',
     *' nrbins hisfac'       
      write(ifmt,'(1x,a6,1x,a6,i7,2f7.1,i7,f7.3)')
     *xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      if(nrtrig.gt.0.or.nridco.gt.0)then
      write(ifmt,'(a,a)')'     nr trvari  trmin  trmax',
     *'            nr idcode'
      do nr=1,max(nrtrig,nridco)
      if(nr.le.min(nrtrig,nridco))then
      write(ifmt,'(i7,1x,a6,f7.3,f7.3,7x,i7,i7)')
     *nr,trvari(nr),trmin(nr),trmax(nr),nr,idcode(nr)
      elseif(nr.le.nrtrig)then
      write(ifmt,'(i7,1x,a6,f7.3,f7.3)')
     *nr,trvari(nr),trmin(nr),trmax(nr)
      elseif(nr.le.nridco)then
      write(ifmt,'(28x,7x,i7,i7)')
     *nr,idcode(nr)
      endif
      enddo
      endif
      write(ifmt,'(77a1)')('-',l=1,77)
      endif

           elseif(line(i:j).eq.'stop')then

      stop

           elseif(line(i:j).eq.'string')then

      ns=ns+1
      icinpu=0
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'string: prob icbac1 icbac2 icfor1 icfor2?' 
      call utword(line,i,j,0)
      read(line(i:j),*)val
      prob(ns)=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'string: icbac1 icbac2 icfor1 icfor2?' 
      call utword(line,i,j,0)
      read(line(i:j),*)val
      icbac(ns,1)=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'string: icbac2 icfor1 icfor2?' 
      call utword(line,i,j,0)
      read(line(i:j),*)val
      icbac(ns,2)=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'string: icfor1 icfor2?' 
      call utword(line,i,j,0)
      read(line(i:j),*)val
      icfor(ns,1)=nint(val)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'string: icfor2?' 
      call utword(line,i,j,0)
      read(line(i:j),*)val
      icfor(ns,2)=nint(val)

           elseif(line(i:j).eq.'trigger')then

      if(nrtrig+1.gt.mxtrig)then
      write(ifmt,'(a)')'too many triggers; command ignored'
      j=80
      i=j+1
      goto1
      endif
      nrtrig=nrtrig+1
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'trvari trmin trmax?'
      call utword(line,i,j,0)
      trvari(nrtrig)=line(i:i+5)
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'trmin trmax?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      trmin(nrtrig)=val
      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'trmax?'
      call utword(line,i,j,0)
      read(line(i:j),*)val
      trmax(nrtrig)=val

           elseif(line(i:j).eq.'write')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'text?' 
      call utword(line,i,j,0)
        if(j-6.ge.i)then
      do l=i,j-6
      if(line(l:l+6).eq.'$iversn')
     *write(line(l:l+6),'(f4.2,3x)')iversn/100.
      enddo
        endif
        if(j-5.ge.i)then
      do l=i,j-5
      if(line(l:l+5).eq.'$yield')then
      if(yield.lt.1.0)then
      write(line(l:l+5),'(f5.3,1x)')yield
      elseif(yield.lt.100.)then
      write(line(l:l+5),'(f5.2,1x)')yield
      elseif(yield.lt.1000.)then
      write(line(l:l+5),'(f5.1,1x)')yield
      elseif(yield.lt.10000.)then
      write(line(l:l+5),'(f6.1)')yield
      else
      write(line(l:l+5),'(i6)')nint(yield)
      endif
      endif
      enddo
        endif
      write(ifhi,'(a)')line(i:j)

           elseif(line(i:j).eq.'writearray')then

      call utworn(line,j,ne)
      if(ne.eq.0.and.iprmpt.gt.0)
     *write(ifmt,'(a)')'# of columns?' 
      call utword(line,i,j,0)
      read(line(i:j),*)val
      nco=nint(val)
      write(ifhi,'(a,i3)')'array',nco
      do k=1,nrbins
      if(nco.eq.2)write(ifhi,'(3e12.4)')ar(k,1),ar(k,3)
      if(nco.eq.3)write(ifhi,'(3e12.4)')ar(k,1),ar(k,3),ar(k,4)
      enddo
      write(ifhi,'(a)')'  endarray'

           else

      write(ifmt,'(a,a,a)')'command "',line(i:j),'" not found'
      j=80

           endif

      i=j+1
      goto1
    
      end

c-----------------------------------------------------------------------
      subroutine aseed
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/parob/istore,iprmpt,iecho
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      double precision seedi,seedj,seedf,seedc
      common /cseed/seedi,seedj,seedc 
      call ranfgt(seedf)
      if(mod(nrevt,modsho).eq.0)
     *write(ifmt,'(a,i10,10x,a,d26.15)')'nrevt:',nrevt,'seedf:',seedf
      seedc=seedf
      return
      end

c-----------------------------------------------------------------------
      subroutine astat
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      character*80 fnch,fnhi,fndt
      common/cfname/fnch,nfnch,fnhi,nfnhi,fndt,nfndt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/parob/istore,iprmpt,iecho
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cptlu/nptlu

      if(iprmpt.le.0)goto1000

      write(ifmt,'(77a1)')('-',i=1,77)
      write(ifmt,'(a)')'statistics'
      write(ifmt,'(77a1)')('-',i=1,77)
      write(ifmt,'(a,i6,a)')'nr of messages:',imsg
     *,'    (written to file '//fnch(1:nfnch)//')'      
      write(ifmt,'(a,i8)')'maximum nptl:',nptlu
      if(inoiac.gt.0)write(ifmt,'(a,6x,i8)')'inoiac:',inoiac
      if(ilamas.gt.0)write(ifmt,'(a,6x,i8)')'ilamas:',ilamas
      write(ifmt,'(77a1)')('-',i=1,77)
 
1000  continue
      return
      end

c-----------------------------------------------------------------------
      subroutine atitle
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/parob/istore,iprmpt,iecho
      common/cvsn/iversn,iverso
      if(iprmpt.gt.0)write(ifmt,'(77a1/a1,23x,a,f5.2,7x,a,23x,a1/77a1)')
     *('#',l=1,78),'VENUS',float(iversn)/100,'Klaus WERNER',('#',l=1,78)    
      if(iversn.eq.iverso)return
      if(iprmpt.gt.0)write(ifmt,'(a1,2x,a,f5.2,a,a,1x,a1
     *               /a1,11x,a,22x,a1/a1,11x,a,21x,a1/77a1)')
     *'#','WARNING: VENUS',float(iversn)/100
     *               ,' is highly preliminary,'
     *               ,' to be used ONLY for technical','#'
     *,'#','studies, in collaboration with the author.','#'
     *,'#','physics results have no meaning whatsoever.'
     *,('#',l=1,78)    
      return
      end

c-----------------------------------------------------------------------
      subroutine avenus
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      double precision seedi,seedj,seedc
      common /cseed/seedi,seedj,seedc

      ish0=ish
      ishs0=ishsub
      if(ishevt.ne.0.and.nrevt+1.ne.ishevt)ish=0
      if(ishevt.ne.0.and.nrevt+1.ne.ishevt)ishsub=0
      if(ish.eq.17)write(ifch,*)'avenus (entry)'
      ipage=0
      call utpage
      if(ish.ge.91)
     *write(ifch,113)('#',l=1,79),ipage,nrevt+1,seedc,('#',l=1,79)
113   format(1x,79a1/1x,i5,'.page            '
     *,5x,'evt:',i5,5x,'seedc:',d25.15/1x,79a1)

      bimevt=-1
      ntevt0=ntevt
3     ntevt=ntevt0
2     ntevt=ntevt+1

      if(iappl.eq.1.or.iappl.eq.2)call nucoll

      if(iappl.eq.3)then
      call jetgen(ier)
      if(ier.eq.1)goto3
      endif


      if(iappl.eq.1.or.iappl.eq.3)then
      call jfrade(ier)
      if(ier.eq.1)goto3
      endif

      if((iappl.eq.1.or.iappl.eq.2).and.nevt.eq.0)then
      bimevt=-1
      goto2
      endif

      if(jerr.gt.0)
     *call utstop('avenus: jerr > 0                        ')

      nrevt=nrevt+1
 
1000  continue
      if(ish.eq.17)write(ifch,*)'avenus (exit)'
      ish=ish0
      ishsub=ishs0
      return
      end

c-----------------------------------------------------------------------
      subroutine hahabs(proj,targ,iap,iat,iskip,irethh)
c     performs a basic (one color exchange) hadron-hadron collision
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      common/ckol/kol
      parameter (nsi=6,nsix=100,nvmax=10)
      parameter (nha=nsix+2)
      real proj(nsi,nha),targ(nsi,nha)
     *,strap(nsi,nsix+1),strat(nsi,nsix+1),str0(nsi,nsix+1)
     *,strlp(nsi,nsix+1),strlt(nsi,nsix+1)
      parameter (mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      real silp(nsi),sixp(nsi,nsix),
     *silt(nsi),sixt(nsi,nsix)
      common /cnew/kotri,newico,newcol
     */clead/lead,cooav3,cooav4
     */cnce/nces,ncolex
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt       
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      character xflap*3,xflbp*3,xflat*3,xflbt*3
      parameter (nflav=6)
      integer icvp(2),icvt(2),jcvp(nflav,2),jcvt(nflav,2)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cstsh/nstsh
      common/cpzstr/pzstrl,estrl,istrl,isea
      real psum(5),psumx(5)
      r=rangen()
      ntry=0
      irethh=0
      call utremb(proj,targ,2)
      goto9
 
9994  iret=0
      call utrest(proj,targ,2)
9     do 8 n=1,nsi
      do 8 nx=1,nsix
      sixp(n,nx)=0
8     sixt(n,nx)=0
      call utkprj(proj,kmaxp)
      call utkprj(targ,kmaxt)
         if(ish.ge.91)then
      write(ifch,'(a,4f11.5,2f8.0)')' proj:',(proj(i,2),i=1,nsi)
      if(3.le.kmaxp+2)then
      do j=3,kmaxp+2
      write(ifch,'(6x,4f11.5,2f8.0)')(proj(i,j),i=1,nsi)
      enddo
      endif
      write(ifch,'(a,4f11.5,2f8.0)')' targ:',(targ(i,2),i=1,nsi)
      if(3.le.kmaxt+2)then
      do j=3,kmaxt+2
      write(ifch,'(6x,4f11.5,2f8.0)')(targ(i,j),i=1,nsi)
      enddo
      endif
         endif
      if(iskip.eq.1)goto9999
 
9997  call hdecmp(proj,silp,sixp)
      call hdecmp(targ,silt,sixt)
1     icvp(1)=nint(proj(5,1))
      icvp(2)=nint(proj(6,1))
      icvt(1)=nint(targ(5,1))
      icvt(2)=nint(targ(6,1))
      call hapapa(silp,iflap,xflap,ptap,ptahp,iflbp,xflbp,ptbp,ptbhp
     *,nqaqp,icvp)
      call hapapa(silt,iflat,xflat,ptat,ptaht,iflbt,xflbt,ptbt,ptbht
     *,nqaqt,icvt)
      if(nqaqp*nqaqt.eq.-1)goto1
 
9998  ntry=ntry+1
      if(ntry.gt.ntrymx+1)then
      iskip=1
      goto9999
      endif
 
      call hdecmp(proj,silp,sixp)
      call hdecmp(targ,silt,sixt)
      call hastfs(silp,sixp,iflap,xflap,ptap,ptahp
     *,silt,sixt,iflbt,xflbt,ptbt,ptbht,strap,iret,1)
      if(iret.eq.1)then
      iskip=1
      goto9999
      endif
      if(iret.eq.2)goto9997
      if(iret.eq.3)then
      if(xflbp(1:2).eq.'va')xflbp(1:2)='se'
      if(xflat(1:2).eq.'va')xflat(1:2)='se'
      iflat=iflbt
      iflbp=iflap
      goto9998
      endif
      if(iret.eq.5)then
      if(xflap(1:2).eq.'se'.and.xflbp(1:2).eq.'se')then
      iflbp=-iabs(iflbp)
      iflap=iflbp
      endif
      if(xflat(1:2).eq.'se'.and.xflbt(1:2).eq.'se')then
      iflat=-iabs(iflat)
      iflbt=iflat
      endif
      goto9998
      endif
      call hastfs(silt,sixt,iflat,xflat,ptat,ptaht
     *,silp,sixp,iflbp,xflbp,ptbp,ptbhp,strat,iret,2)
      if(iret.eq.1)then
      iskip=1
      goto9999
      endif
      if(iret.eq.2)goto9997
      if(iret.eq.3)then
      if(xflap(1:2).eq.'va')xflap(1:2)='se'
      if(xflbt(1:2).eq.'va')xflbt(1:2)='se'
      iflbt=iflat
      iflap=iflbp
      goto9998
      endif
      if(iret.eq.5)then
      if(xflap(1:2).eq.'se'.and.xflbp(1:2).eq.'se')then
      iflap=-iabs(iflap)
      iflbp=iflap
      endif
      if(xflat(1:2).eq.'se'.and.xflbt(1:2).eq.'se')then
      iflbt=-iabs(iflbt)
      iflat=iflbt
      endif
      goto9998
      endif
 
      call hastfc(silp,sixp,iret)
      if(iret.eq.0)
     *call hastfc(silt,sixt,iret)
           if(iret.eq.1)then
      if(ish.ge.91)write(ifch,*)'light string --> redo hastfs'
      goto9998
           elseif(iret.eq.2)then
      if(ish.ge.91)write(ifch,*)'jc>9 --> redo hapapa'
      goto9997
           endif
 
9999  continue
 
      if(iskip.eq.1)then
      if(ish.ge.91)write(ifch,*)'skip'
      colevt=colevt-1./ncolex
      call hdecmp(proj,silp,sixp)
      call hdecmp(targ,silt,sixt)
      call hastfw(silp,sixp)
      call hastfw(silt,sixt)
      endif
 
      call hastfl(silp,sixp,strlp)
      call hastfl(silt,sixt,strlt)
 
      if(iskip.eq.1)goto9995
 
      call iddeco(icvp,jcvp)
      call iddeco(icvt,jcvt)
      if(xflap(1:2).eq.'va')jcvp(iabs(iflap),1)=jcvp(iabs(iflap),1)-1
      if(xflbp(1:2).eq.'va')jcvp(iabs(iflbp),2)=jcvp(iabs(iflbp),2)-1
      if(xflat(1:2).eq.'va')jcvt(iabs(iflat),1)=jcvt(iabs(iflat),1)-1
      if(xflbt(1:2).eq.'va')jcvt(iabs(iflbt),2)=jcvt(iabs(iflbt),2)-1
      call idenco(jcvp,icvp,ireten)
      if(ireten.eq.1)
     *call utstop('hapapa: idenco ret code = 1             ')
      call idenco(jcvt,icvt,ireten)
      if(ireten.eq.1)
     *call utstop('hapapa: idenco ret code = 1             ')
 
      proj(5,1)=icvp(1)
      proj(6,1)=icvp(2)
      targ(5,1)=icvt(1)
      targ(6,1)=icvt(2)
      call utksix(sixp,kmaxpn)
      call utksix(sixt,kmaxtn)
      if(kmaxpn+1.gt.nsix)
     *call utstop('hahabs: dimension nsix too small        ')
      if(kmaxtn+1.gt.nsix)
     *call utstop('hahabs: dimension nsix too small        ')
      kmxp=kmaxpn+1
      kmxt=kmaxtn+1
      do 5 n=1,nsi
      proj(n,2)=strlp(n,1)
      targ(n,2)=strlt(n,1)
      do 6 j=1,kmxp
6     proj(n,2+j)=sixp(n,j)
      do 7 j=1,kmxt
7     targ(n,2+j)=sixt(n,j)
      kproja(2,iproj)=kol
      if(kmaxpn.gt.kmaxp)then
      do 2 k=kmaxp+1,kmaxpn
2     kproja(2+k,iproj)=kol
      endif
      ktarga(2,itarg)=kol
      if(kmaxtn.gt.kmaxt)then
      do 3 k=kmaxt+1,kmaxtn
3     ktarga(2+k,itarg)=kol
      endif
5     continue
 
      nstsh=0
      lead=0
      isplt=0
11    call hastpr(strap,isplt)
      if(isplt.ne.0)
     *call utstop('hahabs: isplt /= 0                      ')
      isplt=0
12    call hastpr(strat,isplt)
      if(isplt.ne.0)
     *call utstop('hahabs: isplt /= 0                      ')
 
      if(kmaxpn.gt.kmaxp)then
      proj(3,1)=proj(3,1)+coord(3,kol)
      proj(4,1)=proj(4,1)+coord(4,kol)
      endif
      if(kmaxtn.gt.kmaxt)then
      targ(3,1)=targ(3,1)+coord(3,kol)
      targ(4,1)=targ(4,1)+coord(4,kol)
      endif
      kmaxp=kmaxpn
      kmaxt=kmaxtn
 
9995  lead=1
 
c     write leading string (proj)
c     ---------------------------
 
           if(iap.eq.1)then
 
      if(kmaxp.gt.0)then
      cooav3=proj(3,1)/kmaxp
      cooav4=proj(4,1)/kmaxp
      else
      cooav3=coord(3,kol)
      cooav4=coord(4,kol)
      endif
      call utkstr(strlp,kmaxor)
      pzstrl=0
      estrl=0
      isea=1
      do i=1,4
      psum(i)=0
      enddo
           do k=1,kmaxor
      do i=1,4
      psum(i)=psum(i)+strlp(i,k)
      enddo
      ampr=0
c-c   if(k.eq.1)ampr=.94
      pzstrl=pzstrl+strlp(3,k)
      estrl=estrl+sqrt(strlp(4,k)**2+ampr**2)
      if(k.ge.2.and.strlp(5,k).gt.0..and.strlp(6,k).gt.0.)isea=0
           enddo
      psum(5)=psum(4)**2-psum(3)**2-psum(2)**2-psum(1)**2
      if(psum(5).gt.0.)psum(5)=sqrt(psum(5))
      if(pzstrl/pnllx.gt.1.-0.850/engy**2.and.iskip.ne.1)goto1002
      istrl=0
      if(nstsh.eq.0.and.isea.eq.1
     *.and.pzstrl/pnllx.gt.1.-10.000/engy**2)istrl=1
      do i=1,nsi
      do k=1,kmaxor
      str0(i,k)=strlp(i,k)
      enddo
      enddo
      nstr0=nstr
17    nstr=nstr0
      isplt=0
15    continue
      do i=1,nsi
      do k=1,kmaxor
      strlp(i,k)=str0(i,k)
      enddo
      enddo
13    call hastpr(strlp,isplt)
      if(isplt.eq.-1.and.iskip.eq.1)goto9996
      if(isplt.eq.-1)goto9994
      if(isplt.eq.-3)goto15
      if(isplt.eq.-4)goto1001
      if(isplt.eq.-5)goto17
      if(isplt.gt.0)goto13
           if(nstr.gt.nstr0+1)then
      do i=1,4
      psumx(i)=0
      do j=nstr0+1,nstr
      psumx(i)=psumx(i)+pstr(i,j)
      enddo
      enddo
      psumx(5)=psumx(4)**2-psumx(3)**2-psumx(2)**2-psumx(1)**2
      if(psumx(5).gt.0.)psumx(5)=sqrt(psumx(5))
c-c   write(ifmt,*)' '
c-c   do j=nstr0+1,nstr
c-c   write(ifmt,109)j,(icstr(k,j),k=1,4)
c-c  *,sqrt(pstr(1,j)**2+pstr(2,j)**2),pstr(3,j)/pnllx,pstr(5,j)
c-c  *,irlstr(j)
c-c   enddo
c-c   write(ifmt,*)psum
c-c   write(ifmt,*)psumx
      do i=1,4
      do j=nstr0+1,nstr
      pstr(i,j+nstr-nstr0)=pstr(i,j)
      enddo
      enddo
      call hrescl(nstr0+1,nstr,psum,ifail)
      if(ifail.gt.0)then
c-c   write(ifmt,*)'rescale failed'
      do i=1,4
      do j=nstr0+1,nstr
      pstr(i,j)=pstr(i,j+nstr-nstr0)
      enddo
      enddo
      endif
      do i=1,4
      psumx(i)=0
      do j=nstr0+1,nstr
      psumx(i)=psumx(i)+pstr(i,j)
      enddo
      enddo
      psumx(5)=psumx(4)**2-psumx(3)**2-psumx(2)**2-psumx(1)**2
      if(psumx(5).gt.0.)psumx(5)=sqrt(psumx(5))
c-c   do j=nstr0+1,nstr
c-c   write(ifmt,109)j,(icstr(k,j),k=1,4)
c-c  *,sqrt(pstr(1,j)**2+pstr(2,j)**2),pstr(3,j)/pnllx,pstr(5,j)
c-c  *,irlstr(j)
c-c   enddo
c-c   write(ifmt,*)psum
c-c   write(ifmt,*)psumx
           endif
109   format(' /cstr/',i4,3x,4i7,2x,3(1x,e9.3),i4)
 
           endif
 
c     write leading string (targ)
c     ---------------------------
 
           if(iat.eq.1)then
 
      if(kmaxt.gt.0)then
      cooav3=targ(3,1)/kmaxt
      cooav4=targ(4,1)/kmaxt
      else
      cooav3=coord(3,kol)
      cooav4=coord(4,kol)
      endif
      call utkstr(strlt,kmaxor)
      pzstrl=0
      estrl=0
      isea=1
      do i=1,4
      psum(i)=0
      enddo
           do k=1,kmaxor
      do i=1,4
      psum(i)=psum(i)+strlt(i,k)
      enddo
      ampr=0
c-c   if(k.eq.1)ampr=.94
      pzstrl=pzstrl+strlt(3,k)
      estrl=estrl+sqrt(strlt(4,k)**2+ampr**2)
      if(k.ge.2.and.strlt(5,k).gt.0..and.strlt(6,k).gt.0.)isea=0
           enddo
      psum(5)=psum(4)**2-psum(3)**2-psum(2)**2-psum(1)**2
      if(psum(5).gt.0.)psum(5)=sqrt(psum(5))
      if(-pzstrl/pnllx.gt.1.-0.850/engy**2.and.iskip.ne.1)goto1002
      istrl=0
      if(nstsh.eq.0.and.isea.eq.1
     *.and.-pzstrl/pnllx.gt.1.-10.000/engy**2)istrl=1
      do i=1,nsi
      do k=1,kmaxor
      str0(i,k)=strlt(i,k)
      enddo
      enddo
      nstr0=nstr
18    nstr=nstr0
      isplt=0
16    continue
      do i=1,nsi
      do k=1,kmaxor
      strlt(i,k)=str0(i,k)
      enddo
      enddo
14    call hastpr(strlt,isplt)
      if(isplt.eq.-1.and.iskip.eq.1)goto9996
      if(isplt.eq.-1)goto9994
      if(isplt.eq.-3)goto16
      if(isplt.eq.-4)goto1001
      if(isplt.eq.-5)goto18
      if(isplt.gt.0)goto14
           if(nstr.gt.nstr0+1)then
      do i=1,4
      psumx(i)=0
      do j=nstr0+1,nstr
      psumx(i)=psumx(i)+pstr(i,j)
      enddo
      enddo
      psumx(5)=psumx(4)**2-psumx(3)**2-psumx(2)**2-psumx(1)**2
      if(psumx(5).gt.0.)psumx(5)=sqrt(psumx(5))
c-c   write(ifmt,*)' '
c-c   do j=nstr0+1,nstr
c-c   write(ifmt,109)j,(icstr(k,j),k=1,4)
c-c  *,sqrt(pstr(1,j)**2+pstr(2,j)**2),pstr(3,j)/pnllx,pstr(5,j)
c-c  *,irlstr(j)
c-c   enddo
c-c   write(ifmt,*)psum
c-c   write(ifmt,*)psumx
      do i=1,4
      do j=nstr0+1,nstr
      pstr(i,j+nstr-nstr0)=pstr(i,j)
      enddo
      enddo
      call hrescl(nstr0+1,nstr,psum,ifail)
      if(ifail.gt.0)then
c-c   write(ifmt,*)'rescale failed'
      do i=1,4
      do j=nstr0+1,nstr
      pstr(i,j)=pstr(i,j+nstr-nstr0)
      enddo
      enddo
      endif
      do i=1,4
      psumx(i)=0
      do j=nstr0+1,nstr
      psumx(i)=psumx(i)+pstr(i,j)
      enddo
      enddo
      psumx(5)=psumx(4)**2-psumx(3)**2-psumx(2)**2-psumx(1)**2
      if(psumx(5).gt.0.)psumx(5)=sqrt(psumx(5))
c-c   do j=nstr0+1,nstr
c-c   write(ifmt,109)j,(icstr(k,j),k=1,4)
c-c  *,sqrt(pstr(1,j)**2+pstr(2,j)**2),pstr(3,j)/pnllx,pstr(5,j)
c-c  *,irlstr(j)
c-c   enddo
c-c   write(ifmt,*)psum
c-c   write(ifmt,*)psumx
           endif
 
           endif
 
c     exit
c     ----
 
      if(.not.(ish.ge.91))return
 
      write(ifch,'(a,4f11.5,2f8.0)')' proj:',(proj(i,2),i=1,nsi)
      if(3.le.kmaxp+2)then
      do j=3,kmaxp+2
      write(ifch,'(6x,4f11.5,2f8.0)')(proj(i,j),i=1,nsi)
      enddo
      endif
      write(ifch,'(a,4f11.5,2f8.0)')' targ:',(targ(i,2),i=1,nsi)
      if(3.le.kmaxt+2)then
      do j=3,kmaxt+2
      write(ifch,'(6x,4f11.5,2f8.0)')(targ(i,j),i=1,nsi)
      enddo
      endif

      return
 
9996  continue
      if(ish.ge.91)then
      write(ifch,*)'skip impossible, strl cant be stored'
      endif
      iskip=2
      return
 
1001  continue
      if(ish.ge.91)then
      write(ifch,*)
     *'sr hahabs: not justified complete absorption -> irethh=1'
      endif
      irethh=1
      if(iskip.eq.1)goto9996
      return
 
1002  continue
      if(ish.ge.91)then
      write(ifch,*)'sr hahabs: too fast multi-string -> irethh=1'
      endif
      irethh=1
      if(iskip.eq.1)
     *call utstop('hahabs: iskip=1 should not happen       ')
      return
 
      end
 
c-----------------------------------------------------------------------
      subroutine hapapa(sil,ifla,xfla,pta,ptah,iflb,xflb,ptb,ptbh,nqaq
     *,icv)
c     determines momenta and flavor of participating partons in a hadron
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (nsi=6,nsix=100,nflav=6)
      parameter (nha=nsix+2)
      real sil(nsi),probab(nflav),probsu(nflav+1)
      integer jc(nflav,2),id(2)
     *,icv(2),jcv(nflav,2)
      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      parameter (nstru=2049)
      common/stru/xstru(nstru),wstru(nstru)
     *,qvap(nstru),qvat(nstru),qsep(nstru),qset(nstru)
      common /cnew/kotri,newico,newcol
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      character xfla*3,xflb*3,chpt*1
 
      sgn=sign(1.,sil(3))
      if(sgn.gt.0.)qvalc=qvapc
      if(sgn.gt.0.)qseac=qsepc
      if(sgn.lt.0.)qvalc=qvatc
      if(sgn.lt.0.)qseac=qsetc
      if(sgn.gt.0.)chpt='p'
      if(sgn.lt.0.)chpt='t'
      if(sgn.gt.0.)w=wproj
      if(sgn.lt.0.)w=wtarg
      call iddeco(icv,jcv)
      nvq=0
      nva=0
      do 12 i=1,nflav
           if(ish.ge.0.and. (jcv(i,1).lt.0.or.jcv(i,2).lt.0) )then
      call utmsg('hahabs')
      write(ifch,*)'*****  negative jcv'
      write(ifch,*)'jcv:'
      write(ifch,*)jcv
      call utmsgf
           endif
      nvq=nvq+jcv(i,1)
12    nva=nva+jcv(i,2)
      do 13 i=1,2
13    id(i)=nint(sil(4+i))
      call iddeco(id,jc)
      nqu=0
      naq=0
      do 11 i=1,nflav
      nqu=nqu+jc(i,1)
11    naq=naq+jc(i,2)
      neq=nqu-naq
 
c     determine nqaq,ival1,ival2
c     --------------------------
      nqaq=0
      ival=0
            if(rangen().gt.w)then
      pq=1
      pa=1
      if(neq.le.neqmn)pq=0
      if(neq.ge.neqmx)pa=0
      sum=(nvq*pq+nva*pa*iaqu)*qvalc+qseac*(.5*pq+.5*pa*iaqu)
      if(sum.eq.0.)goto14
      pvq=nvq*pq*qvalc/sum
      pva=nva*pa*iaqu*qvalc/sum
      psq=.5*pq*qseac/sum
      psa=.5*pa*iaqu*qseac/sum
      r=rangen()
      if(r.lt.pvq+pva)ival=1
      nqaq=-1
      if(ival.eq.1.and.r.lt.pvq)nqaq=1
      if(ival.eq.0.and.r.lt.pvq+pva+psq)nqaq=1
           if(nqu-naq-nqaq.lt.neqmn)then
      if(ish.ge.90)then
      call utmsg('hapapa')
      write(ifch,*)'*****  neq < neqmn'
      write(ifch,*)'pvq/a psq/a:',pvq,pva,psq,psa
      write(ifch,*)'nqaq:',nqaq
      call utmsgf
      endif
      nqaq=0
           endif
           if(nqu-naq-nqaq.gt.neqmx)then
      if(ish.ge.90)then
      call utmsg('hapapa')
      write(ifch,*)'*****  neq > neqmx'
      write(ifch,*)'pvq/a psq/a:',pvq,pva,psq,psa
      write(ifch,*)'nqaq:',nqaq
      endif
      nqaq=0
           endif
      if(nqu-naq-nqaq.lt.neqmn)
     *call utstop('hapapa: neq.lt.neqmn                    ')
      if(nqu-naq-nqaq.gt.neqmx)
     *call utstop('hapapa: neq.gt.neqmx                    ')
           endif
14    continue
      ival1=ival
      ival2=ival
           if(nqaq.eq.0)then
      ival1=0
      sum=nvq*qvalc+qseac*.5
      if(sum.eq.0.)goto15
      pvq=nvq*qvalc/sum
      psq=.5*qseac/sum
      r=rangen()
      if(r.lt.pvq)ival1=1
   15 continue
c-c   if(nvq.gt.0.and.rangen().lt.pvalen)ival1=1
      ival2=0
      sum=nva*iaqu*qvalc+qseac*.5*iaqu
      if(sum.eq.0.)goto16
      pva=nva*iaqu*qvalc/sum
      psa=.5*iaqu*qseac/sum
      r=rangen()
      if(r.lt.pva)ival2=1
   16 continue
c-c   if(nva.gt.0.and.rangen().lt.pvalen)ival2=1
      if(ival1.eq.1.and.ival2.eq.1)then
      r=rangen()
      if(r.lt..5)then
      ival1=0
      else
      ival2=0
      endif
      endif
           endif
 
c     quark
c     -----
      xfla='---'
           if(nqaq.ge.0)then
      su=0
      do 20 i=1,4
      probab(i)=0
      if(ival1.eq.1)probab(i)=jcv(i,1)
      if(ival1.eq.0.and.i.le.2)probab(i)=1
      if(ival1.eq.0.and.i.eq.3)probab(i)=rstras
20    su=su+probab(i)
      probsu(1)=0
      do 30 i=1,4
30    probsu(i+1)=probsu(i)+probab(i)/su
      r=rangen()
      do 50 i=1,4
      if(probsu(i).le.r.and.r.le.probsu(i+1))ifla=i
50    continue
      if(ifla.gt.4)ifla=4
      if(ival1.eq.1)xfla='va'//chpt
      if(ival1.eq.0)xfla='se'//chpt
      pta=utinvt(nptq,xptq,qptq,rangen()*qptq(nptq))
      ptah=utinvt(nptq,xptq,qpth,rangen()*qpth(nptq))
           endif
 
c     antiquark
c     ---------
      xflb='---'
           if(nqaq.le.0)then
      su=0
      do 60 i=1,4
      probab(i)=0
      if(ival2.eq.1)probab(i)=jcv(i,2)
      if(ival2.eq.0.and.i.le.2)probab(i)=1
      if(ival2.eq.0.and.i.eq.3)probab(i)=rstras
60    su=su+probab(i)
      probsu(1)=0
      do 70 i=1,4
70    probsu(i+1)=probsu(i)+probab(i)/su
      r=rangen()
      do 80 i=1,4
      if(probsu(i).le.r.and.r.le.probsu(i+1))iflb=i
80    continue
      if(iflb.gt.4)iflb=4
      if(nqaq.eq.0.and.ival1.eq.0.and.ival2.eq.0)iflb=ifla
c-c   if(nqaq.eq.0.and.ival2.eq.0)iflb=ifla
c-c   if(nqaq.eq.0.and.ival1.eq.0)ifla=iflb
      if(ival2.eq.1)xflb='va'//chpt
      if(ival2.eq.0)xflb='se'//chpt
      ptb=utinvt(nptq,xptq,qptq,rangen()*qptq(nptq))
      ptbh=utinvt(nptq,xptq,qpth,rangen()*qpth(nptq))
           endif
 
      if(nqaq.eq.0.and.ival1.eq.0.and.ival2.eq.0)ifla=-ifla
      if(nqaq.eq.0.and.ival1.eq.0.and.ival2.eq.0)iflb=-iflb
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine hasi(qaq,sil,ifl,xfl,pt0,pt0h,si,iret,jord,ixflab
     *               ,ptdiff)
c     determines string ingredients (=jets)
c iret=0: ok
c iret=1: remnant changes direction
c iret=2: jc(,)=10
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      character xfl*3,qaq*5
      parameter (nsi=6,nflav=6,nsix=100)
      real sil(nsi),si(nsi)
      integer jc(nflav,2),id(2),ic(2)
      real ptdiff(2)
      iret=0
      iretx=0
      ntry=0
      ptdiff(1)=0.
      ptdiff(2)=0.
      ifla=iabs(ifl)
           if(xfl.eq.'000')then
      do 11 j=3,nsi
11    si(j)=0
           else
      do 10 j=1,nsi
10    si(j)=0
           endif
      if(xfl.eq.'---')goto1000
      if(jord.eq.1.and.qaq.eq.'quark')ams=amproj
      if(jord.eq.2.and.qaq.eq.'antiq')ams=amproj
      if(jord.eq.2.and.qaq.eq.'quark')ams=amtarg
      if(jord.eq.1.and.qaq.eq.'antiq')ams=amtarg
c-c   enll=sqrt(ams**2+pnllx**2)
      enll=sil(4)
      sgn=sign(1.,sil(3))
      ihard=0
      if(rangen().le.phard)ihard=1
      do 1 i=1,2
1     id(i)=nint(sil(4+i))
      call iddeco(id,jc)
9999  ntry=ntry+1
      if(ntry.gt.20)then
      iret=1
      goto1000
      endif
           if(ihard.eq.0)then
      pt=pt0
      if(pt.gt.enll)pt=rangen()*enll
      enmin=0
           elseif(ihard.eq.1)then
      pt=pt0h
      if(pt.gt.enll)pt=rangen()*enll
      enmin=pt
           endif
      ptfull=pt
      lo=0
51    lo=lo+1
      if(xfl.eq.'000')then
      en=0.
      pt=0.
      else
      en=ranstc(xfl,enmin/enll)*enll
      endif
      if(pt.gt.en.and.rangen().lt.0.5.and.lo.le.10)goto51
      if(pt.gt.en)pt=rangen()*en
      phi=2.*pi*rangen()
      if(xfl.ne.'000')then
      si(1)=pt*cos(phi)
      si(2)=pt*sin(phi)
      si(3)=sgn*sqrt(en**2-pt**2)
      if(si(1).eq.0..and.si(2).eq.0..and.si(3).eq.0.)then
      iretx=1
      iret=1
      goto1000
      endif
c-c   ptdiff(1)=(ptfull-pt)*cos(phi)
c-c   ptdiff(2)=(ptfull-pt)*sin(phi)
      endif
      if((sil(3)-si(3))*sil(3).lt.0.)then
      if(ish.ge.94)write(ifch,*)'sil,si,enll,en',sil,si,enll,en
      goto9999
      endif
      si(4)=sqrt(si(1)**2+si(2)**2+si(3)**2)
      if(qaq.eq.'quark')si(5)=10**(nflav-ifla)
      if(qaq.eq.'antiq')si(6)=10**(nflav-ifla)
 
      if(ixflab.eq.0)then
      sil(1)=sil(1)-si(1)
      sil(2)=sil(2)-si(2)
      sil(3)=sil(3)-si(3)
      sil(4)=sqrt(sil(1)**2+sil(2)**2+sil(3)**2)
      endif
 
      l1=0
      l2=0
      if(jc(ifla,1).gt.0)l1=1
      if(jc(ifla,2).gt.0)l2=1
      if(qaq.eq.'quark'.and.l1.eq.0)jc(ifla,2)=jc(ifla,2)+1
      if(qaq.eq.'quark'.and.l1.eq.1)jc(ifla,1)=jc(ifla,1)-1
      if(qaq.eq.'antiq'.and.l2.eq.0)jc(ifla,1)=jc(ifla,1)+1
      if(qaq.eq.'antiq'.and.l2.eq.1)jc(ifla,2)=jc(ifla,2)-1
      if(jc(ifla,1).eq.10.or.jc(ifla,2).eq.10)goto9998
      call idenco(jc,ic,ireten)
      if(ireten.eq.1)
     *call utstop('hasi  : idenco ret code = 1             ')
      sil(5)=ic(1)
      sil(6)=ic(2)
      goto1000
 
9998  iret=2
      if(ish.ge.90)then
      call utmsg('hasi  ')
      write(ifch,*)'*****  jc(,)=10'
      write(ifch,*)jc
      call utmsgf
      endif
 
1000  continue
           if(iretx.eq.1)then
      call utmsg('hasi')
      write(ifch,*)'*****  si(1/2/3)=0'
      write(ifch,*)si
      call utmsgf
           endif
      if(ish.ge.93)then
      if(iret.ne.0.or.ntry.gt.1)
     *write(ifch,*)'iret=',iret,'  ntry=',ntry
      if(ntry.gt.0)
     *write(ifch,100)xfl,pt0,pt,en,en/enll
100   format(1x,'hasi: xfl=',a3
     *,' pt0=',e8.3,' pt=',e8.3,' en=',e8.3,' x=',e8.3)
      if(ntry.eq.0)
     *write(ifch,101)xfl
101   format(1x,'hasi: xfl=',a3)
      endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine hastfc(sil,six,iret)
c     checks long strings
c     iret=0: ok
c     iret=1: mass .lt. minimal mass
c     iret=2: jc(,) .gt. 9
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common /cnce/nces,ncolex
      parameter (nsi=6,nflav=6,nsix=100)
      real sil(nsi),six(nsi,nsix),si(nsi)
      integer ic(2),jc(nflav,2),jcp(nflav,2)

      if(ish.ge.94)write(ifch,*)('-',l=1,10)
     *,' entry sr hastfc ',('-',l=1,30)
 
      iret=0
 
      call utksix(six,kmax)
      if(kmax.lt.1)goto9000
 
      do 110 n=1,4
      si(n)=sil(n)
      do 110 k=1,kmax
110   si(n)=si(n)+six(n,k)
      s=si(4)**2-si(1)**2-si(2)**2-si(3)**2
 
      ic(1)=nint(sil(5))
      ic(2)=nint(sil(6))
      call iddeco(ic,jc)
      do 130 k=1,kmax
      ic(1)=nint(abs(six(5,k)))
      ic(2)=nint(abs(six(6,k)))
      call iddeco(ic,jcp)
      do 120 i=1,nflav
      do 120 j=1,2
120   jc(i,j)=jc(i,j)+jcp(i,j)
130   continue
 
      do 125 i=1,nflav
      if(jc(i,1)-jc(i,2).ge.10)iret=2
      if(jc(i,2)-jc(i,1).ge.10)iret=2
125   continue
 
      if(iret.eq.2)goto9000
 
      keu=jc(1,1)-jc(1,2)
      ked=jc(2,1)-jc(2,2)
      kes=jc(3,1)-jc(3,2)
      kec=jc(4,1)-jc(4,2)
      amin=utamnu(keu,ked,kes,kec,4)/2
 
c-c   if(s.lt.amin**2)iret=1
 
9000  continue
 
           if(ish.ge.94)then
      write(ifch,*)'iret= ',iret ,'  kmax= ',kmax,'  nrevt= ',nrevt
     *,'  nces= ',nces
      write(ifch,8004)sil
8004  format(' sil: ',4f11.5,2f8.0)
           if(kmax.gt.0)then
      write(ifch,8007)(six(i,1),i=1,nsi)
8007  format(' six: ',4f11.5,2f8.0)
      if(kmax.gt.1)then
      do 1 j=2,kmax
1     write(ifch,8008)(six(i,j),i=1,nsi)
8008  format('      ',4f11.5,2f8.0)
      endif
      write(ifch,8005)(si(i),i=1,4)
8005  format('  si: ',4f11.5)
      write(ifch,8006)(si(i)**2,i=1,4)
8006  format(' si^2:',4f11.5)
      write(ifch,*)'jc:'
      write(ifch,*)jc
      write(ifch,*)'keu,ked,kes,kec: ',keu,ked,kes,kec
      write(ifch,*)'s= ',s,'    amin**2= ',amin**2,'   amin= ',amin
           endif
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr hastfc ',('-',i=1,10)
           endif
 
      if(iret.ne.0)return
 
      entry hastfw(sil,six)
      if(.not.(ish.ge.91))return
      call utksix(six,kmax)
      write(ifch,103)sil,((six(i,j),i=1,nsi),j=1,kmax)
103   format(' sil: ',4f11.5,2f8.0/,' six: ',4f11.5,2f8.0/
     *,50('      ',4f11.5,2f8.0/))
      return
      end
 
c-----------------------------------------------------------------------
      subroutine hastfl(sil,six,strl)
c     forms a leading string
c-----------------------------------------------------------------------
      parameter (nsi=6,nsix=100)
      real sil(nsi),six(nsi,nsix),strl(nsi,nsix+1)
      do 100 n=1,nsi
      strl(n,1)=sil(n)
100   strl(n,2)=0
      call utksix(six,kmax)
      if(kmax.eq.0)return
      do 110 k=1,kmax
      do 110 n=1,nsi
110   strl(n,k+1)=six(n,k)
      do 120 n=1,nsi
120   strl(n,kmax+2)=0
      return
      end
 
c-----------------------------------------------------------------------
      subroutine hastfr(sila,silb,sia,sib,iret)
c-----------------------------------------------------------------------
c  subtracts sia-sib from sila-silb (only momentum components)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea

      parameter (nsi=6)
      real sila(nsi),silb(nsi),sia(nsi),sib(nsi)
      double precision a,d,b,put,pux,puy,pot,pox,poy,pep,pem,pyp,pym
     *,pip,pim,pi3,pi4,pap,pam,pa3,pa4,sla,slb
      double precision ssila(nsi),ssilb(nsi),ssia(nsi),ssib(nsi)
     *,plab(4),pab(4)
 
      iret=0

      do i=1,4
      ssila(i)=sila(i)
      ssilb(i)=silb(i)
      ssia(i)=sia(i)
      ssib(i)=sib(i)
      plab(i)=ssila(i)+ssilb(i)
      pab(i)=ssia(i)+ssib(i)
      enddo
      sla=ssila(3)/dabs(ssila(3))
      slb=-sla
 
      if(ssia(3)*ssila(3).lt.0.)then
      call utmsg('hastfr')
      write(ifch,*)'*****  ssia(3)*ssila(3)<0'
      write(ifch,*)ssia(3),ssila(3)
      call utmsgf
      endif
 
      pep=0
      pem=0
      pot=0
      put=0
      a=0
      d=0
      pyp=0
      pym=0
      pip=0
      pam=0
 
      pox=ssila(1)-ssia(1)
      poy=ssila(2)-ssia(2)
      pot=dsqrt(pox**2+poy**2)
      pux=ssilb(1)-ssib(1)
      puy=ssilb(2)-ssib(2)
      put=dsqrt(pux**2+puy**2)
      pep= plab(4)+sla*plab(3) - (pab(4)+sla*pab(3))
      pem= plab(4)-sla*plab(3) - (pab(4)-sla*pab(3))
      if(pep.lt.0.)goto9001
      if(pem.lt.0.)goto9001
      a=(pem*pep-put**2-pot**2)/2.
      d=put*pot
      if(a**2-d**2.lt.0.)goto9001
      b=dsqrt(a**2-d**2)
      pyp=(a+put**2-b)/pem
      pym=(a+pot**2-b)/pep
      if(pyp.lt.0.)goto9001
      if(pym.lt.0.)goto9001
      pip=pep-pyp
      if(pip.lt.0.)goto9001
      pim=pym
      pap=pyp
      pam=pem-pym
      if(pam.lt.0.)goto9001
      pi3=(pip-pim)/2*sla
      pi4=(pip+pim)/2
      pa3=(pap-pam)/2*sla
      pa4=(pap+pam)/2
      if(pi3*ssila(3).lt.0.)goto9001
      if(pa3*ssilb(3).lt.0.)goto9001
      ssila(1)=pox
      ssila(2)=poy
      ssila(3)=pi3
      ssila(4)=pi4
      ssilb(1)=pux
      ssilb(2)=puy
      ssilb(3)=pa3
      ssilb(4)=pa4
 
      do i=1,4
      sila(i)=ssila(i)
      silb(i)=ssilb(i)
      sia(i)=ssia(i)
      sib(i)=ssib(i)
      enddo

      if(ish.lt.90)goto9000
 
      if(dabs(pip*pim-pot**2).gt.1e-4)then
      call utmsg('hastfr')
      write(ifch,*)'*****  pip*pim /= pot**2'
      write(ifch,*)'pip*pim=',pip*pim
      write(ifch,*)'pot**2=',pot**2
      write(ifch,*)'pip=',pip
      write(ifch,*)'pim=',pim
      write(ifch,*)'pot=',pot
      call utmsgf
      endif
      if(dabs(pap*pam-put**2).gt.1e-4)then
      call utmsg('hastfr')
      write(ifch,*)'*****  pap*pam /= put**2'
      write(ifch,*)'pap*pam=',pap*pam
      write(ifch,*)'put**2=',put**2
      write(ifch,*)'pap=',pap
      write(ifch,*)'pam=',pam
      write(ifch,*)'put=',put
      call utmsgf
      endif
      if(dabs(ssila(4)**2
     *-ssila(1)**2-ssila(2)**2-ssila(3)**2).gt.1e-4)then
      call utmsg('hastfr')
      write(ifch,*)'*****  mass**2 of ssila nonzero'
      write(ifch,*)'mass**2=',ssila(4)**2
     *-ssila(1)**2-ssila(2)**2-ssila(3)**2
      call utmsgf
      endif
      if(dabs(ssilb(4)**2
     *-ssilb(1)**2-ssilb(2)**2-ssilb(3)**2).gt.1e-4)then
      call utmsg('hastfr')
      write(ifch,*)'*****  mass**2 of ssilb nonzero'
      write(ifch,*)'mass**2=',ssilb(4)**2
     *-ssilb(1)**2-ssilb(2)**2-ssilb(3)**2
      call utmsgf
      endif
      do 14 n=1,4
      if(dabs(plab(n)
     *-ssila(n)-ssilb(n)-ssia(n)-ssib(n)).gt.1e-4)goto15
14    continue
      goto16
15    continue
      call utmsg('hastfr')
      write(ifch,*)'***** ssila+ssilb /= ssila_new+ssilb_new+ssia+ssib'
      write(ifch,*)'ssila_old+ssilb_old:'
      write(ifch,*)plab
      write(ifch,*)'ssila_new+ssilb_new+ssia+ssib:'
      write(ifch,*)((ssila(k)+ssilb(k)+ssia(k)+ssib(k)),k=1,4)
      write(ifch,*)'ssila_new:'
      write(ifch,*)(ssila(n),n=1,4)
      write(ifch,*)'ssilb_new:'
      write(ifch,*)(ssilb(n),n=1,4)
      write(ifch,*)'ssia:'
      write(ifch,*)(ssia(n),n=1,4)
      write(ifch,*)'ssib:'
      write(ifch,*)(ssib(n),n=1,4)
      call utmsgf
16    continue
 
9000  return
 
9001  iret=1
      if(ish.lt.90)return
      call utmsg('hastfr')
      write(ifch,*)'string subtraction not possible'
      write(ifch,*)'sngl(ssila):'
      write(ifch,*)(sngl(ssila(n)),n=1,4)
      write(ifch,*)'sngl(ssilb):'
      write(ifch,*)(sngl(ssilb(n)),n=1,4)
      write(ifch,*)'sngl(ssia):'
      write(ifch,*)(sngl(ssia(n)),n=1,4)
      write(ifch,*)'sngl(ssib):'
      write(ifch,*)(sngl(ssib(n)),n=1,4)
      write(ifch,*)'pep,pem:',pep,pem
      write(ifch,*)'pot,put:',pot,put
      write(ifch,*)'a,d:',a,d
      write(ifch,*)'a**2-d**2:',a**2-d**2
      write(ifch,*)'pyp,pym:',pyp,pym
      write(ifch,*)'pip,pam:',pip,pam
      write(ifch,*)'pi3,sila(3):',pi3,sila(3)
      write(ifch,*)'pa3,silb(3):',pa3,silb(3)
      call utmsgf
      return
      end
 
c-----------------------------------------------------------------------
      subroutine hastfs(sila,sixa,ifla,xfla,pta,ptah
     *                 ,silb,sixb,iflb,xflb,ptb,ptbh  ,str,iret  ,jord)
c     forms a short (=q-qbar) string
c iret=0: ok
c iret=1: in sr hasi: remnant changes direction or zero si().
c iret=2: in sr hasi: jc(,)=10 .
c iret=3: string mass too small. not 4 , 5 .
c iret=4: string mass too small. equal flavour,zero momentum,xfl='000'.
c iret=5: string mass too small. valence quarks involved.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nsi=6,nsix=100)
      real sila(nsi),sixa(nsi,nsix),silb(nsi),sixb(nsi,nsix)
     *,sia(nsi),sib(nsi),str(nsi,nsix+1)
     *,sts(nsi,2)
      character xfla*3,xflb*3
      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      real ptdiff(2)
 
      do 50 n=1,nsi
      str(n,1)=0
      str(n,2)=0
      str(n,3)=0
50    continue
 
      call utksix(sixa,kmaxa)
      call utksix(sixb,kmaxb)
 
      pt=utinvt(nptq,xptq,qptq,rangen()*qptq(nptq))
      phi=2.*pi*rangen()
      sia(1)=pt*cos(phi)
      sia(2)=pt*sin(phi)
      sib(1)=-sia(1)
      sib(2)=-sia(2)
 
      ixflab=0
      if(  xfla.ne.'---'.and.xflb.ne.'---'
     *.and.(xfla.ne.'000'.or.xflb.ne.'000'))ixflab=1
      call hasi('quark',sila,ifla,xfla,pta,ptah,sia,iret,jord,ixflab
     *,ptdiff)
      if(iret.eq.1)goto9001
      if(iret.eq.2)goto9002
      sila(1)=sila(1)-ptdiff(1)
      sila(2)=sila(2)-ptdiff(2)
      silb(1)=silb(1)+ptdiff(1)
      silb(2)=silb(2)+ptdiff(2)
      call hasi('antiq',silb,iflb,xflb,ptb,ptbh,sib,iret,jord,ixflab
     *,ptdiff)
      if(iret.eq.1)goto9001
      if(iret.eq.2)goto9002
      silb(1)=silb(1)-ptdiff(1)
      silb(2)=silb(2)-ptdiff(2)
      sila(1)=sila(1)+ptdiff(1)
      sila(2)=sila(2)+ptdiff(2)
      if(sila(5).eq.0..and.sila(6).eq.0.)then
      sila(5)=100000
      sila(6)=100000
      endif
      if(silb(5).eq.0..and.silb(6).eq.0.)then
      silb(5)=100000
      silb(6)=100000
      endif
           if(xfla.eq.'000'.and.xflb.eq.'000')then
      goto9003
           elseif(xfla.ne.'---'.and.xflb.ne.'---')then
      call hastfr(sila,silb,sia,sib,iret)
      if(iret.eq.1)goto9003
      call utfost(sia,sib,sts)
      call utamst(sts,am,amin,iret)
      if(iret.ne.0)goto9003
      do 70 n=1,nsi
      str(n,1)=sia(n)
70    str(n,2)=sib(n)
           elseif(xfla.ne.'---'.and.xflb.eq.'---')then
      if(kmaxb+1.gt.nsix)
     *call utstop('hastfs: nsix too small                  ')
      do 60 n=1,nsi
      sixb(n,kmaxb+1)=sia(n)
      if(kmaxb+2.le.nsix)sixb(n,kmaxb+2)=0
60    continue
      if(ifla.lt.0)sixb(5,kmaxb+1)=-sixb(5,kmaxb+1)
      if(ifla.lt.0)sixb(6,kmaxb+1)=-sixb(6,kmaxb+1)
      kmaxb=kmaxb+1
           elseif(xfla.eq.'---'.and.xflb.ne.'---')then
      if(kmaxa+1.gt.nsix)
     *call utstop('hastfs: nsix too small                  ')
      do 80 n=1,nsi
      sixa(n,kmaxa+1)=sib(n)
      if(kmaxa+2.le.nsix)sixa(n,kmaxa+2)=0
80    continue
      if(iflb.lt.0)sixa(5,kmaxa+1)=-sixa(5,kmaxa+1)
      if(iflb.lt.0)sixa(6,kmaxa+1)=-sixa(6,kmaxa+1)
      kmaxa=kmaxa+1
           elseif(xfla.eq.'---'.and.xflb.eq.'---')then
c     no action
           else
      call utstop('hastfs: if/else error                   ')
           endif
 
9000  iret=0
      if(ish.ge.91)write(ifch,102)sia,xfla,sib,xflb
102   format(' sia: ',4f11.5,2f8.0,2x,a3
     *      /' sib: ',4f11.5,2f8.0,2x,a3)
      return
 
9001  iret=1
      if(ish.ge.91)write(ifch,102)sia,xfla,sib,xflb
      if(ish.ge.91)write(ifch,*)'remnant changes direction'
      if(ish.ge.91)write(ifch,*)' or zero si(1/2/3)'
      return
 
9002  iret=2
c     jc(,)=10
      if(ish.ge.91)write(ifch,102)sia,xfla,sib,xflb
      return
 
9003  iret=3
      if(xfla(1:2).eq.'va'.or.xflb(1:2).eq.'va')goto9005
      wt=abs(sia(1)+sib(1))+abs(sia(2)+sib(2))
     *+abs(sia(3)+sib(3))
      if(iabs(ifla).eq.iabs(iflb).and.wt.lt.1e-3
     *.and.xfla.eq.'000'.and.xflb.eq.'000')goto9004
      iflb=iabs(ifla)*iflb/iabs(iflb)
      if(ish.ge.91)write(ifch,102)sia,xfla,sib,xflb
      if(ish.ge.91)write(ifch,*)'string mass too small (3)'
      xfla='000'
      xflb='000'
      return
 
9004  iret=4
      if(ish.ge.91)write(ifch,102)sia,xfla,sib,xflb
      if(ish.ge.91)write(ifch,*)'string mass too small (4)'
      return
 
9005  iret=5
      if(ish.ge.91)write(ifch,102)sia,xfla,sib,xflb
      if(ish.ge.91)write(ifch,*)'string mass too small (5)'
      xfla(1:2)='se'
      xflb(1:2)='se'
      return
 
      end
 
c-----------------------------------------------------------------------
      subroutine hastpr(stro,isplt)
c-----------------------------------------------------------------------
c processes a string
c out: isplt=0 : processing of stro finished
c           >0 : hastpr to be redone to process reduced string
c           -1 : error
c           -3 : hastpr to be redone with new epart,apart
c           -4 : haha collision to be redone bec of too fast ld ptl
c                             (only for iappl=2)
c           -5 : hastpr to be redone because of produced s=3/2 particle
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (nsi=6,nsix=100)
      real str(nsi,2),stro(nsi,nsix+1)
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      parameter (nflav=6)
      integer jc1(nflav,2),jc2(nflav,2),jc(nflav,2),ic1(2),ic2(2),ic(2)
     *,ic1x(2)
      parameter (ngau=129)
      common /cgau/xgau(ngau),qgau(ngau),wgau(ngau)
     */clead/lead,cooav3,cooav4 /ckol/kol
      real p1(4),p2(4)
      double precision ppx,ppy,ppz,ppe,ppm,pp1(4),pp2(4),ppp
      parameter (nha=nsix+2,mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      common/cjspli/jpart,nscc,nsccx,apart,epart,alead,elead,sgnsil
      common/cnsuc/nsuc
      common/cstsh/nstsh
      common/cpzstr/pzstrl,estrl,istrl,isea
      zero=0.
      if(isplt.eq.-3)then
      jpart=1
      isplt=0
      elseif(isplt.eq.0)then
      jpart=0
      endif
      isplt0=isplt
 
      call utkstr(stro,kmax)
 
c     zero string
c     -----------
      if(kmax.eq.0)then
      if(ish.ge.91)write(ifch,*)'zero string'
      isplt=0
      return
      endif
 
c     print
c     -----
      if(ish.ge.91)then
      write(ifch,105)(stro(i,1),i=1,4),(nint(stro(i,1)),i=5,6)
      if(kmax.gt.1)then
      do 8 k=2,kmax
8     write(ifch,104)(stro(i,k),i=1,4),(nint(stro(i,k)),i=5,6)
      endif
105   format(' str: ',4f11.5,2i8)
104   format('      ',4f11.5,2i8)
      endif
 
c     check leading object
c     --------------------
      do 3 i=1,2
3     ic1(i)=nint(stro(4+i,1))
      call idcomi(ic1,ic1x)
      do 4 i=1,2
4     stro(4+i,1)=ic1x(i)
      call iddeco(ic1x,jc1)
      npa=0
      do 7 n=1,nflav
      do 7 i=1,2
7     npa=npa+jc1(n,i)
 
c     split string
c     ------------
           if(kmax.gt.2.or.kmax.eq.2.and.isplt.gt.0
     *                 .or.kmax.eq.2.and.npa.gt.3
     *                 .or.kmax.eq.2.and.lead.eq.1)then
           if(isplt.eq.0)then
      nsuc=0
      kmaxor=kmax
      sgnsil=1
      if(stro(3,1).lt.0.)sgnsil=-1
      do 17 n=1,nflav
      do 17 i=1,2
17    jc2(n,i)=0
      do 15 k=1,kmax
      do 14 i=1,2
14    ic1(i)=nint(abs(stro(4+i,k)))
      call iddeco(ic1,jc1)
      do 16 n=1,nflav
      do 16 i=1,2
      jc2(n,i)=jc2(n,i)+jc1(n,i)
      if(n.gt.4.and.jc2(n,i).ne.0)
     *call utstop('hastpr: flavour > 4                     ')
16    continue
15    continue
      keu=jc2(1,1)-jc2(1,2)
      ked=jc2(2,1)-jc2(2,2)
      kes=jc2(3,1)-jc2(3,2)
      kec=jc2(4,1)-jc2(4,2)
      alead=utamnu(keu,ked,kes,kec,0)
c-c   alead=0
      elead=stro(4,1)
      if(jpart.eq.0)then
      nsccx=kmaxor-1
      nstr0=nstr
      else
      nsccx=max(1,nscc)
      nstr=nstr0
      endif
      apart=alead/nsccx
      epart=elead/nsccx
      if(alead.gt.elead)then
      if(ish.ge.90)then
      call utmsg('hastpr')
      write(ifch,*)'*****  alead > elead  ==> ret code = -1'
      write(ifch,*)'alead=',alead,'   elead=',elead
      call utmsgf
      endif
      isplt=-1
      return
      endif
           endif
      call jsplit(stro,str,kolsp,ier,kmaxor)
      if(ier.eq.1)then
      isplt=-1
      return
      endif
      if(ier.eq.3)then
      isplt=-3
      return
      endif
      isplt=isplt+1
      if(ier.eq.2)then
      if(ish.ge.91)write(ifch,*)'multistring: absorption of one leg'
      return
      endif
      kolz=kolsp
      if(kolz.lt.1.or.kolz.gt.kollmx)write(ifch,*)'kolz=',kolz
      if(kolz.lt.1.or.kolz.gt.kollmx)
     *call utstop('hastpr: kolz out of range (1)           ')
      kin=1
           else
      kin=0
      if(isplt.gt.0)kin=1
      do 5 n=1,nsi
      do 5 i=1,2
      str(n,i)=stro(n,i)
      if(n.ge.5)str(n,i)=abs(str(n,i))
5     continue
      if(isplt.gt.0)then
      kolz=kproja(2,iproj)
      if(sgnsil.lt.0.)kolz=ktarga(2,itarg)
      elseif(lead.eq.1.and.kmax.eq.2)then
      kolz=kproja(2,iproj)
      if(str(3,1).lt.0.)kolz=ktarga(2,itarg)
      else
      kolz=kol
      endif
      if(kolz.lt.1.or.kolz.gt.kollmx)then
      call utmsg('hastpr')
      write(ifch,*)'*****  kolz out of range;   kolz:',kolz
     *,'   set to:',kol
      write(ifch,105)(str(i,1),i=1,4),(nint(str(i,1)),i=5,6)
      write(ifch,104)(str(i,2),i=1,4),(nint(str(i,2)),i=5,6)
      call utmsgf
      kolz=kol
      endif
      isplt=0
           endif
 
c     add two jets
c     ------------
      do 1 i=1,2
      ic1(i)=nint(str(4+i,1))
1     ic2(i)=nint(str(4+i,2))
      ia1=ic1(1)+ic1(2)
      ia2=ic2(1)+ic2(2)
           if(ia1.eq.0.and.ia2.eq.0)then
      write(ifch,*)'stro:'
      write(ifch,105)(stro(i,1),i=1,4),(nint(stro(i,1)),i=5,6)
      if(kmax.gt.1)then
      do 18 k=2,kmax
18    write(ifch,104)(stro(i,k),i=1,4),(nint(stro(i,k)),i=5,6)
      endif
      write(ifch,*)'str:'
      write(ifch,105)(str(i,1),i=1,4),(nint(str(i,1)),i=5,6)
      write(ifch,104)(str(i,2),i=1,4),(nint(str(i,2)),i=5,6)
      call utstop('hastpr: str=0                           ')
           endif
      call iddeco(ic1,jc1)
      call iddeco(ic2,jc2)
      n1=0
      n2=0
      do 9 n=1,nflav
      do 9 i=1,2
      n1=n1+jc1(n,i)
      n2=n2+jc2(n,i)
      jc(n,i)=jc1(n,i)+jc2(n,i)
      if(n.gt.4.and.jc(n,i).ne.0)
     *call utstop('hastpr: flavour > 4                     ')
9     continue
      keu=jc(1,1)-jc(1,2)
      ked=jc(2,1)-jc(2,2)
      kes=jc(3,1)-jc(3,2)
      kec=jc(4,1)-jc(4,2)
      amstr=utamnu(keu,ked,kes,kec,6)
      call idenco(jc,ic,ireten)
      if(ireten.eq.1)
     *call utstop('hastpr: idenco ret code = 1             ')
      idstr=idtra(ic,0,0,3)
 
c     determine rot
c     -------------
      do 10 i=1,4
      pp1(i)=str(i,1)
10    pp2(i)=str(i,2)
      ppx=pp1(1)+pp2(1)
      ppy=pp1(2)+pp2(2)
      ppz=pp1(3)+pp2(3)
      ppp=dsqrt(ppx**2+ppy**2+ppz**2)
           if(ia1.ne.0.and.ia2.ne.0)then
      pp1(4)=dsqrt(pp1(1)**2+pp1(2)**2+pp1(3)**2)
      pp2(4)=dsqrt(pp2(1)**2+pp2(2)**2+pp2(3)**2)
      ppe=pp1(4)+pp2(4)
      ppm=dsqrt((ppe-ppp)*(ppe+ppp))
      call utlob2(1,ppx,ppy,ppz,ppe,ppm,pp1(1),pp1(2),pp1(3),pp1(4))
      call utlob2(1,ppx,ppy,ppz,ppe,ppm,pp2(1),pp2(2),pp2(3),pp2(4))
           else
      ppe=pp1(4)+pp2(4)
      if((ppe-ppp)*(ppe+ppp).gt.0d0)then
      ppm=dsqrt((ppe-ppp)*(ppe+ppp))
      else
      ppm=0
      ppe=ppp
      endif
      do 12 i=1,4
      pp1(i)=0.
12    pp2(i)=0.
           endif
      px=ppx
      py=ppy
      pz=ppz
      e=ppe
      am=ppm
      p=ppp
      do 6 i=1,4
      p1(i)=pp1(i)
6     p2(i)=pp2(i)
      if(ish.ge.92)then
      write(ifch,107)(p1(i),i=1,4),ic1
      write(ifch,107)(p2(i),i=1,4),ic2
107   format(' jet: ',4f11.5,2i8)
      endif
      if(p1(3).ne.0.)then
      dircn=p1(3)/abs(p1(3))
      else
      dircn=1
      endif
 
c     mass adjustment
c     ---------------
      amzero=0
      call idres(idstr,amzero,idzero,iadj)
      idaz=iabs(idzero)/10
      wthisp=0.33   !weight for high spin state
      amadd=0.300   !mass to be added to get high spin state
      if(n1+n2.eq.2)then
      wthisp=0.33   !mesons
      amadd=0.650   !mesons
      if(idaz.eq.13.or.idaz.eq.23)then
      wthisp=0.33   !kaons
      amadd=0.650   !kaons
      endif
      endif

      am0=am
      call idres(idstr,am,idstrr,iadj)
 
c diffractive particle:
c-c   if((ia1.eq.0.or.ia2.eq.0).and.isplt0.eq.0)then
c-c   if(mod(idstrr,10).eq.0.and.rangen().lt.0.1)then
c-c   am=0
c-c   call idres(idstr,am,idstrr,iadj)
c-c   am=am+.300
c-c   call idres(idstr,am,idstrr,iadj)
c-c   endif
c-c   endif
 
c remnant:
      if((ia1.eq.0.or.ia2.eq.0).and.isplt0.gt.0.and.nsuc.gt.0)then
      am=0
      call idres(idstr,am,idstrr,iadj)
      if(n1+n2.eq.2.or.mod(idstrr,10).eq.0)then
      if(rangen().lt.wthisp)am=am+amadd
      else
      if(rangen().gt.wthisp)then
      isplt=-5
      return
      endif
      endif
      call idres(idstr,am,idstrr,iadj)
      endif
 
c all legs absorbed
      if((ia1.eq.0.or.ia2.eq.0).and.isplt0.gt.0.and.nsuc.eq.0)then
      if(isea.eq.0)then
      isplt=-4
      return
      endif
      am=0
      if(istrl.eq.1)am=sqrt( engy**2*(1-abs(pzstrl)/pnllx)+.94**2 )
      call idres(idstr,am,idstrr,iadj)
      if(istrl.ne.1)then
      if(mod(idstrr,10).eq.0)am=am+rangen()
      if(mod(idstrr,10).ne.0)am=am+rangen()-.300
      call idres(idstr,am,idstrr,iadj)
      endif
      endif
 
c leading string = hadron:
      if(lead.eq.1.and.ia1.ne.0.and.ia2.ne.0
     *.and.isplt0.eq.0.and.isplt.eq.0.and.idstrr.ne.0)then
      am=0
      call idres(idstr,am,idstrr,iadj)
      if(mod(idstrr,10).eq.0.and.rangen().lt.wthisp)am=am+amadd
      call idres(idstr,am,idstrr,iadj)
      endif
 
      idstr=idstrr
      if(idstr.eq.0.and.am.lt.amstr)am=amstr
      ppm=am
      e=dsqrt(ppp**2+ppm**2)
 
c     write /cstr/
c     ------------
      nstr=nstr+1
      if(nstr.gt.mxstr)
     *call utstop('hastpr: nstr>mxstr                      ')
      if(lead.eq.0)nstsh=1
      nsuc=nsuc+1
      irlstr(nstr)=nsuc
      if(isplt0.eq.0.and.isplt.eq.0)irlstr(nstr)=0
      icstr(1,nstr)=ic1(1)
      icstr(2,nstr)=ic1(2)
      icstr(3,nstr)=ic2(1)
      icstr(4,nstr)=ic2(2)
      pstr(1,nstr)=px
      pstr(2,nstr)=py
      pstr(3,nstr)=pz
      pstr(4,nstr)=e
      pstr(5,nstr)=am
      rotstr(1,nstr)=p1(1)
      rotstr(2,nstr)=p1(2)
      rotstr(3,nstr)=p1(3)
      if(p1(1).eq.0..and.p1(2).eq.0..and.p1(3).eq.0.)
     *rotstr(3,nstr)=1.
      do 2 i=1,4
2     xorstr(i,nstr)=coord(i,kolz)
      rd=0.8*sqrt(rangen())
      ph=2*3.14159*rangen()
      xorstr(1,nstr)=xorstr(1,nstr)+rd*cos(ph)
      xorstr(2,nstr)=xorstr(2,nstr)+rd*sin(ph)
      iorstr(nstr)=-kolz
      amsac=amsac+am
      if(ish.ge.91)then
      if(isplt.gt.0)then
      write(ifch,*)'split off string:'
      elseif(isplt0.gt.0)then
      write(ifch,*)'remainder:'
      elseif(lead.eq.1)then
      write(ifch,*)'ordinary baryonic string:'
      else
      write(ifch,*)'ordinary mesonic string:'
      endif
      write(ifch,106)nstr,(icstr(k,nstr),k=1,4)
     *,(pstr(i,nstr),i=3,5)
106   format(' /cstr/',i4,3x,4i7,2x,3(1x,e9.3))
      endif

      return
      end
 
c-----------------------------------------------------------------------
      subroutine hdecmp(bar,sil,six)
c     decomposes bar into sil,six
c-----------------------------------------------------------------------
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      real bar(nsi,nha),sil(nsi),six(nsi,nsix)
      do 1 n=1,nsi
1     sil(n)=bar(n,2)
      do 3 m=1,nsix
      sixsqr=0
      do 2 n=1,nsi
      six(n,m)=bar(n,2+m)
2     sixsqr=sixsqr+six(n,m)**2
      if(sixsqr.le.1e-5)return
3     continue
      return
      end
 
c-----------------------------------------------------------------------
      subroutine hrescl(j1,j2,psum,ifail)
c     rescales string momenta of strings j1-j2 to have total mom psum.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      dimension psum(5)
      double precision ppsum(5),padd(5),pp(5)
      data errlim/.001/

      ifail=1

      do k=1,5
      ppsum(k)=psum(k)
      enddo

      if(j1.ge.j2)
     *call utstop('hrescl: j1 .ge. j2                      ')

      do 100 k=1,5
100   padd(k)=0.
      do 110 j=j1,j2
      do 110 k=1,5
      padd(k)=padd(k)+pstr(k,j)
110   continue
           if(padd(5).ge.ppsum(5))then
      if(ish.ge.90)then
      call utmsg('hrescl')
      write(ifch,*)'*****  sum of str masses .ge. psum(5)'
      do 1 j=j1,j2
1     write(ifch,109)j,(icstr(k,j),k=1,4)
     *,(pstr(l,j),l=3,5)
109   format(' /cstr/',i4,3x,4i7,2x,3(1x,e9.3))
      write(ifch,*)'ppsum(345):',(sngl(ppsum(k)),k=3,5)
      call utmsgf
      endif
      return
           endif
      padd(5)=padd(4)**2-padd(1)**2-padd(2)**2-padd(3)**2
           if(padd(5).le.0)then
      do 2 j=j1,j2
2     write(ifch,108)j,(pstr(l,j),l=1,5)
108   format(' /cstr/',i4,2x,5(1x,e9.3))
      call utstop('hrescl: mass**2 of string-sum negative  ')
           endif
      padd(5)=sqrt(padd(5))
 
c     boost strings to rest
c     ---------------------
           do j=j1,j2
      do k=1,4
      pp(k)=pstr(k,j)
      enddo 
      call utlob2(1,padd(1),padd(2),padd(3),padd(4),padd(5)    
     *,pp(1),pp(2),pp(3),pp(4))
      do k=1,4
      pstr(k,j)=pp(k)
      enddo 
           enddo  
 
c     rescale momenta in rest frame
c     -----------------------------
      scal=1.
      do 301 ipass=1,200
      sum=0.
      do 310 j=j1,j2
      do 320 k=1,3
320   pstr(k,j)=scal*pstr(k,j)
      pstr(4,j)=sqrt(pstr(1,j)**2+pstr(2,j)**2+pstr(3,j)**2
     *+pstr(5,j)**2)
      sum=sum+pstr(4,j)
310   continue
      scal=psum(5)/sum
      if(abs(scal-1.).le.errlim) goto300
301   continue
      if(ish.ge.90)then
      call utmsg('hrescl')
      write(ifch,*)'*****  scal=',scal
      call utmsgf
      endif
300   continue
 
c     boost back with ppsum
c     ---------------------
           do j=j1,j2
      do k=1,4
      pp(k)=pstr(k,j)
      enddo 
      call utlob2(-1,ppsum(1),ppsum(2),ppsum(3),ppsum(4),ppsum(5)    
     *,pp(1),pp(2),pp(3),pp(4))
      do k=1,4
      pstr(k,j)=pp(k)
      enddo 
           enddo  

      ifail=0
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idchrg(id,chrg)
c     computes charge of particle with ident code id
c     ichrg must be dimensioned nqlep+12
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      dimension ichrg(53),ifl(3)
      data ichrg/0,2,-1,-1,2,-1,2,-1,2,0,0,0,-3,0,-3,0,-3,0,-3,3*0
     *,2,-1,-1,2,-1,2,-1,2,0,0,0,-3,0,-3,0,-3,0,-3,3,0
     *,3,0,0,0,3,3,3,6,6,6,0/
      idabs=iabs(id)
      call idflav(id,ifl(1),ifl(2),ifl(3),jspin,index)
      if(idabs.lt.100) goto200
      isum=0
      do 100 i=1,3
      isum=isum+ichrg(iabs(ifl(i))+1)*isign(1,ifl(i))
  100 continue
      chrg=isum/3.
      return
200   chrg=ichrg(index+1)*isign(1,id)
      chrg=chrg/3.
      return
      end

c-----------------------------------------------------------------------
      subroutine idcomk(ic)
c     compactifies ic
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer ic(2),icx(2),jc(nflav,2)
      call idcomp(ic,icx,jc,1)
      ic(1)=icx(1)
      ic(2)=icx(2)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idcomi(ic,icx)
c     compactifies ic
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer ic(2),icx(2),jc(nflav,2)
      call idcomp(ic,icx,jc,1)
      return
      end
  
c-----------------------------------------------------------------------
      subroutine idcomj(jc)
c     compactifies jc
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer ic(2),icx(2),jc(nflav,2)
      call idcomp(ic,icx,jc,2)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idcomp(ic,icx,jc,im)
c-----------------------------------------------------------------------
c     compactifies ic,jc
c     input: im (1 or 2)
c            ic (if im=1)
c            jc (if im=2) 
c     output: icx (if im=1)
c             jc  
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer ic(2),icx(2),jc(nflav,2)
      if(im.eq.1)call iddeco(ic,jc)
      icx(1)=0
      icx(2)=0
           do n=1,nflav
           do j=1,2
      if(jc(n,j).ne.0)goto1
           enddo
           enddo
      return
1     continue
      nq=0
      na=0
           do n=1,nflav
      nq=nq+jc(n,1)
      na=na+jc(n,2)
           enddo
      l=0
           do n=1,nflav
      k=min0(jc(n,1),jc(n,2))
      if(nq.eq.1.and.na.eq.1)k=0
      jc(n,1)=jc(n,1)-k
      jc(n,2)=jc(n,2)-k
      if(jc(n,1).lt.0.or.jc(n,2).lt.0)
     *call utstop('idcomp: jc negative                     ')
      l=l+jc(n,1)+jc(n,2)
           enddo
           if(l.eq.0)then
      jc(1,1)=1              
      jc(1,2)=1
           endif
           if(im.eq.1)then
      call idenco(jc,icx,ireten)
      if(ireten.eq.1)
     *call utstop('idcomp: idenco ret code = 1             ')
           endif
      return
      end
 
c-----------------------------------------------------------------------
      integer function idcon(ifl1,ifl2,ifail)
c     constructs particle identification code for ifl1+ifl2
c-----------------------------------------------------------------------
      common/didib/ndidib
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      real pmix1(3,2),pmix2(3,2),pspin1(8)
      data pmix1/.25,.25,.5,0.,.5,1./,pmix2/.5,.5,1.,0.,0.,1./
      do 1 i=1,8
      if(i.le.3)pspin1(i)=pspinl
      if(i.gt.3)pspin1(i)=pspinh
1     continue
      ifail=0
 
c               meson:
                if(iabs(ifl1).le.6.and.iabs(ifl2).le.6)then
      if(ifl1*ifl2.gt.0)
     *call utstop('idcon : ifl1*ifl2.gt.0                  ')
      ihigh=max0(iabs(ifl1),iabs(ifl2))
      jspin=int(rangen()+pspin1(ihigh))
      id1=ifl1
      id2=ifl2
           if(id1+id2.eq.0)then
      rnd=rangen()
      id1=iabs(id1)
      id1=int(pmix1(id1,jspin+1)+rnd)+int(pmix2(id1,jspin+1)+rnd)+1
      id2=-id1
           endif
           if(iabs(id1).gt.iabs(id2))then
      isave=id1
      id1=id2
      id2=isave
           endif
      idcon=isign(100*iabs(id1)+10*iabs(id2)+jspin,id1)
 
c               baryon:
                elseif(mod(ifl1,100).eq.0.and.iabs(ifl2).le.6
     *             .or.mod(ifl2,100).eq.0.and.iabs(ifl1).le.6)then
           if(mod(ifl1,100).eq.0)then
      id3=mod(ifl1/100,10)
      id2=ifl1/1000
      id1=ifl2
           elseif(mod(ifl2,100).eq.0)then
      id3=mod(ifl2/100,10)
      id2=ifl2/1000
      id1=ifl1
           endif
           if(iabs(id1).gt.iabs(id2))then
      iswap=id1
      id1=id2
      id2=iswap
           endif
           if(iabs(id2).gt.iabs(id3))then
      iswap=id2
      id2=id3
      id3=iswap
           endif
           if(iabs(id1).gt.iabs(id2))then
      iswap=id1
      id1=id2
      id2=iswap
           endif
      jspin=1
           if(.not.(id1.eq.id2.and.id2.eq.id3))then
      ihigh=iabs(id3)
      jspin=int(rangen()+pspin1(ihigh))
           endif
           if(jspin.ne.1.and.id1.ne.id2.and.id2.ne.id3
     *      .and.rangen().le.pispn)then
      iswap=id1
      id1=id2
      id2=iswap
           endif
      idcon=1000*iabs(id1)+100*iabs(id2)+10*iabs(id3)+jspin
      idcon=isign(idcon,ifl1)
 
               else
      ifail=1
      r=rangen()
      if(r.le..5)idcon=121
      if(r.gt..5)idcon=-121
                endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine iddeco(ic,jc)
c     decode particle id
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer jc(nflav,2),ic(2)
      ici=ic(1)
      jc(6,1)=mod(ici,10)
      jc(5,1)=mod(ici/10,10)
      jc(4,1)=mod(ici/100,10)
      jc(3,1)=mod(ici/1000,10)
      jc(2,1)=mod(ici/10000,10)
      jc(1,1)=mod(ici/100000,10)
      ici=ic(2)
      jc(6,2)=mod(ici,10)
      jc(5,2)=mod(ici/10,10)
      jc(4,2)=mod(ici/100,10)
      jc(3,2)=mod(ici/1000,10)
      jc(2,2)=mod(ici/10000,10)
      jc(1,2)=mod(ici/100000,10)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idenco(jc,ic,ireten)
c     encode particle id
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer jc(nflav,2),ic(2)
      ireten=0
      ic(1)=0
      do 20 i=1,nflav
      if(jc(i,1).ge.10)goto22
20    ic(1)=ic(1)+jc(i,1)*10**(nflav-i)
      ic(2)=0
      do 21 i=1,nflav
      if(jc(i,2).ge.10)goto22
21    ic(2)=ic(2)+jc(i,2)*10**(nflav-i)
      return
22    ireten=1
      ic(1)=0
      ic(2)=0
      return
      end

c-----------------------------------------------------------------------
      subroutine idenct(jc,id,ib1,ib2,ib3,ib4)
c     encode particle id
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer jc(nflav,2),ic(2)

      do 40 nf=1,nflav
      do 40 ij=1,2
      if(jc(nf,ij).ge.10)id=7*10**8
40    continue
           if(id/10**8.ne.7)then
      call idenco(jc,ic,ireten)
      if(ireten.eq.1)
     *call utstop('idenct: idenco ret code = 1             ')
      if(mod(ic(1),100).ne.0.or.mod(ic(2),100).ne.0)then
      id=9*10**8
      else
      id=8*10**8+ic(1)*100+ic(2)/100
      endif
           else
      call idtrbi(jc,ib1,ib2,ib3,ib4)
      id=id
     *+mod(jc(1,1)+jc(2,1)+jc(3,1)+jc(4,1),10**4)*10**4
     *+mod(jc(1,2)+jc(2,2)+jc(3,2)+jc(4,2),10**4)
           endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idflav(id,ifl1,ifl2,ifl3,jspin,index)
c     unpacks the ident code id=+/-ijkl
c
c          mesons--
c          i=0, j<=k, +/- is sign for j
c          id=110 for pi0, id=220 for eta, etc.
c
c          baryons--
c          i<=j<=k in general
c          j<i<k for second state antisymmetric in (i,j), eg. l = 2130
c
c          other--
c          id=1,...,6 for quarks
c          id=9 for gluon
c          id=10 for photon
c          id=11,...,16 for leptons
c          id=20 for ks, id=-20 for kl
c
c          i=21...26 for scalar quarks
c          i=29 for gluino
c          i=30 for photino
c          i=31...36 for scalar leptons
c          i=39 for wino
c          i=40 for zino
c
c          id=80 for w+
c          id=81,...,89 for higgs mesons
c          id=90 for z0
c
c          diquarks--
c          id=+/-ij00, i<j for diquark composed of i,j.
c
c          index is a sequence number used internally
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      parameter ( nqlep=41,nmes=2,nbary=2)
      idabs=iabs(id)
      i=idabs/1000
      j=mod(idabs/100,10)
      k=mod(idabs/10,10)
      jspin=mod(idabs,10)
      if(id.ne.0.and.mod(id,100).eq.0) goto300
      if(j.eq.0) goto200
      if(i.eq.0) goto100
c          baryons
c          only x,y baryons are qqx, qqy, q=u,d,s.
      ifl1=isign(i,id)
      ifl2=isign(j,id)
      ifl3=isign(k,id)
      if(k.le.6) then
        index=max0(i-1,j-1)**2+i+max0(i-j,0)+(k-1)*k*(2*k-1)/6
     1  +109*jspin+36*nmes+nqlep+11
      else
        index=max0(i-1,j-1)**2+i+max0(i-j,0)+9*(k-7)+91
     1  +109*jspin+36*nmes+nqlep+11
      endif
      return
c          mesons
100   continue
      ifl1=0
      ifl2=isign(j,id)
      ifl3=isign(k,-id)
      index=j+k*(k-1)/2+36*jspin+nqlep
      index=index+11
      return
c          quarks, leptons, etc
200   continue
      ifl1=0
      ifl2=0
      ifl3=0
      jspin=0
      index=idabs
      if(idabs.lt.20) return
c          define index=20 for ks, index=21 for kl
      index=idabs+1
      if(id.eq.20) index=20
c          index=nqlep+1,...,nqlep+11 for w+, higgs, z0
      if(idabs.lt.80) return
      index=nqlep+idabs-79
      return
300   ifl1=isign(i,id)
      ifl2=isign(j,id)
      ifl3=0
      jspin=0
      index=0
      return
      end
 
c-----------------------------------------------------------------------
      function idlabl(id)
c     returns the character*8 label for the particle id
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      parameter ( nqlep=41,nmes=2,nbary=2)
c
      character*8 idlabl
      character*8 llep,lmes0,lmes1,lbar0,labar0,lbar1,labar1
      character*8 lqq,laqq
      dimension llep(104)
      dimension lmes0(64),lmes1(64)
      dimension lbar0(109),labar0(109),lbar1(109),labar1(109)
      dimension lqq(21),laqq(21)
c          diquark labels
      data lqq/
     1'uu0. ','ud0. ','dd0. ','us0. ','ds0. ','ss0. ','uc0. ','dc0. ',
     2'sc0. ','cc0. ','ub0. ','db0. ','sb0. ','cb0. ','bb0. ','ut0. ',
     3'dt0. ','st0. ','ct0. ','bt0. ','tt0. '/
      data laqq/
     1'auu0.','aud0.','add0.','aus0.','ads0.','ass0.','auc0.','adc0.',
     2'asc0.','acc0.','aub0.','adb0.','asb0.','acb0.','abb0.','aut0.',
     3'adt0.','ast0.','act0.','abt0.','att0.'/
c          quark and lepton labels
      data llep/
     *'     ','up   ','ub   ','dn   ','db   ','st   ','sb   ','ch   ',
     *'cb   ','bt   ','bb   ','tp   ','tb   ','y    ','yb   ','x    ',
     *'xb   ','gl   ','err  ','gm   ','err  ','nue  ','anue ','e-   ',
     *'e+   ','num  ','anum ','mu-  ','mu+  ','nut  ','anut ','tau- ',
     *'tau+ ','err  ','err  ','err  ','err  ','err  ','err  ','ks   ',
     *'err  ','err  ','kl   ',
     *'upss ','ubss ','dnss ','dbss ','stss ','sbss ','chss ','cbss ',
     *'btss ','bbss ','tpss ','tbss ','err  ','err  ','err  ','err  ',
     *'glss ','err  ','gmss ','err  ','ness ','aness','e-ss ','e+ss ',
     *'nmss ','anmss','mu-ss','mu+ss','ntss ','antss','t-ss ','t+ss ',
     *'err  ','err  ','err  ','err  ','w+ss ','w-ss ','z0ss ','err  ',
     *'w+   ','w-   ','h10  ','ah10 ','h20  ','ah20 ','h30  ','ah30 ',
     *'h4+  ','h4-  ','h5+  ','h5-  ','h6+  ','h6-  ','h7++ ','h7-- ',
     *'h8++ ','h8-- ','h9++ ','h9-- ','z0   '/
c          0- meson labels
      data lmes0/
     1'pi0  ','pi+  ','eta  ','pi-  ','k+   ','k0   ','etap ','ak0  ',
     2'k-   ','ad0  ','d-   ','f-   ','etac ','f+   ','d+   ','d0   ',
     2'ub.  ','db.  ','sb.  ','cb.  ','bb.  ','bc.  ','bs.  ','bd.  ',
     3'bu.  ','ut.  ','dt.  ','st.  ','ct.  ','bt.  ','tt.  ','tb.  ',
     4'tc.  ','ts.  ','td.  ','tu.  ','uy.  ','dy.  ','sy.  ','cy.  ',
     5'by.  ','ty.  ','yy.  ','yt.  ','yb.  ','yc.  ','ys.  ','yd.  ',
     6'yu.  ','ux.  ','dx.  ','sx.  ','cx.  ','bx.  ','tx.  ','yx.  ',
     7'xx.  ','xy.  ','xt.  ','xb.  ','xc.  ','xs.  ','xd.  ','xu.  '/
c          1- meson labels
      data lmes1/
     1'rho0 ','rho+ ','omeg ','rho- ','k*+  ','k*0  ','phi  ','ak*0 ',
     2'k*-  ','ad*0 ','d*-  ','f*-  ','jpsi ','f*+  ','d*+  ','d*0  ',
     3'ub*  ','db*  ','sb*  ','cb*  ','upsl ','bc*  ','bs*  ','bd*  ',
     4'bu*  ','ut*  ','dt*  ','st*  ','ct*  ','bt*  ','tt*  ','tb*  ',
     5'tc*  ','ts*  ','td*  ','tu*  ','uy*  ','dy*  ','sy*  ','cy*  ',
     6'by*  ','ty*  ','yy*  ','yt*  ','yb*  ','yc*  ','ys*  ','yd*  ',
     7'yu*  ','ux*  ','dx*  ','sx*  ','cx*  ','bx*  ','tx*  ','yx*  ',
     8'xx*  ','xy*  ','xt*  ','xb*  ','xc*  ','xs*  ','xd*  ','xu*  '/
c          1/2+ baryon labels
      data lbar0/
     1'err  ','p    ','n    ','err  ','err  ','s+   ','s0   ','s-   ',
     2'l    ','xi0  ','xi-  ','err  ','err  ','err  ','sc++ ','sc+  ',
     3'sc0  ','lc+  ','usc. ','dsc. ','ssc. ','sdc. ','suc. ','ucc. ',
     4'dcc. ','scc. ','err  ','err  ','err  ','err  ','uub. ','udb. ',
     5'ddb. ','dub. ','usb. ','dsb. ','ssb. ','sdb. ','sub. ','ucb. ',
     6'dcb. ','scb. ','ccb. ','csb. ','cdb. ','cub. ','ubb. ','dbb. ',
     7'sbb. ','cbb. ','err  ','err  ','err  ','err  ','err  ','utt. ',
     8'udt. ','ddt. ','dut. ','ust. ','dst. ','sst. ','sdt. ','sut. ',
     9'uct. ','dct. ','sct. ','cct. ','cst. ','cdt. ','cut. ','ubt. ',
     1'dbt. ','sbt. ','cbt. ','bbt. ','bct. ','bst. ','bdt. ','but. ',
     2'utt. ','dtt. ','stt. ','ctt. ','btt. ','err  ','err  ','err  ',
     3'err  ','err  ','err  ','uuy. ','udy. ','ddy. ','duy. ','usy. ',
     4'dsy. ','ssy. ','sdy. ','suy. ','uux. ','udx. ','ddx. ','dux. ',
     5'usx. ','dsx. ','ssx. ','sdx. ','sux. '/
      data labar0/
     1'err  ','ap   ','an   ','err  ','err  ','as-  ','as0  ','as+  ',
     2'al   ','axi0 ','axi+ ','err  ','err  ','err  ','asc--','asc- ',
     3'asc0 ','alc- ','ausc.','adsc.','assc.','asdc.','asuc.','aucc.',
     4'adcc.','ascc.','err  ','err  ','err  ','err  ','auub.','audb.',
     5'addb.','adub.','ausb.','adsb.','assb.','asdb.','asub.','aucb.',
     6'adcb.','ascb.','accb.','acsb.','acdb.','acub.','aubb.','adbb.',
     7'asbb.','acbb.','err  ','err  ','err  ','err  ','err  ','autt.',
     8'audt.','addt.','adut.','aust.','adst.','asst.','asdt.','asut.',
     9'auct.','adct.','asct.','acct.','acst.','acdt.','acut.','aubt.',
     1'adbt.','asbt.','acbt.','abbt.','abct.','abst.','abdt.','abut.',
     2'autt.','adtt.','astt.','actt.','abtt.','err  ','err  ','err  ',
     3'err  ','err  ','err  ','auuy.','audy.','addy.','aduy.','ausy.',
     4'adsy.','assy.','asdy.','asuy.','auux.','audx.','addx.','adux.',
     5'ausx.','adsx.','assx.','asdx.','asux.'/
c          3/2+ baryon labels
      data lbar1/
     1'dl++ ','dl+  ','dl0  ','dl-  ','err  ','s*+  ','s*0  ','s*-  ',
     2'err  ','xi*0 ','xi*- ','om-  ','err  ','err  ','uuc* ','udc* ',
     3'ddc* ','err  ','usc* ','dsc* ','ssc* ','err  ','err  ','ucc* ',
     4'dcc* ','scc* ','ccc* ','err  ','err  ','err  ','uub* ','udb* ',
     5'ddb* ','err  ','usb* ','dsb* ','ssb* ','err  ','err  ','ucb* ',
     6'dcb* ','scb* ','ccb* ','err  ','err  ','err  ','ubb* ','dbb* ',
     7'sbb* ','cbb* ','bbb* ','err  ','err  ','err  ','err  ','utt* ',
     8'udt* ','ddt* ','err  ','ust* ','dst* ','sst* ','err  ','err  ',
     9'uct* ','dct* ','sct* ','cct* ','err  ','err  ','err  ','ubt* ',
     1'dbt* ','sbt* ','cbt* ','bbt* ','err  ','err  ','err  ','err  ',
     2'utt* ','dtt* ','stt* ','ctt* ','btt* ','ttt* ','err  ','err  ',
     3'err  ','err  ','err  ','uuy* ','udy* ','ddy* ','err  ','usy* ',
     4'dsy* ','ssy* ','err  ','err  ','uux* ','udx* ','ddx* ','err  ',
     5'usx* ','dsx* ','ssx* ','err  ','err  '/
      data labar1/
     1'adl--','adl- ','adl0 ','adl+ ','err  ','as*- ','as*0 ','as*+ ',
     2'err  ','axi*0','axi*+','aom+ ','err  ','err  ','auuc*','audc*',
     3'addc*','err  ','ausc*','adsc*','assc*','err  ','err  ','aucc*',
     4'adcc*','ascc*','accc*','err  ','err  ','err  ','auub*','audb*',
     5'addb*','err  ','ausb*','adsb*','assb*','err  ','err  ','aucb*',
     6'adcb*','ascb*','accb*','err  ','err  ','err  ','aubb*','adbb*',
     7'asbb*','acbb*','abbb*','err  ','err  ','err  ','err  ','autt*',
     8'audt*','addt*','err  ','aust*','adst*','asst*','err  ','err  ',
     9'auct*','adct*','asct*','acct*','err  ','err  ','err  ','aubt*',
     1'adbt*','asbt*','acbt*','abbt*','err  ','err  ','err  ','err  ',
     2'autt*','adtt*','astt*','actt*','abtt*','attt*','err  ','err  ',
     3'err  ','err  ','err  ','auuy*','audy*','addy*','err  ','ausy*',
     4'adsy*','assy*','err  ','err  ','auux*','audx*','addx*','err  ',
     5'ausx*','adsx*','assx*','err  ','err  '/
c          entry
      call idflav(id,ifl1,ifl2,ifl3,jspin,index)
      if(iabs(id).lt.100) goto200
      if(iabs(id).lt.1000) goto100
      if(id.ne.0.and.mod(id,100).eq.0) goto300
c          baryons
      index=index-109*jspin-36*nmes-nqlep
      index=index-11
      if(jspin.eq.0.and.id.gt.0) idlabl=lbar0(index)
      if(jspin.eq.0.and.id.lt.0) idlabl=labar0(index)
      if(jspin.eq.1.and.id.gt.0) idlabl=lbar1(index)
      if(jspin.eq.1.and.id.lt.0) idlabl=labar1(index)
      return
c          mesons
100   continue
      i=max0(ifl2,ifl3)
      j=-min0(ifl2,ifl3)
      index=max0(i-1,j-1)**2+i+max0(i-j,0)
      if(jspin.eq.0) idlabl=lmes0(index)
      if(jspin.eq.1) idlabl=lmes1(index)
      return
c          quarks, leptons, etc.
200   continue
      index=2*index
      if(id.le.0) index=index+1
      idlabl=llep(index)
      return
300   i=iabs(ifl1)
      j=iabs(ifl2)
      index=i+j*(j-1)/2
      if(id.gt.0) idlabl=lqq(index)
      if(id.lt.0) idlabl=laqq(index)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idmass(id,amass)
c     returns the mass of the particle with ident code id.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      dimension ammes0(10),ammes1(10),ambar0(30),ambar1(30)
      dimension amlep(52)
      parameter ( nqlep=41,nmes=2,nbary=2)
c   quark masses changed to bare masses (m.hladik)
c     data amlep/.3,.3,.5,1.6,4.9,30.,-1.,-1.,0.,0.,
      data amlep/.005,.009,.180,1.6,4.9,30.,-1.,-1.,0.,0.,
     *0.,.511003e-3,0.,.105661,0.,1.807,3*-1.,.49767,.49767,
     *100.3,100.3,100.5,101.6,104.9,130.,2*-1.,100.,0.,
     *100.,100.005,100.,100.1,100.,101.8,2*-1.,100.,100.,
     *11*0./
c          0- meson mass table
      data ammes0/.13496,.13957,.5488,.49367,.49767,.9576,1.8633
     1,1.8683,2.030,2.976/
c          1- meson mass table
      data ammes1/.770,.770,.7826,.8881,.8922,1.0196,2.006,2.0086
     1,2.140,3.097/
c          1/2+ baryon mass table
      data ambar0/-1.,.93828,.93957,2*-1.,1.1894,1.1925,1.1974
     1,1.1156,1.3149,1.3213,3*-1.,2.43,2.43,2.43,2.26
     2,2.50,2.50,2.60,2.40,2.40,3.55,3.55,3.70,4*-1./
c          3/2+ baryon mass table
      data ambar1/1.232,1.232,1.232,1.232,-1.,1.3823,1.3820
     1,1.3875,-1.,1.5318,1.5350,1.6722,2*-1.
     2,2.63,2.63,2.63,-1.,2.70,2.70,2.80,2*-1.,3.75,3.75
     3,3.90,4.80,3*-1./
c          entry
      call idflav(id,ifl1,ifl2,ifl3,jspin,index)
      if(id.ne.0.and.mod(id,100).eq.0) goto400
      if(iabs(ifl1).gt.4.or.iabs(ifl2).gt.4.or.iabs(ifl3).gt.4)
     1goto300
      if(ifl2.eq.0) goto200
      if(ifl1.eq.0) goto100
c          baryons
      index=index-109*jspin-36*nmes-nqlep
      index=index-11
      amass=(1-jspin)*ambar0(index)+jspin*ambar1(index)
      return
c          mesons
100   continue
      index=index-36*jspin-nqlep
      index=index-11
      amass=(1-jspin)*ammes0(index)+jspin*ammes1(index)
      return
c          quarks and leptons
200   continue
      amass=amlep(index)
      return
c          b and t particles
300   continue
      amass=amlep(iabs(ifl2))+amlep(iabs(ifl3))-.03+.04*jspin
      if(ifl1.ne.0) amass=amass+amlep(iabs(ifl1))
      return
c          diquarks
400   amass=amlep(iabs(ifl1))+amlep(iabs(ifl2))
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idmix(ic,jspin,icm,idm)
c     accounts for flavour mixing
c-----------------------------------------------------------------------
      parameter (nflav=6)
      real pmix1(3,2),pmix2(3,2)
      integer ic(2),icm(2)
      data pmix1/.25,.25,.5,0.,.5,1./,pmix2/.5,.5,1.,0.,0.,1./
      icm(1)=0
      icm(2)=0
      idm=0
      i=ic(1)
      if(i.ne.ic(2))return
      id=0
      if(i.eq.100000)id=1
      if(i.eq. 10000)id=2
      if(i.eq.  1000)id=3
      if(id.eq.0)return
      rnd=rangen()
      idm=int(pmix1(id,jspin+1)+rnd)+int(pmix2(id,jspin+1)+rnd)+1
      icm(1)=10**(nflav-idm)
      icm(2)=ic(1)
      idm=idm*100+idm*10+jspin
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idquac(i,nq,ns,na,jc)
c     returns quark content of ptl i from /cptl/ .
c        nq = # quarks - # antiquarks
c        ns = # strange quarks - # strange antiquarks
c        na = # quarks + # antiquarks
c        jc(nflav,2) = jc-type particle identification code.
c-----------------------------------------------------------------------
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (nflav=6)
      integer jc(nflav,2),ic(2)
 
      if(iabs(idptl(i)).eq.20)then
      idptl(i)=230
      if(rangen().lt..5)idptl(i)=-230
      goto9999
      endif
 
      if(iabs(idptl(i)).lt.100)then
      nq=0
      ns=0
      do 1 n=1,nflav
      jc(n,1)=0
1     jc(n,2)=0
      return
      endif
 
9999  if(idptl(i)/10**8.ne.7)then
      call idtr4(idptl(i),ic)
      call iddeco(ic,jc)
      else
      call idtrb(ibptl(1,i),ibptl(2,i),ibptl(3,i),ibptl(4,i),jc)
      endif
      na=0
      nq=0
      do 53 n=1,nflav
      na=na+jc(n,1)+jc(n,2)
53    nq=nq+jc(n,1)-jc(n,2)
      ns=   jc(3,1)-jc(3,2)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idres(id,am,idr,iadj)
c     returns resonance id idr corresponding to mass am.
c     performs mass adjustment, if necessary (if so iadj=1, 0 else)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxindx=1000,mxre=100,mxma=11,mxmx=6)
      common/crema/indx(mxindx),rema(mxre,mxma),rewi(mxre,mxma)
     *,idmx(mxma,mxmx),icre1(mxre,mxma),icre2(mxre,mxma)
 
      if(am.eq.0.)am=1e-5
 
      idi=id
      ami=am
      idr=0
      iadj=0
 
      if(id.eq.0)goto9999
 
      do 3 k=1,mxmx
      do 3 m=2,mxma
      if(iabs(id).eq.idmx(m,k))id=idmx(1,k)*10*id/iabs(id)
3     continue
 
      ix=iabs(id)/10
      if(ix.lt.1.or.ix.gt.mxindx)
     *call utstop('idres: ix out of range.                 ')
      i=indx(ix)
      if(i.lt.1.or.i.gt.mxre)
     *call utstop('idres: particle not in table            ')
      do 1 j=1,mxma-1
      if(am.ge.rema(i,j).and.am.le.rema(i,j+1))then
      if(j-1.gt.9)
     *call utstop('idres: spin > 9                         ')
      idr=id/10*10+(j-1)*id/iabs(id)
      goto2
      endif
1     continue
      goto9999
2     continue
 
      do 4 k=1,mxmx
      if(ix.eq.idmx(1,k))then
      if(j.lt.1.or.j.gt.mxma-1)
     *call utstop('idres: index j out of range             ')
      if(idmx(j+1,k).ne.0)idr=idmx(j+1,k)*id/iabs(id)
      endif
4     continue
 
      iy=mod(iabs(idr),10)
      if(iy.gt.maxres)then
      iadj=0
      idr=0
      goto9999
      endif
 
      if(iy.ne.0.and.iy.ne.1)goto9999
 
      call idmass(idr,am)
      if(am.lt.0.)then
      write(ifch,*)'*****  error in idres: '
     *,'neg mass returned from idmass'
      write(ifch,*)'id,am(input):',idi,ami
      write(ifch,*)'idr,am:',idr,am
      call utstop('idres: neg mass returned from idmass    ')
      endif
      iadj=1
 
9999  id=idi
      if(.not.(ish.ge.93))return
      write(ifch,*)'return from idres. id,ami,am,idr,iadj:'
      write(ifch,*)id,ami,am,idr,iadj
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idresi
c     initializes /crema/
c-----------------------------------------------------------------------
      parameter (mxindx=1000,mxre=100,mxma=11,mxmx=6)
      common/crema/indx(mxindx),rema(mxre,mxma),rewi(mxre,mxma)
     *,idmx(mxma,mxmx),icre1(mxre,mxma),icre2(mxre,mxma)
      parameter (n=29)
      dimension remai(n,mxma),rewii(n,mxma),idmxi(mxma,mxmx)
     *,icrei(n,2*mxma)
 
      data (idmxi(j,1),j=1,mxma)/ 11, 110, 111,   0,   0,   0,   0, 4*0/
      data (idmxi(j,2),j=1,mxma)/ 22, 220, 330, 331,   0,   0,   0, 4*0/
      data (idmxi(j,3),j=1,mxma)/123,2130,1230,1231,   0,   0,   0, 4*0/
      data (idmxi(j,4),j=1,mxma)/124,2140,1240,1241,   0,   0,   0, 4*0/
      data (idmxi(j,5),j=1,mxma)/134,3140,1340,1341,   0,   0,   0, 4*0/
      data (idmxi(j,6),j=1,mxma)/234,3240,2340,2341,   0,   0,   0, 4*0/
 
      data ((icrei(k,m),m=1,2*mxma),k=1,10)/
     *111,000000, 9*300000,    11*0,
     *222,000000, 9*030000,    11*0,
     *112,       10*210000,    11*0,
     *122,       10*120000,    11*0,
     *113,       10*201000,    11*0,
     *223,       10*021000,    11*0,
     *123,       10*111000,    11*0,
     *133,       10*102000,    11*0,
     *233,       10*012000,    11*0,
     *333,000000, 9*003000,    11*0/
      data ((icrei(k,m),m=1,2*mxma),k=11,20)/
     *114,       10*200100,    11*0,
     *124,       10*110100,    11*0,
     *224,       10*020100,    11*0,
     *134,       10*101100,    11*0,
     *234,       10*011100,    11*0,
     *334,       10*002100,    11*0,
     *144,       10*100200,    11*0,
     *244,       10*010200,    11*0,
     *344,       10*001200,    11*0,
     *444,000000, 9*000300,    11*0/
      data ((icrei(k,m),m=1,2*mxma),k=21,29)/
     * 11,  10*100000,    0,   10*100000,
     * 22,  10*001000,    0,   10*001000,
     * 12,  10*100000,    0,   10*010000,
     * 13,  10*100000,    0,   10*001000,
     * 23,  10*010000,    0,   10*001000,
     * 14,  10*100000,    0,   10*000100,
     * 24,  10*010000,    0,   10*000100,
     * 34,  10*001000,    0,   10*000100,
     * 44,  10*000100,    0,   10*000100/
 
      data ((remai(k,m),m=1,mxma),k=1,10)/
     *111.,0.000,1.425,1.660,1.825,2.000,0.000,0.000,0.000,0.000,0.000,
     *222.,0.000,1.425,1.660,1.825,2.000,0.000,0.000,0.000,0.000,0.000,
     *112.,1.080,1.315,1.485,1.575,1.645,1.685,1.705,1.825,2.000,0.000,
     *122.,1.080,1.315,1.485,1.575,1.645,1.685,1.705,1.825,2.000,0.000,
     *113.,1.300,1.500,1.700,1.850,2.000,0.000,0.000,0.000,0.000,0.000,
     *223.,1.300,1.500,1.700,1.850,2.000,0.000,0.000,0.000,0.000,0.000,
     *123.,1.117,1.300,1.395,1.465,1.540,1.655,1.710,1.800,1.885,2.000,
     *133.,1.423,2.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *233.,1.428,2.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *333.,0.000,2.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000/
      data ((remai(k,m),m=1,mxma),k=11,20)/
     *114.,2.530,2.730,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *124.,2.345,2.530,2.730,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *224.,2.530,2.730,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *134.,2.450,2.600,2.800,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *234.,2.450,2.600,2.800,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *334.,2.700,2.900,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *144.,3.650,3.850,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *244.,3.650,3.850,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *344.,3.800,4.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     *444.,0.000,5.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000/
      data ((remai(k,m),m=1,mxma),k=21,29)/
     * 11.,0.450,0.950,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 22.,0.750,0.965,1.080,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 12.,0.450,0.950,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 13.,0.700,1.050,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 23.,0.700,1.050,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 14.,1.935,2.077,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 24.,1.938,2.079,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 34.,2.085,2.195,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000,
     * 44.,3.037,3.158,0.000,0.000,0.000,0.000,0.000,0.000,0.000,0.000/
 
      data ((rewii(k,m),m=1,mxma),k=1,5)/
     *111.,0.000e+00,0.115e+00,0.140e+00,0.250e+00,0.250e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *222.,0.000e+00,0.115e+00,0.140e+00,0.250e+00,0.250e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *112.,0.000e+00,0.115e+00,0.200e+00,0.140e+00,0.140e+00,
     *     0.145e+00,0.250e+00,0.140e+00,0.250e+00,0.000e+00,
     *122.,0.000e+00,0.115e+00,0.200e+00,0.140e+00,0.140e+00,
     *     0.145e+00,0.250e+00,0.140e+00,0.250e+00,0.000e+00,
     *113.,0.824e-14,0.036e+00,0.080e+00,0.100e+00,0.170e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00/
      data ((rewii(k,m),m=1,mxma),k=6,10)/
     *223.,0.445e-14,0.039e+00,0.080e+00,0.100e+00,0.170e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *123.,0.250e-14,0.890e-05,0.036e+00,0.040e+00,0.016e+00,
     *     0.090e+00,0.080e+00,0.100e+00,0.145e+00,0.170e+00,
     *133.,0.227e-14,0.009e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *233.,0.400e-14,0.010e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *333.,0.000e+00,0.800e-14,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00/
      data ((rewii(k,m),m=1,mxma),k=11,15)/
     *114.,0.400e-11,0.010e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *124.,0.400e-11,0.400e-11,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *224.,0.400e-11,0.010e+00,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *134.,0.150e-11,0.400e-11,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *234.,0.150e-11,0.400e-11,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00/
      data ((rewii(k,m),m=1,mxma),k=16,20)/
     *334.,0.400e-11,0.010e+00,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *144.,0.400e-11,0.010e+00,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *244.,0.400e-11,0.010e+00,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *344.,0.400e-11,0.010e+00,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     *444.,0.400e-11,0.010e+00,0.010e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00/
      data ((rewii(k,m),m=1,mxma),k=21,25)/
     * 11.,0.757e-08,0.153e+00,0.057e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     * 22.,0.105e-05,0.210e-03,0.034e+00,0.004e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     * 12.,0.000e+00,0.153e+00,0.057e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     * 13.,0.000e+00,0.051e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     * 23.,0.197e-02,0.051e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00/
      data ((rewii(k,m),m=1,mxma),k=26,29)/
     * 14.,0.154e-11,0.002e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     * 24.,0.615e-12,0.002e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     * 34.,0.150e-11,0.020e+00,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00,
     * 44.,0.010e+00,0.068e-03,0.000e+00,0.000e+00,0.000e+00,
     *     0.000e+00,0.000e+00,0.000e+00,0.000e+00,0.000e+00/

      do 3 i=1,mxindx
3     indx(i)=0
      do 4 k=1,mxre
      do 4 m=1,mxma
4     rema(k,m)=0
 
      do 2 j=1,mxma
      do 2 i=1,mxmx
2     idmx(j,i)=idmxi(j,i)
 
      ntec=n
      if(ntec.gt.mxre)
     *call utstop('idresi: dimension mxre too small        ')
      do 1 k=1,n
      ix=nint(remai(k,1))
      ix2=nint(rewii(k,1))
      ix3=icrei(k,1)
      if(ix.ne.ix2)
     *call utstop('idresi: ix /= ix2                       ')
      if(ix.ne.ix3)
     *call utstop('idresi: ix /= ix3                       ')
      if(ix.lt.1.or.ix.gt.mxindx)
     *call utstop('idresi: ix out of range.                ')
      indx(ix)=k
      rema(k,1)=0.
      rewi(k,1)=0.
      icre1(k,1)=0
      icre2(k,1)=0
      do 1 m=2,mxma
      rema(k,m)=remai(k,m)
      rewi(k,m)=rewii(k,m)
      icre1(k,m)=icrei(k,m)
1     icre2(k,m)=icrei(k,mxma+m)
 
      indx(33) =indx(22)
      indx(213)=indx(123)
      indx(214)=indx(124)
      indx(314)=indx(134)
      indx(324)=indx(234)
 
      return
      end
 
c-----------------------------------------------------------------------
      integer function idsgl(ic,gen,cmp)
c     returns 1 for singlets (qqq or qqbar) 0 else.
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer ic(2),jcx(nflav,2),icx(2)
      character gen*6,cmp*6
      idsgl=0
      if(cmp.eq.'cmp-ys')then
      call idcomi(ic,icx)
      else
      icx(1)=ic(1)
      icx(2)=ic(2)
      endif
      call iddeco(icx,jcx)
      nq=0
      na=0
      do 1 i=1,nflav
      nq=nq+jcx(i,1)
1     na=na+jcx(i,2)
      if(nq.eq.0.and.na.eq.0)return
      if(gen.eq.'gen-no')then
      if(nq.eq.3.and.na.eq.0.or.nq.eq.1.and.na.eq.1
     *.or.nq.eq.0.and.na.eq.3)idsgl=1
      elseif(gen.eq.'gen-ys')then
      if(mod(nq-na,3).eq.0)idsgl=1
      endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idspin(ii,ic,id,jspin)
c     determines particle spin
c-----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      parameter (nflav=6)
      integer ic(2),jc(nflav,2)
      real pspin1(8)
      do 3 i=1,8
      if(i.le.3)pspin1(i)=pspinl
      if(i.gt.3)pspin1(i)=pspinh
3     continue
      call iddeco(ic,jc)
      ihigh=0
      if(ii.eq.1)then
      do 4 i=1,nflav
      if(jc(i,1).ne.0)ihigh=i
      if(jc(i,2).ne.0)ihigh=i
4     continue
      else
      call idflav(id,i1,i2,i3,idu1,idu2)
      do 5 i=1,nflav
      if(i.eq.iabs(i1).or.i.eq.iabs(i2).or.i.eq.iabs(i3))ihigh=i
5     continue
      endif
      jspin=int(rangen()+pspin1(ihigh))
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idtau(id,p4,p5,taugm)
c     returns lifetime*gamma for id with energy p4, mass p5
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxindx=1000,mxre=100,mxma=11,mxmx=6)
      common/crema/indx(mxindx),rema(mxre,mxma),rewi(mxre,mxma)
     *,idmx(mxma,mxmx),icre1(mxre,mxma),icre2(mxre,mxma)
           if(iabs(id).lt.100.and.id.ne.20)then
      wi=0
           elseif(id.eq.20)then
      wi=.197/2.675e13
           elseif(iabs(id).lt.1e8)then
      ix=iabs(id)/10
      if(ix.lt.1.or.ix.gt.mxindx)
     *call utstop('idtau: ix out of range.                 ')
      ii=indx(ix)
      jj=mod(iabs(id),10)+2
      do 75 imx=1,mxmx
      do 75 ima=2,mxma
      if(iabs(id).eq.idmx(ima,imx))jj=ima
75    continue
      if(ii.lt.1.or.ii.gt.mxre.or.jj.lt.1.or.jj.gt.mxma)then
      write(ifch,*)'id,ii,jj:',id,'   ',ii,jj
      call utstop('idtau: ii or jj out of range            ')
      endif
      wi=rewi(ii,jj)
           else
      tauz=taunll
c-c   tauz=amin1(9./p5**2,tauz)
c-c   tauz=amax1(.2,tauz)
      wi=.197/tauz
           endif
      if(wi.eq.0.)then
      tau=ainfin
      else
      tau=.197/wi
      endif
      if(p5.ne.0.)then
      gm=p4/p5
      else
      gm=ainfin
      endif
      if(tau.ge.ainfin.or.gm.ge.ainfin)then
      taugm=ainfin
      else
      taugm=tau*gm
      endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idtr4(id,ic)
c     transforms generalized paige_id -> werner_id  (for < 4 flv)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      parameter (mxindx=1000,mxre=100,mxma=11,mxmx=6)
      common/crema/indx(mxindx),rema(mxre,mxma),rewi(mxre,mxma)
     *,idmx(mxma,mxmx),icre1(mxre,mxma),icre2(mxre,mxma)
      integer ic(2)
c input for options 11/12 in qnbspd (m.hladik)
      if(iabs(id).lt.10)then
       if(id.eq.1)then
       ic(1)=100000
       ic(2)=000000
       elseif(id.eq.-1)then
       ic(1)=000000
       ic(2)=100000
       elseif(id.eq.2)then 
       ic(1)=010000
       ic(2)=000000
       elseif(id.eq.-2)then 
       ic(1)=000000
       ic(2)=010000
       elseif(id.eq.3)then
       ic(1)=001000
       ic(2)=000000
       elseif(id.eq.-3)then
       ic(1)=000000
       ic(2)=001000
       endif
      return
      endif
c end input for options 11/12 in qnbspd (m.hladik)
           if(iabs(id).lt.1e8)then
      ix=iabs(id)/10
      if(ix.lt.1.or.ix.gt.mxindx)goto9999
      ii=indx(ix)
      if(ii.eq.0)goto9998
      jj=mod(iabs(id),10)+2
      do 27 imx=1,mxmx
      do 27 ima=2,mxma
      if(iabs(id).eq.idmx(ima,imx))jj=ima
27    continue
      if(id.gt.0)then
      ic(1)=icre1(ii,jj)
      ic(2)=icre2(ii,jj)
      else
      ic(2)=icre1(ii,jj)
      ic(1)=icre2(ii,jj)
      endif
      if(ic(1).eq.100000.and.ic(2).eq.100000.and.rangen().lt.0.5)then
      ic(1)=010000
      ic(2)=010000
      endif
           elseif(mod(id/10**8,10).eq.8)then
      ic(1)=mod(id,10**8)/10000*100
      ic(2)=mod(id,10**4)*100
           else
      write(ifch,*)'***** id: ',id
      call utstop('idtr4: unrecognized id                  ')
           endif
      return
 
9998  continue
      write(ifch,*)'id: ',id
      call utstop('idtr4: indx=0.                          ')
 
9999  continue
      write(ifch,*)'id: ',id
      call utstop('idtr4: ix out of range.                 ')
      end
 
c-----------------------------------------------------------------------
      integer function idtra(ic,ier,ires,imix)
c     tranforms from werner-id to paige-id
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (nidt=44,nflav=6)
      integer idt(3,nidt),ic(2),icm(2)
      data idt/
     * 100000,000000,   1   ,010000,000000,   2   ,001000,000000,   3
     *,000100,000000,   4
     *,200000,000000,1100   ,110000,000000,1200   ,020000,000000,2200
     *,101000,000000,1300   ,011000,000000,2300   ,002000,000000,3300
     *,100100,000000,1400   ,010100,000000,2400   ,001100,000000,3400
     *,000200,000000,4400
     *,100000,100000, 110   ,100000,010000, 120   ,010000,010000, 220
     *,100000,001000, 130   ,010000,001000, 230   ,001000,001000, 330
     *,100000,000100, 140   ,010000,000100, 240   ,001000,000100, 340
     *,000100,000100, 440
     *,300000,000000,1111   ,210000,000000,1120   ,120000,000000,1220
     *,030000,000000,2221   ,201000,000000,1130   ,111000,000000,1230
     *,021000,000000,2230   ,102000,000000,1330   ,012000,000000,2330
     *,003000,000000,3331   ,200100,000000,1140   ,110100,000000,1240
     *,020100,000000,2240   ,101100,000000,1340   ,011100,000000,2340
     *,002100,000000,3340   ,100200,000000,1440   ,010200,000000,2440
     *,001200,000000,3440   ,000300,000000,4441/
 
      idtra=0
      if(ic(1).eq.0.and.ic(2).eq.0)return
      do 1 i=1,nidt
      if(ic(2).eq.idt(1,i).and.ic(1).eq.idt(2,i))idtra=-idt(3,i)
      if(ic(1).eq.idt(1,i).and.ic(2).eq.idt(2,i))idtra=idt(3,i)
1     continue
      isi=1
      if(idtra.ne.0)isi=idtra/iabs(idtra)
 
      jspin=0
      if(idtra.ne.0.and.mod(idtra,10).eq.0.and.ires.eq.1)
     *call idspin(1,ic,idu,jspin)
 
      if(imix.eq.1)then
      call idmix(ic,jspin,icm,idtram)
      if(idtram.ne.0)idtra=idtram
      if(jspin.eq.0)then
      if(iabs(idtra).eq.1230.and.rangen().lt.pispn)idtra=isi*2130
      if(iabs(idtra).eq.1240.and.rangen().lt.pispn)idtra=isi*2140
      if(iabs(idtra).eq.1340.and.rangen().lt.pispn)idtra=isi*3140
      if(iabs(idtra).eq.2340.and.rangen().lt.pispn)idtra=isi*3240
      endif
      elseif(imix.eq.2)then
      if(idtra.eq.220)idtra=110
      if(idtra.eq.330)idtra=110
      elseif(imix.eq.3)then
      if(idtra.eq.220)idtra=110
      if(idtra.eq.330)idtra=220
      endif
 
      if(idtra.ne.0)idtra=idtra+jspin*isi
 
      if(idtra.ne.0)return
      if(ier.ne.1)return
      jerr=jerr+1
      write(ifch,*)'***** error in idtra: unknown code'
      write(ifch,*)'ic = ',ic
      return
 
      entry idtrai(num,id,ier)
      idtrai=0
      j=0
      do 2 i=1,nidt
      if(iabs(id).eq.idt(3,i))j=i
2     continue
      if(j.ne.0)then
      if(id.lt.0)then
      idtrai=idt(3-num,j)
      else
      idtrai=idt(num,j)
      endif
      return
      endif
      if(ier.ne.1)return
      jerr=jerr+1
      write(ifch,*)'***** error in idtrai: unknown code'
      write(ifch,*)'id = ',id
      return
      end
 
c-----------------------------------------------------------------------
      subroutine idtrb(ib1,ib2,ib3,ib4,jc)
c     id transformation ib -> jc
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      parameter (nflav=6)
      integer jc(nflav,2)
      jc(1,1)=ib1/10**4
      jc(2,1)=ib2/10**4
      jc(3,1)=ib3/10**4
      jc(4,1)=ib4/10**4
      jc(5,1)=0
      jc(6,1)=0
      jc(1,2)=mod(ib1,10**4)
      jc(2,2)=mod(ib2,10**4)
      jc(3,2)=mod(ib3,10**4)
      jc(4,2)=mod(ib4,10**4)
      jc(5,2)=0
      jc(6,2)=0
      return
      entry idtrbi(jc,ib1,ib2,ib3,ib4)
      ib1=jc(1,1)*10**4+jc(1,2)
      ib2=jc(2,1)*10**4+jc(2,2)
      ib3=jc(3,1)*10**4+jc(3,2)
      ib4=jc(4,1)*10**4+jc(4,2)
      ib5=jc(5,1)*10**4+jc(5,2)
      ib6=jc(6,1)*10**4+jc(6,2)
      if(ib5.ne.0.or.ib6.ne.0)then
      write(ifch,*)'***** error in idtrbi: bottom or top quarks'
      write(ifch,*)'jc:'
      write(ifch,*)jc
      call utstop('idtrbi: bottom or top quarks            ')
      endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine jambr1(ns,nsg,iret)
c-----------------------------------------------------------------------
c "AMOR" (koch+werner, 89)
c breaks string ns according to a-m model.
c ns: points to the current fragmenting string.
c nsg: points to the last produced substring (succ increased)
c     input:
c psg(,ns): momentum of string ns in pp-cms
c pjt(,2*ns-1),pjt(,2*ns): momentum of end of string ns in pp-cms
c     output:
c xorsg(,nsg): origin of substring nsg in pp-cms
c pjt(,2*nsg-1),pjt(,2*nsg): momentum of end of substring nsg in pp-cms
c icjt(,2*nsg-1),icjt(,2*nsg): ic-code  of end of substrg nsg in pp-cms
c psg(,nsg): momentum of substring nsg in pp-cms
c xbkptl(,nptlc-nptl2): breakpoint of ns in pp-cms
c isptl(,nptlc-nptl2): 1 if string ns breaks
c-----------------------------------------------------------------------
      parameter (mxpc=500,mxsg=500)
      parameter (mxjt=2*mxsg)
      double precision xbkptl
      common/cpc/xbkptl(2,mxpc),isptl(mxpc)
      double precision psg,rotsg,pjt,xorsg
      common/csg/psg(5,mxsg),rotsg(3,mxsg),pjt(5,mxjt),xorsg(4,mxsg)
     *,isg(mxsg),icjt(2,mxjt),iorsg(mxsg)
      common/cjambr/nptl2,nptlc
 
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,taufos
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/cdelre/delrer
      parameter (nptf=129)
      common /cptf/xptf(nptf),wptf(nptf)
     *,qptfu(nptf),qptfs(nptf),qptfuu(nptf),qptfus(nptf),qptfss(nptf)
 
      parameter (nflav=6)
      integer jcp(nflav,2),jcm(nflav,2),jcpm(nflav,2),jcmp(nflav,2)
     *,jcps(nflav,2),jcms(nflav,2),icp(2),icm(2),icpm(2),icmp(2)
     *,icps(2),icms(2),ic(2),jc(nflav,2),icx(2),jcx(nflav,2)
     *,icy(2),jcy(nflav,2)
      double precision xor1(4),xor2(4),xbk(4)
      double precision p0p,p0m,pp,pm,pt,px,py,phi,eeta
 
      ncnt3=0
      nsg0=nsg
9993  nsg=nsg0
 
      iret=0
      do i=1,2
      icp(i)=icjt(i,2*ns-1)
      icm(i)=icjt(i,2*ns)
      enddo
      call iddeco(icp,jcp)
      call iddeco(icm,jcm)
      np=0
      do 7 nf=1,nflav
      jc(nf,1)=jcp(nf,1)+jcm(nf,1)
      jc(nf,2)=jcp(nf,2)+jcm(nf,2)
7     np=np+jcp(nf,1)-jcp(nf,2)
      call idenco(jc,ic,ireten)
      id=idtra(ic,0,0,3)
      amms=utamnx(jcp,jcm)
      am=psg(5,ns)
 
c     split string
c     ------------
      p0p=0
      p0m=0
      do 5 j=2*ns-1,2*ns
      p0p=p0p+pjt(4,j)+pjt(3,j)
5     p0m=p0m+pjt(4,j)-pjt(3,j)
9994  ncnt3=ncnt3+1
           if(ncnt3.gt.100)then
           if(ish.ge.90)then
           call utmsg('jambr1')
           write(ifch,*)'*****  split kinem not possible.'
           write(ifch,112)
     *     (icjt(j,2*ns-1),j=1,2),(icjt(j,2*ns),j=1,2),psg(5,ns)
112        format(1x,2i8,4x,2i8,4x,f7.2)
           call utmsgf
           endif
           if(ns.eq.1)iret=9999
           if(ns.eq.1)return
           iret=9996
           return
           endif
      do 17 nf=1,nflav
      do 17 i=1,2
      jcpm(nf,i)=0
17    jcmp(nf,i)=0
 
c     determine flavour
c     -----------------
      nqu=0
      if(ish.ge.93)write(ifch,*)'ordinary string fragmentation'
      nqu=1
      if(rangen().le.pdiqua)nqu=2
      ii=0
      if(mod(np+nqu,3).eq.0)ii=1
      if(mod(np-nqu,3).eq.0)ii=2
           if(ii.eq.0)
     *     call utstop('jambr1: no singlet construction possible')
      ifltt=0
      do 8 n=1,nqu
      ifl=int(rangen()/pud)+1
      if(n.eq.2)ifl=int(rangen()/0.50)+1
      ifltt=ifltt*10+(ifl+1)/2
      jcpm(ifl,ii)=jcpm(ifl,ii)+1
8     jcmp(ifl,3-ii)=jcmp(ifl,3-ii)+1
      call idenco(jcpm,icpm,ireten)
           if(ireten.eq.1)
     *     call utstop('jambr1: idenco(jcpm... ret.code=1       ')
      call idenco(jcmp,icmp,ireten)
           if(ireten.eq.1)
     *     call utstop('jambr1: idenco(jcmp... ret.code=1       ')
      do 25 nf=1,nflav
      do 25 i=1,2
      jcps(nf,i)=jcp(nf,i)+jcpm(nf,i)
25    jcms(nf,i)=jcm(nf,i)+jcmp(nf,i)
      call idenco(jcps,icps,ireten)
      call idenco(jcms,icms,ireten)
      call idcomk(icps)
      call idcomk(icms)

c     calculate p+,p-,pt of string breaking
c     -------------------------------------
           if(ish.ge.93)write(ifch,109)
     *     icm(1),icmp(1),icpm(1),icp(1)
     *     ,icm(2),icmp(2),icpm(2),icp(2)
109        format(1x,'flavors:',2(i11,i7)/9x,2(i11,i7))
           if(ish.ge.93)write(ifch,*)'ifltt:',ifltt
      idp=idtra(icps,0,0,3)
      idm=idtra(icms,0,0,3)
      ammp=utamny(jcp,jcpm)
      ammm=utamny(jcm,jcmp)
      if(ifltt.eq.1)then
      pt=utinvt(nptf,xptf,qptfu ,rangen()*qptfu(nptf))
      elseif(ifltt.eq.2)then
      pt=utinvt(nptf,xptf,qptfs ,rangen()*qptfs(nptf))
      elseif(ifltt.eq.11)then
      pt=utinvt(nptf,xptf,qptfuu,rangen()*qptfuu(nptf))
      elseif(ifltt.eq.12.or.ifltt.eq.21)then
      pt=utinvt(nptf,xptf,qptfus,rangen()*qptfus(nptf))
      elseif(ifltt.eq.22)then
      pt=utinvt(nptf,xptf,qptfss,rangen()*qptfss(nptf))
      endif
      lo=1
      goto48
47    lo=lo+1
      pt=rangen()*pt
48    continue
      phi=2.*pi*rangen()
      px=pt*dcos(phi)
      py=pt*dsin(phi)
      tmmp=sqrt(pt**2+ammp**2)
      tmmm=sqrt(pt**2+ammm**2)
      area=-alog(rangen())/parea
      etam=dlog((tmmm**2+area)/(sqrt(area)*p0m))
      etap=dlog((sqrt(area)*p0p)/(tmmp**2+area))
      if(etam.gt.etap.and.lo.lt.5)goto47
      if(etam.gt.etap)goto9994
      if(nqu.eq.2)then
      endif
      eta=etam+rangen()*(etap-etam)
      amp=sqrt(p0p*sqrt(area)*exp(-eta)-area-pt**2)
      amm=sqrt(p0m*sqrt(area)*exp( eta)-area-pt**2)
      call idres(idp,amp,idpr,iadjp)
      call idres(idm,amm,idmr,iadjm)
      r=rangen()
           if(idpr.eq.110.and.r.lt.0.5)then
      idp=220
      amp=.549
      if(r.lt.0.6667)amp=.958
      call idres(idp,amp,idpr,iadjp)
      iadjp=1
           endif
      r=rangen()
           if(idmr.eq.110.and.r.lt.0.5)then
      idm=220
      amm=.549
      if(r.lt.0.6667)amm=.958
      call idres(idm,amm,idmr,iadjm)
      iadjm=1
           endif
      tmp=sqrt(pt**2+amp**2)
      tmm=sqrt(pt**2+amm**2)
           if(iadjp.eq.1.and.iadjm.ne.1)then
      eta=dlog((sqrt(area)*p0p)/(tmp**2+area))
      if(eta.lt.etam)goto9994
      amm=sqrt(p0m*sqrt(area)*exp(eta)-area-pt**2)
      call idres(idm,amm,idmr,iadjm)
      tmm=sqrt(pt**2+amm**2)
           endif
           if(iadjm.eq.1.and.iadjp.ne.1)then
      eta=dlog((tmm**2+area)/(sqrt(area)*p0m))
      if(eta.gt.etap)goto9994
      amp=sqrt(p0p*sqrt(area)*exp(-eta)-area-pt**2)
      call idres(idp,amp,idpr,iadjp)
      tmp=sqrt(pt**2+amp**2)
           endif
           if(iadjp.eq.1.and.iadjm.eq.1)then
      tm=(p0p*p0m-tmm**2-tmp**2)/2
      if(tm.lt.0.)goto9994
      if((tm-tmp*tmm)*(tm+tmp*tmm).lt.0.)goto9994
      area=tm-sqrt((tm-tmp*tmm)*(tm+tmp*tmm))
      eeta=p0p*sqrt(area)/(tmp**2+area)
      if(eeta.le.0.)goto9994
      eta=dlog(eeta)
           endif
      pp=sqrt(area)*exp(eta)
      pm=sqrt(area)*exp(-eta)
           if(p0p-pp-pt**2/pm.lt.0..or.p0m-pm-pt**2/pp.lt.0.)then
           if(ish.ge.90)then
           call utmsg('jambr1')
           write(ifch,*)'*****  negative jet energy. split again.'
           if(p0p-pp-pt**2/pm.lt.0.)
     *     write(ifch,*)'p0p: ',p0p,'   pp+pt**2/pm: ',pp+pt**2/pm
           if(p0m-pm-pt**2/pp.lt.0.)
     *     write(ifch,*)'p0m: ',p0m,'   pm+pt**2/pp: ',pm+pt**2/pp
           call utmsgf
           endif
           goto9994
           endif
 
c     determine break pnt and new origins
c     -----------------------------------
      xor1(1)=0
      xor1(2)=0
      xor1(3)=pp/2/tensn
      xor1(4)=pp/2/tensn
           if(ish.ge.93)write(ifch,118)xor1
118        format(' new origin +: ',13x,4f7.2)
      call utrot2(-1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,xor1(1),xor1(2),xor1(3))
      call utlob2(-1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,xor1(1),xor1(2),xor1(3),xor1(4))
      do 44 j=1,4
44    xor1(j)=xor1(j)+xorsg(j,ns)
           if(ish.ge.93)write(ifch,117)xor1
      xbk(1)=0
      xbk(2)=0
      xbk(3)=(pp-pm)/2/tensn
      xbk(4)=(pp+pm)/2/tensn
           if(ish.ge.93)write(ifch,116)xbk
116        format(' breaking pnt: ',13x,4f7.2)
      call utrot2(-1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,xbk(1),xbk(2),xbk(3))
      call utlob2(-1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,xbk(1),xbk(2),xbk(3),xbk(4))
      do 45 j=1,4
45    xbk(j)=xbk(j)+xorsg(j,ns)
      if(ish.ge.93)write(ifch,117)xbk
117   format(28x,4f7.2)
      xor2(1)=0
      xor2(2)=0
      xor2(3)=-pm/2/tensn
      xor2(4)= pm/2/tensn
      if(ish.ge.93)write(ifch,119)xor2
119   format(' new origin -: ',13x,4f7.2)
      call utrot2(-1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,xor2(1),xor2(2),xor2(3))
      call utlob2(-1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,xor2(1),xor2(2),xor2(3),xor2(4))
      do 46 j=1,4
46    xor2(j)=xor2(j)+xorsg(j,ns)
           if(ish.ge.93)write(ifch,117)xor2
 
c     store the two substrings
c     ------------------------
      nsgb=nsg
      do 9998 m=1,2
      nsg=nsg+1
           if(nsg.gt.mxsg)
     *     call utstop('jambr1: nsg>mxsg                        ')
      isg(nsg)=nptlc
      if(m.eq.1)then !-------------------
      do 41 j=1,4
41    xorsg(j,nsg)=xor1(j)
      pjt(1,2*nsg-1)=0
      pjt(2,2*nsg-1)=0
      pjt(3,2*nsg-1)=(p0p-pp-pt**2/pm)/2
      pjt(4,2*nsg-1)=(p0p-pp-pt**2/pm)/2
      pjt(5,2*nsg-1)=0
      do 10 j=1,2
10    icjt(j,2*nsg-1)=icjt(j,2*ns-1)
      pjt(1,2*nsg)=px
      pjt(2,2*nsg)=py
      pjt(3,2*nsg)=(pt**2/pm-pm)/2
      pjt(4,2*nsg)=(pt**2/pm+pm)/2
      pjt(5,2*nsg)=0
      do 11 j=1,2
      icjt(j,2*nsg)=icpm(j)
      do 11 nf=1,nflav
      jc(nf,j)=jcps(nf,j)
11    continue
      else !-------------------------
      do 42 j=1,4
42    xorsg(j,nsg)=xor2(j)
      pjt(1,2*nsg-1)=-px
      pjt(2,2*nsg-1)=-py
      pjt(3,2*nsg-1)=(pp-pt**2/pp)/2
      pjt(4,2*nsg-1)=(pp+pt**2/pp)/2
      pjt(5,2*nsg-1)=0
      do 12 j=1,2
      icjt(j,2*nsg-1)=icmp(j)
      do 12 nf=1,nflav
      jc(nf,j)=jcms(nf,j)
12    continue
      pjt(1,2*nsg)=0
      pjt(2,2*nsg)=0
      pjt(3,2*nsg)=-(p0m-pm-pt**2/pp)/2
      pjt(4,2*nsg)= (p0m-pm-pt**2/pp)/2
      pjt(5,2*nsg)=0
      do 13 j=1,2
13    icjt(j,2*nsg)=icjt(j,2*ns)
      endif !---------------------
      do 18 i=1,2
      icx(i)=icjt(i,2*nsg-1)
18    icy(i)=icjt(i,2*nsg)
      call iddeco(icx,jcx)
      call iddeco(icy,jcy)
      do 28 n=1,nflav
      do 28 i=1,2
      k=jcx(n,i)+jcy(n,i)-jc(n,i)
      if(k.le.0)goto28
      do 19 l=1,k
      jx=jcx(n,i)
      jy=jcy(n,i)
      if(jx.gt.0.and.jy.eq.0)then
      jcx(n,i)=jcx(n,i)-1
      elseif(jx.eq.0.and.jy.gt.0)then
      jcy(n,i)=jcy(n,i)-1
      elseif(jx.gt.0.and.jy.gt.0)then
      if(rangen().lt..5)then
      jcx(n,i)=jcx(n,i)-1
      else
      jcy(n,i)=jcy(n,i)-1
      endif
      else
           write(ifch,*)('*',lp=1,71)
           write(ifch,*)'*****  ic;  ',ic
           write(ifch,*)'*****  icx: ',icx
           write(ifch,*)'*****  icy: ',icy
           write(ifch,*)'*****  n,i,k,l: ',n,i,k,l
           write(ifch,*)'*****  jx,jy: ',jx,jy
           write(ifch,*)('*',lp=1,71)
           call utstop('jambr1: error during jet compactificatn ')
      endif
19    continue
28    continue
      call idenco(jcx,icx,ireten)
           if(ireten.eq.1)
     *     call utstop('jambr1: idenco(jcx... ret.code=1        ')
      call idenco(jcy,icy,ireten)
           if(ireten.eq.1)
     *     call utstop('jambr1: idenco(jcy... ret.code=1        ')
      do 20 i=1,2
      icjt(i,2*nsg-1)=icx(i)
20    icjt(i,2*nsg)=icy(i)
           if(ish.ge.93)write(ifch,108)nsg,ns
     *     ,(icjt(j,2*nsg-1),j=1,2),(sngl(pjt(j,2*nsg-1)),j=1,5)
108        format(2x,i3,i4,2x,2i7,3x,5(e8.2,1x))
           if(ish.ge.93)write(ifch,108)nsg,ns
     *     ,(icjt(j,2*nsg  ),j=1,2),(sngl(pjt(j,2*nsg  )),j=1,5)
      do 14 j=1,4
14    psg(j,nsg)=pjt(j,2*nsg-1)+pjt(j,2*nsg)
      psg(5,nsg)=
     *sqrt(psg(4,nsg)**2-psg(3,nsg)**2-psg(2,nsg)**2-psg(1,nsg)**2)
           if(ish.ge.93)write(ifch,110)nsg,ns,(psg(j,nsg),j=1,5)
110        format(2x,i3,i4,19x,5(e8.2,1x))
      call utrot2(-1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,psg(1,nsg),psg(2,nsg),psg(3,nsg))
      call utlob2(-1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,psg(1,nsg),psg(2,nsg),psg(3,nsg),psg(4,nsg))
      iorsg(nsg)=ns
           if(ish.ge.93)
     *     write(ifch,102)nsg,iorsg(nsg),(psg(j,nsg),j=1,5)
102        format(2x,i3,i4,19x,5(e8.2,1x))
c-c   do 56 i=1,2
c-c   icpz(i)=icjt(i,2*nsg-1)
c-c56 icmz(i)=icjt(i,2*nsg)
c-c   call iddeco(icpz,jcpz)
c-c   call iddeco(icmz,jcmz)
c-c   do 57 nf=1,nflav
c-c   jcz(nf,1)=jcpz(nf,1)+jcmz(nf,1)
c-c57 jcz(nf,2)=jcpz(nf,2)+jcmz(nf,2)
c-c   call idenco(jcz,icz,ireten)
c-c   idz=idtra(icz,0,0,3)
c-c   amz=psg(5,nsg)
c-c   call idres(idz,amz,idrz,iadj)
c-c   if(idrz.eq.110.and.rangen().lt.0.5)goto9993
9998  continue
 
      xbkptl(1,nptlc-nptl2)=xbk(3)
      xbkptl(2,nptlc-nptl2)=xbk(4)
      isptl(nptlc-nptl2)=1
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine jambr2(ns,nsg,iret)
c-----------------------------------------------------------------------
c "SAMBA" (scholten+werner, mar 92)
c breaks string ns according to a-m model.
c ns: points to the current fragmenting string.
c nsg: points to the last produced substring (succ increased)
c     input:
c psg(,ns): momentum of string ns in pp-cms
c pjt(,2*ns-1),pjt(,2*ns): momentum of end of string ns in pp-cms
c     output:
c xorsg(,nsg): origin of substring nsg in pp-cms
c pjt(,2*nsg-1),pjt(,2*nsg): momentum of end of substring nsg in pp-cms
c icjt(,2*nsg-1),icjt(,2*nsg): ic-code  of end of substrg nsg in pp-cms
c psg(,nsg): momentum of substring nsg in pp-cms
c xbkptl(,nptlc-nptl2): breakpoint of ns in pp-cms
c isptl(,nptlc-nptl2): 1 if string ns breaks
c-----------------------------------------------------------------------
      parameter (mxpc=500,mxsg=500)
      parameter (mxjt=2*mxsg)
      double precision xbkptl
      common/cpc/xbkptl(2,mxpc),isptl(mxpc)
      double precision psg,rotsg,pjt,xorsg
      common/csg/psg(5,mxsg),rotsg(3,mxsg),pjt(5,mxjt),xorsg(4,mxsg)
     *,isg(mxsg),icjt(2,mxjt),iorsg(mxsg)
      common/cjambr/nptl2,nptlc
 
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,taufos
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/cdelre/delrer
      parameter (nptf=129)
      common /cptf/xptf(nptf),wptf(nptf)
     *,qptfu(nptf),qptfs(nptf),qptfuu(nptf),qptfus(nptf),qptfss(nptf)
 
      parameter (nflav=6)
      integer jcp(nflav,2),jcm(nflav,2),jcpm(nflav,2),jcmp(nflav,2)
     *,icp(2),icm(2),icpm(2),icmp(2)
     *,jcz(nflav,2),icz(2),icmz(2),icpz(2),jcpz(nflav,2),jcmz(nflav,2)
      double precision xor2(4),xbk(4),pw(4)
      double precision p0p,p0m,pp,pm,pt,px,py,phi
 
 
      DIMENSION XBREAK(100),YBREAK(100),pxbrk(100),pybrk(100),
     & ITRD(100),ICPMBR(2,100),ICMPBR(2,100)
      PARAMETER (NBRM=99)
      REAL*8 DY,DR,DALFA,DYST,DONE,AMAX,RA,ANEW,DY1,DY2,DYT,AMAM,
     & RMX,RMY,A,XP,YP,XBR,YBR,YTD,DTWO,DFOUR,DA,YNEW,
     & DATWID,X2
      LOGICAL LAST
      DATA DONE,DTWO,DFOUR,fyst/1.D0,2.D0,4.D0,0.0001/
 
      ncnt3=0
      nsg0=nsg
9993  nsg=nsg0
 
c initialization
c --------------
 
      iret=0
      do 6 i=1,2
      icp(i)=icjt(i,2*ns-1)
      icm(i)=icjt(i,2*ns)
      ICPMBR(i,1)=ICM(I)	!flavor of left side,=break#1
    6 CONTINUE
 
      call iddeco(icp,jcp)
      call iddeco(icm,jcm)
      np=0
      do 7 nf=1,nflav
      np=np+jcp(nf,1)-jcp(nf,2)
7     continue
      AMMS=UTAMNX(JCP,JCM)
      am=psg(5,ns)
C
      p0p=0
      p0m=0
      do 5 j=2*ns-1,2*ns
      p0p=p0p+pjt(4,j)+pjt(3,j)
5     p0m=p0m+pjt(4,j)-pjt(3,j)
      XT=P0M
      YT=P0P
C
      DALFA=DBLE(PAREA)
C	initialize
      XBREAK(1)=XT
      YBREAK(1)=0.
      pxbrk(1)=0
      pybrk(1)=0
 
c  redo
c  ----
 
      ncnt3=0
9994  ncnt3=ncnt3+1
      IBR=1
      XP=XT
      YP=0.
      YTD=YT
           if(ncnt3.gt.100)then
           if(ish.ge.90)then
           call utmsg('jambr2')
           write(ifch,*)'*****  split kinem not possible.'
           write(ifch,112)
     *     (icjt(j,2*ns-1),j=1,2),(icjt(j,2*ns),j=1,2),psg(5,ns)
112        format(1x,2i8,4x,2i8,4x,f7.2)
           call utmsgf
           endif
           if(ns.eq.1)iret=9999
           if(ns.eq.1)return
           iret=9996
           return
           endif
 
c  search for breakpoints
c  ----------------------
 
    9 CONTINUE
      do 17 nf=1,nflav
      do 17 i=1,2
      jcpm(nf,i)=0
17    jcmp(nf,i)=0
 
c     ordinary strings
c     ----------------
      nqu=0
      if(ish.ge.92)write(ifch,*)'ordinary string fragmentation'
      nqu=1
      if(rangen().le.pdiqua)nqu=2
      ii=0
      if(mod(np+nqu,3).eq.0)ii=1
      if(mod(np-nqu,3).eq.0)ii=2
           if(ii.eq.0)
     *     call utstop('jambr2: no singlet construction possible')
      ifltt=0
      do 18 n=1,nqu
      ifl=int(rangen()/pud)+1
      ifltt=ifltt*10+(ifl+1)/2
      jcpm(ifl,ii)=jcpm(ifl,ii)+1
18    jcmp(ifl,3-ii)=jcmp(ifl,3-ii)+1
      call idenco(jcpm,icpm,ireten)
           if(ireten.eq.1)
     *     call utstop('jambr2: idenco(jcpm... ret.code=1       ')
      call idenco(jcmp,icmp,ireten)
           if(ireten.eq.1)
     *     call utstop('jambr2: idenco(jcmp... ret.code=1       ')
 
c  calculate pt of string breaking
c  -------------------------------------
           if(ish.ge.92)write(ifch,109)
     *     icm(1),icmp(1),icpm(1),icp(1)
     *     ,icm(2),icmp(2),icpm(2),icp(2)
109        format(1x,'flavors:',2(i11,i7)/9x,2(i11,i7))
           if(ish.ge.92)write(ifch,*)'ifltt:',ifltt
      ammp=UTAMNY(JCP,JCPM)
      ammm=UTAMNY(JCMP,JCM)
      if(ifltt.eq.1)then
      pt=utinvt(nptf,xptf,qptfu ,rangen()*qptfu(nptf))
      elseif(ifltt.eq.2)then
      pt=utinvt(nptf,xptf,qptfs ,rangen()*qptfs(nptf))
      elseif(ifltt.eq.11)then
      pt=utinvt(nptf,xptf,qptfuu,rangen()*qptfuu(nptf))
      elseif(ifltt.eq.12.or.ifltt.eq.21)then
      pt=utinvt(nptf,xptf,qptfus,rangen()*qptfus(nptf))
      elseif(ifltt.eq.22)then
      pt=utinvt(nptf,xptf,qptfss,rangen()*qptfss(nptf))
      endif
 
      RMX=0.001
      IF(IBR.EQ.1) RMX=AMMM**2+PT**2
      RMY=AMMP**2+PT**2
      IF(RMY.GT.(XP*(YTD-YP))) GOTO 8
 
C     Single Artru-Mennessier Break (SAMB)
c     ------------------------------------
C     in: XP,YP,YTD,RMX,RMY
C     in: FYST,DALFA
C     out: XBR,YBR
 
      DYT=YTD-YP
C	Y-stepping range
      AMAM=DONE+(RMX-RMY)/(XP*DYT)
      A=(DONE-DFOUR*RMX/(AMAM*AMAM*XP*DYT))
      IF(A .LT. 0.) GOTO 8	! string cannot break any further
      A=SQRT(A)
      DY1=DYT*AMAM*(DONE-A)/DTWO
      DY2=DYT*AMAM*(DONE+A)/DTWO
      DYST=DBLE(FYST)*(DYT-DY2)
    3 CONTINUE
      DR=DBLE(1.-RANgen())
      AMAX=XP*DYT*A*AMAM+RMX*LOG(DY1/DY2)+RMY*LOG((DYT-DY2)/(DYT-DY1))
      RA=-DLOG(DR)/DALFA
      IF(RA.GT.AMAX .AND. IBR.GT.1) GOTO 8
      RA=MOD(RA,AMAX)
      DY=DYT*DSQRT(RMX)/(DSQRT(RMX)+DSQRT(RMY))	! second derivative area
      DATWID=XP*(DY-DY1)+RMX*LOG(DY1/DY)+RMY*LOG((DYT-DY)/(DYT-DY1))
      ANEW=RA-DATWID	! DATWID= area for which second derivative=0
      ICOUNT=0
      IF(ANEW .GT. 0.) GOTO 1
    2 CONTINUE
      DA=XP - RMX/DY - RMY/(DYT-DY)	! first derivative of area
      YNEW=DY+ANEW/DA	! always approach from above
      DY=INT(YNEW/DYST)*DYST
      IF(DY.LT.DY1) THEN
      	WRITE(IFCH,*) 'DY,DY1',DY,DY1
      	GOTO 4
      ENDIF
      ANEW=RA-XP*(DY-DY1)-RMX*LOG(DY1/DY)-RMY*LOG((DYT-DY)/(DYT-DY1))	!
      IF(ANEW .LT. 0.) GOTO 2
      GOTO 4
    1 CONTINUE
      DA=XP - RMX/DY - RMY/(DYT-DY)
      YNEW=DY+ANEW/DA	! always approach from BELOW
      DY=(INT(YNEW/DYST)+1)*DYST
      ANEW=RA-XP*(DY-DY1)-RMX*LOG(DY1/DY)-RMY*LOG((DYT-DY)/(DYT-DY1))
      IF(ANEW .GT. 0.) GOTO 1
      DY=DY-DYST
      ANEW=RA-XP*(DY-DY1)-RMX*LOG(DY1/DY)-RMY*LOG((DYT-DY)/(DYT-DY1))
    4 CONTINUE
      YBR=DY+RANgen()*DYST
      IF(YBR .GT. DY2) YBR=DY2
      XBR=ANEW/DYST+RMY/(DYT-YBR)
      X2=(XP-RMX/YBR)
      IF(XBR .GT. X2) XBR=X2
C	BETTER:	solve for YBR from ANEW=0
C		find XBR from homogeneous (not exp) distr, x1<x<x2
C      X1=RMY/(DYT-YBR)
      YBR=YBR+YP
C
C	end SAMB
C
      IF(IBR.GE.NBRM)
     *     call utstop('jambr2: IBR>NBRM                        ')
      IBR=IBR+1
      XBREAK(IBR)=XBR
      YBREAK(IBR)=YBR
      phi=2.*pi*rangen()
      pxbrk(IBR)=pt*dcos(phi)
      pybrk(IBR)=pt*dsin(phi)
      DO I=1,2
      ICPMBR(i,IBR)=ICPM(I)
      ICMPBR(i,IBR)=ICMP(I)
      ENDDO
      XP=XBR	! better, but longer: =XT
      YP=YBR
      amleft=sqrt(ybr*(xt-xbr))
	amrght=sqrt(xbr*(yt-ybr))
      GOTO 9	! search for next break point
    8 CONTINUE  ! ibr loop finished
 
      IF(IBR.EQ.1) GOTO 9994	! no breakpoint found
 
C  initialize tail end
C  -------------------
      XBREAK(IBR+1)=0.
      YBREAK(IBR+1)=YT
      pxbrk(IBR+1)=0
      pybrk(IBR+1)=0
      DO I=1,2
      ICmpBR(i,IBR+1)=ICP(I)	!flavor of right side,=break#last
      ENDDO
 
c          print
c          -----
           if(ish.ge.92)then
           write(ifch,*)'icmpbr(1/2,)  x/ybreak  px/ybreak:'
           do 52 ib=1,ibr+1
           write(ifch,100)icmpbr(1,ib),icmpbr(2,ib)
     *     ,xbreak(ib),ybreak(ib),pxbrk(ib),pybrk(ib)
52         write(ifch,100)icpmbr(1,ib),icpmbr(2,ib)
100        format(1x,i10,i10,2e11.3,7x,2e11.3)
           endif
 
C  Time order breakpoints
C  ----------------------
 
      T1=0.
      DO J=2,IBR
      T2=2.
      DO 20 I=2,IBR
C      T=XBREAK(I)/XT+YBREAK(I)/YT	! time
      T=XBREAK(I)*YBREAK(I)/(XT*YT)	! area past
      IF(T.LE.T1 .OR. T.GT.T2) GOTO 20
      T2=T
      NT=I
   20 CONTINUE
      T1=T2
      ITRD(J)=NT
      ENDDO	! LOOP J
 
c          print
c          -----
           if(ish.ge.92)then
           write(ifch,*)'itrd:'
           write(ifch,*)(itrd(ib),ib=2,ibr)
           endif
 
C  Check acceptance criterium
c  --------------------------
 
      nbr=0
      DO 12 J=2,IBR	! check points according to T ordering
      I=ITRD(J)
      XBRI=XBREAK(I)
      YBRI=YBREAK(I)
C	Find neighboring earlier break points
      IR=IBR+1
      IL=1
      DO JN=2,J
      IN=ITRD(JN)
      IF(IN.GT.I .AND. IN.LT.IR) IR=IN
      IF(IN.LT.I .AND. IN.GT.IL) IL=IN
      ENDDO	! loop JN
      PML=(XBREAK(IL)-XBRI)
      PPL=(YBRI-YBREAK(IL))
      PMR=(XBRI-XBREAK(IR))
      PPR=(YBREAK(IR)-YBRI)
      PX=pxbrk(I)
      PY=pybrk(I)
      PXL=PX-pxbrk(IL)
      PYL=PY-pybrk(IL)
      ammm=PML*PPL-(PXL*PXL+PYL*PYL)
      PXR=pxbrk(IR)-PX
      PYR=pybrk(IR)-PY
      ammp=PMR*PPR-(PXR*PXR+PYR*PYR)
      if(ammp.le.0.) goto 13
      if(ammm.le.0.) goto 13
      ammp=sqrt(ammp)
      ammm=sqrt(ammm)
      amp=ammp
      amm=ammm
      call utresm(icmpbr(1,ir),icmpbr(2,ir)
     *,icpmbr(1,i),icpmbr(2,i),amp,idpr,iadjp,ireten)
      call utresm(icpmbr(1,il),icpmbr(2,il)
     *,icmpbr(1,i),icmpbr(2,i),amm,idmr,iadjm,ireten)
      IF(amp.gt.ammp) GOTO 13
      IF(amm.gt.ammm) GOTO 13
      pt2=px*px+py*py
      ptl2=pxbrk(il)**2+pybrk(il)**2
      ptr2=pxbrk(ir)**2+pybrk(ir)**2
      d=(pml*ppl-ptl2-pt2)/2.
      d=d*d-pt2*ptl2
      if(d.le.0.)goto13
      d=(pmr*ppr-ptr2-pt2)/2.
      d=d*d-pt2*ptr2
      if(d.le.0.)goto13
      nbr=nbr+1
      GOTO 12
   13 CONTINUE
      ITRD(J)=-1
      XBREAK(I)=-1
      YBREAK(I)=-1
   12 CONTINUE	! loop J
      if(nbr.eq.0) goto 9994
 
c          print
c          -----
           if(ish.ge.92)then
           write(ifch,*)'icmpbr(1/2,)  x/ybreak  px/ybreak:'
           do 53 ib=1,ibr+1
           write(ifch,100)icmpbr(1,ib),icmpbr(2,ib)
     *     ,xbreak(ib),ybreak(ib),pxbrk(ib),pybrk(ib)
53         write(ifch,100)icpmbr(1,ib),icpmbr(2,ib)
           endif
 
c  Build new strings
c  -----------------
 
      NBREAK=0
      TAUAVE=0.
      IL=1
      IN=IL
      IR=1
      LAST=.FALSE.
   11 CONTINUE
      IR=IR+1
      IF(IR.EQ.(IBR+1)) LAST=.TRUE.
      IF(XBREAK(IR).LT.0.) GOTO 11	! breakpoint was not acceptable
      IF(IN.EQ.1) GOTO 14
   15 CONTINUE
           if(ish.ge.92)then
           write(ifch,*)'il in ir:  ',il,in,ir
           endif
      PML=XBREAK(IL)-XBREAK(IN)
      PPL=YBREAK(IN)-YBREAK(IL)
      PMR=(XBREAK(IN)-XBREAK(IR))
      PPR=(YBREAK(IR)-YBREAK(IN))
      PX=pxbrk(IN)
      PY=pybrk(IN)
      PXL=PX-pxbrk(IL)
      PYL=PY-pybrk(IL)
      PTL2=(PXL*PXL+PYL*PYL)
      AMM=SQRT(PML*PPL-PTL2)
      PXR=pxbrk(IR)-PX
      PYR=pybrk(IR)-PY
      PTR2=(PXR*PXR+PYR*PYR)
      AMP=SQRT(PMR*PPR-PTR2)
      call utresm(icmpbr(1,ir),icmpbr(2,ir)
     *,icpmbr(1,in),icpmbr(2,in),amp,idpr,iadjp,ireten)
      call utresm(icpmbr(1,il),icpmbr(2,il)
     *,icmpbr(1,in),icmpbr(2,in),amm,idmr,iadjm,ireten)
      AREA=PPL*PMR
      P0P=PPL+PPR
      P0M=PML+PMR
      tmm=sqrt(ptL2+amm**2)
      tmP=sqrt(ptR2+amP**2)
           if(tmp*tmp.gt.ppr*pmr+1e-4)then
           call utmsg('jambr2')
           write(ifch,*)'*****  tmp*tmp.gt.ppr*pmr'
           write(ifch,*)'tmp*tmp ppr*pmr: ',tmp*tmp,ppr*pmr
           call utmsgf
           endif
	   if(tmm*tmm.gt.ppl*pml+1e-4)then
           call utmsg('jambr2')
           write(ifch,*)'*****  tmm*tmm.gt.ppl*pml'
           write(ifch,*)'tmm*tmm ppl*pml: ',tmm*tmm,ppl*pml
           call utmsgf
           endif
      ETA=0.5*ALOG(PPL/PMR)
c ------ adjust p not m
      if(iadjp.eq.1.and.iadjm.ne.1)then
           if(ish.ge.92)write(ifch,*)'adjust p not m'
      etaNEW=dlog((sqrt(area)*p0p)/(tmp**2+area))
      if(etaNEW.lt.eta-1e-4) THEN	!should never happen
           call utmsg('jambr2')
           write(ifch,*)'*****  etanew.lt.eta'
           write(ifch,*)'etanew eta: ',etanew,eta
           call utmsgf
      ETANEW=ETA
      ENDIF
      xbreak(IN)=SQRT(AREA)*EXP(-ETANEW)+XBREAK(IR)
      YBREAK(IN)=SQRT(AREA)*EXP(ETANEW)+YBREAK(IL)
c ------ adjust m not p
      ELSEif(iadjM.eq.1.and.iadjP.ne.1)then
           if(ish.ge.92)write(ifch,*)'adjust m not p'
      etaNEW=-dlog((sqrt(area)*p0M)/(tmM**2+area))
      if(etaNEW.Gt.eta+1e-4) THEN	!should never happen
           call utmsg('jambr2')
           write(ifch,*)'*****  etanew.gt.eta'
           write(ifch,*)'etanew eta: ',etanew,eta
           call utmsgf
      ETANEW=ETA
      ENDIF
      xbreak(IN)=SQRT(AREA)*EXP(-ETANEW)+XBREAK(IR)
      YBREAK(IN)=SQRT(AREA)*EXP(ETANEW)+YBREAK(IL)
c ------ adjust both
      ELSEif(iadjp.eq.1.and.iadjm.eq.1)then
           if(ish.ge.92)write(ifch,*)'adjust both'
      tmp2=tmp**2
      tmm2=tmm**2
      d=(p0p*p0m-tmp2-tmm2)**2-4*tmp2*tmm2
           if(d.lt.0.)then
           call utmsg('jambr2')
           write(ifch,*)'***** negative d'
           write(ifch,*)'d: ',d
           call utmsgf
           d=0.
           endif
      d=sqrt(d)
      t=p0p*p0m-tmp2+tmm2
      p1=(t+d)/(2*p0m)
      p2=(t-d)/(2*p0m)
      if((p1+p2)/2.gt.ppl)then
      p=p2
      else
      p=p1
      endif
      xbreak(in)=p0m-tmm2/p+xbreak(ir)
      ybreak(in)=p+ybreak(il)
      endif
c ------
           if(ish.ge.92)then
           write(ifch,*)'x/ybreak:',xbreak(in),ybreak(in)
           endif
 
c     write sg
c     --------
      NBREAK=NBREAK+1
      TAUAVE=TAUAVE+XBREAK(IN)*YBREAK(IN)
      MM=1
      IF(LAST) MM=2
      do 9998 m=1,MM
      nsg=nsg+1
           if(nsg.gt.mxsg)
     *     call utstop('jambr2: nsg>mxsg                        ')
           if(ish.ge.92)write(ifch,*)'nsg last:  ',nsg,last
      isg(nsg)=nptlc
      IF(M.EQ.2) THEN
      IL=IN
      IN=IR
      ENDIF
      xor2(1)=0
      xor2(2)=0
      xor2(3)=(YBREAK(IL)-XBREAK(IN))/2/tensn
      xor2(4)=(YBREAK(IL)+XBREAK(IN))/2/tensn
      if(ish.ge.92)write(ifch,119)xor2
119   format(' origin: ',13x,4f7.2)
      call utrot2(-1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,xor2(1),xor2(2),xor2(3))
      call utlob2(-1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,xor2(1),xor2(2),xor2(3),xor2(4))
      do 46 j=1,4
46    xor2(j)=xor2(j)+xorsg(j,ns)
c          if(ish.ge.92)write(ifch,117)xor2
      do 42 j=1,4
42    xorsg(j,nsg)=xor2(j)
      PP=YBREAK(IN)-YBREAK(IL)
      PM=XBREAK(IL)-XBREAK(IN)
      psg(1,nsg)=pxbrk(in)-pxbrk(il)
      psg(2,nsg)=pybrk(in)-pybrk(il)
      psg(3,nsg)=(pp-pm)/2
      psg(4,nsg)=(pp+pm)/2
      ss=
     *psg(4,nsg)**2-psg(3,nsg)**2-psg(2,nsg)**2-psg(1,nsg)**2
      psg(5,nsg)=sqrt(ss)
           if(ish.ge.92)write(ifch,110)nsg,ns,(psg(j,nsg),j=1,5)
110        format(2x,i3,i4,19x,5(e8.2,1x))
      do 55 k=1,4
55    pw(k)=psg(k,nsg)
      call utrot2(-1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,psg(1,nsg),psg(2,nsg),psg(3,nsg))
      call utlob2(-1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,psg(1,nsg),psg(2,nsg),psg(3,nsg),psg(4,nsg))
      iorsg(nsg)=ns
           if(ish.ge.92)
     *     write(ifch,102)nsg,iorsg(nsg),(psg(j,nsg),j=1,5)
102        format(2x,i3,i4,19x,5(e8.2,1x))
 
c     write jt
c     --------
      do 41 j=1,2
      icjt(j,2*nsg-1)=icmpBR(j,IN)
41    continue
      do 43 j=1,2
43    icjt(j,2*nsg)=icPMBR(j,IL)
      idx=iadjm
      if(m.eq.2)idx=iadjp
           if(idx.eq.0)then
      pjt(1,2*nsg-1)=pxbrk(IN)            ! right jet
      pjt(2,2*nsg-1)=pybrk(IN)
      PTJR2=pxbrk(IN)**2+pybrk(IN)**2
      PTJL2=pxbrk(IL)**2+pybrk(IL)**2
      AA=PM*PP-PTJL2-PTJR2
      PRM=(AA/2+PTJR2-SQRT(AA*AA/4-PTJL2*PTJR2))/PP
      PLP=(AA/2+PTJL2-SQRT(AA*AA/4-PTJL2*PTJR2))/PM
      pjt(3,2*nsg-1)=(PP-PLP-PRM)/2
      pjt(4,2*nsg-1)=(pp-PLP+PRM)/2
      pjt(5,2*nsg-1)=0
      pjt(1,2*nsg)=-pxbrk(IL)             ! left jet
      pjt(2,2*nsg)=-pybrk(IL)
      pjt(3,2*nsg)=(PLP-PM+PRM)/2
      pjt(4,2*nsg)= (PLP+PM-PRM)/2
      pjt(5,2*nsg)=0
           if(ish.ge.92)then
      write(ifch,108)nsg,ns
     *,(icjt(j,2*nsg-1),j=1,2),(sngl(pjt(j,2*nsg-1)),j=1,5)
108   format(2x,i3,i4,2x,2i7,3x,5(e8.2,1x))
      write(ifch,108)nsg,ns
     *,(icjt(j,2*nsg  ),j=1,2),(sngl(pjt(j,2*nsg  )),j=1,5)
           endif
      err=0
      do 44 j=1,4
44    err=err+(pw(j)-pjt(j,2*nsg-1)-pjt(j,2*nsg))**2
           endif
      do 56 i=1,2
      icpz(i)=icjt(i,2*nsg-1)
56    icmz(i)=icjt(i,2*nsg)
      call iddeco(icpz,jcpz)
      call iddeco(icmz,jcmz)
      do 57 nf=1,nflav
      jcz(nf,1)=jcpz(nf,1)+jcmz(nf,1)
57    jcz(nf,2)=jcpz(nf,2)+jcmz(nf,2)
      call idenco(jcz,icz,ireten)
      idz=idtra(icz,0,0,3)
      amz=psg(5,nsg)
      call idres(idz,amz,idrz,iadj)
      if(idrz.eq.110.and.rangen().lt.0.5)goto9993
9998  CONTINUE
 
   14 CONTINUE
      IF(LAST) GOTO 10
      IL=IN
      IN=IR
      GOTO 11
   10 CONTINUE
c
c     determine break pnt
c     -----------------------------------
      TAUAVE=TAUAVE/NBREAK
      xbk(1)=0
      xbk(2)=0
      xbk(3)=(SQRT(TAUAVE*YT/XT)-SQRT(TAUAVE*XT/YT))/2/tensn
      xbk(4)=(SQRT(TAUAVE*YT/XT)+SQRT(TAUAVE*XT/YT))/2/tensn
           if(ish.ge.92)write(ifch,116)xbk
116        format(' breaking pnt: ',13x,4f7.2)
      call utrot2(-1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,xbk(1),xbk(2),xbk(3))
      call utlob2(-1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,xbk(1),xbk(2),xbk(3),xbk(4))
      do 45 j=1,4
45    xbk(j)=xbk(j)+xorsg(j,ns)
      if(ish.ge.92)write(ifch,117)xbk
117   format(28x,4f7.2)
C
      xbkptl(1,nptlc-nptl2)=xbk(3)
      xbkptl(2,nptlc-nptl2)=xbk(4)
      isptl(nptlc-nptl2)=1
      return
      end
 
c-----------------------------------------------------------------------
      subroutine jamfra(js,newevt)
c-----------------------------------------------------------------------
c     fragments string js according to a-m model.
c     version mar-92 (calls jambr1 or jambr2)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,taufos
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cdelre/delrer
 
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
 
      parameter (mxpc=500,mxsg=500)
      parameter (mxjt=2*mxsg)
      double precision xbkptl
      common/cpc/xbkptl(2,mxpc),isptl(mxpc)
      double precision psg,rotsg,pjt,xorsg
      common/csg/psg(5,mxsg),rotsg(3,mxsg),pjt(5,mxjt),xorsg(4,mxsg)
     *,isg(mxsg),icjt(2,mxjt),iorsg(mxsg)
      common/cjambr/nptl2,nptlc
 
      parameter (nptf=129)
      common /cptf/xptf(nptf),wptf(nptf)
     *,qptfu(nptf),qptfs(nptf),qptfuu(nptf),qptfus(nptf),qptfss(nptf)
      parameter (ngau=129)
      common /cgau/xgau(ngau),qgau(ngau),wgau(ngau)
 
      parameter (nflav=6)
      integer jcp(nflav,2),jcm(nflav,2)
     *,icp(2),icm(2),icup(2),icum(2)
     *,ic(2),jc(nflav,2)
     *,icj1(2),icj2(2),jcj1(nflav,2),jcj2(nflav,2)
      double precision tor,zor
      double precision arp(4),arm(4)
 
      delrec=0.600
      zero=0
      newevt=0
 
      psg(1,1)=pstr(1,js)
      psg(2,1)=pstr(2,js)
      psg(3,1)=pstr(3,js)
      psg(4,1)=pstr(4,js)
      psg(5,1)=pstr(5,js)
      rotsg(1,1)=rotstr(1,js)
      rotsg(2,1)=rotstr(2,js)
      rotsg(3,1)=rotstr(3,js)
      xorsg(1,1)=xorstr(1,js)
      xorsg(2,1)=xorstr(2,js)
      xorsg(3,1)=xorstr(3,js)
      xorsg(4,1)=xorstr(4,js)
      s=pstr(5,js)**2
      isg(1)=iorstr(js)
      iorsg(1)=0
      do 1 i=1,2
      pjt(1,i)=0
      pjt(2,i)=0
      pjt(3,i)=sqrt(s)/2*(3-2*i)
      pjt(4,i)=sqrt(s)/2
      pjt(5,i)=0
      j0=2*i-2
      if(rotstr(3,js).lt.0.)j0=4-2*i
      do 3 j=1,2
3     icjt(j,i)=icstr(j0+j,js)
1     continue
      icj1(1)=icjt(1,1)
      icj1(2)=icjt(2,1)
      icj2(1)=icjt(1,2)
      icj2(2)=icjt(2,2)
      call iddeco(icj1,jcj1)
      call iddeco(icj2,jcj2)
      nqj1=0
      nqj2=0
      do 4 nf=1,nflav
      nqj1=nqj1+jcj1(nf,1)-jcj1(nf,2)
4     nqj2=nqj2+jcj2(nf,1)-jcj2(nf,2)
      if(nqj1.ge.0.and.nqj2.ge.0)isi=0
      if(nqj1.ge.0.and.nqj2.lt.0)isi=1
      if(nqj1.lt.0.and.nqj2.ge.0)isi=2
      if(nqj1.lt.0.and.nqj2.lt.0)isi=3
      nqjstr=isi*10**6+iabs(nqj1)*10**3+iabs(nqj2)
 
c     entry string fragmentation
c     --------------------------
      nptl2=nptl
      ncnt2=0
9996  ncnt2=ncnt2+1
      nptl=nptl2
      nsg=1
      if(ncnt2.gt.1000)goto1001
 
c     entry substring processing
c     --------------------------
      ns=0
9999  ns=ns+1
      if(ns.gt.nsg)goto9997
      if(ish.ge.92)write(ifch,*)'entry substring processing'
      if(ish.ge.92)write(ifch,107)ns
     *,(icjt(j,2*ns-1 ),j=1,2),(sngl(pjt(j,2*ns-1 )),j=1,5)
107   format(2x,i3,3x,3x,2i7,3x,5(e8.2,1x))
      if(ish.ge.92)write(ifch,107)ns
     *,(icjt(j,2*ns   ),j=1,2),(sngl(pjt(j,2*ns   )),j=1,5)
      if(ish.ge.92)write(ifch,101)ns,(psg(j,ns),j=1,5)
101   format(2x,i3,23x,5(e8.2,1x))
114   format(' left breaking pnt: ',8x,4f7.2)
 
c     determine id and min.mass
c     -------------------------
      delrer=delrex+rangen()*(delrem-delrex)
      do 6 i=1,2
      icp(i)=icjt(i,2*ns-1)
6     icm(i)=icjt(i,2*ns)
      call idcomk(icp)
      call idcomk(icm)
      do 29 i=1,2
      icjt(i,2*ns-1)=icp(i)
29    icjt(i,2*ns)=icm(i)
      call iddeco(icp,jcp)
      call iddeco(icm,jcm)
      amms=utamnx(jcm,jcp)
      nubar=0
      do 7 nf=1,nflav
      jc(nf,1)=jcp(nf,1)+jcm(nf,1)
      jc(nf,2)=jcp(nf,2)+jcm(nf,2)
      if(nf.gt.4.and.(jc(nf,1).ne.0.or.jc(nf,2).ne.0))
     *call utstop('jamfra: flavour > 4                     ')
7     nubar=nubar+jc(nf,1)-jc(nf,2)
      if(ns.eq.1)numbar=nubar
      call idenco(jc,ic,ireten)
      call idcomk(ic)
      id=idtra(ic,0,0,3)
      idk=id
      am=psg(5,ns)
      iremn=0
      if(icp(1)+icp(2).eq.0.or.icm(1)+icm(2).eq.0)iremn=1
      if(am.gt.amms+delrer.and.iremn.eq.0)id=0
 
c     mass adjustment => fragment again
c     ---------------------------------
      amc=psg(5,ns)
      call idres(id,amc,idr,iadj)
      if(abs(amc-psg(5,ns)).gt.1e-3)then
      if(ish.ge.90)then
      call utmsg('jamfra')
      write(ifch,*)'*****  mass changed. fragment again.'
      write(ifch,*)'mass before, after: ',psg(5,ns),amc
      write(ifch,*)'ic,id,idr: ',ic,id,idr
      call utmsgf
      endif
      goto9996
      endif
 
c     if mass < min.mass ==> again
c     ----------------------------
c-c        if(idr.eq.0.and.am.lt.amms-1e-3)then
c-c   if(ish.ge.90)then
c-c   call utmsg('jamfra')
c-c   write(ifch,*)'*****  mass < min mass. ',am,amms
c-c   write(ifch,*)'idk,id,idr,psg(5,ns):'
c-c   write(ifch,*)idk,id,idr,psg(5,ns)
c-c   write(ifch,*)'p_jet:'
c-c   write(ifch,107)ns
c-c  *,(icjt(j,2*ns-1 ),j=1,2),(sngl(pjt(j,2*ns-1 )),j=1,5)
c-c   write(ifch,107)ns
c-c  *,(icjt(j,2*ns   ),j=1,2),(sngl(pjt(j,2*ns   )),j=1,5)
c-c   write(ifch,*)'p_str:'
c-c   write(ifch,101)ns,(psg(j,ns),j=1,5)
c-c   call utmsgf
c-c   endif
c-c   if(ns.eq.1)goto1001
c-c   goto9996
c-c        endif
 
c     write on /cptl/ (1)
c     -------------------
      iori=isg(ns)
      nptlb=nptl
9995  nptl=nptl+1
      nptlc=nptl
      if(nptl.gt.mxptl)
     *call utstop('jamfra: nptl>mxptl                      ')
      if(nptl-nptl2.gt.mxpc)
     *call utstop('jamfra: nptl-nptl2>mxpc                 ')
      do 40 j=1,5
40    pptl(j,nptl)=psg(j,ns)
      do 38 j=1,4
38    xorptl(j,nptl)=xorsg(j,ns)
      xbkptl(1,nptl-nptl2)=ainfin
      xbkptl(2,nptl-nptl2)=ainfin
      tivptl(1,nptl)=-ainfin
      tivptl(2,nptl)=ainfin
      ifrptl(1,nptl)=0
      ifrptl(2,nptl)=0
      iclptl(nptl)=0
      isptl(nptl-nptl2)=0
      iorptl(nptl)=iori
      jorptl(nptl)=0
      idptl(nptl)=idr
           if(idr.eq.0)then
      if(ic(1).eq.0.and.ic(2).eq.0)then
      idptl(nptl)=7*10**8
      call idtrbi(jc,ibptl(1,nptl),ibptl(2,nptl)
     *,ibptl(3,nptl),ibptl(4,nptl))
      else
      call idtrbi(jc,idum1,idum2,idum3,idum4)
      idptl(nptl)=8*10**8+ic(1)*100+ic(2)/100
      endif
           endif
           if(ish.ge.91)then
      write(ifch,115)nptl,iorptl(nptl),idptl(nptl)
     *,(pptl(j,nptl),j=1,5)
115   format(1x,'/cptl/',i4,i6,i10,3x,5(e8.2,1x))
      if(idptl(nptl).eq.7e8)write(ifch,*)(ibptl(i,nptl),i=1,4)
           endif

c     if resonance: finished. goto9999 and treat next substring.
c     ----------------------------------------------------------
      if(idr.ne.0)goto9999

c     if quark-cluster: finished. goto9999 and treat next substring.
c     --------------------------------------------------------------
      if(icp(1)+icp(2).eq.0.or.icm(1)+icm(2).eq.0)goto78
      if(am.gt.amms+delrer)goto77
      if(idk.eq.0.and.am.gt.amms+delrec)goto77
      if(idk.ne.0.and.am.gt.amms+delrex)goto77
78    continue
      if(ish.ge.92)write(ifch,*)'quark-cluster'
      if(icp(1)+icp(2).eq.0.and.icm(1)+icm(2).eq.0)
     *call utstop('jamfra: zero string.                    ')
      goto9999
77    continue

c     break substring
c     ---------------

      nqp=0
      nqm=0
      nap=0
      nam=0
      np=0
      nm=0
      jp=0
      jm=0
      do 23 nf=1,nflav
      nqp=nqp+jcp(nf,1)
      nqm=nqm+jcm(nf,1)
      nap=nap+jcp(nf,2)
      nam=nam+jcm(nf,2)
      np=np+jcp(nf,1)-jcp(nf,2)
      nm=nm+jcm(nf,1)-jcm(nf,2)
      jp=jp+jcp(nf,1)+jcp(nf,2)
23    jm=jm+jcm(nf,1)+jcm(nf,2)
 
c     jet trafos
c     ----------
      if(ish.ge.92)write(ifch,*)'jet trafos'
      do 75 k=1,4
      arp(k)=pjt(k,2*ns-1)
75    arm(k)=pjt(k,2*ns)
      if(ish.ge.90)call utchm(arp,arm,1)
      nso=iorsg(ns)
      if(nso.gt.0)then
      call utrot2(-1,rotsg(1,nso),rotsg(2,nso),rotsg(3,nso)
     *,arp(1),arp(2),arp(3))
      call utlob2(-1,psg(1,nso),psg(2,nso),psg(3,nso),psg(4,nso)
     *,psg(5,nso),arp(1),arp(2),arp(3),arp(4))
      call utrot2(-1,rotsg(1,nso),rotsg(2,nso),rotsg(3,nso)
     *,arm(1),arm(2),arm(3))
      call utlob2(-1,psg(1,nso),psg(2,nso),psg(3,nso),psg(4,nso)
     *,psg(5,nso),arm(1),arm(2),arm(3),arm(4))
      if(ish.ge.90)call utchm(arp,arm,2)
      call utlob2(1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,arp(1),arp(2),arp(3),arp(4))
      call utlob2(1,psg(1,ns),psg(2,ns),psg(3,ns),psg(4,ns),psg(5,ns)
     *,arm(1),arm(2),arm(3),arm(4))
      if(ish.ge.90)call utchm(arp,arm,3)
      rotsg(1,ns)=(arp(1)-arm(1))/2
      rotsg(2,ns)=(arp(2)-arm(2))/2
      rotsg(3,ns)=(arp(3)-arm(3))/2
      call utrot2(1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,arp(1),arp(2),arp(3))
      call utrot2(1,rotsg(1,ns),rotsg(2,ns),rotsg(3,ns)
     *,arm(1),arm(2),arm(3))
      endif
      do i=1,2
      icup(i)=icjt(i,2*ns-1)
      icum(i)=icjt(i,2*ns)
      enddo
           if(arp(3).lt.0)then
      do i=1,2
      icjt(i,2*ns-1)=icum(i)
      icjt(i,2*ns)=icup(i)
      enddo
      do k=1,4
      pjt(k,2*ns-1)=arm(k)
      pjt(k,2*ns)=arp(k)
      enddo
           else
      do i=1,2
      icjt(i,2*ns-1)=icup(i)
      icjt(i,2*ns)=icum(i)
      enddo
      do k=1,4
      pjt(k,2*ns-1)=arp(k)
      pjt(k,2*ns)=arm(k)
      enddo
           endif
      s=psg(5,ns)**2
           if(abs(sngl(pjt(3,2*ns-1)-pjt(3,2*ns))-sqrt(s)).gt.1e-2
     *.and.abs(sngl(pjt(3,2*ns-1)-pjt(3,2*ns))-sqrt(s)).gt.
     *1e-2*sqrt(s))then
      if(ish.ge.90)then
      call utmsg('jamfra')
      write(ifch,*)'*****  pz1-pz2-sqrt(s) nonzero'
      write(ifch,*)'value:   ',sngl(pjt(3,2*ns-1)-pjt(3,2*ns))-sqrt(s)
      write(ifch,*)'sqrt(s): ',sqrt(s)
      write(ifch,*)(sngl(pjt(i,2*ns-1)),i=1,4)
      write(ifch,*)(sngl(pjt(i,2*ns)),i=1,4)
      call utmsgf
      endif
           endif
      if(ish.ge.92)write(ifch,103)ns,nso
     *,(icjt(j,2*ns-1 ),j=1,2),(sngl(pjt(j,2*ns-1 )),j=1,5)
103   format(2x,i3,i3,3x,2i7,3x,5(e8.2,1x))
      if(ish.ge.92)write(ifch,104)
     * (icjt(j,2*ns   ),j=1,2),(sngl(pjt(j,2*ns   )),j=1,5)
104   format(2x,9x,2i7,3x,5(e8.2,1x))
 
      if(iopbrk.eq.1)call jambr1(ns,nsg,iret)   ! AMOR
      if(iopbrk.eq.2)call jambr2(ns,nsg,iret)   ! SAMBA
      if(iret.eq.9996)goto9996
      goto9999
 
c     write on /cptl/ (2)
c     -------------------
9997  continue
      if(nptl.le.nptl2)
     *call utstop('jamfra: nptl<=nptl2                     ')
      do 67 i=nptl2+1,nptl
      istptl(i)=isptl(i-nptl2)
      io=iorptl(i)
      if(io.gt.0)then
      if(ifrptl(1,io).eq.0)ifrptl(1,io)=i
      ifrptl(2,io)=i
      else
      iorptl(i)=0
      endif
      zor=xorsg(3,1)
      tor=xorsg(4,1)
      r=rangen()
      tauran=-taurea*alog(r)
      call uttaix(i,tauran,zor,tor,zis,tis)
      tivptl(1,i)=amax1(tis,xorptl(4,i))
           if(istptl(i).ne.0)then
      taubr=
     *dsqrt((xbkptl(2,i-nptl2)-tor)**2-(xbkptl(1,i-nptl2)-zor)**2)
      call uttaix(i,taubr,zor,tor,zis,tis)
      tivptl(2,i)=tis
           elseif(io.gt.0)then
      taubr=
     *dsqrt((xbkptl(2,io-nptl2)-tor)**2-(xbkptl(1,io-nptl2)-zor)**2)
      call uttaix(i,taubr,xorsg(3,1),xorsg(4,1),zisbr,tisbr)
      call idtau(idptl(i),pptl(4,i),pptl(5,i),taugm)
      tivptl(2,i)=tisbr+taugm*(-alog(rangen()))
           else
      call idtau(idptl(i),pptl(4,i),pptl(5,i),taugm)
      tivptl(2,i)=xorptl(4,i)+taugm*(-alog(rangen()))
           endif
      if(ish.ge.92)write(ifch,120)i,iorptl(i),idptl(i)
     *,(sngl(xbkptl(j,i-nptl2)),j=1,2)
     *,(xorptl(j,i),j=3,4),(tivptl(j,i),j=1,2)
120   format(1x,'/cptl/',i4,i5,i10
     *,2x,e8.2,1x,e8.2,1x,e8.2,1x,e8.2,1x,e8.2,1x,e8.2)
67    continue
 
1000  return
 
1001  newevt=1
      goto1000
 
      end

c----------------------------------------------------------------------
      subroutine jcentd
c----------------------------------------------------------------------
      parameter (kparx=15,nquax=12)
      common/centro/entro(1+kparx,1+nquax)
      real entrox(1+kparx,1+nquax)

      data (entrox( 1,1+n),n=0,12)/
     *.00000E+00,.00000E+00,.00000E+00,.00000E+00,.00000E+00,.00000E+00,
     *.00000E+00,.00000E+00,.00000E+00,.00000E+00,.00000E+00,.00000E+00,
     *.00000E+00/
      data (entrox( 4,1+n),n=0,12)/
     *.40254E+01,.59349E+01,.74855E+01,.87464E+01,.98267E+01,.10770E+02,
     *.11611E+02,.12368E+02,.13058E+02,.13691E+02,.14277E+02,.14822E+02,
     *.15332E+02/
      data (entrox( 7,1+n),n=0,12)/
     *.61944E+01,.89306E+01,.11114E+02,.12969E+02,.14595E+02,.16054E+02,
     *.17380E+02,.18599E+02,.19728E+02,.20780E+02,.21767E+02,.22696E+02,
     *.23574E+02/
      data (entrox(10,1+n),n=0,12)/
     *.68876E+01,.10183E+02,.12855E+02,.15159E+02,.17201E+02,.19046E+02,
     *.20734E+02,.22296E+02,.23751E+02,.25115E+02,.26400E+02,.27616E+02,
     *.28771E+02/
      data (entrox(13,1+n),n=0,12)/
     *.61944E+01,.99602E+01,.13068E+02,.15784E+02,.18208E+02,.20406E+02,
     *.22425E+02,.24295E+02,.26041E+02,.27681E+02,.29228E+02,.30696E+02,
     *.32092E+02/
      data (entrox(16,1+n),n=0,12)/
     *.40254E+01,.82375E+01,.11781E+02,.14923E+02,.17745E+02,.20311E+02,
     *.22669E+02,.24853E+02,.26890E+02,.28803E+02,.30609E+02,.32320E+02,
     *.33948E+02/

      kparxt=kparx
      nquaxt=nquax  

      do 1 k=1,1+kparx
      do 1 n=1,1+nquax
1     entro(k,n)=entrox(k,n)
 
      if(kparxt.ne.15)
     *call utstop('icentd: insufficient initialization;   k')
      if(nquaxt.ne.12)
     *call utstop('icentd: insufficient initialization;   n')
 
      return
      end
 
c----------------------------------------------------------------------
      subroutine jcentp
c     plots entro(,) and ftn sjcent
c----------------------------------------------------------------------
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (kparx=15,nquax=12)
      common/centro/entro(1+kparx,1+nquax)
      real xplot(101),yplot(101)
      character text*50
      common/cjcent/nsymx,igx
      integer iplot(5)
      data iplot/0,0,0,1,1/
      ish0=ish
      if(ishsub/100.eq.13)ish=mod(ishsub,100)
      if(.not.(ish.ge.95))goto1000
 
      if(iplot(1).eq.1)then
      text='title ig=     nsym=     k= 3,6,9,12,15$       '
      write(text(10:11),122)igx
      write(text(20:21),122)nsymx
122   format(i2)
      do 54 kx=1,5
      k=3*kx
      do 55 n=0,12
      xplot(1+n)=n
55    yplot(1+n)=entro(1+k,1+n)
      call uthist(0.,12.,0.,40.,1+12,xplot,yplot,'lin','linlin'
     *,'xaxis energy / omega            $                 '
     *,'yaxis entropy                   $                 ',text)
54    continue
      endif
 
      if(iplot(2).eq.1)then
      text='title ig=     nsym=     k= 3,6,9,12,15$       '
      write(text(10:11),122)igx
      write(text(20:21),122)nsymx
      do 64 kx=1,5
      k=3*kx
      do 65 n=0,12
      xplot(1+n)=n*1./k
65    yplot(1+n)=entro(1+k,1+n)/k
      call uthist(0.,5.,0.,6.,1+12,xplot,yplot,'lin','linlin'
     *,'xaxis energy / omega / particle $                 '
     *,'yaxis entropy / particle        $                 ',text)
64    continue
      endif
 
      if(iplot(3).eq.1)then
      do 62 kx=1,10
      k=3*kx
      do 63 n=0,100
      x=n/20.
      xplot(1+n)=x
63    yplot(1+n)=sjcent(k,k,k*x)/k
      call uthist(0.,5.,0.,6.,101,xplot,yplot,'lin','linlin'
     *,'xaxis energy / omega / particle $                 '
     *,'yaxis entropy / particle        $                 '
     *,'title inter(extra)polated       $                 ')
62    continue
      endif
 
      if(iplot(4).eq.1)then
      do 58 kx=1,10
      k=3*kx
      do 59 n=0,100
      x=n/2.
      xplot(1+n)=x
59    yplot(1+n)=sjcent(k,k,x)
      call uthist(0.,50.,0.,80.,101,xplot,yplot,'lin','linlin'
     *,'xaxis energy / omega            $                 '
     *,'yaxis entropy                   $                 '
     *,'title inter(extra)polated  ku=k $                 ')
58    continue
      endif
 
      if(iplot(5).eq.1)then
      do 48 kx=1,10
      k=3*kx
      do 49 n=0,100
      x=n/2.
      xplot(1+n)=x
49    yplot(1+n)=sjcent(k,0,x)
      call uthist(0.,50.,0.,80.,101,xplot,yplot,'lin','linlin'
     *,'xaxis energy / omega            $                 '
     *,'yaxis entropy                   $                 '
     *,'title inter(extra)polated  ku=0 $                 ')
48    continue
      endif
 
1000  continue
      ish=ish0
      return
      end
 
c----------------------------------------------------------------------
      subroutine jcentr(nsym,ig,idi,init)
c     fills array degen(1+k,1+n) containing
c       the number of mixed symmetric (box of width nsym)
c       k-particle states with energy n (units of omega), based on
c       oscillator (dim: idi) wavefunctions (addit. degeneracy: ig) .
c     fills array entro(1+k,1+n) = alog( degen(1+k,1+n) )  .
c     entro(,) written in form of data for jcentd if ish=95
c       (ishsub=12.. selects this).
c     init must be set 1 for the first call of jcentr
c       (init=1 calls uttucl, utpart, utovel)
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (kparx=15,nquax=12)
      parameter (nymax=1000)
      parameter (jpamax=nquax*nquax*nquax,kkpmax=nquax*nquax*2)
      common/cdegen/degen(1+kparx,1+nquax)
      common/centro/entro(1+kparx,1+nquax)
      character*1 deli(1+kparx,1+nquax)
      common/cparta/parta(nquax),ipart(nquax,jpamax)
      integer iyo(nymax,kparx),kkk(kparx),iyox(kparx)
     *,jyo(nymax)
      character*1 cyox(1+nquax,kparx)
      double precision tucl
      parameter (ktumax=100,ntumax=100)
      common/ctucl/tucl(1+ktumax,1+ntumax)
      parameter (itamax=1000,nsymax=20)
      integer iyoz(itamax,nsymax,2),iyol(itamax),iyor(nsymax)
     *,iyoo(itamax),iyom(itamax)
      real yofa(itamax)
      parameter (iovmax=100,jovmax=100)
      common/covel/ovel(1+iovmax,1+jovmax)
      character ciga*7
      common/cjcent/nsymx,igx
 
      if(init.eq.1)then
      call uttucl
      call utpart
      call utovel
      endif
 
      write(ifmt,*)'execute sr jcentr ...'
      if(nsym.ge.nsymax)
     *call utstop('jcentr: dimension nsymax too small      ')
      if(mod(kparx,nsym).ne.0)
     *call utstop('jcentr: kparx should be multiple of nsym')
      mmax=1+(kparx-1)/nsym
      izero=0
      nsymx=nsym
      igx=ig
 
c     loop over energy
c     ----------------
      do 6001 n=0,nquax
      write(ifmt,*)'sr jcentr: n=',n
      if(n.eq.0)then
      kkp=1
      else
      kkp=-ipart(n,1)
      if(kkp.gt.kkpmax)
     *call utstop('jcentr: dimension kkpmax too small      ')
      if(-ipart(n,1).ne.nint(parta(n)))
     *call utstop('jcentr: # of partitions disagree        ')
      endif
      if(ish.ge.93)then
      write(ifch,103)('-',ic=1,79),n,kkp,('-',ic=1,79)
103   format(79a1/7x,'n = ',i2,'   --->   ',i4,' partition(s)'/79a1)
      jj=2
      do 8 kk=1,kkp
      if(n.gt.0)ll=-ipart(n,jj)
      if(n.eq.0)write(ifch,102)kk,izero
      if(n.gt.0)
     *write(ifch,102)kk,(ipart(n,i),i=jj+1,jj+min0(20,ll))
102   format(2x,i2,'. partition:',3x,20i3)
8     jj=jj+ll+1
      endif
 
c     zero particles
c     --------------
      degen(1,1+n)=1
      entro(1,1+n)=0
      deli(1,1+n)=','
      if(n.eq.nquax)deli(1,1+n)='/'
 
c     loop over particle number
c     -------------------------
      do 6001 kx=1,kparx/3
      k=3*kx
      if(k.gt.kparx)
     *call utstop('jcentr: dimension kparx too small       ')
      degen(1+k,1+n)=0
      deli(1+k,1+n)=','
      if(n.eq.nquax)deli(1+k,1+n)='/'
      kkk(k)=kkp
      mmaxk=1+(k-1)/nsym
      if(k.lt.kparx)then
      do 5 ny=1,nymax
      do 5 l=k+1,kparx
5     iyo(ny,l)=-1
      endif
      if(ish.ge.93)write(ifch,100)('-',ic=1,11),k,('-',ic=1,11)
100   format(11a1/3x,'k = ',i2/11a1)
 
c     loop over partitions
c     --------------------
      jj=2
      do 6002 kk=1,kkp
      if(n.gt.0)ll=-ipart(n,jj)
      if(n.eq.0)ll=1
      if(ll.gt.k)then
      kkk(k)=kk-1
      goto6003
      endif
      if(ish.ge.93.and.n.eq.0)
     *write(ifch,102)kk,izero
      if(ish.ge.93.and.n.gt.0)
     *write(ifch,102)kk,(ipart(n,i),i=jj+1,jj+min0(20,ll))
 
c     construct young tableaus
c     ------------------------
      ny1=1
      ny2=1
      jyo(ny1)=1
      do 46 l=1,k
46    iyo(ny1,l)=0
           if(n.gt.0)then
      iyo(ny1,k)=ipart(n,jj+1)
      ny1=ny1-1
45    ny1=ny1+1
      if(jyo(ny1).eq.ll)goto51
      do 43 lx=1,nsym
      do 44 mx=1,mmaxk
      l=nsym+1-lx
      m=mmaxk+1-mx
           if(iyo(ny1,(m-1)*nsym+l).eq.0)then
      if(l.eq.nsym.or.(l.ne.nsym.and.iyo(ny1,(m-1)*nsym+l+1).ne.0))then
      ny2=ny2+1
      if(ny2.gt.nymax)
     *call utstop('jcentr: dimension nymax too small       ')
      jy=jyo(ny1)+1
      jyo(ny2)=jy
      do 50 lp=1,k
50    iyo(ny2,lp)=iyo(ny1,lp)
      iyo(ny2,(m-1)*nsym+l)=ipart(n,jj+jy)
      if(ny2.gt.1)then
      do 47 ny3=1,ny2-1
      if(jyo(ny3).ne.jyo(ny2))goto47
      do 48 lp=1,k
48    if(iyo(ny3,lp).ne.iyo(ny2,lp))goto47
      ny2=ny2-1
      goto49
47    continue
49    continue
      endif
      endif
      goto43
           endif
44    continue
43    continue
      goto45
51    continue
          endif
 
c     loop over young tableaus
c     ------------------------
      do 6005 ny=ny1,ny2
 
      if(ish.ge.93.and.nsym.eq.3)then
      write(ifch,117)ny-ny1+1,((iyo(ny,(m-1)*nsym+i),i=1,nsym),m=1,1)
117   format(2x,i2,'. tableau:',5x,3i2)
      if(mmaxk.gt.1)
     *write(ifch,110)((iyo(ny,(m-1)*nsym+i),i=1,nsym),m=2,mmaxk)
110   format(19x,3i2)
      endif
 
c     select nn-subtableaus
c     ---------------------
      ditab=1
      do 6004 nn=0,n
      iga=ig*nint(dexp(tucl(1+idi,1+nn)))
 
      do 15 l=1,kparx
15    iyox(l)=-1
      do 11 m=1,mmaxk
      memp=m-1
      ms=(m-1)*nsym
      do 11 i=1,nsym
      if(iyo(ny,ms+i).eq.nn)goto12
11    continue
12    continue
      mmaxks=mmaxk-memp
      do 13 l=ms+1,k
      if(iyo(ny,l).eq.nn)iyox(l-ms)=iyo(ny,l)
13    continue
      if(nsym.gt.1)then
      do 17 i=1,nsym-1
      do 16 m=1,mmaxk
      ms=(m-1)*nsym
      if(iyox(ms+1).ne.-1)goto18
16    continue
      do 19 m=1,mmaxk
      ms=(m-1)*nsym
      do 20 l=1,nsym-1
20    iyox(ms+l)=iyox(ms+l+1)
      iyox(ms+nsym)=-1
19    continue
17    continue
18    continue
      endif
      do 38 l=1,kparx
      cyox(1+nn,l)=' '
      if(iyox(l).ge.0)cyox(1+nn,l)='X'
38    continue
 
c     skip for empty tableaus
c     -----------------------
      do 21 l=1,k
      if(iyox(l).ne.-1)goto22
21    continue
      goto6004
22    continue
 
c     print
c     -----
      if(ish.ge.93.and.nsym.eq.3)then
      if(iga.ge.  1.and.iga.lt.  10)write(ciga,107)iga
      if(iga.gt. 10.and.iga.lt. 100)write(ciga,108)iga
      if(iga.gt.100.and.iga.lt.1000)write(ciga,109)iga
107   format('GL(',i1,')  ')
108   format('GL(',i2,') ')
109   format('GL(',i3,')')
      write(ifch,116)nn,((cyox(1+nn,(m-1)*nsym+i),i=1,nsym),m=1,1)
     *,ciga
116   format(2x,i2,'-subtableau:',3x,3(1x,a1),3x,a7)
      if(mmaxks.gt.1)then
      do 39 m=2,mmaxks
39    write(ifch,106)(cyox(1+nn,(m-1)*nsym+i),i=1,nsym)
106   format(19x,3(1x,a1))
      endif
      endif
 
c     reductions of tableaus 1+iyox()
c     -------------------------------
      disuta=0
      n1=1
      n3=1
      n300=1
      do 24 i=1,nsym
      ma=0
      me=0
      do 25 m=1,mmaxk
      if(1+iyox((m-1)*nsym+i).le.0)goto25
      if(ma.eq.0)ma=m
      me=m
25    continue
      iyoz(n1,i,1)=ma
      iyoz(n1,i,2)=me
24    continue
      iyol(n1)=0
      yofa(n1)=1
      iyoo(n1)=0
      iyom(n1)=1
      n1=n1-1
5000  n1=n1+1
      n30=n3
      if(n1.gt.1.and.iyol(n1).gt.iyol(n1-1))n300=n3
      n2=n1
      do 26 i=1,nsym
26    iyor(i)=max0(0,iyoz(n1,i,2)-1)
      lev=iyol(n1)
      fac=yofa(n1)
      if(iga-1.le.iovmax.and.lev.le.jovmax.and.lev.le.iga-1)then
      facx=exp(ovel(iga,1+lev))
      if(abs(facx-fac).gt.1e-5*fac)then
      write(ifch,*)'n=',n,'    k=',k,'    kk=',kk,'    ny=',ny
      write(ifch,*)'facx=',facx,'     fac=',fac
      call utstop('jcentr: binomials differ                ')
      endif
      endif
      if(lev.gt.iga-1)goto5003
      iheim=0
      do 32 i=1,nsym
      ihei=iyoz(n1,i,2)-iyoz(n1,i,1)
      if(ihei.gt.iheim)iheim=ihei
32    continue
      if(iheim.eq.0)disuta=disuta+fac*iyom(n1)
           if(ish.ge.94.and.nsym.eq.3)then
      write(ifch,112)n1,(iyoz(n1,i,1),i=1,nsym),iyoo(n1)
     *,lev,fac,iyom(n1),iheim,disuta
     *,(iyoz(n1,i,2),i=1,nsym)
112   format(3x,i3,2x,3i2,2x,'ori:',i3,3x,'lev:',i2,3x,'fac:',f8.1
     *,3x,'mul:',i2,3x,'hei:',i2,3x,'sum:',f8.1/8x,3i2)
           endif
      n2=n2-1
5001  n2=n2+1
      do 27 i=1,nsym
      if(iyoz(n2,i,2)-1.lt.iyor(i))goto27
      if(i.lt.nsym.and.iyoz(n2,i,2)-1.lt.iyoz(n2,i+1,2))goto27
      n3=n3+1
      if(n3.gt.itamax)
     *call utstop('jcentr: dimension itamax too small      ')
      iyol(n3)=iyol(n1)+1
      yofa(n3)=( yofa(n1)*(iga-iyol(n3)) )/iyol(n3)
      iyoo(n3)=n1
      iyom(n3)=iyom(n1)
      do 28 j=1,nsym
      iyoz(n3,j,1)=iyoz(n2,j,1)
28    iyoz(n3,j,2)=iyoz(n2,j,2)
      iyoz(n3,i,2)=iyoz(n3,i,2)-1
      if(iyoz(n3,i,2).lt.iyoz(n3,i,1))then
      iyoz(n3,i,1)=0
      iyoz(n3,i,2)=0
      endif
      if(n30+1.lt.n3)then
      do 30 nch=n30+1,n3-1
      do 31 ij=1,nsym
      if(iyoz(nch,ij,1).ne.iyoz(n3,ij,1)
     *.or.iyoz(nch,ij,2).ne.iyoz(n3,ij,2))goto30
31    continue
      n3=n3-1
      goto27
30    continue
      endif
           if(ish.ge.95.and.nsym.eq.3)then
      write(ifch,113)n1,(iyoz(n1,ij,1),ij=1,nsym)
     *,n2,(iyoz(n2,ij,1),ij=1,nsym),n3,(iyoz(n3,ij,1),ij=1,nsym)
     *,(iyoz(n1,ij,2),ij=1,nsym)
     *,(iyoz(n2,ij,2),ij=1,nsym),(iyoz(n3,ij,2),ij=1,nsym)
113   format(3x,'n1: ',i2,3x,3i2,4x,'n2: ',i2,3x,3i2,4x,'n3: ',i2,3x,3i2
     */12x,3i2,13x,3i2,13x,3i2)
           endif
27    continue
      if(n2.eq.n1)n2=n30
      if(n2.lt.n3)goto5001
           if(n30.lt.n3.and.n300.lt.n30)then
      n3s=n3
      n3m=n3
      n3=n30
      do 33 nch3=n30+1,n3s
      n3=n3+1
      do 34 nch=n300+1,n30
      do 35 ij=1,nsym
      if(iyoz(nch,ij,1).ne.iyoz(n3,ij,1)
     *.or.iyoz(nch,ij,2).ne.iyoz(n3,ij,2))goto34
35    continue
      iyom(nch)=iyom(nch)+iyom(n3)
      n3=n3-1
      n3m=n3m-1
      if(n3+1.le.n3m)then
      do 36 nm=n3+1,n3m
      iyol(nm)=iyol(nm+1)
      yofa(nm)=yofa(nm+1)
      iyoo(nm)=iyoo(nm+1)
      iyom(nm)=iyom(nm+1)
      do 37 j=1,nsym
      iyoz(nm,j,1)=iyoz(nm+1,j,1)
37    iyoz(nm,j,2)=iyoz(nm+1,j,2)
36    continue
      endif
      goto33
34    continue
33    continue
           endif
      if(n1.lt.n3)goto5000
5003  continue
 
      dialt=-99999.
      if(iyoz(1,1,1).eq.1)then
      dialt=1
      do 40 i=1,nsym
      if(iyoz(1,i,1).gt.1)
     *call utstop('jcentr: iyoz(,,1).gt.1                  ')
      if(iyoz(1,i,1).eq.0)goto40
      do 41 m=1,iyoz(1,i,2)
      hak=iyoz(1,i,2)-m+1
      if(i.lt.nsym)then
      do 42 j=i+1,nsym
      if(iyoz(1,j,2).ge.m)hak=hak+1
42    continue
      endif
      dialt=dialt*(iga+i-m)/hak
41    continue
40    continue
      endif
 
      if(dialt.ne.-99999.
     *.and.abs(dialt-disuta).gt.1e-5*dialt)then
      write(ifch,*)'n=',n,'    k=',k,'    kk=',kk,'    ny=',ny
      write(ifch,*)'disuta=',disuta,'     dialt=',dialt
      call utstop('jcentr: dimensions differ               ')
      endif
 
      if(ish.ge.93)write(ifch,118)disuta
118   format(28x,'subtab.-dimension:',f8.1)
 
      ditab=ditab*disuta
 
6004  continue
 
      if(ish.ge.93)write(ifch,119)ditab
119   format(35x,'tableau-dimension:',f11.1)
 
      degen(1+k,1+n)=degen(1+k,1+n)+ditab
 
6005  continue
 
6002  jj=jj+ll+1
6003  continue
      if(kkk(k).eq.0)
     *call utstop('jcentr: no allowed partition            ')
 
      entro(1+k,1+n)=-9999999.
      if(degen(1+k,1+n).gt.0.)entro(1+k,1+n)=alog(degen(1+k,1+n))
 
      if(ish.ge.93)write(ifch,120)degen(1+k,1+n)
120   format(49x,'degeneracy:',f11.1)
 
6001  continue
 
      if(ish.ge.92)then
121   format(79a1)
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   degeneracy(k,n)         ig=',ig,'          '
     *,'nsym=',nsym
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   k:','   3','   6','   9','   12'
      write(ifch,121)('-',ic=1,79)
      do 52 n=0,nquax
52    write(ifch,*)n,(degen(1+3*k,1+n),k=1,4)
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   entropy(k,n)            ig=',ig,'          '
     *,'nsym=',nsym
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   k:','   3','   6','   9','   12'
      write(ifch,121)('-',ic=1,79)
      do 53 n=0,nquax
53    write(ifch,*)n,(entro(1+3*k,1+n),k=1,4)
      endif
 
      ish0=ish
      if(ishsub/100.eq.12)ish=mod(ishsub,100)
      if(ish.ge.95)then
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   entropy(k,n)            ig=',ig
     *,'          nsym=',nsym
      write(ifch,121)('-',ic=1,79)
      write(ifch,125)kparx
125   format(6x,'if(kparx.ne.',i2,')'/5x,
     *56h*call utstop('jcentd: insufficient initialization;   k'))
      write(ifch,126)nquax
126   format(6x,'if(nquax.ne.',i2,')'/5x,
     *56h*call utstop('jcentd: insufficient initialization;   n'))
      do 58 kx=0,kparx/3
      k=3*kx
      write(ifch,123)k+1,nquax
123   format(6x,'data (entro(',i2,',1+n),n=0,',i2,')/')
      write(ifch,124)(entro(1+k,1+n),deli(1+k,1+n), n=0,nquax)
124   format(10(5x,'*',6(e10.5,a1)/))
58    continue
      endif
      ish=ish0
 
      return
      end

c----------------------------------------------------------------------
      subroutine jcheps(ip,x,y,z,t,sz,eps,rho)
c----------------------------------------------------------------------
c  returns the intersection (x,y,z,t,sz) with tau-line 
c  and calculates energy density eps (gev/fm**3) 
c  and net quark density rho (1/fm**3)
c  for given particle number ip.
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 
      parameter (nflav=6)
      integer jc(nflav,2)

      tiv1ip=tivptl(1,ip)
      tivptl(1,ip)=-ainfin
      call uttain(ip,x,y,z,t,n,0)
      if(n.eq.1.or.n.eq.2.or.n.eq.9)
     *call utstop('jcheps: from uttain: n = 1 2 or 9       ')
      tivptl(1,ip)=tiv1ip
      call uttaus(z,ttest,sz)
      if(abs(t-ttest).gt.1e-5.and.abs(t-ttest).gt.1e-5 *abs(t))then
      call utmsg('jcheps')
      write(ifch,*)'*****  t /= ttest'                       
      write(ifch,*)t,ttest
      call utmsgf
      endif
      desptl(ip)=dezptl(ip)*ttaus          
      oi=sz+desptl(ip)
      ui=sz-desptl(ip)
      vol=(oi-ui)*radptl(ip)**2*pi
      call uttauh(ip,ui,oi,g,dum,wsmax)
      amu=pptl(5,ip)/g
c-c   eps=amu/vol*(oi-ui)*wsmax    !maximum energy density (gauss distr)
      eps=amu/vol                  !average energy density 

      call idquac(ip,nq,ns,na,jc)
      nud=0
      do nf=1,2
      nud=nud+jc(nf,1)-jc(nf,2)
      enddo
      rho=nud/vol

      return
      end

c-----------------------------------------------------------------------
      subroutine jchinv(epsx,rhox,tem,che)
c-----------------------------------------------------------------------
c  (droplet hadronization)
c  inverts ideal gas equations for ud-QGP:
c     epsx=epsx(tem,che)
c     rhox=rhox(tem,che)
c  input:  epsx = energy density (gev/fm**3)
c          rhox = u+d quark density (1/fm**3)
c  output: tem = temperatur (gev)
c          che = u or d chemical potential (gev)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg

      eps=epsx*0.197**3
      rho=rhox*0.197**3

      bag=bag4rt**4
      ups=eps-bag

      tem=0
      che=0

      if(ups.lt.0..or.ups.eq.0..and.rho.ne.0.)then
      call utmsg('jchinv')
      write(ifch,*)'*****  ups.lt.0 .or. ups.eq.0.and.rho.ne.0'
      write(ifch,*)'               -->   ups=0 and rho=0 assumed'
      write(ifch,*)'bag:',bag,'   eps:',eps,'   ups:',ups
      write(ifch,*)'rho:',rho
      call utmsgf
      goto1000
      endif

      if(ups.le.0.)return
 
      if(rho.lt.0.)then
      call utmsg('jchinv')
      write(ifch,*)'*****  rho.lt.0 ; rho changed to 0'
      write(ifch,*)'rho:',rho
      call utmsgf
      rho=0.
      endif

      if(rho.le.0.)then
      tem=(30./37./3.14159**2*ups)**(1./4.)
      goto1000
      endif

      o1=(2./3.*3.14159**2*ups)**(1./4.)
      o2=(1./2.*3.14159**2*rho)**(1./3.)

      if(o1.lt.o2)then
      call utmsg('jchinv')
      write(ifch,*)'*****  o1.lt.o2 > no solution; tem=0,che=0 returned'
      write(ifch,*)'bag:',bag,'   eps:',eps,'   ups:',ups
      write(ifch,*)'rho:',rho
      write(ifch,*)'o1: ',o1,'   o2: ',o2
      call utmsgf
      goto1000
      endif

      if(o1.eq.o2)then
      che=o1
      goto1000
      endif

      u=1e-6
      o=o2
   
      che=u
      y1=15./37./3.14159**2*
     *(-3*che**2+sqrt(8./5.*che**4.+74./15.*3.14159**2*ups))
      y2=(rho-2./3.14159**2*che**3)/2./che
      if(y2.le.y1)then
      che=0
      tem=(30./37./3.14159**2*ups)**(1./4.)
      goto1000
      endif

      ncount=0
1     ncount=ncount+1
      if(ncount.gt.1000)goto1001 
      che=(u+o)/2
      y1=15./37./3.14159**2*
     *(-3*che**2+sqrt(8./5.*che**4.+74./15.*3.14159**2*ups))
      y2=(rho-2./3.14159**2*che**3)/2./che
      if(abs(y1-y2).lt.1e-5)goto2
      if(y1.gt.y2)o=che
      if(y2.gt.y1)u=che
      goto1

2     continue      
      tem=sqrt( (y1+y2)/2 )

1000  continue

c     checks
c     ------
      epsp=37./30.*3.14159**2*tem**4
     * + 3.*che**2*tem**2 + 3./2./3.14159**2*che**4 + bag
      rhop=2.*che*tem**2 + 2./3.14159**2*che**3
      if(abs(epsp-eps).gt.1e-3.or.abs(rhop-rho).gt.1e-3)then
      call utmsg('jchinv')
      write(ifch,*)'*****  eps /= epsp  or  rho /= rhop'
      write(ifch,*)'eps:',eps,'    epsp:',epsp
      write(ifch,*)'rho:',rho,'    rhop:',rhop
      call utmsgf
      endif
      return

1001  continue  
      call utmsg('jchinv')
      write(ifch,*)'*****  no solution found ; tem=0, che=0 returned'
      write(ifch,*)'eps:',eps,'    rho:',rho
      write(ifch,*)'ncount:',ncount
      write(ifch,*)'u:',u,'    o:',o
      write(ifch,*)'y1:',y1,'    y2:',y2
      call utmsgf
      tem=0
      che=0      
      return

      end

c-----------------------------------------------------------------------
      subroutine jchprt
c-----------------------------------------------------------------------
      common/paro5/bag4rt,corlen,dezzer,amuseg
      character text*50
      real xplota(101),yplota(101),xplotb(101),yplotb(101)
     *,xplotc(101),yplotc(101),xplotd(101),yplotd(101)
      bag=bag4rt**4
           do k=1,4
      chex=(k-1)*0.2         
      text='title chemical potential = _____ gev             '             
      write(text(28:32),122)chex
122   format(f5.3)
      do n=0,100
      temx=n/100.*.300
      eps=37./30.*3.14159**2*temx**4
     * + 3.*chex**2*temx**2 + 3./2./3.14159**2*chex**4 + bag
      rho=2.*chex*temx**2 + 2./3.14159**2*chex**3
      epsx=eps/0.197**3
      rhox=rho/0.197**3
      call jchinv(epsx,rhox,tem,che)
      xplota(1+n)=temx
      yplota(1+n)=epsx
      xplotb(1+n)=temx
      yplotb(1+n)=rhox
      xplotc(1+n)=temx
      yplotc(1+n)=tem
      xplotd(1+n)=temx
      yplotd(1+n)=che
      enddo
      call uthist(0.,.300,0.,0.,101,xplota,yplota,'lin','linlin'
     *,'xaxis temperature                                 '
     *,'yaxis energy density                              ',text)
      call uthist(0.,.300,0.,0.,101,xplotb,yplotb,'lin','linlin'
     *,'xaxis temperature                                 '
     *,'yaxis net quark density                          ',text)
      call uthist(0.,.300,0.,0.,101,xplotc,yplotc,'lin','linlin'
     *,'xaxis temperature                                 '
     *,'yaxis temperature                                 ',text)
      call uthist(0.,.300,0.,0.,101,xplotd,yplotd,'lin','linlin'
     *,'xaxis temperature                                 '
     *,'yaxis chemical potential                          ',text)
           enddo
      return
      end

c-----------------------------------------------------------------------
      subroutine jcludc(ip,iret)
c-----------------------------------------------------------------------
c  selects droplet decay method
c-----------------------------------------------------------------------
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      if(ioclud.eq.1)call jclude(ip,iret)
      if(ioclud.eq.2)call qnbdec(ip,iret)
      return
      end

c-----------------------------------------------------------------------
      subroutine jclude(ip,iret)
c-----------------------------------------------------------------------
c  decays cluster ip from /cptl/ .
c  reqires jcentr or jcentd to be called before.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (nflav=6)
      integer ica(2)
     *,jca(nflav,2),jce(nflav,2),jco(nflav,2),jca0(nflav,2)
      common/csjcga/amnull,nsuha(7),asuha(7),entrpy,amegam,nopha
      common/centex/entexp
      common/cscal/scal
      common/ctimel/ntc
      common/cutinv/lutinv
  
      parameter (moxmax=30,iomaxm=25,iomax=54)
      parameter (momax=moxmax*iomax)
      integer idol(iomax),idolis(iomax),ifolis(iomax)
      real pol(iomax)
      real xo(1+momax),qo(1+momax),yo(1+momax),oo(1+momax),go(1+momax)
      real pa(5),pa0(5),pe(5),u(3)
      integer ko(1+momax)

      data idolis/
     * 110,120,-120,220,130,-130,230,-230,330
     *,1120,-1120,1220,-1220,1130,-1130,1230,-1230,2130,-2130
     *,2230,-2230,1330,-1330,2330,-2330
     *,1111,-1111,2221,-2221,3331,-3331
     *,111,121,-121,221,131,-131,231,-231,331
     *,1121,-1121,1221,-1221,1131,-1131,1231,-1231
     *,2231,-2231,1331,-1331,2331,-2331/
      data ifolis/9*1,16*2,6*4,9*3,14*4/

      ish0=ish

      if(ish.ge.91)
     *write(ifch,'(1x,a,i10,a,f10.4)')'sequential decay; id:'
     *,idptl(ip),'   mass:',pptl(5,ip)

      delpox=.01
      nptlb=nptl
      loop=0
      iret=0
      ipoxra=0
      istpfl=0
      ebamin=1.5 
      loopmx=20
      nophax=20

c     original cluster --> pa,jca
c     ---------------------------

6010  continue
      do 1 j=1,5
1     pa(j)=pptl(j,ip)
      call idquac(ip,ndu,ndu,ndu,jca)
      if(ish.ge.95)then
      write(ifch,*)'initial jca:'
      write(ifch,*)jca
      endif
      call idcomj(jca)
      if(ish.ge.95)then
      write(ifch,*)'initial jca after compactification:'
      write(ifch,*)jca
      endif
 
      do 18 j=1,5
18    pa0(j)=pa(j)
      nqn=0
      do 2 nf=1,nflav
      nqn=nqn+jca(nf,1)-jca(nf,2)
      do 2 j=1,2
2     jca0(nf,j)=jca(nf,j)
      nba=iabs(nqn)/3
      eba=ainfin
      if(nba.gt.0)eba=pa(5)/nba
      ish00=ish

c     initialization for redo
c     -----------------------

6001  loop=loop+1
           if(ish00.eq.90)then
      if(loop.eq.loopmx)then
      ish=92
      write(ifch,117)('-',l=1,75),loop,ish
117   format(/1x,75a1/1x,i2,'. attempt to decay this cluster!'
     */1x,'ish set to: ',i2)
      endif 
           endif
      if(loop.gt.loopmx)then
      ish=ish00
      goto1001
      endif
      nopha=0
      ismall=0
      nptl=nptlb
      do 20 j=1,5
20    pa(j)=pa0(j)
      do 3 nf=1,nflav
      do 3 j=1,2
3     jca(nf,j)=jca0(nf,j)
 
c     loop over sequential decays
c     ---------------------------

6002  nptl=nptl+1
      if(nptl.gt.mxptl)
     *call utstop('jclude: nptl>mxptl                      ')
      isjca=0
      do nf=1,nflav
      do j=1,2
      isjca=isjca+iabs(jca(nf,j))
      enddo
      enddo
      if(isjca.eq.0)then
      jca(1,1)=1
      jca(1,2)=1
      endif
      call idcomj(jca)
      call idenco(jca,ica,ireten)
           if(ish.ge.92)then
      write(ifch,100)(pa(k),k=1,5)
100   format(1x,'pa:',22x,5(e10.3,1x))
      write(ifch,116)jca
116   format(1x,'jca: ',6i5,3x,6i5)
           endif

      kau=jca(1,1)-jca(1,2)
      kad=jca(2,1)-jca(2,2)
      kas=jca(3,1)-jca(3,2)
      kac=jca(4,1)-jca(4,2)
      amicl=utamnu(kau,kad,kas,kac,5)

           if(nopha.eq.nophax.or.nopha.gt.1.and.pa(5).gt.2*amicl)then
      if(ish.ge.90)then
      call utmsg('jclude')
      if(ismall.eq.0)write(ifch,*)'*****  no phase space --> redo decay'
      if(ismall.eq.1)write(ifch,*)'*****  small phase sp --> redo decay'
      if(ismall.ne.0.and.ismall.ne.1)
     *write(ifch,*)'***** small/no phase space --> redo decay'
      if(ismall.ne.0.and.ismall.ne.1)write(ifch,*)'ismall=',ismall
      write(ifch,*)'nt=',ntc,'   loop=',loop,'   nopha=',nopha
      if(ismall.eq.1)
     *write(ifch,*)'yo_hit=',yohit,'   yo_max=',yomax
      write(ifch,*)'m_min=',amicl,'   m=',pa(5)
      write(ifch,100)(pa(k),k=1,5)
      write(ifch,116)jca
      call utmsgf
      endif
      goto6001
           endif

           if(nopha.gt.1)then
      if(ish.ge.90)then
      call utmsg('jclude')
      if(ismall.eq.0)write(ifch,*)'*****  no phase space --> incrs mass'
      if(ismall.eq.1)write(ifch,*)'*****  small phase sp --> incrs mass'
      if(ismall.ne.0.and.ismall.ne.1)
     *write(ifch,*)'***** small/no phase space --> incrs mass'
      if(ismall.ne.0.and.ismall.ne.1)write(ifch,*)'ismall=',ismall
      write(ifch,*)'nt=',ntc,'   loop=',loop,'   nopha=',nopha
      if(ismall.eq.1)
     *write(ifch,*)'yo_hit=',yohit,'   yo_max=',yomax
      write(ifch,*)'m_min=',amicl,'   m=',pa(5)
      write(ifch,100)(pa(k),k=1,5)
      write(ifch,116)jca
      endif
      pa(5)=pa(5)*1.05
      pa(4)=sqrt(pa(1)**2+pa(2)**2+pa(3)**2+pa(5)**2)
      if(ish.ge.90)then
      write(ifch,100)(pa(k),k=1,5)
      call utmsgf
      endif
           endif

      idar=0
           if(nopha.eq.0)then
      ich=0
      ida=0
      if(ireten.eq.0)ida=idtra(ica,0,0,3)
      ama=pa(5)
      call idres(ida,ama,idar,iadj)
      if(idar.ne.0.and.eba.lt.ebamin)then
      ama=0
      call idres(ida,ama,idar,iadj)
      if(mod(idar,10).ne.0.and.loop.lt.loopmx/2)goto6001 
      endif
      if(idar.ne.idptl(ip))ich=1
           endif

           if(idar.ne.0)then
      if(nptl.gt.nptlb+1.or.ich.eq.1)then
      idptl(nptl)=idar
      pa(5)=ama
      pa(4)=sqrt(ama**2+pa(1)**2+pa(2)**2+pa(3)**2)
      do 15 j=1,5
15    pptl(j,nptl)=pa(j)
      if(ish.ge.92)
     *write(ifch,101)nptl,idptl(nptl),(pptl(k,nptl),k=1,5)
101   format(1x,'/cptl/',i6,i11,2x,5(e10.3,1x))
      else
      nptl=nptl-1
      endif
      goto7001
           endif

      ama=pa(5)
 
c     loop over hadrons
c     -----------------

      mo=1
      po=0
      xo(1)=0
      qo(1)=0
      iom=0
      qol=0
      if(ish.ge.95)write(ifch,106)
106   format(' partial decay widths:')

      do 6003 io=1,iomax
      if(eba.lt.ebamin.and.io.gt.iomaxm)goto6003

      ido=idolis(io)
      iom=iom+1
      pol(iom)=po
      idol(iom)=ido
      call idmass(ido,amo)
      idptl(nptl)=ido
      call idquac(nptl,ndu,ndu,ndu,jco)
      do 12 nf=1,nflav
      do 12 j=1,2
12    jce(nf,j)=jca(nf,j)-jco(nf,j)
      do 22 nf=1,nflav
      do 22 j=1,2
      if(jce(nf,j).lt.0)then
      jce(nf,3-j)=jce(nf,3-j)-jce(nf,j)
      jce(nf,j)=0
      endif
22    continue
      ke=0
      do 13 i=5,nflav
      if(jce(i,1).ne.0.or.jce(i,2).ne.0.or.istpfl.gt.0)then
      if(istpfl.eq.2)
     *call utstop('jclude: more than 4 flavours            ')
      istpfl=istpfl+1
      ish=93
      if(istpfl.eq.2)goto6010
      goto6001
      endif
13    continue
      keu=jce(1,1)-jce(1,2)
      ked=jce(2,1)-jce(2,2)
      kes=jce(3,1)-jce(3,2)
      kec=jce(4,1)-jce(4,2)
 
c     loop over hadron momenta
c     ------------------------

      if(mo+moxmax.gt.1+momax)
     *call utstop('jclude: dimension momax too small       ')
      pox=0.
      mom=mo
      pom=po
      qom=qo(mo)
      gamxm=0.
      poxm=0.
      gamy=0.
      gamx=0.
      delpo=delpox
      yoint=0

      do 6004 mox=1,moxmax

      delpo=delpo*1.2
      mo=mo+1
      po=po+delpo
      ko(mo)=ido
      if(mox.gt.1)ko(mo)=0
      xo(mo)=po
      pox=pox+delpo
      gamx=sjcgam(keu,ked,kes,kec,ama,amo,pox,mox)
      go(mo)=gamx
      yo(mo)=ifolis(io)*delpo*gamx
      oo(mo)=entexp 
      yoint=yoint+ifolis(io)*delpo*(gamx+gamy)/2.         
      i6005=0
      if(gamx.eq.0.)i6005=1
      if(mox.gt.1.and.gamxm.gt.0..and.gamx.lt.1e-2*gamxm)i6005=1
           if((i6005.eq.0.or.mox.gt.1).and.ish.ge.96)then
      if(mox.eq.1)
     *write(ifch,109)keu,ked,kes,nsuha,ama,asuha,amnull,ido,amo
      write(ifch,105)mo,xo(mo),pox,sqrt(pox**2+amo**2)
     *,ama-sqrt(pox**2+amo**2),entrpy,gamx
     *,yo(mo),oo(mo)
109   format(' u_d_s:',3i3,'  n:',7i5,11x,'  a:',f10.2
     */18x,'m:',7f5.2,'  m0:',f6.2,'  o:',i5,f5.2
     */'   mo     xo   pox      eo ama-eo     entrpy     '
     *,'  gamx         yo         oo ')
105   format(i5,1x,f6.2,f6.2,2x,f6.2,f6.2
     *,3x,e9.3,2x,e9.3,2x,e9.3,2x,e9.3)
           endif
      if(i6005.eq.1)goto6005
      if(gamx.gt.gamxm)then
      gamxm=gamx
      poxm=pox
      endif
      gamy=gamx

6004  continue

      if(ish.ge.90.and.ipoxra.eq.0)then
      ipoxra=1
      call utmsg('jclude')
      write(ifch,*)'*****  pox-range too small'
      write(ifch,100)(pa(k),k=1,5)
      write(ifch,*)'jca:'
      write(ifch,*)jca
      write(ifch,*)'pox= ',pox, '   gamx= ',gamx
      write(ifch,*)'poxm=',poxm,'   gamxm=',gamxm
      call utmsgf
      endif

6005  continue
 
      if(yoint.eq.0.)then
      if(ish.ge.96)write(ifch,*)'no phase space for',ido
      mo=mom
      po=pom
      iom=iom-1
      else
      if(ish.ge.95)write(ifch,107)ido,poxm,yoint,oo(mo)
107   format(' ido,poxm,yoint,oo:',i6,3x,f5.2,3x,e9.3,3x,e9.3)
      endif
 
6003  continue

c     no phase space
c     --------------
      if(iom.eq.0)then
      if(ish.ge.95)write(ifch,*)'no phase space'
      nptl=nptl-1
      nopha=nopha+1
      goto6002
      endif

c     determine qo(m)
c     ---------------
      oomax=0
      do m=1,mo
      if(oo(m).gt.oomax)oomax=oo(m)
      enddo
      oox=oomax-10
      do m=1,mo
      if(oo(m).lt.oox)then
      yo(m)=0
      else
      yo(m)=yo(m)*exp(oo(m)-oox)
      endif
      enddo       
      qo(1)=0        
      do m=2,mo
      if(ko(m).ne.0)yom=0
      qo(m)=qo(m-1)+(yom+yo(m))/2.
      yom=yo(m)
      enddo
      yomax=0
      do m=1,mo
      if(yo(m).gt.yomax)yomax=yo(m)
      enddo

      if(ish.ge.96)then
      ido=0
      xox=0
      write(ifch,*)'accumulated decay probabilities:'
      do m=1,mo
      if(ko(m).ne.0)ido=ko(m)
      write(ifch,110)m,ido,xo(m)-xox,qo(m)
110   format(1x,'m,ido,xo,qo: ',i6,i6,3x,f6.2,3x,e9.3)  
      if(m.lt.mo.and.ko(m+1).ne.0)xox=xo(m)
      enddo
      endif

c     select randomly hadron+momentum
c     -------------------------------

      lo=0
24    lo=lo+1
           if(lo.gt.10)then
      if(ish.ge.95)write(ifch,*)'small phase space:',yohit,yomax
      nptl=nptl-1
      nopha=nopha+1
      ismall=1
      goto6002
           endif

      xos=utinvt(mo,xo,qo,rangen()*qo(mo))
      mhit=lutinv
      do 28 i=2,iom
      ix=i-1
      if(xos.lt.pol(i))goto29
28    continue
      ix=iom
29    continue
      pos=xos-pol(ix)
      ids=idol(ix)
      call idmass(ids,ams)
      idptl(nptl)=ids

      call idquac(nptl,ndu,ndu,ndu,jco)
      do 25 nf=1,nflav
      do 25 j=1,2
25    jce(nf,j)=jca(nf,j)-jco(nf,j)
      do 26 nf=1,nflav
      do 26 j=1,2
      if(jce(nf,j).lt.0)then
      jce(nf,3-j)=jce(nf,3-j)-jce(nf,j)
      jce(nf,j)=0
      endif
26    continue
      ke=0
      do 27 i=5,nflav
      if(jce(i,1).ne.0.or.jce(i,2).ne.0)
     *call utstop('jclude: flavour > 4                     ')
27    continue

      keu=jce(1,1)-jce(1,2)
      ked=jce(2,1)-jce(2,2)
      kes=jce(3,1)-jce(3,2)
      kec=jce(4,1)-jce(4,2)
      gamxht=sjcgam(keu,ked,kes,kec,ama,ams,pos,1)
      yohit=gamxht*(xo(mhit+1)-xo(mhit))*4
      if(yohit.lt.1e-5.and.yohit.lt.1e-5*yomax)goto24
 
      pptl(5,nptl)=ams
      u(3)=2.*rangen()-1.
      phi=2.*pi*rangen()
      u(1)=sqrt(1.-u(3)**2)*cos(phi)
      u(2)=sqrt(1.-u(3)**2)*sin(phi)
      do 5 j=1,3
5     pptl(j,nptl)=pos*u(j)
      pptl(4,nptl)=sqrt(pptl(5,nptl)**2+pos**2)
      if(ish.ge.96)
     *write(ifch,101)nptl,idptl(nptl),(pptl(k,nptl),k=1,5)
      call utlobo(-1,pa(1),pa(2),pa(3),pa(4),pa(5)
     *,pptl(1,nptl),pptl(2,nptl),pptl(3,nptl),pptl(4,nptl))
      if(ish.ge.92)
     *write(ifch,101)nptl,idptl(nptl),(pptl(k,nptl),k=1,5)
 
      pe(5)=amegam
      do 16 j=1,3
16    pe(j)=-pos*u(j)
      pe(4)=sqrt(pe(5)**2+pos**2)
      call utlobo(-1,pa(1),pa(2),pa(3),pa(4),pa(5)
     *,pe(1),pe(2),pe(3),pe(4))
 
      do 17 j=1,5
17    pa(j)=pe(j)
      do 23 nf=1,nflav
      do 23 j=1,2
23    jca(nf,j)=jce(nf,j)
      nopha=0
 
      goto6002
 
7001  continue
 
c     check energy conservation
c     -------------------------
      if(nptl.le.nptlb)goto1000

      ifail=1
           if(nptl.gt.nptlb+1)then                          
      ishrsc=ish
      ish=0
      call jrescl(nptlb+1,nptl,pa0,ifail)
      ish=ishrsc
c-c   if(ifail.ne.0.and.ish.ge.90)then
c-c   call utmsg('jclude')
c-c   write(ifch,*)'*****  ifail_jrescl=',ifail
c-c   call utmsgf
c-c   endif
           endif

c-c   do n=nptlb+1,nptl
c-c   p=pptl(3,n)
c-c   e=pptl(4,n)
c-c   y=100
c-c   dy=3
c-c   if(e-p.ne.0..and.e+p.ne.0.)y=.5*alog((e+p)/(e-p))+dy
c-c   ida=iabs(idptl(n))
c-c   if(ida.gt.1000.and.mod(ida,10).ne.0.and.y.lt.10.)then
c-c   write(ifmt,*)('-',k=1,69)
c-c   p=pptl(3,ip)
c-c   e=pptl(4,ip)
c-c   y=100
c-c   dy=3
c-c   if(e-p.ne.0..and.e+p.ne.0.)y=.5*alog((e+p)/(e-p))+dy
c-c   write(ifmt,115)ip,idptl(ip)
c-c  *,(pptl(k,ip),k=3,5),y
c-c   write(ifmt,*)'-------> ' 
c-c   do m=nptlb+1,nptl
c-c   p=pptl(3,m)
c-c   e=pptl(4,m)
c-c   y=100
c-c   dy=3
c-c   if(e-p.ne.0..and.e+p.ne.0.)y=.5*alog((e+p)/(e-p))+dy
c-c   write(ifmt,115)m,idptl(m)
c-c  *,(pptl(k,m),k=3,5),y
c-c   enddo
c-c   endif
c-c   enddo
 115  format(1x,'/cptl/',i4,i10
     *,3x,3(e8.2,1x),2x,2(e8.2,1x))

      if(ish.ge. 0)then
      px=0
      py=0
      pz=0
      e=0
      do 10 n=nptlb+1,nptl
      px=px+pptl(1,n)
      py=py+pptl(2,n)
      pz=pz+pptl(3,n)
10    e=e+  pptl(4,n)
      endif
 
      if(ish.ge.96)then
      write(ifch,*)'check energy conservation'
      do 11 n=nptlb+1,nptl
11    write(ifch,101)n,idptl(n),(pptl(k,n),k=1,5)
      am=sqrt(e**2-px**2-py**2-pz**2)
      write(ifch,103)px,py,pz,e,am
      write(ifch,104)(pa0(k),k=1,5)
103   format(1x,'p_sum:  ',15x,5(e10.3,1x))
104   format(1x,'p_clu:  ',15x,5(e10.3,1x))
      endif
 
      if(ish.ge.90.and.(
     *ifail.eq.0.and. 
     *   (abs(px-pa0(1)).gt.1e-2*abs(px).and.abs(px-pa0(1)).gt.1e-2
     *.or.abs(py-pa0(2)).gt.1e-2*abs(py).and.abs(py-pa0(2)).gt.1e-2
     *.or.abs(pz-pa0(3)).gt.1e-2*abs(pz).and.abs(pz-pa0(3)).gt.1e-2
     *.or.abs(e -pa0(4)).gt.1e-2*abs(e ).and.abs(e -pa0(4)).gt.1e-2)
     *.or.
     *ifail.ne.0.and.
     *   (abs(px-pa0(1)).gt.1e-2*abs(px).and.abs(px-pa0(1)).gt.1e-2
     *.or.abs(py-pa0(2)).gt.1e-2*abs(py).and.abs(py-pa0(2)).gt.1e-2
     *.or.abs(pz-pa0(3)).gt.1e-2*abs(pz).and.abs(pz-pa0(3)).gt.1e-2
     *.or.abs(e -pa0(4)).gt.35e-1*abs(e ).and.abs(e -pa0(4)).gt.35e-1)
     *   ))then
      call utmsg('jclude')
      write(ifch,*)'*****  p_sum /= p_clu'
      write(ifch,*)'ifail_jrescl:',ifail,'   scal:',scal
      do 30 n=nptlb+1,nptl
30    write(ifch,101)n,idptl(n),(pptl(k,n),k=1,5)
      write(ifch,103)px,py,pz,e
      write(ifch,104)(pa0(k),k=1,4)
      call utmsgf
      endif
  
1000  continue
      ish=ish0
      return
 
1001  iret=1
      if(ish.ge.90)then
      call utmsg('jclude')
      write(ifch,*)'*****  cluster decay not possible  --> iret=1'
      write(ifch,100)(pa0(k),k=1,5)
      write(ifch,*)'jca:'
      write(ifch,*)jca0
      call utmsgf
      endif
      return
 
      end

c-----------------------------------------------------------------------
      subroutine jdecas(i,iret)
c-----------------------------------------------------------------------
c  if no qm droplet: calls jdecax (no qm cl: abs(dezptl(i)) .lt. 1.0e-4)
c  if small qm droplet: calls jdecax (small qm cl.: nr of breaks = 1)
c  if big qm droplet: splits i into k subdroplets (big=.not.small)  
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter (myptl=1000,nzeta=60)
      common/c4ptl/iyptlx,iyptl(mxptl),weiptl(myptl,nzeta),iyiptl(myptl)
      parameter (nflav=6)
      common/c5ptl/iflptl(myptl,nzeta,nflav)
      integer iprb(nflav)
      real weisum(nzeta)
      parameter (nsuclu=1000)
      integer jc(nflav,2),jcs(nsuclu,nflav,2),ifli(nzeta,nflav)
     *,jcx(nflav,2),jcy(nflav,2)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 
      double precision ttaux,ttauz,zor,tor
      real psum(5),psumx(5)
      common/ciutot/iutotc,iutote
      character c2*2

      iret=0
      if(istptl(i).ne.0)goto1001

      if(ish.ge.94)write(ifch,*)('-',l=1,10)
     *,' entry sr jdecas ',('-',l=1,30)

         if(ioclud.eq.1)then
      amusef=1.7
      if(pptl(5,i).gt.2*amuseg)amusef=amuseg
      if(iorptl(i).gt.0)then
      if(i.gt.1.and.iorptl(i-1).eq.iorptl(i).or.
     *   (i.lt.nptl .and. iorptl(i+1).eq.iorptl(i)) )amusef=amuseg
      endif
         else
      amusef=amuseg
         endif

      amux=2.5*amusef

      ttauz=ttaus
      nptlb=nptl

      ish0=ish
      if(ishsub/100.eq.20)ish=mod(ishsub,100)
      if(ishsub/100.eq.21)ish=mod(ishsub,100)

c     no quark matter droplet (droplet from fusion)
c     ---------------------------------------------
      if(abs(dezptl(i)).lt. dezzer )goto5001

c     determine k (nr of breaks) 
c     --------------------------
      iy=0
      if(iyptl(i).ne.0)iy=iyptl(i)
           if(iy.ne.0)then
      if(iy.le.0)
     *call utstop('jdecas: iy .le. 0                       ')
      nh=0
      do 8 n=1,nzeta
      if(weiptl(iy,n).le.0.)goto8
      nh=nh+1
8     continue
           if(nh.eq.0)then
      if(ish.ge.0)then
      call utmsg('jdecas')
      write(ifch,*)'*****  empty weiptl(iy,)  -->  iy set 0' 
      write(ifch,*)'i dez iy:',i,dezptl(i),iy
      write(ifch,*)'id mass:',idptl(i),pptl(5,i)
      write(ifch,*)'wei:'
      write(ifch,*)(weiptl(iy,n),n=1,nzeta)
      call utmsgf
      endif
      iy=0
      iyptl(i)=0
           endif
           endif
      t=tivptl(2,i)
      x=xorptl(1,i)+(t-xorptl(4,i))*pptl(1,i)/pptl(4,i)
      y=xorptl(2,i)+(t-xorptl(4,i))*pptl(2,i)/pptl(4,i)
      z=xorptl(3,i)+(t-xorptl(4,i))*pptl(3,i)/pptl(4,i)
      call uttaux(t,z,ttaux)
      ttaus=ttaux
      call uttaus(z,ttest,sz)
           if(abs(t-ttest).gt.1e-5)then
      call utmsg('jdecas')
      write(ifch,*)'*****  t /= ttest'                       
      write(ifch,*)t,ttest
      call utmsgf
           endif
      dez=dezptl(i)
      desptl(i)=dez*ttaux
      des=desptl(i)
      utot=sz-des
      otot=sz+des
c adjust utot and otot
           if(iy.ne.0)then
      nh=0
      do n=1,nzeta
      if(weiptl(iy,n).gt.1e-5)then
      nh=nh+1
      if(nh.eq.1)n1=n
      n2=n
      endif
      enddo
      sn1=(-nzeta/2+n1-0.001)*dlzeta*ttaus 
      sn2=(-nzeta/2+n2-0.999)*dlzeta*ttaus 
      iutotc=iutotc+1
      if(utot.gt.sn1)then
      iutote=iutote+1
      rerr=iutote*1./iutotc
      utotod=utot
      utot=sn1   
      shft=utotod-utot
      rshft=shft
      if(utot.ne.0)rshft=shft/abs(utot)
      if(ish.ge.90.and.shft.gt.0.10.and.rshft.gt.0.10
     *.and.rerr.gt.0.03)then
      call utmsg('jdecas')
      write(ifch,*)'*****  utot reduced (1)'
      write(ifch,*)'utot utot_old:',utot,utotod
      write(ifch,*)'n1 taus:',n1,sngl(ttaus) 
      write(ifch,*)'errors counts:',iutote,iutotc          
      call utmsgf
      endif
      endif
      iutotc=iutotc+1
      if(otot.lt.sn2)then
      iutote=iutote+1
      rerr=iutote*1./iutotc
      ototod=otot
      otot=sn2       
      shft=ototod-otot
      rshft=shft
      if(otot.ne.0)rshft=shft/abs(otot)
      if(ish.ge.90.and.shft.gt.0.10.and.rshft.gt.0.10
     *.and.rerr.gt.0.03)then
      call utmsg('jdecas')
      write(ifch,*)'*****  otot increased (1)'
      write(ifch,*)'otot otot_old:',otot,ototod
      write(ifch,*)'n2 taus:',n2,sngl(ttaus) 
      write(ifch,*)'errors counts:',iutote,iutotc          
      call utmsgf
      endif
      endif
           endif
           if(utot.ge.otot.and.ish.ge.0)then
      call utmsg('jdecas')
      write(ifch,*)'*****  utot .ge. otot'
      write(ifch,*)'utot otot:',utot,otot
      write(ifch,*)'i des dez:',i,desptl(i),dezptl(i)
      call utmsgf
           endif
      call uttaug(utot,otot,gtotx,rapx)
      call uttauh(i,utot,otot,gtot,raptot,wsmax)
      if(ish.ge.95)write(ifch,*)'jdecas> gg yy:',gtotx,gtot,rapx,raptot  
      if(ish.ge.95)write(ifch,*)'jdecas> 1/(o-u) wsmax:',1/(otot-utot)
     *,wsmax 
      amu=pptl(5,i)/gtot
      vol=2*des*pi*radptl(i)**2
      eps=amu/vol*(otot-utot)*wsmax
      k1=2*des/corlen
      k2=amu/amusef
c-c   k=min0(k1,k2)
      k=nint(amu/amusef)
c-c   if(abs(idptl(i)).ge.10000)
c-c  *write(ifmt,*)i,iorptl(i),idptl(i),pptl(5,i),k 
           if(iy.ne.0.and.k.gt.1)then
      k3=myptl-iyptlx
      if(k3.lt.2)then
      write(ifch,100)sngl(ttaus),amu,vol,eps,k
      write(ifch,*)'myptl - iyptlx < 2    '    
     *,'myptl iyptlx:',myptl,iyptlx
      call utstop('jdecas: myptl too small (1)             ')
      endif
      if(k3.lt.k)then
      call utmsg('jdecas')
      write(ifch,*)'***** myptl too small. try with reduced k...' 
      write(ifch,100)sngl(ttaus),amu,vol,eps,k
      write(ifch,*)'myptl iyptlx:',myptl,iyptlx
      call utmsgf
      amusef=amux/0.74999
      k2=nint(amu/amusef)
      k=min0(k2,k3)
      if(k.lt.2)
     *call utstop('jdecas: myptl too small (2)             ')
      endif
           endif
2     continue
      if(k.le.1)goto5000
      dz=dez/k
      ds=des/k
      amuk=amu/k
           if(amuk.lt.amusef*0.74999)then
      call utmsg('jdecas')
      write(ifch,*)'*****  amuk < amusef*0.74999'                     
      write(ifch,*)amuk,amusef*0.74999
      call utmsgf
           endif

c     droplet breaking for k > 1
c     --------------------------

      if(k.gt.nsuclu)
     *call utstop('jdecas: nsuclu too small                ')
      call idquac(i,ndu1,ndu2,ndu3,jc)
           do n=1,nflav
      l=min0(jc(n,1),jc(n,2))
      jc(n,1)=jc(n,1)-l
      jc(n,2)=jc(n,2)-l
      if(jc(n,1).lt.0.or.jc(n,2).lt.0)
     *call utstop('jdecas: jc negative                     ')
           enddo
      if(ish.ge.91)write(ifch,100)sngl(ttaus),amu,vol,eps,k
100   format(1x,'>>> breaking of fusion cl; taus:',f6.2
     *,' mu vol eps:',f7.2,f8.2,f6.2,' k:',i3)
           if(ish.ge.92)then
      if(idptl(i)/10**8.ne.7)write(ifch,121)idptl(i),jc
121   format(1x,'id:',i10/1x,'jc:',6i4,8x,6i4)
      if(idptl(i)/10**8.eq.7)
     *write(ifch,122)idptl(i),(ibptl(l,i),l=1,4),jc
122   format(1x,'id:',i10,5x,'ib:',4i8/1x,'jc:',6i4,8x,6i4)
      if(iy.ne.0)then
      write(ifch,*)'long droplet; local flavour consrv; iy:',iy
     *,' dez:',dezptl(i)
      else
      write(ifch,*)'short droplet; no local flv consrv; iy:',iy
     *,' dez:',dezptl(i)
      endif
           endif

c     split jc
c     --------
      do n=1,nflav
      do j=1,2
      jcx(n,j)=jc(n,j)
      enddo
      enddo
      do kk=1,k
      do n=1,nflav
      do j=1,2
      jcs(kk,n,j)=0
      enddo
      enddo
      enddo
      do n=1,nflav
      do j=1,2
      if(jc(n,j).gt.0)then
      do l=1,jc(n,j)
      kk=1+rangen()*k
      kk=min(kk,k)
      jcs(kk,n,j)=jcs(kk,n,j)+1
      enddo
      endif
      enddo
      enddo
c check
      if(iy.eq.0)then
      if(ish.ge.92)then
      do kk=1,k
      write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
      enddo
103   format(1x,'jc:',6i4,5x,6i4,5x,'k:',i3)
      endif
      endif
      do n=1,nflav
      do j=1,2
      jtest=jc(n,j)
      do kk=1,k
      jtest=jtest-jcs(kk,n,j)
      enddo
      if(jtest.ne.0)then
      write(ifch,*)'jc:'
      write(ifch,103)jc
      write(ifch,*)'jcs:'
      do kk=1,k
      write(ifch,103)((jcs(kk,nx,jx),nx=1,nflav),jx=1,2),kk
      enddo
      call utstop('jdecas: jtest /= 0    (1)               ')
      endif
      enddo
      enddo
c integer baryon number for subdroplets
      do kk=1,k-1
      jq=0
      do n=1,nflav
      jq=jq+jcs(kk,n,1)-jcs(kk,n,2)
      enddo
      if(mod(jq,3).ne.0)then
      if(mod(jq+1,3).eq.0)then
      n=int(rangen()/pud)+1
      jcs(kk,n,1)=jcs(kk,n,1)+1
      jcs(kk+1,n,2)=jcs(kk+1,n,2)+1
      elseif(mod(jq-1,3).eq.0)then
      n=int(rangen()/pud)+1
      jcs(kk,n,2)=jcs(kk,n,2)+1
      jcs(kk+1,n,1)=jcs(kk+1,n,1)+1
      else
      call utstop('jdecas: error                           ')
      endif
      endif
      enddo
c nonvanishing id for subdroplets
      do kk=1,k
      ja=0
      do n=1,nflav
      do j=1,2
      ja=ja+jcs(kk,n,j)
      enddo
      enddo
      if(ja.eq.0)then
      n=int(rangen()/0.5)+1
      jcs(kk,n,1)=1
      jcs(kk,n,2)=1
      endif
      enddo
c check minimum mass
      do kk=1,k
      ku=jcs(kk,1,1)-jcs(kk,1,2)
      kd=jcs(kk,2,1)-jcs(kk,2,2)
      ks=jcs(kk,3,1)-jcs(kk,3,2)
      kc=jcs(kk,4,1)-jcs(kk,4,2)
      amini=utamnu(ku,kd,ks,kc,5)
      if(amuk.lt.amini)then 
      if(ish.ge.91)then
      write(ifch,*)'amuk<amini ---> k=k-1'
      if(ish.ge.92)then
      write(ifch,*)'amuk amini:',amuk,amini
      write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
      endif
      endif
      k=k-1
      goto2
      endif
      enddo
c check
      if(iy.eq.0)then
      if(ish.ge.92)then
      do kk=1,k
      write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
      enddo
      endif
      endif
      do n=1,nflav
      jtest=jc(n,1)-jc(n,2)
      do kk=1,k
      jtest=jtest-jcs(kk,n,1)+jcs(kk,n,2)
      enddo
      if(jtest.ne.0)
     *call utstop('jdecas: jtest /= 0  (2)                 ')
      enddo
      do kk=1,k
      jq=0
      do n=1,nflav
      jq=jq+jcs(kk,n,1)-jcs(kk,n,2)
      enddo
      if(mod(jq,3).ne.0)
     *call utstop('jdecas: noninteger baryon number        ')
      enddo

c     test for iy /= 0
c     ----------------
           if(iy.ne.0)then
      if(iy.le.0)
     *call utstop('jdecas: iy .le. 0                       ')
      do nf=1,nflav
      itst=0
      do n=1,nzeta
      itst=itst+iflptl(iy,n,nf)
      enddo
      if(itst.ne.jcx(nf,1)-jcx(nf,2))then
      write(ifch,108)i,idptl(i),jcx
108   format(1x,'nptl:',i10/1x,'id:',i10/1x,'jc:',3x,6i4,8x,6i4)
      write(ifch,*)'nz iflptl(,nz,1-6):'
      do nz=1,nzeta
      write(ifch,106)nz,weiptl(iy,nz),(iflptl(iy,nz,nfx),nfx=1,6)
106   format(1x,i6,f6.2,1x,6i2)
      enddo
      call utstop('jdecas: sum iflptl /= jc(,1)-jc(,2)     ')
      endif                                                  
      enddo
           endif

c     print
c     -----
      zero=0
           if(iy.ne.0)then
      if(ish.ge.93)then
      write(ifch,104)(n,n=nzeta/2-5,nzeta/2+6)
104   format(1x,'nr: ',12i5)
      write(ifch,101)i,iy
     *,(weiptl(iy,n),n=nzeta/2-5,nzeta/2+6)
     *,((iflptl(iy,n,nf),n=nzeta/2-5,nzeta/2+6),nf=1,3)
101   format(1x,'nptl:',i4,'   iyptl:',i4
     */1x,'wei:',12f5.1/1x,'u:  ',12i5
     */1x,'d:  ',12i5/1x,'s:  ',12i5)
      write(ifch,115)i,idptl(i)
     *,zero,zero,zero,pptl(5,i),pptl(5,i)
      write(ifch,116)('  ------',l=1,5)
116   format(1x,'      ',6x,10x,3x,5(a8,1x))
      endif
           endif

c     construct k sub-droplets: initialization
c     ----------------------------------------
      iyptly=iyptlx
      do l=1,4
      psumx(l)=0
      psum(l)=0
      enddo
           if(iy.ne.0)then
      w=0
      nh=0
      do 7 n=1,nzeta
      if(weiptl(iy,n).le.0.)goto7
      nh=nh+1
      if(nh.eq.1)n1=n
      n2=n
      w=w+weiptl(iy,n)
7     continue
      if(nh.eq.0)
     *call utstop('jdecas: empty weiptl(iy,)               ')
      if(ish.ge.96)write(ifch,*)'n1 n2 w amuk:',n1,n2,w,amuk
      if(ish.ge.96)write(ifch,*)'utot otot:',utot,otot
      zutot=utot/ttaus
      nutot=zutot/dlzeta+nzeta/2+1
      iutotc=iutotc+1
      if(nutot.gt.n1)then
      iutote=iutote+1
      rerr=iutote*1./iutotc
      utotod=utot
      utot=(-nzeta/2+n1-0.001)*dlzeta*ttaus 
      shft=utotod-utot
      rshft=shft
      if(utot.ne.0)rshft=shft/abs(utot)
      if(ish.ge.90.and.shft.gt.0.10.and.rshft.gt.0.10
     *.and.rerr.gt.0.03)then
      call utmsg('jdecas')
      write(ifch,*)'*****  utot reduced'
      write(ifch,*)'utot utot_old:',utot,utotod
      write(ifch,*)'n1 taus:',n1,sngl(ttaus) 
      write(ifch,*)'errors counts:',iutote,iutotc          
      call utmsgf
      endif
      endif
      do n=1,nzeta
      do nf=1,nflav
      ifli(n,nf)=iflptl(iy,n,nf)
      enddo
      enddo
      do n=1,nflav
      do j=1,2
      jcy(n,j)=0
      enddo
      enddo
           endif
      oi=utot
      nh1=0
      nh2=0

c     construct k sub-droplets: loop
c     ------------------------------
           do kk=1,k
      nptl=nptl+1
      if(nptl.gt.mxptl)
     *call utstop('jdecas: nptl>mxptl                      ')
      nzmini=1
      do n=1,nflav
      do j=1,2
      jc(n,j)=jcs(kk,n,j)
      enddo
      enddo
      call idenct(jc,idptl(nptl)
     *,ibptl(1,nptl),ibptl(2,nptl),ibptl(3,nptl),ibptl(4,nptl))
           if(iy.le.0)then
      iyptl(nptl)=0
      s=sz+des*(-1 - 1./k + kk*2./k)
      ui=s-ds
      oi=s+ds
           else  
      if(iyptlx+1.gt.myptl)call utcley(1)
      iyptlx=iyptlx+1
      if(iyptlx.gt.myptl)
     *call utstop('jdecas: myptl too small (3)             ')
      iyptl(nptl)=iyptlx
      iyi=iyptl(nptl)
      if(kk.eq.1)iyix=iyi
      iyiptl(iyptlx)=nptl
      do n=1,nzeta
      weiptl(iyptlx,n)=0
      do nf=1,nflav
      iflptl(iyi,n,nf)=0
      enddo
      enddo
      ui=oi
      zui=ui/ttaus
      if(ish.ge.96)write(ifch,*)'kk:',kk,'   nptl:',nptl
      if(ish.ge.96)write(ifch,*)'ui:',ui,'   utot:',utot
     *,'    otot:',otot
      nz=zui/dlzeta+nzeta/2+1
      if(kk.eq.1.and.nz.gt.n1)
     *call utstop('jdecas: nz > n1                         ')
      zn=(-nzeta/2-0.5+nz)*dlzeta 
      on=(zn+0.5*dlzeta)*ttaus
      un=ui
      wi=1./k
      ncount=0
5     continue
      ws=weiptl(iy,nz)/w/(dlzeta*ttaus)
      if(un.ge.otot.and.nz.gt.n2)then
      oi=otot
      dw=0
      goto6
      endif
      if(nz.eq.n1)then
      nh1=nh1+1
      if(nh1.eq.1)onmiun=on-un
      endif
      if(nz.eq.n2)then
      nh2=nh2+1
      if(nh2.eq.1)onmiun=on-un
      endif
      if(nz.eq.n1.or.nz.eq.n2)ws=ws/(onmiun)*(dlzeta*ttaus)
      dw=ws*(on-un)
      if(wi-dw.gt.0.)then
      if(ish.ge.96)write(ifch,*)'     nz:',nz
     *,'    un:',un,'    on:',on,'    dw:',dw
      if(iyi.ne.0)weiptl(iyi,nz)=weiptl(iyi,nz)+dw*w
      nz=nz+1
      un=on
      on=on+dlzeta*ttaus
      wi=wi-dw
      goto5
      endif
      dw=wi
      oi=dw/ws+un
      if(iyi.ne.0)weiptl(iyi,nz)=weiptl(iyi,nz)+dw*w
6     continue
      oi=amin1(oi,otot)
      if(kk.eq.k)oi=otot
      if(oi.le.ui)oi=ui+0.001
      if(ish.ge.96)write(ifch,*)'                                  '
     *,'oi:',oi,'    dw:',dw
      s=(oi+ui)/2
      if(ish.ge.96)write(ifch,*)'ui:',ui,'  oi:',oi
c modify jcs to account for local flavour conservation
      zoi=oi/ttaus
      zui=ui/ttaus
      if(nzmini.le.nzeta)then
      nzm=nzmini
      do nz=nzmini,nzeta
      z=(nz-nzeta/2)*dlzeta
      if(z.le.zoi.or.kk.eq.k)then
      do nf=1,nflav
      iflptl(iyi,nz,nf)=iflptl(iyi,nz,nf)+ifli(nz,nf)
      ifli(nz,nf)=0
      enddo
      nzm=nz+1
      elseif(z-dlzeta.lt.zoi)then
      zmin=max(zui,z-dlzeta)  
      p=(zoi-zmin)/(z-zmin)
      if(p.lt.0..or.p.gt.1.)then
      call utmsg('jdecas')
      write(ifch,*)'*****  p<0 or p>1'            
      write(ifch,*)'p=',p
      call utmsgf
      endif
      do nf=1,nflav
      iflix=ifli(nz,nf)
      if(iflix.gt.0)then
      do j=1,iflix
      if(rangen().le.p)then
      iflptl(iyi,nz,nf)=iflptl(iyi,nz,nf)+1
      ifli(nz,nf)=ifli(nz,nf)-1
      endif
      enddo
      elseif(iflix.lt.0)then
      do j=1,-iflix
      if(rangen().le.p)then
      iflptl(iyi,nz,nf)=iflptl(iyi,nz,nf)-1
      ifli(nz,nf)=ifli(nz,nf)+1
      endif
      enddo
      endif
      enddo
      endif
      enddo
      nzmini=nzm
      endif
      do nf=1,nflav
      jcs(kk,nf,1)=0
      jcs(kk,nf,2)=0
      do n=1,nzeta
      ifliyi=iflptl(iyi,n,nf)
      if(ifliyi.ge.0)jcs(kk,nf,1)=jcs(kk,nf,1)+ifliyi
      if(ifliyi.lt.0)jcs(kk,nf,2)=jcs(kk,nf,2)-ifliyi
      enddo
      if(jcs(kk,nf,1).ge.jcs(kk,nf,2))then
      jcs(kk,nf,1)=jcs(kk,nf,1)-jcs(kk,nf,2)
      jcs(kk,nf,2)=0
      endif
      if(jcs(kk,nf,2).gt.jcs(kk,nf,1))then
      jcs(kk,nf,2)=jcs(kk,nf,2)-jcs(kk,nf,1)
      jcs(kk,nf,1)=0
      endif
      enddo
      if(ish.ge.93)then
      write(ifch,101)nptl,iyi
     *,(weiptl(iyi,n),n=nzeta/2-5,nzeta/2+6)
     *,((iflptl(iyi,n,nf),n=nzeta/2-5,nzeta/2+6),nf=1,3)
      if(ish.ge.94)write(ifch,105)
     *((ifli(n,nf),n=nzeta/2-5,nzeta/2+6),nf=1,3)
105   format(1x,'ifli:'/1x,'u:  ',12i5/1x,'d:  ',12i5/1x,'s:  ',12i5)
      endif
      if(ish.ge.92)
     *write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
c account for former baryon number correction
      do nz=1,nzeta
      nzmin=nz
      if(weiptl(iyi,nz).gt.0.)goto20
      enddo
20    continue
      do nz=1,nzeta
      if(weiptl(iyi,nz).gt.0.)nzmax=nz
      enddo
      do n=1,nflav
      do j=1,2
      jcs(kk,n,j)=jcs(kk,n,j)+jcy(n,j)
      if(jcy(n,j).gt.0)then
      if(j.eq.1)iflptl(iyi,nzmin,n)=iflptl(iyi,nzmin,n)+jcy(n,j)
      if(j.eq.2)iflptl(iyi,nzmin,n)=iflptl(iyi,nzmin,n)-jcy(n,j)
      endif
      enddo
      enddo
      if(ish.ge.93)
     *write(ifch,*)'account for former baryon number correction:'
      if(ish.ge.94)write(ifch,109)
     *((iflptl(iyi,n,nf),n=nzeta/2-5,nzeta/2+6),nf=1,3)
109   format(1x,'u:  ',12i5/1x,'d:  ',12i5/1x,'s:  ',12i5)
      if(ish.ge.93)
     *write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
c integer baryon number for subdroplets
      if(ish.ge.93)write(ifch,*)'integer baryon number:'
      if(kk.lt.k)then
      idir=1
      if(rangen().le.0.5)idir=2
      do n=1,nflav
      do j=1,2
      jcy(n,j)=0
      enddo
      enddo
      move15=0
      move16=0
      ncount=0
12    ncount=ncount+1                                      
      if(ncount.gt.5)                                      
     *call utstop('jdecas: ncount.ge.5                     ')
      jq=0
      do n=1,nflav
      jq=jq+jcs(kk,n,1)-jcs(kk,n,2)
      enddo
      if(mod(jq,3).ne.0)then
      if(move15.eq.1)goto15
      if(move16.eq.1)goto16
      if(mod(jq+1,3).eq.0)goto15
      if(mod(jq-1,3).eq.0)goto16
15    continue                   !add quark or subtract antiquark
      do n=1,nflav
      iv=0
      if(idir.eq.1)then
      iflief=ifli(nzmini,n)+jcy(n,1)-jcy(n,2)
      else
      iflief=-iflptl(iyi,nzmax,n) 
      endif
      if(iflief.ge.1)iv=iflief
      if(n.eq.1)iprb(n)=iv
      if(n.gt.1)iprb(n)=iprb(n-1)+iv
      enddo
      if(idir.eq.1)then
      c2='+q'
      else
      c2='-a'
      endif
      if(ish.ge.96)write(ifch,138)ncount,c2,iprb 
138   format(1x,'attempt',i2,3x,a2,3x,'iprb:',6i3) 
      if(iprb(nflav).ge.1)then
      r=rangen()*iprb(nflav)
      do nx=1,nflav
      n=nx
      if(r.le.iprb(n))goto10
      enddo
10    continue
      jcs(kk,n,1)=jcs(kk,n,1)+1
      jcy(n,2)=jcy(n,2)+1
      iflptl(iyi,nzmax,n)=iflptl(iyi,nzmax,n)+1
      if(ish.ge.95)write(ifch,136)c2,idir,nzmini,iprb,r,n
136   format(1x,a2,i4,3x,'nzmini:',i3,3x,'iprb:',6i3,3x
     *,'r:',f6.2,3x,'n:',i3)
      move15=0
      move16=0
      else
      move16=1
      endif
      goto12
16    continue                  !add antiquark or subtract quark
      do n=1,nflav
      iv=0
      if(idir.eq.1)then
      iflief=-( ifli(nzmini,n)+jcy(n,1)-jcy(n,2) )
      else
      iflief=iflptl(iyi,nzmax,n)
      endif
      if(iflief.ge.1)iv=iflief
      if(n.eq.1)iprb(n)=iv
      if(n.gt.1)iprb(n)=iprb(n-1)+iv
      enddo
      if(idir.eq.1)then
      c2='+a'
      else
      c2='-q'
      endif
      if(ish.ge.96)write(ifch,138)ncount,c2,iprb 
      if(iprb(nflav).ge.1)then
      r=rangen()*iprb(nflav)
      do nx=1,nflav
      n=nx
      if(r.le.iprb(n))goto11
      enddo
11    continue
      jcs(kk,n,2)=jcs(kk,n,2)+1
      jcy(n,1)=jcy(n,1)+1
      iflptl(iyi,nzmax,n)=iflptl(iyi,nzmax,n)-1
      if(ish.ge.95)write(ifch,137)c2,idir,nzmini,iprb,r,n
137   format(1x,a2,i4,3x,'nzmini:',i3,3x,'iprb:',6i3,3x
     *,'r:',f6.2,3x,'n:',i3)
      move15=0
      move16=0
      else
      move15=1
      endif
      goto12
      endif
      endif
      if(ish.ge.94)write(ifch,109)
     *((iflptl(iyi,n,nf),n=nzeta/2-5,nzeta/2+6),nf=1,3)
      if(ish.ge.93)
     *write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
c nonvanishing id for subdroplets
      do n=1,nflav
      if(jcs(kk,n,1).ge.jcs(kk,n,2))then
      jcs(kk,n,1)=jcs(kk,n,1)-jcs(kk,n,2)
      jcs(kk,n,2)=0
      endif
      if(jcs(kk,n,2).gt.jcs(kk,n,1))then
      jcs(kk,n,2)=jcs(kk,n,2)-jcs(kk,n,1)
      jcs(kk,n,1)=0
      endif
      enddo
      ja=0
      do n=1,nflav
      do j=1,2
      ja=ja+jcs(kk,n,j)
      enddo
      enddo
      if(ja.eq.0)then
      n=int(rangen()/0.5)+1
      jcs(kk,n,1)=1
      jcs(kk,n,2)=1
      endif
      if(ish.ge.93)then
      write(ifch,*)'nonvanishing id:'
      write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
      endif
c check minimum mass
      ku=jcs(kk,1,1)-jcs(kk,1,2)
      kd=jcs(kk,2,1)-jcs(kk,2,2)
      ks=jcs(kk,3,1)-jcs(kk,3,2)
      kc=jcs(kk,4,1)-jcs(kk,4,2)
      if(ish.ge.93)
     *write(ifch,*)'ku kd ks kc:',ku,kd,ks,kc
      amini=utamnu(ku,kd,ks,kc,5)
      if(amuk.lt.amini)then 
      if(ish.ge.91)then
      write(ifch,*)'amuk<amini ---> k=k-1'
      if(ish.ge.92)then
      write(ifch,*)'amuk amini:',amuk,amini
      write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2)
      endif
      endif
      k=k-1
      iyptlx=iyptly          
      nptl=nptlb
      goto2
      endif
c determine idptl
      do n=1,nflav
      do j=1,2
      jc(n,j)=jcs(kk,n,j)
      enddo
      enddo
      call idenct(jc,idptl(nptl)
     *,ibptl(1,nptl),ibptl(2,nptl),ibptl(3,nptl),ibptl(4,nptl))
           endif
      dezptl(nptl)=(oi-ui)/2/ttaus
      call uttauh(nptl,ui,oi,g,rap,ws)
      am=amuk*g
      rap=rap-raptot
      pptl(1,nptl)=0
      pptl(2,nptl)=0
      pptl(3,nptl)=am*sinh(rap)
      pptl(4,nptl)=am*cosh(rap)
      pptl(5,nptl)=am
      if(ish.ge.93)write(ifch,115)nptl,idptl(nptl),(pptl(l,nptl),l=1,5)
115   format(1x,'/cptl/',i6,i10,3x,5(f8.2,1x))
      do l=1,4
      psumx(l)=psumx(l)+pptl(l,nptl)
      enddo
      call utlob3(-1,pptl(1,i),pptl(2,i),pptl(3,i),pptl(4,i),pptl(5,i)
     *,pptl(1,nptl),pptl(2,nptl),pptl(3,nptl),pptl(4,nptl))
      do l=1,4
      psum(l)=psum(l)+pptl(l,nptl)
      enddo
      call uttaui(s,ti,zi)
      xorptl(1,nptl)=x
      xorptl(2,nptl)=y
      xorptl(3,nptl)=zi
      xorptl(4,nptl)=ti
      iorptl(nptl)=i
      jorptl(nptl)=0
      istptl(nptl)=0
      ifrptl(1,nptl)=0
      ifrptl(2,nptl)=0
      io=nptl
1     io=iorptl(io)
      if(iorptl(io).gt.0)goto1
      zor=xorptl(3,io)
      tor=xorptl(4,io)
      r=rangen()
      tauran=-taurea*alog(r)
      call uttaix(nptl,tauran,zor,tor,zis,tis)
      tivptl(1,nptl)=amax1(ti,tis)
      tivptl(2,nptl)=ti
      iclptl(nptl)=1
      radptl(nptl)=radptl(i)
      desptl(nptl)=(oi-ui)/2
           enddo

c     check
c     -----
           if(ish.ge.92.and.iy.ne.0)then
      do kk=1,k
      write(ifch,103)((jcs(kk,n,j),n=1,nflav),j=1,2),kk
      enddo
           endif
           if(iy.ne.0)then
      do n=1,nflav
      jtest=jcx(n,1)-jcx(n,2)
      do kk=1,k
      jtest=jtest-jcs(kk,n,1)+jcs(kk,n,2)
      enddo
      if(jtest.ne.0)
     *call utstop('jdecas: sum jcs /= jcx  (1)             ')
      enddo
      do kk=1,k
      jq=0
      do n=1,nflav
      jq=jq+jcs(kk,n,1)-jcs(kk,n,2)
      enddo
      if(mod(jq,3).ne.0)
     *call utstop('jdecas: noninteger baryon number        ')
      enddo
           endif
      psumx(5)=psumx(4)**2-psumx(3)**2-psumx(2)**2-psumx(1)**2
      if(psumx(5).gt.0.)then
      psumx(5)=sqrt(psumx(5))
      else
      call utstop('jdecas: psumx(5) .le. 0                  ')
      endif
      psum(5)=psum(4)**2-psum(3)**2-psum(2)**2-psum(1)**2
      if(psum(5).gt.0.)then
      psum(5)=sqrt(psum(5))
      else
      call utstop('jdecas: psum(5) .le. 0                   ')
      endif
           if(ish.ge.93)then
      write(ifch,116)('  ------',l=1,5)
      write(ifch,119)psumx
119   format(1x,'psumx:',19x,5(f8.2,1x))
           endif
           if(ish.ge.91)then
      write(ifch,115)i,idptl(i),(pptl(l,i),l=1,5)
      if(ish.ge.92)write(ifch,116)('  ------',l=1,5)
      do n=nptlb+1,nptl
      write(ifch,115)n,idptl(n),(pptl(l,n),l=1,5)
      enddo
           endif
           if(ish.ge.92)then
      write(ifch,116)('  ------',l=1,5)
      write(ifch,120)psum
120   format(1x,'psum: ',19x,5(f8.2,1x))
         if(iy.gt.0)then 
      do n=1,nzeta
      do nf=1,nflav
      if(ifli(n,nf).ne.0)             
     *call utstop('jdecas: ifli .ne. 0                      ')
      enddo
      enddo
      do n=1,nzeta
      do nf=1,nflav
      isu=iflptl(iy,n,nf)
      do l=iyix,iyi
      isu=isu-iflptl(l,n,nf)
      enddo
      if(isu.ne.0)then
      write(ifch,*)'main droplet; iy:',iy
      write(ifch,109)
     *((iflptl(iy,nx,nfx),nx=nzeta/2-5,nzeta/2+6),nfx=1,3)
      do l=iyix,iyi
      write(ifch,*)'subdroplet; iy:',l
      write(ifch,109)
     *((iflptl(l,nx,nfx),nx=nzeta/2-5,nzeta/2+6),nfx=1,3)
      enddo
      call utstop('jdecas: sum iflptl(iyi,) =/ iflptl(iy,)  ')
      endif
      enddo
      enddo
      do n=1,nflav
      jtest1=0
      jtest2=0
      do kk=1,k
      jtest1=jtest1+jcs(kk,n,1)
      jtest2=jtest2+jcs(kk,n,2)
      enddo
      if(jtest1.ge.jtest2)then
      jtest1=jtest1-jtest2
      jtest2=0
      endif
      if(jtest2.gt.jtest1)then
      jtest2=jtest2-jtest1
      jtest1=0
      endif
      if(jtest1.ne.jcx(n,1).or.jtest2.ne.jcx(n,2))then
      write(ifch,*)'flavour:',n 
      write(ifch,*)'jtest1 jtest2:',jtest1,jtest2
      write(ifch,*)'jcx(n,):',jcx(n,1),jcx(n,2)
      write(ifch,*)'jcx:'
      write(ifch,103)((jcx(nf,j),nf=1,nflav),j=1,2)
      write(ifch,*)'jcs:'
      do kk=1,k
      write(ifch,103)((jcs(kk,nf,j),nf=1,nflav),j=1,2),kk
      enddo
      call utstop('jdecas: sum jcs /= jcx                    ')
      endif
      enddo
         endif
      write(ifch,*)' '
      write(ifch,117)i,sz-desptl(i),sz+desptl(i),x,y,z,t               
117   format(1x,'n s+-des x:',i6,2x,2(f8.2,1x),4x,4(f8.2,1x))
      write(ifch,124)('  ------',l=1,6)
124   format(1x,'           ',6x,2x,2(a8,1x),4x,4(a8,1x))
      do n=nptlb+1,nptl
      zi=xorptl(3,n)
      call uttaus(zi,ti,s)
      write(ifch,117)n,s-desptl(n),s+desptl(n)
     *,(xorptl(l,n),l=1,4)
      enddo
      zt=sz/ttaux
      wei=0
      if(iyptl(nptl).ne.0)then
      do nn=1,nzeta
      wei=wei+weiptl(iyptl(i),nn)
      enddo
      endif
      write(ifch,118)i,zt-dezptl(i),zt+dezptl(i),radptl(i),wei
118   format(1x,'n zt+-dez rad wei:',i6,2x,2(f8.2,1x),4x,f8.2,4x,f8.2)
      write(ifch,125)('  ------',l=1,4)
125   format(1x,'                  ',6x,2x,2(a8,1x),4x,a8,4x,a8)
      weis=0
      do n=nptlb+1,nptl
      zi=xorptl(3,n)
      call uttaus(zi,ti,s)
      zt=s/ttaux
      wei=0
      if(iyptl(nptl).ne.0)then
      do nn=1,nzeta
      wei=wei+weiptl(iyptl(n),nn)
      enddo
      endif
      weis=weis+wei
      write(ifch,118)n,zt-dezptl(n),zt+dezptl(n),radptl(n),wei
      enddo
      write(ifch,134)'  ------'
134   format(1x,'                  ',6x,2x,18x,4x,8x,4x,a8)
      write(ifch,135)weis
135   format(1x,'                  ',6x,2x,18x,4x,8x,4x,f8.2)
           endif
           if(ish.ge.92.and.iyptl(nptl).ne.0)then
      iy=iyptl(i)
      write(ifch,130)i,iy,(weiptl(iy,nn),nn=nzeta/2-3,nzeta/2+4) 
130   format(1x,'wei:',i6,i6,8f7.1)
      write(ifch,131)('  -----',l=1,8)
131   format(17x,8a7)
      do nn=1,nzeta
      weisum(nn)=0
      enddo
      do n=nptlb+1,nptl
      iyi=iyptl(n)
      do nn=1,nzeta
      weisum(nn)=weisum(nn)+weiptl(iyi,nn)
      enddo
      write(ifch,132)n,iyi,(weiptl(iyi,nn),nn=nzeta/2-3,nzeta/2+4) 
132   format(1x,'    ',i6,i6,8f7.1)
      enddo
      write(ifch,131)('  -----',l=1,8)
      write(ifch,133)(weisum(nn),nn=nzeta/2-3,nzeta/2+4) 
133   format(1x,'    ',12x,8f7.1)
           endif
           if(ish.ge.90)then
      err=1e-2
      do l=1,3
      if(abs(psumx(l)).gt.err
     *.and.abs(psumx(l)).gt.err*pptl(5,i))goto3
      enddo
      do l=4,5
      if(abs(psumx(l)-pptl(5,i)).gt.err
     *.and.abs(psumx(l)-pptl(5,i)).gt.err*pptl(5,i))goto3
      enddo
      do l=1,5
      if(abs(psum(l)-pptl(l,i)).gt.err
     *.and.abs(psum(l)-pptl(l,i)).gt.err*abs(pptl(l,i)))goto3
      enddo
      goto4
3     call utmsg('jdecas')
      write(ifch,*)'*****  psumx/=p(i)(cm)  or  psum/=p(i)'            
      do l=1,3
      write(ifch,123)psumx(l),zero,psum(l),pptl(l,i)
      enddo
      do l=4,5
      write(ifch,123)psumx(l),pptl(5,i),psum(l),pptl(l,i)
      enddo
123   format(1x,2f12.6,8x,2f12.6)
      call utmsgf
4     continue
      if(iyptl(nptl).ne.0)then
      iy=iyptl(i)
      do nn=1,nzeta
      weisum(nn)=weiptl(iy,nn)
      enddo
      do n=nptlb+1,nptl
      iyi=iyptl(n)
      do nn=1,nzeta
      weisum(nn)=weisum(nn)-weiptl(iyi,nn)
      if(n.eq.nptl.and.weisum(nn).gt.1e-3)then
      call utmsg('jdecas')
      write(ifch,*)'*****  sum weiptl(iyi,nn) /= weiptl(iy,nn)'
      write(ifch,*)'weiptl(iy,nn) - sum weiptl(iyi,nn)=',weisum(nn)
     *,'       nn=',nn
      write(ifch,*)'iy=',iy,'   weiptl(iy,nn)=',weiptl(iy,nn)
     *,'   otot=',otot
      do m=nptlb+1,nptl
      zm=xorptl(3,m)
      call uttaus(zm,tm,s)
      iym=iyptl(m)
      write(ifch,*)'iyi=',iym,'   weiptl(iyi,nn)=',weiptl(iym,nn)
     *,'   oi=',s+desptl(m)
      enddo
      call utmsgf
      endif
      enddo
      enddo
      endif
           endif

           if(iy.gt.0.and.amuk.lt.amux)then
      iyptlx=iyptly
      do n=nptlb+1,nptl
      iyi=iyptl(n)
      iyiptl(iyi)=0
      iyptl(n)=0
      do nn=1,nzeta
      weiptl(iyi,nn)=0
      enddo
      enddo
           endif

      istptl(i)=1
      ifrptl(1,i)=nptlb+1
      ifrptl(2,i)=nptl

      ttaus=ttauz
      ish=ish0
      goto1000

5000  continue
           if(ish.ge.91)then
      write(ifch,102)sngl(ttaux),i,idptl(i),amu
102   format(' >>> decay of fusion droplet; taus=',f6.2
     *,' i=',i6,' id=',i9,' amu=',f6.2)
           endif
      ttaus=ttauz
      goto5003

5001  continue
           if(ish.ge.91)then
      write(ifch,140)sngl(ttaus),i,idptl(i)
140   format(' >>> decay; taus=',f6.2
     *,' i=',i6,' id=',i9)
           endif

5003  continue
      if(ishsub/100.eq.21)ish=ish0
      call jdecax(i,iret)
      ish=ish0

1000  continue
      if(ish.ge.94)write(ifch,*)('-',l=1,30)
     *,' exit sr jdecas ',('-',l=1,10)
1001  continue

      end
                               
c-----------------------------------------------------------------------
      subroutine jdecax(i,iret)
c-----------------------------------------------------------------------
c  checks whether decay is suppressed
c  if not: calls jdecay
c          writes to /c?ptl/ for particle i and for decay products
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(mxnody=200)
      common/p13/nrnody,nody(mxnody),ndecay 
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter (myptl=1000,nzeta=60)
      common/c4ptl/iyptlx,iyptl(mxptl),weiptl(myptl,nzeta),iyiptl(myptl)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat,zor,tor
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 
      double precision ttaux,ttauz
      common/cclude/kclude
      parameter (nflav=6)
      integer jcdu(nflav,2)
      if(ish.ge.94)write(ifch,*)('-',l=1,10)
     *,' entry sr jdecax ',('-',l=1,30)

      iret=0
      if(istptl(i).ne.0)goto1001

      ttauz=ttaus

      ish0=ish
      if(ishsub/100.eq.5)ish=mod(ishsub,100)

      call idmass(111,amrho0)
      call idmass(221,amomeg)
      ioi=iorptl(i)
      if(.not.(ioi.gt.0.and.iabs(idptl(ioi)).lt.10000
     *.and.jorptl(i).eq.0))then
      if(idptl(i).eq.111)idptl(i)=221
      if(idptl(i).eq.221.and.rangen().gt.0.5)idptl(i)=111
      endif

      ida=iabs(idptl(i))
      if(mod(ndecay        ,10).eq.1
     *.and.ida.ne.0.and.ida.lt.10000)goto1000
      if(mod(ndecay/10     ,10).eq.1.and.ida.eq.  20)goto1000
      if(mod(ndecay/100    ,10).eq.1.and.ida.eq.2130)goto1000
      if(mod(ndecay/1000   ,10).eq.1.and.ida.eq.1130)goto1000
      if(mod(ndecay/1000   ,10).eq.1.and.ida.eq.2230)goto1000
      if(mod(ndecay/10000  ,10).eq.1.and.ida.eq.2330)goto1000
      if(mod(ndecay/10000  ,10).eq.1.and.ida.eq.1330)goto1000
      if(mod(ndecay/100000 ,10).eq.1.and.ida.eq.3331)goto1000
      if(mod(ndecay/1000000,10).eq.1.and.ida.eq. 110)goto1000

      if(nrnody.gt.0)then
      do nod=1,nrnody
      if(idptl(i).eq.nody(nod))goto1000
      enddo 
      endif

      t=tivptl(2,i)
      nptlb=nptl
      if(nptl.gt.mxptl-10)
     *call utstop('jdecax: mxptl too small                  ')
      ish=ish0
      call jdecay(i,iret)
      if(ishsub/100.eq.5)ish=mod(ishsub,100)
      if(iret.eq.1)goto1000
      if(nptl.le.nptlb)goto1000

      ish00=ish
      if(ishsub/100.eq.14.and.kclude.eq.1)ish=mod(ishsub,100)
      if(ish.ge.91)write(ifch,115)i,idptl(i)
     *,(pptl(k,i),k=1,5)
115   format(1x,'/cptl/',i6,i10,3x,5(e8.2,1x))

      istptl(i)=1
      ifrptl(1,i)=nptlb+1
      ifrptl(2,i)=nptl

      x=xorptl(1,i)+(t-xorptl(4,i))*pptl(1,i)/pptl(4,i)
      y=xorptl(2,i)+(t-xorptl(4,i))*pptl(2,i)/pptl(4,i)
      z=xorptl(3,i)+(t-xorptl(4,i))*pptl(3,i)/pptl(4,i)
      if(ish.ge.93)write(ifch,*)
     *'loop over decay products ',nptlb+1,' - ',nptl,' :'
      call uttaux(t,z,ttaux)
      ttaus=ttaux
      call uttaus(z,ttest,sz)
      do 20 n=nptlb+1,nptl
      if(ish.ge.93)write(ifch,*)'particle: ',n,idptl(n)
      iorptl(n)=i
      jorptl(n)=0
      istptl(n)=0
      ifrptl(1,n)=0
      ifrptl(2,n)=0
           if(dezptl(i).ne.0.)then
      rad=radptl(i)*sqrt(rangen())
      phi=2*pi*rangen()
      s=sz+desptl(i)*(2*rangen()-1)
      call uttaui(s,ti,zi)
           else
      rad=0
      phi=0
      ti=t
      zi=z
           endif
      xorptl(1,n)=x + rad*cos(phi)
      xorptl(2,n)=y + rad*sin(phi)
      xorptl(3,n)=zi
      xorptl(4,n)=ti
      io=n
1     io=iorptl(io)
      if(ish.ge.93)write(ifch,*)'io = ',io
      if(iorptl(io).gt.0)goto1
      if(ish.ge.93)write(ifch,*)'origin: ',io,idptl(io)
      zor=xorptl(3,io)
      tor=xorptl(4,io)
      call idquac(io,nq,ndu,ndu,jcdu)
      r=rangen()
      tauran=-taurea*alog(r)
      call uttaix(n,tauran,zor,tor,zis,tis)
      tivptl(1,n)=amax1(t,tis)
      call idtau(idptl(n),pptl(4,n),pptl(5,n),taugm)
      r=rangen()
      tivptl(2,n)=t+taugm*(-alog(r))
      iclptl(n)=1
      radptl(n)=0.
      desptl(n)=0.
      dezptl(n)=0.
      iyptl(n)=0          
      if(ish.ge.91)write(ifch,115)n,idptl(n)
     *,(pptl(k,n),k=1,5)
20    continue

      ish=ish00
 
1000  continue
      ttaus=ttauz
      ish=ish0
1001  continue
      if(ish.ge.94)write(ifch,*)('-',l=1,30)
     *,' exit sr jdecax ',('-',l=1,10)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine jdecay(ip,iret)
c-----------------------------------------------------------------------
c  decays particle ip from /cptl/
c  for ip being no resonance: call jcludc
c  for ip being resonance: standard resonance decay  procedure
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/wco/wmass2,wgam2
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (mxlook=10000,mxdky=2000)
      common/dkytab/look(mxlook),cbr(mxdky),mode(5,mxdky)
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      dimension pgen(5,5),rnd(5),u(3),beta(3)
     1,reduce(5)
      dimension prest(4,5)
      common/cclude/kclude
      data reduce/1.,1.,2.,5.,15./
      data twome/1.022006e-3/
 
c          fctn definitions
      dot(i1,i2)=prest(4,i1)*prest(4,i2)-prest(1,i1)*prest(1,i2)
     *-prest(2,i1)*prest(2,i2)-prest(3,i1)*prest(3,i2)
c          charged w propagator.
      wprop(z)=(z-wmass2**2)**2+(wmass2*wgam2)**2
 
      ish0=ish
      if(ishsub/100.eq.4)ish=mod(ishsub,100)
      if(ish.ge.94)write(ifch,*)('-',l=1,10)
     *,' entry sr jdecay ',('-',l=1,30)
      if(ish.ge.93)write(ifch,*)'decay of',ip,idptl(ip),pptl(5,ip)
      ish=ish0
 
      iret=0
      kclude=0
 
c     no k_long decay
c     ---------------
      if(idptl(ip).eq.-20)goto1000
 
c     droplet decay
c     -------------
      if(iabs(idptl(ip)).gt.1e8)then
      ishclu=ish
      if(ishsub/100.eq.1)ish=mod(ishsub,100)
      kclude=1
      call jcludc(ip,iret)
      ish=ishclu
      goto1000
      endif
 
      if(ishsub/100.eq.4)ish=mod(ishsub,100)
      if(ish.ge.93)write(ifch,*)'ordinary decay'
 
c     select decay mode
c     -----------------
      ntry=0
2     ntry=ntry+1
           if(ntry.gt.100)then
      if(ish.ge.90)then
      call utmsg('jdecay')
      write(ifch,*)'*****  decay not possible. iret = 1.'
      write(ifch,*)'id,mass: ',idptl(ip),pptl(5,ip)
      call utmsgf
      endif
      iret=1
      goto1000
           endif
      idlv1=idptl(ip)
      amss=pptl(5,ip)
1     continue
      ipoint=look(iabs(idlv1))-1
      if(ipoint.lt.0) goto1000
      try=rangen()
100   ipoint=ipoint+1
      if(ish.ge.93)write(ifch,*)'ipoint,cbr,try',ipoint,cbr(ipoint),try
      if(try.gt.cbr(ipoint)) goto100
      nadd=0
      sum=0.
      nstart=nptl+1
      do 110 i=1,5
      if(mode(i,ipoint).eq.0) goto110
      if(nptl+nadd+1.gt.mxptl) goto9999
      nadd=nadd+1
      new=nptl+nadd
      idptl(new)=mode(i,ipoint)
      idlv1=idptl(new)
      call idmass(idlv1,pptl(5,new))
      sum=sum+pptl(5,new)
110   continue
      if(nadd.ne.1.and.sum+1e-2.ge.amss)goto2
      nadd1=nadd-1
      do 120 j=1,5
      pgen(j,1)=pptl(j,ip)
120   continue
      pgen(5,nadd)=pptl(5,nptl+nadd)
      if(nadd.eq.1) goto700
      if(nadd.eq.2) goto400
 
c     use kroll-wada distribution for dalitz decays.
c     ----------------------------------------------
      if(ish.ge.93)write(ifch,*)'>= 3 body decay'
      if(.not.((idptl(ip).eq.110.or.idptl(ip).eq.220).and.
     1iabs(idptl(nptl+2)).eq.12)) goto130
      ntry=0
125   ntry=ntry+1
           if(ntry.gt.10)then
      if(ish.ge. 0)then
      call utmsg('jdecay')
      write(ifch,*)'*****  ntry > 10. iret = 1.'
      write(ifch,*)'amee,ree,wtee',amee,ree,wtee                        
      call utmsgf
      endif
      iret=1
      goto1000
           endif
      amee=twome*(pptl(5,ip)/twome)**rangen()
      ree=(twome/amee)**2
      wtee=(1.-(amee/pptl(5,ip))**2)**3*sqrt(1.-ree)*(1.+.5*ree)
      if(wtee.lt.rangen()) goto125
      pgen(5,2)=amee
      goto400
130   continue
 
c     calculate maximum phase-space weight
c     ------------------------------------
      wtmax=1./reduce(nadd)
      sum1=pgen(5,1)
      sum2=sum-pptl(5,nptl+1)
      do 200 i=1,nadd1
      wtmax=wtmax*utpcm(sum1,sum2,pptl(5,nptl+i))
      sum1=sum1-pptl(5,nptl+i)
      sum2=sum2-pptl(5,nptl+i+1)
200   continue
 
c     generate uniform nadd-body phase space
c     --------------------------------------
      ntry=0
300   ntry=ntry+1
           if(ntry.gt.10000)then
      if(ish.ge. 0)then
      call utmsg('jdecay')
      write(ifch,*)'*****  infinite loop (2). iret = 1.'
      write(ifch,*)'ip,idptl(ip),pptl(5,ip):',ip,idptl(ip),pptl(5,ip)
      write(ifch,*)'wt,wtmax:',wt,wtmax
      write(ifch,*)'i,pgen(5,i),pptl(5,nptl+i),idptl(nptl+i):'
      do i=1,nadd
      write(ifch,*)i,pgen(5,i),pptl(5,nptl+i),idptl(nptl+i)
      enddo
      call utmsgf
      endif
      iret=1
      goto1000
           endif
      rnd(1)=1.
      do 310 i=2,nadd1
      rnew=rangen()
      i1=i-1
      do 320 jj1=1,i1
      j=i-jj1
      jsave=j+1
      if(rnew.le.rnd(j)) goto310
      rnd(jsave)=rnd(j)
320   continue
310   rnd(jsave)=rnew
      rnd(nadd)=0.
      wt=1.
      sum1=sum
      do 330 i=2,nadd
      sum1=sum1-pptl(5,nptl+i-1)
      pgen(5,i)=sum1+rnd(i)*(pgen(5,1)-sum)
      a=pgen(5,i-1)
      b=pgen(5,i)
      c=pptl(5,nptl+i-1)
      wt=wt*utpcm(a,b,c)
330   continue
      if(wt.lt.rangen()*wtmax) goto300
 
c     carry out two-body decays in pgen frames
c     ----------------------------------------
400   continue
      if(ish.ge.93)write(ifch,*)'2 body decay'
      do 410 i=1,nadd1
      qcm=utpcm(pgen(5,i),pgen(5,i+1),pptl(5,nptl+i))
      u(3)=2.*rangen()-1.
      phi=2.*pi*rangen()
      u(1)=sqrt(1.-u(3)**2)*cos(phi)
      u(2)=sqrt(1.-u(3)**2)*sin(phi)
      do 420 j=1,3
      pptl(j,nptl+i)=qcm*u(j)
      pgen(j,i+1)=-pptl(j,nptl+i)
420   continue
      pptl(4,nptl+i)=sqrt(qcm**2+pptl(5,nptl+i)**2)
      pgen(4,i+1)=sqrt(qcm**2+pgen(5,i+1)**2)
410   continue
      do 430 j=1,4
      pptl(j,nptl+nadd)=pgen(j,nadd)
430   continue
 
c     boost pgen frames to lab frame
c          also save momenta in rest frame (last frame)
c     -------------------------------------------------
      do 500 ii=1,nadd1
      i=nadd-ii
      do 510 j=1,3
      beta(j)=pgen(j,i)/pgen(4,i)
510   continue
      gamma=pgen(4,i)/pgen(5,i)
      do 520 k=i,nadd
      k1=nptl+k
      bp=beta(1)*pptl(1,k1)+beta(2)*pptl(2,k1)+beta(3)*pptl(3,k1)
      do 530 j=1,3
      prest(j,k)=pptl(j,k1)
      pptl(j,k1)=pptl(j,k1)+gamma*beta(j)*(pptl(4,k1)
     1+bp*gamma/(gamma+1.))
530   continue
      prest(4,k)=pptl(4,k1)
      pptl(4,k1)=gamma*(pptl(4,k1)+bp)
520   continue
500   continue
 
c     matrix elements
c     ---------------
      if(nadd.eq.3.and.(idptl(ip).eq.221.or.idptl(ip).eq.331)) goto610
      if(nadd.eq.3.and.iabs(idptl(nptl+1)).lt.20.and.
     1idptl(nptl+1).ne.10) goto620
      goto800
 
c     omeg and phi decay
c          use vectors in rest frame
c     ------------------------------
610   wt=(pptl(5,nptl+1)*pptl(5,nptl+2)*pptl(5,nptl+3))**2
     1-(pptl(5,nptl+1)*dot(2,3))**2
     2-(pptl(5,nptl+2)*dot(1,3))**2
     3-(pptl(5,nptl+3)*dot(1,2))**2
     4+2.*dot(1,2)*dot(2,3)*dot(1,3)
      if(wt.lt.rangen()*pptl(5,ip)**6/108.) goto300
      goto800
 
c     semileptonic and quark decays
c          use vectors in rest frame, where ip has (m,0,0,0)
c          include w propagator
c     ------------------------------------------------------
620   wt=(pptl(5,ip)*prest(4,2))*dot(1,3)
      s12=pptl(5,nptl+1)**2+pptl(5,nptl+2)**2+2.*dot(1,2)
      s12max=pptl(5,ip)**2
      wt=wt*wprop(s12max)/wprop(s12)
      if(wt.lt.rangen()*pptl(5,ip)**4/16.) goto300
      goto800
 
c     one-particle decays
c     -------------------
700   continue
      do 710 j=1,5
      pptl(j,nptl+1)=pptl(j,ip)
710   continue
 
c     swap particles and antiparticles if idptl(ip)<0
c     -----------------------------------------------
800   continue
      if(idptl(ip).ge.0.or.iabs(idptl(ip)).eq.20) goto900
      do 810 i=1,nadd
      idabs=iabs(idptl(nptl+i))
      ifl1=idabs/1000
      ifl2=mod(idabs/100,10)
      ifl3=mod(idabs/10,10)
      if(ifl1.eq.0.and.ifl2.ne.0.and.ifl2.eq.ifl3) goto810
      if(idabs.eq.9.or.idabs.eq.10.or.idabs.eq.20) goto810
      if(idabs.eq.29.or.idabs.eq.30.or.idabs.eq.40) goto810
      idptl(nptl+i)=-idptl(nptl+i)
810   continue
 
900   continue
      nptl=nptl+nadd
      if(nptl.gt.mxptl)
     *call utstop('jdecay: nptl>mxptl                      ')
      nqk=0
      if(iabs(idptl(nptl)).lt.10.or.mod(idptl(nptl),100).eq.0)
     *call utstop('jdecay: decay ptcl is parton            ')
 
1000  continue
      ish=ish0
      if(ish.ge.94)write(ifch,*)('-',l=1,30)
     *,' exit sr jdecay ',('-',l=1,10)
      return
 
9999  call utstop('jdecay: mxptl too small                 ')
      end
 
c-----------------------------------------------------------------------
      subroutine jdecin(lprint)
c     sets up /dkytab/
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/wco/wmass2,wgam2
      dimension imode(6)
      character*8 idlabl,lmode(6),lres
      character*8 iquit,iblank
      logical lprint
      parameter (mxlook=10000,mxdky=2000)
      common/dkytab/look(mxlook),cbr(mxdky),mode(5,mxdky)
      common/nodcay/nodcay,noeta,nopi0,nonunu,noevol,nohadr
      logical nodcay,noeta,nopi0,nonunu,noevol,nohadr
      parameter (ndectb=1171)
      real dectab(7,ndectb)
 
      data ((dectab(i,j),i=1,7),j=  1, 18)/
     *  110., .98850,  10.,  10.,   0.,   0.,   0.
     *, 110.,1.00000,  10.,  12., -12.,   0.,   0.
     *, 220., .38000,  10.,  10.,   0.,   0.,   0.
     *, 220., .71000, 110., 110., 110.,   0.,   0.
     *, 220., .94600, 120.,-120., 110.,   0.,   0.
     *, 220., .99500, 120.,-120.,  10.,   0.,   0.
     *, 220.,1.00000,  10.,  12., -12.,   0.,   0.
     *, 330., .44100, 220., 120.,-120.,   0.,   0.
     *, 330., .66100, 220., 110., 110.,   0.,   0.
     *, 330., .95900, 111.,  10.,   0.,   0.,   0.
     *, 330., .98000, 221.,  10.,   0.,   0.,   0.
     *, 330.,1.00000,  10.,  10.,   0.,   0.,   0.
     *, 121.,1.00000, 120., 110.,   0.,   0.,   0.
     *, 111., .99989, 120.,-120.,   0.,   0.,   0.
     *, 111., .99993,  12., -12.,   0.,   0.,   0.
     *, 111.,1.00000,  14., -14.,   0.,   0.,   0.
     *, 221., .89900, 120.,-120., 110.,   0.,   0.
     *, 221., .91200, 120.,-120.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j= 19, 36)/
     *  221., .99992, 110.,  10.,   0.,   0.,   0.
     *, 221.,1.00000,  12., -12.,   0.,   0.,   0.
     *, 331., .48600, 130.,-130.,   0.,   0.,   0.
     *, 331., .83700, 230.,-230.,   0.,   0.,   0.
     *, 331., .98400, 120.,-120., 110.,   0.,   0.
     *, 331., .99944, 220.,  10.,   0.,   0.,   0.
     *, 331., .99975,  12., -12.,   0.,   0.,   0.
     *, 331.,1.00000,  14., -14.,   0.,   0.,   0.
     *, 230., .50000,  20.,   0.,   0.,   0.,   0.
     *, 230.,1.00000, -20.,   0.,   0.,   0.,   0.
     *, 131., .66670, 230., 120.,   0.,   0.,   0.
     *, 131.,1.00000, 130., 110.,   0.,   0.,   0.
     *, 231., .66670, 130.,-120.,   0.,   0.,   0.
     *, 231.,1.00000, 230., 110.,   0.,   0.,   0.
     *, 240., .11000,  12., -11., 230.,   0.,   0.
     *, 240., .17000,  12., -11., 231.,   0.,   0.
     *, 240., .28000,  14., -13., 230.,   0.,   0.
     *, 240., .34000,  14., -13., 231.,   0.,   0./
      data ((dectab(i,j),i=1,7),j= 37, 54)/
     *  240., .37800, 230.,-120.,   0.,   0.,   0.
     *, 240., .56300, 230.,-121.,   0.,   0.,   0.
     *, 240., .60800, 231.,-120.,   0.,   0.,   0.
     *, 240., .62100, 230.,-120., 110.,   0.,   0.
     *, 240., .71000, 130.,-120.,-120.,   0.,   0.
     *, 240., .80100, 230.,-120.,-120., 120.,   0.
     *, 240., .87900, 130.,-120.,-120., 110.,   0.
     *, 240., .95400, 230.,-120., 110., 110.,   0.
     *, 240., .96600, 230.,-130.,   0.,   0.,   0.
     *, 240., .97600, 331.,-120.,   0.,   0.,   0.
     *, 240., .98800,-130., 231.,   0.,   0.,   0.
     *, 240.,1.00000,-131., 230.,   0.,   0.,   0.
     *, 140., .04500,  12., -11., 130.,   0.,   0.
     *, 140., .07500,  12., -11., 131.,   0.,   0.
     *, 140., .12000,  14., -13., 130.,   0.,   0.
     *, 140., .15000,  14., -13., 131.,   0.,   0.
     *, 140., .20300, 130.,-120.,   0.,   0.,   0.
     *, 140., .22700, 230., 110.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j= 55, 72)/
     *  140., .24700, 230., 220.,   0.,   0.,   0.
     *, 140., .28900, 230., 221.,   0.,   0.,   0.
     *, 140., .45100, 130.,-121.,   0.,   0.,   0.
     *, 140., .53600, 131.,-120.,   0.,   0.,   0.
     *, 140., .56200, 231., 110.,   0.,   0.,   0.
     *, 140., .57600, 230., 111.,   0.,   0.,   0.
     *, 140., .58700, 130.,-120., 110.,   0.,   0.
     *, 140., .60300, 230.,-120., 120.,   0.,   0.
     *, 140., .72700, 130.,-120.,-120., 120.,   0.
     *, 140., .87600, 230.,-120., 120., 110.,   0.
     *, 140., .96900, 130.,-120., 110., 110.,   0.
     *, 140.,1.00000, 230., 110., 110., 110.,   0.
     *, 340., .03250,  12., -11., 220.,   0.,   0.
     *, 340., .06500,  12., -11., 331.,   0.,   0.
     *, 340., .09750,  14., -13., 220.,   0.,   0.
     *, 340., .13000,  14., -13., 331.,   0.,   0.
     *, 340., .17900,-130., 230.,   0.,   0.,   0.
     *, 340., .22800,-120., 220.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j= 73, 90)/
     *  340., .33800,-131., 230.,   0.,   0.,   0.
     *, 340., .44800,-130., 231.,   0.,   0.,   0.
     *, 340., .55800,-120., 331.,   0.,   0.,   0.
     *, 340., .57500,-130., 230., 110.,   0.,   0.
     *, 340., .59200,-230., 230.,-120.,   0.,   0.
     *, 340., .69400,-130., 230.,-120., 120.,   0.
     *, 340., .79600,-130., 230., 110., 110.,   0.
     *, 340., .89800,-130., 130.,-120., 110.,   0.
     *, 340.,1.00000,-230., 230.,-120., 110.,   0.
     *, 241., .64000, 140.,-120.,   0.,   0.,   0.
     *, 241., .92000, 240., 110.,   0.,   0.,   0.
     *, 241.,1.00000, 240.,  10.,   0.,   0.,   0.
     *, 141., .55000, 140., 110.,   0.,   0.,   0.
     *, 141.,1.00000, 140.,  10.,   0.,   0.,   0.
     *, 341.,1.00000, 340.,  10.,   0.,   0.,   0.
     *, 441., .07400,  12., -12.,   0.,   0.,   0.
     *, 441., .14800,  14., -14.,   0.,   0.,   0.
     *, 441., .15210,-121., 120.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j= 91,108)/
     *  441., .15620, 111., 110.,   0.,   0.,   0.
     *, 441., .16020, 121.,-120.,   0.,   0.,   0.
     *, 441., .16300,-121., 111., 120.,   0.,   0.
     *, 441., .16580, 121.,-121., 110.,   0.,   0.
     *, 441., .16860, 121., 111.,-120.,   0.,   0.
     *, 441., .28740, 120.,-120., 130.,-130.,   0.
     *, 441., .40620, 110., 110., 130.,-130.,   0.
     *, 441., .52500, 120.,-120., 120.,-120.,   0.
     *, 441., .64380, 120.,-120., 110., 110.,   0.
     *, 441., .76260, 110., 110., 110., 110.,   0.
     *, 441., .88130, 120.,-120., 230.,-230.,   0.
     *, 441.,1.00000, 110., 110., 230., 230.,   0.
     *, 150., .06000, -12.,  11., 140.,   0.,   0.
     *, 150., .12000, -12.,  11., 141.,   0.,   0.
     *, 150., .18000, -14.,  13., 140.,   0.,   0.
     *, 150., .24000, -14.,  13., 141.,   0.,   0.
     *, 150., .25500, -16.,  15., 140.,   0.,   0.
     *, 150., .27000, -16.,  15., 141.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=109,126)/
     *  150., .28050, 140., 120.,   0.,   0.,   0.
     *, 150., .29100, 140., 121.,   0.,   0.,   0.
     *, 150., .30150, 141., 120.,   0.,   0.,   0.
     *, 150., .31200, 141., 121.,   0.,   0.,   0.
     *, 150., .32650, 140.,-340.,   0.,   0.,   0.
     *, 150., .34100, 140.,-341.,   0.,   0.,   0.
     *, 150., .35550, 141.,-340.,   0.,   0.,   0.
     *, 150., .37000, 141.,-341.,   0.,   0.,   0.
     *, 150., .39800, 140., 120., 110.,   0.,   0.
     *, 150., .42600, 140., 120., 220.,   0.,   0.
     *, 150., .45400, 140., 120., 111.,   0.,   0.
     *, 150., .48200, 140., 120., 221.,   0.,   0.
     *, 150., .51000, 140., 121., 110.,   0.,   0.
     *, 150., .53800, 140., 121., 220.,   0.,   0.
     *, 150., .56600, 140., 121., 111.,   0.,   0.
     *, 150., .59400, 140., 121., 221.,   0.,   0.
     *, 150., .62200, 141., 120., 110.,   0.,   0.
     *, 150., .65000, 141., 120., 220.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=127,144)/
     *  150., .67800, 141., 120., 111.,   0.,   0.
     *, 150., .70600, 141., 120., 221.,   0.,   0.
     *, 150., .73400, 141., 121., 110.,   0.,   0.
     *, 150., .76200, 141., 121., 220.,   0.,   0.
     *, 150., .79000, 141., 121., 111.,   0.,   0.
     *, 150., .81800, 141., 121., 221.,   0.,   0.
     *, 150., .83200, 140., 130.,-230.,   0.,   0.
     *, 150., .84600, 140., 130.,-231.,   0.,   0.
     *, 150., .86000, 140., 131.,-230.,   0.,   0.
     *, 150., .87400, 140., 131.,-231.,   0.,   0.
     *, 150., .88800, 141., 130.,-230.,   0.,   0.
     *, 150., .90200, 141., 130.,-231.,   0.,   0.
     *, 150., .91600, 141., 131.,-230.,   0.,   0.
     *, 150., .93000, 141., 131.,-231.,   0.,   0.
     *, 150., .93300, 140.,-140., 130.,   0.,   0.
     *, 150., .93600, 140.,-140., 131.,   0.,   0.
     *, 150., .93900, 140.,-141., 130.,   0.,   0.
     *, 150., .94200, 140.,-141., 131.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=145,162)/
     *  150., .94500, 141.,-140., 130.,   0.,   0.
     *, 150., .94800, 141.,-140., 131.,   0.,   0.
     *, 150., .95100, 141.,-141., 130.,   0.,   0.
     *, 150., .95400, 141.,-141., 131.,   0.,   0.
     *, 150., .95700, 140.,-240., 230.,   0.,   0.
     *, 150., .96000, 140.,-240., 231.,   0.,   0.
     *, 150., .96300, 140.,-241., 230.,   0.,   0.
     *, 150., .96600, 140.,-241., 231.,   0.,   0.
     *, 150., .96900, 141.,-240., 230.,   0.,   0.
     *, 150., .97200, 141.,-240., 231.,   0.,   0.
     *, 150., .97500, 141.,-241., 230.,   0.,   0.
     *, 150., .97800, 141.,-241., 231.,   0.,   0.
     *, 150., .97950, 140.,-340., 330.,   0.,   0.
     *, 150., .98100, 140.,-340., 331.,   0.,   0.
     *, 150., .98250, 140.,-341., 331.,   0.,   0.
     *, 150., .98400, 140.,-341., 331.,   0.,   0.
     *, 150., .98550, 141.,-340., 330.,   0.,   0.
     *, 150., .98700, 141.,-340., 331.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=163,180)/
     *  150., .98850, 141.,-341., 331.,   0.,   0.
     *, 150., .99000, 141.,-341., 331.,   0.,   0.
     *, 150., .99200, 441., 130., 110.,   0.,   0.
     *, 150., .99400, 441., 131., 110.,   0.,   0.
     *, 150., .99600, 441., 230., 120.,   0.,   0.
     *, 150., .99800, 441., 231., 120.,   0.,   0.
     *, 150., .99900, 441., 330., 130.,   0.,   0.
     *, 150.,1.00000, 441., 331., 130.,   0.,   0.
     *, 250., .06000, -12.,  11., 240.,   0.,   0.
     *, 250., .12000, -12.,  11., 241.,   0.,   0.
     *, 250., .18000, -14.,  13., 240.,   0.,   0.
     *, 250., .24000, -14.,  13., 241.,   0.,   0.
     *, 250., .25500, -16.,  15., 240.,   0.,   0.
     *, 250., .27000, -16.,  15., 241.,   0.,   0.
     *, 250., .28050, 240., 120.,   0.,   0.,   0.
     *, 250., .29100, 240., 121.,   0.,   0.,   0.
     *, 250., .30150, 241., 120.,   0.,   0.,   0.
     *, 250., .31200, 241., 121.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=181,198)/
     *  250., .32650, 240.,-340.,   0.,   0.,   0.
     *, 250., .34100, 240.,-341.,   0.,   0.,   0.
     *, 250., .35550, 241.,-340.,   0.,   0.,   0.
     *, 250., .37000, 241.,-341.,   0.,   0.,   0.
     *, 250., .39800, 240., 120., 110.,   0.,   0.
     *, 250., .42600, 240., 120., 220.,   0.,   0.
     *, 250., .45400, 240., 120., 111.,   0.,   0.
     *, 250., .48200, 240., 120., 221.,   0.,   0.
     *, 250., .51000, 240., 121., 110.,   0.,   0.
     *, 250., .53800, 240., 121., 220.,   0.,   0.
     *, 250., .56600, 240., 121., 111.,   0.,   0.
     *, 250., .59400, 240., 121., 221.,   0.,   0.
     *, 250., .62200, 241., 120., 110.,   0.,   0.
     *, 250., .65000, 241., 120., 220.,   0.,   0.
     *, 250., .67800, 241., 120., 111.,   0.,   0.
     *, 250., .70600, 241., 120., 221.,   0.,   0.
     *, 250., .73400, 241., 121., 110.,   0.,   0.
     *, 250., .76200, 241., 121., 220.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=199,216)/
     *  250., .79000, 241., 121., 111.,   0.,   0.
     *, 250., .81800, 241., 121., 221.,   0.,   0.
     *, 250., .83200, 240., 130.,-230.,   0.,   0.
     *, 250., .84600, 240., 130.,-231.,   0.,   0.
     *, 250., .86000, 240., 131.,-230.,   0.,   0.
     *, 250., .87400, 240., 131.,-231.,   0.,   0.
     *, 250., .88800, 241., 130.,-230.,   0.,   0.
     *, 250., .90200, 241., 130.,-231.,   0.,   0.
     *, 250., .91600, 241., 131.,-230.,   0.,   0.
     *, 250., .93000, 241., 131.,-231.,   0.,   0.
     *, 250., .93300, 240.,-140., 130.,   0.,   0.
     *, 250., .93600, 240.,-140., 131.,   0.,   0.
     *, 250., .93900, 240.,-141., 130.,   0.,   0.
     *, 250., .94200, 240.,-141., 131.,   0.,   0.
     *, 250., .94500, 241.,-140., 130.,   0.,   0.
     *, 250., .94800, 241.,-140., 131.,   0.,   0.
     *, 250., .95100, 241.,-141., 130.,   0.,   0.
     *, 250., .95400, 241.,-141., 131.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=217,234)/
     *  250., .95700, 240.,-240., 230.,   0.,   0.
     *, 250., .96000, 240.,-240., 231.,   0.,   0.
     *, 250., .96300, 240.,-241., 230.,   0.,   0.
     *, 250., .96600, 240.,-241., 231.,   0.,   0.
     *, 250., .96900, 241.,-240., 230.,   0.,   0.
     *, 250., .97200, 241.,-240., 231.,   0.,   0.
     *, 250., .97500, 241.,-241., 230.,   0.,   0.
     *, 250., .97800, 241.,-241., 231.,   0.,   0.
     *, 250., .97950, 240.,-340., 330.,   0.,   0.
     *, 250., .98100, 240.,-340., 331.,   0.,   0.
     *, 250., .98250, 240.,-341., 331.,   0.,   0.
     *, 250., .98400, 240.,-341., 331.,   0.,   0.
     *, 250., .98550, 241.,-340., 330.,   0.,   0.
     *, 250., .98700, 241.,-340., 331.,   0.,   0.
     *, 250., .98850, 241.,-341., 331.,   0.,   0.
     *, 250., .99000, 241.,-341., 331.,   0.,   0.
     *, 250., .99200, 441., 130.,-120.,   0.,   0.
     *, 250., .99400, 441., 131.,-120.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=235,252)/
     *  250., .99600, 441., 230., 220.,   0.,   0.
     *, 250., .99800, 441., 231., 221.,   0.,   0.
     *, 250., .99900, 441., 330., 230.,   0.,   0.
     *, 250.,1.00000, 441., 331., 230.,   0.,   0.
     *, 350., .06000, -12.,  11., 340.,   0.,   0.
     *, 350., .12000, -12.,  11., 341.,   0.,   0.
     *, 350., .18000, -14.,  13., 340.,   0.,   0.
     *, 350., .24000, -14.,  13., 341.,   0.,   0.
     *, 350., .25500, -16.,  15., 340.,   0.,   0.
     *, 350., .27000, -16.,  15., 341.,   0.,   0.
     *, 350., .28050, 340., 120.,   0.,   0.,   0.
     *, 350., .29100, 340., 121.,   0.,   0.,   0.
     *, 350., .30150, 341., 120.,   0.,   0.,   0.
     *, 350., .31200, 341., 121.,   0.,   0.,   0.
     *, 350., .32650, 340.,-340.,   0.,   0.,   0.
     *, 350., .34100, 340.,-341.,   0.,   0.,   0.
     *, 350., .35550, 341.,-340.,   0.,   0.,   0.
     *, 350., .37000, 341.,-341.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=253,270)/
     *  350., .39800, 340., 120., 110.,   0.,   0.
     *, 350., .42600, 340., 120., 220.,   0.,   0.
     *, 350., .45400, 340., 120., 111.,   0.,   0.
     *, 350., .48200, 340., 120., 221.,   0.,   0.
     *, 350., .51000, 340., 121., 110.,   0.,   0.
     *, 350., .53800, 340., 121., 220.,   0.,   0.
     *, 350., .56600, 340., 121., 111.,   0.,   0.
     *, 350., .59400, 340., 121., 221.,   0.,   0.
     *, 350., .62200, 341., 120., 110.,   0.,   0.
     *, 350., .65000, 341., 120., 220.,   0.,   0.
     *, 350., .67800, 341., 120., 111.,   0.,   0.
     *, 350., .70600, 341., 120., 221.,   0.,   0.
     *, 350., .73400, 341., 121., 110.,   0.,   0.
     *, 350., .76200, 341., 121., 220.,   0.,   0.
     *, 350., .79000, 341., 121., 111.,   0.,   0.
     *, 350., .81800, 341., 121., 221.,   0.,   0.
     *, 350., .83200, 340., 130.,-230.,   0.,   0.
     *, 350., .84600, 340., 130.,-231.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=271,288)/
     *  350., .86000, 340., 131.,-230.,   0.,   0.
     *, 350., .87400, 340., 131.,-231.,   0.,   0.
     *, 350., .88800, 341., 130.,-230.,   0.,   0.
     *, 350., .90200, 341., 130.,-231.,   0.,   0.
     *, 350., .91600, 341., 131.,-230.,   0.,   0.
     *, 350., .93000, 341., 131.,-231.,   0.,   0.
     *, 350., .93300, 340.,-140., 130.,   0.,   0.
     *, 350., .93600, 340.,-140., 131.,   0.,   0.
     *, 350., .93900, 340.,-141., 130.,   0.,   0.
     *, 350., .94200, 340.,-141., 131.,   0.,   0.
     *, 350., .94500, 341.,-140., 130.,   0.,   0.
     *, 350., .94800, 341.,-140., 131.,   0.,   0.
     *, 350., .95100, 341.,-141., 130.,   0.,   0.
     *, 350., .95400, 341.,-141., 131.,   0.,   0.
     *, 350., .95700, 340.,-240., 230.,   0.,   0.
     *, 350., .96000, 340.,-240., 231.,   0.,   0.
     *, 350., .96300, 340.,-241., 230.,   0.,   0.
     *, 350., .96600, 340.,-241., 231.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=289,306)/
     *  350., .96900, 341.,-240., 230.,   0.,   0.
     *, 350., .97200, 341.,-240., 231.,   0.,   0.
     *, 350., .97500, 341.,-241., 230.,   0.,   0.
     *, 350., .97800, 341.,-241., 231.,   0.,   0.
     *, 350., .97950, 340.,-340., 330.,   0.,   0.
     *, 350., .98100, 340.,-340., 331.,   0.,   0.
     *, 350., .98250, 340.,-341., 331.,   0.,   0.
     *, 350., .98400, 340.,-341., 331.,   0.,   0.
     *, 350., .98550, 341.,-340., 330.,   0.,   0.
     *, 350., .98700, 341.,-340., 331.,   0.,   0.
     *, 350., .98850, 341.,-341., 331.,   0.,   0.
     *, 350., .99000, 341.,-341., 331.,   0.,   0.
     *, 350., .99200, 441., 130.,-130.,   0.,   0.
     *, 350., .99400, 441., 131.,-130.,   0.,   0.
     *, 350., .99600, 441., 230.,-230.,   0.,   0.
     *, 350., .99800, 441., 231.,-230.,   0.,   0.
     *, 350., .99900, 441., 330., 330.,   0.,   0.
     *, 350.,1.00000, 441., 331., 331.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=307,324)/
     *  160., .33330,  -1.,   2.,  -5.,   0.,   0.
     *, 160., .66660,  -4.,   3.,  -5.,   0.,   0.
     *, 160., .77770, -11.,  12.,  -5.,   0.,   0.
     *, 160., .88880, -13.,  14.,  -5.,   0.,   0.
     *, 160.,1.00000, -15.,  16.,  -5.,   0.,   0.
     *, 260., .33330,  -1.,   2.,  -5.,   0.,   0.
     *, 260., .66660,  -4.,   3.,  -5.,   0.,   0.
     *, 260., .77770, -11.,  12.,  -5.,   0.,   0.
     *, 260., .88880, -13.,  14.,  -5.,   0.,   0.
     *, 260.,1.00000, -15.,  16.,  -5.,   0.,   0.
     *, 360., .33330,  -1.,   2.,  -5.,   0.,   0.
     *, 360., .66660,  -4.,   3.,  -5.,   0.,   0.
     *, 360., .77770, -11.,  12.,  -5.,   0.,   0.
     *, 360., .88880, -13.,  14.,  -5.,   0.,   0.
     *, 360.,1.00000, -15.,  16.,  -5.,   0.,   0.
     *, 151.,1.00000, 150.,  10.,   0.,   0.,   0.
     *, 251.,1.00000, 250.,  10.,   0.,   0.,   0.
     *, 351.,1.00000, 350.,  10.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=325,342)/
     *  161.,1.00000, 160.,  10.,   0.,   0.,   0.
     *, 261.,1.00000, 260.,  10.,   0.,   0.,   0.
     *, 361.,1.00000, 360.,  10.,   0.,   0.,   0.
     *,1230.,1.00000,2130.,  10.,   0.,   0.,   0.
     *,1111.,1.00000,1120., 120.,   0.,   0.,   0.
     *,1121., .66670,1120., 110.,   0.,   0.,   0.
     *,1121.,1.00000,1220., 120.,   0.,   0.,   0.
     *,1221., .66670,1220., 110.,   0.,   0.,   0.
     *,1221.,1.00000,1120.,-120.,   0.,   0.,   0.
     *,2221.,1.00000,1220.,-120.,   0.,   0.,   0.
     *,1131., .88000,2130., 120.,   0.,   0.,   0.
     *,1131., .94000,1130., 110.,   0.,   0.,   0.
     *,1131.,1.00000,1230., 120.,   0.,   0.,   0.
     *,1231., .88000,2130., 110.,   0.,   0.,   0.
     *,1231., .94000,1130.,-120.,   0.,   0.,   0.
     *,1231.,1.00000,2230., 120.,   0.,   0.,   0.
     *,2231., .88000,2130.,-120.,   0.,   0.,   0.
     *,2231., .94000,1230.,-120.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=343,360)/
     * 2231.,1.00000,2230., 110.,   0.,   0.,   0.
     *,1331., .66670,2330., 120.,   0.,   0.,   0.
     *,1331.,1.00000,1330., 110.,   0.,   0.,   0.
     *,2331., .66670,1330.,-120.,   0.,   0.,   0.
     *,2331.,1.00000,2330., 110.,   0.,   0.,   0.
     *,  16., .18000,  12., -11.,  15.,   0.,   0.
     *,  16., .36000,  14., -13.,  15.,   0.,   0.
     *,  16., .45100,-120.,  15.,   0.,   0.,   0.
     *,  16., .66000,-121.,  15.,   0.,   0.,   0.
     *,  16., .78000, 110., 110.,-120.,  15.,   0.
     *,  16., .83600, 120.,-120.,-120.,  15.,   0.
     *,  16.,1.00000, 120., 110.,-120.,-120.,  15.
     *,2140., .03750, -12.,  11.,2130.,   0.,   0.
     *,2140., .07500, -12.,  11.,1231.,   0.,   0.
     *,2140., .11250, -14.,  13.,2130.,   0.,   0.
     *,2140., .15000, -14.,  13.,1231.,   0.,   0.
     *,2140., .18200,2130., 120.,   0.,   0.,   0.
     *,2140., .21300,1230., 110.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=361,378)/
     * 2140., .24400,1120.,-230.,   0.,   0.,   0.
     *,2140., .29500,1131., 110.,   0.,   0.,   0.
     *,2140., .34600,1231., 120.,   0.,   0.,   0.
     *,2140., .39700,1121.,-230.,   0.,   0.,   0.
     *,2140., .44800,1111.,-130.,   0.,   0.,   0.
     *,2140., .49900,1130., 111.,   0.,   0.,   0.
     *,2140., .55000,1230., 121.,   0.,   0.,   0.
     *,2140., .60100,1120.,-231.,   0.,   0.,   0.
     *,2140., .65800,1120.,-230., 120.,-120.,   0.
     *,2140., .71500,1120.,-230., 110., 110.,   0.
     *,2140., .77200,1120.,-130., 120., 110.,   0.
     *,2140., .82900,1220.,-230., 120., 110.,   0.
     *,2140., .88600,1220.,-130., 120., 120.,   0.
     *,2140., .94300,2130., 120., 120.,-120.,   0.
     *,2140.,1.00000,2130., 120., 110., 110.,   0.
     *,1140.,1.00000,2140., 120.,   0.,   0.,   0.
     *,1240.,1.00000,2140., 110.,   0.,   0.,   0.
     *,2240.,1.00000,2140.,-120.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=379,396)/
     * 1340., .03750, -12.,  11.,1330.,   0.,   0.
     *,1340., .07500, -12.,  11.,1331.,   0.,   0.
     *,1340., .11250, -14.,  13.,1330.,   0.,   0.
     *,1340., .15000, -14.,  13.,1331.,   0.,   0.
     *,1340., .19900,1330., 120.,   0.,   0.,   0.
     *,1340., .24800,1231.,-230.,   0.,   0.,   0.
     *,1340., .28800,1330., 120.,   0.,   0.,   0.
     *,1340., .32800,1131.,-230.,   0.,   0.,   0.
     *,1340., .36800,1330., 121.,   0.,   0.,   0.
     *,1340., .40800,1130.,-230.,   0.,   0.,   0.
     *,1340., .44800,1330., 120., 110.,   0.,   0.
     *,1340., .48800,2330., 120., 120.,   0.,   0.
     *,1340., .52800,1130.,-130., 120.,   0.,   0.
     *,1340., .56800,1130.,-230., 110.,   0.,   0.
     *,1340., .60800,1230.,-230., 120.,   0.,   0.
     *,1340., .66400,2130.,-230., 120., 110.,   0.
     *,1340., .72000,2130.,-130., 120., 120.,   0.
     *,1340., .77600,1130.,-230., 120., 120.,   0./
      data ((dectab(i,j),i=1,7),j=397,414)/
     * 1340., .83200,1130.,-230., 110., 110.,   0.
     *,1340., .88800,1330., 120., 120.,-120.,   0.
     *,1340., .94400,1330., 120., 110., 110.,   0.
     *,1340.,1.00000,2330., 120., 120., 110.,   0.
     *,3140., .03750, -12.,  11.,1330.,   0.,   0.
     *,3140., .07500, -12.,  11.,1331.,   0.,   0.
     *,3140., .11250, -14.,  13.,1330.,   0.,   0.
     *,3140., .15000, -14.,  13.,1331.,   0.,   0.
     *,3140., .19900,1330., 120.,   0.,   0.,   0.
     *,3140., .24800,1231.,-230.,   0.,   0.,   0.
     *,3140., .28800,1330., 120.,   0.,   0.,   0.
     *,3140., .32800,1131.,-230.,   0.,   0.,   0.
     *,3140., .36800,1330., 121.,   0.,   0.,   0.
     *,3140., .40800,1130.,-230.,   0.,   0.,   0.
     *,3140., .44800,1330., 120., 110.,   0.,   0.
     *,3140., .48800,2330., 120., 120.,   0.,   0.
     *,3140., .52800,1130.,-130., 120.,   0.,   0.
     *,3140., .56800,1130.,-230., 110.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=415,432)/
     * 3140., .60800,1230.,-230., 120.,   0.,   0.
     *,3140., .66400,2130.,-230., 120., 110.,   0.
     *,3140., .72000,2130.,-130., 120., 120.,   0.
     *,3140., .77600,1130.,-230., 120., 120.,   0.
     *,3140., .83200,1130.,-230., 110., 110.,   0.
     *,3140., .88800,1330., 120., 120.,-120.,   0.
     *,3140., .94400,1330., 120., 110., 110.,   0.
     *,3140.,1.00000,2330., 120., 120., 110.,   0.
     *,2340., .03750, -12.,  11.,2330.,   0.,   0.
     *,2340., .07500, -12.,  11.,2331.,   0.,   0.
     *,2340., .11250, -14.,  13.,2330.,   0.,   0.
     *,2340., .15000, -14.,  13.,2331.,   0.,   0.
     *,2340., .17500,2330., 120.,   0.,   0.,   0.
     *,2340., .20000,1330., 110.,   0.,   0.,   0.
     *,2340., .22500,1130.,-130.,   0.,   0.,   0.
     *,2340., .25000,1230.,-230.,   0.,   0.,   0.
     *,2340., .29500,2331., 120.,   0.,   0.,   0.
     *,2340., .34000,1331., 110.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=433,450)/
     * 2340., .38500,1131.,-130.,   0.,   0.,   0.
     *,2340., .43000,1231.,-230.,   0.,   0.,   0.
     *,2340., .47500,2330., 121.,   0.,   0.,   0.
     *,2340., .52000,1330., 111.,   0.,   0.,   0.
     *,2340., .56500,1130.,-131.,   0.,   0.,   0.
     *,2340., .61000,1230.,-231.,   0.,   0.,   0.
     *,2340., .64900,2130.,-230., 120.,-120.,   0.
     *,2340., .68800,2130.,-230., 110., 110.,   0.
     *,2340., .72700,2130.,-130., 120., 110.,   0.
     *,2340., .76600,1130.,-230.,-120., 110.,   0.
     *,2340., .80500,1130.,-130., 120.,-120.,   0.
     *,2340., .84400,1130.,-130., 110., 110.,   0.
     *,2340., .88300,1330., 120.,-120., 110.,   0.
     *,2340., .92200,1330., 110., 110., 110.,   0.
     *,2340., .96100,2330., 120., 120.,-120.,   0.
     *,2340.,1.00000,2330., 120., 110., 110.,   0.
     *,3240., .03750, -12.,  11.,2330.,   0.,   0.
     *,3240., .07500, -12.,  11.,2331.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=451,468)/
     * 3240., .11250, -14.,  13.,2330.,   0.,   0.
     *,3240., .15000, -14.,  13.,2331.,   0.,   0.
     *,3240., .17500,2330., 120.,   0.,   0.,   0.
     *,3240., .20000,1330., 110.,   0.,   0.,   0.
     *,3240., .22500,1130.,-130.,   0.,   0.,   0.
     *,3240., .25000,1230.,-230.,   0.,   0.,   0.
     *,3240., .29500,2331., 120.,   0.,   0.,   0.
     *,3240., .34000,1331., 110.,   0.,   0.,   0.
     *,3240., .38500,1131.,-130.,   0.,   0.,   0.
     *,3240., .43000,1231.,-230.,   0.,   0.,   0.
     *,3240., .47500,2330., 121.,   0.,   0.,   0.
     *,3240., .52000,1330., 111.,   0.,   0.,   0.
     *,3240., .56500,1130.,-131.,   0.,   0.,   0.
     *,3240., .61000,1230.,-231.,   0.,   0.,   0.
     *,3240., .64900,2130.,-230., 120.,-120.,   0.
     *,3240., .68800,2130.,-230., 110., 110.,   0.
     *,3240., .72700,2130.,-130., 120., 110.,   0.
     *,3240., .76600,1130.,-230.,-120., 110.,   0./
      data ((dectab(i,j),i=1,7),j=469,486)/
     * 3240., .80500,1130.,-130., 120.,-120.,   0.
     *,3240., .84400,1130.,-130., 110., 110.,   0.
     *,3240., .88300,1330., 120.,-120., 110.,   0.
     *,3240., .92200,1330., 110., 110., 110.,   0.
     *,3240., .96100,2330., 120., 120.,-120.,   0.
     *,3240.,1.00000,2330., 120., 110., 110.,   0.
     *,3340., .07500, -12.,  11.,3331.,   0.,   0.
     *,3340., .15000, -12.,  11.,3331.,   0.,   0.
     *,3340., .25000,1330.,-230.,   0.,   0.,   0.
     *,3340., .31000,3331., 120.,   0.,   0.,   0.
     *,3340., .37000,1331.,-230.,   0.,   0.,   0.
     *,3340., .43000,1330.,-231.,   0.,   0.,   0.
     *,3340., .49000,2330.,-230., 120.,   0.,   0.
     *,3340., .55000,1330.,-230., 110.,   0.,   0.
     *,3340., .61000,1330.,-130., 120.,   0.,   0.
     *,3340., .67500,3331., 120., 120.,-120.,   0.
     *,3340., .74000,3331., 120., 110., 110.,   0.
     *,3340., .80500,1330.,-230., 120.,-120.,   0./
      data ((dectab(i,j),i=1,7),j=487,504)/
     * 3340., .87000,1330.,-230., 110., 110.,   0.
     *,3340., .93500,2330.,-230., 120., 110.,   0.
     *,3340.,1.00000,2330.,-130., 120., 120.,   0.
     *,1141.,1.00000,2140., 120.,   0.,   0.,   0.
     *,1241.,1.00000,2140., 110.,   0.,   0.,   0.
     *,2241.,1.00000,2140.,-120.,   0.,   0.,   0.
     *,1341., .66670,2340., 120.,   0.,   0.,   0.
     *,1341.,1.00000,1340., 110.,   0.,   0.,   0.
     *,2341., .66670,1340.,-120.,   0.,   0.,   0.
     *,2341.,1.00000,2340., 110.,   0.,   0.,   0.
     *,3341.,1.00000,3340., 110.,   0.,   0.,   0.
     *,1150., .06000,  12., -11.,1140.,   0.,   0.
     *,1150., .12000,  12., -11.,1141.,   0.,   0.
     *,1150., .18000,  14., -13.,1140.,   0.,   0.
     *,1150., .24000,  14., -13.,1141.,   0.,   0.
     *,1150., .25500,  16., -15.,1140.,   0.,   0.
     *,1150., .27000,  16., -15.,1141.,   0.,   0.
     *,1150., .28925,1140.,-120.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=505,522)/
     * 1150., .30850,1140.,-121.,   0.,   0.,   0.
     *,1150., .32775,1141.,-120.,   0.,   0.,   0.
     *,1150., .34700,1141.,-121.,   0.,   0.,   0.
     *,1150., .35775,1140., 340.,   0.,   0.,   0.
     *,1150., .36850,1140., 341.,   0.,   0.,   0.
     *,1150., .37925,1141., 340.,   0.,   0.,   0.
     *,1150., .39000,1141., 341.,   0.,   0.,   0.
     *,1150., .42050,1140.,-120., 110.,   0.,   0.
     *,1150., .45100,1140.,-120., 220.,   0.,   0.
     *,1150., .48150,1140.,-120., 111.,   0.,   0.
     *,1150., .51200,1140.,-120., 221.,   0.,   0.
     *,1150., .54250,1140.,-121., 110.,   0.,   0.
     *,1150., .57300,1140.,-121., 220.,   0.,   0.
     *,1150., .60350,1140.,-121., 111.,   0.,   0.
     *,1150., .63400,1140.,-121., 221.,   0.,   0.
     *,1150., .66450,1141.,-120., 110.,   0.,   0.
     *,1150., .69500,1141.,-120., 220.,   0.,   0.
     *,1150., .72550,1141.,-120., 111.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=523,540)/
     * 1150., .75600,1141.,-120., 221.,   0.,   0.
     *,1150., .78650,1141.,-121., 110.,   0.,   0.
     *,1150., .81700,1141.,-121., 220.,   0.,   0.
     *,1150., .84750,1141.,-121., 111.,   0.,   0.
     *,1150., .87800,1141.,-121., 221.,   0.,   0.
     *,1150., .89325,1140.,-130., 230.,   0.,   0.
     *,1150., .90850,1140.,-130., 231.,   0.,   0.
     *,1150., .92375,1140.,-131., 230.,   0.,   0.
     *,1150., .93900,1140.,-131., 231.,   0.,   0.
     *,1150., .95425,1141.,-130., 230.,   0.,   0.
     *,1150., .96950,1141.,-130., 231.,   0.,   0.
     *,1150., .98475,1141.,-131., 230.,   0.,   0.
     *,1150.,1.00000,1141.,-131., 231.,   0.,   0.
     *,1250., .06000,  12., -11.,1240.,   0.,   0.
     *,1250., .12000,  12., -11.,1241.,   0.,   0.
     *,1250., .18000,  14., -13.,1240.,   0.,   0.
     *,1250., .24000,  14., -13.,1241.,   0.,   0.
     *,1250., .25500,  16., -15.,1240.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=541,558)/
     * 1250., .27000,  16., -15.,1241.,   0.,   0.
     *,1250., .28925,1240.,-120.,   0.,   0.,   0.
     *,1250., .30850,1240.,-121.,   0.,   0.,   0.
     *,1250., .32775,1241.,-120.,   0.,   0.,   0.
     *,1250., .34700,1241.,-121.,   0.,   0.,   0.
     *,1250., .35775,1240., 340.,   0.,   0.,   0.
     *,1250., .36850,1240., 341.,   0.,   0.,   0.
     *,1250., .37925,1241., 340.,   0.,   0.,   0.
     *,1250., .39000,1241., 341.,   0.,   0.,   0.
     *,1250., .42050,1240.,-120., 110.,   0.,   0.
     *,1250., .45100,1240.,-120., 220.,   0.,   0.
     *,1250., .48150,1240.,-120., 111.,   0.,   0.
     *,1250., .51200,1240.,-120., 221.,   0.,   0.
     *,1250., .54250,1240.,-121., 110.,   0.,   0.
     *,1250., .57300,1240.,-121., 220.,   0.,   0.
     *,1250., .60350,1240.,-121., 111.,   0.,   0.
     *,1250., .63400,1240.,-121., 221.,   0.,   0.
     *,1250., .66450,1241.,-120., 110.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=559,576)/
     * 1250., .69500,1241.,-120., 220.,   0.,   0.
     *,1250., .72550,1241.,-120., 111.,   0.,   0.
     *,1250., .75600,1241.,-120., 221.,   0.,   0.
     *,1250., .78650,1241.,-121., 110.,   0.,   0.
     *,1250., .81700,1241.,-121., 220.,   0.,   0.
     *,1250., .84750,1241.,-121., 111.,   0.,   0.
     *,1250., .87800,1241.,-121., 221.,   0.,   0.
     *,1250., .89325,1240.,-130., 230.,   0.,   0.
     *,1250., .90850,1240.,-130., 231.,   0.,   0.
     *,1250., .92375,1240.,-131., 230.,   0.,   0.
     *,1250., .93900,1240.,-131., 231.,   0.,   0.
     *,1250., .95425,1241.,-130., 230.,   0.,   0.
     *,1250., .96950,1241.,-130., 231.,   0.,   0.
     *,1250., .98475,1241.,-131., 230.,   0.,   0.
     *,1250.,1.00000,1241.,-131., 231.,   0.,   0.
     *,1350., .06000,  12., -11.,1340.,   0.,   0.
     *,1350., .12000,  12., -11.,1341.,   0.,   0.
     *,1350., .18000,  14., -13.,1340.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=577,594)/
     * 1350., .24000,  14., -13.,1341.,   0.,   0.
     *,1350., .25500,  16., -15.,1340.,   0.,   0.
     *,1350., .27000,  16., -15.,1341.,   0.,   0.
     *,1350., .28925,1340.,-120.,   0.,   0.,   0.
     *,1350., .30850,1340.,-121.,   0.,   0.,   0.
     *,1350., .32775,1341.,-120.,   0.,   0.,   0.
     *,1350., .34700,1341.,-121.,   0.,   0.,   0.
     *,1350., .35775,1340., 340.,   0.,   0.,   0.
     *,1350., .36850,1340., 341.,   0.,   0.,   0.
     *,1350., .37925,1341., 340.,   0.,   0.,   0.
     *,1350., .39000,1341., 341.,   0.,   0.,   0.
     *,1350., .42050,1340.,-120., 110.,   0.,   0.
     *,1350., .45100,1340.,-120., 220.,   0.,   0.
     *,1350., .48150,1340.,-120., 111.,   0.,   0.
     *,1350., .51200,1340.,-120., 221.,   0.,   0.
     *,1350., .54250,1340.,-121., 110.,   0.,   0.
     *,1350., .57300,1340.,-121., 220.,   0.,   0.
     *,1350., .60350,1340.,-121., 111.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=595,612)/
     * 1350., .63400,1340.,-121., 221.,   0.,   0.
     *,1350., .66450,1341.,-120., 110.,   0.,   0.
     *,1350., .69500,1341.,-120., 220.,   0.,   0.
     *,1350., .72550,1341.,-120., 111.,   0.,   0.
     *,1350., .75600,1341.,-120., 221.,   0.,   0.
     *,1350., .78650,1341.,-121., 110.,   0.,   0.
     *,1350., .81700,1341.,-121., 220.,   0.,   0.
     *,1350., .84750,1341.,-121., 111.,   0.,   0.
     *,1350., .87800,1341.,-121., 221.,   0.,   0.
     *,1350., .89325,1340.,-130., 230.,   0.,   0.
     *,1350., .90850,1340.,-130., 231.,   0.,   0.
     *,1350., .92375,1340.,-131., 230.,   0.,   0.
     *,1350., .93900,1340.,-131., 231.,   0.,   0.
     *,1350., .95425,1341.,-130., 230.,   0.,   0.
     *,1350., .96950,1341.,-130., 231.,   0.,   0.
     *,1350., .98475,1341.,-131., 230.,   0.,   0.
     *,1350.,1.00000,1341.,-131., 231.,   0.,   0.
     *,2150., .06000,  12., -11.,2140.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=613,630)/
     * 2150., .12000,  12., -11.,1241.,   0.,   0.
     *,2150., .18000,  14., -13.,2140.,   0.,   0.
     *,2150., .24000,  14., -13.,1241.,   0.,   0.
     *,2150., .25500,  16., -15.,2140.,   0.,   0.
     *,2150., .27000,  16., -15.,1241.,   0.,   0.
     *,2150., .28925,2140.,-120.,   0.,   0.,   0.
     *,2150., .30850,2140.,-121.,   0.,   0.,   0.
     *,2150., .32775,1241.,-120.,   0.,   0.,   0.
     *,2150., .34700,1241.,-121.,   0.,   0.,   0.
     *,2150., .35775,2140., 340.,   0.,   0.,   0.
     *,2150., .36850,2140., 341.,   0.,   0.,   0.
     *,2150., .37925,1241., 340.,   0.,   0.,   0.
     *,2150., .39000,1241., 341.,   0.,   0.,   0.
     *,2150., .42050,2140.,-120., 110.,   0.,   0.
     *,2150., .45100,2140.,-120., 220.,   0.,   0.
     *,2150., .48150,2140.,-120., 111.,   0.,   0.
     *,2150., .51200,2140.,-120., 221.,   0.,   0.
     *,2150., .54250,2140.,-121., 110.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=631,648)/
     * 2150., .57300,2140.,-121., 220.,   0.,   0.
     *,2150., .60350,2140.,-121., 111.,   0.,   0.
     *,2150., .63400,2140.,-121., 221.,   0.,   0.
     *,2150., .66450,1241.,-120., 110.,   0.,   0.
     *,2150., .69500,1241.,-120., 220.,   0.,   0.
     *,2150., .72550,1241.,-120., 111.,   0.,   0.
     *,2150., .75600,1241.,-120., 221.,   0.,   0.
     *,2150., .78650,1241.,-121., 110.,   0.,   0.
     *,2150., .81700,1241.,-121., 220.,   0.,   0.
     *,2150., .84750,1241.,-121., 111.,   0.,   0.
     *,2150., .87800,1241.,-121., 221.,   0.,   0.
     *,2150., .89325,2140.,-130., 230.,   0.,   0.
     *,2150., .90850,2140.,-130., 231.,   0.,   0.
     *,2150., .92375,2140.,-131., 230.,   0.,   0.
     *,2150., .93900,2140.,-131., 231.,   0.,   0.
     *,2150., .95425,1241.,-130., 230.,   0.,   0.
     *,2150., .96950,1241.,-130., 231.,   0.,   0.
     *,2150., .98475,1241.,-131., 230.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=649,666)/
     * 2150.,1.00000,1241.,-131., 231.,   0.,   0.
     *,2250., .06000,  12., -11.,2240.,   0.,   0.
     *,2250., .12000,  12., -11.,2241.,   0.,   0.
     *,2250., .18000,  14., -13.,2240.,   0.,   0.
     *,2250., .24000,  14., -13.,2241.,   0.,   0.
     *,2250., .25500,  16., -15.,2240.,   0.,   0.
     *,2250., .27000,  16., -15.,2241.,   0.,   0.
     *,2250., .28925,2240.,-120.,   0.,   0.,   0.
     *,2250., .30850,2240.,-121.,   0.,   0.,   0.
     *,2250., .32775,2241.,-120.,   0.,   0.,   0.
     *,2250., .34700,2241.,-121.,   0.,   0.,   0.
     *,2250., .35775,2240., 340.,   0.,   0.,   0.
     *,2250., .36850,2240., 341.,   0.,   0.,   0.
     *,2250., .37925,2241., 340.,   0.,   0.,   0.
     *,2250., .39000,2241., 341.,   0.,   0.,   0.
     *,2250., .42050,2240.,-120., 110.,   0.,   0.
     *,2250., .45100,2240.,-120., 220.,   0.,   0.
     *,2250., .48150,2240.,-120., 111.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=667,684)/
     * 2250., .51200,2240.,-120., 221.,   0.,   0.
     *,2250., .54250,2240.,-121., 110.,   0.,   0.
     *,2250., .57300,2240.,-121., 220.,   0.,   0.
     *,2250., .60350,2240.,-121., 111.,   0.,   0.
     *,2250., .63400,2240.,-121., 221.,   0.,   0.
     *,2250., .66450,2241.,-120., 110.,   0.,   0.
     *,2250., .69500,2241.,-120., 220.,   0.,   0.
     *,2250., .72550,2241.,-120., 111.,   0.,   0.
     *,2250., .75600,2241.,-120., 221.,   0.,   0.
     *,2250., .78650,2241.,-121., 110.,   0.,   0.
     *,2250., .81700,2241.,-121., 220.,   0.,   0.
     *,2250., .84750,2241.,-121., 111.,   0.,   0.
     *,2250., .87800,2241.,-121., 221.,   0.,   0.
     *,2250., .89325,2240.,-130., 230.,   0.,   0.
     *,2250., .90850,2240.,-130., 231.,   0.,   0.
     *,2250., .92375,2240.,-131., 230.,   0.,   0.
     *,2250., .93900,2240.,-131., 231.,   0.,   0.
     *,2250., .95425,2241.,-130., 230.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=685,702)/
     * 2250., .96950,2241.,-130., 231.,   0.,   0.
     *,2250., .98475,2241.,-131., 230.,   0.,   0.
     *,2250.,1.00000,2241.,-131., 231.,   0.,   0.
     *,2350., .06000,  12., -11.,2340.,   0.,   0.
     *,2350., .12000,  12., -11.,2341.,   0.,   0.
     *,2350., .18000,  14., -13.,2340.,   0.,   0.
     *,2350., .24000,  14., -13.,2341.,   0.,   0.
     *,2350., .25500,  16., -15.,2340.,   0.,   0.
     *,2350., .27000,  16., -15.,2341.,   0.,   0.
     *,2350., .28925,2340.,-120.,   0.,   0.,   0.
     *,2350., .30850,2340.,-121.,   0.,   0.,   0.
     *,2350., .32775,2341.,-120.,   0.,   0.,   0.
     *,2350., .34700,2341.,-121.,   0.,   0.,   0.
     *,2350., .35775,2340., 340.,   0.,   0.,   0.
     *,2350., .36850,2340., 341.,   0.,   0.,   0.
     *,2350., .37925,2341., 340.,   0.,   0.,   0.
     *,2350., .39000,2341., 341.,   0.,   0.,   0.
     *,2350., .42050,2340.,-120., 110.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=703,720)/
     * 2350., .45100,2340.,-120., 220.,   0.,   0.
     *,2350., .48150,2340.,-120., 111.,   0.,   0.
     *,2350., .51200,2340.,-120., 221.,   0.,   0.
     *,2350., .54250,2340.,-121., 110.,   0.,   0.
     *,2350., .57300,2340.,-121., 220.,   0.,   0.
     *,2350., .60350,2340.,-121., 111.,   0.,   0.
     *,2350., .63400,2340.,-121., 221.,   0.,   0.
     *,2350., .66450,2341.,-120., 110.,   0.,   0.
     *,2350., .69500,2341.,-120., 220.,   0.,   0.
     *,2350., .72550,2341.,-120., 111.,   0.,   0.
     *,2350., .75600,2341.,-120., 221.,   0.,   0.
     *,2350., .78650,2341.,-121., 110.,   0.,   0.
     *,2350., .81700,2341.,-121., 220.,   0.,   0.
     *,2350., .84750,2341.,-121., 111.,   0.,   0.
     *,2350., .87800,2341.,-121., 221.,   0.,   0.
     *,2350., .89325,2340.,-130., 230.,   0.,   0.
     *,2350., .90850,2340.,-130., 231.,   0.,   0.
     *,2350., .92375,2340.,-131., 230.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=721,738)/
     * 2350., .93900,2340.,-131., 231.,   0.,   0.
     *,2350., .95425,2341.,-130., 230.,   0.,   0.
     *,2350., .96950,2341.,-130., 231.,   0.,   0.
     *,2350., .98475,2341.,-131., 230.,   0.,   0.
     *,2350.,1.00000,2341.,-131., 231.,   0.,   0.
     *,3150., .06000,  12., -11.,3140.,   0.,   0.
     *,3150., .12000,  12., -11.,1341.,   0.,   0.
     *,3150., .18000,  14., -13.,3140.,   0.,   0.
     *,3150., .24000,  14., -13.,1341.,   0.,   0.
     *,3150., .25500,  16., -15.,3140.,   0.,   0.
     *,3150., .27000,  16., -15.,1341.,   0.,   0.
     *,3150., .28925,3140.,-120.,   0.,   0.,   0.
     *,3150., .30850,3140.,-121.,   0.,   0.,   0.
     *,3150., .32775,1341.,-120.,   0.,   0.,   0.
     *,3150., .34700,1341.,-121.,   0.,   0.,   0.
     *,3150., .35775,3140., 340.,   0.,   0.,   0.
     *,3150., .36850,3140., 341.,   0.,   0.,   0.
     *,3150., .37925,1341., 340.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=739,756)/
     * 3150., .39000,1341., 341.,   0.,   0.,   0.
     *,3150., .42050,3140.,-120., 110.,   0.,   0.
     *,3150., .45100,3140.,-120., 220.,   0.,   0.
     *,3150., .48150,3140.,-120., 111.,   0.,   0.
     *,3150., .51200,3140.,-120., 221.,   0.,   0.
     *,3150., .54250,3140.,-121., 110.,   0.,   0.
     *,3150., .57300,3140.,-121., 220.,   0.,   0.
     *,3150., .60350,3140.,-121., 111.,   0.,   0.
     *,3150., .63400,3140.,-121., 221.,   0.,   0.
     *,3150., .66450,1341.,-120., 110.,   0.,   0.
     *,3150., .69500,1341.,-120., 220.,   0.,   0.
     *,3150., .72550,1341.,-120., 111.,   0.,   0.
     *,3150., .75600,1341.,-120., 221.,   0.,   0.
     *,3150., .78650,1341.,-121., 110.,   0.,   0.
     *,3150., .81700,1341.,-121., 220.,   0.,   0.
     *,3150., .84750,1341.,-121., 111.,   0.,   0.
     *,3150., .87800,1341.,-121., 221.,   0.,   0.
     *,3150., .89325,3140.,-130., 230.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=757,774)/
     * 3150., .90850,3140.,-130., 231.,   0.,   0.
     *,3150., .92375,3140.,-131., 230.,   0.,   0.
     *,3150., .93900,3140.,-131., 231.,   0.,   0.
     *,3150., .95425,1341.,-130., 230.,   0.,   0.
     *,3150., .96950,1341.,-130., 231.,   0.,   0.
     *,3150., .98475,1341.,-131., 230.,   0.,   0.
     *,3150.,1.00000,1341.,-131., 231.,   0.,   0.
     *,3250., .06000,  12., -11.,3240.,   0.,   0.
     *,3250., .12000,  12., -11.,2341.,   0.,   0.
     *,3250., .18000,  14., -13.,3240.,   0.,   0.
     *,3250., .24000,  14., -13.,2341.,   0.,   0.
     *,3250., .25500,  16., -15.,3240.,   0.,   0.
     *,3250., .27000,  16., -15.,2341.,   0.,   0.
     *,3250., .28925,3240.,-120.,   0.,   0.,   0.
     *,3250., .30850,3240.,-121.,   0.,   0.,   0.
     *,3250., .32775,2341.,-120.,   0.,   0.,   0.
     *,3250., .34700,2341.,-121.,   0.,   0.,   0.
     *,3250., .35775,3240., 340.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=775,792)/
     * 3250., .36850,3240., 341.,   0.,   0.,   0.
     *,3250., .37925,2341., 340.,   0.,   0.,   0.
     *,3250., .39000,2341., 341.,   0.,   0.,   0.
     *,3250., .42050,3240.,-120., 110.,   0.,   0.
     *,3250., .45100,3240.,-120., 220.,   0.,   0.
     *,3250., .48150,3240.,-120., 111.,   0.,   0.
     *,3250., .51200,3240.,-120., 221.,   0.,   0.
     *,3250., .54250,3240.,-121., 110.,   0.,   0.
     *,3250., .57300,3240.,-121., 220.,   0.,   0.
     *,3250., .60350,3240.,-121., 111.,   0.,   0.
     *,3250., .63400,3240.,-121., 221.,   0.,   0.
     *,3250., .66450,2341.,-120., 110.,   0.,   0.
     *,3250., .69500,2341.,-120., 220.,   0.,   0.
     *,3250., .72550,2341.,-120., 111.,   0.,   0.
     *,3250., .75600,2341.,-120., 221.,   0.,   0.
     *,3250., .78650,2341.,-121., 110.,   0.,   0.
     *,3250., .81700,2341.,-121., 220.,   0.,   0.
     *,3250., .84750,2341.,-121., 111.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=793,810)/
     * 3250., .87800,2341.,-121., 221.,   0.,   0.
     *,3250., .89325,3240.,-130., 230.,   0.,   0.
     *,3250., .90850,3240.,-130., 231.,   0.,   0.
     *,3250., .92375,3240.,-131., 230.,   0.,   0.
     *,3250., .93900,3240.,-131., 231.,   0.,   0.
     *,3250., .95425,2341.,-130., 230.,   0.,   0.
     *,3250., .96950,2341.,-130., 231.,   0.,   0.
     *,3250., .98475,2341.,-131., 230.,   0.,   0.
     *,3250.,1.00000,2341.,-131., 231.,   0.,   0.
     *,3350., .06000,  12., -11.,3340.,   0.,   0.
     *,3350., .12000,  12., -11.,3341.,   0.,   0.
     *,3350., .18000,  14., -13.,3340.,   0.,   0.
     *,3350., .24000,  14., -13.,3341.,   0.,   0.
     *,3350., .25500,  16., -15.,3340.,   0.,   0.
     *,3350., .27000,  16., -15.,3341.,   0.,   0.
     *,3350., .28925,3340.,-120.,   0.,   0.,   0.
     *,3350., .30850,3340.,-121.,   0.,   0.,   0.
     *,3350., .32775,3341.,-120.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=811,828)/
     * 3350., .34700,3341.,-121.,   0.,   0.,   0.
     *,3350., .35775,3340., 340.,   0.,   0.,   0.
     *,3350., .36850,3340., 341.,   0.,   0.,   0.
     *,3350., .37925,3341., 340.,   0.,   0.,   0.
     *,3350., .39000,3341., 341.,   0.,   0.,   0.
     *,3350., .42050,3340.,-120., 110.,   0.,   0.
     *,3350., .45100,3340.,-120., 220.,   0.,   0.
     *,3350., .48150,3340.,-120., 111.,   0.,   0.
     *,3350., .51200,3340.,-120., 221.,   0.,   0.
     *,3350., .54250,3340.,-121., 110.,   0.,   0.
     *,3350., .57300,3340.,-121., 220.,   0.,   0.
     *,3350., .60350,3340.,-121., 111.,   0.,   0.
     *,3350., .63400,3340.,-121., 221.,   0.,   0.
     *,3350., .66450,3341.,-120., 110.,   0.,   0.
     *,3350., .69500,3341.,-120., 220.,   0.,   0.
     *,3350., .72550,3341.,-120., 111.,   0.,   0.
     *,3350., .75600,3341.,-120., 221.,   0.,   0.
     *,3350., .78650,3341.,-121., 110.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=829,846)/
     * 3350., .81700,3341.,-121., 220.,   0.,   0.
     *,3350., .84750,3341.,-121., 111.,   0.,   0.
     *,3350., .87800,3341.,-121., 221.,   0.,   0.
     *,3350., .89325,3340.,-130., 230.,   0.,   0.
     *,3350., .90850,3340.,-130., 231.,   0.,   0.
     *,3350., .92375,3340.,-131., 230.,   0.,   0.
     *,3350., .93900,3340.,-131., 231.,   0.,   0.
     *,3350., .95425,3341.,-130., 230.,   0.,   0.
     *,3350., .96950,3341.,-130., 231.,   0.,   0.
     *,3350., .98475,3341.,-131., 230.,   0.,   0.
     *,3350.,1.00000,3341.,-131., 231.,   0.,   0.
     *,1160., .33300,   1.,  -2.,1500.,   0.,   0.
     *,1160., .66700,   4.,  -3.,1500.,   0.,   0.
     *,1160., .77800, -12.,  11.,1500.,   0.,   0.
     *,1160., .88900, -14.,  13.,1500.,   0.,   0.
     *,1160.,1.00000, -16.,  15.,1500.,   0.,   0.
     *,1260., .33300,   1.,  -2.,2500.,   0.,   0.
     *,1260., .66700,   4.,  -3.,2500.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=847,864)/
     * 1260., .77800, -12.,  11.,2500.,   0.,   0.
     *,1260., .88900, -14.,  13.,2500.,   0.,   0.
     *,1260.,1.00000, -16.,  15.,2500.,   0.,   0.
     *,2260., .33300,   1.,  -2.,2500.,   0.,   0.
     *,2260., .66700,   4.,  -3.,2500.,   0.,   0.
     *,2260., .77800, -12.,  11.,2500.,   0.,   0.
     *,2260., .88900, -14.,  13.,2500.,   0.,   0.
     *,2260.,1.00000, -16.,  15.,2500.,   0.,   0.
     *,2160., .33300,   1.,  -2.,1500.,   0.,   0.
     *,2160., .66700,   4.,  -3.,1500.,   0.,   0.
     *,2160., .77800, -12.,  11.,1500.,   0.,   0.
     *,2160., .88900, -14.,  13.,1500.,   0.,   0.
     *,2160.,1.00000, -16.,  15.,1500.,   0.,   0.
     *,1360., .33300,   1.,  -2.,3500.,   0.,   0.
     *,1360., .66700,   4.,  -3.,3500.,   0.,   0.
     *,1360., .77800, -12.,  11.,3500.,   0.,   0.
     *,1360., .88900, -14.,  13.,3500.,   0.,   0.
     *,1360.,1.00000, -16.,  15.,3500.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=865,882)/
     * 2360., .33300,   1.,  -2.,3500.,   0.,   0.
     *,2360., .66700,   4.,  -3.,3500.,   0.,   0.
     *,2360., .77800, -12.,  11.,3500.,   0.,   0.
     *,2360., .88900, -14.,  13.,3500.,   0.,   0.
     *,2360.,1.00000, -16.,  15.,3500.,   0.,   0.
     *,3360., .33300,   1.,  -2.,3500.,   0.,   0.
     *,3360., .66700,   4.,  -3.,3500.,   0.,   0.
     *,3360., .77800, -12.,  11.,3500.,   0.,   0.
     *,3360., .88900, -14.,  13.,3500.,   0.,   0.
     *,3360.,1.00000, -16.,  15.,3500.,   0.,   0.
     *,1151.,1.00000,1150.,  10.,   0.,   0.,   0.
     *,1251.,1.00000,1250.,  10.,   0.,   0.,   0.
     *,2251.,1.00000,2250.,  10.,   0.,   0.,   0.
     *,1351.,1.00000,1350.,  10.,   0.,   0.,   0.
     *,2351.,1.00000,2350.,  10.,   0.,   0.,   0.
     *,3351.,1.00000,3350.,  10.,   0.,   0.,   0.
     *,1161.,1.00000,1160.,  10.,   0.,   0.,   0.
     *,1261.,1.00000,1260.,  10.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=883,886)/
     * 2261.,1.00000,2260.,  10.,   0.,   0.,   0.
     *,1361.,1.00000,1360.,  10.,   0.,   0.,   0.
     *,2361.,1.00000,2360.,  10.,   0.,   0.,   0.
     *,3361.,1.00000,3360.,  10.,   0.,   0.,   0./
c    *---------------------------------------------
c    *    delta++ resonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=887,900)/
c    *--dl++(1620)---------------------------------
     * 1112., .30000,1120., 120.,   0.,   0.,   0.
     *,1112., .66000,1111., 110.,   0.,   0.,   0.
     *,1112., .90000,1121., 120.,   0.,   0.,   0.
     *,1112.,1.00000,1120., 120., 110.,   0.,   0.
c    *--dl++(1700)---------------------------------
     *,1113., .15000,1120., 120.,   0.,   0.,   0.
     *,1113., .51000,1111., 110.,   0.,   0.,   0.
     *,1113., .75000,1121., 120.,   0.,   0.,   0.
     *,1113.,1.00000,1120., 120., 110.,   0.,   0.
c    *--dl++(1925)---------------------------------
     *,1114., .28000,1120., 120.,   0.,   0.,   0.
     *,1114., .40600,1111., 110.,   0.,   0.,   0.
     *,1114., .49000,1121., 120.,   0.,   0.,   0.
     *,1114., .69000,1120., 121.,   0.,   0.,   0.
     *,1114., .70000,1130., 130.,   0.,   0.,   0.
     *,1114.,1.00000,1122., 120.,   0.,   0.,   0./
c    *---------------------------------------------
c    *    delta- resonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=901,914)/
c    *--dl-(1620)----------------------------------
     * 2222., .30000,1220.,-120.,   0.,   0.,   0.
     *,2222., .66000,2221., 110.,   0.,   0.,   0.
     *,2222., .90000,1221.,-120.,   0.,   0.,   0.
     *,2222.,1.00000,1220., 110.,-120.,   0.,   0.
c    *--dl-(1700)----------------------------------
     *,2223., .15000,1220.,-120.,   0.,   0.,   0.
     *,2223., .51000,2221., 110.,   0.,   0.,   0.
     *,2223., .75000,1221.,-120.,   0.,   0.,   0.
     *,2223.,1.00000,1220., 110.,-120.,   0.,   0.
c    *--dl-(1925)----------------------------------
     *,2224., .28000,1220.,-120.,   0.,   0.,   0.
     *,2224., .40600,2221., 110.,   0.,   0.,   0.
     *,2224., .49000,1221.,-120.,   0.,   0.,   0.
     *,2224., .69000,1220.,-121.,   0.,   0.,   0.
     *,2224., .70000,2230., 230.,   0.,   0.,   0.
     *,2224.,1.00000,1222.,-120.,   0.,   0.,   0./
c    *---------------------------------------------
c    *    n*+ resonances + delta+ resonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=915,931)/
c    *--n*+(1440)----------------------------------
     * 1122., .20000,1120., 110.,   0.,   0.,   0.
     *,1122., .60000,1220., 120.,   0.,   0.,   0.
     *,1122., .68000,1111.,-120.,   0.,   0.,   0.
     *,1122., .73000,1121., 110.,   0.,   0.,   0.
     *,1122., .76000,1221., 120.,   0.,   0.,   0.
     *,1122., .84000,1120., 120.,-120.,   0.,   0.
     *,1122., .87000,1120., 110., 110.,   0.,   0.
     *,1122.,1.00000,1220., 120., 110.,   0.,   0.
c    *--n*+(1530)----------------------------------
     *,1123., .17000,1120., 110.,   0.,   0.,   0.
     *,1123., .51000,1220., 120.,   0.,   0.,   0.
     *,1123., .57000,1111.,-120.,   0.,   0.,   0.
     *,1123., .61000,1121., 110.,   0.,   0.,   0.
     *,1123., .63000,1221., 120.,   0.,   0.,   0.
     *,1123., .67000,1120., 120.,-120.,   0.,   0.
     *,1123., .68000,1120., 110., 110.,   0.,   0.
     *,1123., .75000,1220., 120., 110.,   0.,   0.
     *,1123.,1.00000,1120., 220.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=932,948)/
c    *--dl+(1620)----------------------------------
     * 1124., .20000,1120., 110.,   0.,   0.,   0.
     *,1124., .30000,1220., 120.,   0.,   0.,   0.
     *,1124., .54000,1111.,-120.,   0.,   0.,   0.
     *,1124., .58000,1121., 110.,   0.,   0.,   0.
     *,1124., .90000,1221., 120.,   0.,   0.,   0.
     *,1124., .96000,1120., 120.,-120.,   0.,   0.
     *,1124.,1.00000,1220., 120., 110.,   0.,   0.
c    *--n*+(1665)----------------------------------
     *,1125., .16700,1120., 110.,   0.,   0.,   0.
     *,1125., .49970,1220., 120.,   0.,   0.,   0.
     *,1125., .62470,1111.,-120.,   0.,   0.,   0.
     *,1125., .70800,1121., 110.,   0.,   0.,   0.
     *,1125., .74970,1221., 120.,   0.,   0.,   0.
     *,1125., .82080,1120., 120.,-120.,   0.,   0.
     *,1125., .85190,1120., 110., 110.,   0.,   0.
     *,1125., .96300,1220., 120., 110.,   0.,   0.
     *,1125., .97300,1120., 220.,   0.,   0.,   0.
     *,1125.,1.00000,2130., 130.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=949,955)/
c    *--dl+(1700)----------------------------------
     * 1126., .10000,1120., 110.,   0.,   0.,   0.
     *,1126., .15000,1220., 120.,   0.,   0.,   0.
     *,1126., .39000,1111.,-120.,   0.,   0.,   0.
     *,1126., .43000,1121., 110.,   0.,   0.,   0.
     *,1126., .75000,1221., 120.,   0.,   0.,   0.
     *,1126., .91500,1120., 120.,-120.,   0.,   0.
     *,1126.,1.00000,1220., 120., 110.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=956,969)/
c    *--n*+(1710)----------------------------------
     * 1127., .04430,1120., 110.,   0.,   0.,   0.
     *,1127., .13290,1220., 120.,   0.,   0.,   0.
     *,1127., .23790,1111.,-120.,   0.,   0.,   0.
     *,1127., .30790,1121., 110.,   0.,   0.,   0.
     *,1127., .34290,1221., 120.,   0.,   0.,   0.
     *,1127., .41190,1120., 120.,-120.,   0.,   0.
     *,1127., .48090,1120., 110., 110.,   0.,   0.
     *,1127., .54990,1220., 120., 110.,   0.,   0.
     *,1127., .66070,1120., 220.,   0.,   0.,   0.
     *,1127., .72800,2130., 130.,   0.,   0.,   0.
     *,1127., .74930,1230., 130.,   0.,   0.,   0.
     *,1127., .76000,1130., 230.,   0.,   0.,   0.
     *,1127., .84000,1120., 111.,   0.,   0.,   0.
     *,1127.,1.00000,1220., 121.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=970,980)/
c    *--dl+(1925)----------------------------------
     * 1128., .18700,1120., 110.,   0.,   0.,   0.
     *,1128., .28000,1220., 120.,   0.,   0.,   0.
     *,1128., .36400,1111.,-120.,   0.,   0.,   0.
     *,1128., .37800,1121., 110.,   0.,   0.,   0.
     *,1128., .49000,1221., 120.,   0.,   0.,   0.
     *,1128., .62300,1120., 111.,   0.,   0.,   0.
     *,1128., .69000,1220., 121.,   0.,   0.,   0.
     *,1128., .69350,1130., 230.,   0.,   0.,   0.
     *,1128., .69900,1230., 130.,   0.,   0.,   0.
     *,1128., .89900,1122., 110.,   0.,   0.,   0.
     *,1128.,1.00000,1222., 120.,   0.,   0.,   0./
c    *---------------------------------------------
c    *    n*0  resonances + delta0 resonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=981,997)/
c    *----------n*0(1440)--------------------------
     * 1222., .20000,1220., 110.,   0.,   0.,   0.
     *,1222., .60000,1120.,-120.,   0.,   0.,   0.
     *,1222., .68000,2221., 120.,   0.,   0.,   0.
     *,1222., .73000,1221., 110.,   0.,   0.,   0.
     *,1222., .76000,1121.,-120.,   0.,   0.,   0.
     *,1222., .84000,1220., 120.,-120.,   0.,   0.
     *,1222., .87000,1220., 110., 110.,   0.,   0.
     *,1222.,1.00000,1120.,-120., 110.,   0.,   0.
c    *----------n*0(1530)--------------------------
     *,1223., .17000,1220., 110.,   0.,   0.,   0.
     *,1223., .51000,1120.,-120.,   0.,   0.,   0.
     *,1223., .57000,2221., 120.,   0.,   0.,   0.
     *,1223., .61000,1221., 110.,   0.,   0.,   0.
     *,1223., .63000,1121.,-120.,   0.,   0.,   0.
     *,1223., .67000,1220., 120.,-120.,   0.,   0.
     *,1223., .68000,1220., 110., 110.,   0.,   0.
     *,1223., .75000,1120.,-120., 110.,   0.,   0.
     *,1223.,1.00000,1220., 220.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=998,1014)/
c    *----------dl0(1620)--------------------------
     * 1224., .20000,1220., 110.,   0.,   0.,   0.
     *,1224., .30000,1120.,-120.,   0.,   0.,   0.
     *,1224., .54000,2221., 120.,   0.,   0.,   0.
     *,1224., .58000,1221., 110.,   0.,   0.,   0.
     *,1224., .90000,1121.,-120.,   0.,   0.,   0.
     *,1224., .96500,1220., 120.,-120.,   0.,   0.
     *,1224.,1.00000,1120.,-120., 110.,   0.,   0.
c    *----------n*0(1665)--------------------------
     *,1225., .16700,1220., 110.,   0.,   0.,   0.
     *,1225., .49970,1120.,-120.,   0.,   0.,   0.
     *,1225., .62470,2221., 120.,   0.,   0.,   0.
     *,1225., .70800,1221., 110.,   0.,   0.,   0.
     *,1225., .74970,1121.,-120.,   0.,   0.,   0.
     *,1225., .82080,1220., 120.,-120.,   0.,   0.
     *,1225., .85190,1220., 110., 110.,   0.,   0.
     *,1225., .96300,1120.,-120., 110.,   0.,   0.
     *,1225., .97300,1220., 220.,   0.,   0.,   0.
     *,1225.,1.00000,2130., 230.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1015,1021)/
c    *----------dl0(1700)--------------------------
     * 1226., .10000,1220., 110.,   0.,   0.,   0.
     *,1226., .15000,1120.,-120.,   0.,   0.,   0.
     *,1226., .39000,2221., 120.,   0.,   0.,   0.
     *,1226., .43000,1221., 110.,   0.,   0.,   0.
     *,1226., .75000,1121.,-120.,   0.,   0.,   0.
     *,1226., .91500,1220., 120.,-120.,   0.,   0.
     *,1226.,1.00000,1120.,-120., 110.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1022,1035)/
c    *----------n*0(1710)--------------------------
     * 1227., .04430,1220., 110.,   0.,   0.,   0.
     *,1227., .13290,1120.,-120.,   0.,   0.,   0.
     *,1227., .23790,2221., 120.,   0.,   0.,   0.
     *,1227., .30790,1221., 110.,   0.,   0.,   0.
     *,1227., .34290,1121.,-120.,   0.,   0.,   0.
     *,1227., .41190,1220., 120.,-120.,   0.,   0.
     *,1227., .48090,1220., 110., 110.,   0.,   0.
     *,1227., .54990,1120.,-120., 110.,   0.,   0.
     *,1227., .66070,1220., 220.,   0.,   0.,   0.
     *,1227., .72800,2130., 230.,   0.,   0.,   0.
     *,1227., .73870,1230., 230.,   0.,   0.,   0.
     *,1227., .76000,2230., 130.,   0.,   0.,   0.
     *,1227., .92000,1120.,-121.,   0.,   0.,   0.
     *,1227.,1.00000,1220., 111.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1036,1046)/
c    *----------dl0(1925)--------------------------
     * 1228., .18700,1220., 110.,   0.,   0.,   0.
     *,1228., .28000,1120.,-120.,   0.,   0.,   0.
     *,1228., .36400,2221., 120.,   0.,   0.,   0.
     *,1228., .37800,1221., 110.,   0.,   0.,   0.
     *,1228., .49000,1121.,-120.,   0.,   0.,   0.
     *,1228., .55700,1220., 111.,   0.,   0.,   0.
     *,1228., .69000,1120.,-121.,   0.,   0.,   0.
     *,1228., .69350,2230., 130.,   0.,   0.,   0.
     *,1228., .70000,1230., 230.,   0.,   0.,   0.
     *,1228., .80000,1122.,-120.,   0.,   0.,   0.
     *,1228.,1.00000,1222., 110.,   0.,   0.,   0./
c    *---------------------------------------------
c    *   lambda resonances + sigma0 resonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=1047,1059)/
c    *----------lambda(1405)-----------------------
     * 1233., .33000,1230., 110.,   0.,   0.,   0.
     *,1233., .66000,2230., 120.,   0.,   0.,   0.
     *,1233.,1.00000,1130.,-120.,   0.,   0.,   0.
c    *----------lambda(1520)-----------------------
     *,1234., .22500,1120.,-130.,   0.,   0.,   0.
     *,1234., .48000,1220.,-230.,   0.,   0.,   0.
     *,1234., .62000,1230., 110.,   0.,   0.,   0.
     *,1234., .76000,2230., 120.,   0.,   0.,   0.
     *,1234., .90000,1130.,-120.,   0.,   0.,   0.
     *,1234., .96000,2130., 120.,-120.,   0.,   0.
     *,1234., .99000,2130., 110., 110.,   0.,   0.
     *,1234., .99330,1130.,-120., 110.,   0.,   0.
     *,1234., .99660,2230., 120., 110.,   0.,   0.
     *,1234.,1.00000,1230., 120.,-120.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1060,1075)/
c    *----------lambda(1645)-----------------------
     * 1235., .10000,1120.,-130.,   0.,   0.,   0.
     *,1235., .20000,1220.,-230.,   0.,   0.,   0.
     *,1235., .35000,1230., 110.,   0.,   0.,   0.
     *,1235., .50000,2230., 120.,   0.,   0.,   0.
     *,1235., .65000,1130.,-120.,   0.,   0.,   0.
     *,1235., .75000,2130., 120.,-120.,   0.,   0.
     *,1235., .80000,2130., 110., 110.,   0.,   0.
     *,1235., .84500,1130.,-120., 110.,   0.,   0.
     *,1235., .89000,2230., 120., 110.,   0.,   0.
     *,1235., .93500,1230., 120.,-120.,   0.,   0.
     *,1235.,1.00000,2130., 220.,   0.,   0.,   0.
c    *----------sigma0(1665)-----------------------
     *,1236., .10000,1120.,-130.,   0.,   0.,   0.
     *,1236., .20000,1220.,-230.,   0.,   0.,   0.
     *,1236., .40000,2230., 120.,   0.,   0.,   0.
     *,1236., .60000,1130.,-120.,   0.,   0.,   0.
     *,1236.,1.00000,2130., 110.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1076,1084)/
c    *----------sigma0(1776)-----------------------
     * 1237., .17500,1120.,-130.,   0.,   0.,   0.
     *,1237., .35000,1220.,-230.,   0.,   0.,   0.
     *,1237., .38750,2230., 120.,   0.,   0.,   0.
     *,1237., .42500,1130.,-120.,   0.,   0.,   0.
     *,1237., .57500,2130., 110.,   0.,   0.,   0.
     *,1237., .60000,2231., 120.,   0.,   0.,   0.
     *,1237., .62500,1131.,-120.,   0.,   0.,   0.
     *,1237., .75000,1234., 110.,   0.,   0.,   0.
     *,1237.,1.00000,1230., 220.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1085,1094)/
c    *----------lambda(1845)-----------------------
     * 1238., .17000,1120.,-130.,   0.,   0.,   0.
     *,1238., .34000,1220.,-230.,   0.,   0.,   0.
     *,1238., .44000,1230., 110.,   0.,   0.,   0.
     *,1238., .54000,2230., 120.,   0.,   0.,   0.
     *,1238., .64000,1130.,-120.,   0.,   0.,   0.
     *,1238., .70000,1231., 110.,   0.,   0.,   0.
     *,1238., .76000,2231., 120.,   0.,   0.,   0.
     *,1238., .82000,1131.,-120.,   0.,   0.,   0.
     *,1238., .91000,1120.,-131.,   0.,   0.,   0.
     *,1238.,1.00000,1220.,-231.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1095,1106)/
c    *----------sigma0(1930)-----------------------
     * 1239., .07500,1120.,-130.,   0.,   0.,   0.
     *,1239., .15000,1220.,-230.,   0.,   0.,   0.
     *,1239., .20000,1121.,-130.,   0.,   0.,   0.
     *,1239., .25000,1221.,-230.,   0.,   0.,   0.
     *,1239., .32500,1120.,-131.,   0.,   0.,   0.
     *,1239., .40000,1220.,-231.,   0.,   0.,   0.
     *,1239., .47500,2230., 120.,   0.,   0.,   0.
     *,1239., .55000,1130.,-120.,   0.,   0.,   0.
     *,1239., .70000,2130., 110.,   0.,   0.,   0.
     *,1239., .77500,2231., 120.,   0.,   0.,   0.
     *,1239., .85000,1131.,-120.,   0.,   0.,   0.
     *,1239.,1.00000,1234., 110.,   0.,   0.,   0./
c    *---------------------------------------------
c    *            sigma+ resonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=1107,1118)/
c    *----------sigma+(1665)-----------------------
     * 1132., .20000,1120.,-230.,   0.,   0.,   0.
     *,1132., .40000,1130., 110.,   0.,   0.,   0.
     *,1132., .60000,1230., 120.,   0.,   0.,   0.
     *,1132.,1.00000,2130., 120.,   0.,   0.,   0.
c    *----------sigma+(1776)-----------------------
     *,1133., .35000,1120.,-230.,   0.,   0.,   0.
     *,1133., .38750,1130., 110.,   0.,   0.,   0.
     *,1133., .42500,1230., 120.,   0.,   0.,   0.
     *,1133., .57500,2130., 120.,   0.,   0.,   0.
     *,1133., .60000,1131., 110.,   0.,   0.,   0.
     *,1133., .62500,1231., 120.,   0.,   0.,   0.
     *,1133., .75000,1234., 120.,   0.,   0.,   0.
     *,1133.,1.00000,1130., 220.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1119,1128)/
c    *----------sigma+(1930)-----------------------
     * 1134., .15000,1120.,-230.,   0.,   0.,   0.
     *,1134., .22500,1111.,-130.,   0.,   0.,   0.
     *,1134., .25000,1121.,-230.,   0.,   0.,   0.
     *,1134., .40000,1120.,-231.,   0.,   0.,   0.
     *,1134., .47500,1130., 110.,   0.,   0.,   0.
     *,1134., .55000,1230., 120.,   0.,   0.,   0.
     *,1134., .70000,2130., 120.,   0.,   0.,   0.
     *,1134., .77500,1131., 110.,   0.,   0.,   0.
     *,1134., .85000,1231., 120.,   0.,   0.,   0.
     *,1134.,1.00000,1234., 120.,   0.,   0.,   0./
c    *---------------------------------------------
c    *            sigma- resonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=1129,1140)/
c    *----------sigma-(1665)-----------------------
     * 2232., .20000,1220.,-130.,   0.,   0.,   0.
     *,2232., .40000,2230., 110.,   0.,   0.,   0.
     *,2232., .60000,1230.,-120.,   0.,   0.,   0.
     *,2232.,1.00000,2130.,-120.,   0.,   0.,   0.
c    *----------sigma-(1776)-----------------------
     *,2233., .35000,1220.,-130.,   0.,   0.,   0.
     *,2233., .38750,2230., 110.,   0.,   0.,   0.
     *,2233., .42500,1230.,-120.,   0.,   0.,   0.
     *,2233., .57500,2130.,-120.,   0.,   0.,   0.
     *,2233., .60000,2231., 110.,   0.,   0.,   0.
     *,2233., .62500,1231.,-120.,   0.,   0.,   0.
     *,2233., .75000,1234.,-120.,   0.,   0.,   0.
     *,2233.,1.00000,2230., 220.,   0.,   0.,   0./
      data ((dectab(i,j),i=1,7),j=1141,1150)/
c    *----------sigma-(1930)-----------------------
     * 2234., .15000,1220.,-130.,   0.,   0.,   0.
     *,2234., .17500,1221.,-130.,   0.,   0.,   0.
     *,2234., .25000,2221.,-230.,   0.,   0.,   0.
     *,2234., .40000,1220.,-131.,   0.,   0.,   0.
     *,2234., .47500,2230., 110.,   0.,   0.,   0.
     *,2234., .55000,1230.,-120.,   0.,   0.,   0.
     *,2234., .70000,2130.,-120.,   0.,   0.,   0.
     *,2234., .77500,2231., 110.,   0.,   0.,   0.
     *,2234., .85000,1231.,-120.,   0.,   0.,   0.
     *,2234.,1.00000,1234.,-120.,   0.,   0.,   0./
c    *---------------------------------------------
c    *      additional mesonresonances
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=1151,1159)/
c    *-----------f0(975)---------------------------
     *  332., .50000, 120.,-120.,   0.,   0.,   0.
     *, 332., .75000, 110., 110.,   0.,   0.,   0.
     *, 332., .87500, 130.,-130.,   0.,   0.,   0.
     *, 332.,1.00000, 230.,-230.,   0.,   0.,   0.
c    *-----------a0(980)---------------------------
     *, 112., .56000, 110., 220.,   0.,   0.,   0.
     *, 112., .78000, 130.,-130.,   0.,   0.,   0.
     *, 112.,1.00000, 230.,-230.,   0.,   0.,   0.
c    *-----------a+(980)---------------------------
     *, 122., .60000, 120., 220.,   0.,   0.,   0.
     *, 122.,1.00000, 130.,-230.,   0.,   0.,   0./
c    *---------------------------------------------
c    *      weak baryon decays
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=1160,1169)/
c    *-----------lambda(1116)----------------------
     * 2130.,0.64200,1120.,-120.,   0.,   0.,   0.
     *,2130.,1.00000,1220., 110.,   0.,   0.,   0.
c    *-----------sigma+(1180)----------------------
     *,1130.,0.51580,1120., 110.,   0.,   0.,   0.
     *,1130.,1.00000,1220., 120.,   0.,   0.,   0.
c    *-----------sigma-(1180)----------------------
     *,2230.,1.00000,1220.,-120.,   0.,   0.,   0.
c    *---------kaskade-(1360)----------------------
     *,2330.,1.00000,2130.,-120.,   0.,   0.,   0.
c    *---------kaskade0(1360)----------------------
     *,1330.,1.00000,2130., 110.,   0.,   0.,   0.
c    *---------omega-(1680)------------------------
     *,3331.,0.68000,2130.,-130.,   0.,   0.,   0.
     *,3331.,0.82000,1330.,-120.,   0.,   0.,   0.
     *,3331.,1.00000,2330., 110.,   0.,   0.,   0./
c    *---------------------------------------------
c    *      weak meson decays
c    *---------------------------------------------
      data ((dectab(i,j),i=1,7),j=1170,1171)/
c    *-----------k0s(975)--------------------------
     *   20., .68610, 120.,-120.,   0.,   0.,   0.
     *,  20.,1.00000, 110., 110.,   0.,   0.,   0./
c    *---------------------------------------------

      if(ientro.eq.1)call jcentr(3,6,3,1)
      if(ientro.eq.2)call jcentd
      call jcentp
 
      call idresi
 
c     determine wmass2,wgam2
c     ----------------------
      alfa=1./137.036
      gf=1.16570e-5
      sin2w=.215
      sinw=sqrt(sin2w)
      cosw=sqrt(1.-sin2w)
      amw=sqrt(pi*alfa/(.9304*sqrt(2.)*gf))/sinw
      wmass2=amw
      call idmass(5,amlep5)
      call idmass(6,amlep6)
      ngam=12
      if(amlep5+amlep6.gt.amw) ngam=9
      wgam2=gf*amw**3/(6.*pi*sqrt(2.))*ngam
 
      data iquit/'////'/,iblank/' '/
      ird=0
      do 1 i=1,mxlook
1     look(i)=0
      do 2 i=1,mxdky
      do 3 j=1,5
3     mode(j,i)=0
2     cbr(i)=0.
      nodcay=.false.
      noeta=.false.
      nopi0=.false.
      nonunu=.false.
      noevol=.false.
      nohadr=.false.
      if(lprint) write(ifch,10)
10    format('1',30('*')/' *',28x,'*'/
     1' *',5x,'isajet decay table',5x,'*'/
     2' *',28x,'*'/' ',30('*')//
     36x,'part',18x,'decay mode',19x,'cum br',15x,'ident',17x,
     4'decay ident')
      loop=0
      iold=0
      if(nodcay) return
 
200   loop=loop+1
      if(loop.gt.mxdky) goto9999
220   do 210 i=1,5
      imode(i)=0
      lmode(i)=iblank
210   continue
      ird=ird+1
      if(ird.gt.ndectb)return
      ires=nint(dectab(1,ird))
      br=dectab(2,ird)
      do 215 i=1,5
215   imode(i)=nint(dectab(2+i,ird))
      if(nopi0.and.ires.eq.110) goto220
      if(noeta.and.ires.eq.220) goto220
      if(ires.eq.iold) goto230
      if(ires.lt.0.or.ires.gt.mxlook)
     *call utstop('jdecin: ires out of range               ')
      look(ires)=loop
230   iold=ires
      cbr(loop)=br
      do 240 i=1,5
      mode(i,loop)=imode(i)
      if(imode(i).ne.0) lmode(i)=idlabl(imode(i))
240   continue
      lres=idlabl(ires)
      if(lprint) write(ifch,20) lres,(lmode(k),k=1,5),
     1br,ires,(imode(k),k=1,5)
20    format(6x,a5,6x,5(a5,2x),3x,f8.5,15x,i5,4x,5(i5,2x))
      goto200
 
9999  write(ifch,*)'loop=', loop
      call utstop('jdecin: loop > mxdky                    ')

      end
 
c-----------------------------------------------------------------------
      subroutine jestpr(ic1,ic2,am,ier)
c     processes strings
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      parameter (nflav=6)
      integer ic1(2),ic2(2),jc(nflav,2),jc1(nflav,2),jc2(nflav,2)
      parameter (nsi=6,nsix=100)
      real stro(nsi,nsix+1)
      common/ckol/kol /clead/lead,cooav3,cooav4
      parameter (nha=nsix+2,mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      common/cpzstr/pzstrl,estrl,istrl,isea
 
      ier=0
 
c     pmax
c     ----
      call iddeco(ic1,jc)
      nq=0
      do 7 nf=1,nflav
7     nq=nq+jc(nf,1)-jc(nf,2)
      am1min=piom
      if(iabs(nq).ge.2)am1min=prom
      call iddeco(ic2,jc)
      nq=0
      do 8 nf=1,nflav
8     nq=nq+jc(nf,1)-jc(nf,2)
      am2min=piom
      if(iabs(nq).ge.2)am2min=prom
      if(am.le.am1min+am2min)then
      pmax=am/2
      else
      pmax=utpcm(am,am1min,am2min)
      endif
 
c     hastpr
c     ------
      iproj=1          !this leads to iorstr()=-1
      itarg=1          !
      kproja(2,1)=1    !
      ktarga(2,1)=1    !
      kproja(3,1)=1    !
      ktarga(3,1)=1    !
      kol=1            !
      do 1 i=1,4         !this leads to xorstr()=0
1     coord(i,1)=0       !
      call iddeco(ic1,jc1)
      npa1=0
      do 2 n=1,nflav
      do 2 i=1,2
2     npa1=npa1+jc1(n,i)
      call iddeco(ic2,jc2)
      npa2=0
      do 3 n=1,nflav
      do 3 i=1,2
3     npa2=npa2+jc2(n,i)
      ii=1
      if(npa2.gt.npa1)ii=2
      lead=0
      if(npa1.gt.1.or.npa2.gt.1)lead=1
 
      nstr0=nstr
17    nstr=nstr0
 
      do 15 n=1,nsi
      do 15 i=1,3
15    stro(n,i)=0
      stro(3,ii)=am/2
      stro(4,ii)=am/2
      stro(5,ii)=ic1(1)
      stro(6,ii)=ic1(2)
      stro(3,3-ii)=-am/2
      stro(4,3-ii)=am/2
      stro(5,3-ii)=ic2(1)
      stro(6,3-ii)=ic2(2)
 
      pzstrl=stro(3,1)
      estrl=stro(4,1)
      pzstrl=pzstrl+stro(3,2)
      estrl=estrl+stro(4,2)
      istrl=0
 
      call utpage
      if(ish.ge.91)then
      write(ifch,110)('#',l=1,79),ipage,('#',l=1,79)
110   format(1x,79a1/1x,i5,'.page            '
     *,'string generation'/1x,79a1)
      write(ifch,105)(stro(i,1),i=1,4),(nint(stro(i,1)),i=5,6)
      write(ifch,104)(stro(i,2),i=1,4),(nint(stro(i,2)),i=5,6)
105   format(' str: ',4f11.5,2i8)
104   format('      ',4f11.5,2i8)
      endif
 
      isea=1
      isplt=0
14    call hastpr(stro,isplt)
      if(isplt.eq.-1)goto9001
      if(isplt.eq.-3)goto9001
      if(isplt.eq.-4)
     *call utstop('jestpr: isplt=-4                        ')
      if(isplt.eq.-5)goto17
      if(isplt.gt.0)goto14
 
      pmxevt=pmax
      egyevt=am
 
      return
 
9001  ier=1
      return
 
      end
 
c-----------------------------------------------------------------------
      subroutine jetgen(ier)
c     generates strings
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paroe/prob(99),icbac(99,2),icfor(99,2)
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      common/partnr/summas,iptnr,ic4,px4,py4,js4,pex,pey,pezet,pe0,nps
     */cntevm/ntevm
      common /clep/icinpu,idscat
      integer ic1(2),ic2(2),ic4(2)
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      parameter (mamx=250)
      real targa(nsi,nha,mamx)
      integer icstri(4,18)
 
      data ((icstri(i,j),i=1,4),j=1,8)/
     *100000,000000,110000,000000,
     *010000,000000,200000,000000,
     *100000,000000,210000,100000,
     *010000,000000,210000,010000,
     *001000,000000,210000,001000,
     *000000,100000,310000,000000,
     *000000,010000,220000,000000,
     *000000,001000,211000,000000/
      data ((icstri(i,j),i=1,4),j=11,18)/
     *100000,000000,020000,000000,
     *010000,000000,110000,000000,
     *100000,000000,120000,100000,
     *010000,000000,120000,010000,
     *001000,000000,120000,001000,
     *000000,100000,220000,000000,
     *000000,010000,130000,000000,
     *000000,001000,121000,000000/
 
      ish0=ish
      if(ishsub/100.eq.2)ish=mod(ishsub,100)
      if(ish.ge.93)write(ifch,*)('-',i=1,10)
     *,' entry sr jetgen ',('-',i=1,30)
 
      ier=0
 
      nevt=1
      nstr=0
      nptl=0
 
           if(icinpu.ge.1)then
3     call lepexp(xbj,qsq)
      rnu=qsq/(2.* prom * xbj)
      wsq=prom**2 + 2.* prom * rnu - qsq
      amst=sqrt(wsq)
      elepto=elepti-rnu
      cosang=1-qsq/2/elepti/elepto
      if(ish.ge.93)write(ifch,*)'elepti,elepto,rnu: ',elepti,elepto,rnu
      if(ish.ge.93)write(ifch,*)'xbj,qsq,cosang=1-qsq/2/elepti/elepto: '
     *,xbj,qsq,cosang
      if(rnu.gt.elepti)then
      if(ish.ge.93)write(ifch,*)'*****  q0 too large: ',rnu
      goto3
      endif
      if(cosang.gt.1.and.cosang.lt.1+1e-3)then
      cosang=1.
      elseif(cosang.lt.-1..or.cosang.gt.1.)then
      cosano=cosang
      if(cosang.ge.0.)cosang=1.
      if(cosang.lt.0.)cosang=-1.
      call utmsg('jetgen')
      write(ifch,*)'*****  cosang out of range'
      write(ifch,*)'elepti,elepto,rnu: ',elepti,elepto,rnu
      write(ifch,*)'xbj,qsq,cosang=1-qsq/2/elepti/elepto: '
     *,xbj,qsq,cosano
      write(ifch,*)'cosang_new: ',cosang
      call utmsgf
      endif
      angmue=acos(cosang)
      call leptar(xbj,qsq,matarg,latarg,idscat)
           endif
 
      if(matarg.gt.0)then
      call nucoge
      nptl=0
      call nucini('str',targa,latarg,matarg,-1)
      call nucstr(ier)
      if(ier.eq.1)goto99999
      endif
 
           if(icinpu.eq.0)then
      r=rangen()
      ps=0
      do 1 k=1,99
      ps=ps+prob(k)
      if(r.le.ps)goto2
1     continue
      call utstop('jetgen: no k found                      ')
2     ic1(1)=icfor(k,1)
      ic1(2)=icfor(k,2)
      ic2(1)=icbac(k,1)
      ic2(2)=icbac(k,2)
      amst=engy
           elseif(icinpu.gt.0)then
      call lepstr(idscat,xbj,qsq,ids)
      ic1(1)=icstri(1,ids)
      ic1(2)=icstri(2,ids)
      ic2(1)=icstri(3,ids)
      ic2(2)=icstri(4,ids)
           endif
      call jestpr(ic1,ic2,amst,ier)
 
99999 continue
      if(ish.ge.93)write(ifch,*)('-',i=1,30)
     *,' exit sr jetgen ',('-',i=1,10)
      ish=ish0
      return
      end

c-----------------------------------------------------------------------
      subroutine jfrade(ier)
c-----------------------------------------------------------------------
c  performs string fragmentation/decay and fin. state interactions
c-----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      parameter(mxpri=200) 
      character*6 subpri
      common/p14/nrpri,subpri(mxpri),ishpri(mxpri) 
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt

      common/partnr/summas,iptnr,ic4,px4,py4,js4,pex,pey,pezet,pe0,nps
     */didib/ndidib /cnfr/nrfra
      parameter (mxindx=1000,mxre=100,mxma=11,mxmx=6)
      common/crema/indx(mxindx),rema(mxre,mxma),rewi(mxre,mxma)
     *,idmx(mxma,mxmx),icre1(mxre,mxma),icre2(mxre,mxma)
      parameter (mxlook=10000,mxdky=2000)
      common/dkytab/look(mxlook),cbr(mxdky),mode(5,mxdky)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat /ctimel/ntc
      parameter(nprbms=20)
      common /cprbms/prbms(nprbms) /cnce/nces,ncolex /ckol/kol 
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      parameter (mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      common/cjint/nptl0,rnup,rnut,velp,velt,bx,by,xaver(4)

      real psum(5)
      parameter (nflav=6)
      integer ic4(2)

      ishini=ish
      if(nrpri.gt.0)then
      do nr=1,nrpri
      if(subpri(nr).eq.'jfrade')then
      ish=ishpri(nr)
      endif       
      enddo
      endif
      

c     initialization
c     --------------
      ier=0
      iret=0
      ish0=ish
      if(ishsub/100.eq.16)ish=mod(ishsub,100)
      if(nevt.ne.1.or.ifrade.eq.0)goto1000
      nptlpt=iabs(maproj)+iabs(matarg)
      if(iappl.eq.3)then
      ycmmax=alog(egyevt/.4)
      etapro=ycmmax*etafac
      etatar=-etapro
      else
      etapro=(ypjtl-yhaha)*etafac
      etatar=-yhaha*etafac
      endif
      detap=etapro
      detat=etatar
      tpro=dcosh(detap)
      zpro=dsinh(detap)
      ttar=dcosh(detat)
      ztar=dsinh(detat)
 
c     write
c     -----
      call utpage
      if(ish.ge.91)write(ifch,110)('#',l=1,79),('#',l=1,79)
110   format(1x,79a1/1x,'strings before rescaling'/1x,79a1)
      do 9 j=1,nstr
      if(ish.ge.91)write(ifch,109)j,(icstr(k,j),k=1,4)
     *,sqrt(pstr(1,j)**2+pstr(2,j)**2),pstr(3,j),pstr(5,j)
     *,irlstr(j)
9     continue
109   format(' /cstr/',i4,3x,4i7,2x,3(1x,e9.3),i4)
 
c     string rescaling
c     ----------------
      if((iappl.eq.1.or.iappl.eq.3).and.nstr.gt.1.and.irescl.eq.1)then  
      psum(1)=0
      psum(2)=0
      if(iappl.eq.3)then
      psum(3)=0
      psum(4)=egyevt
      else
      psum(3)=(npjevt-ntgevt)*pnllx
      psum(4)=npjevt*sqrt(amproj**2+pnllx**2)
     *+ntgevt*sqrt(amtarg**2+pnllx**2)
      endif
      psum(5)=sqrt(psum(4)**2-psum(3)**2)
      call hrescl(1,nstr,psum,ifail)
      if(ifail.ne.0)goto1001
      endif
 
c     write
c     -----
      call utpage
      if(ish.ge.91)write(ifch,111)('#',l=1,79),('#',l=1,79)
111   format(1x,79a1/1x,'strings after rescaling'/1x,79a1)
      do 10 j=1,nstr
      if(ish.ge.91)write(ifch,109)j,(icstr(k,j),k=1,4)
     *,sqrt(pstr(1,j)**2+pstr(2,j)**2),pstr(3,j),pstr(5,j)
     *,irlstr(j)
10    continue
 
c     fragmentation
c     -------------
      if(ishsub/100.eq.3)ish=mod(ishsub,100)
      if(ish.eq.13)call uttima('               ')
      if(ish.eq.91)write(ifch,'(1x,79a1)')('#',l=1,79)
      if(ish.eq.91)write(ifch,'(1x,a)')'string fragmentation'
      if(ish.eq.91)write(ifch,'(1x,79a1)')('#',l=1,79)
      do 3 j=1,nstr
      call utpage
      if(ish.ge.92)write(ifch,'(1x,79a1)')('#',l=1,79)
      if(ish.ge.91)write(ifch,102)j
     *,(icstr(k,j),k=1,4),sqrt(pstr(1,j)**2+pstr(2,j)**2
     *+pstr(3,j)**2),pstr(4,j),pstr(5,j)
102   format(1x,'>>> string:',i3,4i7,1x,3(e8.2,1x))
      if(ish.ge.92)write(ifch,'(1x,79a1)')('#',l=1,79)
      call jamfra(j,newevt)
      if(newevt.eq.1)goto10011
3     continue
      if(ish.eq.13)call uttima('fragmentation  ')
 
c     print /cptl/
c     ------------
      if(ishsub/100.eq.9)ish=mod(ishsub,100)
      call utpage
      if(ish.ge.91)then
      write(ifch,104)('#',l=1,79),('#',l=1,79)
104   format(1x,79a1/1x,'ptls after fragmentation'/1x,79a1)
      do 18 n=1,nptl
      if(n.gt.nptlpt.or.istptl(n).ne.0)
     *write(ifch,116)iorptl(n),jorptl(n),n,ifrptl(1,n),ifrptl(2,n)
     *,idptl(n),pptl(3,n),pptl(4,n),pptl(5,n),istptl(n)
18    continue
      endif
 
c     initial decay
c     -------------
      if(ishsub/100.eq.10)ish=mod(ishsub,100)
      call utpage
      if(ish.eq.13)call uttima('               ')
      iacn=1
      if(rinmes.gt.0..and.rinbar.gt.0.)then
      ttaus=taumin
      else
      ttaus=ainfin
      iacn=0
      endif
      if(ish.ge.91)then
      write(ifch,119)('#',l=1,79),sngl(ttaus),('#',l=1,79)
119   format(1x,79a1/1x,'decay before tau =',e10.3/1x,79a1)
      endif
      do n=1,nptl
      radptl(n)=0.
      desptl(n)=0.
      dezptl(n)=0.
      enddo
      np1=1
21    np2=nptl
      do 5 i=np1,np2
      zfi=xorptl(3,i)+(tivptl(2,i)-xorptl(4,i))*pptl(3,i)/pptl(4,i)
      call uttaus(zfi,tzfi,szfi)
      if(tivptl(2,i).le.tzfi.or.iacn.eq.0)call jdecas(i,iret)
      if(iret.eq.1)goto1001
5     continue
      np1=np2+1
      if(np1.le.nptl) goto21
      if(ish.eq.13)call uttima('decay ini      ')

c     interaction and decay 
c     ---------------------
      if(iacn.eq.0)goto5000
      call utpage
           if(ish.ge.91)then
      write(ifch,118)('#',l=1,79),('#',l=1,79)
118   format(1x,79a1/1x,'interactions and decay'/1x,79a1)
           endif
      do 23 nt=1,numtau
      ntc=nt
      dtaus=deltau*factau**(nt-1.)
      ttaus=taumin+deltau*(factau**(nt-1.)-1)/(factau-1)
      if(iojint.eq.1)call jinta1
      if(iojint.eq.2)call jinta2
      ttaus=ttaus+dtaus
      np1=1
36    np2=nptl
      do 37 ip=np1,np2
      zfi=xorptl(3,ip)+(tivptl(2,ip)-xorptl(4,ip))*pptl(3,ip)/pptl(4,ip)
      call uttaus(zfi,tzfi,szfi)
      iclde=0
           if(abs(dezptl(ip)).gt.1e-5.and.istptl(ip).eq.0
     *.and.tivptl(2,ip).gt.tzfi)then
           if(tivptl(2,ip).gt.tzfi*1.000001)then
      call jcheps(ip,x,y,z,t,sz,eps,rho) 
      if(eps.le.epscri.or.nt.eq.numtau)then
      tivptl(2,ip)=t
      iclde=1
      zfi=xorptl(3,ip)+(tivptl(2,ip)-xorptl(4,ip))*pptl(3,ip)/pptl(4,ip)
      call uttaus(zfi,tzfi,szfi)
      if(abs(t-tzfi).gt.1e-5   .and.abs(t-tzfi).gt.1e-5 *abs(t)
     *.or.abs(z-zfi).gt.1e-5   .and.abs(z-zfi).gt.1e-5  *abs(z)
     *.or.abs(sz-szfi).gt.1e-5 .and.abs(sz-szfi).gt.1e-5*abs(sz))then
      call utmsg('jfrade')
      write(ifch,*)'*****  t /= tzfi or z /= zfi or sz /= szfi' 
      write(ifch,*)t,tzfi,z,zfi,sz,szfi 
      call utmsgf
      endif
      endif
           else
      iclde=1       
           endif   
           endif
      if(tivptl(2,ip).le.tzfi.or.iclde.eq.1)then
      call jdecas(ip,iret)
      if(tivptl(2,ip).gt.tzfi-1e-5)tivptl(2,ip)=tzfi-1e-5
      endif
      if(iret.eq.1)goto1001
37    continue
      np1=np2+1
      if(np1.le.nptl)then
      do 4 ip=np1,nptl
4     iaaptl(ip)=1        
      goto36
      endif
      if(ish.eq.13)call uttima('decay          ')
      call utcley(1)
23    continue

c     final decay
c     -----------
      call utpage
      if(ish.ge.91)then
      write(ifch,120)('#',l=1,79),('#',l=1,79)
120   format(1x,79a1/1x,'final decay'/1x,79a1)
      endif
      nbef=nptl
      n0bef=nptl0
      naft=nptl
      n0aft=nptl0
           if(nclean.gt.0)then
      call utclea(nptl0)
      naft=nptl
      n0aft=nptl0
           endif
      ishnpt=ish
      if(ishsub/100.eq.19)ish=mod(ishsub,100)
           if(ish.eq.22)then
      write(ifmt,131)nbef,naft
131   format(1x,'bef fin de: nbef=',i8,4x,'naft=',i8)
           endif
      ish=ishnpt
      np1=1
41    np2=nptl
      do 42 ip=np1,np2
      call jdecas(ip,iret)
      if(iret.eq.1)goto1001
42    continue
      np1=np2+1
      if(np1.le.nptl)then
      do 6 ip=np1,nptl
6     iaaptl(ip)=1
      goto41
      endif
      ishnpt=ish
      if(ishsub/100.eq.19)ish=mod(ishsub,100)
           if(ish.eq.22)then
      write(ifmt,132)nptl      
132   format(1x,'aft fin de: nptl=',i8)                  
           endif
      ish=ishnpt
      if(ish.eq.13)call uttima('decay fin      ')
 
c     print /cptl/
c     ------------
      call utpage
      if(ishsub/100.eq.11)ish=mod(ishsub,100)
      if(ish.ge.91)then
      write(ifch,117)('#',l=1,79),('#',l=1,79)
117   format(1x,79a1/1x
     *,'ptls after performing interactions'/1x,79a1/
     *1x,' ior',' jor',3x,'   n',3x,'ifr1ifr2',10x,'id',3x,
     *5x,'ptr',7x,'pz',4x,' mass ','ist    time')
      do 34 n=1,nptl
      if(n.gt.nptlpt.or.istptl(n).ne.0)
     *write(ifch,116)iorptl(n),jorptl(n),n,ifrptl(1,n),ifrptl(2,n)
     *,idptl(n),sqrt(pptl(1,n)**2+pptl(2,n)**2),pptl(3,n),pptl(5,n)
     *,istptl(n),tivptl(1,n)
34    continue
116   format(1x,i4,i4,3x,i4,3x,i4,i4,i12,3x,3(e8.2,1x),i2,1x,e8.2)
      endif

c     trafo -> lab cm
c     ---------------
5000  continue
      if(labsys.eq.1)then
      do 7 i=1,nptl
      amt=sqrt(pptl(5,i)**2+pptl(1,i)**2+pptl(2,i)**2)
      pz=pptl(3,i)
      e=pptl(4,i)
      yi=sign(1.,pz)*alog((e+abs(pz))/amt)
      y=yi+yhaha
      pptl(3,i)=amt*sinh(y)
7     pptl(4,i)=amt*cosh(y)
      endif
 
c     finish
c     ------

      call utpage
      if(ish.ge.91)write(ifch,113)('#',l=1,79),('#',l=1,79)
113   format(1x,79a1/1x,'particle production finished'/1x,79a1)
      if(ish.eq.13)call uttima('               ')
      goto1000
 
10011 continue
      if(ish.ge.90)then
      call utmsg('jfrade')
      write(ifch,*)'*****  fragmentation not possible'
      write(ifch,112)j,(icstr(k,j)/100,k=1,4)
     *,(pstr(l,j),l=1,5)
112   format( ' str:',3x,i3,3x,3x,4i5,5f7.2,f7.2)
      call utmsgf
      endif
      ier=1
      goto1000
 
1001  continue
      ier=1
      goto1000
 
1000  continue
           if(ier.eq.1)then
      call utmsg('jfrade')
      write(ifmt,'(a)') 'redo event'
      write(ifch,'(a,i9)')' *****  redo event',nrevt+1
      call utmsgf
           endif
      if(ish.eq.14)call uttima('    jfrade f   ')
      ish=ish0

      ish=ishini
      return
      end
 
c----------------------------------------------------------------------
      subroutine jinta1
c----------------------------------------------------------------------
c  secondary interactions
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      write(ifmt,'(a,a)')'subr jinta1 no longer available '
     *,'for version > 5.09'
      stop
      end

c----------------------------------------------------------------------
      subroutine jinta2
c----------------------------------------------------------------------
c  secondary interactions: percolation method
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      integer iacptl(mxptl+10)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter (myptl=1000,nzeta=60)
      common/c4ptl/iyptlx,iyptl(mxptl),weiptl(myptl,nzeta),iyiptl(myptl)
      parameter (nflav=6)
      common/c5ptl/iflptl(myptl,nzeta,nflav)
      parameter (mxifr=mxptl)
      common/cifrij/ifrij(mxifr)
      real xptl(mxptl),yptl(mxptl),zptl(mxptl),tptl(mxptl)
     *,optl(mxptl),uptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt

      common/partnr/summas,iptnr,ic4,px4,py4,js4,pex,pey,pezet,pe0,nps
     */didib/ndidib /cnfr/nrfra
      parameter (mxindx=1000,mxre=100,mxma=11,mxmx=6)
      common/crema/indx(mxindx),rema(mxre,mxma),rewi(mxre,mxma)
     *,idmx(mxma,mxmx),icre1(mxre,mxma),icre2(mxre,mxma)
      parameter (mxlook=10000,mxdky=2000)
      common/dkytab/look(mxlook),cbr(mxdky),mode(5,mxdky)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat /ctimel/ntc
      parameter(nprbms=20)
      common /cprbms/prbms(nprbms) /cnce/nces,ncolex /ckol/kol 
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      parameter (mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      common/cjint/nptl0,rnup,rnut,velp,velt,bx,by,xaver(4)
      integer jci(nflav,2),jc(nflav,2),ic4(2),jcdu(nflav,2)
     *,jcxi(nflav,2),jcxj(nflav,2)
      common/ctfi/tin,tfi

c     initialization for ntc=1
c     ------------------------
           if(ntc.eq.1)then
      nptl0=nptl
      do 51 i=1,nptl0
      iaaptl(i)=1
      call idquac(i,nqi,ndummy,ndummy,jci)
      if(nqi.eq.0)then
      radptl(i)=amax1(radmes,rinmes)                         
      desptl(i)=amax1(radmes,rinmes)
      amiptl(i)=piom+amsiac
      else
      radptl(i)=amax1(radbar,rinbar)
      desptl(i)=amax1(radbar,rinbar)
      amiptl(i)=prom+amsiac
      endif
      dezptl(i)=0          
      iyptl(i)=0          
c    ---shad correction
      amt=sqrt(pptl(5,i)**2+pptl(1,i)**2+pptl(2,i)**2)
      pz=pptl(3,i)
      e=pptl(4,i)
      ya=sign(1.,pz)*alog((e+abs(pz))/amt)
      call uttain(i,x,y,z,t,n,0)
      call uttaus(z,tz,shad)
      ii=0
   53 ii=ii+1
      if(ii.ge.2)ybb=yb
      if(ii.ge.2)shadb=shad
      o=shad+desptl(i)
      u=shad-desptl(i)
      call uttauh(i,u,o,g,yb,ws)
c     write(ifch,'(f6.2,i10,f7.2,i4,3f8.2,f11.2)')
c    *sngl(ttaus),idptl(i),desptl(i),ii,ya,yb,ya-yb,shad/sngl(ttaus)
      shad=shad+(ya-yb)*ttaus
      if(ii.lt.10.and.abs(ya-yb).gt.1e-2.and.
     *( ii.eq.1
     *  .or.ii.gt.1.and.yb.ne.ybb
     *  .or.ii.gt.1.and.yb.eq.ybb.and.abs(shad).lt.abs(shadb) ))goto53 
      shad=shad-(ya-yb)*ttaus
      call uttaui(shad,t,z)
      xorptl(3,i)=z
      xorptl(4,i)=t
      call uttain(i,x,y,z,t,n,0)
      call uttaus(z,tz,shad)
c     write(ifch,'(f7.2)')shad/sngl(ttaus)
c    ---
   51 continue
      iyptlx=0
           endif

c     initialization for each ntc
c     ---------------------------
      nt=ntc
      iyptli=iyptlx      
      nbef=nptl
      n0bef=nptl0
      naft=nptl
      n0aft=nptl0
           if(nclean.gt.0.and.mod(nt-1,nclean).eq.0)then
      call utclea(nptl0)
      naft=nptl
      n0aft=nptl0
           endif
      taus=ttaus
      nptlpt=iabs(maproj)+iabs(matarg)
           if(nptl.gt.nptl0)then
      do 52 i=nptl0+1,nptl
      if(radptl(i).ne.0.)goto52            !not true for decay products
      call idquac(i,nqi,ndummy,ndummy,jci)
      if(nqi.eq.0)then
      radptl(i)=amax1(radmes,rinmes)
      desptl(i)=amax1(radmes,rinmes)
      amiptl(i)=piom+amsiac
      else
      radptl(i)=amax1(radbar,rinbar)
      desptl(i)=amax1(radbar,rinbar)
      amiptl(i)=prom+amsiac
      endif
      dezptl(i)=0
      iyptl(i)=0          
c    ---shad correction
      amt=sqrt(pptl(5,i)**2+pptl(1,i)**2+pptl(2,i)**2)
      pz=pptl(3,i)
      e=pptl(4,i)
      ya=sign(1.,pz)*alog((e+abs(pz))/amt)
      call uttain(i,x,y,z,t,n,0)
      call uttaus(z,tz,shad)
      ii=0
   54 ii=ii+1
      if(ii.ge.2)ybb=yb
      o=shad+desptl(i)
      u=shad-desptl(i)
      call uttauh(i,u,o,g,yb,ws)
c     write(ifch,'(f6.2,i10,f7.2,i4,3f8.2,f11.2)')
c    *sngl(ttaus),idptl(i),desptl(i),ii,ya,yb,ya-yb,shad/sngl(ttaus)
      shad=shad+(ya-yb)*ttaus
      if(ii.lt.10.and.abs(ya-yb).gt.1e-2.and.
     *( ii.eq.1
     *  .or.ii.gt.1.and.yb.ne.ybb
     *  .or.ii.gt.1.and.yb.eq.ybb.and.abs(shad).lt.abs(shadb) ))goto54
      shad=shad-(ya-yb)*ttaus
      call uttaui(shad,t,z)
      xorptl(3,i)=z
      xorptl(4,i)=t
      call uttain(i,x,y,z,t,n,0)
      call uttaus(z,tz,shad)
c     write(ifch,'(f7.2)')shad/sngl(ttaus)
c    ---
   52 continue
           endif
      do 44 i=1,nptl
      iacptl(i)=1
      if(iclptl(i).eq.0.and.pptl(5,i).gt.amiptl(i))iacptl(i)=0
      if(iorptl(i).eq.-1)iacptl(i)=0
      call uttain(i,x,y,z,t,n,0)
      if(n.eq.1.or.n.eq.2.or.n.eq.9)iacptl(i)=0
      if(nclean.gt.0
     *.and.mod(nt,nclean).eq.0.and.n.eq.2)istptl(i)=2
      if(istptl(i).eq.2)iacptl(i)=0
      if(iaaptl(i).eq.0)iacptl(i)=0
      if(iabs(idptl(i)).lt.100.and.iabs(idptl(i)).ne.20)iacptl(i)=0
      if(idptl(i).eq.441.and.jpsifi.eq.0)iacptl(i)=0
      xptl(i)=x
      yptl(i)=y
      zptl(i)=z
      tptl(i)=t
      call uttaus(z,tz,sz)
      if(dezptl(i).ne.0.)desptl(i)=dezptl(i)*ttaus          
      optl(i)=sz+desptl(i)
      uptl(i)=sz-desptl(i)
cc    if(iacptl(i).eq.1)
cc   *write(ifch,'(a,i4,i10,2f8.3,i4,i4,2f10.3,4x,2f10.3)')'i=',i
cc   *,idptl(i),radptl(i),desptl(i),iacptl(i),n
cc   *,amin1(100.,tin),amin1(100.,tfi)
cc   *,amax1(-100.,tivptl(1,i)),amin1(100.,tivptl(2,i))
   44 continue
      nptl0=nptl
      i=0

c     i loop --> 24
c     -------------
9999  i=i+1
      if(iacptl(i).eq.0)goto24
      j0=nptlpt+1
      if(i.gt.nptlpt)j0=i+1
      if(i.gt.nptl0)j0=1
      optli = optl(i)
      uptli = uptl(i)

c     j loop
c     ------
      j=j0-1
   25 j=j+1
      if(j.gt.nptl) goto 24      

      if(iacptl(j).eq.0)then
      if(iacptl(j+1).eq.0)then
      if(iacptl(j+2).eq.0)then
      if(iacptl(j+3).eq.0)then
      if(iacptl(j+4).eq.0)then
      if(iacptl(j+5).eq.0)then
      if(iacptl(j+6).eq.0)then
      if(iacptl(j+7).eq.0)then
      if(iacptl(j+8).eq.0)then
      if(iacptl(j+9).eq.0)then
      if(iacptl(j+10).eq.0)then
      j=j+10
      goto 25
      else
      j=j+10
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+9
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+8
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+7
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+6
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+5
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+4
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+3
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+2
      if(j.gt.nptl) goto 24
      goto 28
      endif
      else
      j=j+1
      if(j.gt.nptl) goto 24
      goto 28
      endif
      endif
   28 continue

      if(optli.lt.uptl(j))goto25
      if(optl(j).lt.uptli)goto25
      radsqr=(radptl(i)+radptl(j))**2
      if((yptl(i)-yptl(j))**2.gt.radsqr)goto25
      if((xptl(i)-xptl(j))**2.gt.radsqr)goto25
      if(i.eq.j)goto25
      if(iorptl(i).gt.0.and.iorptl(j).eq.iorptl(i))goto25
      if(iorptl(i).eq.j)goto25
      if(iorptl(j).eq.i)goto25
      if((xptl(i)-xptl(j))**2+(yptl(i)-yptl(j))**2.gt.radsqr)goto25
      call jintcc(i,j,iret)  !check children
      if(iret.eq.1)goto25                   

      iactn=0
      nptl00=nptl
      nstr00=nstr
      if((pptl(5,i)+pptl(5,j)).ne.0.)then
      wi=pptl(5,i)/(pptl(5,i)+pptl(5,j))
      wj=pptl(5,j)/(pptl(5,i)+pptl(5,j))
      else
      wi=0.5
      wj=0.5
      endif
      xaver(1)=wi*xptl(i)+wj*xptl(j)
      xaver(2)=wi*yptl(i)+wj*yptl(j)    
      xaver(3)=wi*zptl(i)+wj*zptl(j)    
      xaver(4)=wi*tptl(i)+wj*tptl(j)    

      call jintfs(i,j,nqifus,jc,amim,iret)    !fusion (1)        
      if(iret.eq.1)goto25                 

      call jintce(i,j,amim,iactn,iret)        !colour exchange (1)
      if(iret.eq.25)goto25             

      if(dezptl(i).eq.0..and.iyptl(i).eq.0.and.
     *   dezptl(j).eq.0..and.iyptl(j).eq.0)then    !hadron-hadron
      iihaha=1
      rmes=radmes
      rbar=radbar
      else                                         !not hadron-hadron
      iihaha=0
      rmes=rinmes
      rbar=rinbar
      endif
      if(dezptl(i).eq.0..and.iyptl(i).eq.0)then    !i hadron
      call idquac(i,nqi,ndummy,ndummy,jci)
      if(nqi.eq.0)then
      radi=rmes
      desi=rmes
      else
      radi=rbar
      desi=rbar
      endif
      else                                         !i not hadron
      radi=radptl(i)
      desi=desptl(i)
      endif
      if(dezptl(j).eq.0..and.iyptl(j).eq.0)then    !j hadron
      call idquac(j,nqi,ndummy,ndummy,jci)
      if(nqi.eq.0)then
      radj=rmes
      desj=rmes
      else
      radj=rbar
      desj=rbar
      endif
      else                                         !j not hadron
      radj=radptl(j)
      desj=desptl(j)
      endif
      oi=(uptl(i)+optl(i))/2.+desi
      ui=(uptl(i)+optl(i))/2.-desi
      oj=(uptl(j)+optl(j))/2.+desj
      uj=(uptl(j)+optl(j))/2.-desj

      if(oi.lt.uj)goto26
      if(oj.lt.ui)goto26
      radsqr=(radi+radj)**2
      if((yptl(i)-yptl(j))**2.gt.radsqr)goto26
      if((xptl(i)-xptl(j))**2.gt.radsqr)goto26
      if((xptl(i)-xptl(j))**2+(yptl(i)-yptl(j))**2.gt.radsqr)goto26
      goto27
   26 continue
      nptl=nptl00
      nstr=nstr00
      goto25
   27 continue

cc    write(ifch,'(//a,i4,i10,4f9.3,5x,2f9.3)')'i=',i,idptl(i),radptl(i)       
cc   *,desptl(i),uptl(i),optl(i),rinmes,rinbar
cc    write(ifch,'(a,i4,i10,4f9.3)')'j=',j,idptl(j),radptl(j),desptl(j)
cc   *,uptl(j),optl(j)

      optl(i)=oi
      uptl(i)=ui
      radptl(i)=radi
      desptl(i)=desi
      optl(j)=oj
      uptl(j)=uj
      radptl(j)=radj
      desptl(j)=desj

cc    write(ifch,'(a,i4,i10,4f9.3,5x,2f9.3)')'i=',i,idptl(i),radptl(i)
cc   *,desptl(i),uptl(i),optl(i),rinmes,rinbar
cc    write(ifch,'(a,i4,i10,4f9.3)')'j=',j,idptl(j),radptl(j),desptl(j)
cc   *,uptl(j),optl(j)

      tivptl(2,i)=tptl(i)
      tivptl(2,j)=tptl(j)
      istptl(i)=1
      istptl(j)=1
      iacptl(i)=0
      iacptl(j)=0

      call jintch(i,j,kmax)  
      do k=1,kmax
      n=ifrij(k)            ! n: i,j child
      istptl(n)=2
      iacptl(n)=0
      enddo
      call jintpa(i,j,kmax) ! also: iaaptl(n)=0 
      do k=1,kmax
      n=ifrij(k)            ! n: i,j parent
      iacptl(n)=0
      enddo

      if(iactn.eq.1)then                     !colour exchange (2)
      if(ish.ge.91)then
      write(ifch,'(1x,a,i3,a,f6.2)')
     *'>>> co.ex. interaction; nt=',ntc,' taus=',taus
      write(ifch,115)i,idptl(i),(pptl(k,i),k=3,5),desptl(i),radptl(i)       
      write(ifch,115)j,idptl(j),(pptl(k,j),k=3,5),desptl(j),radptl(j)
  115 format(' /cptl/',i6,i10,3x,2(e9.3,1x),e8.3,3x,2(e9.3,1x),2x,i4)       
      endif
      do n=nptl00+1,nptl
      iaaptl(n)=1
      call idquac(n,nqi,ndummy,ndummy,jcdu)
      if(nqi.eq.0)then
      radptl(n)=amax1(radmes,rinmes)
      desptl(n)=amax1(radmes,rinmes)
      amiptl(n)=piom+amsiac
      else
      radptl(n)=amax1(radbar,rinbar)
      desptl(n)=amax1(radbar,rinbar)
      amiptl(n)=prom+amsiac
      endif
      dezptl(n)=0          
      iyptl(n)=0          
      iacptl(n)=1
      if(pptl(5,n).gt.amiptl(n))iacptl(n)=0
      call uttain(n,x,y,z,t,k,0)
      if(k.eq.1.or.k.eq.2.or.k.eq.9)iacptl(n)=0
      if(iabs(idptl(n)).lt.100.and.iabs(idptl(n)).ne.20)iacptl(n)=0
      xptl(n)=x
      yptl(n)=y
      zptl(n)=z
      tptl(n)=t
      call uttaus(z,tz,sz)
      optl(n)=sz+desptl(n)
      uptl(n)=sz-desptl(n)
      if(ish.ge.91)
     *write(ifch,115)n,idptl(n),(pptl(k,n),k=3,5),desptl(n),radptl(n)
      enddo
      goto24                   
      endif

      call jintel(i,j,amim,iactn)            !elastic scattering
      if(iactn.eq.2)then
      if(ish.ge.91)then
      write(ifch,'(1x,a,i3,a,f6.2)')
     *'>>> elastic scattering; nt=',ntc,' taus=',sngl(ttaus)
      write(ifch,115)i,idptl(i),(pptl(k,i),k=3,5),desptl(i),radptl(i)       
      write(ifch,115)j,idptl(j),(pptl(k,j),k=3,5),desptl(j),radptl(j)
      endif
      do n=nptl00+1,nptl
      if(n.eq.nptl00+1)ij=i
      if(n.eq.nptl00+2)ij=j
      radptl(n)=radptl(ij)
      desptl(n)=desptl(ij)
      dezptl(n)=dezptl(ij) 
      iyptl(n)=iyptl(ij)          
      amiptl(n)=amiptl(ij)
      xptl(n)=xaver(1)
      yptl(n)=xaver(2)
      zptl(n)=xaver(3)                    
      tptl(n)=xaver(4)
      z=zptl(n)
      call uttaus(z,tz,sz)
      optl(n)=sz+desptl(n)
      uptl(n)=sz-desptl(n)
      iacptl(n)=1
      iaaptl(n)=1
      iy=iyptl(ij)
      iyptl(n)=iy
      if(iy.gt.0)iyiptl(iy)=n
      if(ish.ge.91)
     *write(ifch,115)n,idptl(n),(pptl(k,n),k=3,5),desptl(n),radptl(n)
      enddo
      goto24
      endif
                                             ! fusion (2)
      if(ish.ge.91)then
      write(ifch,'(1x,a,i3,a,f6.2)')
     *'>>> fusion interaction; nt=',ntc,' taus=',sngl(ttaus)
      write(ifch,115)i,idptl(i),(pptl(k,i),k=3,5),desptl(i),radptl(i)       
      write(ifch,115)j,idptl(j),(pptl(k,j),k=3,5),desptl(j),radptl(j)
      endif
      call jintfu(i,j,jc,iactn)                                    
      if(nptl00+1.ne.nptl)
     *call utstop('jinta2: nptl00+1.ne.nptl                ')
      n=nptl
      call jintov(
     *xptl(i),yptl(i),radptl(i),optl(i),uptl(i),
     *xptl(j),yptl(j),radptl(j),optl(j),uptl(j),dov,fov,vov,vfu)
      if(nqifus.eq.0)then
      amiptl(n)=piom+amsiac
      rad=radmes
      rep=rinmes
      else
      amiptl(n)=prom+amsiac
      rad=radbar
      rep=rinbar
      endif
      optl(n)=amax1( optl(i) , optl(j) )
      uptl(n)=amin1( uptl(i) , uptl(j) )
      uo2=(uptl(n)+optl(n))/2
           if(iabs(idptl(n)).le.10000)then   !no droplet
      desptl(n)=amax1(rad,rep)
      radptl(n)=amax1(rad,rep)
      optl(n)=uo2+desptl(n)
      uptl(n)=uo2-desptl(n)
      dezptl(n)=0.
      iyptl(n)=0          
           else                              !droplet
      if(iihaha.eq.1)then
      desptl(n)=rep*1.5**(1./3.)
      radptl(n)=rep*1.5**(1./3.)
      optl(n)=uo2+desptl(n)
      uptl(n)=uo2-desptl(n)
      dezptl(n)=desptl(n)/ttaus          
      iyptl(n)=0          
      else
      desptl(n)=(optl(n)-uptl(n))/2
      dezptl(n)=desptl(n)/ttaus          
      iyptl(n)=0          
      radptl(n)=sqrt( vfu/(2*desptl(n)*pi) )
      endif
           if(2*desptl(n).gt.corlen)then     !long droplet
      if(iyptlx+1.gt.myptl)call utcley(1)
      iyptlx=iyptlx+1
      if(iyptlx.gt.myptl)
     *call utstop('jinta2: myptl too small (4)             ')
      iyptl(n)=iyptlx
      iyiptl(iyptlx)=n
      zi=(optl(i)+uptl(i))/2/ttaus
      zj=(optl(j)+uptl(j))/2/ttaus
      ni=zi/dlzeta+nzeta/2+1
      nj=zj/dlzeta+nzeta/2+1
      if(ni.lt.1.or.ni.gt.nzeta)                            
     *call utstop('jinta2: nzeta too small (1)             ')
      if(nj.lt.1.or.nj.gt.nzeta)                            
     *call utstop('jinta2: nzeta too small (2)             ')
      iyi=0
      if(dezptl(i).ne.0..and.iyptl(i).ne.0)iyi=iyptl(i)
      iyj=0
      if(dezptl(j).ne.0..and.iyptl(j).ne.0)iyj=iyptl(j)
      do nz=1,nzeta
      weiptl(iyptlx,nz)=0
      do nf=1,nflav
      iflptl(iyptlx,nz,nf)=0
      enddo
      enddo
      call idquac(i,nqdum,nsdum,nadum,jcxi)
      if(iyi.ne.0)then
      do nz=1,nzeta
      weiptl(iyptlx,nz)=weiptl(iyptlx,nz)+weiptl(iyi,nz)
      do nf=1,nflav
      iflptl(iyptlx,nz,nf)=iflptl(iyptlx,nz,nf)+iflptl(iyi,nz,nf)
      enddo
      enddo
      else
      call uttaug(uptl(i),optl(i),gi,dum)
      weiptl(iyptlx,ni)=weiptl(iyptlx,ni)+pptl(5,i)/gi
      do nf=1,nflav
      iflptl(iyptlx,ni,nf)=iflptl(iyptlx,ni,nf)+jcxi(nf,1)-jcxi(nf,2)
      enddo
      endif
      call idquac(j,nqdum,nsdum,nadum,jcxj)
      if(iyj.ne.0)then
      do nz=1,nzeta
      weiptl(iyptlx,nz)=weiptl(iyptlx,nz)+weiptl(iyj,nz)
      do nf=1,nflav
      iflptl(iyptlx,nz,nf)=iflptl(iyptlx,nz,nf)+iflptl(iyj,nz,nf)
      enddo
      enddo          
      else
      call uttaug(uptl(j),optl(j),gj,dum)
      weiptl(iyptlx,nj)=weiptl(iyptlx,nj)+pptl(5,j)/gj
      do nf=1,nflav
      iflptl(iyptlx,nj,nf)=iflptl(iyptlx,nj,nf)+jcxj(nf,1)-jcxj(nf,2)
      enddo
      endif
           if(ish.ge.92)then   
      write(ifch,*)'check fusion:'
      write(ifch,*)'tau:',sngl(ttaus)
      write(ifch,122)i,idptl(i),jcxi,pptl(5,i)
     *,j,idptl(j),jcxj,pptl(5,j),n,idptl(n),jc,pptl(5,n)
122   format(
     * 1x,'i   nptl:',i6,'   id:',i9,'   jc:',6i2,2x,6i3,'   m:',f5.1
     */1x,'j   nptl:',i6,'   id:',i9,'   jc:',6i2,2x,6i3,'   m:',f5.1
     */1x,'n   nptl:',i6,'   id:',i9,'   jc:',6i2,2x,6i3,'   m:',f5.1)
      write(ifch,*)
     *' n    iy  wei   u d s    iy  wei   u d s    iy  wei   u d s'
      do nn=nzeta/2-9,nzeta/2+10
      iyi=iyptl(i)
       if(iyi.eq.0)then
      if(nn.ne.ni)then
      weii=0
      kiu=0
      kid=0
      kis=0
      kic=0
      else
      weii=pptl(5,i)/gi
      kiu=jcxi(1,1)-jcxi(1,2)
      kid=jcxi(2,1)-jcxi(2,2)
      kis=jcxi(3,1)-jcxi(3,2)
      kic=jcxi(4,1)-jcxi(4,2)
      endif
       else
      weii=weiptl(iyi,nn)
      kiu=iflptl(iyi,nn,1)
      kid=iflptl(iyi,nn,2)
      kis=iflptl(iyi,nn,3)
      kic=iflptl(iyi,nn,4)
       endif
      iyj=iyptl(j)
       if(iyj.eq.0)then
      if(nn.ne.nj)then
      weij=0
      kju=0
      kjd=0
      kjs=0
      kjc=0
      else
      weij=pptl(5,j)/gj
      kju=jcxj(1,1)-jcxj(1,2)
      kjd=jcxj(2,1)-jcxj(2,2)
      kjs=jcxj(3,1)-jcxj(3,2)
      kjc=jcxj(4,1)-jcxj(4,2)
      endif
       else
      weij=weiptl(iyj,nn)
      kju=iflptl(iyj,nn,1)
      kjd=iflptl(iyj,nn,2)
      kjs=iflptl(iyj,nn,3)
      kjc=iflptl(iyj,nn,4)
       endif
      iyn=iyptl(n)
      wein=0
      knu=0
      knd=0
      kns=0
      knc=0
      if(iyn.gt.0)then
      wein=weiptl(iyn,nn)
      knu=iflptl(iyn,nn,1)
      knd=iflptl(iyn,nn,2)
      kns=iflptl(iyn,nn,3)
      knc=iflptl(iyn,nn,4)
      endif
      write(ifch,135)nn,iyi,weii,kiu,kid,kis,iyj,weij,kju,kjd,kjs
     *,iyn,wein,knu,knd,kns
135   format(1x,i2,3(i6,f6.2,1x,3i2))
      enddo
           endif 
      iyn=iyptl(n)
      do nf=1,nflav
      itst=0
      do nz=1,nzeta
      itst=itst+iflptl(iyn,nz,nf)
      enddo
      if(itst.ne.jc(nf,1)-jc(nf,2))then
      write(ifch,121)n,idptl(n),jc
121   format(1x,'nptl:',i10/1x,'id:',i10/1x,'jc:',3x,6i4,8x,6i4)
      write(ifch,*)'nz iflptl(,nz,1-6):'
      do nz=1,nzeta
      write(ifch,106)nz,weiptl(iyn,nz),(iflptl(iyn,nz,nfx),nfx=1,6)
106   format(1x,i6,f6.2,1x,6i2)
      enddo
      call utstop('jinta2: sum iflptl /= jc(,1)-jc(,2)     ')
      endif                                                  
      enddo
           endif                      !long droplet
           endif                      !droplet
      s=(optl(n)+uptl(n))/2
      call uttaui(s,t,z)
      xptl(n)=xaver(1)
      yptl(n)=xaver(2)
      zptl(n)=z                    
      tptl(n)=t
      xorptl(1,n)=xptl(n)
      xorptl(2,n)=yptl(n)
      xorptl(3,n)=zptl(n)
      xorptl(4,n)=tptl(n)
      tivptl(1,n)=tptl(n) 
           if(dezptl(n).ne.0.)then
      taugm=pptl(4,n)/pptl(5,n)*ainfin
           else
      call idtau(idptl(n),pptl(4,n),pptl(5,n),taugm)
           endif
      tivptl(2,n)=tivptl(1,n)+taugm*(-alog(rangen()))
      iacptl(n)=1
      iaaptl(n)=1
           if(ish.eq.23.and.pptl(5,nptl).ge.amprif)then
      call jintfp(i,j,n,
     *xptl(i),yptl(i),radptl(i),optl(i),uptl(i),
     *xptl(j),yptl(j),radptl(j),optl(j),uptl(j),
     *xptl(n),yptl(n),radptl(n),optl(n),uptl(n),dov,fov,vov)
           endif
      if(ish.ge.91)
     *write(ifch,115)n,idptl(n),(pptl(k,n),k=3,5),desptl(n),radptl(n)
 
   24 continue

      if(i.lt.nptl-1)goto9999

      if(ish.eq.24)then
      if(taus.eq.1..or.taus.eq.2..or.taus.eq.4.)then
      do 45 i=1,nptl
      if(iacptl(i).eq.0.and.i.ne.nptl)goto45
      call jintcl(i,xptl(i),yptl(i),radptl(i),optl(i),uptl(i),iacptl(i))
45    continue
      endif
      endif

      if(idensi.eq.1)then
      if(mod(ntc-1,5).eq.0)then
      itau=1+(ntc-1)/5
      do i=1,nptl
      if(iacptl(i).ne.0.and.iabs(idptl(i)).gt.10000)
     *call xdense(1,itau,i,xptl(i),yptl(i),radptl(i),optl(i),uptl(i))
      enddo
      endif
      endif

      ishnpt=ish
      if(ishsub/100.eq.19)ish=mod(ishsub,100)
      if(ish.eq.22)then
      write(ifmt,131)nt,nbef,naft,nptl
131   format(1x,'nt=',i5,4x,'nbef=',i8,4x,'naft=',i8,4x,'nptl=',i8)
      endif
      ish=ishnpt
      if(ish.eq.13)call uttima('interactions   ')
      return
      end

c---------------------------------------------------------------------
      subroutine jintcc(i,j,iret)
c---------------------------------------------------------------------
c  iret=1 if i = child of j
c  iret=1 if j = child of i
c  iret=0 else
c---------------------------------------------------------------------
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter (mxifr=mxptl)
      integer ifr(mxifr)

      iret=0    
           do kk=1,2
      k1=0
      k2=0
      if(kk.eq.1)n=i
      if(kk.eq.2)n=j
      if(kk.eq.1)m=j
      if(kk.eq.2)m=i
      loop=0
55    loop=loop+1
           if(ifrptl(1,n).gt.0)then
      do 56 k=ifrptl(1,n),ifrptl(2,n)
      if(k.eq.m)goto1001
      k2=k2+1
      if(k2.gt.mxifr)
     *call utstop('jintcc: k2 > mxifr                      ')
56    ifr(k2)=k
           endif
      k1=k1+1
      if(k1.le.k2)then
      n=ifr(k1)
      goto55
      endif
           enddo
      goto1000

1001  iret=1

1000  return
      end

c----------------------------------------------------------------------
      subroutine jintce(i,j,amim,iactn,iret)
c----------------------------------------------------------------------
c  colour exchange interaction of ptls i,j
c  input:
c  i,j: ptl numbers; amf: mass, amim: min mass, of fused obj 
c  iactn=1: ce done
c  iret=0: ce done  or  criteria for ce not fulfilled
c  iret=25: skip
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt

      common/partnr/summas,iptnr,ic4,px4,py4,js4,pex,pey,pezet,pe0,nps
     */didib/ndidib /cnfr/nrfra
      parameter (mxindx=1000,mxre=100,mxma=11,mxmx=6)
      common/crema/indx(mxindx),rema(mxre,mxma),rewi(mxre,mxma)
     *,idmx(mxma,mxmx),icre1(mxre,mxma),icre2(mxre,mxma)
      parameter (mxlook=10000,mxdky=2000)
      common/dkytab/look(mxlook),cbr(mxdky),mode(5,mxdky)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat /ctimel/ntc
      parameter(nprbms=20)
      common /cprbms/prbms(nprbms) /cnce/nces,ncolex /ckol/kol 
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      parameter (mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      common/cjint/nptl0,rnup,rnut,velp,velt,bx,by,xaver(4)

      real psum(5)
      parameter (nflav=6)
      integer ic(2),ic4(2)
      parameter (mxifr=mxptl)
      real proj(nsi,nha),targ(nsi,nha)
      double precision arp(5),arm(5),arq(5),rot(3),boo(5)

c     initialization
c     --------------      
      iret=0
      delrap=1.5
      delamf=1.0
      ish00=ish
      pnllx0=pnllx
      taus=ttaus
      amf=pptl(5,nptl+1)

c     check whether ce criteria fulfilled
c     -----------------------------------
      vei=pptl(3,i)/pptl(4,i)
      vej=pptl(3,j)/pptl(4,j)
      rai=0.
      raj=0.
      if(abs(vei).lt.1..and.abs(vej).lt.1.)then
      rai=0.5*alog((1+vei)/(1-vei))
      raj=0.5*alog((1+vej)/(1-vej))
      endif
      if(.not.(amf.gt.amim+delamf.and.abs(rai-raj).gt.delrap
     *.and.iabs(idptl(i)).lt.10000
     *.and.iabs(idptl(j)).lt.10000))goto1000
      iactn=1

c     print
c     -----
      if(ish.ge.92)write(ifch,'(1x,a,i3,a,f6.2)')
     *'>>> co.ex. interaction; nt=',ntc,' taus=',taus
      if(ish.ge.92)write(ifch,115)i,idptl(i)
     *,(pptl(k,i),k=3,5),desptl(i),radptl(i)
      if(ish.ge.92)write(ifch,115)j,idptl(j)
     *,(pptl(k,j),k=3,5),desptl(j),radptl(j)
      if(ish.ge.92)write(ifch,*)'amf,amim: ',amf,amim
      if(ish.ge.92)write(ifch,*)'rai,raj:  ',rai,raj
115   format(' /cptl/',i6,i10,3x,2(e9.3,1x),e8.3,3x,2(e9.3,1x),2x,i4)       

c     initialization      
c     --------------
      nptl00=nptl
      nstr00=nstr

      iproj=1          !this leads to iorstr()=-1
      itarg=1          !
      do l=1,nha       !
      kproja(l,1)=1    !
      ktarga(l,1)=1    !
      enddo            !
      kol=1            !
      coord(1,1)=xaver(1)    ! to determine xorstr()
      coord(2,1)=xaver(2)    !
      coord(3,1)=xaver(3)    !
      coord(4,1)=xaver(4)    !

      idp=idptl(i)
      idm=idptl(j)
      do l=1,5
      arp(l)=pptl(l,i)
      arm(l)=pptl(l,j)
      enddo
      do l=1,4
      boo(l)=arp(l)+arm(l)
      enddo
      boo(5)=boo(4)**2-boo(3)**2-boo(2)**2-boo(1)**2
      s=boo(5)
        if(boo(5).gt.0.)then
      boo(5)=sqrt(boo(5))
        else
      if(ish.ge. 0)then
      call utmsg('jintce')
      write(ifch,*)'*****  boo(5) .le.0'
      write(ifch,*)boo
      call utmsgf
      endif
      goto10025
        endif

c     boosts into i-j cm
c     ------------------
      call utlob2(1,boo(1),boo(2),boo(3),boo(4),boo(5)    
     *,arp(1),arp(2),arp(3),arp(4))
      call utlob2(1,boo(1),boo(2),boo(3),boo(4),boo(5)    
     *,arm(1),arm(2),arm(3),arm(4))
      if(arp(3).lt.0.)then
      idq=idm
      idm=idp
      idp=idq
      do l=1,5
      arq(l)=arm(l)
      arm(l)=arp(l)
      arp(l)=arq(l)
      enddo
      endif
      pnllx=arp(3)
      rot(1)=(arp(1)-arm(1))/2
      rot(2)=(arp(2)-arm(2))/2
      rot(3)=(arp(3)-arm(3))/2
      call utrot2(1,rot(1),rot(2),rot(3)
     *,arp(1),arp(2),arp(3))
      call utrot2(1,rot(1),rot(2),rot(3)
     *,arm(1),arm(2),arm(3))

c     checks
c     ------
           if(arp(3).lt.0)then
      if(ish.ge. 0)then
      call utmsg('jintce')
      write(ifch,*)'*****  z-comp of +jet < 0.'
      write(ifch,*)(sngl(arp(l)),l=1,4)
      write(ifch,*)(sngl(arm(l)),l=1,4)
      call utmsgf
      endif
      goto10025
           endif
           if(abs(sngl(arp(4)+arm(4))-sqrt(s)).gt.
     *2.e-2*sqrt(s))then
      if(ish.ge. 0)then
      call utmsg('jintce')
      write(ifch,*)'*****  arp(4)+arm(4)-sqrt(s) nonzero'
      write(ifch,*)'value:   ',sngl(arp(4)+arm(4))-sqrt(s)
      write(ifch,*)'sqrt(s): ',sqrt(s)
      write(ifch,*)(sngl(arp(i)),i=1,4)
      write(ifch,*)(sngl(arm(i)),i=1,4)
      call utmsgf
      endif
           endif

c     fill proj, targ
c     ---------------
      call idtr4(idp,ic)
      do m=1,nha
      do n=1,nsi
      proj(n,m)=0
      enddo
      enddo
      proj(5,1)=ic(1)
      proj(6,1)=ic(2)
      proj(5,2)=ic(1)
      proj(6,2)=ic(2)
      do l=1,4
      proj(l,2)=arp(l)
      enddo
      call idtr4(idm,ic)
      do m=1,nha
      do n=1,nsi
      targ(n,m)=0
      enddo
      enddo
      targ(5,1)=ic(1)
      targ(6,1)=ic(2)
      targ(5,2)=ic(1)
      targ(6,2)=ic(2)
      do l=1,4
      targ(l,2)=arm(l)
      enddo

c     redo
c     ----
      call utremb(proj,targ,1)
      loo=1
      goto58
57    continue
      loo=loo+1
      if(loo.gt.20)goto10025
      if(ish.ge.92)write(ifch,*)'redo hh collision'
      call utrest(proj,targ,1)
58    continue
 
c     number of ce's
c     --------------
      ncolex=1
c-c   lo=0
c-c16 lo=lo+1
c-c        if(lo.eq.3)then
c-c   if(ish.ge.90)then
c-c   call utmsg('jintce')
c-c   write(ifch,*)'*****  lo=3'
c-c   call utmsgf
c-c   endif
c-c        endif
c-c   r=rangen()
c-c   ncolex=0
c-c15 ncolex=ncolex+1
c-c   if(ncolex.gt.nprbms)goto16
c-c   if(r.gt.prbms(ncolex))goto15
 
c     do ncolex ce's
c     --------------
      iskip=0
      do nce=1,ncolex
      if(ish.ge.92.and.nce.gt.1)write(ifch,*)nce,'. colour exchange'
      nces=nce
      ish=ish-2
      call hahabs(proj,targ,nce/ncolex,nce/ncolex,iskip,irethh)
      ish=ish+2
      if(iskip.ne.0)goto10025
      if(irethh.eq.1)goto57
      enddo

c     fragmentation
c     -------------
      if(nstr.le.nstr00)goto10050

      if(ish.ge.92)write(ifch,123)
123   format(1x,'strings before rescaling')
      do l=nstr00+1,nstr
      if(ish.ge.92)write(ifch,109)l,(icstr(k,l),k=1,4)
     *,sqrt(pstr(1,l)**2+pstr(2,l)**2),pstr(3,l),pstr(5,l)
     *,irlstr(l)
      enddo
109   format(' /cstr/',i4,3x,4i7,2x,3(1x,e9.3),i4)
 
      if(irescl.eq.1)then
      psum(1)=0
      psum(2)=0
      psum(3)=0
      psum(4)=sqrt(s)
      psum(5)=sqrt(s)
      ish=ish-2
      call hrescl(nstr00+1,nstr,psum,ifail)
      ish=ish+2
      if(ifail.ne.0)goto57
      endif
 
      if(ish.ge.92)write(ifch,124)
124   format(1x,'strings after rescaling')
      do l=nstr00+1,nstr
      if(ish.ge.92)write(ifch,109)l,(icstr(k,l)/100,k=1,4)
     *,sqrt(pstr(1,l)**2+pstr(2,l)**2),pstr(3,l),pstr(5,l)
     *,irlstr(l)
      enddo
  
      do l=nstr00+1,nstr
      if(ish.ge.92)write(ifch,122)l
     *,(icstr(k,l),k=1,4)
     *,sqrt(pstr(1,l)**2+pstr(2,l)**2),pstr(3,l),pstr(5,l)
122   format('str:',i7,4i7,1x,3(e8.2,1x))
      ish=ish-2
      call jamfra(l,newevt)
      ish=ish+2
      if(newevt.eq.1)goto10025
      enddo

c     boost particles from string fragmentation
c     -----------------------------------------
      if(nptl.le.nptl00)goto10051

      iorptl(nptl00+1)=i
      jorptl(nptl00+1)=j
      do l=1,5
      psum(l)=0
      enddo

           do n=nptl00+1,nptl

      do l=1,5
      arp(l)=pptl(l,n)
      enddo
      call utrot2(-1,rot(1),rot(2),rot(3)
     *,arp(1),arp(2),arp(3))
      call utlob2(-1,boo(1),boo(2),boo(3),boo(4),boo(5)    
     *,arp(1),arp(2),arp(3),arp(4))
      do l=1,5
      pptl(l,n)=arp(l)
      enddo

      do l=1,4
      if(istptl(n).eq.0)psum(l)=psum(l)+pptl(l,n)
      enddo

           enddo

      if(ish.ge.92)then
      write(ifch,*)'p_fin:',(psum(k),k=1,4)
      write(ifch,*)'p_ini:',(sngl(boo(k)),k=1,4)
      endif
               
      if(ish.ge.90.and.
     *   (abs(boo(1)-psum(1)).gt.5.e-3*abs(boo(1))
     *.or.abs(boo(2)-psum(2)).gt.5.e-3*abs(boo(2))
     *.or.abs(boo(3)-psum(3)).gt.5.e-3*abs(boo(3))
     *.or.abs(boo(4)-psum(4)).gt.1.e-1*abs(boo(4)))
     *.and.
     *   (abs(boo(1)-psum(1)).gt.5.e-3
     *.or.abs(boo(2)-psum(2)).gt.5.e-3
     *.or.abs(boo(3)-psum(3)).gt.5.e-3
     *.or.abs(boo(4)-psum(4)).gt.1.e-1))then
      call utmsg('jintce')
      write(ifch,*)'*****  p_ini /= p_fin'
      write(ifch,*)'final particles:'
      do n=nptl00+1,nptl
      write(ifch,125)n,idptl(n),(pptl(k,n),k=1,5),istptl(n)                                   
125   format(1x,'/cptl/',i6,i10,3x,5(e9.3,1x),i2)
      enddo 
      write(ifch,*)'p_fin:',(psum(k),k=1,4)
      write(ifch,*)'p_ini:',(sngl(boo(k)),k=1,4)
      call utmsgf
      endif
      goto1000

c     finish
c     ------

10025 continue
      iret=25
      if(ish.ge.91)write(ifch,*)'skip'
      nptl=nptl00
      nstr=nstr00
      goto1000

10050 continue      
      call utmsg('jintce')
      write(ifch,*)'incident particles:'
      write(ifch,115)i,idptl(i),(pptl(k,i),k=3,5)      
      write(ifch,115)j,idptl(j),(pptl(k,j),k=3,5)
      write(ifch,*)'nstr=',nstr,'    nstr00=',nstr00
      call utmsgf
      call utstop('jintce: nstr .le. nstr00                ')

10051 continue      
      call utmsg('jintce')
      write(ifch,*)'incident particles:'
      write(ifch,115)i,idptl(i),(pptl(k,i),k=3,5)      
      write(ifch,115)j,idptl(j),(pptl(k,j),k=3,5)
      write(ifch,*)'nstr=',nstr,'    nstr00=',nstr00
      write(ifch,*)'nptl=',nptl,'    nptl00=',nptl00
      call utmsgf
      call utstop('jintce: nptl .le. nptl00                ')
  
1000  continue
      pnllx=pnllx0
      ish=ish00
      return
      end

c----------------------------------------------------------------------
      subroutine jintch(i,j,kmax)
c----------------------------------------------------------------------
c  writes children of i,j to ifrij(1-kmax)
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (mxifr=mxptl)
      common/cifrij/ifrij(mxifr)

      kcur=0
      kmax=0
      loop=0
31    loop=loop+1
      if(loop.eq.1)n=i
      if(loop.eq.2)n=j
           if(ifrptl(1,n).gt.0)then
      if(ish.ge.93)write(ifch,*)n,' ---> ',ifrptl(1,n),ifrptl(2,n)
      do 30 k=ifrptl(1,n),ifrptl(2,n)
      kmax=kmax+1
      if(kmax.gt.mxifr)
     *call utstop('jintch: kmax > mxifr                    ')
30    ifrij(kmax)=k
           endif
      if(loop.eq.1)goto31
      kcur=kcur+1
      if(kcur.le.kmax)then
      n=ifrij(kcur)
      goto31
      endif

      return
      end

c---------------------------------------------------------------------
      subroutine jintcl(i,x,y,rad,o,u,iac)
c---------------------------------------------------------------------
c  fills histogram concerning droplet characteristics
c---------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cvsn/iversn,iverso

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)

      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 
      parameter (mxtau=4,mxvol=10,mxeps=16)
      common/cjintc/clust(mxtau,mxvol,mxeps)
      common/cjintd/volsum(mxtau),vo2sum(mxtau),nclsum(mxtau)

      character ax*2

      tau=sngl(ttaus)
      itau=1+alog(tau)/alog(2.)

      dlvol=delvol*ttaus
      dleps=deleps/ttaus

      if(iac.eq.0)goto1

      vol=(o-u)*rad**2*pi
      ivol=1+vol/dlvol
      call uttauh(i,u,o,g,dum,ws)
      eps=pptl(5,i)/g/vol*(o-u)*ws
      ieps=1+eps/dleps
      if(itau.ge.1.and.itau.le.mxtau
     *.and.ivol.ge.1.and.ivol.le.mxvol
     *.and.ieps.ge.1.and.ieps.le.mxeps)
     *clust(itau,ivol,ieps)=clust(itau,ivol,ieps)+1
      nclsum(itau)=nclsum(itau)+1
      volsum(itau)=volsum(itau)+vol
      vo2sum(itau)=vo2sum(itau)+vol**2

1     continue
      if(i.lt.nptl.or.nrevt+1.lt.nevent)goto1000

      volave=0
      if(nclsum(itau).ne.0)volave=volsum(itau)/nclsum(itau)
      vo2ave=0
      if(nclsum(itau).ne.0)vo2ave=vo2sum(itau)/nclsum(itau)
      d2vol=vo2ave-volave**2

      if(itau.eq.1)write(ifhi,105)maproj,matarg,engy,iversn/100.
105   format(1x,'proj=',i3,3x,'targ=',i3,3x,'engy=',f7.2
     *,3x,'VENUS ',f4.2)       
      write(ifhi,100)tau,nevent,dlvol,dleps,volave,d2vol
100   format(1x,'tau=',f5.2,'  nevent=',i4,'  dvol=',f5.1
     *,'  deps=',f5.2,'  <vol>=',f6.1,'  <d2vol>=',f8.1)
c     write(ifhi,101)
c101  format(9x,'ivol=1 ivol=2 ivol=3 ivol=4 ivol=5 '
c    *,'ivol=6 ivol=7 ivol=8 ivol=9 ivol=10 ')
      do ie=1,mxeps
      if(ie.le.9)write(ax,102)ie
102   format('0',i1)
      if(ie.gt.9)write(ax,103)ie
103   format(i2)
c     write(ifhi,104)ax
      write(ifhi,104)
     *(nint(clust(itau,iv,ie)),iv=1,mxvol)
 104  format('      ',2x,10i7)
c104  format(' ieps=',a2,10i7)
      enddo

1000  continue
      return
      end

c----------------------------------------------------------------------
      subroutine jintel(i,j,amim,iactn)
c----------------------------------------------------------------------
c  elastic scattering of ptls i,j
c  el scatt requir not fulf: return without action, iactn unchanged
c    else: elastic scattering done, iactn=2
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      common/cjint/nptl0,rnup,rnut,velp,velt,bx,by,xaver(4)

      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat /ctimel/ntc

      parameter (nflav=6)
      real p(5),u(5),pei(5),pej(5)

c     check
c     -----
      amf=pptl(5,nptl+1)
      if(amf.ge.amim)goto1000

c     print
c     -----
      if(ish.ge.92)then
      write(ifch,'(1x,a,i3,a,f6.2)')
     *'>>> elastic scattering; nt=',ntc,' taus=',sngl(ttaus)
      write(ifch,115)i,idptl(i)
     *,(pptl(k,i),k=3,5),desptl(i),radptl(i)
      write(ifch,115)j,idptl(j)
     *,(pptl(k,j),k=3,5),desptl(j),radptl(j)
115   format(' /cptl/',i6,i10,3x,2(e9.3,1x),e8.3,3x,2(e9.3,1x),2x,i4)
      endif

c     initialization
c     --------------
      iactn=2
      do k=1,5
      p(k)=pptl(k,nptl+1)
      enddo

c     determine momenta of outgoing particles (pei,pej)
c     -------------------------------------------------
           if(p(5).le.(pptl(5,i)+pptl(5,j))*.99)then
      if(ish.ge.90)then
      call utmsg('jintel')
      write(ifch,132)p(5),pptl(5,i)+pptl(5,j)
132   format(1x,'*****  m_fus < m_i+m_j ---> qcm set zero    ( '
     *,2f10.3,' )')
      write(ifch,133)'p_i:  ',(pptl(k,i),k=1,5)
      write(ifch,133)'p_j:  ',(pptl(k,j),k=1,5)
      write(ifch,133)'p_fus:',(p(k),k=1,5)
133   format(1x,a6,3x,5f10.3) 
      call utmsgf
      endif
      qcm=0.
           elseif(p(5).le.pptl(5,i)+pptl(5,j))then
      qcm=0.
           else
      qcm=utpcm(p(5),pptl(5,i),pptl(5,j))
           endif

c isotropic
      u(3)=2.*rangen()-1.
      phi=2.*pi*rangen()
      u(1)=sqrt(1.-u(3)**2)*cos(phi)
      u(2)=sqrt(1.-u(3)**2)*sin(phi)
      do 47 k=1,3
      pei(k)= qcm*u(k)
47    pej(k)=-qcm*u(k)

c nonisotropic
c-c   pt=utinvt(nptq,xptq,qptq,rangen()*qptq(nptq))
c-c   if(pt.ge.qcm)pt=rangen()*qcm
c-c   qpl=sqrt(qcm**2-pt**2)
c-c   u(3)=qpl
c-c   phi=2.*pi*rangen()
c-c   u(1)=pt*cos(phi)
c-c   u(2)=pt*sin(phi)
c-c   call utaxis(i,j,a1,a2,a3)
c-c   ivt=1
c-c   if(a3.lt.0.)then
c-c   a1=-a1
c-c   a2=-a2
c-c   a3=-a3
c-c   ivt=-1
c-c   endif 
c-c   call utrota(-1,a1,a2,a3,u(1),u(2),u(3))
c-c   do 47 k=1,3
c-c   pei(k)= u(k)*ivt
c-c47 pej(k)=-u(k)*ivt

      pei(4)=sqrt(qcm**2+pptl(5,i)**2)
      pej(4)=sqrt(qcm**2+pptl(5,j)**2)
      pei(5)=pptl(5,i)
      pej(5)=pptl(5,j)
      call utlobo(-1,p(1),p(2),p(3),p(4),p(5)
     *,pei(1),pei(2),pei(3),pei(4))
      call utlobo(-1,p(1),p(2),p(3),p(4),p(5)
     *,pej(1),pej(2),pej(3),pej(4))

c     fill /cptl/
c     -----------
      do 49 lo=1,2
      nptl=nptl+1
      if(lo.eq.1)ij=i
      if(lo.eq.2)ij=j
      do 48 k=1,5
      if(lo.eq.1)pptl(k,nptl)=pei(k)
      if(lo.eq.2)pptl(k,nptl)=pej(k)
48    continue
      istptl(nptl)=0
      idptl(nptl)=idptl(ij)
      ibptl(1,nptl)=ibptl(1,ij)
      ibptl(2,nptl)=ibptl(2,ij)
      ibptl(3,nptl)=ibptl(3,ij)
      ibptl(4,nptl)=ibptl(4,ij)
      xorptl(1,nptl)=xaver(1)           
      xorptl(2,nptl)=xaver(2)
      xorptl(3,nptl)=xaver(3)
      xorptl(4,nptl)=xaver(4)
      iorptl(nptl)=i
      jorptl(nptl)=j
      tivptl(1,nptl)=xaver(4)                
      call idtau(idptl(nptl),pptl(4,nptl),pptl(5,nptl),taugm)
      tivptl(2,nptl)=tivptl(1,nptl)+taugm
      ifrptl(1,nptl)=0
      ifrptl(2,nptl)=0
      iclptl(nptl)=iclptl(ij)
49    continue

1000  return
      end

c----------------------------------------------------------------------
      subroutine jintfp(i,j,n,
     *xpl1,ypl1,radpl1,opl1,upl1,
     *xpl2,ypl2,radpl2,opl2,upl2,
     *xpl3,ypl3,radpl3,opl3,upl3,dov,fov,vov)
c----------------------------------------------------------------------
c  printout
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)

      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 

      real xpl(3),ypl(3),radpl(3),opl(3),upl(3)
      character ax*10,lin*59,mark*1,txt*8

      xpl(1)  =xpl1
      ypl(1)  =ypl1
      radpl(1)=radpl1
      opl(1)  =opl1
      upl(1)  =upl1
      xpl(2)  =xpl2
      ypl(2)  =ypl2
      radpl(2)=radpl2
      opl(2)  =opl2
      upl(2)  =upl2
      xpl(3)  =xpl3
      ypl(3)  =ypl3
      radpl(3)=radpl3
      opl(3)  =opl3
      upl(3)  =upl3

      v1=(opl1-upl1)*radpl1**2*pi
      v2=(opl2-upl2)*radpl2**2*pi
      v3=(opl3-upl3)*radpl3**2*pi

      call uttauh(i,upl1,opl1,g1,y1,ws1)
      call uttauh(j,upl2,opl2,g2,y2,ws2)
      call uttauh(n,upl3,opl3,g3,y3,ws3)
      uu1=pptl(5,i)/g1
      uu2=pptl(5,j)/g2
      uu3=pptl(5,n)/g3

      write(ifch,103)sngl(ttaus)
103   format(' monitoring fusion interaction at ttaus=',f5.2)
      tx=ttaus
      tx=amax1(tx,1.) 
      write(txt(3:7),104)tx
104   format(f5.2)
      write(ifch,105)'inc:',i,idptl(i),g1,uu1,v1,uu1/v1*(opl1-upl1)*ws1
      write(ifch,105)'inc:',j,idptl(j),g2,uu2,v2,uu2/v2*(opl2-upl2)*ws2
      write(ifch,105)'fus:',n,idptl(n),g3,uu3,v3,uu3/v3*(opl3-upl3)*ws3
105   format(1x,a4,2x,'nr=',i6,2x,'id=',i9,2x,'g=',f4.1
     *,2x,'mass=',f6.2,2x,'vol=',f6.2,2x,'eps=',f5.2)

      do mm=1,3
      if(mm.eq.1)txt(1:2)='s/'
      if(mm.eq.1)txt(8:8)=':'
      if(mm.eq.2)txt='x,y:    '
      ax='---------!'
      if(mm.le.2)
     *write(ifch,100)txt,(ax,l=1,6)
100   format(2x,a8,3x,'-3',8x,'-2',8x,'-1',9x,'0',9x,'1',9x,'2',9x,'3'
     */13x,' !',6a10)
      do k=1,3
      do l=1,59
      wo=-3+l*0.1 + 0.05
      wu=-3+l*0.1 - 0.05
           if(mm.eq.1)then
      o=opl(k)/tx
      u=upl(k)/tx
           elseif(mm.eq.2)then
      o=xpl(k)+radpl(k)
      u=xpl(k)-radpl(k)
           elseif(mm.eq.3)then
      o=ypl(k)+radpl(k)
      u=ypl(k)-radpl(k)
           endif
      mark='O'
      if(k.eq.3)mark='X'
      lin(l:l)=' '
      if(wu.le.o.and.wo.ge.u)lin(l:l)=mark
      enddo
      write(ifch,101)u,o,lin
      enddo
101   format(1x,2f6.2,' !',a59,'!')
      write(ifch,102)('-',l=1,59)
102   format(13x,' !',59a1,'!')
      enddo

           do mm=1,2
           if(mm.eq.1)then
      oi=xpl(1)+radpl(1)
      ui=xpl(1)-radpl(1)
      oj=xpl(2)+radpl(2)
      uj=xpl(2)-radpl(2)
           elseif(mm.eq.2)then
      oi=ypl(1)+radpl(1)
      ui=ypl(1)-radpl(1)
      oj=ypl(2)+radpl(2)
      uj=ypl(2)-radpl(2)
           endif
           if(oj.le.ui)then
      ov=0.
           elseif(uj.le.ui.and.oj.le.oi)then
      ov=oj-ui
           elseif(uj.le.ui.and.oj.ge.oi)then
      ov=oi-ui
           elseif(uj.ge.ui.and.oj.le.oi)then
      ov=oj-uj
           elseif(uj.ge.ui.and.oj.ge.oi)then
      ov=oi-uj
           elseif(uj.ge.oi)then
      ov=0.
           else
      call utstop('jintfp: should not be reached           ')
           endif
      if(mm.eq.1)xov=ov
      if(mm.eq.2)yov=ov
           enddo
      if(fov.gt.xov*yov)                                    
     *call utstop('jintfp: fov.gt.xov*yov                  ')

      xu=amin1(xpl(1)-radpl(1),xpl(2)-radpl(2))
      xo=amax1(xpl(1)+radpl(1),xpl(2)+radpl(2))
      yu=amin1(ypl(1)-radpl(1),ypl(2)-radpl(2))
      yo=amax1(ypl(1)+radpl(1),ypl(2)+radpl(2))
      nxy=400
      dx=(xo-xu)/nxy
      dy=(yo-yu)/nxy
      fnum=0
      do ix=1,nxy
      do iy=1,nxy
      x=xu + dx*(ix-0.5)
      y=yu + dy*(iy-0.5)
      if(   (xpl(1)-x)**2 + (ypl(1)-y)**2 .le. radpl(1)**2
     *.and. (xpl(2)-x)**2 + (ypl(2)-y)**2 .le. radpl(2)**2 )
     *fnum=fnum+dx*dy
      enddo
      enddo

      su=amin1(upl(1),upl(2))
      so=amax1(opl(1),opl(2))
      ns=5000
      ds=(so-su)/ns 
      dnum=0
      do is=1,ns 
      s=su + ds*(is-0.5)
      if(   upl(1).le.s.and.s.le.opl(1)                        
     *.and. upl(2).le.s.and.s.le.opl(2) )                      
     *dnum=dnum+ds      
      enddo

      re1=(dov-dnum      )/dov
      re2=(fov-fnum      )/fov
      re3=(vov-dnum*fnum )/vov
      re4=(v3-(v1+v2-vov))/v3
      if(re1.gt.1e-2.or.re2.gt.1e-2.or.re3.gt.1e-2.or.re4.gt.1e-2)   
     *write(ifmt,106)dov,dnum,fov,fnum,vov,dnum*fnum,v3,v1+v2-vov
106   format(1x,'ov:',2f8.4,2x,2f8.4,2x,2f8.4
     *,2x,'v3:',2f8.4)

      return
      end

c----------------------------------------------------------------------
      subroutine jintfs(i,j,nqi,jc,amim,iret) 
c----------------------------------------------------------------------
c  input: ptl numbers i,j   
c  output: ppfus(5): momentum , nqi: net quark number,                  
c              jc: jc-code, amim: minimum mass,    of fused ptl
c          iret=0 if ok, 1 else
c          ppfus() written to pptl(,nptl+1)
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/cjint/nptl0,rnup,rnut,velp,velt,bx,by,xaver(4)
      parameter (nflav=6)
      integer jci(nflav,2),jcj(nflav,2),jc(nflav,2)
      real p(5)
      double precision ppfus(5),pp52

      iret=0

      do 35 k=1,4
      p(k)=pptl(k,i)+pptl(k,j)
35    ppfus(k)=p(k)
      pp52=ppfus(4)**2-ppfus(3)**2-ppfus(2)**2-ppfus(1)**2
           if(pp52.le.0.)then
      if(ish.ge.90)then
      call utmsg('jintfs')
      write(ifch,*)'*****  mfus**2 < 0    (',pp52,' )'
      write(ifch,*)(ppfus(m),m=1,4)
      call utmsgf
      endif
      goto1001
           endif
      ppfus(5)=dsqrt(pp52)
      if(nptl+1.gt.mxptl)
     *call utstop('jintfs: nptl>mxptl                      ')
      do 36 k=1,5
36    pptl(k,nptl+1)=ppfus(k)
      amf=ppfus(5)

      istptl(nptl+1)=0

      call idquac(i,ndummy,ndummy,ndummy,jci)
      call idquac(j,ndummy,ndummy,ndummy,jcj)
      do 29 n=1,nflav
      do 29 k=1,2
29    jc(n,k)=jci(n,k)+jcj(n,k)
      nqi=0
      do 54 n=1,nflav
54    nqi=nqi+jc(n,1)-jc(n,2)
                       
      call idcomj(jc)
      amim=utamnz(jc,5)+.200
      if(amf.lt.amim.and.i.gt.nptl0.and.j.gt.nptl0)goto1001
      goto1000

1001  iret=1

1000  return
      end

c----------------------------------------------------------------------
      subroutine jintfu(i,j,jc,iactn)
c----------------------------------------------------------------------
c  fusion of ptls i,j: determine droplet
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)

      parameter (mxlook=10000,mxdky=2000)
      common/dkytab/look(mxlook),cbr(mxdky),mode(5,mxdky)
      common/cjint/nptl0,rnup,rnut,velp,velt,bx,by,xaver(4)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat /ctimel/ntc

      parameter (nflav=6)
      integer jc(nflav,2),ic(2),ib(4)

c     print
c     -----
      if(ish.ge.92)then
      write(ifch,'(1x,a,i3,a,f6.2)')
     *'>>> fusion interaction; nt=',ntc,' taus=',sngl(ttaus)
      write(ifch,115)i,idptl(i)
     *,(pptl(k,i),k=3,5),desptl(i),radptl(i)
      write(ifch,115)j,idptl(j)
     *,(pptl(k,j),k=3,5),desptl(j),radptl(j)
115   format(' /cptl/',i6,i10,3x,2(e9.3,1x),e8.3,3x,2(e9.3,1x),2x,i4)
      endif

c     initialization
c     --------------
      iactn=3
      amf=pptl(5,nptl+1)
      nptl=nptl+1

c     determine idr, ib(1-4)
c     ----------------------
      idr=0
      do 40 nf=1,nflav
      do 40 ij=1,2
      if(jc(nf,ij).ge.10)idr=7*10**8
40    continue
           if(idr/10**8.ne.7)then
      call idenco(jc,ic,ireten)
      if(ireten.eq.1)
     *call utstop('jintfu: idenco ret code = 1             ')
      id=idtra(ic,0,0,3)
43    amc=amf
      call idres(id,amc,idr,iadj)
           if(idr.ne.0)then
      lid=look(iabs(idr))
      if((lid.le.0.or.lid.gt.0.and.mode(2,lid).eq.0)
     *.and.pptl(5,nptl).gt.amc+1e-3)then
      amf=amf+0.010
      goto43
      endif
      if((lid.le.0.or.lid.gt.0.and.mode(2,lid).eq.0)
     *.and.abs(amc-pptl(5,nptl)).gt.1e-3)then
      if(ish.ge.90)then
      call utmsg('jintfu')
      write(ifch,*)'*****  not on mass shell after fusion: '
     *,pptl(5,nptl),amc
      call utmsgf
      endif
      endif
           endif
           if(idr.eq.0)then
      if(mod(ic(1),100).ne.0.or.mod(ic(2),100).ne.0)then
      idr=9*10**8
      else
      idr=8*10**8+ic(1)*100+ic(2)/100
      endif
           endif
           else
      call idtrbi(jc,ib(1),ib(2),ib(3),ib(4))
      idr=idr
     *+mod(jc(1,1)+jc(2,1)+jc(3,1)+jc(4,1),10**4)*10**4
     *+mod(jc(1,2)+jc(2,2)+jc(3,2)+jc(4,2),10**4)
      if(ish.ge.93)write(ifch,*)'ib:',(ib(kk),kk=1,4)
           endif

c     fill /cptl/
c     -----------
      idptl(nptl)=idr
      do k=1,4
      ibptl(k,nptl)=ib(k)
      enddo
      iorptl(nptl)=i
      jorptl(nptl)=j
      ifrptl(1,nptl)=0
      ifrptl(2,nptl)=0
      iclptl(nptl)=1
     
      return
      end

c----------------------------------------------------------------------
      subroutine jintov(xi,yi,ri,oi,ui,xj,yj,rj,oj,uj,dov,fov,vov,vfu)
c----------------------------------------------------------------------
c  returns overlap length dov, area fov, volume vov
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin

      u(x)=x-cos(x)*sin(x)

      rij=sqrt( (xj-xi)**2+(yj-yi)**2 )

           if(ri+rj.le.rij)then
      fov=0
           elseif(rij+ri.le.rj)then
      fov=pi*ri**2
           elseif(rij+rj.le.ri)then
      fov=pi*rj**2
           else
      ai=acos( (rij**2+ri**2-rj**2)/(2*rij*ri) )
      aj=acos( (rij**2+rj**2-ri**2)/(2*rij*rj) )
      bi=pi-ai
      bj=pi-aj
           if(ri**2.gt.rj**2+rij**2)then
      fov= rj**2 * (pi-u(bj)) + ri**2 * u(ai)
           elseif(rj**2.gt.ri**2+rij**2)then
      fov= ri**2 * (pi-u(bi)) + rj**2 * u(aj)
           else
      fov= ri**2 * u(ai) + rj**2 * u(aj)
           endif
           endif

           if(oj.le.ui)then
      dov=0.
           elseif(uj.le.ui.and.oj.le.oi)then
      dov=oj-ui
           elseif(uj.le.ui.and.oj.ge.oi)then
      dov=oi-ui
           elseif(uj.ge.ui.and.oj.le.oi)then
      dov=oj-uj
           elseif(uj.ge.ui.and.oj.ge.oi)then
      dov=oi-uj
           elseif(uj.ge.oi)then
      dov=0.
           else
      write(ifch,*)'ui oi:',ui,oi
      write(ifch,*)'uj oj:',uj,oj
      call utstop('jintov: should not be reached           ')
           endif

      vov=fov*dov

      vi=(oi-ui)*ri**2*pi
      vj=(oj-uj)*rj**2*pi
      vfu=vi+vj-vov

      if(vfu.le.0.) 
     *call utstop('jintov: should not happen               ')

      return
      end

c----------------------------------------------------------------------
      subroutine jintpa(i,j,kmax)
c----------------------------------------------------------------------
c  writes parents of i,j to ifrij(1-kmax)
c  sets iaaptl()=0 for parents
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter (mxifr=mxptl)
      common/cifrij/ifrij(mxifr)

      kcur=0
      kmax=0
      loop=0
12    loop=loop+1
      if(loop.eq.1)n=i
      if(loop.eq.2)n=j
           if(iorptl(n).gt.0)then
      if(ish.ge.93)write(ifch,*)n,' <--- ',iorptl(n),jorptl(n)
      if(kmax+2.gt.mxifr)
     *call utstop('jintpa: kmax+2 > mxifr                  ')
      if(iaaptl(iorptl(n)).ne.0)then
      kmax=kmax+1
      ifrij(kmax)=iorptl(n)
      iaaptl(iorptl(n))=0
      endif
      if(jorptl(n).gt.0)then
      if(iaaptl(jorptl(n)).ne.0)then
      kmax=kmax+1
      ifrij(kmax)=jorptl(n)
      iaaptl(jorptl(n))=0
      endif
      endif
           endif
      if(loop.eq.1)goto12
8     kcur=kcur+1
      if(kcur.le.kmax)then
      n=ifrij(kcur)
      if(iaaptl(n).ne.0)
     *call utstop('jintpa: should not happen               ')
      goto12
      endif

      return
      end

c-----------------------------------------------------------------------
      subroutine jrescl(j1,j2,psum,ifail)
c     rescales ptl momenta of ptls j1-j2 to have total mom psum.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      dimension psum(5)
      double precision ppsum(5),padd(5),pp(5)
      common/cscal/scal
      data errlim/.001/

      ifail=1

      do k=1,5
      ppsum(k)=psum(k)
      enddo

      if(j1.ge.j2)
     *call utstop('jrescl: j1 .ge. j2                      ')

      do 100 k=1,5
100   padd(k)=0.
      do 110 j=j1,j2
      do 110 k=1,5
      padd(k)=padd(k)+pptl(k,j)
110   continue
           if(padd(5).ge.ppsum(5))then
      if(ish.ge.90)then
      call utmsg('jrescl')
      write(ifch,*)'*****  sum of ptl masses .ge. ppsum(5)'
      do 1 j=j1,j2
1     write(ifch,109)j,idptl(j)
     *,(pptl(l,j),l=3,5)
109   format(' /cptl/',i6,i10,3x,3(1x,e9.3))
      write(ifch,*)'ppsum(345):',(sngl(ppsum(k)),k=3,5)
      call utmsgf
      endif
      return
           endif
      padd(5)=padd(4)**2-padd(1)**2-padd(2)**2-padd(3)**2
           if(padd(5).le.0)then
      do 2 j=j1,j2
2     write(ifch,108)j,(pptl(l,j),l=1,5)
108   format(' /cptl/',i4,2x,5(1x,e9.3))
      call utstop('jrescl: mass**2 of string-sum negative  ')
           endif
      padd(5)=sqrt(padd(5))

c     boost ptls to rest
c     ------------------
           do j=j1,j2
      do k=1,4
      pp(k)=pptl(k,j)
      enddo 
      call utlob2(1,padd(1),padd(2),padd(3),padd(4),padd(5)    
     *,pp(1),pp(2),pp(3),pp(4))
      do k=1,4
      pptl(k,j)=pp(k)
      enddo 
           enddo  
 
c     rescale momenta in rest frame
c     -----------------------------
      scal=1.
      do 301 ipass=1,200
      sum=0.
      do 310 j=j1,j2
      do 320 k=1,3
320   pptl(k,j)=scal*pptl(k,j)
      pptl(4,j)=sqrt(pptl(1,j)**2+pptl(2,j)**2+pptl(3,j)**2
     *+pptl(5,j)**2)
      sum=sum+pptl(4,j)
310   continue
      scal=psum(5)/sum
      if(abs(scal-1.).le.errlim) goto300
301   continue
      if(ish.ge.90)then
      call utmsg('jrescl')
      write(ifch,*)'*****  scal=',scal
      call utmsgf
      endif
300   continue
 
c     boost back with ppsum
c     ---------------------
           do j=j1,j2
      do k=1,4
      pp(k)=pptl(k,j)
      enddo 
      call utlob2(-1,ppsum(1),ppsum(2),ppsum(3),ppsum(4),ppsum(5)    
     *,pp(1),pp(2),pp(3),pp(4))
      do k=1,4
      pptl(k,j)=pp(k)
      enddo 
           enddo  

      ifail=0
      return
      end
 
c-----------------------------------------------------------------------
      subroutine jsplit(stro,str,kolsp,ier,kmaxor)
c-----------------------------------------------------------------------
c  splits string stro into q-qbar string str and remainder (->stro)
c  dimensions: stro(nsi,nsix+1),str(nsi,2)
c  ier=0: ok ; ier=1: error ; ier=2: absorption ;
c  ier=3: again with new apart, epart
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nptf=129)
      common /cptf/xptf(nptf),wptf(nptf)
     *,qptfu(nptf),qptfs(nptf),qptfuu(nptf),qptfus(nptf),qptfss(nptf)
      parameter (nptq=129)
      common /cptq/xptq(nptq),qptq(nptq),qpth(nptq),wptq(nptq)
      parameter (nsi=6,nsix=100)
      common/cnfusn/nfusn(nsix+1)
      parameter (nflav=6)
      integer jc(nflav,2),ic(2),jcx(nflav,2),icx(2)
      parameter (nha=nsix+2,mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      common/cjspli/jpart,nscc,nsccx,apart,epart,alead,elead,sgnsil
      parameter (nsplit=129)
      common /csplit/xsplit(nsplit),qsplit(nsplit),qsplix(nsplit)
     *,wsplit(nsplit)
      common/cpzstr/pzstrl,estrl,istrl,isea

      real stro(nsi,nsix+1),stro0(nsi,nsix+1),str(nsi,2)
      double precision sstr(nsi,2),po(5),po123,a,d,pnl3,pnl4
     *,pox,poy,pot,pop,pom,pux,puy,put,pup,pum,pap,pam,pep,pem,pyp,pym
     *,pip,pim,pp,pm

      pudx=0.47 
      ish0=ish
      if(ishsub/100.eq.15)ish=mod(ishsub,100)

      if(ish.ge.94)write(ifch,*)('-',i=1,10)
     *,' entry sr jsplit ',('-',i=1,30)
   
      call utkstr(stro,kmax)
           if(kmax.eq.kmaxor)then
      nscc=0
      if(jpart.eq.0)then
      do i=1,kmaxor
      nfusn(i)=0
      enddo
      endif
           endif
      kmax0=kmax
      do 10 i=1,nsi
      do 10 k=1,kmax
10    stro0(i,k)=stro(i,k)
c-c   pdiq=pdiqua/2
c-c   if(kmaxor.gt.kutdiq)pdiq=0.
c-c   pdiq=pdiqua*0.5*nsccx/(kmaxor-1.)
      pdiq=0
      loop=0
      ncorn=0
      xunter=0.
      xober=1.
      messco=0
      kolsp=kproja(kmax+1,iproj)
      if(sgnsil.lt.0.)kolsp=ktarga(kmax+1,itarg)
      pap=2*epart
      pam=0
      pux=stro(1,kmax)
      puy=stro(2,kmax)
      put=dsqrt(pux**2+puy**2)
      pup=stro(4,kmax)-abs(stro(3,kmax))
      pum=stro(4,kmax)+abs(stro(3,kmax))
 
c     print
c     -----
      if(ish.ge.92)then
      write(ifch,*)'multi-string detected. split off q-qbar string:'
      if(ish.ge.93)then
      write(ifch,*)'input string stro:'
      write(ifch,105)(stro(i,1),i=1,4),(nint(stro(i,1)),i=5,6)
      do 8 k=2,kmax
8     write(ifch,104)(stro(i,k),i=1,4),(nint(stro(i,k)),i=5,6)
      endif
      endif
 
c     absorption
c     ----------
      if(istrl.eq.1)goto1002
 
c     reset
c     -----
5001  loop=0
5000  loop=loop+1
      if(loop.ge.5)goto1002
      kmax=kmax0
      do 11 i=1,nsi
      do 11 k=1,kmax
11    stro(i,k)=stro0(i,k)
 
c     split off hadron + determine remainder string (momenta)
c     -------------------------------------------------------
      nqu=1
      if(rangen().lt.pdiq)nqu=2
      ifltt=0
      iflto=0
      do 24 i=1,nqu
      ifl=int(rangen()/pudx)+1
      iflto=iflto*10+ifl
24    ifltt=ifltt*10+(ifl+1)/2
      if(ifltt.eq.1)then
      pt=utinvt(nptf,xptf,qptfu ,rangen()*qptfu(nptf))
      elseif(ifltt.eq.2)then
      pt=utinvt(nptf,xptf,qptfs ,rangen()*qptfs(nptf))
      elseif(ifltt.eq.11)then
      pt=utinvt(nptf,xptf,qptfuu,rangen()*qptfuu(nptf))
      elseif(ifltt.eq.12.or.ifltt.eq.21)then
      pt=utinvt(nptf,xptf,qptfus,rangen()*qptfus(nptf))
      elseif(ifltt.eq.22)then
      pt=utinvt(nptf,xptf,qptfss,rangen()*qptfss(nptf))
      endif
      at=sqrt(apart**2+pt**2)
      r=rangen()
c-c   x=sqrt(1-sqrt(r))                !f(x)=x-x**3
      x=utinvt(nsplit,xsplit,qsplit,r*qsplit(nsplit))
      if(nsccx.ge.2)
     *x=utinvt(nsplit,xsplit,qsplix,r*qsplix(nsplit))
        if(abs(pzstrl).lt.estrl)then
      ystrl=.5*alog((estrl+pzstrl)/(estrl-pzstrl))
      xunter=abs(0.94*sinh(ystrl))/epart
      if(xunter.gt.1)xunter=1.
        else
      xunter=1.
        endif
      xunter=xunter-(xober-xunter)
      x=xunter+x*(xober-xunter)
c-c   en=apart+x*(epart-apart)
      pz=x*epart
      sgnpo=1
      if(pz.lt.0)sgnpo=-1
      en=sqrt(pz**2+at**2)
c-c   if(at.gt.en)at=apart+rangen()*(en-apart)
c-c   pt=sqrt((at-apart)*(at+apart))
      phi=2.*pi*rangen()
      po(1)=pt*cos(phi)
      po(2)=pt*sin(phi)
c-c   po(3)=sgnsil*sqrt(en**2-at**2)
      po(3)=sgnsil*pz
      po(4)=en
      po123=sqrt(po(3)**2+po(2)**2+po(1)**2)
           if(po(4)-po123.gt.0.)then
      po(5)=sqrt((po(4)-po123)*(po(4)+po123))
           else
      po(5)=0
      if(po(4)-po123.lt.-1e-4*po(4))then
      call utmsg('jsplit')
      write(ifch,*)'*****  !po123! > po4'
      write(ifch,*)'po123,po4:',po123,po(4)
      write(ifch,*)'po1:',po(1)
      write(ifch,*)'po2:',po(2)
      write(ifch,*)'po3:',po(3)
      write(ifch,*)'po5:',po(5)
      call utmsgf
      endif
           endif
      pox=po(1)
      poy=po(2)
      pot=dsqrt(pox**2+poy**2)
      pop=po(4)+dabs(po(3))*sgnpo
      pom=po(4)-dabs(po(3))*sgnpo
      stro(3,1)=stro(3,1)-epart*sgnsil
      stro(4,1)=stro(4,1)-epart
      do 2 i=1,4
      stro(i,1)=stro(i,1)+po(i)
2     stro(i,kmax)=0.
      yld=100
      if(po(4)-dabs(po(3)).ne.0.)
     *yld=0.5*dlog((po(4)+dabs(po(3)))/(po(4)-dabs(po(3))))
 
c     determine split string (momenta)
c     --------------------------------
      pp=pap+pup
      pm=pam+pum
      pep=pp-pop
      pem=pm-pom
      if(pep.le.0.)goto5000
      if(pem.le.0.)goto5000
      a=(pem*pep-put**2-pot**2)/2.
      d=put*pot
      if(a**2-d**2.lt.0.)goto5000
      pyp=(a+put**2-dsqrt(a**2-d**2))
      if(pyp.lt.0..and.pyp.gt.-1e-6)pyp=0.
      pyp=pyp/pem
      pym=(a+pot**2-dsqrt(a**2-d**2))
      if(pym.lt.0..and.pym.gt.-1e-6)pym=0.
      pym=pym/pep
      if(pyp.lt.0.)goto5000
      if(pym.lt.0.)goto5000
      pip=pep-pyp
      if(pip.lt.0.)goto5000
      pim=pym
      pap=pyp
      pam=pem-pym
      if(pam.lt.0.)goto5000
      sstr(1,1)=-pox
      sstr(2,1)=-poy
      sstr(3,1)=sgnsil*(pip-pim)/2
      sstr(4,1)=(pip+pim)/2
      sstr(1,2)=pux
      sstr(2,2)=puy
      sstr(3,2)=sgnsil*(pap-pam)/2
      sstr(4,2)=(pap+pam)/2
      pnl3=sstr(3,1)+sstr(3,2)
      pnl4=sstr(4,1)+sstr(4,2)
      ynl=100
      if(pnl4-dabs(pnl3).ne.0.)
     *ynl=0.5*dlog((pnl4+dabs(pnl3))/(pnl4-dabs(pnl3)))
      do n=1,4
      do i=1,2
      str(n,i)=sstr(n,i)
      enddo 
      enddo 

c     checks
c     ------
      if(ish.ge.90)then
      if(dabs(pip*pim-pot**2).gt.1e-4)then
      call utmsg('jsplit')
      write(ifch,*)'*****  pip*pim /= pot**2'
      write(ifch,*)'pip*pim=',pip*pim
      write(ifch,*)'pot**2=',pot**2
      write(ifch,*)'pip=',pip
      write(ifch,*)'pim=',pim
      write(ifch,*)'pot=',pot
      call utmsgf
      endif
      if(dabs(pap*pam-put**2).gt.1e-4)then
      call utmsg('jsplit')
      write(ifch,*)'*****  pap*pam /= put**2'
      write(ifch,*)'pap*pam=',pap*pam
      write(ifch,*)'put**2=',put**2
      write(ifch,*)'pap=',pap
      write(ifch,*)'pam=',pam
      write(ifch,*)'put=',put
      call utmsgf
      endif
      if(dabs(sstr(4,1)**2
     *-sstr(1,1)**2-sstr(2,1)**2-sstr(3,1)**2).gt.1e-4)then
      call utmsg('jsplit')
      write(ifch,*)'*****  mass**2 of string end 1 nonzero'
      write(ifch,*)'mass**2=',sstr(4,1)**2
     *-sstr(1,1)**2-sstr(2,1)**2-sstr(3,1)**2
      call utmsgf
      endif
      if(dabs(sstr(4,2)**2
     *-sstr(1,2)**2-sstr(2,2)**2-sstr(3,2)**2).gt.1e-4)then
      call utmsg('jsplit')
      write(ifch,*)'*****  mass**2 of string end 2 nonzero'
      write(ifch,*)'mass**2=',sstr(4,2)**2
     *-sstr(1,2)**2-sstr(2,2)**2-sstr(3,2)**2
      call utmsgf
      endif
      do 14 n=1,4
      if(abs(str(n,1)+stro(n,1)-stro0(n,1)
     *+str(n,2)+stro(n,kmax)-stro0(n,kmax)).gt.1e-4
     *.and.abs(str(n,1)+stro(n,1)-stro0(n,1)
     *+str(n,2)+stro(n,kmax)-stro0(n,kmax))
     *.gt.1e-4*abs(stro0(n,1)+stro0(n,kmax)))goto15
14    continue
      goto16
15    continue
      call utmsg('jsplit')
      write(ifch,*)'*****  p_str + p_stro /= p_stro0'
      write(ifch,*)'p_str + p_stro:'
      write(ifch,104)((str(n,1)+str(n,2)+stro(n,1)+stro(n,kmax)),n=1,4)
      write(ifch,*)'p_stro0:'
      write(ifch,104)((stro0(n,1)+stro0(n,kmax)),n=1,4)
      write(ifch,*)'str:'
      write(ifch,104)(str(n,1),n=1,4)
      write(ifch,104)(str(n,2),n=1,4)
      write(ifch,*)'stro:'
      write(ifch,104)(stro(n,1),n=1,4)
      write(ifch,104)(stro(n,kmax),n=1,4)
      write(ifch,*)'stro0:'
      write(ifch,104)(stro0(n,1),n=1,4)
      write(ifch,104)(stro0(n,kmax),n=1,4)
      call utmsgf
16    continue
      if(ish.ge.93)then
      write(ifch,100)apart,epart,en,pt
100   format(3x,'apart,epart,en,pt:',4f11.5)
      write(ifch,101)(sngl(po(k)),k=1,5)
101   format(3x,'po:',5f11.5)
      if(nqu.eq.2)then
      write(ifch,*)'diquark-antidiquark break'
      endif
      endif
      endif
 
c     flavour
c     -------
      do 1 i=1,2
1     ic(i)=nint(abs(stro(4+i,kmax)))
      do 3 i=1,2
3     str(4+i,2)=ic(i)
      do 4 n=1,nflav
      do 4 i=1,2
4     jcx(n,i)=0
      m=0
      if(ic(1).gt.0.and.nqu.eq.1)m=2
      if(ic(2).gt.0.and.nqu.eq.1)m=1
      if(ic(1).gt.0.and.nqu.eq.2)m=1
      if(ic(2).gt.0.and.nqu.eq.2)m=2
      if(m.eq.0)
     *call utstop('jsplit: m = 0                           ')
      do 25 n=1,nqu
      if(n.eq.1)nfl=mod(iflto,10)
      if(n.eq.2)nfl=iflto/10
25    jcx(nfl,m)=jcx(nfl,m)+1
      call idenco(jcx,icx,ireten)
      if(ireten.eq.1)
     *call utstop('jsplit: idenco ret code = 1             ')
      do 5 i=1,2
5     str(4+i,1)=icx(i)
           call utamst(str,am,amin,iret)
           if(iret.ne.0)goto1002
           if(nfusn(kmax).eq.1)goto1002
      do 6 i=1,2
6     ic(i)=nint(stro(4+i,1))
      call iddeco(ic,jc)
      do 26 n=1,nqu
      if(n.eq.1)nfl=mod(iflto,10)
      if(n.eq.2)nfl=iflto/10
      if(jc(nfl,m).gt.0)then
      jc(nfl,m)=jc(nfl,m)-1
      else
      jc(nfl,3-m)=jc(nfl,3-m)+1
      endif
26    continue
      nn=0
      do 27 n=1,nflav
      do 27 i=1,2
27    nn=nn+jc(n,i)
      if(nn.eq.0)then
      nfl=int(rangen()/pudx)+1
      jc(nfl,1)=1
      jc(nfl,2)=1
      endif
      call idenco(jc,ic,ireten)
           if(ireten.eq.1)then
      if(ish.ge.90.and.messco.eq.0)then
      call utmsg('jsplit')
      write(ifch,*)'*****  idenco ret code = 1.   redo jsplit'
      write(ifch,*)'jc:'
      write(ifch,*)jc
      call utmsgf
      messco=1
      endif
      goto5000
           endif
      do 7 i=1,2
      stro(4+i,1)=ic(i)
7     stro(4+i,kmax)=0.
 
c     ok
c     --
1000  ier=0
      nscc=nscc+1
      goto10002
 
c     error
c     -----
1001  ier=1
      if(ish.ge.90)then
      call utmsg('jsplit')
      write(ifch,*)'*****  split not possible'
      call utmsgf
      endif
      goto10001
 
c     absorption
c     ----------
1002  ier=2
      nfusn(kmax)=1
      if(ish.ge.92)then
      write(ifch,*)'absorption of stro(,kmax)'
      endif
      do 18 i=1,nsi
      str(i,1)=0
18    str(i,2)=0
      do 17 i=1,nsi
      do 17 k=1,kmax
17    stro(i,k)=stro0(i,k)
      do 19 i=1,4
      stro(i,1)=stro(i,1)+stro(i,kmax)
19    stro(i,kmax)=0.
      do 20 i=1,2
20    ic(i)=nint(stro(4+i,1))
      call iddeco(ic,jc)
      do 21 i=1,2
21    icx(i)=nint(abs(stro(4+i,kmax)))
      call iddeco(icx,jcx)
      do 22 nf=1,nflav
      do 22 i=1,2
22    jc(nf,i)=jc(nf,i)+jcx(nf,i)
      call idenco(jc,ic,ireten)
           if(ireten.eq.1)then
      if(ish.ge.90)then
      call utmsg('jsplit')
      write(ifch,*)'*****  idenco ret code = 1'
     *,'   (after absorption)'
      write(ifch,*)'jc:'
      write(ifch,*)jc
      call utmsgf
      endif
      goto1001
           endif
      do 23 i=1,2
      stro(4+i,1)=ic(i)
23    stro(4+i,kmax)=0.
      goto10002
 
c     check nscc
c     ----------
10002 continue
      if(kmaxor.gt.2.and.kmax.eq.2.and.max(1,nscc).ne.nsccx)then
      ier=3
      if(ish.ge.91)then
      write(ifch,*)'redo string processing with new apart, epart'
      endif
      goto10001
      endif
 
c     print
c     -----
      if(ish.ge.92)then
      if(ish.ge.93)then
      write(ifch,*)'remainder string:'
      write(ifch,105)(stro(i,1),i=1,4),(nint(stro(i,1)),i=5,6)
      do 9 k=2,kmax
9     write(ifch,104)(stro(i,k),i=1,4),(nint(stro(i,k)),i=5,6)
      write(ifch,*)'split string:'
      endif
      write(ifch,105)(str(i,1),i=1,4),(nint(str(i,1)),i=5,6)
      write(ifch,104)(str(i,2),i=1,4),(nint(str(i,2)),i=5,6)
105   format(' str: ',4f11.5,2i8)
104   format('      ',4f11.5,2i8)
      endif
10001 continue
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr jsplit ',('-',i=1,10)
      ish=ish0
      return
 
      end
 
c-----------------------------------------------------------------------
      subroutine jstrij(ipa1,ipa2,jc,idhad,jcx,ier,isame,ierrc)
c     strips hadron idhad off jet jc. isame=1: zero flavor required
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (nflav=6)
      integer jc(nflav,2),jcx(nflav,2),icp(2),jcp(nflav,2)
     *,ic1(2),jc1(nflav,2),ic2(2),jc2(nflav,2)
      ier=0
      call randfl(jc,ipa1,ii,ic1,isame)
      if(ii.eq.0)goto9999
      call iddeco(ic1,jc1)
      do 8 n=1,nflav
      do 8 i=1,2
      jcp(n,i)=jc1(n,i)
8     jcx(n,i)=jc(n,i)-jc1(n,i)
      if(isame.ne.1)then
      call randfl(jcx,ipa2,ii,ic2,0)
      if(ii.eq.0)goto9999
      call iddeco(ic2,jc2)
      do 4 n=1,nflav
      do 4 i=1,2
      jcp(n,i)=jcp(n,i)+jc2(n,i)
4     jcx(n,i)=jcx(n,i)-jc2(n,i)
      endif
      call idenco(jcp,icp,ireten)
      if(ireten.eq.1)
     *call utstop('jstrij: idenco ret code = 1             ')
      idhad=idtra(icp,1,0,3)
      if(idhad.eq.0)goto9998
      return
9998  jerr=jerr+1
      write(ifch,*)'***** error in jstrij: idhad = 0'
      write(ifch,*)'code bef strip:'
      write(ifch,*)jc
      write(ifch,*)'jc1:'
      write(ifch,*)jc1
      write(ifch,*)'jc2:'
      write(ifch,*)jc2
      write(ifch,*)'ptl code:'
      write(ifch,*)jcp
      ier=1
      return
9999  ier=1
      if(ierrc.eq.0)return
      jerr=jerr+1
      write(ifch,*)'***** error in jstrij: no parton found'
      write(ifch,*)'code bef strip:'
      write(ifch,*)jc
      return
      end
 
c-----------------------------------------------------------------------
      subroutine jstrip(ipa1,ipa2,ic,idhad,icx,ier,isame,ierrc)
c     strips hadron idhad off jet ic. isame=1: zero flavor required
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer jc(nflav,2),ic(2),jcx(nflav,2),icx(2)
      call iddeco(ic,jc)
      call jstrij(ipa1,ipa2,jc,idhad,jcx,ier,isame,ierrc)
      call idenco(jcx,icx,ireten)
      if(ireten.eq.1)
     *call utstop('jstrip: idenco ret code = 1             ')
      return
      end
 
c-----------------------------------------------------------------------
      subroutine lepexp(rxbj,rqsq)
c     generates x_bjorken and q**2 according to an experimental
c     distribution ( given in array xq(nxbj,nqsq) ).
c-----------------------------------------------------------------------
      parameter (nxbj=10,nqsq=10)
      parameter (xbjmin=0.,qsqmin=4.)
      parameter (xbjwid=0.025, qsqwid=4.)
      dimension xq(nxbj,nqsq),vxq(nxbj*nqsq)
      equivalence (xq(1,1),vxq(1))
 
      data (vxq(i),i=1,50)/
     &         1304.02,   366.40,    19.84,    10.79,     6.42,
     &            4.54,     4.15,     3.38,     2.03,     1.56,
     &          241.63,  1637.26,   427.36,   164.51,    73.72,
     &           43.07,    20.73,    12.78,     9.34,     5.83,
     &            0.01,   724.66,   563.79,   275.08,   176.13,
     &          106.44,    85.82,    54.52,    37.12,    28.65,
     &            0.01,   202.40,   491.10,   245.13,   157.07,
     &          104.43,    61.05,    49.42,    37.84,    26.79,
     &            0.01,     3.77,   316.38,   226.92,   133.45,
     &           90.30,    63.67,    48.42,    35.73,    28.04/
      data (vxq(i),i=51,100)/
     &            0.01,     0.01,   153.74,   213.09,   114.14,
     &           76.26,    60.02,    43.15,    43.47,    25.60,
     &            0.01,     0.01,    39.31,   185.74,   108.56,
     &           88.40,    47.29,    39.35,    31.80,    22.91,
     &            0.01,     0.01,     0.01,   104.61,   107.01,
     &           66.24,    45.34,    37.45,    33.44,    23.78,
     &            0.01,     0.01,     0.01,    56.58,    99.39,
     &           67.78,    43.28,    35.98,    34.63,    18.31,
     &            0.01,     0.01,     0.01,    13.56,    76.25,
     &           64.30,    42.80,    28.56,    21.19,    20.75 /
 
      data init/0/
      init=init+1
      if(init.eq.1) then
      n=nxbj*nqsq
      sum=0.
      do 1 i=1,n
      sum=sum+vxq(i)
1     continue
      do 2 i=2,n
2     vxq(i)=vxq(i)+vxq(i-1)
      do 3 i=1,n
3     vxq(i)=vxq(i)/sum
      endif
 
      n=nxbj*nqsq
      r=rangen()
      call utloc(vxq,n,r,iloc)
      if(iloc.ge.n) iloc=iloc-1
      i=mod(iloc,nxbj)+1
      if(i.eq.0) i=nxbj
      j=iloc/nxbj + 1
      dxint=vxq(1)
      if(iloc.gt.0) dxint=vxq(iloc+1)-vxq(iloc)
      dxbj=xbjwid*abs(r-vxq(iloc+1))/dxint
      dy=qsqwid*rangen()
      rxbj=xbjmin+xbjwid*float(i-1)+dxbj
      rqsq=qsqmin+qsqwid*float(j-1)+dy
      return
      end
 
c-----------------------------------------------------------------------
      subroutine lepstr(nuclon,xbj,qsq,nstrng)
c     returns string code nstrng for given nuclon, xbj, qsq.
c     nuclon  : the struck nucleon (1120/1220 = proton/neutron)
c     xbj,qsq : x-bjorken and q**2
c     nstrng  : string code:
c        proton strings:          neutron strings:
c        1 : u  ---  ud           11 : u  ---  dd
c        2 : d  ---  uu           12 : d  ---  ud
c        3 : u  ---  ub(uud)      13 : u  ---  ub(udd)
c        4 : d  ---  db(uud)      14 : d  ---  db(udd)
c        5 : s  ---  sb(uud)      15 : s  ---  sb(udd)
c        6 : ub ---  u(uud)       16 : ub ---  u(udd)
c        7 : db ---  d(uud)       17 : db ---  d(udd)
c        8 : sb ---  s(uud)       18 : sb ---  s(udd)
c-----------------------------------------------------------------------
      dimension quarks(9)
      nstrng=0
      if(nuclon.ne.1120.and.nuclon.ne.1220) return
 
c     proton-string (valence part)
      if(nuclon.eq.1120) then
      quarks(1) =  4.*  stxu(xbj,qsq)
      quarks(2) =       stxd(xbj,qsq)
      endif
 
c     neutron-string (valence part)
      if(nuclon.eq.1220) then
      quarks(1) =  4.*  stxd(xbj,qsq)
      quarks(2) =       stxu(xbj,qsq)
      endif
 
c     the sea contributions (proton/neutron)
      udsea     =       stxus(xbj,qsq)
      ssea      =       stxs(xbj,qsq)
      quarks(3) =  4.*  udsea
      quarks(4) =       udsea
      quarks(5) =       ssea
      quarks(6) =  4.*  udsea
      quarks(7) =       udsea
      quarks(8) =       ssea
 
      quarks(9) = 0.
      do 11 i=1,8
11    quarks(9) = quarks(9)+quarks(i)
 
      r=rangen() * quarks(9)
      sumq=0.
      do 12 i=1,8
      nstrng=i
      sumq=sumq+quarks(i)
      if(r.le.sumq) go to 13
12    continue
13    continue
 
      if(nuclon.eq.1220) nstrng=nstrng+10
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine leptar(xbj,qsq,matarg,latarg,nuclon)
c     returns nuclon = id of hit target nucleon.
c     xbj   : x bjorken
c     qsq   : q squared
c     matarg : a  of target
c     latarg : z  of target
c     nuclon: id of target nucleon (1120/1220 for proton/neutron)
c-----------------------------------------------------------------------
c     proton part of the structure function:
      f2p = latarg * stxzpr(xbj,qsq)
c     neutron part of the structure function:
      f2n = (matarg-latarg) * stxzne(xbj,qsq)
c     structure function of nucleus:
      f2a = f2p + f2n
c     select  the target-nucleon  ( proton or neutron ):
      rn = rangen()
      f2rndm = f2a * rn
      nuclon = 1120
      if(f2rndm .lt. f2n) nuclon = 1220
      return
      end
 
c-----------------------------------------------------------------------
      subroutine nucini(opt,nuc,la,ma,isi)
c     initializes nucleon-momenta.
c     writes nucleons on /cptl/ (p,ifr,icl).
c-----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (nsi=6,nsix=100,mamx=250)
      parameter (nha=nsix+2)
      real nuc(nsi,nha,mamx)
      parameter (mamx2=mamx*2)
      common/cnnn/nnnptl(mamx2)
      character opt*3
      nhax=3
      if(nha.lt.nhax)
     *call utstop('nucini: nha too small                   ')
      las=0
      mas=0
 
      do 1 l=1,ma
 
      do 2 m=1,nha
      do 2 n=1,nsi
2     nuc(n,m,l)=0
 
      if(opt.eq.'nuc')then
      if(isi.gt.0.and.laproj.lt.0)then
      id=idproj
      elseif(isi.lt.0.and.latarg.lt.0)then
      id=idtarg
      else
      id=1220
      if(rangen().le.(la-las)*1./(ma-mas))id=1120
      if(id.eq.1120)las=las+1
      mas=mas+1
      endif
      call idmass(id,ams)
      ic1=idtrai(1,id,1)
      ic2=idtrai(2,id,1)
      p1=0
      p2=0
      p3=isi*pnllx
      p0=sqrt(pnllx**2+ams**2)
      p0x=pnllx
      p5=ams
 
      elseif(opt.eq.'str')then
      id=1220
      if(rangen().le.(la-las)*1./(ma-mas))id=1120
      if(id.eq.1120)las=las+1
      mas=mas+1
      ic1=idtrai(1,id,1)
      ic2=idtrai(2,id,1)
      p1=0
      p2=0
      p3=0
      p0=prom
      p5=prom
      endif
 
      nuc(5,1,l)=ic1
      nuc(6,1,l)=ic2
      nuc(3,2,l)=p3
      nuc(4,2,l)=p0x
      nuc(5,2,l)=ic1
      nuc(6,2,l)=ic2
      nptl=nptl+1
      if(nptl.ne.nnnptl(nptl))
     *call utstop('nucini: nptl and nnnptl dont match      ')
      idptl(nptl)=id
      pptl(1,nptl)=p1
      pptl(2,nptl)=p2
      pptl(3,nptl)=p3
      pptl(4,nptl)=p0
      pptl(5,nptl)=p5
      ifrptl(1,nptl)=0
      ifrptl(2,nptl)=0
      iclptl(nptl)=1
1     continue
      return
      end
 
c-----------------------------------------------------------------------
      subroutine nuclco(massnr,n,x,y,z,ynuc)
c     calculates coordinates of the nucleons in a nucleus.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (ndep=129,ndet=129,kollmx=5000)
      common /col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      real x(n),y(n),z(n)
      if(massnr.eq.0)return
      if(massnr.gt.n)
     *call utstop('nuclco: massnr.gt.n                     ')
      if(massnr.eq.1)then
      x(1)=0
      y(1)=0
      z(1)=0
      return
      endif
      do 3 i=1,massnr
10    continue
      if(massnr.eq.maproj)r=utinvt(ndep,xdep,qdep,rangen()*qdep(ndep))
      if(massnr.eq.matarg)r=utinvt(ndet,xdet,qdet,rangen()*qdet(ndet))
      if(massnr.ne.maproj.and.massnr.ne.matarg)
     *call utstop('nuclco: nucleus neither proj nor targ   ')
      theta=acos(1-2*rangen())
      phi=-pi+2*pi*rangen()
      x(i)=r*sin(theta)*cos(phi)
      y(i)=r*sin(theta)*sin(phi)
      z(i)=r*cos(theta)
      if(i.eq.1)goto3
      if(core.eq.0.)goto3
      do 2 j=1,i-1
      if((x(i)-x(j))**2+(y(i)-y(j))**2+(z(i)-z(j))**2.lt.core**2)goto10
2     continue
3     continue
      if(ish.ge.93)write(ifch,*)'nucleon coordinates:'
      do 4 i=1,massnr
      z(i)=z(i)/cosh(ynuc)
      if(ish.ge.93)write(ifch,*)'i x y z: ',i,x(i),y(i),z(i)
4     continue
      return
      end
 
c-----------------------------------------------------------------------
      subroutine nucoge
c     writes nucleons on /cptl/ (xor,tiv,ist,ior,jor,nst).
c     determines sequence of collisions according to nuclear geometry
c       if maproj>0.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paroi/bmaxim,bminim,phimax,phimin
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (ndep=129,ndet=129,kollmx=5000)
      common /col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      parameter (mamx=250)
      common /cncl/xproj(mamx),yproj(mamx),zproj(mamx)
     *,xtarg(mamx),ytarg(mamx),ztarg(mamx)
      parameter (mamx2=mamx*2)
      common/cnnn/nnnptl(mamx2)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      if(ish.eq.17)write(ifch,*)'nucoge (entry)'
      vel=tanh(ypjtl-yhaha)+tanh(yhaha)
      nproj=1
      ntarg=1
      do 15 n=1,kollmx
      nord(n)=n
      do 16 k=1,4
16    coord(k,n)=0
15    continue
 
      if(matarg.lt.0)then
      koll=-matarg
      ntarg=koll
      bimp=0
      nptl=nptl+1
      do 3 i=1,4
3     xorptl(i,nptl)=0
      tivptl(1,nptl)=-ainfin  !formation time of incident nucleons
      tivptl(2,nptl)=0
      istptl(nptl)=1
      iorptl(nptl)=-1
      jorptl(nptl)=0
      nnnptl(nptl)=nptl
      do 1 k=1,koll
      nrproj(k)=1
      nrtarg(k)=k
      nptl=nptl+1
      do 4 i=1,4
4     xorptl(i,nptl)=0
      tivptl(1,nptl)=-ainfin  !formation time of incident nucleons
      tivptl(2,nptl)=0
      istptl(nptl)=1
      iorptl(nptl)=-1
      jorptl(nptl)=0
      nnnptl(nptl)=nptl
1     continue
      goto9999
 
      elseif(maproj.eq.1.and.matarg.eq.1)then
      koll=1
      bimp=0
      nrproj(1)=1
      nrtarg(1)=1
      do 5 ii=1,2
      nptl=nptl+1
      do 2 i=1,4
2     xorptl(i,nptl)=0
      tivptl(1,nptl)=-ainfin  !formation time of incident nucleons
      tivptl(2,nptl)=0
      istptl(nptl)=1
      iorptl(nptl)=-1
      jorptl(nptl)=0
5     nnnptl(nptl)=nptl
      goto9999
      endif
 
      call nuclco(maproj,mamx,xproj,yproj,zproj,ypjtl-yhaha)
      call nuclco(matarg,mamx,xtarg,ytarg,ztarg,yhaha)
      bx=0
      by=0
      if(maproj.gt.0)then
      if(bimevt.lt.0)then
      b1=bminim
      b2=amin1(rmproj+rmtarg,bmaxim)
      if(b1.gt.b2)
     *call utstop('nucoge: bmin > bmax                     ')
      bimp=sqrt(b1**2+(b2**2-b1**2)*rangen())
      phi=phimin+rangen()*(phimax-phimin)
      else
      phi=phievt
      bimp=bimevt
      endif
      bx=cos(phi)*bimp
      by=sin(phi)*bimp
      do 6 i=1,maproj
      nptl=nptl+1
      xorptl(1,nptl)=xproj(i)+bx/2
      xorptl(2,nptl)=yproj(i)+by/2
      xorptl(3,nptl)=zproj(i)
      xorptl(4,nptl)=0
      tivptl(1,nptl)=-ainfin  !formation time of incident nucleons
      tivptl(2,nptl)= ainfin
      istptl(nptl)=0
      iorptl(nptl)=0
      jorptl(nptl)=0
6     nnnptl(nptl)=nptl
      endif
      do 7 i=1,matarg
      nptl=nptl+1
      xorptl(1,nptl)=xtarg(i)-bx/2
      xorptl(2,nptl)=ytarg(i)-by/2
      xorptl(3,nptl)=ztarg(i)
      xorptl(4,nptl)=0
      tivptl(1,nptl)=-ainfin  !formation time of incident nucleons
      tivptl(2,nptl)= ainfin
      istptl(nptl)=0
      iorptl(nptl)=0
      jorptl(nptl)=0
7     nnnptl(nptl)=nptl
      if(maproj.eq.0)goto1000
      koll=0
      do 12 i=1,maproj
      do 12 j=1,matarg
      if((xproj(i)+bx-xtarg(j))**2
     *+(yproj(i)+by-ytarg(j))**2.gt.sigppi/pi)goto12
           if(koll.gt.0)then
      kp=0
      kt=0
      do kl=1,koll
      if(nrproj(kl).eq.i)kp=1
      if(nrtarg(kl).eq.j)kt=1
      enddo
      ftrsig=1.0
      ftr=1.0
      if(kp.eq.1)ftr=ftr*ftrsig
      if(kt.eq.1)ftr=ftr*ftrsig
      if(kp.eq.1.or.kt.eq.1)then
      if((xproj(i)+bx-xtarg(j))**2
     *+(yproj(i)+by-ytarg(j))**2.gt.ftr*sigppi/pi)goto12
      endif
           endif
      koll=koll+1
      if(koll.gt.kollmx)
     *call utstop('nucoge: kollmx too small                ')
      nrproj(koll)=i
      nrtarg(koll)=j
      distce(koll)=ztarg(j)-zproj(i)
      coord(1,koll)=(xproj(i)+xtarg(j))/2
      coord(2,koll)=(yproj(i)+ytarg(j))/2
      coord(3,koll)=(zproj(i)+ztarg(j))/2
      coord(4,koll)=distce(koll)/vel
      istptl(i)=1
      iorptl(i)=-1
      tivptl(2,i)=coord(4,koll)
      istptl(maproj+j)=1
      iorptl(maproj+j)=-1
      tivptl(2,maproj+j)=coord(4,koll)
12    continue
      if(koll.le.1)goto9999
 
      do 21 n=2,koll
      do 22 m=1,n-1
      if(nrproj(m).eq.nrproj(n))goto21
22    continue
      nproj=nproj+1
21    continue
      do 23 n=2,koll
      do 24 m=1,n-1
      if(nrtarg(m).eq.nrtarg(n))goto23
24    continue
      ntarg=ntarg+1
23    continue
 
      do 20 n=1,koll-1
      do 20 m=n+1,koll
      if(distce(nord(m)).lt.distce(nord(n)))then
      nordm=nord(m)
      nord(m)=nord(n)
      nord(n)=nordm
      endif
20    continue
 
9999  continue
      if(koll.le.0)goto1000
      if(koll.lt.ko1ko2/10000.or.koll.gt.mod(ko1ko2,10000))goto1000
      nevt=1
      bimevt=bimp
      phievt=phi
      kolevt=koll
      npjevt=nproj
      ntgevt=ntarg
      pmxevt=pnll
      egyevt=engy
 
1000  continue
      if(ish.eq.17)write(ifch,*)'nucoge (exit)'
      return
      end
 
c-----------------------------------------------------------------------
      subroutine nucoll
c     performs a  nucleus-nucleus collision (incl. nucleon-nucleon)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      common/civ/ivp0,ivp,ivt0,ivt /cntevm/ntevm
     */cnew/kotri,newico,newcol
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      real proj(nsi,nha),targ(nsi,nha)
      parameter(nprbms=20)
      common /cprbms/prbms(nprbms)
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      common/partnr/summas,iptnr,ic4,px4,py4,js4,pex,pey,pezet,pe0,nps
     */cnce/nces,ncolex /cnfr/nrfra /ckol/kol
      parameter (mamx=250)
      real proja(nsi,nha,mamx),targa(nsi,nha,mamx)
      integer ic4(2)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      character dash*1
      dash='-'
      ish0=ish
      if(ishsub/100.eq.6)ish=mod(ishsub,100)
      if(ish.eq.14)call uttima('*** nucoll *** ')
      if(ish.eq.17)write(ifch,*)'nucoll (entry)'
 
      if(ishsub/100.eq.7)ish=mod(ishsub,100)
      nevt=0
      nptl=0
      call nucoge
      if(iappl.eq.2.or.koll.eq.0)goto1000
      naevt=naevt+1
      if(koll.lt.ko1ko2/10000.or.koll.gt.mod(ko1ko2,10000))goto1000
      colevt=koll
      amsev=sqrt((npjevt*sqrt(amproj**2+pnllx**2)
     *+ntgevt*sqrt(amtarg**2+pnllx**2))**2
     *-((npjevt-ntgevt)*pnllx)**2)
      if(ishsub/100.eq.7)ish=ish0
 
      itry=0
38    continue
      itry=itry+1
      nstr=0
      nptl=0
      sumpt2=0
      amsac=0
      call nucini('nuc',proja,laproj,maproj,1)
      call nucini('nuc',targa,latarg,iabs(matarg),-1)
 
      kolran=rangen()*koll+1
      kolran=min0(kolran,koll)
      kolran=max0(kolran,1)
 
c--------------------------
      do 2 kols=1,koll
c--------------------------
 
      kol=kols
      iskip=0
      i=nrproj(nord(kol))
      j=nrtarg(nord(kol))
      iproj=i
      itarg=j
      nrecop=0
      nrecot=0
      if(kol.lt.koll)then
      do 33 k=kol+1,koll
      if(nrproj(nord(k)).eq.i)nrecop=nrecop+1
      if(nrtarg(nord(k)).eq.j)nrecot=nrecot+1
33    continue
      endif
      ncop=0
      ncot=0
      if(kol.gt.1)then
      do 34 k=1,kol-1
      if(nrproj(nord(k)).eq.i)ncop=ncop+1
      if(nrtarg(nord(k)).eq.j)ncot=ncot+1
34    continue
      endif
      if(ncop.ge.ncolmx.or.ncot.ge.ncolmx)iskip=1
 
      do 56 m=1,nha
      sm=0
      do 57 n=1,nsi
      sm=sm+proja(n,m,i)**2
57    proj(n,m)=proja(n,m,i)
      if(m.ge.3.and.sm.lt.1e-5)goto58
56    continue
58    continue
      do 53 m=1,nha
      sm=0
      do 54 n=1,nsi
      sm=sm+targa(n,m,j)**2
54    targ(n,m)=targa(n,m,j)
      if(m.ge.3.and.sm.lt.1e-5)goto55
53    continue
55    continue
 
      if(kol.eq.kolran.and.jpsi.eq.1)then
      call pvjpsf(proj,targ)
      iskip=1
      endif
 
c     nr of colour exchanges
c     ----------------------
 
      amsac0=amsac
      iskip0=iskip
      smpt2=sumpt2
      call utremb(proj,targ,1)
      goto4
3     continue
      if(ish.ge.91)write(ifch,*)'redo hh collision'
      call utrest(proj,targ,1)
      sumpt2=smpt2
      iskip=iskip0
      amsac=amsac0
4     continue
 
      ncolex=1
      if(iskip.ne.1)then
      lo=0
16    lo=lo+1
      if(lo.eq.3)then
      if(ish.ge.90)then
      call utmsg('nucoll')
      write(ifch,*)'*****  lo=3'
      call utmsgf
      endif
      endif
      r=rangen()
      ncolex=0
15    ncolex=ncolex+1
      if(ncolex.gt.nprbms)goto16
      if(r.gt.prbms(ncolex))goto15
      endif
 
c     hadron-hadron collision
c     -----------------------
 
      do 31 nce=1,ncolex
      nces=nce
      call utpage
      if(ish.ge.91)then
      write(ifch,101)('#',l=1,79),ipage,kol,nce
     *,nrproj(nord(kol)),nrtarg(nord(kol)),('#',l=1,79)
101   format(1x,79a1/1x,i5,'.page             col: ',i2
     *,'   cex: ',i2,'   prj: ',i3,'   trg: ',i3/1x,79a1)
      endif
      istorp=0
      istort=0
      if(nrecop.eq.0)istorp=1
      if(nrecot.eq.0)istort=1
      call hahabs(proj,targ
     *,istorp*(nce/ncolex),istort*(nce/ncolex),iskip,irethh)
      if(iskip.ge.2)goto9997
      if(irethh.eq.1)goto3
      if(irescl.eq.1.and.amsac.gt.amsev)goto9998
      sumpt2=sumpt2+
     *proj(1,2)**2+proj(2,2)**2+targ(1,2)**2+targ(2,2)**2
      if(ish.eq.11)write(ifch,*)'sumpt2:',nrevt,kol,nce,sumpt2
      iskip=0
31    continue
 
      do 40 m=1,nha
      sm=0
      do 41 n=1,nsi
      sm=sm+proj(n,m)**2
41    proja(n,m,i)=proj(n,m)
      if(m.ge.3.and.sm.lt.1e-5)goto42
40    continue
42    continue
      do 43 m=1,nha
      sm=0
      do 44 n=1,nsi
      sm=sm+targ(n,m)**2
44    targa(n,m,j)=targ(n,m)
      if(m.ge.3.and.sm.lt.1e-5)goto45
43    continue
45    continue
 
c------------------
  2   continue
c------------------
 
      if(sumpt2.lt.1e-5)goto9999
 
1000  continue
      if(ish.eq.14)call uttima('    nucoll f   ')
      if(ish.eq.17)write(ifch,*)'nucoll (exit)'
      ish=ish0
      return
 
9999  inoiac=inoiac+1
      if(ish.ge.91
     *.or.ish.eq.11)then
      call utmsg('nucoll')
      write(ifch,*)'*****  no interaction. redo nucoll'
      call utmsgf
      endif
      goto38
 
9998  ilamas=ilamas+1
      if(ish.ge.91
     *.or.ish.eq.12)then
      call utmsg('nucoll')
      write(ifch,*)'*****  amsac>amsev: ',amsac,amsev
     *,' . redo nucoll'
      do 1 ns=1,nstr
1     write(ifch,109)ns,(icstr(l,ns)/100,l=1,4)
     *,(pstr(l,ns),l=3,5)
109   format(1x,i3,3x,4i5,2x,3(1x,e9.3))
      call utmsgf
      endif
      goto38
 
9997  continue
      if(ish.ge.90)then
      call utmsg('nucoll')
      write(ifch,*)'*****  iskip>=2. redo nucoll'
      call utmsgf
      endif
      goto38
 
      end
 
c-----------------------------------------------------------------------
      subroutine nucstr(ier)
c     performes x and p trafos for nucleons for string decay in nucleus
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common /clep/icinpu,idscat
      if(matarg.ne.nptl)
     *call utstop('nucstr: matarg /= nptl                  ')
      ncnt=0
3     ncnt=ncnt+1
      if(ncnt.ge.10)goto1001
      n0=min0(1+int(rangen()*nptl),nptl)
      if(icinpu.gt.0.and.idptl(n0).ne.idscat)goto3
      istptl(n0)=1
      iorptl(n0)=-1
      tivptl(2,n0)=0
      do 2 n=1,nptl
      do 1 j=1,3
1     xorptl(j,n)=xorptl(j,n)-xorptl(j,n0)
      phi=rangen()*2*pi
      p1=-elepto*sin(angmue)*sin(phi)
      p2=-elepto*sin(angmue)*cos(phi)
      p3=elepti-elepto*cos(angmue)
      call utrota(1,p1,p2,p3,xorptl(1,n),xorptl(2,n),xorptl(3,n))
      p3=sqrt(p1**2+p2**2+p3**2)
      p1=0
      p2=0
      p4=prom+elepti-elepto
      call utlobo(1,p1,p2,p3,p4,sqrt(p4**2-p3**2-p2**2-p1**2)
     *,xorptl(1,n),xorptl(2,n),xorptl(3,n),xorptl(4,n))
2     call utlobo(1,p1,p2,p3,p4,sqrt(p4**2-p3**2-p2**2-p1**2)
     *,pptl(1,n),pptl(2,n),pptl(3,n),pptl(4,n))
 
      ier=0
      return
 
1001  ier=1
      if(ish.ge.90)then
      call utmsg('nucstr')
      write(ifch,*)'*****  idscat not possible ==> redo event.'
      call utmsgf
      endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine pvjpsf(proj,targ)
c     forms a jpsi
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      common/ckol/kol
      parameter (nptj=129)
      common /cptj/xptj(nptj),qptj(nptj),wptj(nptj)
      parameter (ngau=129)
      common /cgau/xgau(ngau),qgau(ngau),wgau(ngau)
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      real proj(nsi,nha),targ(nsi,nha),pr(4),tg(4)
      parameter (mamx=250)
      common/cproja/iproj,itarg,kproja(nha,mamx),ktarga(nha,mamx)
      pax=0.19
      prox=proj(3,2)
      tarx=targ(3,2)
      loop=0
      do 3 n=1,4
      pr(n)=proj(n,2)
3     tg(n)=targ(n,2)
 
 
      call utpage
      if(ish.ge.91)write(ifch,110)('#',l=1,79),ipage,('#',l=1,79)
110   format(1x,79a1/1x,i5,'.page            '
     *,'jpsi formation'/1x,79a1)
 
5000  loop=loop+1
      if(loop.gt.100)then
      call utmsg('pvjpsf')
      write(ifch,*)'*****  jpsi formation not possible'
      call utmsgf
      goto1000
      endif
      do 4 n=1,4
      proj(n,2)=pr(n)
4     targ(n,2)=tg(n)
 
c     jpsi momenta
c     ------------
      id=441
      call idmass(id,am)
      s=am**2
      pt=utinvt(nptj,xptj,qptj,rangen()*qptj(nptj))
      phi=2.*pi*rangen()
      px=pt*cos(phi)
      py=pt*sin(phi)
      lo=0
1     lo=lo+1
      if(lo.gt.10)
     *call utstop('pvjpsf: lo > 10                         ')
      z=pax*utinvt(ngau,xgau,qgau,rangen()*qgau(ngau))
      if(z.gt.1.)goto1
      pz=z*pnllx*ransig()
      e=sqrt(s+px**2+py**2+pz**2)
      pp=e+pz
      pm=e-pz
 
c     proj and targ momenta
c     ---------------------
      r=rangen()
      pox=proj(1,2)-r*px
      poy=proj(2,2)-r*py
      pot=sqrt(pox**2+poy**2)
      r=1-r
      pux=targ(1,2)-r*px
      puy=targ(2,2)-r*py
      put=sqrt(pux**2+puy**2)
      pop=proj(4,2)+proj(3,2)
      pom=proj(4,2)-proj(3,2)
      pup=targ(4,2)+targ(3,2)
      pum=targ(4,2)-targ(3,2)
      pep=pop+pup-pp
      pem=pom+pum-pm
      a=(pem*pep-put**2-pot**2)/2.
      d=put*pot
      if(a**2-d**2.lt.0.)goto5000
      pyp=(a+put**2-sqrt(a**2-d**2))/pem
      pym=(a+pot**2-sqrt(a**2-d**2))/pep
      if(pyp.lt.0.)goto5000
      if(pym.lt.0.)goto5000
      pip=pep-pyp
      if(pip.lt.0.)goto5000
      pim=pym
      pap=pyp
      pam=pem-pym
      if(pam.lt.0.)goto5000
      proj(1,2)=pox
      proj(2,2)=poy
      proj(3,2)=(pip-pim)/2
      proj(4,2)=(pip+pim)/2
      targ(1,2)=pux
      targ(2,2)=puy
      targ(3,2)=(pap-pam)/2
      targ(4,2)=(pap+pam)/2
      kproja(2,iproj)=kol
      ktarga(2,itarg)=kol
      if(proj(3,2)*prox.lt.0.)goto5000
      if(targ(3,2)*tarx.lt.0.)goto5000
 
c     jpsi string
c     -----------
      nstr=nstr+1
      if(nstr.gt.mxstr)
     *call utstop('pvjpsf: nstr>mxstr                      ')
      iorstr(nstr)=-kol
      icstr(1,nstr)=000100
      icstr(2,nstr)=0
      icstr(3,nstr)=0
      icstr(4,nstr)=000100
      pstr(1,nstr)=px
      pstr(2,nstr)=py
      pstr(3,nstr)=pz
      pstr(4,nstr)=e
      pstr(5,nstr)=am
      rotstr(1,nstr)=0
      rotstr(2,nstr)=0
      rotstr(3,nstr)=1
      do 2 i=1,4
2     xorstr(i,nstr)=coord(i,kol)
           if(ish.ge.91)then
      j=nstr
      write(ifch,100)j,(icstr(k,j),k=1,4)
     *,pstr(3,j),pstr(4,j),pstr(5,j)
100   format(' /cstr/',i4,3x,4i7,2x,3(1x,e9.3))
           endif
 
c     checks
c     ------
      if(ish.ge.90)then
      if(abs(pip*pim-pot**2).gt.1e-4)then
      call utmsg('pvjpsf')
      write(ifch,*)'*****  pip*pim /= pot**2'
      write(ifch,*)'pip*pim=',pip*pim,'pot**2=',pot**2
      write(ifch,*)'pip=',pip,'   pim=',pim,'   pot=',pot
      call utmsgf
      endif
      if(abs(pap*pam-put**2).gt.1e-4)then
      call utmsg('pvjpsf')
      write(ifch,*)'*****  pap*pam /= put**2'
      write(ifch,*)'pap*pam=',pap*pam,'put**2=',put**2
      write(ifch,*)'pap=',pap,'   pam=',pam,'   put=',put
      call utmsgf
      endif
      if(abs(proj(4,2)**2
     *-proj(1,2)**2-proj(2,2)**2-proj(3,2)**2).gt.1e-4)then
      call utmsg('pvjpsf')
      write(ifch,*)'*****  mass**2 of proj nonzero'
      write(ifch,*)'mass**2=',proj(4,2)**2
     *-proj(1,2)**2-proj(2,2)**2-proj(3,2)**2
      call utmsgf
      endif
      if(abs(targ(4,2)**2
     *-targ(1,2)**2-targ(2,2)**2-targ(3,2)**2).gt.1e-4)then
      call utmsg('pvjpsf')
      write(ifch,*)'*****  mass**2 of targ nonzero'
      write(ifch,*)'mass**2=',targ(4,2)**2
     *-targ(1,2)**2-targ(2,2)**2-targ(3,2)**2
      call utmsgf
      endif
      do 14 n=1,4
      if(abs(pr(n)+tg(n)
     *-proj(n,2)-targ(n,2)-pstr(n,nstr)).gt.1e-4)goto15
14    continue
      goto16
15    continue
      call utmsg('pvjpsf')
      write(ifch,*)'*****  proj + targ /= proj_new + targ_new +jpsi'
      write(ifch,*)'proj,targ:'
      write(ifch,*)pr
      write(ifch,*)tg
      write(ifch,*)'proj_new,targ_new,jpsi:'
      write(ifch,*)(proj(n,2),n=1,4)
      write(ifch,*)(targ(n,2),n=1,4)
      write(ifch,*)(pstr(n,nstr),n=1,4)
      call utmsgf
16    continue
      endif
 
      amsac=amsac+am
 
1000  return
      end

c-----------------------------------------------------------------------
      subroutine qgcaaa
c-----------------------------------------------------------------------
c hadronic resonance gas in grand canonical treatment
c returns T, chemical potentials and hadronic yield 
c (hadron chemical potentials as combinations of quark chemical potentials) 
c
c input:
c   iostat: 1: Boltzmann approximation, 0: quantum statistics  /paroc/
c   tecm:                    droplet energy      /config/
c   volu:                    droplet volume      /config/
c   keu ked kes kec keb ket: net flavor number   /cinfla/
c
c output:
c   tem    : temperature [GeV]                            /cgchg/
c   chem(1:nflav): quark chem. pot. [GeV]                 /cflav/
c   chemgc(1:nspecs): hadron chem. pot. [GeV]             /cgchg/
c   ptlngc(1:nspecs): hadron number                       /cgchg/
c   rmsngc(1:nspecs): standard deviation of hadron number /cgchg/
c
c exact treatment (iostat=0):
c for massive hadrons     : first in Boltzmann approximation with analytical
c                           expressions for particle and energy densities,
c                           then by using quantum statistics in integral form,
c                           extracting mu and T using numerical integration
c                           and an iterative procedure to solve for mu, T
c for massless hadrons    : using analytic expressions for massles particles
c                           and employing the  same algorithm as for massive
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/parod/ioinfl,ioinct,iowidn,epsgc
      common/cnsta/pi,pii,hquer,prom,piom,ainfin 
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cbol/rmsbol(mspecs),ptlbol(mspecs),chebol(mspecs),tembol
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/ciakt/gen,iafs,ians,genm
      common/cnrit/nrit
      gen=10.0**(-epsgc)
      genm=gen/100.

      isho=ish
      if(ishsub/100.eq.51)ish=mod(ishsub,100)

      iug=(1+iospec)/2*2-1


c     initialization
c     --------------
      kef(1)=keu
      kef(2)=ked
      kef(3)=kes
      kef(4)=kec
      kef(5)=keb
      kef(6)=ket

      if(iug.eq.1)nflavs=1
      if(iug.eq.3)nflavs=2
      if(iug.eq.5)nflavs=2
      if(iug.eq.7)nflavs=3
      if(iug.eq.9)nflavs=3
      if(iug.eq.11)nflavs=3
      tem=0.0
      do i=1,nflavs
      chem(i)=0.0
      enddo
      call qgchac(0)
      do i=1,nspecs
      ptlngc(i)=0.0
      rmsngc(i)=0.0
      enddo
      nrit=0

      if(ish.ge.94)write(ifch,*)('-',l=1,10)
     *,' entry sr qgcaaa ',('-',l=1,30)
      if(ish.ge.94)write(ifch,'(1x,a,2x,3i3)')
     *'>>> grand canonical hadron gas for droplet with u d s content:'
     *,keu,ked,kes
      if(ish.ge.94)write(ifch,'(1x,a,2x,f7.3,2x,a,2x,f7.3)')
     *'mass [GeV]:',tecm,'volume [fm^3]:',volu

      if(iug.eq.1.and.keu.ne.0.and.ish.ge.94)then
      write(ifch,*)'inversion impossible !!!'
      write(ifch,*)'keu=0 required for this option'
      write(ifch,*)'T = mu(i) = 0 returned'
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      return
      endif
      if(iug.eq.3.and.(keu+ked).ne.0.and.ish.ge.94)then
      write(ifch,*)'inversion impossible !!!'
      write(ifch,*)'keu+ked=0 required for this option'
      write(ifch,*)'T = mu(i) = 0 returned'
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      return
      endif
      kf=keu+ked+kes+kec+keb+ket
      kf=abs(kf)
      if(kf.ne.0)then
      if(mod(kf,3).ne.0.and.ish.ge.94)then
      write(ifch,*)'inversion impossible !!!'
      write(ifch,*)'sum must be multiple of three'
      write(ifch,*)'T = mu(i) = 0 returned'
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      return
      endif
      endif


c     initial T (m=0, baryon free)
c     -------------------------------
      gfac=0.0

       if(iostat.eq.0.and.iospec.eq.iug)then
      do i=1,nspecs
      igsp=gspecs(i)
      if(mod(igsp,2).eq.0)then
      gfac=gfac+7.*gspecs(i)/8.
      else
      gfac=gfac+gspecs(i)
      endif
      enddo
      if(iabs(ispecs(nspecs)).lt.10)gfac=gfac+16. 
      tem=(tecm/volu*hquer**3*30./pi**2/gfac)**.25
       else
      do i=1,nspecs
      gfac=gfac+gspecs(i)
      enddo
      if(iabs(ispecs(nspecs)).lt.10)gfac=gfac+16.
      tem=(tecm/volu*hquer**3*pi**2/gfac/3.)**.25
      tem=2.*tem
       endif

      if(ish.ge.94)write(ifch,1)'initial T :',tem
1     format(1x,a,3x,f9.6)

      if(ish.ge.94)write(ifch,*)'iospec: ',iospec

       if(ish.ge.94.and.iospec.ne.iug)then
      write(ifch,*)'inversion in Boltzmann approx. :'
       elseif(ish.ge.94.and.iospec.eq.iug)then 
      write(ifch,*)'inversion for massless hadrons :'
       endif

       if(ish.ge.94)then 
      if(nflavs.eq.1)write(ifch,'(3x,a,8x,a)')
     *'T:','chemu:'
      if(nflavs.eq.2)write(ifch,'(3x,a,8x,a,5x,a)')
     *'T:','chemu:','chemd:'
      if(nflavs.eq.3)write(ifch,'(3x,a,8x,a,5x,a,5x,a)')
     *'T:','chemu:','chemd:','chems:'
       endif

      k=1
10    continue
      if(ish.ge.98.and.mod(k,10).eq.0)
     *write(ifch,*)'qgc iteration:',k
      if(ish.ge.94)call qgccch(1)

c     search for temperature (chem=const)
c     -----------------------------------
      idt=0
      temo=tem

       if(iospec.eq.iug)then

c     massless particles
c     ------------------
      if(iostat.eq.0)then
      if(ish.ge.98)
     *write(ifch,*)'iteration (massless):',k
      call qgctm0
      elseif(iostat.eq.1)then
      if(ish.ge.98)
     *write(ifch,*)'iteration (Boltzmann, massless):',k
      call qgctbo(ibna)
      if(ibna.eq.1)then
      tem=temo
      goto20
      endif
      endif

       else

c     Boltzmann approxiamtion (massive particles)
c     -------------------------------------------
      if(ish.ge.98)
     *write(ifch,*)'iteration (Boltzmann, massive):',k
      call qgctbo(ibna)
      if(ibna.eq.1)then
      tem=temo
      goto20
      endif

       endif

      if(tem.le.1.e-6.and.ish.ge.94)then
      write(ifch,*)'inversion imposssible'
      write(ifch,*)'T:',tem
      if(ioinco.ge.1)call qnbmin(keu,ked,kes,kec)
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      ish=isho
      return
      endif

      dt=abs(temo-tem)
      if(dt.le.gen*temo.or.dt.le.genm)idt=1

c     search for chemical potentials (tem=const)
c     ------------------------------------------
      idch=0
      ibna=0

        do iafs=1,nflavs
      chemo=chem(iafs)

       if(iospec.eq.iug)then

c     massless particles
c     ------------------
      if(iostat.eq.0)then
      call qgccm0
      elseif(iostat.eq.1)then
      call qgccbo(ibna)
      endif

       else

c     Boltzmann approxiamtion (massive particles)
c     -------------------------------------------
      call qgccbo(ibna)

       endif

      dch=abs(chemo-chem(iafs))
      if(ish.ge.98)write(ifch,*)'dch:',dch
      if(dch.le.abs(gen*chemo).or.dch.le.genm)idch=idch+1
      if(ibna.eq.1)then
      chem(iafs)=chemo
      call qgchac(0)
      goto20
      endif

        enddo


c     new hadron chem. potentials
c     ---------------------------
      call qgchac(0)


      if(ish.ge.94.and.nflavs.eq.1)
     *write(ifch,'(1x,f8.6,2x,f9.6)')
     *tem,chem(1)
      if(ish.ge.94.and.nflavs.eq.2)
     *write(ifch,'(1x,f8.6,2x,f9.6,2x,f9.6)')
     *tem,chem(1),chem(2)
      if(ish.ge.94.and.nflavs.eq.3)
     *write(ifch,'(1x,f8.6,2x,f9.6,2x,f9.6,2x,f9.6)')
     *tem,chem(1),chem(2),chem(3)
      if(idch.eq.nflavs.and.idt.eq.1)goto20


      k=k+1

       if(k.gt.300)then
       if(ish.ge.94)
     *write(ifch,*)'failure in approximate solution'
      goto20
       endif

      goto10

20    continue
      if(ish.ge.94)call qgccch(0)
      if(ish.ge.94)write(ifch,'(1x,a,1x,f9.6)')'  T  :',tem
      do i=1,nflavs
      if(i.eq.1.and.ish.ge.94)
     *write(ifch,'(1x,a,1x,f9.6)')'chemu:',chem(1)
      if(i.eq.2.and.ish.ge.94)
     *write(ifch,'(1x,a,1x,f9.6)')'chemd:',chem(2)
      if(i.eq.3.and.ish.ge.94)
     *write(ifch,'(1x,a,1x,f9.6)')'chems:',chem(3)
      enddo
 

c     checking results
c     ----------------
      if(ish.ge.94)call qgcchb

c     particle yield
c     --------------
      call qgcpyi(1)

c     checking flavor conservation
c     ----------------------------
      if(ish.ge.94)call qgccfc

      if(iug.eq.iospec.and.iostat.eq.0)then
      if(ish.ge.94)write(ifch,*)
     *'approximation and exact treatment equal'
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      ish=isho
      return
      endif

c     continue or return approximate values
c     -------------------------------------
      do i=1,nspecs
      rmsbol(i)=rmsngc(i)
      ptlbol(i)=ptlngc(i)
      chebol(i)=chemgc(i)
      enddo
      tembol=tem
      if(iostat.eq.1)then
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      ish=isho
      return
      endif
 

c     quantum statistics
c     ------------------
      if(ish.ge.94)write(ifch,*)'quantum statistics:'
      if(ish.ge.94.and.nflavs.eq.1)write(ifch,'(3x,a,8x,a)') 
     *'T:','chemu:'
      if(ish.ge.94.and.nflavs.eq.2)write(ifch,'(3x,a,8x,a,6x,a)') 
     *'T:','chemu:','chemd:'
      if(ish.ge.94.and.nflavs.eq.3)write(ifch,'(3x,a,8x,a,6x,a,6x,a)') 
     *'T:','chemu:','chemd:','chems:'
      k=1

30    continue
      if(ish.ge.98.and.mod(k,10).eq.0)
     *write(ifch,*)'qgc iteration:',k

c     new temperature
c     ---------------
      idt=0
      temo=tem
      call qgctex
      if(ish.ge.94.and.nflavs.eq.1)
     *write(ifch,'(1x,f10.8,2x,f10.7)')
     *tem,chem(1)
      if(ish.ge.94.and.nflavs.eq.2)
     *write(ifch,'(1x,f10.8,2x,f10.7,2x,f10.7)')
     *tem,chem(1),chem(2)
      if(ish.ge.94.and.nflavs.eq.3)
     *write(ifch,'(1x,f10.8,2x,f10.7,2x,f10.7,2x,f10.7)')
     *tem,chem(1),chem(2),chem(3)

      if(tem.le.1.e-6.and.ish.ge.94)then
      write(ifch,*)'inversion imposssible'
      write(ifch,*)'T:',tem
      call qnbmin(keu,ked,kes,kec)
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      ish=isho
      return
      endif

      dt=abs(temo-tem)
      if(dt.le.gen*temo.or.dt.le.genm)idt=1
      if(ish.ge.98)write(ifch,*)'dtem:',dt

c     new quark chem. potentials
c     --------------------------
      idch=0
      do iafs=1,nflavs
      chemo=chem(iafs)
      call qgccex
      dch=abs(chemo-chem(iafs))
      if(ish.ge.98)write(ifch,*)'dche:',dch
      if(dch.le.abs(gen*chemo).or.dch.le.genm)idch=idch+1
      enddo

c     new hadron chem. potentials
c     ---------------------------
      call qgchac(0)

       if(idch.eq.nflavs.and.idt.eq.1)then
50    continue
      if(ish.ge.94)write(ifch,*)'results:'
      if(ish.ge.94)write(ifch,51)'  T  :',tem
      if(nflavs.ge.1.and.ish.ge.94)write(ifch,51)'chemu:',chem(1)
      if(nflavs.ge.2.and.ish.ge.94)write(ifch,51)'chemd:',chem(2)
      if(nflavs.ge.3.and.ish.ge.94)write(ifch,51)'chems:',chem(3)
51    format(1x,a,3x,f9.6)

c     checking results
c     ----------------
      if(ish.ge.94)call qgcchh(i)  

c     particle yield
c     --------------
      call qgcpyi(0)

c     checking flavor conservation
c     ----------------------------
      call qgccfc

      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      ish=isho
      return 
       endif

       if(k.gt.300)then
       if(ish.ge.94)
     *write(ifch,*)'failure in exact solution'
      if(ish.ge.94)write(ifch,*)'results:'
      if(ish.ge.94)write(ifch,51)'  T  :',tem
      if(nflavs.ge.1.and.ish.ge.94)write(ifch,51)'chemu:',chem(1)
      if(nflavs.ge.2.and.ish.ge.94)write(ifch,51)'chemd:',chem(2)
      if(nflavs.ge.3.and.ish.ge.94)write(ifch,51)'chems:',chem(3)

c     particle yield
c     --------------
      call qgcpyi(0)

      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcaaa ',('-',i=1,10)
      ish=isho
      return

       endif

      k=k+1
      goto30

      end


c---------------------------------------------------------------------
      FUNCTION qgcbi0(x)
c---------------------------------------------------------------------
      DOUBLE PRECISION p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9
      DATA p1,p2,p3,p4,p5,p6,p7/1.0d0,3.5156229d0,3.0899424d0,
     *1.2067492d0,0.2659732d0,0.360768d-1,0.45813d-2/
      DATA q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,0.1328592d-1,
     *0.225319d-2,-0.157565d-2,0.916281d-2,-0.2057706d-1,0.2635537d-1,
     *-0.1647633d-1,0.392377d-2/
      if (abs(x).lt.3.75) then
        y=(x/3.75)**2
        qgcbi0=p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7)))))
      else
        ax=abs(x)
        y=3.75/ax
        qgcbi0=(exp(ax)/sqrt(ax))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *(q7+y*(q8+y*q9))))))))
      endif
      return
      end


c------------------------------------------------------------------------
      FUNCTION qgcbi1(x)
c------------------------------------------------------------------------
      DOUBLE PRECISION p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9
      DATA p1,p2,p3,p4,p5,p6,p7/0.5d0,0.87890594d0,0.51498869d0,
     *0.15084934d0,0.2658733d-1,0.301532d-2,0.32411d-3/
      DATA q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,-0.3988024d-1,
     *-0.362018d-2,0.163801d-2,-0.1031555d-1,0.2282967d-1,-0.2895312d-1,
     *0.1787654d-1,-0.420059d-2/
      if (abs(x).lt.3.75) then
        y=(x/3.75)**2
        qgcbi1=x*(p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7))))))
      else
        ax=abs(x)
        y=3.75/ax
        qgcbi1=(exp(ax)/sqrt(ax))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *(q7+y*(q8+y*q9))))))))
        if(x.lt.0.)qgcbi1=-qgcbi1
      endif
      return
      END


c---------------------------------------------------------------------
      FUNCTION qgcbk0(x)
c------------------------------------------------------------------------
      DOUBLE PRECISION p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7
      DATA p1,p2,p3,p4,p5,p6,p7/-0.57721566d0,0.42278420d0,0.23069756d0,
     *0.3488590d-1,0.262698d-2,0.10750d-3,0.74d-5/
      DATA q1,q2,q3,q4,q5,q6,q7/1.25331414d0,-0.7832358d-1,0.2189568d-1,
     *-0.1062446d-1,0.587872d-2,-0.251540d-2,0.53208d-3/
      if (x.le.2.0) then
        y=x*x/4.0
        qgcbk0=(-log(x/2.0)*qgcbi0(x))+(p1+y*(p2+y*(p3+y*(p4+y*(p5+y*
     *(p6+y*p7))))))
      else
        y=(2.0/x)
        qgcbk0=(exp(-x)/sqrt(x))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *q7))))))
      endif
      return
      END


c---------------------------------------------------------------
      FUNCTION qgcbk1(x)
c--------------------------------------------------------------------
      DOUBLE PRECISION p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7
      DATA p1,p2,p3,p4,p5,p6,p7/1.0d0,0.15443144d0,-0.67278579d0,
     *-0.18156897d0,-0.1919402d-1,-0.110404d-2,-0.4686d-4/
      DATA q1,q2,q3,q4,q5,q6,q7/1.25331414d0,0.23498619d0,-0.3655620d-1,
     *0.1504268d-1,-0.780353d-2,0.325614d-2,-0.68245d-3/
      if (x.le.2.0) then
        y=x*x/4.0
        qgcbk1=(log(x/2.0)*qgcbi1(x))+(1.0/x)*(p1+y*(p2+y*(p3+y*(p4+y*
     *(p5+y*(p6+y*p7))))))
      else
        y=2.0/x
        qgcbk1=(exp(-x)/sqrt(x))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *q7))))))
      endif
      return
      END


c-------------------------------------------------------------------
      FUNCTION qgcbk(n,x)
c------------------------------------------------------------------ 
      tox=2.0/x
      bkm=qgcbk0(x)
      bk=qgcbk1(x)
      do 11 j=1,n-1
        bkp=bkm+j*tox*bk
        bkm=bk
        bk=bkp
11    continue
      qgcbk=bk
      return
      END


c----------------------------------------------------------------          
      subroutine qgccbo(iba)
c----------------------------------------------------------------
c returns new chem(iafs) for boltzmann statistics
c  input:
c    tem
c    kef/volu
c  output:
c    chem(iafs)
c-----------------------------------------------------------------------
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/ciakt/gen,iafs,ians,genm
      external qgcbk
      k=1
      iba=0
      c1=-0.5
      c2=0.5
      goto11

c     new chemical potential
c     ----------------------
10    chem(iafs)=c1+0.5*(c2-c1)
11    continue
      fd=0.0
      call qgchac(0)
 
        do i=1,nspecs

        if(ifok(iafs,i).ne.0)then
       if((chemgc(i)/tem).gt.70.)then
      hpd=1.e30
       else
      hpd=exp(chemgc(i)/tem)
       endif
       if(aspecs(i).ne.0.)then
      fk2=qgcbk(2,aspecs(i)/tem)
      hpd=hpd*gspecs(i)*aspecs(i)**2*tem*fk2
     */2./pi**2/hquer**3
       else
      hpd=hpd*gspecs(i)*tem**3/pi**2/hquer**3
       endif
      hfd=ifok(iafs,i)*hpd
      fd=fd+hfd
        endif

        enddo

      dfd=abs(fd-(kef(iafs)/volu))
      if(dfd.le.abs(gen*(kef(iafs)/volu)).or.dfd.le.genm)return
c     if(abs(fd).ge.100.)then
c     iba=1
c     return
c     endif


       if(fd.gt.(kef(iafs)/volu))then
      c2=chem(iafs)
      else
      c1=chem(iafs)
       endif

      k=k+1
      if(k.gt.300)return

      goto10
      
      end


c----------------------------------------------------------------------
      subroutine qgccch(iii)
c----------------------------------------------------------------------
c checks convergence of iterative algorithm
c plots iteration values for T and mu_i
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common /clatt/nlattc,npmax
      common/cgctot/rmstot,ptltot
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter (mspecs=54)
      parameter (nflav=6)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      parameter(maxit=50000)
      common/count/nacc,nrej,naccit(maxit),nptot,npit(maxit)
      parameter (nbin=500)
      common/cdatc/data(nbin),datb(nbin),datc(nbin),datd(nbin)
     *,date(nbin),datf(nbin),datg(nbin),dath(nbin),dati(nbin)
      common/cnrit/nrit
      character cen*4,cvol*4,cu*3,cd*3,cs*3

           if(iii.gt.0)then

      nrit=nrit+1
      data(nrit)=nrit
      datb(nrit)=tem
      datc(nrit)=chem(1)
      datd(nrit)=chem(2)
      date(nrit)=chem(3)

           elseif(iii.eq.0)then

      nrit=nrit+1
      data(nrit)=nrit
      datb(nrit)=tem
      datc(nrit)=chem(1)
      datd(nrit)=chem(2)
      date(nrit)=chem(3)
      do i=1,nrit
      datf(i)=datb(nrit)
      datg(i)=datc(nrit)
      dath(i)=datd(nrit)
      dati(i)=date(nrit)
      enddo

      x1=data(1)
      x2=data(nrit)
      write(cen,'(f4.1)')tecm
      write(cvol,'(f4.1)')volu
      write(cu,'(i3)')keu
      write(cd,'(i3)')ked
      write(cs,'(i3)')kes
 

      write(ifhi,'(a)')       'newpage zone 1 4 1 openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis Iteration"'
      write(ifhi,'(a)')       'text 0 0 "yaxis T (GeV)"'
      write(ifhi,'(a)')       'text 0.15 0.9 "E= '//cen//'"'
      write(ifhi,'(a)')       'text 0.4 0.9 "V= '//cvol//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),datb(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0-'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),datf(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0'
 
      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis Iteration"'
      write(ifhi,'(a)')       'text 0 0 "yaxis [m]^1! (GeV)"'
      write(ifhi,'(a)')       'text 0.15 0.9 "Q^1!= '//cu//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),datc(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0-'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),datg(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis Iteration"'
      write(ifhi,'(a)')       'text 0 0 "yaxis [m]^2! (GeV)"'
      write(ifhi,'(a)')       'text 0.15 0.9 "Q^2!= '//cd//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),datd(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0-'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),dath(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis Iteration"'
      write(ifhi,'(a)')       'text 0 0 "yaxis [m]^3! (GeV)"'
      write(ifhi,'(a)')       'text 0.15 0.9 "Q^3!= '//cs//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),date(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0-'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(3a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrit
      write(ifhi,'(2e12.4)')data(j),dati(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0'

           endif
  
       return
 
       end

c-----------------------------------------------------------------------
      subroutine qgccex
c-----------------------------------------------------------------------
c returns new chem(iafs) for massive quantum statistics
c  input:
c    tem
c    kef/volu
c  output:
c    chem(iafs)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/ciakt/gen,iafs,ians,genm
      external qgcfhn

      k=1

      c1=-0.5
      c2=0.5
      goto11

c     new chemical potential
c     ----------------------
10    chem(iafs)=c1+0.5*(c2-c1)
11    continue

      fd=0.0
        do ians=1,nspecs
       if(ifok(iafs,ians).ne.0)then

      call qgchac(0)
      call qgclim(a,b) 
      if(b.eq.0.0)then
      hpd=0.0
      else
      call uttraq(qgcfhn,a,b,hpd)
      endif
      hpd=hpd*gspecs(ians)/2./pi**2/hquer**3
      fd=fd+hpd*ifok(iafs,ians)

       endif
        enddo

      dfd=abs(fd-(kef(iafs)/volu))
      if(dfd.le.abs(gen*(kef(iafs)/volu)).or.dfd.le.genm)return

       if(fd.gt.(kef(iafs)/volu))then
      c2=chem(iafs)
      else
      c1=chem(iafs)
       endif

      k=k+1
      if(k.gt.300)then
      if(ish.ge.94)
     *write(ifch,*)'failure at cex at iafs:',iafs
      return
      endif

      goto10

      end


c------------------------------------------------------------------
      subroutine qgccfc
c------------------------------------------------------------------
c checks flavor conservation in particle yield
c------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom

      if(ish.ge.94)write(ifch,*)'checking flavor conservation'
      do i=1,nflavs
      ckef=0.0
      do ii=1,nspecs
      ckef=ckef+ifok(i,ii)*ptlngc(ii)
      enddo
      dkef=abs(ckef-kef(i))
      if(dkef.le.1.e-2)then
      if(i.eq.1.and.ish.ge.94)write(ifch,*)'u conserved'
      if(i.eq.2.and.ish.ge.94)write(ifch,*)'d conserved'
      if(i.eq.3.and.ish.ge.94)write(ifch,*)'s conserved' 
      else
      if(i.eq.1.and.ish.ge.94)write(ifch,*)'u not conserved'
      if(i.eq.2.and.ish.ge.94)write(ifch,*)'d not conserved'
      if(i.eq.3.and.ish.ge.94)write(ifch,*)'s not conserved'
      if(ish.ge.94)write(ifch,*)'df:',dkef
      endif
      enddo

      return 
      end

c----------------------------------------------------------------
      subroutine qgcchb
c----------------------------------------------------------------
c checks results by numerical integration
c----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/ciakt/gen,iafs,ians,genm
      external qgcfbe
      external qgcfbn
      if(ish.ge.94)write(ifch,*)
     *'check by numer. calc. of expect. values:'
      iced=0
      ceden=0.0
       do ians=1,nspecs
      call qgclim(a,b)
      if(b.eq.0.0)then
      cedh=0.0
      else
      call uttraq(qgcfbe,a,b,cedh)
      endif
      if(ish.ge.98)write(ifch,*)'cedh:',cedh
      ced=cedh*gspecs(ians)/2./pi**2/hquer**3
      ceden=ceden+ced
       enddo

      if(iabs(ispecs(nspecs)).lt.10)
     *ceden=ceden+(8.*pi**2*tem**4/15.+bag4rt**4)/hquer**3

      if(ish.ge.94)write(ifch,*)'energy density   :',ceden
      ded=abs((tecm/volu)-ceden)
      if((tecm/volu)*gen.ge.ded.or.ded.le.gen)iced=1
      icfd=0

       do i=1,nflavs
      cfd=0.0
      do ians=1,nspecs
      call qgclim(a,b)
      if(b.eq.0.0)then
      hpd=0.0
      else
      call uttraq(qgcfbn,a,b,hpd)
      endif
      hfd=ifok(i,ians)*hpd*gspecs(ians)/2./pi**2/hquer**3
      if(ish.ge.98)write(ifch,*)'hfd:',hfd
      cfd=cfd+hfd
      enddo
      if(i.eq.1.and.ish.ge.94)write(ifch,5)'flavor density u :',cfd
      if(i.eq.2.and.ish.ge.94)write(ifch,5)'flavor density d :',cfd
      if(i.eq.3.and.ish.ge.94)write(ifch,5)'flavor density s :',cfd
5     format(1x,a,1x,f12.6)
      dfd=abs(cfd-(kef(i)/volu))
      if(abs(gen*(kef(i)/volu)).ge.dfd.or.dfd.le.gen)
     *icfd=icfd+1
       enddo

       if(iced.eq.1.and.icfd.eq.nflavs)then
      if(ish.ge.94)write(ifch,*)'results agree'
      else
      if(ish.ge.94)write(ifch,*)'results disagree'
       endif

      return
      end

c----------------------------------------------------------------
      subroutine qgcchh(icorr)
c----------------------------------------------------------------
c checks results by numerical integration
c----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/ciakt/gen,iafs,ians,genm
      external qgcfhe
      external qgcfhn
      icorr=0
      if(ish.ge.94)write(ifch,*)
     *'check by numer. calc. of expect. values:'

      iced=0
      ceden=0.0
       do ians=1,nspecs
      call qgclim(a,b)
      if(b.eq.0.0)then
      cedh=0.0
      else
      call uttraq(qgcfhe,a,b,cedh)
      endif
      if(ish.ge.98)write(ifch,*)'cedh:',cedh
      ced=cedh*gspecs(ians)/2./pi**2/hquer**3
      ceden=ceden+ced
       enddo

      if(iabs(ispecs(nspecs)).lt.10)
     *ceden=ceden+(8.*pi**2*tem**4/15.+bag4rt**4)/hquer**3

      if(ish.ge.94)write(ifch,*)'energy density   :',ceden
      ded=abs((tecm/volu)-ceden)
      if((tecm/volu)*gen.ge.ded.or.ded.le.gen)iced=1

      icfd=0

       do i=1,nflavs
      cfd=0.0
      do ians=1,nspecs
      call qgclim(a,b)
      if(b.eq.0.0)then
      hpd=0.0
      else
      call uttraq(qgcfhn,a,b,hpd)
      endif
      hfd=ifok(i,ians)*hpd*gspecs(ians)/2./pi**2/hquer**3
      if(ish.ge.98)write(ifch,*)'hfd:',hfd
      cfd=cfd+hfd 
      enddo
      if(i.eq.1.and.ish.ge.94)write(ifch,5)'flavor density u :',cfd
      if(i.eq.2.and.ish.ge.94)write(ifch,5)'flavor density d :',cfd
      if(i.eq.3.and.ish.ge.94)write(ifch,5)'flavor density s :',cfd
5     format(1x,a,1x,f9.6)
      dfd=abs(cfd-(kef(i)/volu))
      if(abs(gen*(kef(i)/volu)).ge.dfd.or.dfd.le.gen)
     *icfd=icfd+1
       enddo

       if(iced.eq.1.and.icfd.eq.nflavs)then
      if(ish.ge.94)write(ifch,*)'results agree'
      icorr=1
      else
      if(ish.ge.94)write(ifch,*)'results disagree'
       endif

      return 
      end


c--------------------------------------------------------------------
      subroutine qgccm0
c--------------------------------------------------------------------
c returns new quark chemical potentials for massless quantum statistics
c input:
c  tem
c  kef/volu
c output:
c  chem
c---------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/ciakt/gen,iafs,ians,genm
      external qgcfhn
      k=1
      z3=1.2020569

      c1=-0.5
      c2=0.5
      goto11

c     new chemical potential
c     ----------------------
10    chem(iafs)=c1+0.5*(c2-c1)
11    continue

      fd=0.0
      call qgchac(0)

               do i=1,nspecs
              if(ifok(iafs,i).ne.0)then

           igsp=gspecs(i)
          if(mod(igsp,2).eq.0)then

       if(ispecs(i).gt.0)then
      hpd=gspecs(i)*(chemgc(i)*tem**2+chemgc(i)**3/pi**2)/6./hquer**3
       else
      hpd=0.0
       endif

c            else
c      if(ispecs(i).gt.0)then
c     hpd=gspecs(i)*(chemgc(i)*tem**2/3.-chemgc(i)**3/pi**2/6.)/hquer**3
c      else
c     hpd=0.0
c      endif
c        endif

c     n=1
c0    xx=n*abs(chemgc(i))/tem
c     if(xx.le.60.)then
c     hpd=hpd+(-1.)**(n+1)/n**3/exp(xx)
c     n=n+1
c     goto20
c     endif
c     hpd=hpd*gspecs(i)*tem**3/pi**2/hquer**3
c     if(chemgc(i).eq.abs(chemgc(i)))then
c     hpd=gspecs(i)*(chemgc(i)*tem**2+chemgc(i)**3/pi**2)/6./hquer**3 
c    *-hpd
c     endif

c      else
c     hpd=3.*gspecs(i)*tem**3*z3/4./pi**2/hquer**3
c      endif
  
         else

      hpd=gspecs(i)*tem**3*z3/pi**2/hquer**3

         endif

      hfd=hpd*ifok(iafs,i)
      fd=fd+hfd

       endif
        enddo

      dfd=abs(fd-(kef(iafs)/volu))
      if(dfd.le.abs(gen*(kef(iafs)/volu)).or.dfd.le.genm)return

       if(fd.gt.(kef(iafs)/volu))then
      c2=chem(iafs)
      else
      c1=chem(iafs)
       endif

      k=k+1
      if(k.gt.300)then
      if(ish.ge.94)
     *write(ifch,*)'failure at cm0 at iafs:',iafs
      return
      endif
      goto10
      end

c-----------------------------------------------------------------------
      function qgcfbe(x)
c-----------------------------------------------------------------------
c integrand of energy density
c------------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      eex=81.
      qgcfbe=0.0
      sq=sqrt(x**2+aspecs(ians)**2)
      if(tem.ne.0.0)eex=(sq-chemgc(ians))/tem
      if(eex.gt.60.)return
      if(eex.lt.-60)then
      qgcfbn=1.e25
      return
      endif

      qgcfbe=sq*x**2*exp(-eex)

      return
      end

c-----------------------------------------------------------------
      function qgcfbf(x)
c-----------------------------------------------------------------
c integrand of mean square variance of  energy
c----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      eex=61
      qgcfbf=0.0

      sq=sqrt(x**2+aspecs(ians)**2)
      if(tem.ne.0.0)eex=(sq-chemgc(ians))/tem
      if(eex.gt.60.)return
      if(eex.lt.-60)then
      qgcfbf=1.e25
      return
      endif

      qgcfbf=(aspecs(ians)**2+x**2)*x**2*exp(-eex)

      return
      end

c-----------------------------------------------------------------
      function qgcfbn(x)
c-----------------------------------------------------------------
c integrand of hadron density
c-----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      eex=81.
      qgcfbn=0.0
      igsp=gspecs(ians)

      sq=sqrt(x**2+aspecs(ians)**2)
      if(tem.ne.0.0)eex=(sq-chemgc(ians))/tem
      if(eex.gt.80.)return
      if(eex.lt.-60)then
      qgcfbn=1.e25
      return
      endif


      qgcfbn=x**2*exp(-eex)

      return
      end

c-----------------------------------------------------------------------
      function qgcfhe(x)
c-----------------------------------------------------------------------
c integrand of energy density
c------------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      eex=81.
      qgcfhe=0.0
      igsp=gspecs(ians)

      sq=sqrt(x**2+aspecs(ians)**2)
      if(tem.ne.0.0)eex=(sq-chemgc(ians))/tem
      if(eex.gt.80.)return

       if(mod(igsp,2).ne.0)then
      d=-1.0
      if(eex.lt.1.e-10)return  
       else
      d=1.0
       endif 

      qgcfhe=sq*x**2/(exp(eex)+d)

      return 
      end

c-----------------------------------------------------------------
      function qgcfhf(x)
c-----------------------------------------------------------------
c integrand of mean square variance of  energy
c----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      eex=61
      qgcfhf=0.0
      igsp=gspecs(ians)

      sq=sqrt(x**2+aspecs(ians)**2)
      if(tem.ne.0.0)eex=(sq-chemgc(ians))/tem
      if(eex.gt.60.)return
      if(eex.lt.(-60.))return

       if(mod(igsp,2).ne.0)then
      d=-1.0
      if(eex.lt.1.0e-10.and.eex.gt.(-1.0e-10))return
       else
      d=1.0
       endif

      qgcfhf=(aspecs(ians)**2+x**2)*x**2/(exp(eex)+2.0*d+exp(-eex))

      return
      end

c-----------------------------------------------------------------
      function qgcfhn(x)
c-----------------------------------------------------------------
c integrand of hadron density
c-----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      eex=81.
      qgcfhn=0.0
      igsp=gspecs(ians)

      sq=sqrt(x**2+aspecs(ians)**2)
      if(tem.ne.0.0)eex=(sq-chemgc(ians))/tem
      if(eex.gt.80.)return

       if(mod(igsp,2).ne.0)then
      d=-1.0
      if(eex.lt.1.e-10)return  
       else
      d=1.0
       endif 

      qgcfhn=x**2/(exp(eex)+d)

      return
      end

c-----------------------------------------------------------------
      function qgcfhw(x)
c-----------------------------------------------------------------
c integrand of mean square variance of hadron yield
c----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      eex=61
      qgcfhw=0.0
      igsp=gspecs(ians)

      sq=sqrt(x**2+aspecs(ians)**2)
      if(tem.ne.0.0)eex=(sq-chemgc(ians))/tem
      if(eex.gt.60.)return 
      if(eex.lt.(-60.))return 

       if(mod(igsp,2).ne.0)then
      d=-1.0
      if(eex.lt.1.0e-10.and.eex.gt.(-1.0e-10))return  
       else
      d=1.0
       endif 

      qgcfhw=x**2/(exp(eex)+2.0*d+exp(-eex))

      return
      end


c-----------------------------------------------------------------
      subroutine qgchac(iboco)
c------------------------------------------------------------------
c returns hadronic chemical potentials as combinations of quark
c chemical potentials
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter(nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)

       do i=1,nspecs
      chemgc(i)=0.0
      do ii=1,nflavs
      chemgc(i)=chemgc(i)+ifok(ii,i)*chem(ii)
      if(ish.ge.99)write(ifch,*)'mu_i:',chem(ii),' k_i:',ifok(ii,i)
      enddo
      if(ish.ge.99)write(ifch,*)'mu_nu:',chemgc(i)
      igsp=gspecs(i)
      if(mod(igsp,2).ne.0.and.chemgc(i).gt.aspecs(i).and.iboco.eq.0)
     *chemgc(i)=aspecs(i)
       enddo

      return
      end


c-----------------------------------------------------------------------
      subroutine qgclim(a,b)
c----------------------------------------------------------------------
c returns integration limits for numerical evaluation of particle
c and energy densities using quantum statistics
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm

      iug=(1+iospec)/2*2-1
      igsp=gspecs(ians)

       if(mod(igsp,2).ne.0)then
      a=0.001
       else
      a=0.0
       endif

      b=0.0
      bb=(chemgc(ians)+tem*80.)**2-aspecs(ians)**2
      if(ish.ge.99)write(ifch,*)'bb:',bb
      if(bb.ge.0.0)b=sqrt(bb)
      if(bb.lt.0.0)then
      if(ish.ge.99)write(ifch,*)'failure at qgclim, bb=',bb
      if(ish.ge.99)write(ifch,'(1x,a,i5,a,2x,f12.6,1x,a,2x,f9.6)')
     *'mu(',ispecs(ians),'):',chemgc(ians),' T:',tem
      endif
      if(ish.ge.99)write(ifch,*)'ians:',ians,' a:',a,' b:',b
      return
      end


c------------------------------------------------------------------------
      subroutine qgcnbi
c-----------------------------------------------------------------------
c uses qgcaaa results to generate initial hadron set, nlattc, iozero
c input:
c    ptlngc(1:nspecs): particle number expectation values  /cgchg/
c output:
c     nump:           number of hadrons   /cqnbin/
c     ihadro(1:nump): hadron ids          /cqnbin/
c     nlattc:         lattice size        /clatt/
c     iozero:         zero weight         /paro6/
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/parod/ioinfl,ioinct,iowidn,epsgc
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/cqnbin/nump,ihadro(maxp)
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cgctot/rmstot,ptltot
      common/camgc/amgc,samgc,amtot
      common/cinfla/keu,ked,kes,kec,keb,ket
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      parameter (nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/clatt/nlattc,npmax
      common/cgcnb/nptlgc(mspecs)
      common/ctaue/taue
      integer k(nflav),kp(nflav),kps(nflav)
     *,idp(maxp),ida(mspecs),idb(mspecs)
      integer qgcndn
      isho=ish
      if(ishsub/100.eq.50)ish=mod(ishsub,100)

      if(ish.ge.94)write(ifch,*)('-',l=1,10)
     *,' entry sr qgcnbi ',('-',l=1,30)


      nh=nint(ptltot)
      iug=(1+iospec)/2*2-1

c     determine nlattc
c     ----------------
        if(ionlat.eq.1)then
      s1=ptltot+2.*rmstot
      s2=1.3*ptltot
      s=max(s1,s2,6.)
      nlattc=nint(s)
       elseif(ionlat.eq.2)then
      s1=ptltot+3.*rmstot
      s2=1.5*ptltot
      s=max(s1,s2,6.)
      nlattc=nint(s)
       elseif(ionlat.eq.3)then
      s1=ptltot+4.*rmstot
      s2=2.*ptltot
      s=max(s1,s2,6.)
      nlattc=nint(s)
        elseif(ionlat.eq.0)then
      nlattc=8*(tecm/10)*(1/(tecm/volu))**0.2*(nspecs/3.)**0.3
      if(aspecs(1).lt.0.010)nlattc=nlattc*3
      nlattc=max(nlattc,20)
        endif

      if(ish.ge.94)write(ifch,*)'nlattc:',nlattc

c     determine iozero
c     ----------------
      if(iozero.eq.-1)then
      iozero=nspecs
      elseif(iozero.eq.-2)then
      iozero=nspecs*sqrt(volu/tecm)
      endif

c     modify iozero for testing
c     -------------------------
      if(iozevt.gt.0)then
      iozero=(nrevt/iozevt+1)*iozinc   !nrevt=event number - 1 !!
      write(ifch,*)'nrevt+1:',nrevt+1,'   iozero:',iozero
      endif

c     initial hadron set
c     ------------------
      ammin=2.*aspecs(1)
      if(tecm.lt.ammin)then
      write(ifch,*)'impossible to generate hadron configuration'
      call utstop('qgcnbi: tecm less than two pi0 masses   ')
      endif
      
      kk=1
100   continue

       if(kk.gt.10)then
      if(ish.ge.94)
     *write(ifch,*)'failed to generate hadron set for'
     *,' event:',nrevt+1
      if(ish.ge.94)write(ifch,*)'u d s :',keu,ked,kes,' E:',tecm
      call qnbmin(keu,ked,kes,kec)
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcnbi ',('-',i=1,10)
      ish=isho
      return
        endif

      amtot=0.0
      do i=1,nspecs
      nptlgc(i)=0
      enddo
      do ii=1,nflavs
      k(ii)=kef(ii)
      enddo

      if(ish.ge.96)write(ifch,*)'sample hadron multiplicities and total 
     *mass:'

      kbar=keu+ked+kes
      kpar=iabs(keu)+iabs(ked)+iabs(kes)
      nbar=kbar/3.
      if(ish.ge.96)write(ifch,*)'baryon number:',nbar,' parton number:'
     *,kpar

      nn=2
      if(ioinco.ne.2)then
      nn=qgcndn(0)
      else
      nn=nh
      endif
      nb=iabs(nbar)
      if(ish.ge.96)write(ifch,*)'<n>:',nh,' n_sample:',nn,' n_bar:',nb
      if(nn.gt.nb.and.nb.ne.0.and.nb.ge.nh)nn=nb
      if(nn.lt.nb.and.nb.ne.0)nn=nb
      km=kpar-iabs(kbar)
      nt=km/2+nb
      if(nt.gt.nn)nn=nt
      nn=max(nn,2)

      if(ioinco.eq.2)then
      nit=15*taue
      else
      itpn=50
      nit=nn*itpn
      endif
      nbb=0
      n=0

       do it=1,nit

      if(nbb.lt.nb.and.iug.eq.9)then
      xsp=36
      x0=18.5
      else
      xsp=nspecs
      x0=0.5
      endif
      xib=x0+xsp*rangen()
      ib=nint(xib)
      if(ib.gt.nspecs)ib=nspecs
      if(ib.lt.1)ib=1
      kb=ifok(1,ib)+ifok(2,ib)+ifok(3,ib)
      if(rangen().lt.0.5.and.nptlgc(ib).ge.1)then
      ni=-1
      else
      ni=1
      endif
      as=1.0
      if(nptlgc(ib).eq.0)as=0.5
      if(nptlgc(ib).eq.1.and.ni.eq.(-1))as=2.0
      if(ish.ge.98)write(ifch,*)
     *'id:',ispecs(ib),' <i>:',ptlngc(ib),' ni:',ni

         if(ni.ne.0)then

       if(ptlngc(ib).gt.5.0)then

      pnla=qgcpnl(ib,0)
      pnlb=qgcpnl(ib,ni)
      pnlog=-pnla+pnlb
      if(ish.ge.98)write(ifch,*)'pnlog:',pnlog
      if(pnlog.lt.60)then
      pn=exp(pnlog)
      else 
      pn=1.1
      endif

       else

      if(ni.eq.1)then
      pn=ptlngc(ib)/(nptlgc(ib)+1)
      elseif(ni.eq.(-1).and.ptlngc(ib).gt.1.e-20)then
      pn=nptlgc(ib)/ptlngc(ib)
      elseif(nptlgc(ib).gt.0)then
      pn=1.1
      else
      pn=0.0
      endif

       endif

       pm=1.0
       if(ioinfl.ge.0)then
      pmla=qgcpml(ib,0,ib,0)
      pmlb=qgcpml(ib,ni,ib,0)
      pmlog=-pmla+pmlb
      if(ish.ge.98)write(ifch,*)'pmlog:',pmlog
      if(pmlog.lt.60)then
      pm=exp(pmlog)
      else
      pm=1.1
      endif
       endif

      p=pn*pm*as
      r=rangen()
      if(r.le.p)then
      nptlgc(ib)=nptlgc(ib)+ni
      n=n+ni
      amtot=amtot+ni*aspecs(ib)
      do ii=1,nflavs
      k(ii)=k(ii)-ifok(ii,ib)*ni
      enddo
      if(kb.ne.0)nbb=nbb+ni
      if(ish.ge.96.and.ni.gt.0)write(ifch,*)'add:'
      if(ish.ge.96.and.ni.lt.0)write(ifch,*)'remove:'
      if(ish.ge.96)write(ifch,*)'id:',ispecs(ib),' <n_i>:',ptlngc(ib)
     *,' n_i:',nptlgc(ib)
      if(ish.ge.96)write(ifch,*)'<n>:',nn,' it:',it
      if(ish.ge.96)write(ifch,*)'<M>:',amgc,' M:',amtot
      if(ish.ge.96)write(ifch,*)'p:',p,' r:',r
      if(ish.ge.96)write(ifch,*)'flav defect: u:',k(1),' d:'
     *,k(2),' s:',k(3)
      if(n.ge.nn.and.ioinco.ne.2)goto102
      endif

       endif

       enddo


102   continue
      
       ndd=0
c      if(nbb.lt.nb)then
c      nba=nb-nbb
c     if(nbar.gt.0)then
c     if(ish.ge.96)write(ifch,*)'add protons: nba:',nba 
c     nptlgc(19)=nptlgc(19)+nba
c     n=n+nba
c     amtot=amtot+aspecs(19)*nba
c     elseif(nbar.lt.0)then
c     if(ish.ge.96)write(ifch,*)'add aprotons: nba:',nba 
c     nptlgc(20)=nptlgc(20)+nba
c     n=n+nba
c     amtot=amtot+aspecs(20)*nba
c     endif
c      endif
       if(n.lt.nn.and.ioinco.ne.2)then
      ndd=nn-n
      nd=mod(ndd,4)
      xn=n
      xnn=nn
      xl=(xnn-xn)/4.
      l=aint(xl)
      if(ish.ge.96)write(ifch,*)'add pions/etas: ndd:',ndd
     *,' l:',l,' nd:',nd
        if(l.ge.1)then
       do j=1,l
      nptlgc(1)=nptlgc(1)+1
      nptlgc(2)=nptlgc(2)+1
      nptlgc(3)=nptlgc(3)+1
      nptlgc(4)=nptlgc(4)+1
      amtot=amtot+aspecs(1)+aspecs(2)+aspecs(3)+aspecs(4)
       enddo
        endif
      if(nd.eq.1)then
      nptlgc(1)=nptlgc(1)+1
      amtot=amtot+aspecs(1)
      elseif(nd.eq.2)then
      nptlgc(1)=nptlgc(1)+1
      nptlgc(4)=nptlgc(4)+1
      amtot=amtot+aspecs(1)+aspecs(4)
      elseif(nd.eq.3)then
      nptlgc(2)=nptlgc(2)+1
      nptlgc(3)=nptlgc(3)+1
      nptlgc(4)=nptlgc(4)+1
      amtot=amtot+aspecs(2)+aspecs(3)+aspecs(4)
      endif
       endif

       if(n.eq.0.and.ioinco.eq.2)then
      nptlgc(2)=nptlgc(2)+1
      nptlgc(3)=nptlgc(3)+1
      amtot=amtot+aspecs(2)+aspecs(3)
       elseif(n.eq.1.and.ioinco.eq.2)then
      nptlgc(1)=nptlgc(1)+1
      amtot=amtot+aspecs(1)
       endif

      if(amtot.ge.tecm.and.ioinfl.ge.0)then
      if(ish.ge.96)write(ifch,*)
     *'total mass exceeded , redo configuration'
      kk=kk+1
      goto100
      endif
 

      iii=0
      if(ish.ge.94)write(ifch,*)'u d s :',keu,ked,kes,' E:',tecm
      if(ish.ge.94)write(ifch,*)
     *'hadron set without flavor conservation:'
      do i=1,nspecs
      n=nptlgc(i)
      if(n.ge.1)then
      do j=1,n
      iii=iii+1
      idp(iii)=ispecs(i)
      enddo
      endif
      enddo
      if(ish.ge.94)write(ifch,'(1x,10i6)')(idp(i),i=1,iii)
      if(ish.ge.94)write(ifch,*)'flav defect: u:',k(1),' d:'
     *,k(2),' s:',k(3)
      if(ish.ge.94)write(ifch,*)'M:',amtot,' <M>:',amgc
      if(ioinfl.le.0)goto1000

      ll=1
      llmax=nn*25
      ior=1

120        if(k(1).ne.0.or.k(2).ne.0.or.k(3).ne.0)then

        if(kk.gt.6)ior=0

      if(ish.ge.96)write(ifch,*)
     *'remaining flavor defect before operation:',ll
      if(ish.ge.96)write(ifch,*)'flav defect: u:',k(1),' d:'
     *,k(2),' s:',k(3)

      nida=0
      do i=1,nspecs
      if(nptlgc(i).gt.0)then
      nida=nida+1
      ida(nida)=i
      endif
      enddo

      if(nida.eq.0)then
      if(ish.ge.96)write(ifch,*)'no proposals in a , redo'
      kk=kk+1
      goto100
      endif


      xna=0.5+nida*rangen()
      na=nint(xna)
      if(na.gt.nida)na=nida
      if(na.lt.1)na=1
      ia=ida(na)
      if(ish.ge.96)write(ifch,*)'nida:',nida,' ia:',ia

      nidb=0
      iasym=1
      do ii=1,nflavs
      kp(ii)=k(ii)+ifok(ii,ia)
      kps(ii)=isign(1,kp(ii))
      enddo
      if(ish.ge.96)write(ifch,*)
     *'   assemble: u:',kp(1),' d:',kp(2),' s:',kp(3)
      do i=1,nspecs
      iacc=0
      naccsp=0
      naccmi=1
      do ii=1,nflavs
      naccsp=naccsp+iabs(ifok(ii,i))
      if(kp(ii).ne.0)then
      if(kps(ii)*ifok(ii,i).le.kps(ii)*kp(ii)
     *.and.kps(ii)*ifok(ii,i).gt.0)iacc=iacc+iabs(ifok(ii,i))
      endif
      enddo
      if(kp(1).eq.0.and.kp(2).eq.0.and.kp(3).eq.0)naccmi=0
      if(iacc.eq.naccsp.and.naccsp.ge.naccmi)then
      nidb=nidb+1
      idb(nidb)=i
      if(i.eq.ia)iasym=0
      endif
      enddo

      if(nidb.eq.0)then
      if(ish.ge.96)write(ifch,*)'no proposals in b , redo'
      kk=kk+1
      goto100
      endif

      xnb=0.5+nidb*rangen()
      nb=nint(xnb)
      if(nb.gt.nidb)nb=nidb
      if(nb.lt.1)nb=1
      ib=idb(nb)
      if(ish.ge.96)write(ifch,*)'nidb:',nidb,' ib:',ib
      if(ish.ge.96)write(ifch,*)
     *'proposal:',ispecs(ia),' --> ',ispecs(ib)

      asym=1.0
      nidd=nida
      if(nptlgc(ia).eq.1)nidd=nidd-1
      if(nptlgc(ib).eq.0)nidd=nidd+1
c     if(iasym.eq.1)then
c     nidc=0
c     else
      nidc=nidb
c     endif
      if(nidc.gt.0.and.nidd.gt.0)then
      asym=nida*nidb/nidc/nidd
c     else
c     asym=0.0
      endif

       if(asym.gt.0.0)then

       if(ptlngc(ia).gt.5.0)then
      pnali=qgcpnl(ia,0)
      pnalf=qgcpnl(ia,-1)
      pnalog=-pnali+pnalf
      if(ish.ge.96)write(ifch,*)'pnalog:',pnalog
      if(pnalog.lt.60)then
      pna=exp(pnalog)
      else
      pna=1.1
      endif
       else
      if(ptlngc(ia).gt.1.e-20)then
      pna=nptlgc(ia)/ptlngc(ia)
      elseif(nptlgc(ia).gt.0)then
      pna=1.1
      else
      pna=0.0
      endif
       endif
      
       if(ptlngc(ib).gt.5.0)then
      pnbli=qgcpnl(ib,0)
      pnblf=qgcpnl(ib,1)
      pnblog=-pnbli+pnblf
      if(ish.ge.96)write(ifch,*)'pnblog:',pnblog
      if(pnblog.lt.60)then
      pnb=exp(pnblog)
      else
      pnb=1.1
      endif
       else 
      pnb=ptlngc(ib)/(nptlgc(ib)+1)
       endif


      pmli=qgcpml(ia,0,ib,0)
      pmlf=qgcpml(ia,-1,ib,1)
      pmlog=-pmli+pmlf
      if(ish.ge.96)write(ifch,*)'pmlog:',pmlog
      if(pmlog.lt.60)then
      pm=exp(pmlog)
      else
      pm=1.1
      endif

      p=pna*pnb*pm*asym
      if(ior.eq.0)then
      r=0.0
      else
      r=rangen()
      endif
 
       else

      r=1.0
      p=0.0

       endif

       if(r.lt.p)then
      if(ish.ge.96)write(ifch,*)'p:',p,' r:',r,' asymmetry:',asym
      if(ish.ge.96)write(ifch,*)'remove ',ispecs(ia),'  add ',ispecs(ib)
     *,'  proposal accepted'
      nptlgc(ia)=nptlgc(ia)-1
      nptlgc(ib)=nptlgc(ib)+1
      amtot=amtot-aspecs(ia)+aspecs(ib)
      do ii=1,nflavs
      k(ii)=k(ii)+ifok(ii,ia)-ifok(ii,ib)
      enddo
       endif

        
        if(k(1).ne.0.or.k(2).ne.0.or.k(3).ne.0)then
       ll=ll+1
      if(ll.le.llmax)then
      goto120
      else
      if(ish.ge.96)write(ifch,*)'failed to remove defect, redo'
      kk=kk+1
      goto100
       endif
        endif

         endif

1000  continue

      nump=0
      kcu=0
      kcd=0
      kcs=0
      do i=1,nspecs
      n=nptlgc(i)
      if(n.ge.1)then
      do j=1,n
      nump=nump+1
      ihadro(nump)=ispecs(i)
      kcu=kcu+ifok(1,i)
      kcd=kcd+ifok(2,i)
      kcs=kcs+ifok(3,i)
      enddo
      endif
      enddo

          if(ioinfl.gt.0)then
        if(kcu.ne.keu.or.kcd.ne.ked.or.kcs.ne.kes)then
      if(ish.ge.96)write(ifch,*)
     *'failed to remove flavor defect, redo configuration'
      kk=kk+1
      goto100
        endif
          endif

      if(ioinct.ge.1)then
        chitot=0.0
        nutot=nspecs
        do i=1,nspecs
        chi=0.0
        if(rmsngc(i).gt.1.e-10)chi=(ptlngc(i)-nptlgc(i))/rmsngc(i)
        chitot=chitot+chi**2
        enddo
        call xchico(chitot)
 
        u=0
        d=0
        s=0
        do i=1,nspecs
        u=u+ifok(1,i)*nptlgc(i)
        d=d+ifok(2,i)*nptlgc(i)
        s=s+ifok(3,i)*nptlgc(i)
        enddo
        call xchfla(nint(u),nint(d),nint(s),0)
        call xchamt(amtot,0)
      endif

      if(ish.ge.94)write(ifch,*)
     *'initial hadron set for droplet decay:'
      if(ish.ge.94)write(ifch,'(1x,10i6)')(ihadro(i),i=1,nump)
       if(nump.ge.nlattc)then
       nlattc=nump+1
       if(ish.ge.94)write(ifch,*)'initial set > nlattc !'
       if(ish.ge.94)write(ifch,*)'new nlattc:',nlattc
       endif
      if(ish.ge.94)write(ifch,*)'keu:',kef(1),' kcu:',kcu,' ku:',k(1)
      if(ish.ge.94)write(ifch,*)'ked:',kef(2),' kcd:',kcd,' kd:',k(2)
      if(ish.ge.94)write(ifch,*)'kes:',kef(3),' kcs:',kcs,' ks:',k(3)
      if(ish.ge.94)write(ifch,*)' nh:',nh,' nump:',nump
      if(ish.ge.94)write(ifch,*)' nu:',nutot,'  chi^2:',chitot
      if(ish.ge.94)write(ifch,*)'iozero:',iozero,'  iomom:',iomom
      if(ish.ge.94)write(ifch,*)
     *'total mass:',amtot,' droplet mass:',tecm
      if(ish.ge.94)write(ifch,*)'trials needed:',kk
     *,' operations needed:',ll
      if(ish.ge.94)write(ifch,*)'iterations:',it,' pions added:',ndd
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qgcnbi ',('-',i=1,10)
      ish=isho
      return

      end

c--------------------------------------------------------------------
      integer function qgcndn(i)
c--------------------------------------------------------------------
c returns random multiplicity from gaussian distribution for species i
c---------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (mspecs=54)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cgctot/rmstot,ptltot
      common/clatt/nlattc,npmax
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/parod/ioinfl,ioinct,iowidn,epsgc
      a=iowidn
      kk=0

       if(i.eq.0)then

1     continue
      kk=kk+1
      p=0.0
      nmin=2
      nh=nint(ptltot)
      nmax=nlattc
      xn=1.5+(nmax-nmin)*rangen()
      n=nint(xn)
      x=(n-ptltot)**2/2.0
      y=-70.
      if(rmstot.gt.1.e-15)y=-x/rmstot**2*a**2
      if(y.lt.70.)p=exp(y)
      if(rmstot.gt.1.e-15.and.iowidn.lt.0)p=p/sqrt(2.*pi)/rmstot
      if(p.ge.rangen())then
      qgcndn=n
      if(ish.ge.98)write(ifch,*)'qgcndn: k:',kk,' n:',qgcndn
      return
      else
      if(kk.le.25)goto1
      qgcndn=max(2,nh)
      if(ish.ge.98)write(ifch,*)'qgcndn: k:',kk,' n:',qgcndn
      return
      endif

       else
  
2     continue
      kk=kk+1
      p=0.0
      nmin=0
      nh=nint(ptlngc(i))
      nmax=2*nh
      nmax=max(2,nmax)
      xn=-0.5+(nmax-nmin)*rangen()
      n=nint(xn)
      x=(n-ptlngc(i))**2/2.0
      if(x.lt.1.e-30)then
      p=1.
      else
      y=-70.
      if(rmsngc(i).gt.1.e-15)y=-x/rmsngc(i)**2
      if(y.lt.70.)p=exp(y)
      if(rmsngc(i).gt.1.e-15.and.iowidn.lt.0)
     *p=p/sqrt(2.*pi)/rmsngc(i)
      endif
      if(p.ge.rangen())then
      qgcndn=n
      if(ish.ge.98)write(ifch,*)'qgcndn: k:',kk,' n:',qgcndn
      return
      else
      if(kk.le.25)goto2
      qgcndn=nh
      if(ish.ge.98)write(ifch,*)'qgcndn: k:',kk,' n:',qgcndn
      return
      endif

       endif

      end

c--------------------------------------------------------------------
      function qgcpml(i1,n1,i2,n2)
c--------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/camgc/amgc,samgc,amtot
      common/cgcnb/nptlgc(mspecs)
      if(ish.ge.99)write(ifch,*)'i1:',i1,' i2:',i2
      if(ish.ge.99)write(ifch,*)'n1:',n1,' n2:',n2
      qgcpml=-1.e30
      ampr=n1*aspecs(i1)+n2*aspecs(i2)
      if((amtot+ampr).lt.tecm.and.(amtot+ampr).ge.0
     *.and.nptlgc(i1).ge.(-n1).and.nptlgc(i2).ge.(-n2))then
      qgcpml=0.0
      pl=(amtot-amgc+ampr)**2/2.0
      if(pl.lt.1.e-30)then
      qgcpml=0.0
      return
      endif
      if(samgc.gt.1.e-15)qgcpml=-pl/samgc**2
      endif
      if(ish.ge.99)write(ifch,*)'qgcpml:',qgcpml
      return
      end

c--------------------------------------------------------------------
      function qgcpnl(i,n)
c--------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (mspecs=54)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cgcnb/nptlgc(mspecs)
      if(ish.ge.99)write(ifch,*)'i:',i,' n:',n
      qgcpnl=-1.e30
      if(nptlgc(i).ge.(-n))then
      pl=(nptlgc(i)-ptlngc(i)+n)**2/2.0
      if(pl.lt.1.e-30)then
      qgcpnl=0.0
      return
      endif
      if(rmsngc(i).gt.1.e-15)qgcpnl=-pl/rmsngc(i)**2
      endif
      if(ish.ge.99)write(ifch,*)'qgcpnl:',qgcpnl
      return
      end


c--------------------------------------------------------------------
      subroutine qgcpen
c--------------------------------------------------------------------
c returns array for twodimensional plot of energy- and flavor-
c density
c--------------------------------------------------------------------
c xpar1,xpar2 temperature range
c xpar3       # of bins for temperature
c xpar4,xpar5 chem.pot. range
c xpar6       # of bins for chem.pot.
c xpar7       max. density
c xpar8       strange chem.pot.
c--------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter (nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/cinfla/keu,ked,kes,kec,keb,ket
      common/cgctot/rmstot,ptltot
      common/camgc/amgc,samgc,amtot
      common/ciakt/gen,iafs,ians,genm
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (nbin=100)
      real edensi(nbin,nbin),qdensi(nbin,nbin)
      external qgcfhe
      external qgcfhn
      external qgcfbe
      external qgcfbn

      iug=(1+iospec)/2*2-1

c     initialization
c     --------------

      if(iug.eq.1)nflavs=1
      if(iug.eq.3)nflavs=2
      if(iug.eq.5)nflavs=2
      if(iug.eq.7)nflavs=3
      if(iug.eq.9)nflavs=3
      if(iug.eq.11)nflavs=3
      tem=0.0
      do i=1,nflavs
      chem(i)=0.0
      enddo
      call qgchac(0)
      do i=1,nspecs
      ptlngc(i)=0.0
      rmsngc(i)=0.0
      enddo

      nbt=nint(xpar3)
      nbc=nint(xpar6)
      nbc=min(nbc,100)
      nbt=min(nbt,100)
      dt=(xpar2-xpar1)/nbt
      dc=(xpar5-xpar4)/nbc
      ymax=xpar7
      cs=xpar8


      t0=xpar1+dt/2.
      c0=xpar4+dc/2
      do i=1,nbc
      chem(1)=c0+(i-1)*dc
      chem(2)=chem(1)
      chem(3)=cs
      chem(4)=0.0
      chem(5)=0.0
      chem(6)=0.0
      call qgchac(0)
      do ii=1,nbt
      tem=t0+(ii-1)*dt
      if(ish.ge.94)write(ifch,*)' mu:',chem(1),' T:',tem

       qd=0.0
       ed=0.0

      do ians=1,nspecs

      call qgclim(a,b)

      if(b.eq.0.0)then
      hden=0.0
      elseif(iostat.eq.0)then
      call uttraq(qgcfhn,a,b,hden)
      elseif(iostat.eq.1)then
      call uttraq(qgcfbn,a,b,hden)
      endif
      hd=hden*gspecs(ians)/2./pi**2/hquer**3
      
      if(ish.ge.96)write(ifch,*)'i:',ians,' n_u:',ifok(1,ians),' hd:',hd

      qd=qd+ifok(1,ians)*hd+ifok(2,ians)*hd   
      if(qd.gt.ymax)qd=ymax
c     if(qd.gt.ymax)qd=0.0
      if(qd.lt.-ymax)qd=-ymax
c     if(qd.lt.-ymax)qd=0.0
      

      if(b.eq.0.0)then
      edi=0.0
      elseif(iostat.eq.0)then
      call uttraq(qgcfhe,a,b,edi)
      elseif(iostat.eq.1)then
      call uttraq(qgcfbe,a,b,edi)
      endif
      edi=edi*gspecs(ians)/2./pi**2/hquer**3

      if(ish.ge.96)write(ifch,*)'i:',ians,' mu:',chemgc(ians),' edi:',edi
 
      ed=ed+edi
      if(ed.gt.ymax)ed=ymax
c     if(ed.gt.ymax)ed=0.0
      enddo

      if(ish.ge.94)write(ifch,*)' ed:',ed,' qd:',qd
      edensi(i,ii)=ed
      qdensi(i,ii)=qd
      
      enddo
      enddo

      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a,2e11.3)')'xrange',xpar1,xpar2
      write(ifhi,'(a,2e11.3)')'yrange',xpar4,xpar5
      write(ifhi,'(a)')      'set ityp2d 5'
      write(ifhi,'(a,i4)')   'array2d',nbt
      do j=1,nbc
      do jj=1,nbt
      write(ifhi,'(e11.3)') edensi(j,jj)
      enddo
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot2d'

      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a,2e11.3)')'xrange',xpar1,xpar2
      write(ifhi,'(a,2e11.3)')'yrange',xpar4,xpar5
      write(ifhi,'(a)')      'set ityp2d 5'
      write(ifhi,'(a,i4)')   'array2d',nbt
      do j=1,nbc
      do jj=1,nbt
      write(ifhi,'(e11.3)') qdensi(j,jj)
      enddo
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot2d'

        return
        end

c--------------------------------------------------------------------
      subroutine qgcpfl
c--------------------------------------------------------------------
c returns array for twodimensional plot of energy- and flavor-
c density fluctuations
c--------------------------------------------------------------------
c xpar1,xpar2 temperature range
c xpar3       # of bins for temperature
c xpar4,xpar5 chem.pot. range
c xpar6       # of bins for chem.pot.
c xpar7       max. density
c xpar8       strange chem.pot.
c--------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter (nflav=6)
      common/cflavs/nflavs,kef(nflav),chem(nflav)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/cinfla/keu,ked,kes,kec,keb,ket
      common/cgctot/rmstot,ptltot
      common/camgc/amgc,samgc,amtot
      common/ciakt/gen,iafs,ians,genm
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (nbin=100)
      real efl(nbin,nbin),qfl(nbin,nbin),v(nbin),wn(nbin),we(nbin)
      external qgcfhf
      external qgcfhe
      external qgcfhn
      external qgcfhw
      external qgcfbf
      external qgcfbe
      external qgcfbn

      iug=(1+iospec)/2*2-1

c     initialization
c     --------------

      if(iug.eq.1)nflavs=1
      if(iug.eq.3)nflavs=2
      if(iug.eq.5)nflavs=2
      if(iug.eq.7)nflavs=3
      if(iug.eq.9)nflavs=3
      if(iug.eq.11)nflavs=3
      tem=0.0
      do i=1,nflavs
      chem(i)=0.0
      enddo
      call qgchac(0)
      do i=1,nspecs
      ptlngc(i)=0.0
      rmsngc(i)=0.0
      enddo

      nbt=nint(xpar3)
      nbv=nint(xpar6)
      nbv=min(nbv,100)
      nbt=min(nbt,100)
      dt=(xpar2-xpar1)/nbt
      dv=(xpar5-xpar4)/nbv
      ymax=1.e20
      chem(1)=xpar7
      chem(2)=xpar7
      chem(3)=xpar8
      call qgchac(0)


      t0=xpar1+dt/2.
      v0=xpar4
      do i=1,nbv
      volu=v0+(i-1)*dv
      do ii=1,nbt
      tem=t0+(ii-1)*dt
      if(ish.ge.94)write(ifch,*)'volu:',volu,' tem:',tem

       ev=0.0
       ee=0.0
       qv=0.0
       qe=0.0

      do ians=1,nspecs

      call qgclim(a,b)

      if(b.eq.0.0)then
      hn=0.0
      hv=0.0
      elseif(iostat.eq.0)then
      call uttraq(qgcfhn,a,b,hn)
      call uttraq(qgcfhw,a,b,hv)
      elseif(iostat.eq.1)then
      call uttraq(qgcfbn,a,b,hn)
      hv=hn
      endif
      hn=hn*volu*gspecs(ians)/2./pi**2/hquer**3
      hv=hv*volu*gspecs(ians)/2./pi**2/hquer**3
      if(ish.ge.94)write(ifch,*)'hn:',hn,' hv:',hv
      
      hn=max(hn,1.e-15)
      qv=qv+hv
      qe=qe+hn


      if(qv.gt.ymax)qv=ymax
      if(qe.gt.ymax)qe=ymax
      

      if(b.eq.0.0)then
      eei=0.0
      evi=0.0
      elseif(iostat.eq.0)then
      call uttraq(qgcfhe,a,b,eei)
      call uttraq(qgcfhf,a,b,evi)
      elseif(iostat.eq.1)then
      call uttraq(qgcfbe,a,b,eei)
      call uttraq(qgcfbf,a,b,evi)
      endif
      eei=eei*volu*gspecs(ians)/2./pi**2/hquer**3
      evi=evi*volu*gspecs(ians)/2./pi**2/hquer**3
      if(ish.ge.94)write(ifch,*)'eei:',eei,' evi:',evi

 
      eei=max(eei,1.e-15)
      ev=ev+evi
      ee=ee+eei
      if(ev.gt.ymax)ev=ymax
      if(ee.gt.ymax)ee=ymax
      enddo
      if(ish.ge.94)write(ifch,*)'qv:',qv,' ev:',ev

      qfl(i,ii)=0.
      efl(i,ii)=0.
      if(ev.gt.0.0.and.ee.gt.1.e-15)efl(i,ii)=sqrt(ev)/ee
      if(qv.gt.0.0.and.ee.gt.1.e-15)qfl(i,ii)=sqrt(qv)/qe
      if(tem.eq.0.195)then
      we(i)=efl(i,ii)
      wn(i)=qfl(i,ii)
      v(i)=volu
      endif
      
      enddo
      enddo

      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a,2e11.3)')'xrange',xpar1,xpar2
      write(ifhi,'(a,2e11.3)')'yrange',xpar4,xpar5
      write(ifhi,'(a)')      'set ityp2d 5'
      write(ifhi,'(a,i4)')   'array2d',nbt
      do j=1,nbv
      do jj=1,nbt
      write(ifhi,'(e11.3)') efl(j,jj)
      enddo
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot2d'

      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a,2e11.3)')'xrange',xpar1,xpar2
      write(ifhi,'(a,2e11.3)')'yrange',xpar4,xpar5
      write(ifhi,'(a)')      'set ityp2d 5'
      write(ifhi,'(a,i4)')   'array2d',nbt
      do j=1,nbv
      do jj=1,nbt
      write(ifhi,'(e11.3)') qfl(j,jj)
      enddo
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot2d'

      write(ifhi,'(a)')      'newpage zone 1 2 1'
      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a,2e11.3)')'xrange',xpar4,xpar5
      write(ifhi,'(a)')      'htyp lfu xmod lin ymod lin'
      write(ifhi,'(a,i4)')   'array 2'
      do j=1,nbv
      write(ifhi,'(2e11.3)')v(j),we(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0' 

      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a,2e11.3)')'xrange',xpar4,xpar5
      write(ifhi,'(a)')      'htyp lfu xmod lin ymod lin'
      write(ifhi,'(a,i4)')   'array 2'
      do j=1,nbv
      write(ifhi,'(2e11.3)')v(j),wn(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0'


        return
        end


c------------------------------------------------------------------
      subroutine qgcpyi(ist)
c------------------------------------------------------------------
c returns particle yield 
c input:
c   tem   : temperature
c   chemgc: chemical potentials
c output:
c   ptlngc: expectation value of particle number for each species
c   rmsngc: standard deviation of ptlngc
c   ptltot: total particle number
c   rmstot: standard deviation of ptltot
c works for hadrons and partons
c  ist=1 boltzmann statistics
c  ist=0 quantum statistics
c--------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cgctot/rmstot,ptltot
      common/camgc/amgc,samgc,amtot
      common/ciakt/gen,iafs,ians,genm
      external qgcfhw
      external qgcfhn

       if(iabs(ispecs(nspecs)).lt.10)then

c     parton yield
c     ------------
      if(ish.ge.94)write(ifch,*)'parton yield:'
      gln=16.*1.20206*tem**3/pi**2*volu/hquer**3
      sdg=sqrt(gln)   !!???
      if(ish.ge.94)write(ifch,'(1x,a,f10.4,2x,a,f9.4,a)')
     *'<N(    0)> :',gln,' sigma :',sdg,' (qm-statistics!)'
      ptltot=gln
      rmstot=0.0
      vartot=gln

       else

      if(ish.ge.94)write(ifch,*)'hadronic yield:'
      ptltot=0.0
      rmstot=0.0
      vartot=0.0

       endif

      amgc=0.0
      samgc=0.0

       do ians=1,nspecs

c     hadronic yield
c     --------------
       if(ist.eq.0)then

      call qgclim(a,b)
      if(b.eq.0.0)then
      hden=0.0
      else
      call uttraq(qgcfhn,a,b,hden)
      endif
      ptlngc(ians)=hden*volu*gspecs(ians)/2./pi**2/hquer**3

       else

       if((chemgc(ians)/tem).gt.70.)then
      hpd=1.e30
       else
      hpd=exp(chemgc(ians)/tem)
       endif
       if(aspecs(ians).ne.0.)then
      fk2=qgcbk(2,aspecs(ians)/tem)
      hpd=hpd*gspecs(ians)*aspecs(ians)**2*tem*fk2
     */2./pi**2/hquer**3
       else
      hpd=hpd*gspecs(ians)*tem**3/pi**2/hquer**3
       endif
      ptlngc(ians)=hpd*volu

       endif

      ptltot=ptltot+ptlngc(ians)
      amgc=amgc+ptlngc(ians)*aspecs(ians)
      if(amgc.ge.tecm)amgc=tecm*0.9

c     standard deviation
c     ------------------
      rmsngc(ians)=0.0

       if(ist.eq.0)then
 
      call uttraq(qgcfhw,a,b,var)
      var=var*gspecs(ians)*volu/2./pi**2/hquer**3
      vartot=vartot+var
      if(var.ge.0.0)rmsngc(ians)=sqrt(var)
      samgc=samgc+var*aspecs(ians)
 
       else

      if(ptlngc(ians).ge.0.0)rmsngc(ians)=sqrt(ptlngc(ians))
      vartot=vartot+ptlngc(ians)
      samgc=samgc+ptlngc(ians)*aspecs(ians)

       endif


      if(ish.ge.96)write(ifch,'(2x,a,i5,a,2x,f8.4,5x,a,3x,f8.4)')
     *'m(',ispecs(ians),')  :',aspecs(ians),'mu :',chemgc(ians)
      if(ish.ge.94)write(ifch,'(1x,a,i5,a,2x,f8.4,2x,a,2x,f10.4)')
     *'<N(',ispecs(ians),')> :',ptlngc(ians),'sigma :',rmsngc(ians)

       enddo
      
      if(vartot.ge.0.0)rmstot=sqrt(vartot)
      if(samgc.ge.0.0)samgc=sqrt(samgc)
      if(amgc.ge.tecm)samgc=sqrt(amgc)
      if(ish.ge.94)write(ifch,'(1x,a,2x,f8.4,2x,a,2x,f10.4)')
     *'<N(  all)> :',ptltot,'sigma :',rmstot
      if(ish.ge.94)write(ifch,'(1x,a,2x,f8.4,2x,a,2x,f10.4)')
     *'<M_tot>    :',amgc,'sigma :',samgc

      return 
      end

c------------------------------------------------------------------------
      subroutine qgctbo(iba)
c------------------------------------------------------------------------
c returns new tem using boltzmann statistics in analytic form
c  input:
c    chemgc
c    tecm/volu
c  output:
c    tem
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      external qgcbk
      external qgcbk1
      iba=0
      k=1
      t1=0.0
      t2=1.0
      
      goto15

10    tem=t1+.5*(t2-t1)
      if(tem.le.1.e-7)return
15    eden=0.0

        do i=1,nspecs

       if(aspecs(i).ne.0)then
      if(tem.ne.0.)ar=aspecs(i)/tem
      cba=(aspecs(i)/tem+12.*tem/aspecs(i)-3.*chemgc(i)/aspecs(i))
     **qgcbk(2,ar)+(3.-chemgc(i)/tem)*qgcbk1(ar)
       else
      cba=4.*tem-chemgc(i)
       endif

      if(cba.lt.0.0)then
      iba=1
      return
      endif

      if(tem.ne.0.)x=chemgc(i)/tem

       if(x.le.70.)then
      y=exp(x)
       else
      y=1.e30
       endif

       if(aspecs(i).ne.0.)then
      edi=y*(3./ar*qgcbk(2,ar)+qgcbk1(ar))
     **gspecs(i)*aspecs(i)**3*tem/2./pi**2/hquer**3
       else
      edi=y*3.*gspecs(i)*tem**4/pi**2/hquer**3
       endif

      eden=eden+edi

        enddo

      if(iabs(ispecs(nspecs)).lt.10)
     *eden=eden+(8.*pi**2*tem**4/15.+bag4rt**4)/hquer**3

      de=abs(eden-(tecm/volu))
      if(de.le.gen*(tecm/volu).or.de.le.genm)return
c     if(eden.ge.100.)return

       if(eden.gt.(tecm/volu))then
      t2=tem
      else
      t1=tem
       endif

       if(k.gt.300)return
 
      k=k+1
      goto10
      end

c----------------------------------------------------------------------
      subroutine qgctex
c----------------------------------------------------------------------
c returns new tem using massive quantum statistics in integral form
c  input:
c    chemgc
c    tecm/volu
c  output:
c    tem
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm
      external qgcfhe
      k=1
      t1=0.0
      t2=tem+0.1
      goto15

c     new temperature
c     ---------------
10    tem=t1+.5*(t2-t1)
15    continue
      if(tem.le.1.e-6)return
      eden=0.0
 
       do ians=1,nspecs
      call qgclim(a,b)
      if(b.eq.0.0)then
      edi=0.0
      else
      call uttraq(qgcfhe,a,b,edi)
      endif
      edi=edi*gspecs(ians)/2./pi**2/hquer**3
      eden=eden+edi
       enddo

      if(iabs(ispecs(nspecs)).lt.10)
     *eden=eden+(8.*pi**2*tem**4/15.+bag4rt**4)/hquer**3

      de=abs(eden-(tecm/volu))
      if(de.le.gen*(tecm/volu).or.de.le.genm)return

       if(eden.gt.(tecm/volu))then
      t2=tem
      else
      t1=tem
       endif

       if(k.gt.300)then
       if(ish.ge.94)
     *write(ifch,*)'failure in tex'
      return
       endif
 
      k=k+1
      goto10
      end

c-----------------------------------------------------------------
      subroutine qgctm0
c-----------------------------------------------------------------
c returns new tem using massless quantum statistics in analytic form
c  input:
c    chemgc
c    tecm/volu
c  output:
c    tem
c----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ciakt/gen,iafs,ians,genm

      k=1
      
      t1=0.0
      t2=1.0
10    tem=t1+.5*(t2-t1)
      if(tem.le.1.e-6)return
      eden=0.0

        do i=1,nspecs

      igsp=gspecs(i)
      if(mod(igsp,2).eq.0)then
      edhm0=7./240.*pi**2*tem**4+chemgc(i)**2*tem**2/8.
     *+chemgc(i)**4/pi**2/16.
      else
      edhm0=pi**2*tem**4/30.+chemgc(i)**2*tem**2/4.
     *-chemgc(i)**4/pi**2/16.
      endif
      edi=edhm0*gspecs(i)/hquer**3


      eden=eden+edi
        enddo

      if(iabs(ispecs(nspecs)).lt.10)
     *eden=eden+(8.*pi**2*tem**4/15.+bag4rt**4)/hquer**3

      de=abs(eden-(tecm/volu))
      if(de.le.gen*(tecm/volu).or.de.le.genm)return

       if(eden.gt.(tecm/volu))then
      t2=tem
      else
      t1=tem
       endif

       if(k.gt.300)then
       if(ish.ge.94)
     *write(ifch,*)'failure in tm0'
      return
       endif
 
      k=k+1
      goto10
      end

c----------------------------------------------------------------------
      subroutine qnbaaa
c----------------------------------------------------------------------
c  decays droplet specified via keu...ket, tecm, volu
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common /cttaus/  tpro,zpro,ttar,ztar,ttaus,detap,detat 
      parameter (nflav=6)
      integer jc(nflav,2),ic(2)
      if(ish.ge.94)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbaaa ',('-',i=1,30)

      nevt=0
      nptl=0

      ttaus=1
      taus=ttaus
      ypjtl=6
      yhaha=3
      etapro=(ypjtl-yhaha)*etafac
      etatar=-yhaha*etafac
      detap=etapro
      detat=etatar
      tpro=dcosh(detap)
      zpro=dsinh(detap)
      ttar=dcosh(detat)
      ztar=dsinh(detat)
      labsys=0

      if(ish.ge.94)write(ifch,'(a/6i4)')
     *' keu ked kes kec keb ket:',keu,ked,kes,kec,keb,ket
      do i=1,6
      jc(i,1)=0
      jc(i,2)=0
      enddo
      if(keu.ge.0)jc(1,1)=keu
      if(ked.ge.0)jc(2,1)=ked
      if(kes.ge.0)jc(3,1)=kes
      if(kec.ge.0)jc(4,1)=kec
      if(keb.ge.0)jc(5,1)=keb
      if(ket.ge.0)jc(6,1)=ket
      if(keu.lt.0)jc(1,2)=-keu
      if(ked.lt.0)jc(2,2)=-ked
      if(kes.lt.0)jc(3,2)=-kes
      if(kec.lt.0)jc(4,2)=-kec
      if(keb.lt.0)jc(5,2)=-keb
      if(ket.lt.0)jc(6,2)=-ket
      if(ish.ge.94)write(ifch,'(a/6i4/6i4)')' jc:',jc
      call idenco(jc,ic,ireten)
      if(ireten.eq.1)
     *call utstop('ametro: idenco ret code = 1             ')
      if(ic(1).eq.0.and.ic(2).eq.0)then
      ic(1)=100000
      ic(2)=100000
      endif
      id=8*10**8+ic(1)*100+ic(2)/100
      if(ish.ge.94)write(ifch,'(a,i9)')' id:',id

      pptl(1,1)=0
      pptl(2,1)=0
      pptl(3,1)=0
      pptl(4,1)=tecm
      pptl(5,1)=tecm
      dez=0.5e-4
      call uttauh(1,-dez,dez,g,dum,wsmax)
      if(ish.ge.94)write(ifch,'(a,f9.4)')' g:',g

      nptl=nptl+1
      idptl(nptl)=id
      pptl(1,nptl)=0
      pptl(2,nptl)=0
      pptl(3,nptl)=0
      pptl(4,nptl)=tecm*g
      pptl(5,nptl)=tecm*g
      dezptl(nptl)=dez
      dez=0.5e-4
      radptl(nptl)=sqrt(volu/pi/2/dez/taus)
      istptl(nptl)=0

      np1=1
21    np2=nptl
      do i=np1,np2
      call jdecas(i,iret)
      if(iret.eq.1)stop'STOP: sr qnbaaa: jdecas return code = 1'
      enddo
      np1=np2+1
      if(np1.le.nptl) goto21

      nevt=1
      nrevt=nrevt+1

      if(ioceau.eq.1)call xtauex(0)
      if(iociau.eq.1)call xtauin(0)

      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbaaa ',('-',i=1,10)
      return
      end

c------------------------------------------------------------------------------
      subroutine qnbcor(mode)
c------------------------------------------------------------------------------
c determines(mode=1) and plots (mode=2) two particle  correlations 
c for the configurations /config/
c------------------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      integer bns
      parameter (maxp=1000,bns=100)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtlogx,wtlog
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      dimension phi(bns),zwei(bns),yy(bns),zz(bns)
      common/cor/wert(bns),cwert(bns)
      character*6 cen,cvol
      common/cctcor/nctcor

           if(mode.eq.1)then

      nctcor=nctcor+1

      if(nctcor.eq.1)then
      do nn=1,bns
      wert(nn)=0
      cwert(nn)=0
      enddo
      endif

      ll=0

      do ii=1,np-1
      do jj=ii+1,np

      ll=ll+1
      prod=0

      do kk=1,3
      prod=prod+pcm(kk,ii)*pcm(kk,jj)
      enddo

      cs=prod/pcm(5,ii)/pcm(5,jj)

      if(abs(cs).gt.1.)then
      cs=aint(cs)
      ang=acos(cs) 
      else
      ang=acos(cs)
      endif

      if(cs.eq.1.)then
      nk=bns
      nw=1
      elseif(ang.eq.pi)then
      nk=1
      nw=bns
      else
      nw=1+aint(ang/pi*bns) 
      nk=1+aint((cs+1.)/2.*bns) 
      endif
      nw=min(nw,bns)
      nk=min(nk,bns)

      wert(nw)=wert(nw)+1
      cwert(nk)=cwert(nk)+1

      enddo
      enddo

           elseif(mode.eq.2)then

      do mm=1,bns
      phi(mm)=.5*pi/bns+(mm-1)*pi/bns
      zwei(mm)=.5*2./bns+(mm-1)*2./bns-1.
      yy(mm)=wert(mm)/nctcor
      zz(mm)=cwert(mm)/nctcor
      enddo

      write(cen,'(f6.1)')tecm
      write(cvol,'(f6.1)')volu

      write(ifhi,'(a)')    'newpage zone 1 1 1 openhisto'
      write(ifhi,'(a)')    'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')    'xrange -1 1'
      write(ifhi,'(a)')    'text 0 0 "xaxis cosine"'
      write(ifhi,'(a)')    'text 0 0 "yaxis counts"'
      write(ifhi,'(a)')    'text 0.4 0.91 "V='//cvol//'"'
      write(ifhi,'(a)')    'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a)')    'array 2'
         do mm=1,bns
      write(ifhi,'(2e11.3)')zwei(mm),zz(mm)
         enddo
      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto plot 0'

           endif

      return
      end

c----------------------------------------------------------------------
      subroutine qnbdec(ip,iret)
c----------------------------------------------------------------------
c  decays droplet ip via loop over qnbmet
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/parod/ioinfl,ioinct,iowidn,epsgc
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common /cttaus/  tpro,zpro,ttar,ztar,ttaus,detap,detat 
      parameter (nflav=6)
      integer jc(nflav,2)
      double precision p(5),c(5)
      parameter(maxit=50000)
      common/count/nacc,nrej,naccit(maxit),nptot,npit(maxit)
      if(ish.ge.94)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbdec ',('-',i=1,30)
      iret=0
      do j=1,5
      c(j)=pptl(j,ip)
      enddo

      call idquac(ip,nqi,ndu,ndu,jc)
      keu=jc(1,1)-jc(1,2)
      ked=jc(2,1)-jc(2,2)
      kes=jc(3,1)-jc(3,2)
      kec=jc(4,1)-jc(4,2)
      keb=jc(5,1)-jc(5,2)
      ket=jc(6,1)-jc(6,2)
      tiv1=tivptl(1,ip)
      tiv2=tivptl(2,ip)
      tivptl(1,ip)=-ainfin
      tivptl(2,ip)=ainfin
      call uttain(ip,x,y,z,t,nret,0)
      tivptl(1,ip)=tiv1
      tivptl(2,ip)=tiv2
        if(rinmes.gt.0..and.rinbar.gt.0.)then
      if(nret.eq.1.or.nret.eq.2.or.nret.eq.9)
     *call utstop('qnbdec: uttain ret code = 1 2 or 9      ')
      call uttaus(z,tdu,sz)
        else
      sz=0
        endif
      desptl(ip)=dezptl(ip)*ttaus          
      o=sz+desptl(ip)
      u=sz-desptl(ip)
      volu=(o-u)*radptl(ip)**2*pi
         if(desptl(ip).eq.0.)then
      if(nqi.eq.0)then
      volu=2*pi*radmes**3
      else
      volu=2*pi*radbar**3
      endif
         endif
      volu=amax1(volu,1.)
      call uttauh(ip,u,o,g,dum,wsmax)
      tecm=pptl(5,ip)/g
      if(ish.ge.91)
     *write(ifch,'(1x,a,3i3,3(a,e8.3))')'n-body decay;  uds: '
     *,keu,ked,kes,'  mass/g: ',tecm,'  vol: ',volu,'  dez: '
     *,dezptl(ip)
      if(ish.ge.92)write(ifch,*)'droplet: nr / id / momentum:'
      if(ish.ge.92)write(ifch,'(i5,i10,4x,5(1x,e10.3))')
     *ip,idptl(ip),(pptl(j,ip),j=1,5)

      if(ish.eq.26)then
      call qgcpen
      goto1
      endif
      if(ish.eq.29)then
      call qgcpfl
      goto1
      endif
      call qnbini(iret)
      if(iret.ne.0)return
      if(ioinct.ge.1)goto1
      do iter=1,itermx
      naccit(iter)=0
      call qnbmet
      enddo

1     continue

      if(ioceau.eq.1.and.iappl.eq.1)call xtauex(ip)

      nptlb=nptl
      do n=1,np
      nptl=nptl+1
      if(nptl.gt.mxptl)
     *call utstop('qnbptl: mxptl too small                  ')
      idptl(nptl)=ident(n)
      do j=1,4
      p(j)=pcm(j,n)
      p(5)=amass(n)
      enddo
      call utlob2(-1,c(1),c(2),c(3),c(4),c(5),p(1),p(2),p(3),p(4))
      do j=1,5
      pptl(j,nptl)=p(j)
      enddo
      enddo 

      if(ish.ge.92)then
      write(ifch,*)'decay products: nr / id / momentum:'
      do k=nptlb+1,nptl
      write(ifch,'(i5,i10,4x,5(1x,e10.3))')
     *k,idptl(k),(pptl(j,k),j=1,5)
      enddo
      endif

      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbdec ',('-',i=1,10)
      return
      end

c----------------------------------------------------------------------
      subroutine qnbfac(faclog)
c----------------------------------------------------------------------
c  returns log of factor for phase space weight
c  faclog= log{ prod[ m_i*(2*s_i+1)*volu/4/pi**3/hquer**3/(n_l+1-i) ] }
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      integer ii(maxp)
      common /clatt/nlattc,npmax

      pi=3.1415927
      hquer=0.197327
      ish0=ish
      if(ishsub/100.eq.22)ish=mod(ishsub,100)
      do i=1,np
      ii(i)=1
      enddo

      if(ish.ge.96)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbfac ',('-',i=1,30)
      if(ish.ge.96)write(ifch,*)'configuration:'
      if(ish.ge.96)write(ifch,*)(ident(i),i=1,np)
      if(ish.ge.96)write(ifch,*)'n_l:',nlattc,'   n_0:',nlattc-np   

      faclog=0

c prod n_alpha! -> f   
c      f=1
c      n1=1
c      nx=1
c    1 continue
c      if(ish.ge.96)write(ifch,*)'n1:',n1
c      i=0
c      fac=1
c      do n2=n1,np
c      if(ident(n2).eq.ident(n1))then
c      ii(n2)=0
c      i=i+1
c      fac=fac*i
c      endif
c      if(ii(n2).ne.0.and.n2.gt.n1.and.nx.eq.n1
c     *.and.ident(n2).ne.ident(n1))nx=n2
c      enddo
c      f=f*fac
c      if(ish.ge.96)write(ifch,*)'i:',i,'   fac:',fac,'   f:',f
c      if(nx.gt.n1)then
c      n1=nx
c      goto1
c      endif
c      if(ish.ge.96)write(ifch,*)'f=prod n_alpha!:',f,'   1/f:',1/f

      if(ish.ge.96)write(ifch,*)'faclog:',faclog

c sum_i log m_i*g_i*volu/4/pi**3/hquer**3/(n_l+1-i) -> flog
      flog=0
      do i=1,np
      call qnbfaf(i,gg,am,ioma)
      flog=flog+alog(gg*am*volu/4/pi**3/hquer**3/(nlattc+1-i))
      if(ish.ge.97)write(ifch,*)'flog:',flog
      enddo
      if(ish.ge.96)write(ifch,*)'flog=log(mass-spin-volume-factor):'
     *,flog,'   ( option:',ioma,' )'
      faclog=faclog+flog
      if(ish.ge.96)write(ifch,*)'faclog=faclog+flog:',faclog
      
      if(ish.ge.96)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbfac ',('-',i=1,10)
      ish=ish0
      return
      end

c----------------------------------------------------------------------
      subroutine qnbfaf(i,gg,am,ioma)
c----------------------------------------------------------------------
c  returns degeneracy gg and mass am  for factor f5
c----------------------------------------------------------------------
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog

      ioma=4

      hquer=0.197327
      cc=0.216416
      dd=13.773935

      call qnbspi(ident(i),spideg)
      gg=spideg

      if(ioma.eq.1)am=amass(i)
      if(ioma.eq.2)am=tecm/np
      if(ioma.eq.3)am=1
      if(ioma.eq.4)
     *am=cc*dd*gg**(-0.25)*(tecm/volu)**(0.25)*hquer**(0.75)
      if(iocova.eq.2)then
      am=0.5  !for noncovariant case
      ioma=0
      endif
      return
      end

c----------------------------------------------------------------------
      subroutine qnbids(jc,ids,iwts,i)  
c----------------------------------------------------------------------
c  returns i id-codes ids() corr to jc  and their weights iwts()
c----------------------------------------------------------------------
      parameter (mxids=200,mspecs=54,nflav=6)
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cspec2/jspecs(nflav,2,mspecs)
      integer ids(mxids),jc(nflav,2),iwts(mxids),jc1mi2(nflav)

      if(nspecs+1.gt.mxids)
     *call utstop('qnbids: mxids too small                 ')

      do n=1,nflav
      jc1mi2(n)=jc(n,1)-jc(n,2)
      enddo

      i=0

      do n=1,nflav
      if(jc1mi2(n).ne.0)goto1
      enddo
      i=i+1
      ids(i)=0
      iwts(i)=iozero
    1 continue      

           do j=1,nspecs
      if(jc1mi2(1).ne.ifok(1,j))goto2
      if(jc1mi2(2).ne.ifok(2,j))goto2
      if(jc1mi2(3).ne.ifok(3,j))goto2
      do n=4,nflav
      if(jc1mi2(n).ne.ifok(n,j))goto2
      enddo
      i=i+1
      ids(i)=ispecs(j)
      iwts(i)=1
    2 continue      
           enddo

      return
      end

c----------------------------------------------------------------------
      subroutine qnbiiw(x,f,df)
c----------------------------------------------------------------------
c returns fctn value and first derivative at x of the
c i-th integrated weight fctn minus random number  
c for the asympotic phase space integral.
c input:
c   x:   x-value
c   iii: i-value (via common/ciiw/iii,rrr)
c   rrr: random number   ( " )
c output:
c   f:   fctn value
c   df:  first derivative
c----------------------------------------------------------------------
      common/ciiw/iii,rrr
      i=iii
      f=x**(2*i-2)*(i-(i-1)*x**2)-rrr
      df=2*i*(i-1)*(x**(2*i-3)-x**(2*i-1))
      return
      end

c----------------------------------------------------------------------
      subroutine qnbini(iret)
c----------------------------------------------------------------------
c  generates initial configuration
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/crnoz/rnoz(maxp-1)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt              
      common/cfact/faclog 
      common/cqnbin/nump,ihadro(maxp)
      common /clatt/nlattc,npmax
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter(maxit=50000)
      common/count/nacc,nrej,naccit(maxit),nptot,npit(maxit)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/ctaue/taue
      if(ish.ge.94)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbini ',('-',i=1,30)

      iter=0

      nlattc=8*(tecm/10)*(1/(tecm/volu))**0.2*(nspecs/3.)**0.3
      if(aspecs(1).lt.0.010)nlattc=nlattc*3
      nlattc=max(nlattc,20)
      if(iternc.lt.0)iternc=1.500*nlattc

      itermx=iterma
      if(itermx.le.0)then
      e=tecm/volu
      b=1.1*(e+0.33)**0.66
      a=13.*(e+0.13)**(-0.65)
      tm=34.*(e+0.65)**(-0.61)
      t=a+b*volu
      taue=max(t,tm)
      itermx=(-itermx)*taue
      else
      taue=0
      endif
      if(ish.ge.94)write(ifch,*)'itermx:',itermx

      if(iternc.gt.itermx/2)iternc=itermx/2

      if(ioinco.eq.0)then
      call qnbmin(keu,ked,kes,kec)
      if(iograc.eq.1)call qgcaaa
      elseif(ioinco.ge.1)then
      call qgcaaa
      call qgcnbi
      endif

      np=nump+nadd
      if(np.gt.maxp)stop'np too large'

      nlattc=max(nlattc,1+int(np*1.2))

      do i= 1, nlattc-1
      rnoz(i)=rangen()
      enddo

      if(nadd.gt.0)then
      do i=nump+1,np
      ihadro(i)=110
      enddo
      endif

           do i=1,np
      ident(i)=ihadro(i)
      amass(i)=-1
      do j=1,nspecs
      if(ident(i).eq.ispecs(j))then
      amass(i)=aspecs(j)
      goto1
      endif
      enddo
    1 continue
      if(amass(i).lt.0.)
     *call utstop('qnbini: invalid particle species        ')
           enddo

      if(iocova.eq.1)call qnbody    !covariant
      if(iocova.eq.2)call qnbodz    !noncovariant
      call qnbfac(faclog)
      wtlog=wtxlog+faclog

      iret=0
      if(wtlog.le.-0.99999E+35)then
      call utmsg('qnbmet')
      write(ifch,*)'*****  wtlog for initl config < -1E+35'
      write(ifch,*)'wtlog:',wtlog
      write(ifch,*)'droplet mass:',tecm
      write(ifch,*)'flavour:'
      write(ifch,*)keu,ked,kes,kec,keb,ket
      call utmsgf
      iret=1
      goto1000
      endif

      if(ish.ge.94)write(ifch,*)'initial configuration:'
      if(ish.ge.94)call qnbwri

      itermx=iterma
      if(itermx.le.0)then
      e=tecm/volu
      b=1.1*(e+0.33)**0.66
      a=13.*(e+0.13)**(-0.65)
      tm=34.*(e+0.65)**(-0.61)
      t=a+b*volu
      taue=max(t,tm)
      itermx=(-itermx)*taue
      else
      taue=0
      endif
      if(ish.ge.94)write(ifch,*)'itermx:',itermx

      if(iternc.gt.itermx/2)iternc=itermx/2

      nacc=0
      nrej=0

 1000 continue

      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbini ',('-',i=1,10)

      return
      end

c----------------------------------------------------------------------
      subroutine qnbint(tecmx,nevt,nsho)
c----------------------------------------------------------------------
c  calculates phase space integral of the minimal hadron configuration
c  compatibel with keu, ked, kes, kec for a total mass of tecm
c  by employing nevt simulations and printing results every nsho events
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter(maxp=1000)
      common/cqnbin/nump,ihadro(maxp)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      tecm=tecmx
      write(ifch,*)
      write(ifch,'(1x,a,4i3,a,f10.4)')'droplet id:',keu,ked,kes,kec
     *,'   droplet mass:',tecm
      call qnbmin(keu,ked,kes,kec)
      np=nump
      if(np.gt.maxp)stop'np too large'
      do i=1,np
      call idmass(ihadro(i),amass(i))
      enddo
      wts=0
      n=0
           do ll=1,nevt
      n=n+1
      if(iocova.eq.1)call qnbody
      if(iocova.eq.2)call qnbodz
      wt=exp(wtxlog)
      wts=wts+wt
      if(mod(n,nsho).eq.0)
     *write(ifch,'(a,i7,3x,a,e13.6,3x,a,e13.6,3x,a,e13.6)')
     *'n:',n,'weight:',wt,'wts/n:',wts/n,'error:',wts/n/sqrt(1.*n)
           enddo
      return
      end

c----------------------------------------------------------------------
      subroutine qnbmet
c----------------------------------------------------------------------
c  change (or not) configuration via metropolis
c  configuration=np,tecm,amass(),ident(),pcm(),volu,wtlog
c    (common /config/)
c  nlattc (in /clatt/) must be set before calling this routine
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/ctst/psulog,wtulog
      common/crnoz/rnoz(maxp-1)
      real rnozo(maxp-1)
      common/cfact/faclog 
      logical test
      dimension amasso(maxp),idento(maxp),pcmo(5,maxp)
      parameter (nflav=6)
      integer jc(nflav,2),jc1(nflav,2),jc2(nflav,2)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (mxids=200)
      parameter (literm=500)
      common/cmet/kspecs(mspecs),liter,lspecs(literm,mspecs)
     *,iterl(literm),iterc(literm)
      parameter (mxpair=mspecs**2*4)
      common /clatt/nlattc,npmax
      common/cmetro/keepr
      parameter (nhise=100)
      common/chise/hise(mspecs,nhise)
      integer id1old(2),id2old(2),id1new(2),id2new(2)
      parameter(maxit=50000)
      common/count/nacc,nrej,naccit(maxit),nptot,npit(maxit)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      if(ish.ge.94)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbmet ',('-',i=1,30)
      if(ish.ge.94)write(ifch,'(1x,a,i4)')'iteration:',iter
      if(mod(iter,iterpr).eq.0)write(ifmt,*)'iteration:',iter
      if(maxp.gt.np)then
      do n=np+1,maxp
      ident(n)=0
      enddo
      endif
      test=.false.

c     for iter=1
c     ----------
           if(iter.eq.1)then
      liter=1
      do i=1,nspecs
      kspecs(i)=0
      nptot=0
      do li=1,literm
      lspecs(li,i)=0
      enddo
      enddo
      do li=1,literm
      iterc(li)=0
      enddo
      do j=1,mspecs
      do i=1,nhise
      hise(j,i)=0
      enddo
      enddo
      call qnbzmu(-1)
           endif
      
c     remember old configuration
c     --------------------------
      wtlo=wtlog
      wtlox=wtxlog
      faclo=faclog
      npo=np
      if(np-1.gt.0)then
      do i=1,np-1
      rnozo(i)=rnoz(i)
      enddo
      endif
      if(np.gt.0)then
      do i=1,np
      amasso(i)=amass(i)
      idento(i)=ident(i)
      do j=1,5
      pcmo(j,i)=pcm(j,i)
      enddo
      enddo
      endif

c     determine pair, construct new pair, update ident
c     ------------------------------------------------
           if(iopair.eq.1)then
c     (single pair method)
      call qnbpad(1,n1,n2,n3,n4,mm,jc)
      id1old(1)=ident(n1)
      id2old(1)=ident(n2)
      id1old(2)=0
      id2old(2)=0
      call qnbpaj(jc,iwpair,id1,id2)
      ident(n1)=id1
      ident(n2)=id2
      call qnbrmz
      id1new(1)=id1
      id2new(1)=id2
      id1new(2)=0
      id2new(2)=0
      xab=1
      xba=1
      nzold=0
      if(id1old(1).eq.0)nzold=nzold+1
      if(id2old(1).eq.0)nzold=nzold+1
      nznew=0
      if(id1new(1).eq.0)nznew=nznew+1
      if(id2new(1).eq.0)nznew=nznew+1

c     determine 2 pairs, construct 2 new pairs, update ident
c     ------------------------------------------------------
           elseif(iopair.eq.2)then
c     (double pair method)
      kkk=0
   25 call qnbpad(1,n1,n2,n3,n4,mm,jc)
      kkk=kkk+1
      id1old(1)=ident(n1)
      id2old(1)=ident(n2)
      call qnbpai(id1,id2,jc1)
      ident(n1)=id1
      ident(n2)=id2
      id1new(1)=id1
      id2new(1)=id2
      do i=1,nflav
      do j=1,2
      jc(i,j)=jc(i,j)-jc1(i,j)
      jc2(i,j)=jc(i,j)
      enddo
      enddo
    2 call qnbpad(2,n1,n2,n3,n4,mm,jc1)
      id1old(2)=ident(n3)
      id2old(2)=ident(n4)
      do i=1,nflav
      do j=1,2
      jc(i,j)=jc(i,j)+jc1(i,j)
      enddo
      enddo
      call qnbpaj(jc,iwpair,id1,id2)
      if(iwpair.eq.0)then
      do i=1,nflav
      do j=1,2
      jc(i,j)=jc2(i,j)
      enddo
      enddo
      if(ish.ge.94)write(ifch,*)'no pair possible'
      goto2
      endif
      ident(n3)=id1
      ident(n4)=id2
      id1new(2)=id1
      id2new(2)=id2
      call qnbrmz
      if(ish.ge.94)write(ifch,*)'wt-sum of 2. pairs (-->):',iwpair
     *,'   chosen pair:',id1,id2
      call qnbpaj(jc1,iwpais,idum1,idum2)
      if(ish.ge.94)write(ifch,*)'wt-sum of 2. pairs (<--):',iwpais
      nzold=0
      if(id1old(1).eq.0)nzold=nzold+1
      if(id2old(1).eq.0)nzold=nzold+1
      if(id1old(2).eq.0)nzold=nzold+1
      if(id2old(2).eq.0)nzold=nzold+1
      if(ish.ge.94)write(ifch,*)'number of zeros (old):',nzold
      nznew=0
      if(id1new(1).eq.0)nznew=nznew+1
      if(id2new(1).eq.0)nznew=nznew+1
      if(id1new(2).eq.0)nznew=nznew+1
      if(id2new(2).eq.0)nznew=nznew+1
      if(ish.ge.94)write(ifch,*)'number of zeros (new):',nznew
      if(iorejz.eq.1.and.nzold.eq.4.and.nznew.eq.4.and.kkk.le.50)goto25
      xab=1./iwpair*iozero**nznew
      xba=1./iwpais*iozero**nzold
      if(ish.ge.94)write(ifch,*)'asymmetry factor:',xba/xab
           else
      call utstop('qnbmet: invalid choice for iopair       ')
           endif

c     determine masses/momenta/weight of trial configuration
c     ------------------------------------------------------
      if(np.ge.2)then
           do i=1,np
      amass(i)=-1
      do j=1,nspecs
      if(ident(i).eq.ispecs(j))then
      amass(i)=aspecs(j)
      goto1
      endif
      enddo
    1 continue
      if(amass(i).lt.0.)
     *call utstop('qnbmet: invalid particle species        ')
           enddo
      keepr=0
c-c   call qnbolo(1000) !instead of "call qnbody" for testing 
      keepr=1
      if(iocova.eq.1)call qnbody
      if(iocova.eq.2)call qnbodz
      else
      wtxlog=-1e35
      endif
      call qnbfac(faclog)
      wtlog=wtxlog+faclog
      if(ish.ge.94)write(ifch,*)'trial configuration:'
      if(ish.ge.94)call qnbwri

c     accept or not trial configuration (metropolis)
c     ----------------------------------------------
      if(ish.ge.94)write(ifch,'(1x,a,4i5,a,4i5,a)')
     *'metropolis decision for '
     *,id1old(1),id2old(1),id1old(2),id2old(2),'   -->  '
     *,id1new(1),id2new(1),id1new(2),id2new(2),' :'
      iacc=0
           if(wtlog-wtlo.lt.30.)then
      q=exp(wtlog-wtlo)*xba/xab
      r=rangen()
      if(r.le.q)iacc=1 
      if(ish.ge.94)write(ifch,*)'new weight / old weight:',q,'    '
     *,'random number:',r
           else
      iacc=1
      if(ish.ge.94)write(ifch,*)'log new weight / old weight:'
     *,wtlog-wtlo
           endif
           if(iacc.eq.1)then 
      if(ish.ge.94)write(ifch,*)'new configuration accepted'
      nacc=nacc+1
      naccit(iter)=1
           else
      if(ish.ge.94)write(ifch,*)'old configuration kept'
      nrej=nrej+1
      wtlog=wtlo
      wtxlog=wtlox
      faclog=faclo
      np=npo
      if(np-1.gt.0)then
      do i=1,np-1
      rnoz(i)=rnozo(i)
      enddo
      endif
      if(np.gt.0)then
      do i=1,np
      amass(i)=amasso(i)
      ident(i)=idento(i)
      do j=1,5
      pcm(j,i)=pcmo(j,i)
      enddo
      enddo
      endif
           endif
           if(ioobsv.eq.0)then
      npit(iter)=np
      if(iter.gt.iternc)nptot=nptot+np
      else
      npob=0
      do i=1,np
      if(ioobsv.eq.ident(i))npob=npob+1
      enddo
      npit(iter)=npob
      if(iter.gt.iternc)nptot=nptot+npob
           endif
      if(ish.ge.94)write(ifch,*)'actual configuration:'
      if(ish.ge.94)call qnbwri
      if(ish.eq.27)call qnbcor(1)

c     printout/return
c     ---------------
      if(iosngl.ne.nrevt+1.and.iocite.ne.1)goto1000
      npmax=max(npmax,np)
           if(liter.le.literm)then
      iterc(liter)=iterc(liter)+1
      do i=1,np
      do j=1,nspecs
      if(ident(i).eq.ispecs(j))then
      lspecs(liter,j)=lspecs(liter,j)+1
      goto8
      endif
      enddo
    8 continue
      enddo
      if(mod(iter,iterpl).eq.0)then
      iterl(liter)=iter
      liter=liter+1
c     if(liter.le.literm)then
c     iterc(liter)=iterc(liter-1)
c     do j=1,nspecs
c     lspecs(liter,j)=lspecs(liter-1,j)
c     enddo
c     endif      
      endif
           endif
      if(iter.le.iternc)return

           do i=1,np
      call qnbzen(i)  !fill energy histogram
      do j=1,nspecs
      if(ident(i).eq.ispecs(j))then
      kspecs(j)=kspecs(j)+1
      goto7
      endif
      enddo
    7 continue
           enddo
      call qnbzmu(1)  !fill multiplicity histogram

           if(iter.eq.itermx.and.npmax.ge.nlattc)then
      call utmsg('qnbmet')
      write(ifch,*)'*****  nlattc too small'
      write(ifch,*)'nlattc:',nlattc,'   npmax:',npmax
      call utmsgf
           endif

1000  continue
      if(ish.ge.94)write(ifch,*)'accepted proposals:',nacc
     *,'  rejected proposals:',nrej
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbmet ',('-',i=1,10)
      return
      end

c----------------------------------------------------------------------
      subroutine qnbmin(keux,kedx,kesx,kecx)
c----------------------------------------------------------------------
c  returns min hadron set with given u,d,s,c content
c  input:
c     keux: net u quark number
c     kedx: net d quark number
c     kesx: net s quark number
c     kecx: net c quark number
c  output (written to /cqnbin/):
c     nump: number of hadrons
c     ihadro(n): hadron id for n'th hadron
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/cqnbin/nump,ihadro(maxp)
      logical wri
      character f1*11
      wri=.false.
      if(ish.ge.95)wri=.true.
      if(wri)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbmin ',('-',i=1,30)

      nump=0
      f1='(4i3,i7,i6)'
      ke=iabs(keux+kedx+kesx+kecx)
 
      if(keux+kedx+kesx+kecx.ge.0)then
      keu=keux
      ked=kedx
      kes=kesx
      kec=kecx
      isi=1
      else
      keu=-keux
      ked=-kedx
      kes=-kesx
      kec=-kecx
      isi=-1
      endif
      if(wri)write(ifch,'(4i3)')keux,kedx,kesx,kecx
      if(wri)write(ifch,'(4i3)')keu,ked,kes,kec
 
c get rid of anti-c and c (140, 240, -140, -240)
      if(kec.ne.0)then
   10 continue
      if(kec.lt.0)then
      kec=kec+1
      if(keu.gt.ked)then
      keu=keu-1
      nump=nump+1
      ihadro(nump)=140
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      else
      ked=ked-1
      nump=nump+1
      ihadro(nump)=240
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      endif
      goto10
      endif
   11 continue
      if(kec.gt.0)then
      kec=kec-1
      if(keu.lt.ked)then
      keu=keu+1
      nump=nump+1
      ihadro(nump)=-140
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      else
      ked=ked+1
      nump=nump+1
      ihadro(nump)=-240
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      endif
      goto11
      endif
      endif
 
c get rid of anti-s (130,230)
    5 continue
      if(kes.lt.0)then
      kes=kes+1
      if(keu.ge.ked)then
      keu=keu-1
      nump=nump+1
      ihadro(nump)=130
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      else
      ked=ked-1
      nump=nump+1
      ihadro(nump)=230
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      endif
      goto5
      endif

c get rid of anti-d (120, -230)  
   6  continue
      if(ked.lt.0)then
      ked=ked+1
      if(keu.ge.kes)then
      keu=keu-1
      nump=nump+1
      ihadro(nump)=120
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      else
      kes=kes-1
      nump=nump+1
      ihadro(nump)=-230
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      endif
      goto6
      endif

c get rid of anti-u (-120, -130)
    7 continue
      if(keu.lt.0)then
      keu=keu+1
      if(ked.ge.kes)then
      ked=ked-1
      nump=nump+1
      ihadro(nump)=-120
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      else
      kes=kes-1
      nump=nump+1
      ihadro(nump)=-130
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      endif
      goto7
      endif
 
      if(keu+ked+kes+kec.ne.ke)
     *call utstop('qnbmin: sum_kei /= ke                   ')

      keq=keu+ked
      keqx=keq

c get rid of s (3331, x330, xx30) 
      i=4
    2 i=i-1
    3 continue
      if((4-i)*kes.gt.(i-1)*keq)then
      kes=kes-i
      keq=keq-3+i
      nump=nump+1
      if(i.eq.3)ihadro(nump)=3331
      if(i.eq.2)ihadro(nump)=0330
      if(i.eq.1)ihadro(nump)=0030
           if(i.lt.3)then
      do j=1,3-i
      l=1+2*rangen()
      if(keu.gt.ked)l=1
      if(keu.lt.ked)l=2
      if(l.eq.1)keu=keu-1
      if(l.eq.2)ked=ked-1
      ihadro(nump)=ihadro(nump)+l*10**(4-j)
      enddo
           endif
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      if(kes.lt.0)
     *call utstop('qnbmin: negative kes                     ')
      if(keq.lt.0)
     *call utstop('qnbmin: negative keq                     ')
      goto3
      endif
      if(i.gt.1)goto2

      if(keu+ked.ne.keq)
     *call utstop('qnbmin: keu+ked /= keq                   ')

c get rid of d (2221, 1220, 1120) 
      i=4
   12 i=i-1
   13 continue
      if((4-i)*ked.gt.(i-1)*keu)then
      ked=ked-i
      keu=keu-3+i
      nump=nump+1
      if(i.eq.3)ihadro(nump)=2221
      if(i.eq.2)ihadro(nump)=1220
      if(i.eq.1)ihadro(nump)=1120
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      if(ked.lt.0)
     *call utstop('qnbmin: negative ked                     ')
      if(keu.lt.0)
     *call utstop('qnbmin: negative keu                     ')
      goto13
      endif
      if(i.gt.1)goto12

      if(ked.ne.0)
     *call utstop('qnbmin: ked .ne. 0                       ')

c get rid of u (1111) 
    9 continue
      if(keu.gt.0)then
      keu=keu-3
      nump=nump+1
      ihadro(nump)=1111
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      if(keu.lt.0)
     *call utstop('qnbmin: negative keu                     ')
      goto9
      endif

      if(keu.ne.0)
     *call utstop('qnbmin: keu .ne. 0                       ')

      if(isi.eq.-1)then
      do i=1,nump
      ihadro(i)=isi*ihadro(i)
      enddo
      endif

      do lo=1,2
      if(nump.lt.2)then
      nump=nump+1
      ihadro(nump)=110
      if(wri)write(ifch,f1)keu,ked,kes,kec,nump,ihadro(nump)
      endif
      enddo

      if(wri)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbmin ',('-',i=1,10)
      return
      end

c-------------------------------------------------------------
      subroutine qnbody
c-------------------------------------------------------------  
c   formerly subr genbod from genlib (cernlib).
c   modified by K. Werner, march 94.
c   subr to generate n-body event
c   according to fermi lorentz-invariant phase space.
c   the phase space integral is the sum over the weights wt divided 
c   by the number of events (sum wt / n).
c   adapted from fowl (cern w505) sept. 1974 by f. james.
c   events are generated in their own center-of-mass,
c   but may be transformed to any frame using loren4.
c
c   input to and output from subr thru common block config.
c   input:
c             np=number of outgoing particles 
c             tecm=total energy in center-of-mass
c             amass(i)=mass of ith outgoing particle
c   output:
c             pcm(1,i)=x-momentum if ith particle
c             pcm(2,i)=y-momentum if ith particle
c             pcm(3,i)=z-momentum if ith particle
c             pcm(4,i)=energy of ith particle
c             pcm(5,i)=momentum of ith particle
c             wtxlog=log of weight of event
c--------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common /clatt/nlattc,npmax
      dimension emm(maxp)
      dimension rno(3*maxp-4)
c     !pcm1 is linear equiv. of pcm to avoid double indices
      dimension em(maxp),pd(maxp),ems(maxp),sm(maxp)
     *,pcm1(5*maxp)
      common/cffq/ffqlog(maxp)
      common/ciiw/iii,rrr
      equivalence (nt,np),(amass(1),em(1)),(pcm1(1),pcm(1,1))
      logical wri
      common/cknt/knt
      external qnbiiw
      data twopi,pi/6.2831853073,3.1415927/
      nas=5 !must be at least 3
      knt=0
      kas=1000
      wri=.false.
      if(ish.ge.95)wri=.true.  
      if(wri)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbody ',('-',i=1,30)
      if(wri)write(ifch,1200)np,tecm
      if(wri)write(ifch,*)'particle masses:'
      if(wri)write(ifch,'(1x,10f6.3)')(amass(n),n=1,np)
c initialization
      knt=knt + 1
      if(knt.gt.1) goto 1
c     !ffqlog(n) = log{ pi * (twopi)**(n-2) / (n-2)factorial }
      ffqlog(1)=-1e35
      ffqlog(2)=alog(pi)
      do n=3,maxp
      ffqlog(n)=ffqlog(n-1)+log(twopi/(n-2))
      enddo
    1 continue
      if(nt.lt.2) goto 1001
      if(nt.gt.maxp) goto 1002
      ntm1=nt-1
      ntm2=nt-2
      ntp1=nt+1
      ntnm4=3*nt - 4
      emm(1)=em(1)
      tm=0.0
      do 2 i=1,nt
      ems(i)=em(i)**2
      tm=tm+em(i)
    2 sm(i)=tm
      tecmtm=tecm-tm
      if(tecmtm.le.0.0) goto 1000
      emm(nt)=tecm
c prefactor
      wtmlog=alog(tecmtm)*ntm2 + ffqlog(nt) - alog(tecm)
c fill rno with 3*nt-4 random numbers, the first nt-2 being ordered
      do 3 i= 1, ntnm4
    3 rno(i)=rangen()
      if(ntm2) 9,5,4
    4 continue
           if(nt.le.nas)then
      call qnbsor(rno,ntm2)
           else
      do i=2,nt-1
      iii=i
      rrr=rno(i-1)
      rno(i-1)=utroot(qnbiiw,0.,1.,0.00001)
      enddo
      x=1
      do i=nt-1,2,-1
      rno(i-1)=rno(i-1)*x
      x=rno(i-1)
      enddo
           endif
c calculate emm()
      do 6 j=2,ntm1
    6 emm(j)=rno(j-1)*(tecmtm)+sm(j)
    5 continue
c calculate wtlog
      wtxlog=wtmlog
      ir=ntm2
      do 7 i=1,ntm1
      pd(i)=qnbpdk(emm(i+1),emm(i),em(i+1))
      if(pd(i).gt.0.)then
      pdlog=alog(pd(i))
      else
      pdlog=-1e35
      if(wri)write(ifch,*)'i:',i
      if(wri)write(ifch,*)'emm(i+1):',emm(i+1),'   rno(i):  ',rno(i)
      if(wri)write(ifch,*)'emm(i):  ',emm(i),  '   rno(i-1):',rno(i-1)
      if(wri)write(ifch,*)'em(i+1):',em(i+1)
      endif
      wtxlog=wtxlog+pdlog
    7 continue
c log of factor for asymptotic method
           if(nt.le.nas)then
      famlog=0
      wtlogy=0
           else
      famlog=0
      wtlogy=alog(pi/2.)
      do i=2,nt-1
      if(i.eq.nt-1)then
      zi=rno(i-1)
      else
      zi=rno(i-1)/rno(i)
      endif
      famlog=famlog-alog( 2.*i*rno(i-1)*(1-zi**2) )
      wtlogy=wtlogy+alog( tecm**2*pi/(i-1)/2./i )
      enddo
           if(ish.ge.96)then
      write(ifch,*)'log x(i):'
      write(ifch,'(1x,10f7.2)')(alog(rno(i-1)),i=2,nt-1)
      write(ifch,*)'wtxlog_bef:',wtxlog,'   famlog:',famlog
           endif
           endif
      wtxlog=wtxlog+famlog
c complete specification of event (raubold-lynch method)
      pcm(1,1)=0.0
      pcm(2,1)=pd(1)
      pcm(3,1)=0.0
      do 8 i=2,nt
      pcm(1,i)=0.0
      pcm(2,i)=-pd(i-1)
      pcm(3,i)=0.0
      ir=ir+1
      bang=twopi*rno(ir)
      cb=cos(bang)
      sb=sin(bang)
      ir=ir+1
      c=2.0*rno(ir)-1.0
      s=sqrt(1.0-c*c)
      if(i.eq.nt) goto 10
      esys=sqrt(pd(i)**2+emm(i)**2)
      beta=pd(i)/esys
      gama=esys/emm(i)
      do 12 j=1,i
      ndx=5*j - 5
      aa= pcm1(ndx+1)**2 + pcm1(ndx+2)**2 + pcm1(ndx+3)**2
      pcm1(ndx+5)=sqrt(aa)
      pcm1(ndx+4)=sqrt(aa+ems(j))
      call qnbrt2(c,s,cb,sb,pcm,j)
      psave=gama*(pcm(2,j)+beta*pcm(4,j))
   12 pcm(2,j)=psave
      goto 8
   10 do 11 j=1,i
      aa=pcm(1,j)**2 + pcm(2,j)**2 + pcm(3,j)**2
      pcm(5,j)=sqrt(aa)
      pcm(4,j)=sqrt(aa+ems(j))
      call qnbrt2(c,s,cb,sb,pcm,j)
   11 continue
    8 continue
    9 continue
           if(wri)then
      if(wtxlog.gt.-1e30)then
      write(ifch,'(1x,a,f8.3,4x,a,f8.3)')'wtxlog:',wtxlog
     *,'asymptotic:',wtlogy
      else
      write(ifch,*)'wtxlog:',wtxlog,'   asymptotic:',wtlogy
      endif
           endif
      goto1111

c error returns
 1000 continue
      if(wri)
     *write(ifch,*)'available energy zero or negative -> wtxlog=-1e35'
      wtxlog=-1e35
      goto1111

 1001 continue
      if(wri)
     *write(ifch,*)'less than 2 outgoing particles -> wtxlog=-1e35'
      wtxlog=-1e35
      goto1111

 1002 continue
      write(ifch,*)'too many outgoing particles'
 1050 write(ifch,1150) knt
 1150 format(47h0 above error detected in qnbody at call number,i7)
      write(ifch,1200) np,tecm
 1200 format(' np:',i6/' tecm:',f10.5)
      write(ifch,*)'particle masses:'
      write(ifch,'(1x,10f6.3)')(amass(jk),jk=1,np)
      stop

1111  continue
      if(wri)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbody ',('-',i=1,10)
      return
      end

c-------------------------------------------------------------
      subroutine qnbodz
c-------------------------------------------------------------  
c   subr to generate n-body event
c   according to non-invariant phase space.
c   the phase space integral is the sum over the weights exp(wtxlog)
c   divided by the number of events.
c   ref.: hagedorn, nuov. cim. suppl ix, x (1958) 646.
c   events are generated in their own center-of-mass.
c
c   input to and output from subr is thru common block config.
c   input:
c             np=number of outgoing particles 
c             tecm=total energy in center-of-mass
c             amass(i)=mass of ith outgoing particle
c   output:
c             pcm(1,i)=x-momentum if ith particle
c             pcm(2,i)=y-momentum if ith particle
c             pcm(3,i)=z-momentum if ith particle
c             pcm(4,i)=energy of ith particle
c             pcm(5,i)=momentum of ith particle
c             wtxlog=log of weight of event
c--------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common /clatt/nlattc,npmax
      common/cffq/ffqlog(maxp)
      dimension ti(maxp),xi(maxp),si(maxp),zi(maxp)
      common/crnoz/rnoz(maxp-1)
      double precision ps(5)
      common/cknt/knt
      common/cmetro/keepr
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      data pi/3.1415927/
      if(ish.ge.95)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbodz ',('-',i=1,30)
      if(ish.ge.95)write(ifch,1200)np,tecm
      if(ish.ge.95)write(ifch,*)'particle masses:'
      if(ish.ge.95)write(ifch,'(1x,10f6.3)')(amass(n),n=1,np)
      knt=0
c initialization knt=1
      knt=knt + 1
      if(knt.gt.1) goto 1
c     !ffqlog(n) = log{ (4*pi)**n  / (n-1)! }
      ffqlog(1)=alog(4*pi)
      do n=2,maxp
      ffqlog(n)=ffqlog(n-1)+alog(4*pi/(n-1))
      enddo
    1 continue
c set wtxlog -infinity for np<2 
      if(np.lt.2) goto 1001
c special treatment for np=2
      if(np.eq.2)then
      if(tecm.lt.amass(1)+amass(2)+0.00001)goto1000
      p0=utpcm(tecm,amass(1),amass(2))
      wtxlog=alog( 4*pi*p0
     */(1/sqrt(amass(1)**2+p0**2)+1/sqrt(amass(2)**2+p0**2)) )
      if(ish.ge.96)
     *write(ifch,*)'wtxlog:',wtxlog,'   (np=2 treatment)'
      bang=2*pi*rangen()
      cb=cos(bang)
      sb=sin(bang)
      c=2.0*rangen()-1.0
      s=sqrt(1.0-c*c)
      do 9 i=1,2
      is=2*i-3
      pcm(5,i)=p0
      pcm(1,i)=is*pcm(5,i)*s*cb
      pcm(2,i)=is*pcm(5,i)*s*sb
      pcm(3,i)=is*pcm(5,i)*c
      pcm(4,i)=sqrt(amass(i)**2+p0**2)
    9 continue
      goto1111
      endif
c stop if np too large
      if(np.gt.maxp) goto 1002
c initialization all knt
      tm=0.0
      do 2 i=1,np
      tm=tm+amass(i)
    2 continue
      tt=tecm-tm
      if(tt.le.0.0) goto 1000
c prefactor
      wtxlog=alog(tt)*(np-1) + ffqlog(np)
      if(ish.ge.96)
     *write(ifch,*)'wtxlog:',wtxlog,'   (prefactor)'
c fill rnoz with np-1 random numbers 
      if(keepr.eq.0)then
      do 3 i= 1, np-1
    3 rnoz(i)=rangen()
      else
      do lo=1,iomom
      j=1+rangen()*nlattc
      rnoz(j)=rangen()
      enddo
      endif
c calculate z_i distributed as i*z*(i-1)
      do i= 1, np-1
      zi(i)=rnoz(i)**(1./i)
      enddo
c calculate x_i
      xi(np)=1
      do i=np-1,1,-1
      xi(i)=zi(i)*xi(i+1)
      enddo
c calculate t_i, e_i, p_i
      if(ish.ge.98)write(ifch,*)'calculate t_i, e_i, p_i ...'
      do i=1,np-1
      si(i)=xi(i)*tt
      enddo
      ti(1)=si(1)
      if(ti(1).le.0.)ti(1)=1e-10
      ti(np)=tt-si(np-1)
      if(ti(np).le.0.)ti(np)=1e-10
      do i=np-1,2,-1
      ti(i)=si(i)-si(i-1)
      if(ti(i).le.0.)ti(i)=1e-10
      enddo
      do i=1,np
      pcm(1,i)=0
      pcm(2,i)=0
      pcm(3,i)=0
      pcm(4,i)=ti(i)+amass(i)
      p52=ti(i)*(ti(i)+2*amass(i))
      if(p52.gt.0)then 
      pcm(5,i)=sqrt(p52)
      else
      pcm(5,i)=ti(i)*sqrt(1+2*amass(i)/ti(i))
      endif
      enddo
c calculate wtxlog
      call qnbraw(7,200,w)
      if(w.gt.0.)then
      wtxlog=wtxlog+alog(w)
      else
      wtxlog=wtxlog-1e+30
      endif
      do 7 i=1,np
      wtxlog=wtxlog+alog(pcm(5,i))+alog(ti(i)+amass(i))
    7 continue
      if(ish.ge.96)
     *write(ifch,*)'wtxlog:',wtxlog
c print
      if(ish.ge.96)then
      write(ifch,*)'momenta:'
      do j=1,4
      ps(j)=0
      enddo
      do i=1,np
      do j=1,4
      ps(j)=ps(j)+pcm(j,i)
      enddo
      write(ifch,'(1x,i3,5x,5f12.5)')i,(pcm(j,i),j=1,5)
      enddo
      ps(5)=dsqrt(ps(1)**2+ps(2)**2+ps(3)**2)
      write(ifch,'(1x,a4,8x,5f12.5)')'sum:',(sngl(ps(j)),j=1,5)
      endif
      if(w.le.0.)goto1111
c complete specification of event (random rotations and then deformations)
      call qnbrot 
      if(ish.ge.96)write(ifch,*)'momenta after rotations:'
      call qnbrop(96,0)
      call qnbrod
      if(ish.ge.96)write(ifch,*)'momenta after deformations:'
      call qnbrop(96,1)
      goto1111

c error returns
 1000 continue
      if(ish.ge.95)
     *write(ifch,*)'available energy zero or negative -> wtxlog=-1e35'
      wtxlog=-1e35
      goto1111

 1001 continue
      if(ish.ge.95)
     *write(ifch,*)'less than 2 outgoing particles -> wtxlog=-1e35'
      wtxlog=-1e35
      goto1111

 1002 continue
      write(ifch,*)'too many outgoing particles'
 1050 write(ifch,1150) knt
 1150 format(47h0 above error detected in qnbody at call number,i7)
      write(ifch,1200) np,tecm
 1200 format(' np:',i6/' tecm:',f10.5)
      write(ifch,*)'particle masses:'
      write(ifch,'(1x,10f6.3)')(amass(jk),jk=1,np)
      stop

1111  continue
      if(ish.ge.95)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbodz ',('-',i=1,10)
      return
      end

c-----------------------------------------------------------------------
      subroutine qnbolo(loops)
c-----------------------------------------------------------------------
c  loop over qnbody
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      a=0 
      k=0 
      do j=1,loops
c-c   if(mod(j,iterpr).eq.0)write(ifmt,*)'     iteration:',iter,j
      if(iocova.eq.1)call qnbody
      if(iocova.eq.2)call qnbodz
      if(ish.ge.97)write(ifch,*)'j:',j,'   wtxlog:',wtxlog
           if(wtxlog.gt.-1e30)then 
      k=k+1 
      if(k.eq.1)c=wtxlog
           if(a.gt.0.)then
      if(alog(a).lt.wtxlog-c-20)then
      a=0
      c=wtxlog
      endif
           endif
      a=a+exp(wtxlog-c) 
           endif 
      if(ish.ge.97)write(ifch,*)'k:',k,'   c:',c
      enddo 
      a=a/loops
      wtxlog=alog(a)+c 
      return
      end

c-----------------------------------------------------------------------
      function qnbpdk(a,b,c)
c-----------------------------------------------------------------------
c  formerly pdk from cernlib
c-----------------------------------------------------------------------
      double precision aa,bb,cc,a2,b2,c2
      aa=a
      bb=b
      cc=c
      a2=aa*aa
      b2=bb*bb
      c2=cc*cc
      if(a2 + (b2-c2)**2/a2-2.0*(b2+c2).le.0.)then
      qnbpdk = 0
      else
      qnbpdk = 0.5*dsqrt(a2 + (b2-c2)**2/a2 - 2.0*(b2+c2))
      endif
      return
      end

c----------------------------------------------------------------------
      subroutine qnbpad(k,n1,n2,n3,n4,mm,jc)
c----------------------------------------------------------------------
c  k=1: determ pair indices k1,k2
c  k=2: determ pair indices k3,k4 (.ne. n1,n2)
c  k=1 and k=2: mm: type of pair, jc: flavour of pair
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (nflav=6)
      integer jc(nflav,2),ic(2),jc1(nflav,2),ic1(2),jc2(nflav,2),ic2(2)
      common /clatt/nlattc,npmax
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog

      if(k.eq.2)then
      k1=n1
      k2=n2
      endif

c     determine n1,n2 and mm
c     ----------------------
    1 continue
      n1=1+rangen()*nlattc
      n1=min(n1,nlattc)
    2 continue
      n2=1+rangen()*nlattc
      n2=min(n2,nlattc)
      if(n2.eq.n1)goto2
      if(n2.lt.n1)then
      n1r=n1
      n1=n2
      n2=n1r      
      endif
      if(k.eq.2)then
      if(n1.eq.k1.or.n1.eq.k2.or.n2.eq.k1.or.n2.eq.k2)goto1
      endif
      if(ident(n1).ne.0.and.ident(n2).ne.0)mm=1  ! hadron-hadron 
      if(ident(n1).ne.0.and.ident(n2).eq.0)mm=2  ! hadron-empty 
      if(ident(n1).eq.0.and.ident(n2).ne.0)mm=2  ! empty-hadron 
      if(ident(n1).eq.0.and.ident(n2).eq.0)mm=3  ! empty-empty 
      if(ish.ge.94)write(ifch,'(a,i2)')' mm:',mm
      if(ish.ge.94)write(ifch,*)'to be replaced:',n1,ident(n1)
      if(ish.ge.94)write(ifch,*)'to be replaced:',n2,ident(n2)

c     flavour of n1+n2 --> jc      
c     -----------------------
           if(mm.eq.1)then 
      call idtr4(ident(n1),ic1)
      call iddeco(ic1,jc1)
      call idtr4(ident(n2),ic2)
      call iddeco(ic2,jc2)
      do i=1,nflav
      do j=1,2
      jc(i,j)=jc1(i,j)+jc2(i,j)
      enddo
      enddo
           elseif(mm.eq.2.and.ident(n1).ne.0)then
      call idtr4(ident(n1),ic)
      call iddeco(ic,jc)
           elseif(mm.eq.2.and.ident(n2).ne.0)then
      call idtr4(ident(n2),ic)
      call iddeco(ic,jc)
           else
      do i=1,nflav
      do j=1,2
      jc(i,j)=0
      enddo
      enddo  
           endif

      if(k.eq.2)then
      n3=n1
      n4=n2
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine qnbpai(id1,id2,jc)
c----------------------------------------------------------------------
c  returns arbitrary hadron pair id1,id2, flavour written to jc  
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      parameter (nflav=6)
      integer jc(nflav,2),jc1(nflav,2),ic1(2),jc2(nflav,2),ic2(2)
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)

c     construct pair id1,id2
c     ----------------------
      i1=rangen()*(nspecs+iozero)-(iozero-1)
      i1=max(i1,0)
      i1=min(i1,nspecs)
      if(i1.eq.0)then
      id1=0
      do i=1,nflav
      do j=1,2
      jc1(i,j)=0
      enddo
      enddo
      else
      id1=ispecs(i1)
      call idtr4(id1,ic1)
      call iddeco(ic1,jc1)
      endif
      if(ish.ge.95)write(ifch,'(1x,a,i3,a,i5,a,6i2,3x,6i2)')
     *'i1:',i1,'   id1:',id1,'   jc1:',jc1
      i2=rangen()*(nspecs+iozero)-(iozero-1)
      i2=max(i2,0)
      i2=min(i2,nspecs)
      if(i2.eq.0)then
      id2=0
      do i=1,nflav
      do j=1,2
      jc2(i,j)=0
      enddo
      enddo
      else
      id2=ispecs(i2)
      call idtr4(id2,ic2)
      call iddeco(ic2,jc2)
      endif
      if(ish.ge.95)write(ifch,'(1x,a,i3,a,i5,a,6i2,3x,6i2)')
     *'i2:',i2,'   id2:',id2,'   jc2:',jc2
      if(ish.ge.94)write(ifch,'(a,i6,i6)')' pair:',id1,id2

c     determine jc
c     ------------
      do i=1,nflav
      do j=1,2
      jc(i,j)=jc1(i,j)+jc2(i,j)
      enddo
      enddo
      do i=1,nflav
      j12=jc(i,1)-jc(i,2)
      if(j12.ge.0)then
      jc(i,1)=j12
      jc(i,2)=0
      else
      jc(i,1)=0
      jc(i,2)=-j12
      endif
      enddo
      if(ish.ge.94)write(ifch,'(a,6i2,3x,6i2)')' jc:',jc

      return
      end

c----------------------------------------------------------------------
      subroutine qnbpaj(jc,iwpair,id1,id2)
c----------------------------------------------------------------------
c  returns sum of weights iwpair of possible pairs
c  and randomly chosen hadron pair id1,id2 for given flavour jc
c----------------------------------------------------------------------
      parameter(nflav=6,mspecs=54,mxids=200)
      parameter(mxpair=mspecs**2*4)
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cspec2/jspecs(nflav,2,mspecs)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      dimension ids(mxids),iwts(mxids),jc(nflav,2),jc2(nflav,2)
      dimension idpair(mxpair,2),iwtpai(mxpair)
      dimension jc1mi2(nflav),jcmi(nflav)

      nflv=nflav
      if(nflv.gt.6)
     *call utstop('qnbpaj: nflav.gt.6: modify this routine ')

c     construct possible pairs id1,id2
c     --------------------------------

      ipair=0
      iwpair=0

c  id1=0:

      if(nspecs+1.gt.mxids)
     *call utstop('qnbpaj: mxids too small                 ')

      jc1mi2(1)=jc(1,1)-jc(1,2)
      jc1mi2(2)=jc(2,1)-jc(2,2)
      jc1mi2(3)=jc(3,1)-jc(3,2)
      jc1mi2(4)=jc(4,1)-jc(4,2)
      jc1mi2(5)=jc(5,1)-jc(5,2)
      jc1mi2(6)=jc(6,1)-jc(6,2)

      nids=0

      if(jc1mi2(1).ne.0)goto11
      if(jc1mi2(2).ne.0)goto11
      if(jc1mi2(3).ne.0)goto11
      if(jc1mi2(4).ne.0)goto11
      if(jc1mi2(5).ne.0)goto11
      if(jc1mi2(6).ne.0)goto11
      nids=nids+1
      ids(nids)=0
      iwts(nids)=iozero
   11 continue

      do j=1,nspecs
      if(jc1mi2(1).ne.ifok(1,j))goto22
      if(jc1mi2(2).ne.ifok(2,j))goto22
      if(jc1mi2(3).ne.ifok(3,j))goto22
      if(jc1mi2(4).ne.ifok(4,j))goto22
      if(jc1mi2(5).ne.ifok(5,j))goto22
      if(jc1mi2(6).ne.ifok(6,j))goto22
      nids=nids+1
      ids(nids)=ispecs(j)
      iwts(nids)=1
   22 continue
      enddo

      if(nids.eq.0)goto2
      if(nids.gt.mxpair)
     *call utstop('qnbpaj: mxpair too small                ')
      do k=1,nids
      ipair=ipair+1
      idpair(ipair,1)=0
      idpair(ipair,2)=ids(k)
      iwtpai(ipair)=iozero*iwts(k)
      iwpair=iwpair+iwtpai(ipair)
      if(ish.ge.95)write(ifch,'(a,i5,5x,a,i6,i6,5x,a,i6)')' pair nr:'
     *,ipair,'ids:',0,ids(k),'weight:',iwtpai(ipair)
      enddo
    2 continue

c  id1>0:

        do i1=1,nspecs

        if(ish.ge.95)then
        do i=1,nflav
      jc2(i,1)=jc(i,1)-jspecs(i,1,i1)
      jc2(i,2)=jc(i,2)-jspecs(i,2,i1)
        enddo
      write(ifch,'(1x,a,i3,a,i6,a,6i2,3x,6i2)')
     *'i1:',i1,'   id1:',ispecs(i1),'   jc1:'
     *,(jspecs(i,1,i1),i=1,6),(jspecs(i,2,i1),i=1,6)
      write(ifch,'(a,6i2,3x,6i2)')' jc2:',jc2
        endif

      jcmi(1)=jc1mi2(1)-jspecs(1,1,i1)+jspecs(1,2,i1)
      jcmi(2)=jc1mi2(2)-jspecs(2,1,i1)+jspecs(2,2,i1)
      jcmi(3)=jc1mi2(3)-jspecs(3,1,i1)+jspecs(3,2,i1)
      jcmi(4)=jc1mi2(4)-jspecs(4,1,i1)+jspecs(4,2,i1)
      jcmi(5)=jc1mi2(5)-jspecs(5,1,i1)+jspecs(5,2,i1)
      jcmi(6)=jc1mi2(6)-jspecs(6,1,i1)+jspecs(6,2,i1)

      nids=0

      if(jcmi(1).ne.0)goto111
      if(jcmi(2).ne.0)goto111
      if(jcmi(3).ne.0)goto111
      if(jcmi(4).ne.0)goto111
      if(jcmi(5).ne.0)goto111
      if(jcmi(6).ne.0)goto111
      nids=nids+1
      ids(nids)=0
      iwts(nids)=iozero
  111 continue
      do j=1,nspecs
      if(jcmi(1).ne.ifok(1,j))goto222
      if(jcmi(2).ne.ifok(2,j))goto222
      if(jcmi(3).ne.ifok(3,j))goto222
      if(jcmi(4).ne.ifok(4,j))goto222
      if(jcmi(5).ne.ifok(5,j))goto222
      if(jcmi(6).ne.ifok(6,j))goto222
      nids=nids+1
      ids(nids)=ispecs(j)
      iwts(nids)=1
  222 continue
      enddo

      if(nids.eq.0)goto3
      if(ipair+nids.gt.mxpair)
     *call utstop('qnbpaj: mxpair too small                ')
      do k=1,nids
      ipair=ipair+1
      idpair(ipair,1)=ispecs(i1)
      idpair(ipair,2)=ids(k)
      iwtpai(ipair)=iwts(k)
      iwpair=iwpair+iwtpai(ipair)
      enddo
      if(ish.ge.95)then
      ipair0=ipair-nids
      do k=1,nids
      ipair0=ipair0+1
      write(ifch,'(a,i5,5x,a,i6,i6,5x,a,i6)')' pair nr:'
     *,ipair0,'ids:',ispecs(i1),ids(k),'weight:',iwtpai(ipair0)
      enddo
      endif
    3 continue

        enddo

c     no pair found
c     -------------
      if(ipair.eq.0)then
      if(iwpair.ne.0)
     *call utstop('qnbpaj: iwpair.ne.0                     ')
      return
      endif

c     select pair
c     -----------
      r=rangen()
      ir=1+r*iwpair
      ir=min(ir,iwpair)
      is=0
      do ip=1,ipair
      is=is+iwtpai(ip)
      if(ir.le.is)then
      id1=idpair(ip,1)
      id2=idpair(ip,2)
      if(ish.ge.95)write(ifch,*)'random number:',r
     *,' --> chosen pair:',ip
      goto1000
      endif
      enddo
      call utstop('qnbpaj: no pair selected                ')

1000  continue
      return
      end

c--------------------------------------------------------------------
      subroutine qnbraw(npx,npy,w)
c--------------------------------------------------------------------
c returns random walk fctn w=w(0,p_1,p_2,...,p_n) for noncovariant
c phase space integral (see hagedorn, suppl nuov cim ix(x) (1958)646)
c input: dimension np and momenta p_i=pcm(5,i) via /config/
c    1   < np <= npx : hagedorn method
c    npx < np <= npy : integral method
c    npy < np        : asymptotic method     
c--------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      integer ii(maxp),isi(maxp)
      double precision ppcm(maxp),ww,ppsum,ppmax
      external qnbrax
      common/cepsr/epsr,nepsr
      if(ish.ge.98)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbraw ',('-',i=1,30)

      if(np.lt.3)
     *call utstop('qnbraw: np must be at least 3           ')

      kper=5
      pi=3.1415927
      pmax=0
      do i=1,np
      pmax=pmax+pcm(5,i)
      enddo
      wio=0
      win=0
      whd=0

c     sum p_i - 2*p_max not positive
c     ------------------------------
      px=0
      ps=0
      do i=1,np
      px=max(px,pcm(5,i))
      ps=ps+pcm(5,i)
      enddo
      if(ps-2*px.le.0.)then
      w=0
      if(ish.ge.96)write(ifch,'(1x,a,e12.5,4x)')
     *'sum p_i - 2*p_max not positive -->  w:',w
      goto1000
      endif   

c     asymptotic method
c     -----------------
      was=0
      do i=1,np
      was=was+pcm(5,i)**2
      enddo
      was=(was*2*pi/3)**(-1.5)
      if(ish.ge.96)write(ifch,'(1x,a,e12.5,4x)')
     *'asymptotic method: was:',was

      if(np.gt.npy)then
      w=was
      goto1000
      endif

      if(np.le.npx)goto9

c     integral method
c     ---------------
      if(ish.ge.98)write(ifch,*)'integral method...'
      itmax=8
      it=0
      b=pi*np*kper/pmax
      win=0
      nepsr=0
    3 continue
      it=it+1
      if(ish.ge.98)write(ifch,*)'it:',it
      b=b*5./3.
      wio=win
      call uttrap(qnbrax,0.,b,win) 
      iok=0
      if(abs(win-wio).le.epsr*abs((win+wio)/2))iok=1
      if(it.eq.itmax)iok=1     
      if(ish.ge.97.or.ish.ge.96.and.iok.eq.1)
     *write(ifch,'(1x,2(a,e12.5,2x),a,i2,2x,a,i4)')
     *'integral method:   win:',win
     *,'upper limit:',b,'it:',it,'nepsr:',nepsr
           if(it.eq.itmax
     *.and.abs(win-wio).gt.epsr*abs((win+wio)/2))then
      nepsr=nepsr+1
      if(ish.ge.98)then
      call utmsg('qnbraw')
      write(ifch,*)
     *'*****  requested accuracy could not be achieved'
      write(ifch,*)'achieved accuracy: '
     *,abs(win-wio)/abs((win+wio)/2)
      write(ifch,*)'requested accuracy:',epsr
      call utmsgf
      endif
           endif
      if(it.eq.1.or.iok.eq.0)goto3

      if(nepsr.eq.0)then
      w=win
      goto1000
      endif

      if(np.gt.20)then
      call utmsg('qnbraw')
      write(ifch,*)
     *'*****  requested accuracy could not be achieved'
      write(ifch,*)'achieved accuracy: '
     *,abs(win-wio)/abs((win+wio)/2)
      write(ifch,*)'requested accuracy:',epsr
      call utmsgf
      w=win
      goto1000
      endif

c     hagedorn method (double)
c     ------------------------
    9 continue
      ppmax=0
      do i=1,np
      ppcm(i)=pcm(5,i)
      ppmax=ppmax+ppcm(i)
      enddo
      ww=0
      do i=1,np
      ii(i)=0
      isi(i)=1
      enddo
      ppsum=ppmax
      i=0
      iprosi=1
      ww=iprosi*(ppsum/ppmax)**(np-3)
      if(ish.ge.97)
     *write(ifch,'(4x,i5,12x,f7.2,i5,f11.2)')np,sngl(ppsum)
     *,iprosi,sngl(ww)
    5 continue
      i=i+1
      if(i.gt.np)goto6
      if(ii(i).eq.1)goto5
      iprosi=-iprosi
      isi(i)=-isi(i)
      ppsum=ppsum+2*isi(i)*ppcm(i)
           if(ppsum.gt.0.or.ppsum.eq.0..and.isi(i).gt.0)then
      ww=ww+iprosi*(ppsum/ppmax)**(np-3)
      if(ish.ge.97)
     *write(ifch,'(4x,2i5,2f7.2,i5,f11.2)')
     *np,i,sngl(2*isi(i)*ppcm(i)),sngl(ppsum),iprosi,sngl(ww)
           else
      if(ish.ge.97)
     *write(ifch,'(4x,2i5,2f7.2,i5,4x,a)')
     *np,i,sngl(2*isi(i)*ppcm(i)),sngl(ppsum),iprosi,'not counted'
           endif
      ii(i)=1
      if(i.gt.1)then
      do j=1,i-1
      ii(j)=0
      enddo
      endif
      i=0
      goto5
    6 continue
      do i=1,np
      ww=ww*pmax/ppcm(i)/2./i
      enddo
      ww=-ww/pmax**3/pi/2.*np*(np-1)*(np-2)
      whd=ww      
      if(ish.ge.96)write(ifch,'(1x,a,e12.5,4x,a)')
     *'hagedorn method:   whd:',whd,'double precision'

      w=whd

1000  continue
      if(ish.ge.98)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbraw ',('-',i=1,10)
      return
      end

c--------------------------------------------------------------------
      function qnbrax(x)
c--------------------------------------------------------------------
c returns integrand for random walk fctn w=w(0,p_1,p_2,...,p_n):
c 1./(2*pi**2) * x**2 * prod[sin(p_i*x)/(p_i*x)]
c input: dimension np and momenta p_i=pcm(5,i) via /config/
c--------------------------------------------------------------------
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      qnbrax= pii * x**2
      do i=1,np
      px=pcm(5,i)*x
      if(px.ne.0.)qnbrax=qnbrax*sin(px)/(px)
      enddo
      return
      end

c----------------------------------------------------------------------
      subroutine qnbrmz
c----------------------------------------------------------------------
c  removes intermediate zeros from ident
c  updates np
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      integer identx(maxp)
      common /clatt/nlattc,npmax /cfact/faclog 
      if(ish.ge.98)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbrmz ',('-',i=1,30)
      if(np.eq.0)goto1000

      do i=1,np
      identx(i)=ident(i)
      enddo
      npx=np

      i=0
      np=nlattc+1

    1 i=i+1
      if(i.gt.nlattc)then
      np=nlattc
      goto1000
      endif
      if(ident(i).ne.0)goto1
    2 np=np-1
      if(np.eq.0)goto1000
      if(ident(np).eq.0)goto2

      if(ish.ge.98)then
      write(ifch,*)'ident:'
      write(ifch,'(1x,10i7)')(ident(j),j=1,nlattc)
      write(ifch,'(1x,a,i3,3x,a,i3)')'i:',i,'np:',np
      endif

      if(i.eq.np+1)goto1000
       
      ident(i)=ident(np)
      ident(np)=0
      goto1

1000  continue
      if(ish.ge.98)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbrmz ',('-',i=1,10)
      end

c----------------------------------------------------------------------
      subroutine qnbrod
c----------------------------------------------------------------------
c deformes polygon of a sequence of arbitrarily rotated momentum
c vectors such that the polygon gets closed
c    input: pcm(1-3,i) representing polygon
c    output: pcm(1-3,i) representing closed polygon
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      real x(3),y(3),z(3),w(3)
      if(ish.ge.97)write(ifch,*)'sr qnbrod: polygon deformation:'

      err=0.01

      kmax=1000
      fac=0.30
      x2max=(err*tecm)**2

      if(ish.ge.97)write(ifch,'(a,i4,a,f12.6)')
     *' kmax:',kmax,'   x2max:',x2max

      x(1)=0
      x(2)=0
      x(3)=0
      do i=1,np
      x(1)=x(1)+pcm(1,i)
      x(2)=x(2)+pcm(2,i)
      x(3)=x(3)+pcm(3,i)
      enddo ! i

      k=0
   1  continue

      x2=x(1)**2+x(2)**2+x(3)**2
      if(ish.ge.97)write(ifch,'(a,i3,a,3f9.3,a,f12.6)')
     *' it',k,':   x:',x,'      x2:',x2
      if(x2.le.x2max)goto1000
      if(k.gt.kmax)goto1001

      k=k+1
      ir=1+rangen()*np
      ir=min(ir,np)

      z(1)=-x(1)
      z(2)=-x(2)
      z(3)=-x(3)
      x(1)=x(1)-pcm(1,ir)
      x(2)=x(2)-pcm(2,ir)
      x(3)=x(3)-pcm(3,ir)
      y(1)=pcm(1,ir)
      y(2)=pcm(2,ir)
      y(3)=pcm(3,ir)
      if(ish.ge.98)write(ifch,'(a,i3,a,3f9.3,a,3f9.3,a,i4)')
     *' it',k,':   x:',x,'   y:',y,'  ir:',ir
      xxx=x(1)**2+x(2)**2+x(3)**2
      yyy=y(1)**2+y(2)**2+y(3)**2
      zzz=z(1)**2+z(2)**2+z(3)**2
         if(xxx.gt.0..and.yyy.gt.0..and.zzz.gt.0.)then
      xx=sqrt(xxx)
      yy=sqrt(yyy)
      zz=sqrt(zzz)
      a=min(fac,fac*yy/zz)
      w(1)=y(1)+a*z(1)
      w(2)=y(2)+a*z(2)
      w(3)=y(3)+a*z(3)
      www=w(1)**2+w(2)**2+w(3)**2
         if(www.gt.0.)then
      ww=sqrt(www)
      y(1)=yy/ww*w(1)
      y(2)=yy/ww*w(2)
      y(3)=yy/ww*w(3)
      pcm(1,ir)=y(1)
      pcm(2,ir)=y(2)
      pcm(3,ir)=y(3)
         endif
         endif
      x(1)=x(1)+y(1)
      x(2)=x(2)+y(2)
      x(3)=x(3)+y(3)
      if(ish.ge.98)write(ifch,'(a,i3,a,3f9.3,a,3f9.3,a,i4)')
     *' it',k,':   x:',x,'   y:',y,'  ir:',ir

      goto1

 1001 continue
      call utmsg('qnbrod')
      write(ifch,*)'*****  total 3-momentum nonzero'
      write(ifch,'(3f12.5,5x,2f12.5)')(x(j),j=1,3),x2,x2max
      call utmsgf

 1000 continue
      return

      end

c----------------------------------------------------------------------
      subroutine qnbrop(ishx,ichk)
c----------------------------------------------------------------------
c  prints momenta of configuration (essentially to check rotation procedure)
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      double precision ps(5)
      err=0.01
      do j=1,4
      ps(j)=0
      enddo
      do i=1,np
      do j=1,4
      ps(j)=ps(j)+pcm(j,i)
      enddo
      if(ish.ge.ishx)write(ifch,'(1x,i3,5x,5f12.5)')i,(pcm(j,i),j=1,3)
     *,sqrt(pcm(1,i)**2+pcm(2,i)**2+pcm(3,i)**2),pcm(5,i)
      enddo
      ps(5)=dsqrt(ps(1)**2+ps(2)**2+ps(3)**2)
      if(ish.ge.ishx)write(ifch,'(1x,a4,8x,5f12.5)')
     *'sum:',(sngl(ps(j)),j=1,5)
           if(ichk.eq.1)then
           if(dabs(ps(1)).gt.err*tecm.or.dabs(ps(2)).gt.err*tecm
     *.or.dabs(ps(3)).gt.err*tecm)then
      call utmsg('qnbrop')
      write(ifch,*)'*****  total 3-momentum nonzero'
      write(ifch,'(9x,5f12.5)')(sngl(ps(j)),j=1,5)
      call utmsgf
           endif
           endif
      return
      end

c----------------------------------------------------------------------
      subroutine qnbrot
c----------------------------------------------------------------------
c rotates momenta of /config/ randomly
c   input: pcm(5,i)
c   output: pcm(1-3,i)
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin /cvsn/iversn,iverso 
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      real u(3)

      do i=1,np
      u(3)=2.*rangen()-1.
      phi=2.*pi*rangen()
      u(1)=sqrt(1.-u(3)**2)*cos(phi)
      u(2)=sqrt(1.-u(3)**2)*sin(phi)
      pcm(1,i)=pcm(5,i)*u(1)
      pcm(2,i)=pcm(5,i)*u(2)
      pcm(3,i)=pcm(5,i)*u(3)
      enddo

      return
      end

c-------------------------------------------------------------------
      subroutine qnbrt2(c,s,c2,s2,pr,i)
c-------------------------------------------------------------------
c  formerly subr rotes2 from cernlib      
c  this subr now does two rotations (xy and xz)
c-------------------------------------------------------------------
      parameter(maxp=1000)
      dimension pr(5*maxp)
      k1 = 5*i - 4
      k2 = k1 + 1
      sa = pr(k1)
      sb = pr(k2)
      a      = sa*c - sb*s
      pr(k2) = sa*s + sb*c
      k2 = k2 + 1
      b = pr(k2)
      pr(k1) = a*c2 - b*s2
      pr(k2) = a*s2 + b*c2
      return
      end

c-----------------------------------------------------------------------
      subroutine qnbsor(a,n)
c-----------------------------------------------------------------------
c cern proglib# m103    flpsor          .version kernfor  3.15  820113
c orig. 29/04/78
c-----------------------------------------------------------------------
c   sort the one-dimensional floating point array a(1),...,a(n) by
c   increasing values
c-----------------------------------------------------------------------
      dimension a(*)
      common /slate/ lt(20),rt(20)
      integer r,rt
c
      level=1
      lt(1)=1
      rt(1)=n
   10 l=lt(level)
      r=rt(level)
      level=level-1
   20 if(r.gt.l) go to 200
      if(level) 50,50,10
c
c   subdivide the interval l,r
c     l : lower limit of the interval (input)
c     r : upper limit of the interval (input)
c     j : upper limit of lower sub-interval (output)
c     i : lower limit of upper sub-interval (output)
c
  200 i=l
      j=r
      m=(l+r)/2
      x=a(m)
  220 if(a(i).ge.x) go to 230
      i=i+1
      go to 220
  230 if(a(j).le.x) go to 231
      j=j-1
      go to 230
c
  231 if(i.gt.j) go to 232
      w=a(i)
      a(i)=a(j)
      a(j)=w
      i=i+1
      j=j-1
      if(i.le.j) go to 220
c
  232 level=level+1
      if(level.gt.20)stop'level too large'
      if((r-i).ge.(j-l)) go to 30
      lt(level)=l
      rt(level)=j
      l=i
      go to 20
   30 lt(level)=i
      rt(level)=r
      r=j
      go to 20
   50 return
      end

c-----------------------------------------------------------------------
      subroutine qnbspd(iopt)
c-----------------------------------------------------------------------
c  defines particle species and masses and degeneracies.
c  input:
c    iopt=odd number: massless
c    iopt=even number: same as iopt-1, but massive 
c    iopt= 1: pi0 (massless)
c    iopt= 2: pi0
c    iopt= 3: pi-,pi0,pi+ (massless)
c    iopt= 4: pi-,pi0,pi+
c    iopt= 5: pi-,pi0,pi+,prt,aprt,ntr,antr (massless)
c    iopt= 6: pi-,pi0,pi+,prt,aprt,ntr,antr
c    iopt= 7: 25 hadrons (massless)
c    iopt= 8: 25 hadrons
c    iopt= 9: 54 hadrons (massless)
c    iopt=10: 54 hadrons
c    iopt=11:  3 quarks  (massless) 
c    iopt=12:  3 quarks 
c  output:
c    nspecs: nr of species
c    ispecs: id's
c    aspecs: masses
c-----------------------------------------------------------------------
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter (nflav=6)
      integer jc(nflav,2),ic(2)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/cspec2/jspecs(nflav,2,mspecs)
      parameter(nspe01=1,nspe03=3,nspe05=7,nspe07=25,nspe09=54)
      parameter(nspe11=6)
      real jspe01(nspe01),jspe03(nspe03),jspe05(nspe05),jspe07(nspe07)
     *,jspe09(nspe09),jspe11(nspe11)
      data jspe01/   110 /
      data jspe03/   110,  120, -120 /
      data jspe05/   110,  120, -120, 1120,-1120, 1220,-1220 /
      data jspe07/  
     *   110,  120, -120,  130, -130,  230, -230,  220,  330
     *, 1120,-1120, 1220,-1220, 1130,-1130, 2130,-2130
     *, 1230,-1230, 2230,-2230, 1330,-1330, 2330,-2330 /
      data jspe09/  
     *   110,  120, -120,  130, -130,  230, -230,  220,  330
     *,  111,  121, -121,  131, -131,  231, -231,  221,  331
     *, 1120,-1120, 1220,-1220, 1130,-1130, 2130,-2130
     *, 1230,-1230, 2230,-2230, 1330,-1330, 2330,-2330
     *, 1111,-1111, 1121,-1121, 1221,-1221, 2221,-2221, 1131,-1131
     *, 1231,-1231, 2231,-2231, 1331,-1331, 2331,-2331, 3331,-3331 /
      data jspe11/
     *     1,   -1,    2,   -2,    3,   -3   /

      if(iopt.gt.12)
     *call utstop('qnbspd: invalid option                  ')
      ioptx=(1+iopt)/2*2-1

      if(ioptx.eq.1)nspecs=nspe01
      if(ioptx.eq.3)nspecs=nspe03
      if(ioptx.eq.5)nspecs=nspe05
      if(ioptx.eq.7)nspecs=nspe07
      if(ioptx.eq.9)nspecs=nspe09
      if(ioptx.eq.11)nspecs=nspe11
           do i=1,nspecs
      if(ioptx.eq.1)ispecs(i)=jspe01(i)      
      if(ioptx.eq.3)ispecs(i)=jspe03(i)      
      if(ioptx.eq.5)ispecs(i)=jspe05(i)      
      if(ioptx.eq.7)ispecs(i)=jspe07(i)      
      if(ioptx.eq.9)ispecs(i)=jspe09(i)      
      if(ioptx.eq.11)ispecs(i)=jspe11(i)
      if(ioptx.eq.iopt)then
      aspecs(i)=0
      else
      call idmass(ispecs(i),am)
      aspecs(i)=am
      endif
      call qnbspi(ispecs(i),gg)
      gspecs(i)=gg
           enddo

      do nf=1,nflav
      ifoa(nf)=0
      enddo
           do i=1,nspecs
      id=ispecs(i)
      call idtr4(id,ic)
      call iddeco(ic,jc)
      do nf=1,nflav
      ifok(nf,i)=jc(nf,1)-jc(nf,2)
      ifoa(nf)=ifoa(nf)+iabs(ifok(nf,i))
      jspecs(nf,1,i)=jc(nf,1)
      jspecs(nf,2,i)=jc(nf,2)
      enddo
           enddo

      return
      end

c-------------------------------------------------------------
      subroutine qnbspf(ku,kd,ks,kc,kb,kt,j,n,spelog)
c-------------------------------------------------------------  
c  returns spelog = log of factor for consid. different species
c  spelog is double precision
c  option ioflac determines the method:
c     ioflac=1: ignore flavour conservation
c     ioflac=2: flavour conservation implemented straightforward
c                 (only for nspecs=3,7)
c     ioflac=3: flavour conservation via generating fctn
c  further input: 
c     ku,...,kt (integer) : flavour
c     j (integer) : excluded species 
c     n (integer) : multiplicity 
c-------------------------------------------------------------  
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (nflav=6)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      integer m(7),l(7),ifot(nflav)
      common/csph/ifox(nflav),ifoy(nflav),jx,nx,ifom(nflav,mspecs)
      double precision epsh  
      common/cepsh/epsh
      parameter(nprec=6)
      parameter(mxfacu=200)
      common/cprfa/ifacu(0:nprec,0:mxfacu),ifaci(0:nprec,0:mxfacu)
      parameter(mxhalb=1000)
      common/cprha/ihalb(0:nprec,0:mxhalb),izwei(0:nprec,0:mxhalb)
      parameter(mxnx2=200)
      double precision faci(0:mxfacu)
      double precision utgam2,spelog,spe
      parameter(numax=100,kqmax=100)
      parameter(mxhh=200)
      if(ish.ge.98)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbspf ',('-',i=1,30)
      if(ish.ge.98)write(ifch,'(1x,a,9x,a,4x,a)')
     *' ku kd ks kc kb kt','j','n'
      if(ish.ge.98)write(ifch,'(1x,6i3,5x,2i5)')
     *ku,kd,ks,kc,kb,kt,j,n
      k=nspecs
      jx=j
      nx=n
      ifot(1)=ku
      ifot(2)=kd
      ifot(3)=ks
      ifot(4)=kc
      ifot(5)=kb
      ifot(6)=kt

           if(ioflac.eq.1)then

      if(ish.ge.98)write(ifch,'(1x,a,i1)')'ioflac=',ioflac
      g=0
      do i=1,nspecs
      if(i.ne.j)g=g+gspecs(i)
      enddo
      spelog=n*dlog(1.d0*g)

           elseif(ioflac.eq.2)then

      if(ish.ge.98)write(ifch,'(1x,a,i2)')'ioflac:',ioflac
           if(k.eq.3)then
      if(ish.ge.98)write(ifch,'(1x,a,i2)')'nspecs:',nspecs
      spe=0d0
           if(j.lt.1.or. j.gt.k)then
      do 1 n1=0,n
      do 2 n2=0,n-n1
      n3=n-n1-n2
      do 5 nf=1,nflav
      if(ifoa(nf).eq.0.and.ifot(nf).eq.0)goto5
      if(n1*ifok(nf,1)+n2*ifok(nf,2)+n3*ifok(nf,3).ne.ifot(nf))goto2       
    5 continue
      spe=spe+utgam2(1.d0+n)
     &/utgam2(1.d0+n1)/utgam2(1.d0+n2)/utgam2(1.d0+n3)
     &*gspecs(1)**n1*gspecs(2)**n2*gspecs(3)**n3
    2 continue
    1 continue
           else
      do 3 i1=0,n
      i2=n-i1
      m(1)=0
      m(2)=i1
      m(3)=i2
      do i=1,3
      ii=1+mod(j-2+i,3)
      l(ii)=m(i)
      enddo
      n1=l(1)
      n2=l(2)
      n3=l(3)
      do 6 nf=1,nflav
      if(ifoa(nf).eq.0.and.ifot(nf).eq.0)goto6
      if(n1*ifok(nf,1)+n2*ifok(nf,2)+n3*ifok(nf,3).ne.ifot(nf))goto3
    6 continue
      spe=spe+utgam2(1.d0+n)
     &/utgam2(1.d0+n1)/utgam2(1.d0+n2)/utgam2(1.d0+n3)
     &*gspecs(1)**n1*gspecs(2)**n2*gspecs(3)**n3
    3 continue
           endif
      if(ish.ge.98)write(ifch,*)'spe:',spe
      spelog=-1000
      if(spe.gt.0.d0)spelog=dlog(spe)
      if(ish.ge.98)write(ifch,*)'spelog:',spelog
           elseif(k.eq.7)then
      if(ish.ge.98)write(ifch,'(1x,a,i2)')'nspecs:',nspecs
      if(n.gt.mxfacu)
     *call utstop('qnbspf: mxfacu too small                ')
      do lf=0,n
      faci(lf)=1.d0/utgam2(1d0+lf)
      enddo
      spe=0
           if(j.lt.1.or. j.gt.k)then
      do n1=0,n
      do n2=0,n-n1
      do n3=0,n-n1-n2
      do n4=0,n-n1-n2-n3
      do n5=0,n-n1-n2-n3-n4
      do 12 n6=0,n-n1-n2-n3-n4-n5
      n7=n-n1-n2-n3-n4-n5-n6
      do 15 nf=1,nflav
      if(ifoa(nf).eq.0.and.ifot(nf).eq.0)goto15
      if(n1*ifok(nf,1)+n2*ifok(nf,2)+n3*ifok(nf,3)+n4*ifok(nf,4)
     *+n5*ifok(nf,5)+n6*ifok(nf,6)+n7*ifok(nf,7).ne.ifot(nf))goto12       
   15 continue
      spe=spe+1d0/faci(n)*faci(n1)*faci(n2)*faci(n3)*faci(n4)
     &*faci(n5)*faci(n6)*faci(n7)
     &*gspecs(1)**n1*gspecs(2)**n2*gspecs(3)**n3*gspecs(4)**n4
     &*gspecs(5)**n5*gspecs(6)**n6*gspecs(7)**n7
   12 continue
      enddo
      enddo
      enddo
      enddo
      enddo
           else
      do i1=0,n
      do i2=0,n-i1
      do i3=0,n-i1-i2
      do i4=0,n-i1-i2-i3
      do 13 i5=0,n-i1-i2-i3-i4
      i6=n-i1-i2-i3-i4-i5
      m(1)=0
      m(2)=i1
      m(3)=i2
      m(4)=i3
      m(5)=i4
      m(6)=i5
      m(7)=i6
      do i=1,7
      ii=1+mod(j-2+i,7)
      l(ii)=m(i)
      enddo
      n1=l(1)
      n2=l(2)
      n3=l(3)
      n4=l(4)
      n5=l(5)
      n6=l(6)
      n7=l(7)
      do 16 nf=1,nflav
      if(ifoa(nf).eq.0.and.ifot(nf).eq.0)goto16
      if(n1*ifok(nf,1)+n2*ifok(nf,2)+n3*ifok(nf,3)+n4*ifok(nf,4)
     *+n5*ifok(nf,5)+n6*ifok(nf,6)+n7*ifok(nf,7).ne.ifot(nf))goto13       
   16 continue
      spe=spe+1d0/faci(n)*faci(n1)*faci(n2)*faci(n3)*faci(n4)
     &*faci(n5)*faci(n6)*faci(n7)
     &*gspecs(1)**n1*gspecs(2)**n2*gspecs(3)**n3*gspecs(4)**n4
     &*gspecs(5)**n5*gspecs(6)**n6*gspecs(7)**n7
   13 continue
      enddo
      enddo
      enddo
      enddo
           endif
      if(ish.ge.98)write(ifch,*)'spe:',spe
      spelog=-1000
      if(spe.gt.0.d0)spelog=dlog(spe)
      if(ish.ge.98)write(ifch,*)'spelog:',spelog
           else
      call utstop('qnbspf: ioflac=2 only for nspecs=3,7    ')
           endif

           elseif(ioflac.eq.3)then

      call utstop('qnbspf: ioflac must be 1 or 2           ')

           endif

      if(ish.ge.98)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbspf ',('-',i=1,10)
      return
      end

c-------------------------------------------------------------
      subroutine qnbspg(ku,kd,ks,kc,kb,kt,j,n,spelog)
c-------------------------------------------------------------  
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      double precision spelog,spalog
      if(ioflac.ne.0)return
      ioflac=2
      call qnbspf(ku,kd,ks,kc,kb,kt,j,n,spalog)
      ioflac=3
      call qnbspf(ku,kd,ks,kc,kb,kt,j,n,spelog)
      ioflac=0     
      write(ifch,*)'ioflac=2/3:',spalog,spelog
      return
      end

c----------------------------------------------------------------------
      subroutine qnbspi(id,spideg)
c----------------------------------------------------------------------
c  returns spin degeneracy spideg for particle id-code id
c----------------------------------------------------------------------
      parameter (nspec=60)
      dimension ispec(nspec),spid(nspec)
      data ispec/
     *     1,   -1,    2,   -2,    3,   -3
     *,  110,  120, -120,  220,  130, -130,  230, -230,  330
     *,  111,  121, -121,  221,  131, -131,  231, -231,  331
     *, 1120, 1220, 1130, 2130, 1230, 2230, 1330, 2330
     *, 1111, 1121, 1221, 2221, 1131, 1231, 2231, 1331, 2331, 3331
     *,-1120,-1220,-1130,-2130,-1230,-2230,-1330,-2330
     *,-1111,-1121,-1221,-2221,-1131,-1231,-2231,-1331,-2331,-3331/
      data spid/
     *  6*6.
     *, 9*1.
     *, 9*3.
     *, 8*2.
     *,10*4.
     *, 8*2.
     *,10*4.   /
      do i=1,nspec
      if(id.eq.ispec(i))then
      spideg=spid(i)
      goto1
      endif
      enddo
      call utstop('qnbspi: id not found                    ')
    1 continue
      return
      end

c----------------------------------------------------------------------
      subroutine qnbtst(iof12)
c----------------------------------------------------------------------
c  calculates logs of prefactors and phase space integral
c  for ultrarelativistic limit (massless particles) and (2*s_i+1)=1
c  f12log and w15log=w35log+f12log not calculated calculated for iof12=0
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/ctst/psulog,wtulog
      integer ii(maxp)
      common /clatt/nlattc,npmax
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom

      pi=3.1415927
      hquer=0.197327
      ish0=ish
      if(ishsub/100.eq.23)ish=mod(ishsub,100)
      do i=1,np
      ii(i)=1
      enddo

      if(ish.ge.96)write(ifch,*)('-',i=1,10)
     *,' entry sr qnbtst ',('-',i=1,30)
      if(ish.ge.96)write(ifch,*)'configuration:'
      if(ish.ge.96)write(ifch,*)(ident(i),i=1,np)
      if(ish.ge.96)write(ifch,*)'n_l:',nlattc,'   n_0:',nlattc-np   

c log of prod m_i*volu/4/pi**3/hquer**3 -> f5log
      f5log=0
      do i=1,np
      call qnbfaf(i,gg,am,ioma)
      f5log=f5log+alog(gg*am*volu/4/pi**3/hquer**3)
      enddo
      if(ish.ge.96)write(ifch,*)'log(f5):',f5log

c log f4log=0
      f4log=0
      if(ish.ge.96)write(ifch,*)'log(f4):',f4log

c log of 1/prod n_alpha! -> f3log   
      dbllog=0
      n1=1
      nx=1
    1 continue
      i=0
      x=0
      do n2=n1,np
      if(ident(n2).eq.ident(n1))then
      ii(n2)=0
      i=i+1
      x=x+alog(i*1.)
      endif
      if(ii(n2).ne.0.and.n2.gt.n1.and.nx.eq.n1
     *.and.ident(n2).ne.ident(n1))nx=n2
      enddo
      dbllog=dbllog+x
      if(nx.gt.n1)then
      n1=nx
      goto1
      endif
      f3log=-dbllog
      if(ish.ge.96)write(ifch,*)'log(f3):'
     *,f3log

c log of f3 * f4 * f5
      f35log=f5log+f4log+f3log
      if(ish.ge.96)write(ifch,*)'log(f3*f4*f5):',f35log
      
c log of phase space integral --> psilog
           if(iocova.eq.1)then
      psilog=alog(2.*np*np*(np-1)/tecm**4/pi)
      do i=1,np
      psilog=psilog+alog(tecm**2*pi/2./i/i)
      enddo
           elseif(iocova.eq.2)then
      psilog=-alog(2.*np-1)
      psilog=psilog+(np-1)*alog(pi/2.)
      do i=1,2*np-2
      psilog=psilog+alog((2.*np+i-2)/i)
      enddo
      do i=1,3*np-4
      psilog=psilog+alog(tecm/i)
      enddo
           endif
      if(ish.ge.96)write(ifch,*)'log(psi):',psilog

c log of phase space integral * f3 * f4 * f5
      w35log=f35log+psilog
      if(ish.ge.96)write(ifch,*)'log(f35*psi):',w35log

           if(iof12.ne.0)then

c log of macro/micro factor (f1*f2) --> f12log
      deglog=0
      do i=1,np
      deglog=deglog+alog(1.*i)
      enddo
      deglog=deglog+f3log
      do i=1,np
      deglog=deglog+alog(nlattc+1.-i)-alog(1.*i)
      enddo
      f12log=-deglog
      if(ish.ge.96)write(ifch,*)'log(f1*f2):',f12log

      w15log=w35log+f12log
      if(ish.ge.96)write(ifch,*)'log(f15*psi):',w15log

      if(ish.ge.96)write(ifch,'(1x,4(a,3x))')
     *'log(fac):','log(psi):',' log(wt):','log(wta):'
      if(ish.ge.94)write(ifch,'(1x,4(f9.3,3x))')
     *f12log+f35log,psilog,w15log,w15log-f12log

           endif

      psulog=psilog
      wtulog=w35log
   
      if(ish.ge.96)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbtst ',('-',i=1,10)
      ish=ish0
      return
      end

c----------------------------------------------------------------------
      subroutine qnbuex(x,e)
c----------------------------------------------------------------------
c  x --> x*10.**e with x.lt.10.**10.
c----------------------------------------------------------------------
           if(x.eq.0.)then
      e=0.      
           else
      e=int(alog10(abs(x)))/10*10
      x=x/10.**e
           endif
      return
      end

c----------------------------------------------------------------------
      subroutine qnbwin(n,w,q,i)
c----------------------------------------------------------------------
c  returns random index i according to weight w(i)
c----------------------------------------------------------------------
      real w(n),q(n)
      q(1)=w(1)
      do k=2,n
      q(k)=q(k-1)+w(k)
      enddo
      y=rangen()*q(n)
      do k=1,n
      i=k
      if(q(k).ge.y)goto1000
      enddo
      i=n
1000  return
      end

c----------------------------------------------------------------------
      subroutine qnbwri
c----------------------------------------------------------------------
c  writes (to ifch) an configuration
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/cfact/faclog
      write(ifch,'(1x,a,i5)')'np:',np
      write(ifch,'(1x,3(a,3x))')
     *'log(fac):','log(psi):',' log(wt):'
      if(wtlog.gt.-1e30.and.wtxlog.gt.-1e30)then
      write(ifch,'(1x,3(f9.3,3x))')faclog,wtxlog,wtlog
      else
      write(ifch,*)faclog,wtxlog,wtlog
      endif
      if(np.eq.0)return
      call qnbtst(1)
      write(ifch,*)'particle id codes:'
      write(ifch,'(1x,10i6)')(ident(n),n=1,np)
      write(ifch,*)'particle masses:'
      write(ifch,'(1x,10f6.3)')(amass(n),n=1,np)
      end

c----------------------------------------------------------------------
      subroutine qnbzen(iii)
c----------------------------------------------------------------------
c analysis of events. energy spectra.
c for iii>0: filling histogram considering ptl iii
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (nhise=100)
      common/chise/hise(mspecs,nhise)
      parameter (literm=500)
      common/cmet/kspecs(mspecs),liter,lspecs(literm,mspecs)
     *,iterl(literm),iterc(literm)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      parameter (mxtext=10)
      character*50 ctext(mxtext)
      common/chisto/ntext,xtext(mxtext),ytext(mxtext),ctext
      common/ckhist/khisto
      de=2./nhise/2.

           if(iii.gt.0)then

      i=iii
      do l=1,nspecs
      if(ident(i).eq.ispecs(l))then
      j=l
      goto1
      endif
      enddo
    1 continue
      am=aspecs(j)
      e=pcm(4,i)
      ke=1+int((e-am)/(2*de))
      if(ke.ge.1.and.ke.le.nhise)hise(j,ke)=hise(j,ke)+1
      return

           else

      stop'STOP in qnbzen: iii=0'

           endif

      end

c----------------------------------------------------------------------
      subroutine qnbzmu(iii)
c----------------------------------------------------------------------
c analysis of events. multiplicity spectra.
c for iii<0: settting histograms to zero (should be first call)
c for iii>0: filling histogram considering ptl iii
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/ctst/psulog,wtulog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (nhismu=500)
      parameter (nflav=6)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/chismu/hismu(mspecs,0:nhismu),hismus(nhismu)
      parameter (literm=500)
      common/cmet/kspecs(mspecs),liter,lspecs(literm,mspecs)
     *,iterl(literm),iterc(literm)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common /clatt/nlattc,npmax
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter (mxtext=10)
      character*50 ctext(mxtext)
      common/chisto/ntext,xtext(mxtext),ytext(mxtext),ctext
      common/ckhist/khisto

           if(iii.lt.0)then

      do i=1,nhismu
      hismus(i)=0
      enddo
      do j=1,nspecs
      do i=0,nhismu
      hismu(j,i)=0
      enddo
      enddo
      goto1000

           elseif(iii.gt.0)then

      if(np.ge.1.and.np.le.nhismu)hismus(np)=hismus(np)+1
      do j=1,nspecs
      mu=0
      do i=1,np
      if(ident(i).eq.ispecs(j))mu=mu+1
      enddo
      if(mu.ge.0.and.mu.le.nhismu)hismu(j,mu)=hismu(j,mu)+1
      enddo
      goto1000

           else

      stop'STOP in sr qnbzmu: iii must not be 0'

           endif

1000  continue
      return
      end

c-----------------------------------------------------------------------
      subroutine racpro(n,acprob)
c     returns the array acprob containing accumulated prob for
c       multi pomeron cuts acc to gribov 
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro3/uentro,sigppe,sigppd,asuhax(7),asuhay(7),omega
      common/paro4/grigam,grirsq,gridel,grislo,gricel
      parameter(mxpri=200) 
      character*6 subpri
      common/p14/nrpri,subpri(mxpri),ishpri(mxpri) 
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      real acprob(n)

      ishini=ish
      if(nrpri.gt.0)then
      do nr=1,nrpri
      if(subpri(nr).eq.'racpro')then
      ish=ishpri(nr)
      endif       
      enddo
      endif
       
      xi=alog(engy**2)
      sig=8*pi*grigam*exp(xi*gridel)
      z=2*gricel*grigam/(grirsq+grislo*xi)*exp(xi*gridel)
 
c     0-cut-pomeron probability
c     -------------------------
      qz=1/(-z)
      qzh=1/(-z/2)
      fz=0
      fzh=0
      do 10 i=1,20
      qz=qz*(-z)/i
      qzh=qzh*(-z/2)/i
      fz=fz+qz/i
10    fzh=fzh+qzh/i
      reler=abs(qz/i/fz)
      relerh=abs(qzh/i/fzh)
      sigppz=sig*(fzh-fz)
      sigppe=sigppz/gricel
      sigppd=sigppz/gricel*(gricel-1)
 
c     n-cut-pomeron probabilities, n>0
c     --------------------------------
      r=0
      do 21 i=1,n
      if(i.eq.1)q=1
      if(i.gt.1)q=q*z/(i-1)
      r=r+q
      si=sig/i/z*(1-exp(-z)*r)
      if(i.eq.1)acprob(i)=si
      if(i.gt.1)acprob(i)=acprob(i-1)+si
21    continue
      if(sigppi.lt.0.)sigppi=acprob(n)
      do 22 i=1,n
22    acprob(i)=acprob(i)/acprob(n)
 
c     warnings
c     --------
           if(reler.gt.1e-3)then
      call utmsg('racpro')
      write(ifch,*)'*****  reler too large'
      write(ifch,*)'reler,qz/i,fz:',reler,qz/i,fz
      call utmsgf
           endif
           if(relerh.gt.1e-3)then
      call utmsg('racpro')
      write(ifch,*)'*****  relerh too large'
      write(ifch,*)'relerh,qzh/i,fzh:',relerh,qzh/i,fzh
      call utmsgf
           endif
           if(sigppz.lt.0.)then
      call utmsg('racpro')
      write(ifch,*)'*****  negative sigppz'
      write(ifch,*)'sigppz,sig,fzh,fz:',sigppz,sig,fzh,fz
      call utmsgf
           endif
      do 24 i=1,n
      if(acprob(i).lt.0)then
      call utmsg('racpro')
      write(ifch,*)'*****  negative acprob(i)'
      write(ifch,*)'i,acprob(i):',i,acprob(i)
      call utmsgf
      endif
24    continue
      relern=acprob(n)-acprob(n-1)
           if(relern.gt.1e-3)then
      call utmsg('racpro')
      write(ifch,*)'*****  relern too large'
      write(ifch,*)'relern:',relern      
      call utmsgf
           endif
 
c     print
c     -----
      if(ish.ge.95)then
100   format(79a1)
      write(ifch,*)' '
      write(ifch,100)('-',ic=1,79)
      write(ifch,*)'   cross sections and',
     *' cut-pomeron weights acc to gribov-regge-theory'
      write(ifch,100)('-',ic=1,79)
      write(ifch,*)'   cms-energy [gev]:',engy
      write(ifch,100)('-',ic=1,79)
      write(ifch,*)' '
      write(ifch,*)'   parameters: '
      write(ifch,*)' '
      write(ifch,*)'     gamma: ',grigam
      write(ifch,*)'     r**2:  ',grirsq
      write(ifch,*)'     delta: ',gridel
      write(ifch,*)'     slope: ',grislo
      write(ifch,*)'     c:     ',gricel
      write(ifch,*)' '
      write(ifch,*)'   cross sections:'
      write(ifch,*)' '
      write(ifch,*)'     elastic:     ',sigppe
      write(ifch,*)'     diffractive: ',sigppd
      write(ifch,*)'     inelastic:   ',sigppi
      write(ifch,*)' '
      write(ifch,*)'   weights w(n) of n cut pomerons:'
      write(ifch,*)'      ( n - w(n) )'
      write(ifch,*)' '
      i=1
      write(ifch,*)i,acprob(i)
      do 25 i=2,n
25    write(ifch,*)i,acprob(i)-acprob(i-1)
      write(ifch,*)' '
      endif
 
      ish=ishini
      return
      end
 
c-----------------------------------------------------------------------
      subroutine randfl(jc,iqa0,iflav,ic,isame)
c     returns random flavour ic(2) (iqa0=1:quark,2:antiquark,11:diquark)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (nflav=6)
      real probab(nflav),probsu(nflav+1)
      integer jc(nflav,2),jc0(nflav,2),ic(2)
      character dash*1
      dash='-'
      if(ish.ge.95)then
      write(ifch,*)('-',i=1,10)
     *,' entry sr randfl ',('-',i=1,30)
      write(ifch,*)'iqa0:',iqa0
      write(ifch,*)'jc:'
      write(ifch,*)jc
      endif
      iflav=0
      ic(1)=0
      ic(2)=0
      do 10 n=1,nflav
      do 10 i=1,2
10    jc0(n,i)=0
      iqa1=iqa0*10
9999  iqa1=iqa1/10
      if(iqa1.eq.0)goto9998
      iqa=mod(iqa1,10)
      su=0
      do 20 i=1,nflav
      probab(i)=jc(i,iqa)-jc0(i,iqa)
      if(isame.eq.1)probab(i)=probab(i)*(jc(i,3-iqa)-jc0(i,3-iqa))
20    su=su+probab(i)
      if(su.lt..5)then
      iflav=0
      ic(1)=0
      ic(2)=0
      goto9998
      endif
      probsu(1)=0.
      do 30 i=1,nflav
      probsu(i+1)=probsu(i)+probab(i)/su
      if(probsu(i+1)-probsu(i).lt.1e-5)probsu(i+1)=probsu(i)
30    continue
      r=rangel(probsu(nflav+1))
      do 50 i=1,nflav
      if(probsu(i).le.r.and.r.lt.probsu(i+1))iflav=i
50    continue
      jc0(iflav,iqa)=jc0(iflav,iqa)+1
      if(isame.eq.1)jc0(iflav,3-iqa)=jc0(iflav,3-iqa)+1
      call idenco(jc0,ic,ireten)
      if(ireten.eq.1)
     *call utstop('randfl: idenco ret code = 1             ')
      if(ish.ge.95)then
      write(ifch,*)'probab:'
      write(ifch,*)probab
      write(ifch,*)'probsu:'
      write(ifch,*)probsu
      write(ifch,*)'ran#:',r,'   flav:',iflav
      endif
      goto9999
9998  continue
      if(ish.ge.95)write(ifch,*)('-',i=1,30)
     *,' exit sr qnbdec ',('-',i=1,10)
      return
      end
 
c-----------------------------------------------------------------------
      real function ranf()
c     uniform random number generator from cern library
c-----------------------------------------------------------------------
      double precision    dranf,    g900gt,   g900st
      double precision    ds(2),    dm(2),    dseed
      double precision    dx24,     dx48
      double precision    dl,       dc,       du,       dr
      logical             single
      data      ds     /  1665 1885.d0, 286 8876.d0  /
      data      dm     /  1518 4245.d0, 265 1554.d0  /
      data      dx24   /  1677 7216.d0  /
      data      dx48   /  281 4749 7671 0656.d0  /
      single  =  .true.
      goto 10
      entry dranf()
      single  =  .false.
  10  dl  =  ds(1) * dm(1)
      dc  =  dint(dl/dx24)
      dl  =  dl - dc*dx24
      du  =  ds(1)*dm(2) + ds(2)*dm(1) + dc
      ds(2)  =  du - dint(du/dx24)*dx24
      ds(1)  =  dl
      dr     =  (ds(2)*dx24 + ds(1)) / dx48
      if(single)  then
         ranf  =  sngl(dr)
      else
         dranf  =  dr
      endif
      return
      entry g900gt()
      g900gt  =  ds(2)*dx24 + ds(1)
      return
      entry g900st(dseed)
      ds(2)  =  dint(dseed/dx24)
      ds(1)  =  dseed - ds(2)*dx24
      g900st =  ds(1)
      return
      end
      subroutine ranfgt(seed)
      double precision    seed,     g900gt,   g900st,   dummy
      seed  =  g900gt()
      return
      entry ranfst(seed)
      dummy  =  g900st(seed)
      return
      end
 
c-----------------------------------------------------------------------
      function rangen()
c     generates a random number
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      double precision seedi,seedj,seedc
      common /cseed/seedi,seedj,seedc
1     rangen=ranf()
      if(rangen.le.0.)goto1
      if(rangen.ge.1.)goto1
      if(irandm.eq.1)write(ifch,*)'rangen()= ',rangen
      return
      entry rangel(xmax)
2     rangel=ranf()*xmax
      if(rangel.le.0.)goto2
      if(rangel.ge.xmax)goto2
      return
      end
 
c-----------------------------------------------------------------------
      subroutine ranhvy(x,eps)
c     generates x for heavy particle fragmentation according to
c     the peterson form
c          d(x)=1/(x*(1-1/x-eps/(1-x))**2)
c              =d0(x)*d1(x)*d2(x)
c          d0(x)=(1-x)**2/((1-x)**2+eps)**2
c          d1(x)=x
c          d2(x)=(((1-x)**2+eps)/((1-x)**2+eps*x))**2
c     using x=1-y**pow
c     generates flat in x if eps>1.
c-----------------------------------------------------------------------
      data aln4/1.3863/
      if(eps.lt.1.) then
        pow=alog((3.+eps)/eps)/aln4
        ymx=(eps*(3.*pow-1.)/(pow+1.))**(.5/pow)
        zmx=1-ymx**pow
        d0mx=(1-zmx)**2/((1.-zmx)**2+eps)**2*pow*ymx**(pow-1.)
        d2mx=2./(2.-sqrt(eps))
      else
        pow=1.
        zmx=0.
        d0mx=(1.-zmx)**2/((1.-zmx)**2+eps)**2
        d2mx=1.+eps
      endif
c
c          generate z according to (1-z)**2/((1-z)**2+eps*z)**2
1     continue
      y=rangen()
      z=1.-y**pow
c
      d0z=(1.-z)**2/((1.-z)**2+eps)**2*pow*y**(pow-1.)
      if(d0z.lt.rangen()*d0mx) goto1
c
c          check remaining factors
      d1=z
      d2=(((1.-z)**2+eps)/((1.-z)**2+eps*z))**2
      if(d1*d2.lt.rangen()*d2mx) goto1
c
c          good x
      x=z
      return
      end
 
c-----------------------------------------------------------------------
      function ransig()
c     returns randomly +1 or -1
c-----------------------------------------------------------------------
      ransig=1
      if(rangen().gt.0.5)ransig=-1
      return
      end
 
c-----------------------------------------------------------------------
      function ranstc(xfl,xmin)
c     returns random number according to a quark structure fctn
c       with x>=xmin
c-----------------------------------------------------------------------
      parameter (nstru=2049)
      common/stru/xstru(nstru),wstru(nstru)
     *,qvap(nstru),qvat(nstru),qsep(nstru),qset(nstru)
      character xfl*3
      if(xfl.eq.'vap')ranstc=ranxq(nstru,xstru,qvap,xmin)
      if(xfl.eq.'sep')ranstc=ranxq(nstru,xstru,qsep,xmin)
      if(xfl.eq.'vat')ranstc=ranxq(nstru,xstru,qvat,xmin)
      if(xfl.eq.'set')ranstc=ranxq(nstru,xstru,qset,xmin)
      return
      end
 
c-----------------------------------------------------------------------
      function ranxq(n,x,q,xmin)
c     returns random number according to x(i) q(i) with x>=xmin
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      real x(n),q(n)
      imin=1
      if(xmin.eq.0.)goto3
      i1=1
      i2=n
1     i=i1+(i2-i1)/2
      if(x(i).lt.xmin)then
      i1=i
      elseif(x(i).gt.xmin)then
      i2=i
      else
      imin=i
      goto3
      endif
      if(i2-i1.gt.1)goto1
      imin=i2
3     continue
      if(q(imin).gt.q(n)*.9999)then
      ranxq=xmin
      goto4
      endif
      qran=q(imin)+rangen()*(q(n)-q(imin))
      ranxq=utinvt(n,x,q,qran)
4     continue
 
      if(ranxq.lt.xmin)then
      call utmsg('ranxq ')
      write(ifch,*)'*****  ranxq=',ranxq,' <       xmin=',xmin
      write(ifch,*)'q(imin) q q(n):',q(imin),qran,q(n)
      write(ifch,*)'x(imin) x x(n):',x(imin),ranxq,x(n)
      call utmsgf
      ranxq=xmin
      endif
 
      return
      end
 
c-----------------------------------------------------------------------
      function sbet(z,w)
c-----------------------------------------------------------------------
      sbet=sgam(z)*sgam(w)/sgam(z+w)
      return
      end
 
c-----------------------------------------------------------------------
      function sdensi(r)
c     nuclear density
c-----------------------------------------------------------------------
      common /cden/massnr
      a=0.54
      r0=1.19*massnr**(1./3.)-1.61*massnr**(-1./3.)
      sdensi=r**2/(1+exp((r-r0)/a))
      return
      end
 
c-----------------------------------------------------------------------
      function sgam(x)
c     gamma fctn
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      dimension c(13)
      data c
     1/ 0.00053 96989 58808, 0.00261 93072 82746, 0.02044 96308 23590,
     2  0.07309 48364 14370, 0.27964 36915 78538, 0.55338 76923 85769,
     3  0.99999 99999 99998,-0.00083 27247 08684, 0.00469 86580 79622,
     4  0.02252 38347 47260,-0.17044 79328 74746,-0.05681 03350 86194,
     5  1.13060 33572 86556/
      sgam=0
      z=x
      if(x .gt. 0.0) goto1
      if(x .eq. aint(x)) goto5
      z=1.0-z
    1 f=1.0/z
      if(z .le. 1.0) goto4
      f=1.0
    2 continue
      if(z .lt. 2.0) goto3
      z=z-1.0
      f=f*z
      goto2
    3 z=z-1.0
    4 sgam=
     1 f*((((((c(1)*z+c(2))*z+c(3))*z+c(4))*z+c(5))*z+c(6))*z+c(7))/
     2   ((((((c(8)*z+c(9))*z+c(10))*z+c(11))*z+c(12))*z+c(13))*z+1.0)
      if(x .gt. 0.0) return
      sgam=3.141592653589793/(sin(3.141592653589793*x)*sgam)
      return
    5 write(ifch,10)x
   10 format(1x,'argument of gamma fctn = ',e20.5)
      call utstop('sgam : negative integer argument        ')
      end
 
c-----------------------------------------------------------------------
      function sgau(x)
c     returns gaussian distribution (not normalized)
c-----------------------------------------------------------------------
      sgau=exp(-x**2/2)
      return
      end
 
c----------------------------------------------------------------------
      function sjcent(k,ku,u)
c     returns entropy.
c     input: quark number k; energy u [gev].
c     iopent=1: oscillator model.
c       interpolates and extrapolates entro(1+k,1+n)
c       from sr jcentr (jcentd).
c       sr jcentr (jcentd) has to be called before!!
c     iopent=2,3: fermi gas model; joerg  aichelin.
c       iopent=2: const volume, iopent=3: const density
c     iopent=5: hagedorn
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (kparx=15,nquax=12)
      common/centro/entro(1+kparx,1+nquax)
      common/paro3/uentro,sigppe,sigppd,asuhax(7),asuhay(7),omega
      iop=mod(iopent,10)
 
      sjcent=0
 
           if(iop.eq.0)then
 
      return
 
           elseif(iop.eq.1)then
 
      if(k.le.kentro)return
      if(mod(k,3).ne.0)
     *call utstop('sjcent: k must be multiple of 3         ')
      e=u/omega
      if(k.gt.kparx)goto5001
      n=int(e)
      if(n.lt.0)
     *call utstop('sjcent: negative energy                 ')
      if(n.ge.nquax)n=nquax-1
      sjcent=entro(1+k,1+n)+(e-n)*(entro(1+k,1+n+1)-entro(1+k,1+n))
      if(k.le.3)return
5002  e3=e*3./k
      n3=int(e3)
      if(n3.ge.nquax)n3=nquax-1
      sentr3=entro(1+3,1+n3)+(e3-n3)*(entro(1+3,1+n3+1)-entro(1+3,1+n3))
      sjcent=amin1(sjcent,k/3.*sentr3)
      return
5001  continue
      l=kparx
      el=(e*l)/k
      nl=int(el)
      if(nl.ge.nquax)nl=nquax-1
      sentrl=entro(1+l,1+nl)+(el-nl)*(entro(1+l,1+nl+1)-entro(1+l,1+nl))
      sjcent=(k*1.)/l*sentrl
      goto5002
 
           elseif(iop.eq.2)then
 
c     constant volume 5 fm**3
      if(k.le.kentro)return
      sjcent = 0
      x1=12.96*k**(-.315)
      if(k.lt.19)then
      x2=.785 + .005*k
      else
      x2=.88
      endif
      sjcent=k*x1*(u/k)**x2
      return
 
           elseif(iop.eq.3)then
 
c     constant density
      if(k.le.kentro)return
      sjcent = 0
      x1=9.785
      x2=.7926
      sjcent=k*x1*(u/k)**x2
      return
 
           elseif(iop.eq.5)then
 
c     hagedorn
      if(u.le.uentro.and.k.le.kentro)return
      thaged=.250
      sjcent = u/thaged   
      return
 
           else
 
      call utstop('sjcent: invalid option iop              ')
 
           endif
 
      end
 
c----------------------------------------------------------------------
      function sjcgam(keux,kedx,kesx,kecx,ama,amo,po,mox)
c     returns partial decay width dgamma = phase space * density
c       for decay of cluster into cluster and hadron.
c     ke*x: net quark number
c     ama: cluster mass;  amo: hadron mass;  po: hadron momentum
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/csjcga/amnull,nsuha(7),asuha(7),entrpy,amegam,nopha
      common/paro3/uentro,sigppe,sigppd,asuhax(7),asuhay(7),omega
      common/centex/entexp
      sjcgam=0.
      entrpy=-99.9
      amegam=0.
      if(mox.eq.1)entexp=0.
      entrpy=entrpy-entexp
      ke=iabs(keux+kedx+kesx+kecx)
      ku=iabs(keux+kedx)
      if(mox.eq.1)amnull=utamnu(keux,kedx,kesx,kecx,iopenu)
      eo=sqrt(amo**2+po**2)
      if(eo.gt.ama)return
      ame2=(ama-eo)**2-po**2
      if(ame2.lt.0.)return
      amegam=sqrt(ame2)
      e=amegam-amnull
      if(e.lt.0.)return
      entrpy=sjcent(ke,ku,e)
      if(mox.eq.1)entexp=entrpy
      entrpy=entrpy-entexp
      sjcgam=1./8./pi**2/ama*
     *exp(entrpy)*
     *po**2/sqrt(amo**2+po**2)
      return
      end
 
c-----------------------------------------------------------------------
      function sjctem(k,ku,u)
c     returns temperature. input: quark number k; u,d qurks ku; energy u
c-----------------------------------------------------------------------
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      du=0.001
      beta=(sjcent(k,ku,u+du)-sjcent(k,ku,u))/du
      if(beta.lt.1/ainfin)then
      sjctem=ainfin
      else
      sjctem=1/beta
      endif
      return
      end
 
c-----------------------------------------------------------------------
      function smass(a,y,z)
c     returns cluster mass (in gev) (per cluster, not (!) per nucleon)
c     according to berger/jaffe mass formula, prc35(1987)213 eq.2.31,
c     see also c. dover, BNL-46322, intersections-meeting, tucson, 91.
c     a: massnr, y: hypercharge, z: charge,
c-----------------------------------------------------------------------
      common/cmass/thet,epsi,as,ac,dy,dz,ym,cz,zm,sigma,rzero
      ymin=ym*a
      zmin=cz/(dz/a+zm/a**(1./3.))
      smass=epsi*a+as*a**(2./3.)+(ac/a**(1./3.)+dz/a/2.)*(z-zmin)**2
     *+dy/a/2.*(y-ymin)**2
      return
      end
 
c-----------------------------------------------------------------------
      subroutine smassi(theta)
c     initialization for smass.
c     calculates parameters for berger/jaffe mass formula
c     (prc35(1987)213 eq.2.31, see also c. dover, BNL-46322).
c     theta: parameter that determines all parameters in mass formula.
c-----------------------------------------------------------------------
      common/cmass/thet,epsi,as,ac,dy,dz,ym,cz,zm,sigma,rzero
      thet=theta
 
      astr=.150
      pi=3.14159
      alp=1./137.
 
      co=cos(theta)
      si=sin(theta)
      bet=(1+co**3)/2.
      rzero=si/astr/(  2./3./pi*(1+co**3)  )**(1./3.)
      cs=astr/si
      cz=-astr/si*(  (  .5*(1+co**3)  )**(1./3.)-1  )
      sigma=6./8./pi*(astr/si)**3*(co**2/6.-si**2*(1-si)/3.-
     *1./3./pi*(pi/2.-theta-sin(2*theta)+si**3*alog((1+co)/si)))
 
      epsi=astr*((.5*(1+co**3))**(1./3.)+2)/si
      as=4*pi*sigma*rzero**2
      ac=3./5.*alp/rzero
      dz=astr/si*bet**(1./3.)*co**2*
     *(co**4*(1+bet**(2./3.))+(1+bet)**2)/
     *(  (2*co**2+bet**(1./3.))*(co**4*(1+bet**(2./3.))+(1+bet)**2)-
     *(co**4+bet**(1./3.)*(1+bet))*((2*bet**(2./3.)-1)*co**2+1+bet)  )
      dy=astr/6.*(1+co**3)**3/si*
     *(  1+(1+co)/(4*(1+co**3))**(2./3.)  )/
     *(co**6+co+co*(.5*(1+co**3))**(4./3.))
      zm=6*alp/(5*rzero)
      ym=(1-co**3)/(1+co**3)
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine smassp
c     prints smass.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/cmass/thet,epsi,as,ac,dy,dz,ym,cz,zm,sigma,rzero
      real eng(14),ymi(14),zmi(14)
      pi=3.14159
      write(ifch,*)'parameters of mass formula:'
      write(ifch,*)'theta=',thet,'   epsi=',epsi
      write(ifch,*)'as=',as,'   ac=',ac
      write(ifch,*)'dy=',dy,'   dz=',dz
      write(ifch,*)'ym=',ym
      write(ifch,*)'cz dz zm=',cz,dz,zm
      write(ifch,*)'sigma**1/3=',sigma**(1./3.),'   rzero=',rzero
      write(ifch,*)'mass:'
      write(ifch,5000)(j,j=1,14)
5000  format(5x,'a:',14i5)
      do 4 j=1,14
      a=j
      ymi(j)=ym*a
4     zmi(j)=cz/(dz/a+zm/a**(1./3.))
      write(ifch,5002)(ymi(j),j=1,14)
5002  format(1x,'ymin: ',14f5.2)
      write(ifch,5003)(zmi(j),j=1,14)
5003  format(1x,'zmin: ',14f5.2)
      do 2 i=1,15
      ns=11-i
      do 3 j=1,14
      a=j
      y=a-ns
      z=0.
3     eng(j)=smass(a,y,z)/a
      write(ifch,5001)ns,(eng(j),j=1,14)
5001  format(1x,'s=',i2,2x,14f5.2)
2     continue
      write(ifch,*)'mass-mass(free):'
      write(ifch,5000)(j,j=1,14)
      do 5 i=1,15
      ns=11-i
      do 6 j=1,14
      a=j
      y=a-ns
      z=0.
      call smassu(a,y,z,ku,kd,ks,kc)
6     eng(j)=(smass(a,y,z)-utamnu(ku,kd,ks,kc,3))/a
      write(ifch,5001)ns,(eng(j),j=1,14)
5     continue
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine smasst(kux,kdx,ksx,kcx,a,y,z)
c     input: kux,kdx,ksx,kcx = net quark numbers (for u,d,s,c quarks).
c     output: massnr a, hypercharge y and charge z.
c-----------------------------------------------------------------------
      sg=1
      if(kux+kdx+ksx+kcx.lt.0.)sg=-1
      ku=sg*kux
      kd=sg*kdx
      ks=sg*ksx
      kc=sg*kcx
      k=ku+kd+ks+kc
      if(mod(k,3).ne.0)stop'noninteger baryon number'
      a=k/3
      y=a-ks
      nz=2*ku-kd-ks+2*kc
      if(mod(nz,3).ne.0)stop'noninteger charge'
      z=nz/3
      return
      end
 
c-----------------------------------------------------------------------
      subroutine smassu(ax,yx,zx,ku,kd,ks,kc)
c     input: massnr ax, hypercharge yx and charge zx.
c     output: ku,kd,ks,kc = net quark numbers (for u,d,s,c quarks).
c-----------------------------------------------------------------------
      sg=1
      if(ax.lt.0.)sg=-1
      a=sg*ax
      y=sg*yx
      z=sg*zx
      ku=nint(a+z)
      kd=nint(a-z+y)
      ks=nint(a-y)
      kc=0
      return
      end
 
c-----------------------------------------------------------------------
      function spoc(a,b,c,d,x)
c     power fctn with cutoff
c-----------------------------------------------------------------------
      spoc=0
      if(a.eq.0..and.b.eq.0.)return
      spoc =a+b*x**c
      spoc0=a+b*d**c
      spoc=amin1(spoc,spoc0)
      spoc=amax1(0.,spoc)
      return
      end
 
c-----------------------------------------------------------------------
      function sptf(x)
c     returns pt-distribution for fragmentation
c-----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common /cquama/quama
      if(ioptf.eq.1)sptf=x*exp(-2/ptf*sqrt(x**2+quama**2))
      if(ioptf.eq.2)sptf=x*exp(-pi/4*(x**2+quama**2)/ptf**2)
      return
      end
 
c-----------------------------------------------------------------------
      function spth(x)
c     returns pt-distribution for hard scattering
c-----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      spth=pth**2*2.*x/(x**2+pth**2)**3
      return
      end
 
c-----------------------------------------------------------------------
      function sptj(x)
c     jpsi pt-distribution in 200 gev pp
c-----------------------------------------------------------------------
      a=0.95
      c=1/0.363
      z=x/a
      sptj=1/a*c**c/sgam(c)*z**(c-1)*exp(-c*z)
      return
      end
 
c-----------------------------------------------------------------------
      function sptq(x)
c     returns pt-distribution of quarks in nucleons
c-----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      if(ioptq.eq.1)sptq=4./ptq**2*x*exp(-2/ptq*x)
      if(ioptq.eq.2)sptq=pi/2./ptq**2*x*exp(-pi/4*x**2/ptq**2)
      if(ioptq.eq.3)sptq=ptq**2*2.*x/(x**2+ptq**2)**2
      return
      end
 
c-----------------------------------------------------------------------
      function ssplit(x)
c     returns splitting fctn
c-----------------------------------------------------------------------
c-c   ssplit= ( 0.3 + 10.0*x**7 - 10.3*x**15 ) /sqrt(x**2+.2**2)
c-c   ssplit=   0.2 +  2.4*x**2 -  2.6*x**5
c-c   ssplit=   0.4 +  2.0*x    -  2.4*x**3
c-c   ssplit= ( 0.8 +  2.0*x    -  2.8*x**3  ) /sqrt(x**2+.2**2)
      ssplit= x**3 - x**5
      return
      end
 
c-----------------------------------------------------------------------
      function ssplix(x)
c     returns splitting fctn
c-----------------------------------------------------------------------
c-c   ssplix= (1-x) - (1-x)**7
c-c   ssplix= 1
c-c   z=2*x-.5
c-c   ssplix= z**3 - z**5
c-c   if(z.lt.0.)ssplix=0.
c-c   if(z.gt.1.)ssplix=0.
c-c   ssplix= x**3 - x**5
      ssplix= x**3 * (1-x)**3
      return
      end
 
c-----------------------------------------------------------------------
c     structure functions.
c-----------------------------------------------------------------------

      subroutine staa(x,q2i,z,s)
c     initialization
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      z=sqrt(x**2+xcut**2)
      q2=amax1(4.00001,q2i)
      s=alog(alog(q2/.2**2)/alog(4/.2**2))
      return
      end
 
      function stgdv(x,q2)
c     d-valence structure fctn
      stgdv=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      stgdv=.541*x**.364*(1-x**2)**5.09/z
      return
      end
 
      function stgus(x,q2)
      stgus=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      stgus=0.25*(1-x)**7/z
      return
      end
 
      function stguv(x,q2)
      stguv=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      stguv=1.23*x**.421*(1-x**2)**3.37/z
      return
      end
 
      function stodpr(x,q2)
      stodpr=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      eta3=.763-.237*s+.026*abs(s)**2
      eta4=4.0+.627*s-.019*abs(s)**2
      gd=-.421*s+.033*abs(s)**2
      b=sbet(eta3,eta4+1)*(1+gd*eta3/(eta3+eta4+1))
      stodpr=1/b*x**eta3*(1-x)**eta4*(1+gd*x)/z
      return
      end
 
      function stoglu(x,q2)
c     glue structure fctn
      stoglu=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      stoglu=(1.56-1.71*s+.638*abs(s)**2)*
     *x**(-.949*s+.325*abs(s)**2)*
     *(1-x)**(6.0+1.44*s-1.05*abs(s)**2)*
     *(1+(9.0-7.19*s+.255*abs(s)**2)*x
     *+(-16.5*s+10.9*abs(s)**2)*x**2
     *+(15.3*s-10.1*abs(s)**2)*x**3)
     */z
      return
      end
 
      function stospi(x,q2)
      stospi=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      a=0.9
      b=5
      stospi=a*(1-x)**b/z
      return
      end
 
      function stospr(x,q2)
      stospr=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      stospr=(1.265-1.132*s+.293*abs(s)**2)*
     *x**(-.372*s-.029*abs(s)**2)*
     *(1-x)**(8.05+1.59*s-.153*abs(s)**2)*
     *(1+(6.31*s-.273*abs(s)**2)*x
     *+(-10.5*s-3.17*abs(s)**2)*x**2
     *+(14.7*s+9.80*abs(s)**2)*x**3)
     */z
      return
      end
 
      function stovpi(x,q2)
      stovpi=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      a=0.4
      b=0.7
      stovpi=x**a*(1-x)**b/sbet(a,b+1)/z
      return
      end
 
      function stovpr(x,q2)
      stovpr=0
      if(x.eq.0.)return
      call staa(x,q2,z,s)
      eta1=.419+.004*s-.007*abs(s)**2
      eta2=3.46+.724*s-.066*abs(s)**2
      gud=4.4-4.86*s+1.33*abs(s)**2
      a=sbet(eta1,eta2+1)*(1+gud*eta1/(eta1+eta2+1))
      stovpr=3/a*x**eta1*(1-x)**eta2*(1+gud*x)/z
      return
      end
 
      function stxd(x,q2)
c     valence d-quark distribution.
c     from glueck, hoffmann, reya, z. phys. c13 (1982) 119.
      call staa(x,q2,z,s)
      a=.364-.0368*s
      c=2.-.5414*s**.8
      d=5.09+.3463*s
      stxd=c*x**a*(1.-x**c)**d/sbet(d+1.,a/c)
      return
      end
 
      function stxs(x,q2)
      call staa(x,q2,z,s)
c     s-sea distribution.
c     from glueck, hoffmann, reya, z. phys. c13 (1982) 119.
      a=.0625+.1132*s**1.3
      b=12.64*s-51.70*s**1.8+38.02*s**2
      c=4.448*s
      d=7.+1.562*s
      e=.3081*s**2.5
      f=47.24+67.91*s
      stxs=a*(1.+b*x+c*x**2)*(1.-x)**d + e*exp(-f*x)
      return
      end
 
      function stxu(x,q2)
c     valence u-quark distribution.
c     from glueck, hoffmann, reya, z. phys. c13 (1982) 119.
      call staa(x,q2,z,s)
      a=.421-.0412*s
      c=2.-.6223*s**.8
      d=3.37+.4319*s
      stxu=2.*c*x**a*(1.-x**c)**d/sbet(d+1.,a/c)
      return
      end
 
      function stxus(x,q2)
c     u-sea distribution.
c     from glueck, hoffmann, reya, z. phys. c13 (1982) 119.
      call staa(x,q2,z,s)
      a=.25+.088*s**1.3
      b=.8128*s-2.003*s**1.8+.0831*s**2
      c=3.97*s
      d=7.+1.666*s
      e=.2487*s**2.5
      f=27.8+59.68*s
      stxus=a*(1.+b*x+c*x**2)*(1.-x)**d + e*exp(-f*x)
      return
      end
 
      function stxzne(x,q2)
c     structure function of neutron
      stxzne=(4.*stxd(x,q2) + stxu(x,q2) +
     *10.*stxus(x,q2) + 2.*stxs(x,q2))/9.
      return
      end
 
      function stxzpr(x,q2)
c     structure function of proton
      stxzpr=(4.*stxu(x,q2) + stxd(x,q2) +
     *10.*stxus(x,q2) + 2.*stxs(x,q2))/9.
      return
      end

c----------------------------------------------------------
c  sty1?? = grv = glueck, reya, vogt, z.phys.c 53 (1992), 127
c  e. furler 4.11.94
c-----------------------------------------------------------

      subroutine sty1aa(q2,x,z,s)
c     returns z and s
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
       q2i=q2
       z=sqrt(x**2+xcut**2)
       smu2  = 0.3
       slam2 = 0.248**2
       s = alog(alog(amax1(smu2,q2i)/slam2) / alog(smu2/slam2))
      return
      end

      function sty1bs(x,q2,s)
c  b sea distribution
       call utstop('sty1bs: b distribution not tested!    ')
       sty1bs = 0.0
       if (x.le.0.) return
       rlnx = alog(1/x)
       wx = sqrt(x)
       sb =  1.297
       a  =   0.99
       b  =   0.0
       ak =   0.0  - 0.193 * s
       ag =   0.0
       bg =   0.0
       d  =  3.447 + 0.927 * s
       e  =   4.68 + 1.259 * s
       e1 =  1.892 + 2.199 * s
       sty1bs = (s - sb)**a / rlnx**ak * (1 + ag*wx + bg*x)
     *           * (1 - x)**d *
     *           exp(-e + sqrt(e1 * s**b * rlnx))
      return
      end

      function sty1cs(x,q2,s)
c  c sea distribution
       call utstop('sty1bs: c distribution not tested!    ')
       sty1cs = 0.0
       if (x.le.0.) return
       x  = amin1(1.,x)
       rlnx = alog(1/x)
       wx = sqrt(x)
       s2 = s**2
       sc =  0.820
       a  =   0.98
       b  =   0.0
       ak = -0.625 - 0.523 * s
       ag =   0.0
       bg =  1.896 + 1.616 * s
       d  =   4.12 + 0.683 * s
       e  =   4.36 + 1.328 * s
       e1 =  0.677 + 0.679 * s
       sty1cs = (s - sc)**a / rlnx**ak * (1 + ag*wx + bg*x)
     *           * (1 - x)**d *
     *           exp(-e + sqrt(e1 * s**b * rlnx))
      return
      end

      function sty1dv(x,q2,s)
c  d-valence distribution
       sty1dv = 0.0
       ws = sqrt(s)
       wx = sqrt(x)
       s2 = s**2
       rn = 0.459 + 0.315 * ws + 0.515 * s
       ak = 0.624              - 0.031 * s
       ag =  8.13 -  6.77 * ws +  0.46 * s
       b  =  6.59 - 12.83 * ws +  5.65 * s
       d  =  3.98              +  1.04 * s  -  0.34 * s2
       sty1dv = rn * x**ak * (1 + ag*wx + b*x) * (1 - x)**d
      return
      end

      function sty1gl(x,q2,s)
c  gluon  distribution
       sty1gl = 0.0
       if (x.le.0.) return
       rlnx = alog(1/x)
       s2 = s**2
       a =  1.128
       b =  1.575
       ak =  0.323 + 1.653 * s
       bk =  0.811 + 2.044 * s
       ag =   0.0  + 1.963 * s - 0.519 * s2
       bg =  0.078 +  6.24 * s
       c  =  30.77 - 24.19 * s
       d  =  3.188 + 0.720 * s
       e  = -0.881 + 2.687 * s
       e1 =  2.466
       sty1gl = (x**ak * (ag + bg*x + c*x**2) * rlnx**bk + 
     *          s**a * exp(-e + sqrt(e1 * s**b * rlnx))) * (1 - x)**d
      return
      end
 
      function sty1ss(x,q2,s)
c  s sea distribution
       sty1ss = 0.0
       if (x.le.0.) return
       x  = amin1(1.,x)
       rlnx = alog(1/x)
       if (rlnx.le.0.) rlnx = 1.e-6
       wx = sqrt(x)
       s2 = s**2
       ss =   0.0
       a  =  0.756
       b  =  0.101
       ak =  2.942 - 1.016 * s
       ag =  -4.60 + 1.167 * s
       bg =   9.31 - 1.324 * s
       d  =  11.49 - 1.198 * s + 0.053 * s2
       e  =  2.630 + 1.729 * s
       e1 =   8.12
       sty1ss = (s - ss)**a / rlnx**ak * (1 + ag*wx + bg*x) 
     *           * (1 - x)**d *
     *           exp(-e + sqrt(e1 * s**b * rlnx))
      return
      end

      function sty1us(x,q2,s)
c  light sea distribution
       sty1us = 0.0
       if (x.le.0.0) return
       x  = amin1(1.,x)
       rlnx = alog(1/x)
       if (rlnx.eq.0.) rlnx = 1.e-6
       a =  0.594
       b =  0.614
       ak =  0.636 - 0.084 * s
       bk =   0.0
       ag =  1.121 - 0.193 * s
       bg =  0.751 - 0.785 * s
       c  =   8.57 - 1.763 * s
       d  =  10.22 + 0.668 * s
       e  =  3.784 + 1.280 * s
       e1 =  1.808 + 0.980 * s
       sty1us = ((x**ak) * (ag + bg*x + c*x**2) * (rlnx**bk) + 
     *           (s**a)  * exp(-e + sqrt(e1 * s**b * rlnx))) 
     *        * (1 - x)**d
      return
      end
       
      function sty1uv(x,q2,s)
c  u-valence distribution
      sty1uv=sty1vv(x,q2,s)-sty1dv(x,q2,s)
      return
      end

      function sty1vv(x,q2,s)
c  u-valence + d-valence  distribution
       sty1vv = 0.0
       wx = sqrt(x)
       s2 = s**2
       s3 = s2*s
       rn = 0.330 + 0.151 * s - 0.059 * s2 + 0.027 * s3
       ak = 0.285
       ag = -2.28 + 15.73 * s -  4.58 * s2
       b  =  56.7 -  53.6 * s + 11.21 * s2
       d  =  3.17 +  1.17 * s -  0.47 * s2 +  0.09 * s3
       sty1vv = rn * x**ak * (1 + ag*wx + b*x) * (1 - x)**d
      return
      end

c-------------------------------------------------
c  sty2?? = mrs = martin, roberts, stirling, 
c  ral  preprint ral-93-077 (1993), (dis)
c  phys. lett. b306 (1993) 145      (msbar)
c  uebernommen von der cern library pdflib.for
c  e. furler 4.11.94
c--------------------------------------------------

      subroutine sty2di(x,scale,upv,dnv,usea,dsea,str,chm,bot,glu)
c     this is the new h fit to the 1993 hera f2 data
c        new set  mrs(h-dis) (december 1993)    
c                                                    
c         fitted to the recent nmc     
c         and ccfr  structure function data,           
c         and have ubar < dbar in the sea. 
c         lambda(msbar,nf=4) = 230 mev                 
c                                                      
c     10**-5 < x < 1  and  5 < q**2 < 1.31 * 10**6 
c
      implicit double precision(a-h,o-z)
      parameter(nx=47)
      parameter(ntenth=21)
      dimension f(8,nx,20),g(8),xx(nx),n0(8)
        data ((f(i,1,k),i=1,5),f(7,1,k),f(6,1,k),f(8,1,k),k=1,7)/
     +    0.01508,   0.00234,  28.57361,   2.50233,   0.14546,
     +    0.00000,   1.34132,   2.52798,
     +    0.01551,   0.00248,  41.41998,   2.90445,   0.59834,
     +    0.00001,   1.76941,   2.92978,
     +    0.01591,   0.00262,  56.22984,   3.35828,   1.09605,
     +    0.00001,   2.24573,   3.38329,
     +    0.01629,   0.00274,  72.60197,   3.85731,   1.63369,
     +    0.00001,   2.76459,   3.88201,
     +    0.01665,   0.00287,  90.39349,   4.39562,   2.20645,
     +    0.12740,   3.32062,   4.42005,
     +    0.01698,   0.00300, 109.40785,   4.96962,   2.81165,
     +    0.69649,   3.91070,   4.99380,
     +    0.01730,   0.00311, 129.52321,   5.57897,   3.44960,
     +    1.39599,   4.53479,   5.60291 /
        data ((f(i,1,k),i=1,5),f(7,1,k),f(6,1,k),f(8,1,k),k=8,14)/
     +    0.01761,   0.00324, 150.59039,   6.22099,   4.11796,
     +    2.11291,   5.19041,   6.24470,
     +    0.01791,   0.00335, 172.47517,   6.89236,   4.81379,
     +    2.85689,   5.87442,   6.91586,
     +    0.01820,   0.00346, 195.07593,   7.59054,   5.53473,
     +    3.62358,   6.58438,   7.61385,
     +    0.01848,   0.00357, 218.32326,   8.31348,   6.27895,
     +    4.41183,   7.31835,   8.33663,
     +    0.01874,   0.00367, 242.10217,   9.05856,   7.04403,
     +    5.21909,   8.07380,   9.08157,
     +    0.01901,   0.00378, 266.14624,   9.82123,   7.82554,
     +    6.04096,   8.84624,   9.84408,
     +    0.01927,   0.00389, 290.66772,  10.60346,   8.62559,
     +    6.87977,   9.63772,  10.62618 /
        data ((f(i,1,k),i=1,5),f(7,1,k),f(6,1,k),f(8,1,k),k=15,19)/
     +    0.01951,   0.00399, 315.54395,  11.40309,   9.44207,
     +    7.73350,  10.44611,  11.42568,
     +    0.01975,   0.00408, 340.46313,  12.21363,  10.26866,
     +    8.59568,  11.26500,  12.23610,
     +    0.01999,   0.00419, 365.83740,  13.04181,  11.11208,
     +    9.47343,  12.10109,  13.06417,
     +    0.02021,   0.00428, 391.07251,  13.87698,  11.96183,
     +   10.35593,  12.94386,  13.89926,
     +    0.02044,   0.00438, 416.66797,  14.72739,  12.82614,
     +   11.25187,  13.80151,  14.74957 /
        data ((f(i,2,k),i=1,5),f(7,2,k),f(6,2,k),f(8,2,k),k=1,7)/
     +    0.01910,   0.00337,  23.12250,   2.03841,   0.11542,
     +    0.00000,   1.08670,   2.05327,
     +    0.01968,   0.00359,  33.11473,   2.35319,   0.47102,
     +    0.00000,   1.42219,   2.36782,
     +    0.02022,   0.00378,  44.40931,   2.70360,   0.85671,
     +    0.00001,   1.79052,   2.71801,
     +    0.02072,   0.00397,  56.67399,   3.08411,   1.26826,
     +    0.00000,   2.18684,   3.09833,
     +    0.02118,   0.00415,  69.79973,   3.49027,   1.70207,
     +    0.09624,   2.60708,   3.50430,
     +    0.02163,   0.00433,  83.63523,   3.91914,   2.15598,
     +    0.52392,   3.04873,   3.93300,
     +    0.02205,   0.00451,  98.08751,   4.37030,   2.63008,
     +    1.04547,   3.51159,   4.38401 /
        data ((f(i,2,k),i=1,5),f(7,2,k),f(6,2,k),f(8,2,k),k=8,14)/
     +    0.02246,   0.00467, 113.04887,   4.84163,   3.12254,
     +    1.57516,   3.99372,   4.85519,
     +    0.02285,   0.00484, 128.42445,   5.33066,   3.63115,
     +    2.12040,   4.49277,   5.34410,
     +    0.02323,   0.00499, 144.14645,   5.83550,   4.15422,
     +    2.67801,   5.00693,   5.84881,
     +    0.02359,   0.00515, 160.17074,   6.35465,   4.69040,
     +    3.24717,   5.53480,   6.36785,
     +    0.02394,   0.00531, 176.41951,   6.88621,   5.23795,
     +    3.82612,   6.07457,   6.89931,
     +    0.02428,   0.00546, 192.70798,   7.42673,   5.79356,
     +    4.41154,   6.62283,   7.43974,
     +    0.02461,   0.00560, 209.19710,   7.97800,   6.35906,
     +    5.00551,   7.18140,   7.99092 /
        data ((f(i,2,k),i=1,5),f(7,2,k),f(6,2,k),f(8,2,k),k=15,19)/
     +    0.02493,   0.00575, 225.80480,   8.53840,   6.93295,
     +    5.60657,   7.74873,   8.55127,
     +    0.02524,   0.00589, 242.31673,   9.10323,   7.51058,
     +    6.21002,   8.32013,   9.11601,
     +    0.02555,   0.00602, 259.02999,   9.67763,   8.09716,
     +    6.82136,   8.90079,   9.69039,
     +    0.02584,   0.00616, 275.53486,  10.25374,   8.68491,
     +    7.43260,   9.48288,  10.26643,
     +    0.02613,   0.00629, 292.18452,  10.83787,   9.28013,
     +    8.05038,  10.07273,  10.85050 /
        data ((f(i,3,k),i=1,5),f(7,3,k),f(6,3,k),f(8,3,k),k=1,7)/
     +    0.02417,   0.00488,  18.71066,   1.65943,   0.09145,
     +    0.00000,   0.88033,   1.66758,
     +    0.02494,   0.00517,  26.47142,   1.90530,   0.37020,
     +    0.00000,   1.14277,   1.91330,
     +    0.02563,   0.00547,  35.06683,   2.17515,   0.66844,
     +    0.00000,   1.42694,   2.18303,
     +    0.02629,   0.00574,  44.22931,   2.46448,   0.98272,
     +    0.00000,   1.72885,   2.47225,
     +    0.02690,   0.00600,  53.88190,   2.76996,   1.31043,
     +    0.07253,   2.04556,   2.77763,
     +    0.02748,   0.00626,  63.91248,   3.08932,   1.64989,
     +    0.39322,   2.37508,   3.09689,
     +    0.02804,   0.00650,  74.25426,   3.42216,   2.00114,
     +    0.78114,   2.71723,   3.42965 /
        data ((f(i,3,k),i=1,5),f(7,3,k),f(6,3,k),f(8,3,k),k=8,14)/
     +    0.02857,   0.00675,  84.83257,   3.76689,   2.36283,
     +    1.17147,   3.07054,   3.77432,
     +    0.02908,   0.00698,  95.58434,   4.12171,   2.73335,
     +    1.56995,   3.43331,   4.12907,
     +    0.02957,   0.00721, 106.46631,   4.48525,   3.11149,
     +    1.97427,   3.80425,   4.49254,
     +    0.03004,   0.00743, 117.45285,   4.85648,   3.49637,
     +    2.38397,   4.18240,   4.86371,
     +    0.03050,   0.00765, 128.49417,   5.23406,   3.88676,
     +    2.79781,   4.56646,   5.24124,
     +    0.03093,   0.00786, 139.46341,   5.61542,   4.28018,
     +    3.21339,   4.95394,   5.62256,
     +    0.03136,   0.00807, 150.48344,   6.00210,   4.67826,
     +    3.63247,   5.34640,   6.00920 /
        data ((f(i,3,k),i=1,5),f(7,3,k),f(6,3,k),f(8,3,k),k=15,19)/
     +    0.03178,   0.00827, 161.50014,   6.39301,   5.07995,
     +    4.05410,   5.74279,   6.40008,
     +    0.03218,   0.00847, 172.36797,   6.78466,   5.48185,
     +    4.47482,   6.13963,   6.79170,
     +    0.03256,   0.00866, 183.30180,   7.18108,   5.88799,
     +    4.89894,   6.54099,   7.18809,
     +    0.03295,   0.00885, 194.01912,   7.57645,   6.29266,
     +    5.32057,   6.94109,   7.58344,
     +    0.03332,   0.00904, 204.77129,   7.97556,   6.70065,
     +    5.74476,   7.34471,   7.98253 /
        data ((f(i,4,k),i=1,5),f(7,4,k),f(6,4,k),f(8,4,k),k=1,7)/
     +    0.02772,   0.00605,  16.53067,   1.47072,   0.07972,
     +    0.00000,   0.77826,   1.47640,
     +    0.02861,   0.00643,  23.21927,   1.68322,   0.32122,
     +    0.00000,   1.00530,   1.68881,
     +    0.02944,   0.00679,  30.53673,   1.91450,   0.57752,
     +    0.00000,   1.24915,   1.92001,
     +    0.03020,   0.00712,  38.25038,   2.16056,   0.84557,
     +    0.00000,   1.50626,   2.16601,
     +    0.03091,   0.00744,  46.30000,   2.41869,   1.12326,
     +    0.06140,   1.77423,   2.42408,
     +    0.03159,   0.00777,  54.59346,   2.68694,   1.40923,
     +    0.33205,   2.05139,   2.69229,
     +    0.03224,   0.00807,  63.07721,   2.96498,   1.70347,
     +    0.65786,   2.33758,   2.97028 /
        data ((f(i,4,k),i=1,5),f(7,4,k),f(6,4,k),f(8,4,k),k=8,14)/
     +    0.03286,   0.00837,  71.69281,   3.25146,   2.00488,
     +    0.98389,   2.63158,   3.25673,
     +    0.03345,   0.00865,  80.39113,   3.54491,   2.31214,
     +    1.31506,   2.93198,   3.55014,
     +    0.03402,   0.00893,  89.14072,   3.84423,   2.62431,
     +    1.64953,   3.23778,   3.84943,
     +    0.03456,   0.00921,  97.92462,   4.14861,   2.94069,
     +    1.98698,   3.54821,   4.15380,
     +    0.03509,   0.00948, 106.70447,   4.45696,   3.26029,
     +    2.32641,   3.86223,   4.46210,
     +    0.03560,   0.00973, 115.37932,   4.76715,   3.58110,
     +    2.66585,   4.17777,   4.77228,
     +    0.03610,   0.00998, 124.05480,   5.08060,   3.90455,
     +    3.00694,   4.49626,   5.08572 /
        data ((f(i,4,k),i=1,5),f(7,4,k),f(6,4,k),f(8,4,k),k=15,19)/
     +    0.03658,   0.01022, 132.68862,   5.39638,   4.22981,
     +    3.34887,   4.81683,   5.40148,
     +    0.03704,   0.01047, 141.16511,   5.71167,   4.55410,
     +    3.68886,   5.13665,   5.71676,
     +    0.03750,   0.01071, 149.66236,   6.02989,   4.88087,
     +    4.03056,   5.45919,   6.03497,
     +    0.03794,   0.01093, 157.95304,   6.34620,   5.20535,
     +    4.36910,   5.77962,   6.35127,
     +    0.03836,   0.01116, 166.24386,   6.66467,   5.53162,
     +    4.70876,   6.10203,   6.66975 /
        data ((f(i,5,k),i=1,5),f(7,5,k),f(6,5,k),f(8,5,k),k=1,7)/
     +    0.03054,   0.00704,  15.13950,   1.34971,   0.07229,
     +    0.00000,   0.71307,   1.35417,
     +    0.03154,   0.00749,  21.15556,   1.54117,   0.29027,
     +    0.00000,   0.91781,   1.54558,
     +    0.03246,   0.00790,  27.67900,   1.74829,   0.52029,
     +    0.00000,   1.13641,   1.75265,
     +    0.03330,   0.00830,  34.50014,   1.96745,   0.75956,
     +    0.00000,   1.36565,   1.97178,
     +    0.03410,   0.00868,  41.56969,   2.19628,   1.00629,
     +    0.05451,   1.60346,   2.20059,
     +    0.03485,   0.00904,  48.80835,   2.43308,   1.25928,
     +    0.29431,   1.84837,   2.43736,
     +    0.03558,   0.00940,  56.17104,   2.67753,   1.51854,
     +    0.58199,   2.10025,   2.68179 /
        data ((f(i,5,k),i=1,5),f(7,5,k),f(6,5,k),f(8,5,k),k=8,14)/
     +    0.03626,   0.00974,  63.60899,   2.92847,   1.78313,
     +    0.86870,   2.35804,   2.93273,
     +    0.03692,   0.01008,  71.08234,   3.18464,   2.05191,
     +    1.15891,   2.62055,   3.18888,
     +    0.03755,   0.01039,  78.56602,   3.44508,   2.32410,
     +    1.45101,   2.88688,   3.44931,
     +    0.03817,   0.01072,  86.04810,   3.70914,   2.59913,
     +    1.74479,   3.15645,   3.71337,
     +    0.03875,   0.01102,  93.49756,   3.97587,   2.87615,
     +    2.03942,   3.42836,   3.98010,
     +    0.03932,   0.01132, 100.82874,   4.24340,   3.15337,
     +    2.33317,   3.70076,   4.24764,
     +    0.03987,   0.01161, 108.13600,   4.51309,   3.43219,
     +    2.62757,   3.97504,   4.51732 /
        data ((f(i,5,k),i=1,5),f(7,5,k),f(6,5,k),f(8,5,k),k=15,19)/
     +    0.04040,   0.01189, 115.38472,   4.78414,   3.71189,
     +    2.92197,   4.25042,   4.78836,
     +    0.04092,   0.01217, 122.47620,   5.05405,   3.99002,
     +    3.21392,   4.52447,   5.05827,
     +    0.04141,   0.01244, 129.56682,   5.32595,   4.26973,
     +    3.50673,   4.80029,   5.33019,
     +    0.04190,   0.01270, 136.46161,   5.59553,   4.54677,
     +    3.79609,   5.07362,   5.59977,
     +    0.04239,   0.01297, 143.34029,   5.86647,   4.82482,
     +    4.08586,   5.34813,   5.87072 /
        data ((f(i,6,k),i=1,5),f(7,6,k),f(6,6,k),f(8,6,k),k=1,7)/
     +    0.03292,   0.00793,  14.14132,   1.26262,   0.06697,
     +    0.00000,   0.66628,   1.26634,
     +    0.03400,   0.00843,  19.68086,   1.43912,   0.26826,
     +    0.00000,   0.85518,   1.44282,
     +    0.03500,   0.00890,  25.64561,   1.62915,   0.47969,
     +    0.00000,   1.05592,   1.63285,
     +    0.03593,   0.00934,  31.84277,   1.82937,   0.69869,
     +    0.00000,   1.26554,   1.83306,
     +    0.03679,   0.00977,  38.23102,   2.03768,   0.92373,
     +    0.04969,   1.48221,   2.04136,
     +    0.03760,   0.01018,  44.74008,   2.25249,   1.15367,
     +    0.26791,   1.70461,   2.25618,
     +    0.03839,   0.01058,  51.33093,   2.47356,   1.38857,
     +    0.52899,   1.93261,   2.47725 /
        data ((f(i,6,k),i=1,5),f(7,6,k),f(6,6,k),f(8,6,k),k=8,14)/
     +    0.03914,   0.01097,  57.96184,   2.69985,   1.62760,
     +    0.78837,   2.16528,   2.70355,
     +    0.03986,   0.01134,  64.59860,   2.93022,   1.86976,
     +    1.05019,   2.40156,   2.93392,
     +    0.04055,   0.01170,  71.22144,   3.16385,   2.11435,
     +    1.31302,   2.64067,   3.16756,
     +    0.04120,   0.01206,  77.82121,   3.40016,   2.36090,
     +    1.57670,   2.88212,   3.40388,
     +    0.04184,   0.01239,  84.37195,   3.63832,   2.60867,
     +    1.84053,   3.12509,   3.64206,
     +    0.04245,   0.01272,  90.79819,   3.87666,   2.85606,
     +    2.10294,   3.36796,   3.88041,
     +    0.04305,   0.01305,  97.18675,   4.11645,   3.10437,
     +    2.36541,   3.61203,   4.12021 /
        data ((f(i,6,k),i=1,5),f(7,6,k),f(6,6,k),f(8,6,k),k=15,19)/
     +    0.04362,   0.01337, 103.50786,   4.35699,   3.35299,
     +    2.62735,   3.85662,   4.36077,
     +    0.04418,   0.01368, 109.67487,   4.59604,   3.59971,
     +    2.88658,   4.09952,   4.59983,
     +    0.04473,   0.01397, 115.82816,   4.83648,   3.84742,
     +    3.14614,   4.34359,   4.84027,
     +    0.04525,   0.01427, 121.79561,   5.07441,   4.09230,
     +    3.40214,   4.58501,   5.07821,
     +    0.04576,   0.01456, 127.73788,   5.31318,   4.33771,
     +    3.65810,   4.82710,   5.31700 /
        data ((f(i,7,k),i=1,5),f(7,7,k),f(6,7,k),f(8,7,k),k=1,7)/
     +    0.04155,   0.01147,  11.43973,   1.02530,   0.05266,
     +    0.00000,   0.53956,   1.02817,
     +    0.04294,   0.01218,  15.71670,   1.16188,   0.20932,
     +    0.00000,   0.68616,   1.16481,
     +    0.04422,   0.01287,  20.21892,   1.30675,   0.37153,
     +    0.00000,   0.83970,   1.30975,
     +    0.04541,   0.01351,  24.80116,   1.45733,   0.53732,
     +    0.00000,   0.99787,   1.46039,
     +    0.04652,   0.01411,  29.44316,   1.61220,   0.70572,
     +    0.03716,   1.15949,   1.61531,
     +    0.04758,   0.01471,  34.09891,   1.77021,   0.87597,
     +    0.19950,   1.32362,   1.77338,
     +    0.04858,   0.01527,  38.74529,   1.93122,   1.04817,
     +    0.39205,   1.49020,   1.93444 /
        data ((f(i,7,k),i=1,5),f(7,7,k),f(6,7,k),f(8,7,k),k=8,14)/
     +    0.04954,   0.01582,  43.35803,   2.09451,   1.22177,
     +    0.58144,   1.65864,   2.09777,
     +    0.05045,   0.01635,  47.91843,   2.25933,   1.39610,
     +    0.77090,   1.82820,   2.26264,
     +    0.05133,   0.01687,  52.41753,   2.42513,   1.57078,
     +    0.95950,   1.99845,   2.42850,
     +    0.05218,   0.01736,  56.85410,   2.59160,   1.74553,
     +    1.14723,   2.16904,   2.59501,
     +    0.05299,   0.01784,  61.21371,   2.75817,   1.91985,
     +    1.33365,   2.33948,   2.76162,
     +    0.05378,   0.01831,  65.44678,   2.92364,   2.09264,
     +    1.51769,   2.50860,   2.92714,
     +    0.05454,   0.01876,  69.61964,   3.08911,   2.26500,
     +    1.70058,   2.67750,   3.09264 /
        data ((f(i,7,k),i=1,5),f(7,7,k),f(6,7,k),f(8,7,k),k=15,19)/
     +    0.05527,   0.01920,  73.71400,   3.25409,   2.43649,
     +    1.88195,   2.84573,   3.25766,
     +    0.05598,   0.01964,  77.67226,   3.41700,   2.60559,
     +    2.06026,   3.01172,   3.42061,
     +    0.05668,   0.02005,  81.59584,   3.58002,   2.77450,
     +    2.23785,   3.17768,   3.58367,
     +    0.05735,   0.02046,  85.36723,   3.74035,   2.94044,
     +    2.41191,   3.34080,   3.74404,
     +    0.05801,   0.02087,  89.10036,   3.90050,   3.10595,
     +    2.58510,   3.50363,   3.90423 /
        data ((f(i,8,k),i=1,5),f(7,8,k),f(6,8,k),f(8,8,k),k=1,7)/
     +    0.05244,   0.01662,   9.25090,   0.83129,   0.04108,
     +    0.00000,   0.43682,   0.83477,
     +    0.05421,   0.01766,  12.53508,   0.93613,   0.16225,
     +    0.00000,   0.54983,   0.93974,
     +    0.05583,   0.01863,  15.90942,   1.04568,   0.28587,
     +    0.00000,   0.66642,   1.04938,
     +    0.05735,   0.01955,  19.26860,   1.15795,   0.41050,
     +    0.00000,   0.78486,   1.16175,
     +    0.05876,   0.02041,  22.60866,   1.27205,   0.53561,
     +    0.02762,   0.90447,   1.27594,
     +    0.06010,   0.02124,  25.90247,   1.38718,   0.66069,
     +    0.14764,   1.02458,   1.39118,
     +    0.06138,   0.02206,  29.13887,   1.50330,   0.78589,
     +    0.28871,   1.14523,   1.50737 /
        data ((f(i,8,k),i=1,5),f(7,8,k),f(6,8,k),f(8,8,k),k=8,14)/
     +    0.06261,   0.02282,  32.30612,   1.61994,   0.91089,
     +    0.42601,   1.26604,   1.62408,
     +    0.06378,   0.02357,  35.39610,   1.73659,   1.03531,
     +    0.56207,   1.38658,   1.74082,
     +    0.06490,   0.02429,  38.40730,   1.85300,   1.15890,
     +    0.69633,   1.50657,   1.85729,
     +    0.06598,   0.02499,  41.34314,   1.96893,   1.28157,
     +    0.82886,   1.62588,   1.97329,
     +    0.06700,   0.02567,  44.19688,   2.08407,   1.40301,
     +    0.95943,   1.74416,   2.08850,
     +    0.06801,   0.02632,  46.93665,   2.19758,   1.52245,
     +    1.08731,   1.86063,   2.20206,
     +    0.06897,   0.02695,  49.61301,   2.31035,   1.64083,
     +    1.21354,   1.97620,   2.31489 /
        data ((f(i,8,k),i=1,5),f(7,8,k),f(6,8,k),f(8,8,k),k=15,19)/
     +    0.06991,   0.02756,  52.21521,   2.42209,   1.75786,
     +    1.33788,   2.09057,   2.42668,
     +    0.07081,   0.02816,  54.70546,   2.53165,   1.87246,
     +    1.45927,   2.20265,   2.53630,
     +    0.07169,   0.02874,  57.15674,   2.64074,   1.98632,
     +    1.57950,   2.31412,   2.64543,
     +    0.07255,   0.02931,  59.48952,   2.74729,   2.09742,
     +    1.69655,   2.42294,   2.75202,
     +    0.07337,   0.02985,  61.78384,   2.85322,   2.20772,
     +    1.81243,   2.53105,   2.85801 /
        data ((f(i,9,k),i=1,5),f(7,9,k),f(6,9,k),f(8,9,k),k=1,7)/
     +    0.06010,   0.02068,   8.16771,   0.73475,   0.03533,
     +    0.00000,   0.38595,   0.73899,
     +    0.06215,   0.02195,  10.97002,   0.82411,   0.13917,
     +    0.00000,   0.48259,   0.82850,
     +    0.06401,   0.02315,  13.80556,   0.91663,   0.24418,
     +    0.00000,   0.58137,   0.92113,
     +    0.06575,   0.02427,  16.58923,   1.01064,   0.34917,
     +    0.00000,   0.68089,   1.01526,
     +    0.06736,   0.02534,  19.32477,   1.10552,   0.45380,
     +    0.02313,   0.78066,   1.11023,
     +    0.06891,   0.02637,  21.99377,   1.20060,   0.55771,
     +    0.12326,   0.88017,   1.20542,
     +    0.07039,   0.02735,  24.59062,   1.29587,   0.66105,
     +    0.24031,   0.97949,   1.30079 /
        data ((f(i,9,k),i=1,5),f(7,9,k),f(6,9,k),f(8,9,k),k=8,14)/
     +    0.07179,   0.02830,  27.10899,   1.39101,   0.76363,
     +    0.35349,   1.07836,   1.39601,
     +    0.07313,   0.02921,  29.54529,   1.48566,   0.86514,
     +    0.46499,   1.17645,   1.49073,
     +    0.07442,   0.03008,  31.90111,   1.57960,   0.96547,
     +    0.57442,   1.27358,   1.58475,
     +    0.07566,   0.03093,  34.18138,   1.67272,   1.06457,
     +    0.68189,   1.36970,   1.67794,
     +    0.07685,   0.03175,  36.38255,   1.76477,   1.16222,
     +    0.78726,   1.46455,   1.77004,
     +    0.07800,   0.03254,  38.48053,   1.85508,   1.25780,
     +    0.88994,   1.55750,   1.86042,
     +    0.07911,   0.03329,  40.51802,   1.94443,   1.35214,
     +    0.99087,   1.64936,   1.94984 /
        data ((f(i,9,k),i=1,5),f(7,9,k),f(6,9,k),f(8,9,k),k=15,19)/
     +    0.08019,   0.03403,  42.48756,   2.03263,   1.44504,
     +    1.08989,   1.73990,   2.03808,
     +    0.08122,   0.03475,  44.35997,   2.11873,   1.53561,
     +    1.18613,   1.82825,   2.12426,
     +    0.08223,   0.03545,  46.19490,   2.20419,   1.62532,
     +    1.28114,   1.91584,   2.20976,
     +    0.08321,   0.03613,  47.92944,   2.28731,   1.71250,
     +    1.37323,   2.00098,   2.29293,
     +    0.08415,   0.03680,  49.62872,   2.36971,   1.79876,
     +    1.46414,   2.08531,   2.37537 /
        data ((f(i,10,k),i=1,5),f(7,10,k),f(6,10,k),f(8,10,k),k=1,7)/
     +    0.06624,   0.02414,   7.47539,   0.67282,   0.03164,
     +    0.00000,   0.35348,   0.67779,
     +    0.06847,   0.02562,   9.97223,   0.75245,   0.12455,
     +    0.00000,   0.43978,   0.75754,
     +    0.07053,   0.02702,  12.46965,   0.83431,   0.21790,
     +    0.00000,   0.52743,   0.83953,
     +    0.07245,   0.02832,  14.89576,   0.91699,   0.31065,
     +    0.00000,   0.61516,   0.92231,
     +    0.07424,   0.02955,  17.25882,   0.99998,   0.40261,
     +    0.02033,   0.70266,   1.00540,
     +    0.07594,   0.03074,  19.54593,   1.08273,   0.49346,
     +    0.10814,   0.78948,   1.08824,
     +    0.07758,   0.03188,  21.75473,   1.16525,   0.58341,
     +    0.21035,   0.87574,   1.17084 /
        data ((f(i,10,k),i=1,5),f(7,10,k),f(6,10,k),f(8,10,k),k=8,14)/
     +    0.07913,   0.03297,  23.88213,   1.24729,   0.67228,
     +    0.30870,   0.96120,   1.25296,
     +    0.08061,   0.03401,  25.92714,   1.32856,   0.75988,
     +    0.40519,   1.04565,   1.33430,
     +    0.08203,   0.03501,  27.89279,   1.40891,   0.84611,
     +    0.49949,   1.12895,   1.41473,
     +    0.08340,   0.03598,  29.78505,   1.48828,   0.93096,
     +    0.59175,   1.21108,   1.49416,
     +    0.08471,   0.03691,  31.60204,   1.56645,   1.01430,
     +    0.68189,   1.29184,   1.57239,
     +    0.08598,   0.03780,  33.32407,   1.64286,   1.09557,
     +    0.76940,   1.37070,   1.64887,
     +    0.08720,   0.03868,  34.98917,   1.71825,   1.17554,
     +    0.85516,   1.44840,   1.72431 /
        data ((f(i,10,k),i=1,5),f(7,10,k),f(6,10,k),f(8,10,k),k=15,19)/
     +    0.08839,   0.03952,  36.59158,   1.79243,   1.25405,
     +    0.93903,   1.52475,   1.79855,
     +    0.08954,   0.04034,  38.10712,   1.86463,   1.33036,
     +    1.02028,   1.59900,   1.87079,
     +    0.09064,   0.04113,  39.58728,   1.93610,   1.40575,
     +    1.10029,   1.67244,   1.94231,
     +    0.09172,   0.04189,  40.97926,   2.00539,   1.47878,
     +    1.17760,   1.74361,   2.01165,
     +    0.09277,   0.04264,  42.33871,   2.07394,   1.55089,
     +    1.25374,   1.81395,   2.08024 /
        data ((f(i,11,k),i=1,5),f(7,11,k),f(6,11,k),f(8,11,k),k=1,7)/
     +    0.07148,   0.02735,   6.97721,   0.62867,   0.02893,
     +    0.00000,   0.33006,   0.63388,
     +    0.07391,   0.02902,   9.25154,   0.70124,   0.11395,
     +    0.00000,   0.40888,   0.70660,
     +    0.07615,   0.03059,  11.50403,   0.77545,   0.19890,
     +    0.00000,   0.48849,   0.78094,
     +    0.07822,   0.03204,  13.67303,   0.85001,   0.28289,
     +    0.00002,   0.56780,   0.85563,
     +    0.08017,   0.03342,  15.77015,   0.92449,   0.36577,
     +    0.01834,   0.64650,   0.93022,
     +    0.08202,   0.03473,  17.78607,   0.99845,   0.44732,
     +    0.09732,   0.72428,   1.00429,
     +    0.08378,   0.03600,  19.72060,   1.07192,   0.52773,
     +    0.18896,   0.80122,   1.07783 /
        data ((f(i,11,k),i=1,5),f(7,11,k),f(6,11,k),f(8,11,k),k=8,14)/
     +    0.08547,   0.03720,  21.57309,   1.14469,   0.60690,
     +    0.27677,   0.87721,   1.15070,
     +    0.08708,   0.03836,  23.34392,   1.21652,   0.68465,
     +    0.36262,   0.95201,   1.22261,
     +    0.08861,   0.03947,  25.03760,   1.28729,   0.76095,
     +    0.44622,   1.02557,   1.29346,
     +    0.09009,   0.04054,  26.66037,   1.35700,   0.83580,
     +    0.52777,   1.09786,   1.36324,
     +    0.09151,   0.04156,  28.21049,   1.42545,   0.90909,
     +    0.60720,   1.16874,   1.43176,
     +    0.09289,   0.04255,  29.67432,   1.49217,   0.98035,
     +    0.68409,   1.23774,   1.49854,
     +    0.09421,   0.04351,  31.08479,   1.55783,   1.05030,
     +    0.75924,   1.30555,   1.56426 /
        data ((f(i,11,k),i=1,5),f(7,11,k),f(6,11,k),f(8,11,k),k=15,19)/
     +    0.09549,   0.04444,  32.43724,   1.62229,   1.11882,
     +    0.83256,   1.37205,   1.62877,
     +    0.09673,   0.04533,  33.70975,   1.68483,   1.18523,
     +    0.90340,   1.43653,   1.69137,
     +    0.09793,   0.04620,  34.94885,   1.74664,   1.25070,
     +    0.97302,   1.50018,   1.75323,
     +    0.09909,   0.04704,  36.10852,   1.80639,   1.31395,
     +    1.04010,   1.56169,   1.81303,
     +    0.10022,   0.04786,  37.23849,   1.86539,   1.37629,
     +    1.10603,   1.62236,   1.87207 /
        data ((f(i,12,k),i=1,5),f(7,12,k),f(6,12,k),f(8,12,k),k=1,7)/
     +    0.09073,   0.04042,   5.62636,   0.50845,   0.02147,
     +    0.00000,   0.26664,   0.51491,
     +    0.09386,   0.04279,   7.30762,   0.56213,   0.08530,
     +    0.00000,   0.32544,   0.56877,
     +    0.09674,   0.04498,   8.91820,   0.61605,   0.14799,
     +    0.00000,   0.38379,   0.62283,
     +    0.09943,   0.04702,  10.42288,   0.66934,   0.20897,
     +    0.00001,   0.44095,   0.67625,
     +    0.10194,   0.04893,  11.84112,   0.72179,   0.26828,
     +    0.01314,   0.49687,   0.72882,
     +    0.10432,   0.05074,  13.17321,   0.77315,   0.32585,
     +    0.06936,   0.55135,   0.78028,
     +    0.10659,   0.05247,  14.42432,   0.82347,   0.38187,
     +    0.13384,   0.60455,   0.83069 /
        data ((f(i,12,k),i=1,5),f(7,12,k),f(6,12,k),f(8,12,k),k=8,14)/
     +    0.10875,   0.05410,  15.59858,   0.87272,   0.43636,
     +    0.19482,   0.65644,   0.88002,
     +    0.11082,   0.05566,  16.70023,   0.92077,   0.48926,
     +    0.25373,   0.70694,   0.92816,
     +    0.11280,   0.05715,  17.73563,   0.96761,   0.54062,
     +    0.31046,   0.75605,   0.97507,
     +    0.11469,   0.05858,  18.71164,   1.01326,   0.59050,
     +    0.36522,   0.80384,   1.02079,
     +    0.11650,   0.05994,  19.62926,   1.05765,   0.63887,
     +    0.41804,   0.85023,   1.06525,
     +    0.11825,   0.06125,  20.48106,   1.10048,   0.68543,
     +    0.46864,   0.89493,   1.10812,
     +    0.11994,   0.06250,  21.29117,   1.14226,   0.73075,
     +    0.51769,   0.93851,   1.14998 /
        data ((f(i,12,k),i=1,5),f(7,12,k),f(6,12,k),f(8,12,k),k=15,19)/
     +    0.12155,   0.06371,  22.05750,   1.18294,   0.77476,
     +    0.56513,   0.98086,   1.19070,
     +    0.12312,   0.06487,  22.76675,   1.22204,   0.81705,
     +    0.61056,   1.02156,   1.22985,
     +    0.12464,   0.06600,  23.45059,   1.26041,   0.85845,
     +    0.65489,   1.06146,   1.26826,
     +    0.12611,   0.06708,  24.07961,   1.29717,   0.89809,
     +    0.69723,   1.09967,   1.30507,
     +    0.12753,   0.06813,  24.68698,   1.33324,   0.93692,
     +    0.73859,   1.13713,   1.34117 /
        data ((f(i,13,k),i=1,5),f(7,13,k),f(6,13,k),f(8,13,k),k=1,7)/
     +    0.11612,   0.05999,   4.52734,   0.41093,   0.01511,
     +    0.00002,   0.21501,   0.41883,
     +    0.12021,   0.06325,   5.73271,   0.44951,   0.06199,
     +    0.00001,   0.25778,   0.45759,
     +    0.12400,   0.06625,   6.84270,   0.48755,   0.10722,
     +    0.00002,   0.29946,   0.49577,
     +    0.12750,   0.06900,   7.84306,   0.52442,   0.15040,
     +    0.00002,   0.33952,   0.53278,
     +    0.13076,   0.07156,   8.75681,   0.56014,   0.19174,
     +    0.00920,   0.37808,   0.56858,
     +    0.13384,   0.07397,   9.59102,   0.59451,   0.23122,
     +    0.04814,   0.41504,   0.60305,
     +    0.13679,   0.07622,  10.35507,   0.62768,   0.26910,
     +    0.09226,   0.45058,   0.63631 /
        data ((f(i,13,k),i=1,5),f(7,13,k),f(6,13,k),f(8,13,k),k=8,14)/
     +    0.13957,   0.07834,  11.05375,   0.65970,   0.30544,
     +    0.13337,   0.48477,   0.66841,
     +    0.14222,   0.08035,  11.69351,   0.69051,   0.34025,
     +    0.17253,   0.51760,   0.69926,
     +    0.14474,   0.08225,  12.28241,   0.72013,   0.37364,
     +    0.20976,   0.54912,   0.72894,
     +    0.14715,   0.08406,  12.82666,   0.74866,   0.40568,
     +    0.24529,   0.57943,   0.75753,
     +    0.14945,   0.08578,  13.32714,   0.77610,   0.43642,
     +    0.27918,   0.60853,   0.78501,
     +    0.15165,   0.08741,  13.78036,   0.80221,   0.46564,
     +    0.31125,   0.63620,   0.81116,
     +    0.15378,   0.08897,  14.20426,   0.82743,   0.49379,
     +    0.34202,   0.66290,   0.83643 /
        data ((f(i,13,k),i=1,5),f(7,13,k),f(6,13,k),f(8,13,k),k=15,19)/
     +    0.15582,   0.09047,  14.59854,   0.85172,   0.52089,
     +    0.37151,   0.68861,   0.86077,
     +    0.15777,   0.09188,  14.95536,   0.87485,   0.54668,
     +    0.39948,   0.71307,   0.88391,
     +    0.15966,   0.09324,  15.29492,   0.89736,   0.57172,
     +    0.42656,   0.73685,   0.90646,
     +    0.16148,   0.09456,  15.59800,   0.91864,   0.59542,
     +    0.45213,   0.75934,   0.92777,
     +    0.16324,   0.09583,  15.88724,   0.93937,   0.61846,
     +    0.47693,   0.78125,   0.94853 /
        data ((f(i,14,k),i=1,5),f(7,14,k),f(6,14,k),f(8,14,k),k=1,7)/
     +    0.13480,   0.07546,   3.98165,   0.36243,   0.01180,
     +    0.00002,   0.18930,   0.37118,
     +    0.13962,   0.07935,   4.95415,   0.39371,   0.05048,
     +    0.00003,   0.22434,   0.40262,
     +    0.14405,   0.08290,   5.82556,   0.42416,   0.08736,
     +    0.00003,   0.25805,   0.43319,
     +    0.14816,   0.08611,   6.59158,   0.45332,   0.12219,
     +    0.00003,   0.29007,   0.46246,
     +    0.15199,   0.08906,   7.27642,   0.48123,   0.15518,
     +    0.00735,   0.32055,   0.49046,
     +    0.15559,   0.09181,   7.89017,   0.50779,   0.18635,
     +    0.03830,   0.34945,   0.51711,
     +    0.15899,   0.09438,   8.44100,   0.53318,   0.21598,
     +    0.07307,   0.37698,   0.54255 /
        data ((f(i,14,k),i=1,5),f(7,14,k),f(6,14,k),f(8,14,k),k=8,14)/
     +    0.16221,   0.09679,   8.93561,   0.55739,   0.24411,
     +    0.10512,   0.40316,   0.56683,
     +    0.16525,   0.09904,   9.38150,   0.58047,   0.27084,
     +    0.13539,   0.42807,   0.58997,
     +    0.16815,   0.10115,   9.78434,   0.60251,   0.29627,
     +    0.16395,   0.45183,   0.61204,
     +    0.17092,   0.10315,  10.15014,   0.62354,   0.32051,
     +    0.19100,   0.47448,   0.63312,
     +    0.17354,   0.10505,  10.48254,   0.64358,   0.34355,
     +    0.21658,   0.49603,   0.65319,
     +    0.17604,   0.10683,  10.77755,   0.66252,   0.36532,
     +    0.24065,   0.51639,   0.67216,
     +    0.17843,   0.10853,  11.04940,   0.68067,   0.38616,
     +    0.26358,   0.53589,   0.69034 /
        data ((f(i,14,k),i=1,5),f(7,14,k),f(6,14,k),f(8,14,k),k=15,19)/
     +    0.18072,   0.11016,  11.29810,   0.69804,   0.40607,
     +    0.28540,   0.55454,   0.70771,
     +    0.18292,   0.11169,  11.51828,   0.71437,   0.42484,
     +    0.30592,   0.57210,   0.72408,
     +    0.18504,   0.11317,  11.72573,   0.73020,   0.44299,
     +    0.32569,   0.58909,   0.73992,
     +    0.18708,   0.11455,  11.90742,   0.74503,   0.46004,
     +    0.34424,   0.60504,   0.75478,
     +    0.18905,   0.11590,  12.07889,   0.75942,   0.47655,
     +    0.36216,   0.62048,   0.76917 /
        data ((f(i,15,k),i=1,5),f(7,15,k),f(6,15,k),f(8,15,k),k=1,7)/
     +    0.15029,   0.08868,   3.63159,   0.33129,   0.00955,
     +    0.00002,   0.17277,   0.34067,
     +    0.15572,   0.09304,   4.45618,   0.35796,   0.04311,
     +    0.00002,   0.20293,   0.36749,
     +    0.16071,   0.09698,   5.17942,   0.38370,   0.07481,
     +    0.00003,   0.23168,   0.39331,
     +    0.16532,   0.10049,   5.80246,   0.40808,   0.10448,
     +    0.00003,   0.25873,   0.41781,
     +    0.16960,   0.10372,   6.34982,   0.43121,   0.13235,
     +    0.00621,   0.28426,   0.44102,
     +    0.17362,   0.10670,   6.83238,   0.45303,   0.15850,
     +    0.03227,   0.30828,   0.46291,
     +    0.17737,   0.10946,   7.25862,   0.47370,   0.18312,
     +    0.06136,   0.33094,   0.48360 /
        data ((f(i,15,k),i=1,5),f(7,15,k),f(6,15,k),f(8,15,k),k=8,14)/
     +    0.18092,   0.11204,   7.63543,   0.49326,   0.20637,
     +    0.08796,   0.35234,   0.50320,
     +    0.18427,   0.11442,   7.96998,   0.51174,   0.22828,
     +    0.11291,   0.37255,   0.52173,
     +    0.18745,   0.11666,   8.26775,   0.52927,   0.24902,
     +    0.13631,   0.39168,   0.53928,
     +    0.19046,   0.11876,   8.53430,   0.54588,   0.26865,
     +    0.15834,   0.40982,   0.55592,
     +    0.19332,   0.12074,   8.77298,   0.56160,   0.28720,
     +    0.17905,   0.42696,   0.57166,
     +    0.19603,   0.12258,   8.98090,   0.57635,   0.30461,
     +    0.19841,   0.44305,   0.58642,
     +    0.19861,   0.12434,   9.17000,   0.59041,   0.32122,
     +    0.21678,   0.45837,   0.60048 /
        data ((f(i,15,k),i=1,5),f(7,15,k),f(6,15,k),f(8,15,k),k=15,19)/
     +    0.20108,   0.12602,   9.34047,   0.60376,   0.33697,
     +    0.23416,   0.47293,   0.61383,
     +    0.20344,   0.12760,   9.48812,   0.61623,   0.35176,
     +    0.25042,   0.48656,   0.62632,
     +    0.20572,   0.12911,   9.62581,   0.62826,   0.36599,
     +    0.26602,   0.49969,   0.63835,
     +    0.20789,   0.13052,   9.74323,   0.63944,   0.37927,
     +    0.28058,   0.51193,   0.64954,
     +    0.20999,   0.13188,   9.85289,   0.65023,   0.39209,
     +    0.29459,   0.52374,   0.66034 /
        data ((f(i,16,k),i=1,5),f(7,16,k),f(6,16,k),f(8,16,k),k=1,7)/
     +    0.16352,   0.10032,   3.37937,   0.30848,   0.00790,
     +    0.00002,   0.16083,   0.31828,
     +    0.16954,   0.10506,   4.09838,   0.33198,   0.03789,
     +    0.00004,   0.18760,   0.34191,
     +    0.17509,   0.10927,   4.71769,   0.35442,   0.06601,
     +    0.00004,   0.21292,   0.36448,
     +    0.18013,   0.11305,   5.24194,   0.37553,   0.09213,
     +    0.00003,   0.23653,   0.38562,
     +    0.18480,   0.11647,   5.69572,   0.39538,   0.11650,
     +    0.00545,   0.25865,   0.40554,
     +    0.18916,   0.11960,   6.08940,   0.41395,   0.13920,
     +    0.02810,   0.27928,   0.42415,
     +    0.19324,   0.12247,   6.43224,   0.43142,   0.16047,
     +    0.05329,   0.29867,   0.44166 /
        data ((f(i,16,k),i=1,5),f(7,16,k),f(6,16,k),f(8,16,k),k=8,14)/
     +    0.19707,   0.12514,   6.73166,   0.44780,   0.18039,
     +    0.07616,   0.31682,   0.45808,
     +    0.20067,   0.12760,   6.99285,   0.46321,   0.19907,
     +    0.09751,   0.33388,   0.47350,
     +    0.20405,   0.12990,   7.22282,   0.47771,   0.21664,
     +    0.11741,   0.34990,   0.48799,
     +    0.20727,   0.13205,   7.42642,   0.49135,   0.23319,
     +    0.13604,   0.36498,   0.50165,
     +    0.21033,   0.13405,   7.60415,   0.50417,   0.24875,
     +    0.15348,   0.37919,   0.51450,
     +    0.21319,   0.13594,   7.75704,   0.51611,   0.26326,
     +    0.16968,   0.39241,   0.52642,
     +    0.21592,   0.13772,   7.89445,   0.52745,   0.27703,
     +    0.18501,   0.40497,   0.53774 /
        data ((f(i,16,k),i=1,5),f(7,16,k),f(6,16,k),f(8,16,k),k=15,19)/
     +    0.21854,   0.13938,   8.01655,   0.53817,   0.29009,
     +    0.19948,   0.41686,   0.54848,
     +    0.22102,   0.14094,   8.11928,   0.54810,   0.30224,
     +    0.21292,   0.42788,   0.55840,
     +    0.22342,   0.14243,   8.21397,   0.55762,   0.31388,
     +    0.22578,   0.43847,   0.56792,
     +    0.22569,   0.14384,   8.29227,   0.56642,   0.32472,
     +    0.23772,   0.44828,   0.57670,
     +    0.22787,   0.14517,   8.36434,   0.57488,   0.33513,
     +    0.24917,   0.45771,   0.58514 /
        data ((f(i,17,k),i=1,5),f(7,17,k),f(6,17,k),f(8,17,k),k=1,7)/
     +    0.21657,   0.14561,   2.68652,   0.24625,   0.00301,
     +    0.00001,   0.12777,   0.25730,
     +    0.22475,   0.15119,   3.12191,   0.26105,   0.02355,
     +    0.00002,   0.14543,   0.27214,
     +    0.23206,   0.15600,   3.47119,   0.27473,   0.04231,
     +    0.00000,   0.16162,   0.28581,
     +    0.23858,   0.16017,   3.74667,   0.28716,   0.05927,
     +    0.00002,   0.17630,   0.29820,
     +    0.24448,   0.16382,   3.96980,   0.29849,   0.07471,
     +    0.00345,   0.18965,   0.30949,
     +    0.24989,   0.16704,   4.15146,   0.30875,   0.08872,
     +    0.01754,   0.20177,   0.31970,
     +    0.25484,   0.16990,   4.29869,   0.31804,   0.10152,
     +    0.03288,   0.21279,   0.32897 /
        data ((f(i,17,k),i=1,5),f(7,17,k),f(6,17,k),f(8,17,k),k=8,14)/
     +    0.25940,   0.17247,   4.41846,   0.32651,   0.11326,
     +    0.04651,   0.22287,   0.33740,
     +    0.26362,   0.17480,   4.51544,   0.33427,   0.12405,
     +    0.05900,   0.23212,   0.34510,
     +    0.26753,   0.17691,   4.59424,   0.34134,   0.13397,
     +    0.07041,   0.24060,   0.35210,
     +    0.27118,   0.17885,   4.65814,   0.34781,   0.14314,
     +    0.08088,   0.24839,   0.35851,
     +    0.27460,   0.18061,   4.70790,   0.35373,   0.15160,
     +    0.09055,   0.25557,   0.36440,
     +    0.27777,   0.18220,   4.74467,   0.35906,   0.15935,
     +    0.09936,   0.26207,   0.36965,
     +    0.28076,   0.18365,   4.77407,   0.36398,   0.16656,
     +    0.10755,   0.26812,   0.37453 /
        data ((f(i,17,k),i=1,5),f(7,17,k),f(6,17,k),f(8,17,k),k=15,19)/
     +    0.28359,   0.18500,   4.79632,   0.36848,   0.17327,
     +    0.11516,   0.27372,   0.37901,
     +    0.28621,   0.18622,   4.80952,   0.37255,   0.17942,
     +    0.12214,   0.27881,   0.38300,
     +    0.28870,   0.18739,   4.81959,   0.37638,   0.18523,
     +    0.12874,   0.28361,   0.38676,
     +    0.29102,   0.18844,   4.82134,   0.37976,   0.19051,
     +    0.13475,   0.28792,   0.39010,
     +    0.29325,   0.18943,   4.82118,   0.38294,   0.19552,
     +    0.14045,   0.29200,   0.39324 /
        data ((f(i,18,k),i=1,5),f(7,18,k),f(6,18,k),f(8,18,k),k=1,7)/
     +    0.29475,   0.20433,   2.09615,   0.19190,   0.00001,
     +    0.00012,   0.09825,   0.20318,
     +    0.30508,   0.20942,   2.30651,   0.19956,   0.01189,
     +    0.00014,   0.10874,   0.21073,
     +    0.31388,   0.21346,   2.45504,   0.20630,   0.02372,
     +    0.00009,   0.11796,   0.21726,
     +    0.32137,   0.21670,   2.55577,   0.21203,   0.03400,
     +    0.00010,   0.12588,   0.22279,
     +    0.32791,   0.21931,   2.62536,   0.21694,   0.04302,
     +    0.00184,   0.13278,   0.22753,
     +    0.33360,   0.22141,   2.67192,   0.22103,   0.05095,
     +    0.00979,   0.13874,   0.23148,
     +    0.33867,   0.22308,   2.70094,   0.22454,   0.05796,
     +    0.01828,   0.14396,   0.23481 /
        data ((f(i,18,k),i=1,5),f(7,18,k),f(6,18,k),f(8,18,k),k=8,14)/
     +    0.34318,   0.22445,   2.71617,   0.22747,   0.06418,
     +    0.02557,   0.14851,   0.23759,
     +    0.34720,   0.22557,   2.72121,   0.22993,   0.06974,
     +    0.03208,   0.15249,   0.23995,
     +    0.35078,   0.22647,   2.71849,   0.23203,   0.07470,
     +    0.03789,   0.15598,   0.24191,
     +    0.35401,   0.22722,   2.70995,   0.23382,   0.07916,
     +    0.04311,   0.15905,   0.24354,
     +    0.35694,   0.22781,   2.69663,   0.23531,   0.08320,
     +    0.04784,   0.16176,   0.24491,
     +    0.35953,   0.22823,   2.67878,   0.23646,   0.08675,
     +    0.05202,   0.16407,   0.24591,
     +    0.36190,   0.22858,   2.65897,   0.23739,   0.08998,
     +    0.05581,   0.16613,   0.24674 /
        data ((f(i,18,k),i=1,5),f(7,18,k),f(6,18,k),f(8,18,k),k=15,19)/
     +    0.36407,   0.22884,   2.63718,   0.23811,   0.09290,
     +    0.05923,   0.16793,   0.24741,
     +    0.36600,   0.22898,   2.61265,   0.23864,   0.09549,
     +    0.06236,   0.16945,   0.24787,
     +    0.36781,   0.22906,   2.58826,   0.23910,   0.09789,
     +    0.06528,   0.17084,   0.24823,
     +    0.36939,   0.22908,   2.56176,   0.23934,   0.10000,
     +    0.06783,   0.17200,   0.24833,
     +    0.37087,   0.22905,   2.53572,   0.23952,   0.10196,
     +    0.07022,   0.17305,   0.24837 /
        data ((f(i,19,k),i=1,5),f(7,19,k),f(6,19,k),f(8,19,k),k=1,7)/
     +    0.35617,   0.24127,   1.77708,   0.16036,   0.00001,
     +    0.00000,   0.08124,   0.17114,
     +    0.36684,   0.24466,   1.88087,   0.16450,   0.00655,
     +    0.00001,   0.08835,   0.17499,
     +    0.37554,   0.24696,   1.94096,   0.16790,   0.01526,
     +    0.00005,   0.09434,   0.17806,
     +    0.38259,   0.24840,   1.96970,   0.17044,   0.02262,
     +    0.00003,   0.09919,   0.18041,
     +    0.38844,   0.24925,   1.97969,   0.17243,   0.02889,
     +    0.00132,   0.10323,   0.18207,
     +    0.39329,   0.24964,   1.97654,   0.17385,   0.03423,
     +    0.00664,   0.10655,   0.18323,
     +    0.39733,   0.24973,   1.96431,   0.17479,   0.03891,
     +    0.01216,   0.10933,   0.18407 /
        data ((f(i,19,k),i=1,5),f(7,19,k),f(6,19,k),f(8,19,k),k=8,14)/
     +    0.40074,   0.24955,   1.94539,   0.17543,   0.04292,
     +    0.01685,   0.11157,   0.18453,
     +    0.40353,   0.24924,   1.92203,   0.17585,   0.04636,
     +    0.02097,   0.11345,   0.18472,
     +    0.40590,   0.24878,   1.89570,   0.17604,   0.04938,
     +    0.02456,   0.11499,   0.18468,
     +    0.40797,   0.24816,   1.86760,   0.17599,   0.05205,
     +    0.02772,   0.11626,   0.18446,
     +    0.40973,   0.24742,   1.83844,   0.17574,   0.05440,
     +    0.03057,   0.11731,   0.18409,
     +    0.41108,   0.24666,   1.80774,   0.17538,   0.05642,
     +    0.03303,   0.11807,   0.18357,
     +    0.41222,   0.24584,   1.77732,   0.17492,   0.05824,
     +    0.03524,   0.11869,   0.18297 /
        data ((f(i,19,k),i=1,5),f(7,19,k),f(6,19,k),f(8,19,k),k=15,19)/
     +    0.41321,   0.24494,   1.74710,   0.17436,   0.05987,
     +    0.03722,   0.11917,   0.18230,
     +    0.41396,   0.24396,   1.71651,   0.17370,   0.06122,
     +    0.03895,   0.11950,   0.18152,
     +    0.41461,   0.24300,   1.68711,   0.17302,   0.06244,
     +    0.04053,   0.11975,   0.18072,
     +    0.41504,   0.24200,   1.65738,   0.17225,   0.06347,
     +    0.04189,   0.11985,   0.17981,
     +    0.41539,   0.24101,   1.62891,   0.17147,   0.06443,
     +    0.04314,   0.11991,   0.17891 /
        data ((f(i,20,k),i=1,5),f(7,20,k),f(6,20,k),f(8,20,k),k=1,7)/
     +    0.40731,   0.26516,   1.55169,   0.13673,   0.00001,
     +    0.00008,   0.06860,   0.14665,
     +    0.41708,   0.26630,   1.59176,   0.13869,   0.00350,
     +    0.00006,   0.07363,   0.14818,
     +    0.42450,   0.26648,   1.60210,   0.14000,   0.01038,
     +    0.00005,   0.07764,   0.14908,
     +    0.43018,   0.26579,   1.59256,   0.14067,   0.01602,
     +    0.00009,   0.08071,   0.14955,
     +    0.43449,   0.26486,   1.57257,   0.14097,   0.02069,
     +    0.00103,   0.08314,   0.14959,
     +    0.43765,   0.26354,   1.54594,   0.14093,   0.02457,
     +    0.00483,   0.08501,   0.14929,
     +    0.44000,   0.26203,   1.51525,   0.14062,   0.02785,
     +    0.00876,   0.08641,   0.14871 /
        data ((f(i,20,k),i=1,5),f(7,20,k),f(6,20,k),f(8,20,k),k=8,14)/
     +    0.44171,   0.26037,   1.48214,   0.14016,   0.03064,
     +    0.01200,   0.08749,   0.14790,
     +    0.44295,   0.25858,   1.44787,   0.13944,   0.03301,
     +    0.01480,   0.08828,   0.14702,
     +    0.44380,   0.25672,   1.41340,   0.13858,   0.03503,
     +    0.01719,   0.08881,   0.14603,
     +    0.44431,   0.25485,   1.37933,   0.13764,   0.03678,
     +    0.01926,   0.08914,   0.14496,
     +    0.44452,   0.25297,   1.34583,   0.13660,   0.03831,
     +    0.02106,   0.08937,   0.14380,
     +    0.44438,   0.25107,   1.31263,   0.13557,   0.03954,
     +    0.02266,   0.08942,   0.14258,
     +    0.44406,   0.24922,   1.28073,   0.13453,   0.04061,
     +    0.02406,   0.08939,   0.14133 /
        data ((f(i,20,k),i=1,5),f(7,20,k),f(6,20,k),f(8,20,k),k=15,19)/
     +    0.44362,   0.24737,   1.24994,   0.13348,   0.04153,
     +    0.02527,   0.08927,   0.14004,
     +    0.44301,   0.24547,   1.21982,   0.13229,   0.04231,
     +    0.02634,   0.08904,   0.13884,
     +    0.44234,   0.24362,   1.19131,   0.13113,   0.04300,
     +    0.02732,   0.08879,   0.13767,
     +    0.44142,   0.24184,   1.16340,   0.13000,   0.04355,
     +    0.02811,   0.08844,   0.13630,
     +    0.44049,   0.24009,   1.13695,   0.12889,   0.04404,
     +    0.02883,   0.08809,   0.13499 /
        data ((f(i,21,k),i=1,5),f(7,21,k),f(6,21,k),f(8,21,k),k=1,7)/
     +    0.45065,   0.28029,   1.37264,   0.11722,   0.00001,
     +    0.00008,   0.05790,   0.12589,
     +    0.45864,   0.27879,   1.37121,   0.11760,   0.00136,
     +    0.00007,   0.06146,   0.12598,
     +    0.46405,   0.27661,   1.35088,   0.11760,   0.00694,
     +    0.00007,   0.06415,   0.12555,
     +    0.46745,   0.27411,   1.31920,   0.11713,   0.01138,
     +    0.00011,   0.06605,   0.12479,
     +    0.46955,   0.27137,   1.28289,   0.11649,   0.01504,
     +    0.00078,   0.06745,   0.12370,
     +    0.47069,   0.26833,   1.24451,   0.11555,   0.01800,
     +    0.00362,   0.06838,   0.12254,
     +    0.47104,   0.26525,   1.20543,   0.11440,   0.02042,
     +    0.00644,   0.06898,   0.12127 /
        data ((f(i,21,k),i=1,5),f(7,21,k),f(6,21,k),f(8,21,k),k=8,14)/
     +    0.47085,   0.26217,   1.16658,   0.11318,   0.02246,
     +    0.00878,   0.06937,   0.11987,
     +    0.47018,   0.25911,   1.12864,   0.11195,   0.02414,
     +    0.01076,   0.06953,   0.11839,
     +    0.46915,   0.25612,   1.09201,   0.11067,   0.02552,
     +    0.01242,   0.06955,   0.11691,
     +    0.46785,   0.25321,   1.05697,   0.10936,   0.02666,
     +    0.01384,   0.06947,   0.11544,
     +    0.46627,   0.25043,   1.02352,   0.10808,   0.02764,
     +    0.01513,   0.06927,   0.11385,
     +    0.46453,   0.24759,   0.99127,   0.10674,   0.02847,
     +    0.01618,   0.06896,   0.11231,
     +    0.46274,   0.24481,   0.96083,   0.10537,   0.02918,
     +    0.01708,   0.06859,   0.11086 /
        data ((f(i,21,k),i=1,5),f(7,21,k),f(6,21,k),f(8,21,k),k=15,19)/
     +    0.46092,   0.24206,   0.93196,   0.10397,   0.02976,
     +    0.01784,   0.06817,   0.10952,
     +    0.45885,   0.23947,   0.90429,   0.10267,   0.03022,
     +    0.01852,   0.06770,   0.10802,
     +    0.45677,   0.23700,   0.87835,   0.10144,   0.03061,
     +    0.01915,   0.06723,   0.10655,
     +    0.45459,   0.23449,   0.85343,   0.10014,   0.03092,
     +    0.01967,   0.06672,   0.10518,
     +    0.45246,   0.23209,   0.83000,   0.09890,   0.03119,
     +    0.02013,   0.06620,   0.10384 /
        data ((f(i,22,k),i=1,5),f(7,22,k),f(6,22,k),f(8,22,k),k=1,7)/
     +    0.49430,   0.28888,   1.18740,   0.09657,   0.00001,
     +    0.00004,   0.04705,   0.10414,
     +    0.49866,   0.28440,   1.15255,   0.09585,   0.00001,
     +    0.00006,   0.04952,   0.10290,
     +    0.50065,   0.27954,   1.10937,   0.09489,   0.00436,
     +    0.00008,   0.05112,   0.10140,
     +    0.50086,   0.27457,   1.06249,   0.09356,   0.00787,
     +    0.00010,   0.05217,   0.09981,
     +    0.49985,   0.26976,   1.01611,   0.09213,   0.01055,
     +    0.00061,   0.05278,   0.09826,
     +    0.49798,   0.26496,   0.97144,   0.09066,   0.01274,
     +    0.00259,   0.05310,   0.09649,
     +    0.49544,   0.26037,   0.92871,   0.08918,   0.01451,
     +    0.00457,   0.05313,   0.09462 /
        data ((f(i,22,k),i=1,5),f(7,22,k),f(6,22,k),f(8,22,k),k=8,14)/
     +    0.49267,   0.25580,   0.88821,   0.08760,   0.01587,
     +    0.00617,   0.05300,   0.09290,
     +    0.48956,   0.25140,   0.85014,   0.08599,   0.01704,
     +    0.00751,   0.05277,   0.09117,
     +    0.48617,   0.24725,   0.81454,   0.08446,   0.01800,
     +    0.00865,   0.05241,   0.08944,
     +    0.48264,   0.24330,   0.78133,   0.08298,   0.01876,
     +    0.00960,   0.05198,   0.08774,
     +    0.47901,   0.23951,   0.75025,   0.08153,   0.01938,
     +    0.01036,   0.05155,   0.08608,
     +    0.47539,   0.23574,   0.72100,   0.08009,   0.01986,
     +    0.01102,   0.05104,   0.08446,
     +    0.47175,   0.23215,   0.69387,   0.07870,   0.02024,
     +    0.01161,   0.05050,   0.08292 /
        data ((f(i,22,k),i=1,5),f(7,22,k),f(6,22,k),f(8,22,k),k=15,19)/
     +    0.46802,   0.22874,   0.66857,   0.07737,   0.02053,
     +    0.01212,   0.04995,   0.08148,
     +    0.46432,   0.22542,   0.64465,   0.07601,   0.02076,
     +    0.01250,   0.04941,   0.08002,
     +    0.46072,   0.22226,   0.62242,   0.07472,   0.02098,
     +    0.01282,   0.04888,   0.07860,
     +    0.45707,   0.21916,   0.60137,   0.07346,   0.02113,
     +    0.01311,   0.04827,   0.07726,
     +    0.45352,   0.21621,   0.58173,   0.07226,   0.02125,
     +    0.01336,   0.04768,   0.07596 /
        data ((f(i,23,k),i=1,5),f(7,23,k),f(6,23,k),f(8,23,k),k=1,7)/
     +    0.52775,   0.28932,   1.03032,   0.07908,   0.00001,
     +    0.00012,   0.03823,   0.08528,
     +    0.52784,   0.28186,   0.97512,   0.07771,   0.00001,
     +    0.00016,   0.03979,   0.08353,
     +    0.52556,   0.27483,   0.91944,   0.07616,   0.00270,
     +    0.00014,   0.04073,   0.08172,
     +    0.52203,   0.26787,   0.86550,   0.07450,   0.00542,
     +    0.00013,   0.04120,   0.07970,
     +    0.51775,   0.26122,   0.81547,   0.07282,   0.00758,
     +    0.00051,   0.04136,   0.07765,
     +    0.51272,   0.25505,   0.76943,   0.07117,   0.00922,
     +    0.00195,   0.04122,   0.07569,
     +    0.50733,   0.24916,   0.72705,   0.06947,   0.01043,
     +    0.00335,   0.04097,   0.07383 /
        data ((f(i,23,k),i=1,5),f(7,23,k),f(6,23,k),f(8,23,k),k=8,14)/
     +    0.50185,   0.24349,   0.68803,   0.06785,   0.01145,
     +    0.00447,   0.04064,   0.07193,
     +    0.49626,   0.23822,   0.65228,   0.06622,   0.01226,
     +    0.00539,   0.04014,   0.07018,
     +    0.49066,   0.23320,   0.61950,   0.06464,   0.01288,
     +    0.00616,   0.03967,   0.06846,
     +    0.48510,   0.22838,   0.58943,   0.06315,   0.01336,
     +    0.00680,   0.03927,   0.06676,
     +    0.47959,   0.22378,   0.56185,   0.06174,   0.01375,
     +    0.00732,   0.03873,   0.06519,
     +    0.47406,   0.21946,   0.53623,   0.06034,   0.01406,
     +    0.00773,   0.03812,   0.06371,
     +    0.46868,   0.21536,   0.51271,   0.05904,   0.01431,
     +    0.00809,   0.03750,   0.06228 /
        data ((f(i,23,k),i=1,5),f(7,23,k),f(6,23,k),f(8,23,k),k=15,19)/
     +    0.46345,   0.21144,   0.49099,   0.05783,   0.01452,
     +    0.00841,   0.03688,   0.06085,
     +    0.45831,   0.20762,   0.47080,   0.05658,   0.01464,
     +    0.00869,   0.03631,   0.05954,
     +    0.45334,   0.20401,   0.45219,   0.05537,   0.01473,
     +    0.00893,   0.03576,   0.05830,
     +    0.44839,   0.20055,   0.43474,   0.05426,   0.01475,
     +    0.00906,   0.03520,   0.05708,
     +    0.44363,   0.19724,   0.41857,   0.05318,   0.01476,
     +    0.00918,   0.03466,   0.05594 /
        data ((f(i,24,k),i=1,5),f(7,24,k),f(6,24,k),f(8,24,k),k=1,7)/
     +    0.55155,   0.28393,   0.89347,   0.06418,   0.00001,
     +    0.00004,   0.03060,   0.06934,
     +    0.54675,   0.27390,   0.82658,   0.06247,   0.00001,
     +    0.00001,   0.03167,   0.06735,
     +    0.54026,   0.26460,   0.76486,   0.06075,   0.00167,
     +    0.00002,   0.03222,   0.06515,
     +    0.53281,   0.25604,   0.70869,   0.05902,   0.00384,
     +    0.00001,   0.03238,   0.06298,
     +    0.52514,   0.24797,   0.65873,   0.05718,   0.00552,
     +    0.00035,   0.03224,   0.06104,
     +    0.51712,   0.24061,   0.61410,   0.05548,   0.00670,
     +    0.00139,   0.03204,   0.05906,
     +    0.50906,   0.23377,   0.57406,   0.05373,   0.00769,
     +    0.00243,   0.03170,   0.05722 /
        data ((f(i,24,k),i=1,5),f(7,24,k),f(6,24,k),f(8,24,k),k=8,14)/
     +    0.50111,   0.22729,   0.53806,   0.05208,   0.00844,
     +    0.00324,   0.03117,   0.05558,
     +    0.49325,   0.22126,   0.50557,   0.05063,   0.00902,
     +    0.00397,   0.03062,   0.05387,
     +    0.48554,   0.21562,   0.47630,   0.04920,   0.00946,
     +    0.00453,   0.03010,   0.05227,
     +    0.47808,   0.21029,   0.44993,   0.04777,   0.00978,
     +    0.00493,   0.02960,   0.05082,
     +    0.47092,   0.20519,   0.42605,   0.04650,   0.00998,
     +    0.00528,   0.02905,   0.04944,
     +    0.46381,   0.20055,   0.40396,   0.04528,   0.01021,
     +    0.00558,   0.02847,   0.04800,
     +    0.45701,   0.19610,   0.38391,   0.04409,   0.01038,
     +    0.00582,   0.02792,   0.04670 /
        data ((f(i,24,k),i=1,5),f(7,24,k),f(6,24,k),f(8,24,k),k=15,19)/
     +    0.45048,   0.19178,   0.36563,   0.04290,   0.01046,
     +    0.00598,   0.02739,   0.04559,
     +    0.44408,   0.18771,   0.34877,   0.04182,   0.01052,
     +    0.00611,   0.02688,   0.04442,
     +    0.43793,   0.18391,   0.33331,   0.04081,   0.01055,
     +    0.00623,   0.02638,   0.04328,
     +    0.43177,   0.18034,   0.31892,   0.03987,   0.01055,
     +    0.00631,   0.02589,   0.04221,
     +    0.42593,   0.17694,   0.30570,   0.03898,   0.01053,
     +    0.00639,   0.02541,   0.04118 /
        data ((f(i,25,k),i=1,5),f(7,25,k),f(6,25,k),f(8,25,k),k=1,7)/
     +    0.56657,   0.27385,   0.77290,   0.05154,   0.00001,
     +    0.00000,   0.02418,   0.05592,
     +    0.55642,   0.26187,   0.70031,   0.04989,   0.00001,
     +    0.00002,   0.02490,   0.05366,
     +    0.54561,   0.25099,   0.63701,   0.04826,   0.00079,
     +    0.00006,   0.02519,   0.05146,
     +    0.53441,   0.24104,   0.58177,   0.04645,   0.00265,
     +    0.00004,   0.02521,   0.04939,
     +    0.52340,   0.23207,   0.53394,   0.04461,   0.00394,
     +    0.00029,   0.02498,   0.04760,
     +    0.51261,   0.22376,   0.49229,   0.04305,   0.00492,
     +    0.00105,   0.02463,   0.04576,
     +    0.50181,   0.21638,   0.45567,   0.04167,   0.00558,
     +    0.00181,   0.02422,   0.04398 /
        data ((f(i,25,k),i=1,5),f(7,25,k),f(6,25,k),f(8,25,k),k=8,14)/
     +    0.49159,   0.20941,   0.42329,   0.04020,   0.00616,
     +    0.00240,   0.02379,   0.04234,
     +    0.48193,   0.20276,   0.39454,   0.03868,   0.00655,
     +    0.00283,   0.02326,   0.04105,
     +    0.47246,   0.19676,   0.36890,   0.03738,   0.00687,
     +    0.00317,   0.02269,   0.03970,
     +    0.46331,   0.19126,   0.34601,   0.03622,   0.00713,
     +    0.00347,   0.02213,   0.03832,
     +    0.45478,   0.18593,   0.32555,   0.03498,   0.00724,
     +    0.00373,   0.02162,   0.03721,
     +    0.44627,   0.18111,   0.30698,   0.03395,   0.00738,
     +    0.00390,   0.02118,   0.03593,
     +    0.43822,   0.17649,   0.29019,   0.03298,   0.00747,
     +    0.00407,   0.02073,   0.03478 /
        data ((f(i,25,k),i=1,5),f(7,25,k),f(6,25,k),f(8,25,k),k=15,19)/
     +    0.43059,   0.17204,   0.27493,   0.03203,   0.00751,
     +    0.00424,   0.02026,   0.03380,
     +    0.42318,   0.16794,   0.26109,   0.03112,   0.00753,
     +    0.00434,   0.01979,   0.03283,
     +    0.41613,   0.16410,   0.24849,   0.03025,   0.00755,
     +    0.00442,   0.01935,   0.03191,
     +    0.40919,   0.16048,   0.23672,   0.02947,   0.00750,
     +    0.00442,   0.01889,   0.03107,
     +    0.40260,   0.15705,   0.22597,   0.02874,   0.00746,
     +    0.00443,   0.01845,   0.03027 /
        data ((f(i,26,k),i=1,5),f(7,26,k),f(6,26,k),f(8,26,k),k=1,7)/
     +    0.57320,   0.26055,   0.66597,   0.04112,   0.00001,
     +    0.00008,   0.01880,   0.04445,
     +    0.55803,   0.24685,   0.59189,   0.03946,   0.00001,
     +    0.00008,   0.01958,   0.04212,
     +    0.54300,   0.23468,   0.52989,   0.03771,   0.00022,
     +    0.00008,   0.01959,   0.04049,
     +    0.52818,   0.22397,   0.47756,   0.03616,   0.00172,
     +    0.00009,   0.01944,   0.03848,
     +    0.51431,   0.21407,   0.43304,   0.03457,   0.00274,
     +    0.00021,   0.01917,   0.03689,
     +    0.50084,   0.20543,   0.39519,   0.03308,   0.00349,
     +    0.00076,   0.01884,   0.03532,
     +    0.48800,   0.19750,   0.36238,   0.03172,   0.00409,
     +    0.00133,   0.01833,   0.03383 /
        data ((f(i,26,k),i=1,5),f(7,26,k),f(6,26,k),f(8,26,k),k=8,14)/
     +    0.47584,   0.19028,   0.33366,   0.03051,   0.00452,
     +    0.00171,   0.01791,   0.03232,
     +    0.46421,   0.18367,   0.30864,   0.02942,   0.00479,
     +    0.00198,   0.01748,   0.03093,
     +    0.45326,   0.17753,   0.28664,   0.02836,   0.00502,
     +    0.00223,   0.01702,   0.02972,
     +    0.44293,   0.17179,   0.26714,   0.02734,   0.00522,
     +    0.00247,   0.01653,   0.02863,
     +    0.43306,   0.16640,   0.24979,   0.02637,   0.00535,
     +    0.00270,   0.01609,   0.02767,
     +    0.42367,   0.16147,   0.23419,   0.02545,   0.00540,
     +    0.00284,   0.01568,   0.02665,
     +    0.41474,   0.15688,   0.22028,   0.02459,   0.00542,
     +    0.00292,   0.01529,   0.02574 /
        data ((f(i,26,k),i=1,5),f(7,26,k),f(6,26,k),f(8,26,k),k=15,19)/
     +    0.40624,   0.15257,   0.20783,   0.02376,   0.00539,
     +    0.00296,   0.01492,   0.02494,
     +    0.39801,   0.14859,   0.19640,   0.02306,   0.00537,
     +    0.00301,   0.01449,   0.02416,
     +    0.39022,   0.14486,   0.18606,   0.02241,   0.00535,
     +    0.00305,   0.01409,   0.02343,
     +    0.38288,   0.14112,   0.17672,   0.02172,   0.00534,
     +    0.00309,   0.01375,   0.02271,
     +    0.37586,   0.13766,   0.16816,   0.02106,   0.00531,
     +    0.00312,   0.01341,   0.02205 /
        data ((f(i,27,k),i=1,5),f(7,27,k),f(6,27,k),f(8,27,k),k=1,7)/
     +    0.57227,   0.24490,   0.57098,   0.03244,   0.00001,
     +    0.00001,   0.01463,   0.03454,
     +    0.55263,   0.22950,   0.49826,   0.03067,   0.00001,
     +    0.00005,   0.01506,   0.03305,
     +    0.53356,   0.21649,   0.43921,   0.02912,   0.00022,
     +    0.00002,   0.01518,   0.03109,
     +    0.51563,   0.20511,   0.39068,   0.02775,   0.00131,
     +    0.00009,   0.01496,   0.02952,
     +    0.49888,   0.19512,   0.35043,   0.02633,   0.00215,
     +    0.00023,   0.01454,   0.02817,
     +    0.48317,   0.18618,   0.31648,   0.02522,   0.00272,
     +    0.00064,   0.01426,   0.02665,
     +    0.46845,   0.17813,   0.28761,   0.02403,   0.00316,
     +    0.00095,   0.01388,   0.02544 /
        data ((f(i,27,k),i=1,5),f(7,27,k),f(6,27,k),f(8,27,k),k=8,14)/
     +    0.45458,   0.17093,   0.26274,   0.02294,   0.00338,
     +    0.00125,   0.01352,   0.02436,
     +    0.44172,   0.16420,   0.24127,   0.02195,   0.00359,
     +    0.00150,   0.01315,   0.02324,
     +    0.42952,   0.15810,   0.22253,   0.02113,   0.00370,
     +    0.00167,   0.01275,   0.02223,
     +    0.41797,   0.15256,   0.20607,   0.02041,   0.00374,
     +    0.00178,   0.01232,   0.02135,
     +    0.40730,   0.14727,   0.19155,   0.01957,   0.00388,
     +    0.00189,   0.01188,   0.02055,
     +    0.39716,   0.14243,   0.17887,   0.01882,   0.00390,
     +    0.00201,   0.01149,   0.01979,
     +    0.38760,   0.13795,   0.16751,   0.01810,   0.00390,
     +    0.00209,   0.01115,   0.01910 /
        data ((f(i,27,k),i=1,5),f(7,27,k),f(6,27,k),f(8,27,k),k=15,19)/
     +    0.37851,   0.13376,   0.15724,   0.01742,   0.00390,
     +    0.00210,   0.01084,   0.01848,
     +    0.36999,   0.12980,   0.14796,   0.01679,   0.00390,
     +    0.00213,   0.01052,   0.01778,
     +    0.36193,   0.12613,   0.13959,   0.01621,   0.00389,
     +    0.00215,   0.01022,   0.01713,
     +    0.35404,   0.12274,   0.13203,   0.01570,   0.00384,
     +    0.00218,   0.00994,   0.01657,
     +    0.34657,   0.11954,   0.12517,   0.01521,   0.00379,
     +    0.00219,   0.00967,   0.01603 /
        data ((f(i,28,k),i=1,5),f(7,28,k),f(6,28,k),f(8,28,k),k=1,7)/
     +    0.56538,   0.22677,   0.48679,   0.02493,   0.00001,
     +    0.00010,   0.01125,   0.02674,
     +    0.54059,   0.21110,   0.41754,   0.02362,   0.00001,
     +    0.00004,   0.01150,   0.02509,
     +    0.51810,   0.19754,   0.36280,   0.02226,   0.00004,
     +    0.00009,   0.01152,   0.02372,
     +    0.49725,   0.18600,   0.31868,   0.02104,   0.00095,
     +    0.00012,   0.01126,   0.02253,
     +    0.47797,   0.17605,   0.28261,   0.02006,   0.00146,
     +    0.00015,   0.01103,   0.02126,
     +    0.46054,   0.16712,   0.25296,   0.01899,   0.00183,
     +    0.00047,   0.01069,   0.02030,
     +    0.44439,   0.15914,   0.22776,   0.01801,   0.00221,
     +    0.00077,   0.01039,   0.01917 /
        data ((f(i,28,k),i=1,5),f(7,28,k),f(6,28,k),f(8,28,k),k=8,14)/
     +    0.42925,   0.15205,   0.20665,   0.01728,   0.00247,
     +    0.00090,   0.01003,   0.01807,
     +    0.41522,   0.14553,   0.18838,   0.01656,   0.00259,
     +    0.00109,   0.00965,   0.01726,
     +    0.40223,   0.13955,   0.17264,   0.01580,   0.00266,
     +    0.00126,   0.00933,   0.01652,
     +    0.39029,   0.13407,   0.15908,   0.01501,   0.00274,
     +    0.00135,   0.00905,   0.01579,
     +    0.37934,   0.12907,   0.14720,   0.01415,   0.00281,
     +    0.00136,   0.00872,   0.01512,
     +    0.36836,   0.12445,   0.13660,   0.01371,   0.00285,
     +    0.00144,   0.00843,   0.01446,
     +    0.35837,   0.12013,   0.12722,   0.01320,   0.00284,
     +    0.00149,   0.00815,   0.01390 /
        data ((f(i,28,k),i=1,5),f(7,28,k),f(6,28,k),f(8,28,k),k=15,19)/
     +    0.34934,   0.11605,   0.11888,   0.01254,   0.00279,
     +    0.00149,   0.00786,   0.01342,
     +    0.34021,   0.11244,   0.11151,   0.01209,   0.00279,
     +    0.00155,   0.00765,   0.01297,
     +    0.33161,   0.10908,   0.10490,   0.01167,   0.00279,
     +    0.00159,   0.00747,   0.01255,
     +    0.32368,   0.10578,   0.09886,   0.01125,   0.00273,
     +    0.00156,   0.00722,   0.01213,
     +    0.31617,   0.10270,   0.09342,   0.01087,   0.00269,
     +    0.00153,   0.00699,   0.01171 /
        data ((f(i,29,k),i=1,5),f(7,29,k),f(6,29,k),f(8,29,k),k=1,7)/
     +    0.55161,   0.20834,   0.41234,   0.01882,   0.00001,
     +    0.00010,   0.00846,   0.02046,
     +    0.52309,   0.19192,   0.34778,   0.01775,   0.00001,
     +    0.00000,   0.00868,   0.01921,
     +    0.49732,   0.17840,   0.29790,   0.01675,   0.00001,
     +    0.00010,   0.00874,   0.01797,
     +    0.47384,   0.16703,   0.25853,   0.01590,   0.00063,
     +    0.00007,   0.00861,   0.01677,
     +    0.45313,   0.15697,   0.22705,   0.01487,   0.00118,
     +    0.00016,   0.00842,   0.01571,
     +    0.43434,   0.14821,   0.20114,   0.01398,   0.00150,
     +    0.00036,   0.00808,   0.01494,
     +    0.41707,   0.14043,   0.17977,   0.01323,   0.00173,
     +    0.00055,   0.00778,   0.01414 /
        data ((f(i,29,k),i=1,5),f(7,29,k),f(6,29,k),f(8,29,k),k=8,14)/
     +    0.40096,   0.13356,   0.16184,   0.01256,   0.00186,
     +    0.00073,   0.00748,   0.01348,
     +    0.38638,   0.12729,   0.14652,   0.01194,   0.00193,
     +    0.00084,   0.00721,   0.01278,
     +    0.37288,   0.12164,   0.13346,   0.01139,   0.00203,
     +    0.00090,   0.00692,   0.01209,
     +    0.36030,   0.11654,   0.12224,   0.01086,   0.00208,
     +    0.00094,   0.00662,   0.01149,
     +    0.34863,   0.11186,   0.11247,   0.01034,   0.00199,
     +    0.00101,   0.00645,   0.01102,
     +    0.33783,   0.10752,   0.10394,   0.00976,   0.00205,
     +    0.00105,   0.00619,   0.01061,
     +    0.32769,   0.10350,   0.09643,   0.00932,   0.00210,
     +    0.00109,   0.00594,   0.01019 /
        data ((f(i,29,k),i=1,5),f(7,29,k),f(6,29,k),f(8,29,k),k=15,19)/
     +    0.31810,   0.09977,   0.08980,   0.00902,   0.00212,
     +    0.00115,   0.00574,   0.00976,
     +    0.30922,   0.09632,   0.08383,   0.00857,   0.00208,
     +    0.00112,   0.00554,   0.00936,
     +    0.30094,   0.09312,   0.07851,   0.00814,   0.00201,
     +    0.00109,   0.00534,   0.00900,
     +    0.29306,   0.09006,   0.07380,   0.00784,   0.00200,
     +    0.00111,   0.00516,   0.00864,
     +    0.28559,   0.08722,   0.06954,   0.00758,   0.00199,
     +    0.00113,   0.00499,   0.00832 /
        data ((f(i,30,k),i=1,5),f(7,30,k),f(6,30,k),f(8,30,k),k=1,7)/
     +    0.53275,   0.18980,   0.34717,   0.01472,   0.00001,
     +    0.00007,   0.00592,   0.01533,
     +    0.50046,   0.17362,   0.28792,   0.01377,   0.00001,
     +    0.00004,   0.00624,   0.01432,
     +    0.47216,   0.16028,   0.24361,   0.01288,   0.00001,
     +    0.00012,   0.00630,   0.01339,
     +    0.44743,   0.14867,   0.20883,   0.01192,   0.00037,
     +    0.00006,   0.00612,   0.01259,
     +    0.42526,   0.13897,   0.18143,   0.01116,   0.00069,
     +    0.00005,   0.00601,   0.01182,
     +    0.40536,   0.13054,   0.15956,   0.01046,   0.00095,
     +    0.00014,   0.00583,   0.01110,
     +    0.38734,   0.12327,   0.14159,   0.00977,   0.00120,
     +    0.00033,   0.00561,   0.01048 /
        data ((f(i,30,k),i=1,5),f(7,30,k),f(6,30,k),f(8,30,k),k=8,14)/
     +    0.37117,   0.11660,   0.12646,   0.00918,   0.00131,
     +    0.00042,   0.00528,   0.00991,
     +    0.35595,   0.11079,   0.11384,   0.00889,   0.00140,
     +    0.00054,   0.00511,   0.00921,
     +    0.34224,   0.10551,   0.10312,   0.00838,   0.00145,
     +    0.00062,   0.00497,   0.00873,
     +    0.32984,   0.10063,   0.09393,   0.00778,   0.00146,
     +    0.00065,   0.00479,   0.00841,
     +    0.31791,   0.09624,   0.08602,   0.00755,   0.00147,
     +    0.00068,   0.00460,   0.00799,
     +    0.30715,   0.09225,   0.07912,   0.00720,   0.00144,
     +    0.00072,   0.00439,   0.00750,
     +    0.29710,   0.08857,   0.07306,   0.00682,   0.00142,
     +    0.00072,   0.00420,   0.00711 /
        data ((f(i,30,k),i=1,5),f(7,30,k),f(6,30,k),f(8,30,k),k=15,19)/
     +    0.28766,   0.08516,   0.06772,   0.00642,   0.00141,
     +    0.00068,   0.00404,   0.00685,
     +    0.27867,   0.08201,   0.06308,   0.00613,   0.00139,
     +    0.00071,   0.00391,   0.00665,
     +    0.27031,   0.07911,   0.05895,   0.00588,   0.00139,
     +    0.00074,   0.00379,   0.00646,
     +    0.26267,   0.07633,   0.05519,   0.00564,   0.00138,
     +    0.00077,   0.00362,   0.00614,
     +    0.25542,   0.07375,   0.05182,   0.00545,   0.00136,
     +    0.00078,   0.00349,   0.00585 /
        data ((f(i,31,k),i=1,5),f(7,31,k),f(6,31,k),f(8,31,k),k=1,7)/
     +    0.50914,   0.17015,   0.28913,   0.01049,   0.00001,
     +    0.00012,   0.00455,   0.01127,
     +    0.47426,   0.15417,   0.23659,   0.00972,   0.00001,
     +    0.00008,   0.00473,   0.01058,
     +    0.44395,   0.14141,   0.19727,   0.00918,   0.00001,
     +    0.00003,   0.00476,   0.00965,
     +    0.41794,   0.13044,   0.16745,   0.00837,   0.00030,
     +    0.00010,   0.00466,   0.00919,
     +    0.39481,   0.12128,   0.14411,   0.00784,   0.00062,
     +    0.00014,   0.00446,   0.00864,
     +    0.37437,   0.11346,   0.12554,   0.00749,   0.00080,
     +    0.00024,   0.00426,   0.00798,
     +    0.35603,   0.10655,   0.11047,   0.00693,   0.00090,
     +    0.00033,   0.00408,   0.00759 /
        data ((f(i,31,k),i=1,5),f(7,31,k),f(6,31,k),f(8,31,k),k=8,14)/
     +    0.33928,   0.10054,   0.09801,   0.00665,   0.00100,
     +    0.00040,   0.00397,   0.00701,
     +    0.32454,   0.09503,   0.08765,   0.00610,   0.00102,
     +    0.00043,   0.00382,   0.00674,
     +    0.31096,   0.09012,   0.07896,   0.00560,   0.00106,
     +    0.00048,   0.00363,   0.00646,
     +    0.29826,   0.08579,   0.07158,   0.00536,   0.00112,
     +    0.00053,   0.00346,   0.00604,
     +    0.28656,   0.08189,   0.06527,   0.00524,   0.00110,
     +    0.00050,   0.00331,   0.00567,
     +    0.27593,   0.07821,   0.05978,   0.00492,   0.00111,
     +    0.00050,   0.00316,   0.00536,
     +    0.26610,   0.07483,   0.05497,   0.00465,   0.00109,
     +    0.00051,   0.00302,   0.00512 /
        data ((f(i,31,k),i=1,5),f(7,31,k),f(6,31,k),f(8,31,k),k=15,19)/
     +    0.25692,   0.07173,   0.05074,   0.00445,   0.00106,
     +    0.00055,   0.00286,   0.00497,
     +    0.24814,   0.06890,   0.04703,   0.00437,   0.00105,
     +    0.00056,   0.00277,   0.00464,
     +    0.23998,   0.06631,   0.04375,   0.00430,   0.00104,
     +    0.00056,   0.00269,   0.00433,
     +    0.23238,   0.06386,   0.04090,   0.00415,   0.00096,
     +    0.00055,   0.00257,   0.00423,
     +    0.22539,   0.06157,   0.03833,   0.00398,   0.00091,
     +    0.00055,   0.00247,   0.00412 /
        data ((f(i,32,k),i=1,5),f(7,32,k),f(6,32,k),f(8,32,k),k=1,7)/
     +    0.48189,   0.15218,   0.23951,   0.00794,   0.00001,
     +    0.00001,   0.00301,   0.00802,
     +    0.44468,   0.13685,   0.19286,   0.00734,   0.00001,
     +    0.00002,   0.00327,   0.00772,
     +    0.41316,   0.12445,   0.15879,   0.00682,   0.00001,
     +    0.00002,   0.00328,   0.00718,
     +    0.38650,   0.11416,   0.13337,   0.00631,   0.00005,
     +    0.00003,   0.00322,   0.00661,
     +    0.36315,   0.10543,   0.11373,   0.00580,   0.00035,
     +    0.00006,   0.00316,   0.00616,
     +    0.34227,   0.09806,   0.09825,   0.00542,   0.00053,
     +    0.00013,   0.00305,   0.00590,
     +    0.32419,   0.09170,   0.08583,   0.00488,   0.00064,
     +    0.00019,   0.00293,   0.00555 /
        data ((f(i,32,k),i=1,5),f(7,32,k),f(6,32,k),f(8,32,k),k=8,14)/
     +    0.30779,   0.08615,   0.07577,   0.00471,   0.00068,
     +    0.00020,   0.00272,   0.00506,
     +    0.29284,   0.08123,   0.06732,   0.00455,   0.00070,
     +    0.00023,   0.00254,   0.00467,
     +    0.27940,   0.07676,   0.06023,   0.00423,   0.00076,
     +    0.00026,   0.00246,   0.00445,
     +    0.26727,   0.07269,   0.05431,   0.00390,   0.00080,
     +    0.00029,   0.00238,   0.00428,
     +    0.25617,   0.06899,   0.04930,   0.00371,   0.00073,
     +    0.00033,   0.00226,   0.00398,
     +    0.24576,   0.06577,   0.04492,   0.00358,   0.00069,
     +    0.00033,   0.00213,   0.00381,
     +    0.23624,   0.06281,   0.04116,   0.00337,   0.00068,
     +    0.00033,   0.00202,   0.00367 /
        data ((f(i,32,k),i=1,5),f(7,32,k),f(6,32,k),f(8,32,k),k=15,19)/
     +    0.22755,   0.06005,   0.03791,   0.00311,   0.00067,
     +    0.00035,   0.00192,   0.00353,
     +    0.21927,   0.05753,   0.03502,   0.00308,   0.00065,
     +    0.00034,   0.00184,   0.00325,
     +    0.21158,   0.05520,   0.03247,   0.00309,   0.00065,
     +    0.00032,   0.00177,   0.00298,
     +    0.20438,   0.05302,   0.03025,   0.00299,   0.00063,
     +    0.00033,   0.00171,   0.00287,
     +    0.19773,   0.05103,   0.02826,   0.00289,   0.00062,
     +    0.00034,   0.00165,   0.00274 /
        data ((f(i,33,k),i=1,5),f(7,33,k),f(6,33,k),f(8,33,k),k=1,7)/
     +    0.45140,   0.13384,   0.19629,   0.00549,   0.00001,
     +    0.00002,   0.00212,   0.00592,
     +    0.41297,   0.11927,   0.15587,   0.00502,   0.00001,
     +    0.00002,   0.00234,   0.00551,
     +    0.38107,   0.10777,   0.12669,   0.00472,   0.00001,
     +    0.00007,   0.00245,   0.00504,
     +    0.35339,   0.09828,   0.10534,   0.00434,   0.00015,
     +    0.00004,   0.00233,   0.00485,
     +    0.33017,   0.09046,   0.08908,   0.00403,   0.00032,
     +    0.00006,   0.00224,   0.00444,
     +    0.31005,   0.08364,   0.07629,   0.00368,   0.00041,
     +    0.00011,   0.00215,   0.00419,
     +    0.29212,   0.07787,   0.06611,   0.00352,   0.00042,
     +    0.00012,   0.00201,   0.00377 /
        data ((f(i,33,k),i=1,5),f(7,33,k),f(6,33,k),f(8,33,k),k=8,14)/
     +    0.27577,   0.07283,   0.05790,   0.00343,   0.00039,
     +    0.00015,   0.00195,   0.00355,
     +    0.26156,   0.06824,   0.05116,   0.00319,   0.00043,
     +    0.00018,   0.00184,   0.00331,
     +    0.24884,   0.06423,   0.04558,   0.00285,   0.00046,
     +    0.00021,   0.00173,   0.00312,
     +    0.23713,   0.06074,   0.04090,   0.00259,   0.00048,
     +    0.00021,   0.00167,   0.00298,
     +    0.22628,   0.05757,   0.03692,   0.00257,   0.00047,
     +    0.00022,   0.00156,   0.00274,
     +    0.21651,   0.05464,   0.03349,   0.00246,   0.00045,
     +    0.00025,   0.00150,   0.00257,
     +    0.20756,   0.05197,   0.03055,   0.00235,   0.00047,
     +    0.00026,   0.00144,   0.00240 /
        data ((f(i,33,k),i=1,5),f(7,33,k),f(6,33,k),f(8,33,k),k=15,19)/
     +    0.19928,   0.04956,   0.02803,   0.00226,   0.00051,
     +    0.00025,   0.00139,   0.00222,
     +    0.19158,   0.04737,   0.02583,   0.00220,   0.00050,
     +    0.00028,   0.00133,   0.00212,
     +    0.18449,   0.04538,   0.02390,   0.00211,   0.00048,
     +    0.00029,   0.00126,   0.00203,
     +    0.17789,   0.04346,   0.02220,   0.00194,   0.00046,
     +    0.00023,   0.00118,   0.00197,
     +    0.17175,   0.04171,   0.02069,   0.00181,   0.00043,
     +    0.00019,   0.00110,   0.00190 /
        data ((f(i,34,k),i=1,5),f(7,34,k),f(6,34,k),f(8,34,k),k=1,7)/
     +    0.41866,   0.11687,   0.15953,   0.00362,   0.00001,
     +    0.00008,   0.00143,   0.00412,
     +    0.37996,   0.10336,   0.12489,   0.00345,   0.00001,
     +    0.00007,   0.00161,   0.00391,
     +    0.34768,   0.09286,   0.10038,   0.00318,   0.00001,
     +    0.00000,   0.00163,   0.00375,
     +    0.32090,   0.08415,   0.08252,   0.00289,   0.00001,
     +    0.00002,   0.00151,   0.00349,
     +    0.29787,   0.07684,   0.06909,   0.00281,   0.00011,
     +    0.00006,   0.00144,   0.00319,
     +    0.27790,   0.07097,   0.05877,   0.00267,   0.00018,
     +    0.00007,   0.00143,   0.00283,
     +    0.26062,   0.06573,   0.05059,   0.00256,   0.00026,
     +    0.00012,   0.00134,   0.00264 /
        data ((f(i,34,k),i=1,5),f(7,34,k),f(6,34,k),f(8,34,k),k=8,14)/
     +    0.24556,   0.06109,   0.04403,   0.00228,   0.00025,
     +    0.00013,   0.00124,   0.00250,
     +    0.23178,   0.05713,   0.03870,   0.00219,   0.00027,
     +    0.00012,   0.00120,   0.00234,
     +    0.21940,   0.05363,   0.03428,   0.00212,   0.00031,
     +    0.00011,   0.00112,   0.00215,
     +    0.20837,   0.05050,   0.03058,   0.00201,   0.00034,
     +    0.00011,   0.00104,   0.00199,
     +    0.19835,   0.04765,   0.02748,   0.00192,   0.00037,
     +    0.00015,   0.00111,   0.00189,
     +    0.18919,   0.04511,   0.02488,   0.00182,   0.00033,
     +    0.00011,   0.00098,   0.00163,
     +    0.18073,   0.04283,   0.02264,   0.00178,   0.00028,
     +    0.00010,   0.00091,   0.00151 /
        data ((f(i,34,k),i=1,5),f(7,34,k),f(6,34,k),f(8,34,k),k=15,19)/
     +    0.17287,   0.04077,   0.02070,   0.00180,   0.00025,
     +    0.00014,   0.00087,   0.00151,
     +    0.16586,   0.03887,   0.01901,   0.00166,   0.00027,
     +    0.00014,   0.00084,   0.00137,
     +    0.15945,   0.03712,   0.01752,   0.00153,   0.00029,
     +    0.00013,   0.00081,   0.00124,
     +    0.15322,   0.03551,   0.01623,   0.00153,   0.00027,
     +    0.00010,   0.00075,   0.00120,
     +    0.14755,   0.03403,   0.01508,   0.00148,   0.00026,
     +    0.00009,   0.00069,   0.00117 /
        data ((f(i,35,k),i=1,5),f(7,35,k),f(6,35,k),f(8,35,k),k=1,7)/
     +    0.38446,   0.10083,   0.12828,   0.00241,   0.00001,
     +    0.00001,   0.00104,   0.00289,
     +    0.34532,   0.08856,   0.09880,   0.00224,   0.00001,
     +    0.00003,   0.00110,   0.00262,
     +    0.31328,   0.07871,   0.07835,   0.00229,   0.00001,
     +    0.00006,   0.00108,   0.00252,
     +    0.28709,   0.07086,   0.06381,   0.00208,   0.00003,
     +    0.00001,   0.00114,   0.00223,
     +    0.26527,   0.06441,   0.05285,   0.00181,   0.00011,
     +    0.00002,   0.00110,   0.00206,
     +    0.24629,   0.05896,   0.04453,   0.00184,   0.00017,
     +    0.00004,   0.00101,   0.00191,
     +    0.22988,   0.05437,   0.03809,   0.00174,   0.00020,
     +    0.00007,   0.00094,   0.00163 /
        data ((f(i,35,k),i=1,5),f(7,35,k),f(6,35,k),f(8,35,k),k=8,14)/
     +    0.21536,   0.05041,   0.03290,   0.00156,   0.00025,
     +    0.00008,   0.00092,   0.00158,
     +    0.20238,   0.04693,   0.02871,   0.00150,   0.00026,
     +    0.00007,   0.00085,   0.00151,
     +    0.19088,   0.04387,   0.02531,   0.00147,   0.00026,
     +    0.00008,   0.00079,   0.00133,
     +    0.18064,   0.04116,   0.02248,   0.00141,   0.00026,
     +    0.00009,   0.00077,   0.00117,
     +    0.17127,   0.03876,   0.02011,   0.00131,   0.00024,
     +    0.00009,   0.00071,   0.00111,
     +    0.16283,   0.03658,   0.01811,   0.00128,   0.00024,
     +    0.00011,   0.00073,   0.00110,
     +    0.15522,   0.03460,   0.01641,   0.00119,   0.00025,
     +    0.00011,   0.00070,   0.00108 /
        data ((f(i,35,k),i=1,5),f(7,35,k),f(6,35,k),f(8,35,k),k=15,19)/
     +    0.14831,   0.03281,   0.01494,   0.00102,   0.00026,
     +    0.00010,   0.00064,   0.00104,
     +    0.14181,   0.03117,   0.01368,   0.00099,   0.00025,
     +    0.00012,   0.00061,   0.00099,
     +    0.13584,   0.02969,   0.01257,   0.00098,   0.00025,
     +    0.00013,   0.00059,   0.00094,
     +    0.13044,   0.02832,   0.01162,   0.00087,   0.00026,
     +    0.00014,   0.00057,   0.00089,
     +    0.12541,   0.02706,   0.01077,   0.00079,   0.00026,
     +    0.00014,   0.00055,   0.00084 /
        data ((f(i,36,k),i=1,5),f(7,36,k),f(6,36,k),f(8,36,k),k=1,7)/
     +    0.34970,   0.08634,   0.10210,   0.00144,   0.00001,
     +    0.00002,   0.00044,   0.00217,
     +    0.31109,   0.07503,   0.07750,   0.00164,   0.00001,
     +    0.00002,   0.00061,   0.00193,
     +    0.28053,   0.06625,   0.06078,   0.00141,   0.00001,
     +    0.00006,   0.00065,   0.00179,
     +    0.25532,   0.05930,   0.04893,   0.00126,   0.00003,
     +    0.00001,   0.00068,   0.00175,
     +    0.23414,   0.05361,   0.04021,   0.00122,   0.00005,
     +    0.00001,   0.00067,   0.00163,
     +    0.21632,   0.04889,   0.03361,   0.00122,   0.00009,
     +    0.00001,   0.00065,   0.00129,
     +    0.20086,   0.04483,   0.02853,   0.00114,   0.00014,
     +    0.00003,   0.00062,   0.00119 /
        data ((f(i,36,k),i=1,5),f(7,36,k),f(6,36,k),f(8,36,k),k=8,14)/
     +    0.18713,   0.04140,   0.02447,   0.00115,   0.00009,
     +    0.00000,   0.00055,   0.00115,
     +    0.17529,   0.03838,   0.02126,   0.00114,   0.00012,
     +    0.00002,   0.00052,   0.00100,
     +    0.16486,   0.03575,   0.01864,   0.00104,   0.00014,
     +    0.00003,   0.00049,   0.00086,
     +    0.15552,   0.03344,   0.01648,   0.00093,   0.00013,
     +    0.00003,   0.00045,   0.00077,
     +    0.14717,   0.03134,   0.01469,   0.00086,   0.00015,
     +    0.00005,   0.00045,   0.00068,
     +    0.13953,   0.02948,   0.01318,   0.00075,   0.00017,
     +    0.00006,   0.00044,   0.00072,
     +    0.13246,   0.02783,   0.01191,   0.00073,   0.00016,
     +    0.00006,   0.00041,   0.00072 /
        data ((f(i,36,k),i=1,5),f(7,36,k),f(6,36,k),f(8,36,k),k=15,19)/
     +    0.12589,   0.02633,   0.01081,   0.00079,   0.00014,
     +    0.00005,   0.00038,   0.00067,
     +    0.12026,   0.02496,   0.00986,   0.00069,   0.00016,
     +    0.00005,   0.00037,   0.00059,
     +    0.11513,   0.02372,   0.00904,   0.00056,   0.00016,
     +    0.00005,   0.00037,   0.00050,
     +    0.11021,   0.02256,   0.00833,   0.00048,   0.00013,
     +    0.00005,   0.00034,   0.00052,
     +    0.10569,   0.02152,   0.00770,   0.00042,   0.00011,
     +    0.00006,   0.00032,   0.00054 /
        data ((f(i,37,k),i=1,5),f(7,37,k),f(6,37,k),f(8,37,k),k=1,7)/
     +    0.31340,   0.07287,   0.08010,   0.00120,   0.00001,
     +    0.00000,   0.00026,   0.00137,
     +    0.27667,   0.06280,   0.05995,   0.00110,   0.00001,
     +    0.00001,   0.00043,   0.00136,
     +    0.24737,   0.05510,   0.04647,   0.00115,   0.00001,
     +    0.00001,   0.00044,   0.00107,
     +    0.22381,   0.04895,   0.03700,   0.00105,   0.00001,
     +    0.00005,   0.00050,   0.00094,
     +    0.20373,   0.04404,   0.03013,   0.00121,   0.00001,
     +    0.00005,   0.00043,   0.00079,
     +    0.18738,   0.03996,   0.02500,   0.00103,   0.00003,
     +    0.00002,   0.00040,   0.00075,
     +    0.17334,   0.03646,   0.02106,   0.00095,   0.00005,
     +    0.00001,   0.00036,   0.00067 /
        data ((f(i,37,k),i=1,5),f(7,37,k),f(6,37,k),f(8,37,k),k=8,14)/
     +    0.16108,   0.03352,   0.01798,   0.00088,   0.00009,
     +    0.00003,   0.00036,   0.00061,
     +    0.15027,   0.03098,   0.01552,   0.00074,   0.00007,
     +    0.00003,   0.00033,   0.00065,
     +    0.14064,   0.02873,   0.01354,   0.00071,   0.00008,
     +    0.00001,   0.00031,   0.00054,
     +    0.13214,   0.02678,   0.01192,   0.00072,   0.00011,
     +    0.00002,   0.00031,   0.00045,
     +    0.12473,   0.02501,   0.01058,   0.00051,   0.00009,
     +    0.00004,   0.00030,   0.00057,
     +    0.11790,   0.02348,   0.00945,   0.00043,   0.00010,
     +    0.00004,   0.00028,   0.00051,
     +    0.11168,   0.02208,   0.00849,   0.00039,   0.00010,
     +    0.00004,   0.00026,   0.00044 /
        data ((f(i,37,k),i=1,5),f(7,37,k),f(6,37,k),f(8,37,k),k=15,19)/
     +    0.10602,   0.02082,   0.00769,   0.00039,   0.00009,
     +    0.00003,   0.00024,   0.00040,
     +    0.10082,   0.01970,   0.00700,   0.00038,   0.00009,
     +    0.00004,   0.00023,   0.00039,
     +    0.09609,   0.01867,   0.00640,   0.00039,   0.00011,
     +    0.00005,   0.00023,   0.00040,
     +    0.09175,   0.01774,   0.00588,   0.00036,   0.00009,
     +    0.00005,   0.00021,   0.00035,
     +    0.08775,   0.01688,   0.00543,   0.00034,   0.00010,
     +    0.00005,   0.00020,   0.00033 /
        data ((f(i,38,k),i=1,5),f(7,38,k),f(6,38,k),f(8,38,k),k=1,7)/
     +    0.27888,   0.06083,   0.06204,   0.00057,   0.00001,
     +    0.00005,   0.00015,   0.00079,
     +    0.24346,   0.05199,   0.04573,   0.00083,   0.00001,
     +    0.00003,   0.00020,   0.00080,
     +    0.21650,   0.04514,   0.03499,   0.00051,   0.00001,
     +    0.00001,   0.00022,   0.00075,
     +    0.19412,   0.03987,   0.02754,   0.00064,   0.00001,
     +    0.00006,   0.00024,   0.00061,
     +    0.17601,   0.03561,   0.02221,   0.00056,   0.00001,
     +    0.00006,   0.00020,   0.00052,
     +    0.16082,   0.03211,   0.01828,   0.00058,   0.00003,
     +    0.00002,   0.00025,   0.00050,
     +    0.14782,   0.02920,   0.01528,   0.00054,   0.00002,
     +    0.00002,   0.00025,   0.00052 /
        data ((f(i,38,k),i=1,5),f(7,38,k),f(6,38,k),f(8,38,k),k=8,14)/
     +    0.13676,   0.02670,   0.01295,   0.00045,   0.00007,
     +    0.00001,   0.00021,   0.00041,
     +    0.12707,   0.02456,   0.01112,   0.00040,   0.00009,
     +    0.00001,   0.00022,   0.00039,
     +    0.11846,   0.02270,   0.00964,   0.00037,   0.00008,
     +    0.00003,   0.00023,   0.00042,
     +    0.11088,   0.02105,   0.00844,   0.00033,   0.00009,
     +    0.00003,   0.00022,   0.00042,
     +    0.10430,   0.01963,   0.00746,   0.00024,   0.00008,
     +    0.00002,   0.00023,   0.00039,
     +    0.09809,   0.01833,   0.00665,   0.00031,   0.00009,
     +    0.00002,   0.00021,   0.00029,
     +    0.09258,   0.01719,   0.00596,   0.00031,   0.00009,
     +    0.00003,   0.00018,   0.00025 /
        data ((f(i,38,k),i=1,5),f(7,38,k),f(6,38,k),f(8,38,k),k=15,19)/
     +    0.08775,   0.01616,   0.00538,   0.00021,   0.00009,
     +    0.00004,   0.00018,   0.00030,
     +    0.08322,   0.01524,   0.00488,   0.00021,   0.00009,
     +    0.00004,   0.00018,   0.00027,
     +    0.07909,   0.01442,   0.00445,   0.00021,   0.00009,
     +    0.00005,   0.00017,   0.00025,
     +    0.07531,   0.01364,   0.00408,   0.00020,   0.00010,
     +    0.00005,   0.00016,   0.00023,
     +    0.07185,   0.01294,   0.00376,   0.00019,   0.00008,
     +    0.00005,   0.00015,   0.00021 /
        data ((f(i,39,k),i=1,5),f(7,39,k),f(6,39,k),f(8,39,k),k=1,7)/
     +    0.24440,   0.04987,   0.04722,   0.00033,   0.00001,
     +    0.00005,   0.00006,   0.00047,
     +    0.21132,   0.04215,   0.03429,   0.00041,   0.00001,
     +    0.00006,   0.00011,   0.00044,
     +    0.18597,   0.03635,   0.02586,   0.00051,   0.00001,
     +    0.00004,   0.00014,   0.00039,
     +    0.16571,   0.03185,   0.02013,   0.00040,   0.00004,
     +    0.00001,   0.00018,   0.00049,
     +    0.14944,   0.02827,   0.01609,   0.00030,   0.00004,
     +    0.00003,   0.00020,   0.00034,
     +    0.13588,   0.02534,   0.01313,   0.00022,   0.00003,
     +    0.00002,   0.00018,   0.00031,
     +    0.12427,   0.02291,   0.01090,   0.00023,   0.00005,
     +    0.00001,   0.00018,   0.00030 /
        data ((f(i,39,k),i=1,5),f(7,39,k),f(6,39,k),f(8,39,k),k=8,14)/
     +    0.11431,   0.02089,   0.00917,   0.00024,   0.00007,
     +    0.00001,   0.00019,   0.00027,
     +    0.10559,   0.01912,   0.00783,   0.00027,   0.00005,
     +    0.00001,   0.00013,   0.00019,
     +    0.09806,   0.01760,   0.00676,   0.00023,   0.00005,
     +    0.00000,   0.00012,   0.00018,
     +    0.09151,   0.01627,   0.00589,   0.00016,   0.00007,
     +    0.00002,   0.00016,   0.00020,
     +    0.08567,   0.01509,   0.00519,   0.00013,   0.00009,
     +    0.00004,   0.00016,   0.00016,
     +    0.08042,   0.01407,   0.00461,   0.00009,   0.00008,
     +    0.00003,   0.00013,   0.00016,
     +    0.07566,   0.01315,   0.00412,   0.00008,   0.00008,
     +    0.00003,   0.00012,   0.00015 /
        data ((f(i,39,k),i=1,5),f(7,39,k),f(6,39,k),f(8,39,k),k=15,19)/
     +    0.07131,   0.01232,   0.00370,   0.00012,   0.00008,
     +    0.00002,   0.00012,   0.00014,
     +    0.06746,   0.01159,   0.00335,   0.00011,   0.00007,
     +    0.00003,   0.00013,   0.00015,
     +    0.06399,   0.01093,   0.00304,   0.00009,   0.00007,
     +    0.00003,   0.00013,   0.00015,
     +    0.06076,   0.01032,   0.00279,   0.00007,   0.00005,
     +    0.00003,   0.00010,   0.00014,
     +    0.05782,   0.00976,   0.00256,   0.00005,   0.00005,
     +    0.00002,   0.00009,   0.00013 /
        data ((f(i,40,k),i=1,5),f(7,40,k),f(6,40,k),f(8,40,k),k=1,7)/
     +    0.21180,   0.04060,   0.03552,   0.00040,   0.00001,
     +    0.00010,   0.00001,   0.00035,
     +    0.18146,   0.03395,   0.02538,   0.00045,   0.00001,
     +    0.00006,   0.00002,   0.00030,
     +    0.15829,   0.02909,   0.01893,   0.00044,   0.00001,
     +    0.00007,   0.00008,   0.00024,
     +    0.14038,   0.02532,   0.01458,   0.00021,   0.00001,
     +    0.00007,   0.00007,   0.00017,
     +    0.12549,   0.02231,   0.01155,   0.00023,   0.00001,
     +    0.00004,   0.00009,   0.00026,
     +    0.11345,   0.01988,   0.00936,   0.00021,   0.00002,
     +    0.00005,   0.00008,   0.00016,
     +    0.10312,   0.01790,   0.00771,   0.00027,   0.00003,
     +    0.00002,   0.00012,   0.00015 /
        data ((f(i,40,k),i=1,5),f(7,40,k),f(6,40,k),f(8,40,k),k=8,14)/
     +    0.09442,   0.01622,   0.00646,   0.00023,   0.00003,
     +    0.00002,   0.00010,   0.00016,
     +    0.08701,   0.01480,   0.00549,   0.00015,   0.00004,
     +    0.00000,   0.00008,   0.00014,
     +    0.08055,   0.01356,   0.00472,   0.00010,   0.00004,
     +    0.00000,   0.00009,   0.00013,
     +    0.07481,   0.01248,   0.00410,   0.00008,   0.00004,
     +    0.00000,   0.00008,   0.00013,
     +    0.06965,   0.01154,   0.00359,   0.00013,   0.00004,
     +    0.00001,   0.00006,   0.00014,
     +    0.06513,   0.01073,   0.00317,   0.00012,   0.00003,
     +    0.00001,   0.00007,   0.00013,
     +    0.06113,   0.00999,   0.00283,   0.00011,   0.00003,
     +    0.00000,   0.00007,   0.00011 /
        data ((f(i,40,k),i=1,5),f(7,40,k),f(6,40,k),f(8,40,k),k=15,19)/
     +    0.05755,   0.00934,   0.00254,   0.00007,   0.00002,
     +    0.00001,   0.00007,   0.00010,
     +    0.05429,   0.00876,   0.00229,   0.00006,   0.00002,
     +    0.00001,   0.00006,   0.00007,
     +    0.05135,   0.00824,   0.00208,   0.00004,   0.00002,
     +    0.00000,   0.00006,   0.00006,
     +    0.04866,   0.00776,   0.00190,   0.00001,   0.00004,
     +    0.00001,   0.00006,   0.00009,
     +    0.04620,   0.00733,   0.00174,   0.00002,   0.00004,
     +    0.00002,   0.00006,   0.00011 /
        data ((f(i,41,k),i=1,5),f(7,41,k),f(6,41,k),f(8,41,k),k=1,7)/
     +    0.18063,   0.03217,   0.02606,   0.00015,   0.00001,
     +    0.00004,   0.00001,   0.00014,
     +    0.15337,   0.02677,   0.01841,   0.00024,   0.00001,
     +    0.00003,   0.00001,   0.00016,
     +    0.13272,   0.02277,   0.01358,   0.00022,   0.00001,
     +    0.00004,   0.00002,   0.00018,
     +    0.11667,   0.01970,   0.01038,   0.00022,   0.00001,
     +    0.00002,   0.00002,   0.00012,
     +    0.10374,   0.01728,   0.00815,   0.00023,   0.00001,
     +    0.00001,   0.00003,   0.00008,
     +    0.09320,   0.01534,   0.00655,   0.00017,   0.00001,
     +    0.00002,   0.00001,   0.00012,
     +    0.08453,   0.01373,   0.00537,   0.00009,   0.00001,
     +    0.00002,   0.00000,   0.00009 /
        data ((f(i,41,k),i=1,5),f(7,41,k),f(6,41,k),f(8,41,k),k=8,14)/
     +    0.07706,   0.01240,   0.00447,   0.00008,   0.00001,
     +    0.00001,   0.00002,   0.00010,
     +    0.07062,   0.01126,   0.00378,   0.00009,   0.00001,
     +    0.00000,   0.00004,   0.00011,
     +    0.06505,   0.01029,   0.00323,   0.00010,   0.00000,
     +    0.00000,   0.00003,   0.00009,
     +    0.06018,   0.00946,   0.00279,   0.00012,   0.00000,
     +    0.00001,   0.00003,   0.00007,
     +    0.05588,   0.00872,   0.00244,   0.00012,   0.00000,
     +    0.00001,   0.00003,   0.00005,
     +    0.05210,   0.00807,   0.00215,   0.00010,   0.00001,
     +    0.00000,   0.00003,   0.00007,
     +    0.04872,   0.00750,   0.00191,   0.00008,   0.00001,
     +    0.00000,   0.00003,   0.00007 /
        data ((f(i,41,k),i=1,5),f(7,41,k),f(6,41,k),f(8,41,k),k=15,19)/
     +    0.04565,   0.00699,   0.00171,   0.00009,   0.00001,
     +    0.00000,   0.00002,   0.00007,
     +    0.04298,   0.00653,   0.00154,   0.00006,   0.00001,
     +    0.00000,   0.00002,   0.00007,
     +    0.04058,   0.00612,   0.00139,   0.00002,   0.00001,
     +    0.00000,   0.00002,   0.00007,
     +    0.03832,   0.00576,   0.00127,   0.00003,   0.00001,
     +    0.00000,   0.00002,   0.00006,
     +    0.03629,   0.00542,   0.00116,   0.00003,   0.00001,
     +    0.00000,   0.00002,   0.00005 /
        data ((f(i,42,k),i=1,5),f(7,42,k),f(6,42,k),f(8,42,k),k=1,7)/
     +    0.12459,   0.01917,   0.01314,   0.00001,   0.00001,
     +    0.00001,   0.00001,   0.00005,
     +    0.10362,   0.01561,   0.00898,   0.00008,   0.00001,
     +    0.00000,   0.00001,   0.00002,
     +    0.08808,   0.01306,   0.00644,   0.00006,   0.00001,
     +    0.00002,   0.00000,   0.00002,
     +    0.07623,   0.01111,   0.00480,   0.00003,   0.00001,
     +    0.00000,   0.00001,   0.00005,
     +    0.06687,   0.00962,   0.00370,   0.00001,   0.00000,
     +    0.00000,   0.00001,   0.00007,
     +    0.05939,   0.00844,   0.00292,   0.00001,   0.00000,
     +    0.00000,   0.00002,   0.00006,
     +    0.05313,   0.00748,   0.00236,   0.00001,   0.00001,
     +    0.00000,   0.00002,   0.00005 /
        data ((f(i,42,k),i=1,5),f(7,42,k),f(6,42,k),f(8,42,k),k=8,14)/
     +    0.04796,   0.00668,   0.00194,   0.00001,   0.00002,
     +    0.00000,   0.00002,   0.00001,
     +    0.04353,   0.00600,   0.00162,   0.00001,   0.00002,
     +    0.00000,   0.00002,   0.00002,
     +    0.03969,   0.00543,   0.00137,   0.00001,   0.00001,
     +    0.00000,   0.00002,   0.00004,
     +    0.03639,   0.00495,   0.00118,   0.00003,   0.00001,
     +    0.00000,   0.00002,   0.00002,
     +    0.03356,   0.00453,   0.00102,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00004,
     +    0.03104,   0.00417,   0.00089,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02881,   0.00383,   0.00079,   0.00002,   0.00001,
     +    0.00000,   0.00001,   0.00001 /
        data ((f(i,42,k),i=1,5),f(7,42,k),f(6,42,k),f(8,42,k),k=15,19)/
     +    0.02684,   0.00356,   0.00070,   0.00002,   0.00001,
     +    0.00001,   0.00002,   0.00001,
     +    0.02508,   0.00331,   0.00063,   0.00002,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02350,   0.00309,   0.00056,   0.00002,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02209,   0.00288,   0.00051,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02080,   0.00270,   0.00047,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001 /
        data ((f(i,43,k),i=1,5),f(7,43,k),f(6,43,k),f(8,43,k),k=1,7)/
     +    0.07957,   0.01048,   0.00596,   0.00004,   0.00001,
     +    0.00001,   0.00001,   0.00001,
     +    0.06469,   0.00834,   0.00393,   0.00007,   0.00001,
     +    0.00001,   0.00001,   0.00001,
     +    0.05390,   0.00684,   0.00274,   0.00003,   0.00001,
     +    0.00001,   0.00001,   0.00004,
     +    0.04590,   0.00574,   0.00200,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00003,
     +    0.03967,   0.00488,   0.00151,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.03470,   0.00422,   0.00117,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.03070,   0.00370,   0.00093,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001 /
        data ((f(i,43,k),i=1,5),f(7,43,k),f(6,43,k),f(8,43,k),k=8,14)/
     +    0.02737,   0.00326,   0.00076,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02458,   0.00292,   0.00062,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02222,   0.00261,   0.00052,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02021,   0.00236,   0.00044,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01843,   0.00213,   0.00038,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01690,   0.00194,   0.00033,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01559,   0.00179,   0.00029,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001 /
        data ((f(i,43,k),i=1,5),f(7,43,k),f(6,43,k),f(8,43,k),k=15,19)/
     +    0.01441,   0.00164,   0.00026,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01337,   0.00151,   0.00023,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01244,   0.00140,   0.00021,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01161,   0.00129,   0.00019,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01087,   0.00121,   0.00017,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001 /
        data ((f(i,44,k),i=1,5),f(7,44,k),f(6,44,k),f(8,44,k),k=1,7)/
     +    0.04536,   0.00498,   0.00231,   0.00001,   0.00001,
     +    0.00001,   0.00001,   0.00001,
     +    0.03601,   0.00389,   0.00147,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02947,   0.00313,   0.00100,   0.00000,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02460,   0.00257,   0.00071,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.02086,   0.00217,   0.00052,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.01798,   0.00184,   0.00040,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01568,   0.00160,   0.00031,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,44,k),i=1,5),f(7,44,k),f(6,44,k),f(8,44,k),k=8,14)/
     +    0.01382,   0.00139,   0.00025,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01226,   0.00123,   0.00021,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01096,   0.00109,   0.00017,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00987,   0.00097,   0.00014,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00894,   0.00087,   0.00012,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00812,   0.00079,   0.00011,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00741,   0.00071,   0.00009,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,44,k),i=1,5),f(7,44,k),f(6,44,k),f(8,44,k),k=15,19)/
     +    0.00680,   0.00066,   0.00008,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00627,   0.00060,   0.00007,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00579,   0.00055,   0.00007,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00536,   0.00051,   0.00006,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00498,   0.00047,   0.00005,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,45,k),i=1,5),f(7,45,k),f(6,45,k),f(8,45,k),k=1,7)/
     +    0.02210,   0.00199,   0.00072,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01702,   0.00150,   0.00044,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01355,   0.00117,   0.00029,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.01104,   0.00096,   0.00020,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00919,   0.00077,   0.00014,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00778,   0.00065,   0.00011,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00668,   0.00057,   0.00008,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,45,k),i=1,5),f(7,45,k),f(6,45,k),f(8,45,k),k=8,14)/
     +    0.00579,   0.00049,   0.00007,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00507,   0.00041,   0.00005,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00448,   0.00036,   0.00004,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00398,   0.00031,   0.00004,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00356,   0.00028,   0.00003,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00321,   0.00025,   0.00003,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00289,   0.00022,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,45,k),i=1,5),f(7,45,k),f(6,45,k),f(8,45,k),k=15,19)/
     +    0.00264,   0.00019,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00240,   0.00018,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00220,   0.00017,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00202,   0.00015,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00186,   0.00014,   0.00001,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,46,k),i=1,5),f(7,46,k),f(6,46,k),f(8,46,k),k=1,7)/
     +    0.00204,   0.00011,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00145,   0.00008,   0.00001,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00107,   0.00006,   0.00001,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00084,   0.00004,   0.00001,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00065,   0.00004,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00052,   0.00003,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00044,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,46,k),i=1,5),f(7,46,k),f(6,46,k),f(8,46,k),k=8,14)/
     +    0.00037,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00030,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00026,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00022,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00019,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00016,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00015,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,46,k),i=1,5),f(7,46,k),f(6,46,k),f(8,46,k),k=15,19)/
     +    0.00013,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00012,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00010,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00009,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00008,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
      data xx/1.d-5,2.d-5,4.d-5,6.d-5,8.d-5,
     .        1.d-4,2.d-4,4.d-4,6.d-4,8.d-4,
     .        1.d-3,2.d-3,4.d-3,6.d-3,8.d-3,
     .        1.d-2,2.d-2,4.d-2,6.d-2,8.d-2,
     .     .1d0,.125d0,.15d0,.175d0,.2d0,.225d0,.25d0,.275d0,
     .     .3d0,.325d0,.35d0,.375d0,.4d0,.425d0,.45d0,.475d0,
     .     .5d0,.525d0,.55d0,.575d0,.6d0,.65d0,.7d0,.75d0,
     .     .8d0,.9d0,1.d0/
      data xmin,xmax,qsqmin,qsqmax/1.d-5,1.d0,5.d0,1310720.d0/
      data n0/2,5,5,7,0,0,7,7/
      data zerod/0.d0/,poned/0.1d0/,oned/1.d0/,onedo/1.1d0/,twod/2.d0/
      data init/0/
c
      xsave=x
c
      if(init.ne.0) goto 10
      init=1
      do 20 n=1,nx-1
      do 20 m=1,19
c     read(55,50)f(1,n,m),f(2,n,m),f(3,n,m),f(4,n,m),f(5,n,m),f(7,n,m),
c    .          f(6,n,m),f(8,n,m)
c 1=uv 2=dv 3=glue 4=ubar 5=cbar 7=bbar 6=sbar 8=dbar
         do 25 i=1,8
  25     f(i,n,m)=f(i,n,m)/(oned-xx(n))**n0(i)
  20  continue
      do 31 j=1,ntenth-1
      xx(j)= log10(xx(j))+onedo
      do 31 i=1,8
      if(i.eq.7) go to 31
      do 30 k=1,19
  30  f(i,j,k)= log(f(i,j,k))*f(i,ntenth,k)/ log(f(i,ntenth,k))
  31  continue
  50  format(8f10.5)
      do 40 i=1,8
      do 40 m=1,19
  40  f(i,nx,m)=zerod
  10  continue
      if(x.lt.xmin) x=xmin
      if(x.gt.xmax) x=xmax
c furler 8.8.94
c   for compability with venus is scale = q2
c     qsq=scale**2
      qsq=scale
      if(qsq.lt.qsqmin) qsq=qsqmin
      if(qsq.gt.qsqmax) qsq=qsqmax
      xxx=x
      if(x.lt.poned) xxx= log10(x)+onedo
      n=0
  70  n=n+1
      if(xxx.gt.xx(n+1)) goto 70
      a=(xxx-xx(n))/(xx(n+1)-xx(n))
      rm= log(qsq/qsqmin)/ log(twod)
      b=rm-aint(rm)
      m=1+  int(rm)
      do 60 i=1,8
      g(i)=zerod
      if(xxx.ge.oned) goto 60
      g(i)= (oned-a)*(oned-b)*f(i,n,m)+(oned-a)*b*f(i,n,m+1)
     .    + a*(oned-b)*f(i,n+1,m)  + a*b*f(i,n+1,m+1)
      if(n.ge.ntenth) goto 65
      if(i.eq.7) goto 65
          fac=(oned-b)*f(i,ntenth,m)+b*f(i,ntenth,m+1)
          g(i)=fac**(g(i)/fac)
  65  continue
      g(i)=g(i)*(oned-x)**n0(i)
  60  continue
      upv=g(1)
      dnv=g(2)
      usea=g(4)
      dsea=g(8)
      str=g(6)
      chm=g(5)
      glu=g(3)
      bot=g(7)
c
      x=xsave
c
      return
      end

      subroutine sty2ms(x,scale,upv,dnv,usea,dsea,str,chm,bot,glu)
c                                                            
c     mrs(h) (a preliminary "hera" fit, oct. 1993)
c         to the recent h1 + zeus f2 data at small x,      
c         otherwise similar to d0', d-'. the exponent of the
c         gluon and sea at small x for this new set is -0.3  
c         (cf. -0.5 for d-')                                
c                                                            
c     10**-5 < x < 1  and  5 < q**2 < 1.31 * 10**6            
c                                                            
c   with ubar < dbar (230 mev)

      implicit double precision(a-h,o-z)
      parameter(nx=47)
      parameter(ntenth=21)
      dimension f(8,nx,20),g(8),xx(nx),n0(8)
        data ((f(i,1,k),i=1,5),f(7,1,k),f(6,1,k),f(8,1,k),k=1,7)/
     +    0.01399,   0.00189,  28.57361,   2.52574,   0.13922,
     +    0.00000,   1.35002,   2.55150,
     +    0.01450,   0.00205,  41.41998,   2.93569,   0.60391,
     +    0.00001,   1.78792,   2.96110,
     +    0.01497,   0.00221,  56.22984,   3.40128,   1.11643,
     +    0.00001,   2.27752,   3.42636,
     +    0.01541,   0.00235,  72.60197,   3.91488,   1.67105,
     +    0.00001,   2.81215,   3.93963,
     +    0.01581,   0.00249,  90.39349,   4.46961,   2.26217,
     +    0.12740,   3.38556,   4.49408,
     +    0.01618,   0.00263, 109.40785,   5.06167,   2.88696,
     +    0.69649,   3.99446,   5.08588,
     +    0.01654,   0.00276, 129.52321,   5.69024,   3.54540,
     +    1.39599,   4.63840,   5.71420 /
        data ((f(i,1,k),i=1,5),f(7,1,k),f(6,1,k),f(8,1,k),k=8,14)/
     +    0.01688,   0.00289, 150.59039,   6.35208,   4.23468,
     +    2.11291,   5.31439,   6.37581,
     +    0.01721,   0.00301, 172.47517,   7.04358,   4.95158,
     +    2.85689,   6.01900,   7.06710,
     +    0.01752,   0.00313, 195.07593,   7.76197,   5.69356,
     +    3.62358,   6.74958,   7.78529,
     +    0.01782,   0.00325, 218.32326,   8.50504,   6.45865,
     +    4.41183,   7.50404,   8.52820,
     +    0.01811,   0.00336, 242.10217,   9.27003,   7.24428,
     +    5.21909,   8.27972,   9.29304,
     +    0.01840,   0.00348, 266.14624,  10.05212,   8.04581,
     +    6.04096,   9.07188,  10.07497,
     +    0.01867,   0.00359, 290.66772,  10.85345,   8.86548,
     +    6.87977,   9.88272,  10.87617 /
        data ((f(i,1,k),i=1,5),f(7,1,k),f(6,1,k),f(8,1,k),k=15,19)/
     +    0.01893,   0.00370, 315.54395,  11.67175,   9.70111,
     +    7.73350,  10.71001,  11.69433,
     +    0.01919,   0.00380, 340.46313,  12.50028,  10.54612,
     +    8.59568,  11.54710,  12.52274,
     +    0.01944,   0.00391, 365.83740,  13.34612,  11.40759,
     +    9.47343,  12.40105,  13.36847,
     +    0.01968,   0.00401, 391.07251,  14.19815,  12.27457,
     +   10.35593,  13.26086,  14.22042,
     +    0.01992,   0.00411, 416.66797,  15.06506,  13.15572,
     +   11.25187,  14.13518,  15.08723 /
        data ((f(i,2,k),i=1,5),f(7,2,k),f(6,2,k),f(8,2,k),k=1,7)/
     +    0.01772,   0.00272,  23.12250,   2.05768,   0.11044,
     +    0.00000,   1.09384,   2.07248,
     +    0.01840,   0.00297,  33.11473,   2.37891,   0.47574,
     +    0.00000,   1.43741,   2.39348,
     +    0.01903,   0.00319,  44.40932,   2.73870,   0.87330,
     +    0.00001,   1.81637,   2.75305,
     +    0.01960,   0.00341,  56.67400,   3.13056,   1.29815,
     +    0.00000,   2.22501,   3.14471,
     +    0.02012,   0.00361,  69.79974,   3.54923,   1.74606,
     +    0.09624,   2.65856,   3.56320,
     +    0.02062,   0.00381,  83.63524,   3.99164,   2.21476,
     +    0.52392,   3.11437,   4.00544,
     +    0.02109,   0.00400,  98.08752,   4.45698,   2.70407,
     +    1.04547,   3.59193,   4.47063 /
        data ((f(i,2,k),i=1,5),f(7,2,k),f(6,2,k),f(8,2,k),k=8,14)/
     +    0.02154,   0.00418, 113.04889,   4.94272,   3.21183,
     +    1.57516,   4.08891,   4.95622,
     +    0.02196,   0.00436, 128.42447,   5.44617,   3.73564,
     +    2.12040,   4.60277,   5.45955,
     +    0.02237,   0.00453, 144.14647,   5.96530,   4.27367,
     +    2.67801,   5.13156,   5.97855,
     +    0.02276,   0.00470, 160.17076,   6.49851,   4.82451,
     +    3.24717,   5.67379,   6.51165,
     +    0.02314,   0.00487, 176.41954,   7.04381,   5.38634,
     +    3.82612,   6.22756,   7.05686,
     +    0.02351,   0.00503, 192.70801,   7.59759,   5.95567,
     +    4.41154,   6.78931,   7.61054,
     +    0.02386,   0.00518, 209.19713,   8.16174,   6.53449,
     +    5.00551,   7.36098,   8.17461 /
        data ((f(i,2,k),i=1,5),f(7,2,k),f(6,2,k),f(8,2,k),k=15,19)/
     +    0.02420,   0.00534, 225.80484,   8.73462,   7.12124,
     +    5.60657,   7.94098,   8.74743,
     +    0.02453,   0.00549, 242.31677,   9.31133,   7.71111,
     +    6.21002,   8.52444,   9.32406,
     +    0.02486,   0.00563, 259.03003,   9.89730,   8.30958,
     +    6.82136,   9.11683,   9.91000,
     +    0.02517,   0.00578, 275.53491,  10.48433,   8.90855,
     +    7.43260,   9.70999,  10.49697,
     +    0.02548,   0.00592, 292.18457,  11.07906,   9.51465,
     +    8.05038,  10.31057,  11.09164 /
        data ((f(i,3,k),i=1,5),f(7,3,k),f(6,3,k),f(8,3,k),k=1,7)/
     +    0.02241,   0.00393,  18.71066,   1.67536,   0.08754,
     +    0.00000,   0.88625,   1.68330,
     +    0.02331,   0.00428,  26.47142,   1.92669,   0.37440,
     +    0.00000,   1.15548,   1.93450,
     +    0.02412,   0.00462,  35.06683,   2.20416,   0.68230,
     +    0.00000,   1.44829,   2.21187,
     +    0.02487,   0.00493,  44.22931,   2.50242,   1.00712,
     +    0.00000,   1.75995,   2.51003,
     +    0.02556,   0.00523,  53.88190,   2.81753,   1.34576,
     +    0.07253,   2.08694,   2.82505,
     +    0.02621,   0.00552,  63.91248,   3.14711,   1.69647,
     +    0.39322,   2.42720,   3.15454,
     +    0.02682,   0.00579,  74.25426,   3.49047,   2.05910,
     +    0.78114,   2.78030,   3.49783 /
        data ((f(i,3,k),i=1,5),f(7,3,k),f(6,3,k),f(8,3,k),k=8,14)/
     +    0.02740,   0.00606,  84.83257,   3.84571,   2.43203,
     +    1.17147,   3.14449,   3.85301,
     +    0.02796,   0.00631,  95.58434,   4.21089,   2.81355,
     +    1.56995,   3.51794,   4.21813,
     +    0.02849,   0.00656, 106.46631,   4.58455,   3.20237,
     +    1.97427,   3.89928,   4.59172,
     +    0.02900,   0.00680, 117.45285,   4.96560,   3.59757,
     +    2.38397,   4.28750,   4.97272,
     +    0.02949,   0.00704, 128.49417,   5.35265,   3.99787,
     +    2.79781,   4.68126,   5.35973,
     +    0.02996,   0.00727, 139.46341,   5.74303,   4.40071,
     +    3.21339,   5.07796,   5.75007,
     +    0.03042,   0.00749, 150.48344,   6.13838,   4.80782,
     +    3.63247,   5.47927,   6.14539 /
        data ((f(i,3,k),i=1,5),f(7,3,k),f(6,3,k),f(8,3,k),k=15,19)/
     +    0.03086,   0.00771, 161.50014,   6.53758,   5.21812,
     +    4.05410,   5.88411,   6.54456,
     +    0.03129,   0.00792, 172.36797,   6.93704,   5.62812,
     +    4.47482,   6.28891,   6.94399,
     +    0.03170,   0.00813, 183.30180,   7.34098,   6.04206,
     +    4.89894,   6.69793,   7.34790,
     +    0.03211,   0.00833, 194.01912,   7.74337,   6.45399,
     +    5.32057,   7.10517,   7.75027,
     +    0.03250,   0.00853, 204.77129,   8.14922,   6.86895,
     +    5.74476,   7.51565,   8.15611 /
        data ((f(i,4,k),i=1,5),f(7,4,k),f(6,4,k),f(8,4,k),k=1,7)/
     +    0.02568,   0.00487,  16.53067,   1.48499,   0.07637,
     +    0.00000,   0.78360,   1.49042,
     +    0.02673,   0.00532,  23.21927,   1.70253,   0.32526,
     +    0.00000,   1.01687,   1.70789,
     +    0.02769,   0.00574,  30.53673,   1.94063,   0.59019,
     +    0.00000,   1.26845,   1.94593,
     +    0.02856,   0.00613,  38.25038,   2.19453,   0.86751,
     +    0.00000,   1.53412,   2.19979,
     +    0.02937,   0.00650,  46.30000,   2.46097,   1.15468,
     +    0.06140,   1.81099,   2.46619,
     +    0.03012,   0.00686,  54.59346,   2.73794,   1.45028,
     +    0.33205,   2.09733,   2.74312,
     +    0.03084,   0.00720,  63.07721,   3.02485,   1.75415,
     +    0.65786,   2.39277,   3.02999 /
        data ((f(i,4,k),i=1,5),f(7,4,k),f(6,4,k),f(8,4,k),k=8,14)/
     +    0.03152,   0.00753,  71.69281,   3.32010,   2.06498,
     +    0.98389,   2.69587,   3.32522,
     +    0.03216,   0.00784,  80.39113,   3.62211,   2.38137,
     +    1.31506,   3.00512,   3.62720,
     +    0.03278,   0.00815,  89.14072,   3.92971,   2.70232,
     +    1.64953,   3.31945,   3.93478,
     +    0.03337,   0.00845,  97.92462,   4.24206,   3.02712,
     +    1.98698,   3.63808,   4.24712,
     +    0.03394,   0.00874, 106.70447,   4.55803,   3.35474,
     +    2.32641,   3.95992,   4.56306,
     +    0.03449,   0.00902, 115.37933,   4.87541,   3.68309,
     +    2.66585,   4.28283,   4.88043,
     +    0.03502,   0.00929, 124.05481,   5.19573,   4.01373,
     +    3.00694,   4.60835,   5.20074 /
        data ((f(i,4,k),i=1,5),f(7,4,k),f(6,4,k),f(8,4,k),k=15,19)/
     +    0.03553,   0.00955, 132.68863,   5.51803,   4.34580,
     +    3.34887,   4.93558,   5.52303,
     +    0.03603,   0.00981, 141.16512,   5.83940,   4.67643,
     +    3.68886,   5.26162,   5.84439,
     +    0.03651,   0.01007, 149.66237,   6.16345,   5.00927,
     +    4.03056,   5.59011,   6.16844,
     +    0.03698,   0.01031, 157.95305,   6.48515,   5.33936,
     +    4.36910,   5.91604,   6.49013,
     +    0.03743,   0.01055, 166.24387,   6.80877,   5.67098,
     +    4.70876,   6.24370,   6.81376 /
        data ((f(i,5,k),i=1,5),f(7,5,k),f(6,5,k),f(8,5,k),k=1,7)/
     +    0.02828,   0.00567,  15.13950,   1.36292,   0.06931,
     +    0.00000,   0.71806,   1.36711,
     +    0.02945,   0.00620,  21.15556,   1.55918,   0.29426,
     +    0.00000,   0.92870,   1.56335,
     +    0.03052,   0.00669,  27.67900,   1.77264,   0.53228,
     +    0.00000,   1.15448,   1.77679,
     +    0.03149,   0.00715,  34.50014,   1.99898,   0.78004,
     +    0.00000,   1.39155,   2.00311,
     +    0.03239,   0.00759,  41.56969,   2.23533,   1.03536,
     +    0.05451,   1.63742,   2.23946,
     +    0.03323,   0.00800,  48.80835,   2.47993,   1.29699,
     +    0.29431,   1.89057,   2.48405,
     +    0.03403,   0.00840,  56.17104,   2.73226,   1.56483,
     +    0.58199,   2.15068,   2.73637 /
        data ((f(i,5,k),i=1,5),f(7,5,k),f(6,5,k),f(8,5,k),k=8,14)/
     +    0.03478,   0.00878,  63.60899,   2.99093,   1.83774,
     +    0.86870,   2.41650,   2.99504,
     +    0.03550,   0.00915,  71.08234,   3.25458,   2.11453,
     +    1.15891,   2.68675,   3.25868,
     +    0.03619,   0.00950,  78.56602,   3.52221,   2.39437,
     +    1.45101,   2.96051,   3.52631,
     +    0.03685,   0.00985,  86.04810,   3.79315,   2.67668,
     +    1.74479,   3.23716,   3.79725,
     +    0.03748,   0.01018,  93.49756,   4.06641,   2.96060,
     +    2.03942,   3.51578,   4.07052,
     +    0.03809,   0.01051, 100.82874,   4.34007,   3.24427,
     +    2.33317,   3.79447,   4.34419,
     +    0.03868,   0.01082, 108.13600,   4.61557,   3.52920,
     +    2.62757,   4.07471,   4.61969 /
        data ((f(i,5,k),i=1,5),f(7,5,k),f(6,5,k),f(8,5,k),k=15,19)/
     +    0.03925,   0.01113, 115.38472,   4.89210,   3.81465,
     +    2.92197,   4.35572,   4.89622,
     +    0.03980,   0.01143, 122.47620,   5.16710,   4.09811,
     +    3.21392,   4.63498,   5.17123,
     +    0.04033,   0.01172, 129.56682,   5.44385,   4.38289,
     +    3.50673,   4.91576,   5.44800,
     +    0.04085,   0.01200, 136.46161,   5.71789,   4.66459,
     +    3.79609,   5.19365,   5.72204,
     +    0.04136,   0.01228, 143.34029,   5.99307,   4.94706,
     +    4.08586,   5.47249,   5.99723 /
        data ((f(i,6,k),i=1,5),f(7,6,k),f(6,6,k),f(8,6,k),k=1,7)/
     +    0.03046,   0.00638,  14.14132,   1.27502,   0.06427,
     +    0.00000,   0.67103,   1.27853,
     +    0.03174,   0.00698,  19.68086,   1.45616,   0.27221,
     +    0.00000,   0.86559,   1.45968,
     +    0.03290,   0.00754,  25.64561,   1.65221,   0.49119,
     +    0.00000,   1.07312,   1.65574,
     +    0.03396,   0.00806,  31.84277,   1.85914,   0.71812,
     +    0.00000,   1.29006,   1.86268,
     +    0.03494,   0.00856,  38.23102,   2.07441,   0.95110,
     +    0.04969,   1.51420,   2.07796,
     +    0.03585,   0.00902,  44.74008,   2.29640,   1.18899,
     +    0.26791,   1.74416,   2.29996,
     +    0.03672,   0.00947,  51.33093,   2.52466,   1.43172,
     +    0.52899,   1.97967,   2.52823 /
        data ((f(i,6,k),i=1,5),f(7,6,k),f(6,6,k),f(8,6,k),k=8,14)/
     +    0.03754,   0.00990,  57.96184,   2.75795,   1.67830,
     +    0.78837,   2.21962,   2.76154,
     +    0.03832,   0.01031,  64.59860,   2.99506,   1.92768,
     +    1.05019,   2.46288,   2.99866,
     +    0.03907,   0.01071,  71.22144,   3.23513,   2.17913,
     +    1.31302,   2.70865,   3.23875,
     +    0.03978,   0.01110,  77.82121,   3.47757,   2.43218,
     +    1.57670,   2.95641,   3.48120,
     +    0.04047,   0.01147,  84.37195,   3.72152,   2.68607,
     +    1.84053,   3.20534,   3.72517,
     +    0.04113,   0.01183,  90.79819,   3.96526,   2.93915,
     +    2.10294,   3.45376,   3.96893,
     +    0.04177,   0.01218,  97.18675,   4.21015,   3.19283,
     +    2.36541,   3.70306,   4.21383 /
        data ((f(i,6,k),i=1,5),f(7,6,k),f(6,6,k),f(8,6,k),k=15,19)/
     +    0.04238,   0.01253, 103.50786,   4.45548,   3.44648,
     +    2.62735,   3.95256,   4.45918,
     +    0.04298,   0.01286, 109.67487,   4.69895,   3.69784,
     +    2.88658,   4.19999,   4.70267,
     +    0.04356,   0.01318, 115.82816,   4.94358,   3.94995,
     +    3.14614,   4.44836,   4.94731,
     +    0.04412,   0.01350, 121.79561,   5.18534,   4.19884,
     +    3.40214,   4.69370,   5.18908,
     +    0.04466,   0.01381, 127.73788,   5.42774,   4.44805,
     +    3.65810,   4.93951,   5.43150 /
        data ((f(i,7,k),i=1,5),f(7,7,k),f(6,7,k),f(8,7,k),k=1,7)/
     +    0.03836,   0.00925,  11.43973,   1.03567,   0.05078,
     +    0.00000,   0.54373,   1.03836,
     +    0.04001,   0.01013,  15.71670,   1.17658,   0.21340,
     +    0.00000,   0.69549,   1.17936,
     +    0.04151,   0.01096,  20.21892,   1.32668,   0.38207,
     +    0.00000,   0.85488,   1.32954,
     +    0.04288,   0.01172,  24.80116,   1.48282,   0.55440,
     +    0.00000,   1.01910,   1.48575,
     +    0.04415,   0.01243,  29.44316,   1.64326,   0.72917,
     +    0.03716,   1.18670,   1.64626,
     +    0.04533,   0.01311,  34.09891,   1.80686,   0.90564,
     +    0.19950,   1.35673,   1.80993,
     +    0.04644,   0.01375,  38.74529,   1.97333,   1.08382,
     +    0.39205,   1.52904,   1.97646 /
        data ((f(i,7,k),i=1,5),f(7,7,k),f(6,7,k),f(8,7,k),k=8,14)/
     +    0.04750,   0.01437,  43.35803,   2.14183,   1.26306,
     +    0.58144,   1.70291,   2.14501,
     +    0.04850,   0.01496,  47.91843,   2.31155,   1.44269,
     +    0.77090,   1.87757,   2.31479,
     +    0.04946,   0.01553,  52.41753,   2.48195,   1.62230,
     +    0.95950,   2.05258,   2.48525,
     +    0.05038,   0.01608,  56.85410,   2.65270,   1.80163,
     +    1.14723,   2.22761,   2.65605,
     +    0.05126,   0.01660,  61.21371,   2.82325,   1.98020,
     +    1.33365,   2.40216,   2.82664,
     +    0.05211,   0.01712,  65.44678,   2.99236,   2.15687,
     +    1.51769,   2.57504,   2.99581,
     +    0.05293,   0.01761,  69.61964,   3.16121,   2.33282,
     +    1.70058,   2.74743,   3.16469 /
        data ((f(i,7,k),i=1,5),f(7,7,k),f(6,7,k),f(8,7,k),k=15,19)/
     +    0.05372,   0.01809,  73.71400,   3.32931,   2.50763,
     +    1.88195,   2.91888,   3.33283,
     +    0.05448,   0.01856,  77.67226,   3.49504,   2.67973,
     +    2.06026,   3.08778,   3.49861,
     +    0.05522,   0.01901,  81.59584,   3.66070,   2.85144,
     +    2.23785,   3.25646,   3.66431,
     +    0.05594,   0.01945,  85.36723,   3.82339,   3.01989,
     +    2.41191,   3.42202,   3.82704,
     +    0.05664,   0.01989,  89.10036,   3.98575,   3.18774,
     +    2.58510,   3.58712,   3.98944 /
        data ((f(i,8,k),i=1,5),f(7,8,k),f(6,8,k),f(8,8,k),k=1,7)/
     +    0.04826,   0.01348,   9.25090,   0.84012,   0.04000,
     +    0.00000,   0.44066,   0.84345,
     +    0.05039,   0.01479,  12.53508,   0.94920,   0.16672,
     +    0.00000,   0.55857,   0.95268,
     +    0.05231,   0.01599,  15.90942,   1.06346,   0.29606,
     +    0.00000,   0.68038,   1.06705,
     +    0.05408,   0.01710,  19.26860,   1.18047,   0.42623,
     +    0.00000,   0.80397,   1.18418,
     +    0.05570,   0.01813,  22.60866,   1.29914,   0.55655,
     +    0.02762,   0.92847,   1.30295,
     +    0.05722,   0.01910,  25.90247,   1.41871,   0.68657,
     +    0.14764,   1.05327,   1.42263,
     +    0.05865,   0.02003,  29.13887,   1.53904,   0.81641,
     +    0.28871,   1.17835,   1.54304 /
        data ((f(i,8,k),i=1,5),f(7,8,k),f(6,8,k),f(8,8,k),k=8,14)/
     +    0.06001,   0.02090,  32.30612,   1.65958,   0.94568,
     +    0.42601,   1.30325,   1.66366,
     +    0.06130,   0.02174,  35.39610,   1.77983,   1.07400,
     +    0.56207,   1.42754,   1.78400,
     +    0.06253,   0.02255,  38.40730,   1.89951,   1.20115,
     +    0.69633,   1.55095,   1.90375,
     +    0.06371,   0.02332,  41.34314,   2.01844,   1.32706,
     +    0.82886,   1.67337,   2.02275,
     +    0.06483,   0.02407,  44.19688,   2.13630,   1.45144,
     +    0.95943,   1.79448,   2.14068,
     +    0.06592,   0.02479,  46.93665,   2.25223,   1.57350,
     +    1.08731,   1.91347,   2.25667,
     +    0.06696,   0.02548,  49.61301,   2.36720,   1.69425,
     +    1.21354,   2.03133,   2.37170 /
        data ((f(i,8,k),i=1,5),f(7,8,k),f(6,8,k),f(8,8,k),k=15,19)/
     +    0.06797,   0.02615,  52.21521,   2.48092,   1.81343,
     +    1.33788,   2.14777,   2.48548,
     +    0.06894,   0.02680,  54.70546,   2.59223,   1.92992,
     +    1.45927,   2.26167,   2.59685,
     +    0.06988,   0.02743,  57.15674,   2.70292,   2.04552,
     +    1.57950,   2.37481,   2.70758,
     +    0.07080,   0.02804,  59.48952,   2.81086,   2.15814,
     +    1.69655,   2.48508,   2.81557,
     +    0.07168,   0.02863,  61.78384,   2.91807,   2.26982,
     +    1.81243,   2.59452,   2.92283 /
        data ((f(i,9,k),i=1,5),f(7,9,k),f(6,9,k),f(8,9,k),k=1,7)/
     +    0.05520,   0.01689,   8.16771,   0.74291,   0.03471,
     +    0.00000,   0.38972,   0.74703,
     +    0.05768,   0.01852,  10.97002,   0.83649,   0.14393,
     +    0.00000,   0.49116,   0.84078,
     +    0.05991,   0.02002,  13.80556,   0.93348,   0.25430,
     +    0.00000,   0.59486,   0.93790,
     +    0.06195,   0.02139,  16.58923,   1.03184,   0.36434,
     +    0.00000,   0.69908,   1.03639,
     +    0.06383,   0.02267,  19.32477,   1.13080,   0.47362,
     +    0.02313,   0.80322,   1.13545,
     +    0.06559,   0.02387,  21.99377,   1.22976,   0.58186,
     +    0.12326,   0.90683,   1.23452,
     +    0.06725,   0.02500,  24.59062,   1.32865,   0.68919,
     +    0.24031,   1.00996,   1.33351 /
        data ((f(i,9,k),i=1,5),f(7,9,k),f(6,9,k),f(8,9,k),k=8,14)/
     +    0.06881,   0.02608,  27.10899,   1.42708,   0.79538,
     +    0.35349,   1.11228,   1.43203,
     +    0.07030,   0.02711,  29.54529,   1.52470,   0.90015,
     +    0.46499,   1.21348,   1.52973,
     +    0.07172,   0.02809,  31.90111,   1.62131,   1.00340,
     +    0.57442,   1.31341,   1.62642,
     +    0.07308,   0.02903,  34.18138,   1.71683,   1.10511,
     +    0.68189,   1.41204,   1.72201,
     +    0.07438,   0.02994,  36.38255,   1.81102,   1.20510,
     +    0.78726,   1.50913,   1.81626,
     +    0.07563,   0.03081,  38.48053,   1.90320,   1.30273,
     +    0.88994,   1.60404,   1.90851,
     +    0.07684,   0.03164,  40.51802,   1.99423,   1.39890,
     +    0.99087,   1.69765,   1.99961 /
        data ((f(i,9,k),i=1,5),f(7,9,k),f(6,9,k),f(8,9,k),k=15,19)/
     +    0.07800,   0.03245,  42.48756,   2.08391,   1.49343,
     +    1.08989,   1.78975,   2.08934,
     +    0.07912,   0.03323,  44.35997,   2.17129,   1.58541,
     +    1.18613,   1.87944,   2.17679,
     +    0.08020,   0.03399,  46.19490,   2.25790,   1.67639,
     +    1.28114,   1.96824,   2.26345,
     +    0.08125,   0.03472,  47.92944,   2.34199,   1.76465,
     +    1.37323,   2.05441,   2.34759,
     +    0.08226,   0.03544,  49.62872,   2.42527,   1.85189,
     +    1.46414,   2.13967,   2.43091 /
        data ((f(i,10,k),i=1,5),f(7,10,k),f(6,10,k),f(8,10,k),k=1,7)/
     +    0.06076,   0.01989,   7.47539,   0.68073,   0.03133,
     +    0.00000,   0.35722,   0.68552,
     +    0.06351,   0.02180,   9.97223,   0.76448,   0.12944,
     +    0.00000,   0.44819,   0.76942,
     +    0.06599,   0.02355,  12.46965,   0.85058,   0.22784,
     +    0.00000,   0.54049,   0.85567,
     +    0.06826,   0.02514,  14.89576,   0.93728,   0.32526,
     +    0.00000,   0.63257,   0.94249,
     +    0.07035,   0.02662,  17.25882,   1.02396,   0.42144,
     +    0.02033,   0.72403,   1.02929,
     +    0.07230,   0.02801,  19.54593,   1.11017,   0.51617,
     +    0.10814,   0.81452,   1.11560,
     +    0.07414,   0.02932,  21.75473,   1.19587,   0.60964,
     +    0.21035,   0.90413,   1.20139 /
        data ((f(i,10,k),i=1,5),f(7,10,k),f(6,10,k),f(8,10,k),k=8,14)/
     +    0.07588,   0.03056,  23.88213,   1.28076,   0.70166,
     +    0.30870,   0.99260,   1.28637,
     +    0.07753,   0.03174,  25.92714,   1.36457,   0.79206,
     +    0.40519,   1.07972,   1.37026,
     +    0.07910,   0.03286,  27.89279,   1.44717,   0.88077,
     +    0.49949,   1.16539,   1.45294,
     +    0.08061,   0.03394,  29.78505,   1.52853,   0.96782,
     +    0.59175,   1.24961,   1.53437,
     +    0.08204,   0.03497,  31.60204,   1.60846,   1.05309,
     +    0.68189,   1.33222,   1.61436,
     +    0.08343,   0.03596,  33.32407,   1.68638,   1.13603,
     +    0.76940,   1.41267,   1.69235,
     +    0.08476,   0.03692,  34.98917,   1.76310,   1.21748,
     +    0.85516,   1.49177,   1.76913 /
        data ((f(i,10,k),i=1,5),f(7,10,k),f(6,10,k),f(8,10,k),k=15,19)/
     +    0.08604,   0.03784,  36.59158,   1.83844,   1.29729,
     +    0.93903,   1.56935,   1.84453,
     +    0.08728,   0.03873,  38.10712,   1.91161,   1.37470,
     +    1.02028,   1.64464,   1.91775,
     +    0.08847,   0.03959,  39.58728,   1.98395,   1.45107,
     +    1.10029,   1.71901,   1.99014,
     +    0.08963,   0.04041,  40.97926,   2.05395,   1.52491,
     +    1.17760,   1.79094,   2.06019,
     +    0.09075,   0.04122,  42.33871,   2.12312,   1.59774,
     +    1.25374,   1.86195,   2.12941 /
        data ((f(i,11,k),i=1,5),f(7,11,k),f(6,11,k),f(8,11,k),k=1,7)/
     +    0.06552,   0.02273,   6.97721,   0.63640,   0.02887,
     +    0.00000,   0.33384,   0.64150,
     +    0.06853,   0.02489,   9.25154,   0.71299,   0.11890,
     +    0.00000,   0.41719,   0.71826,
     +    0.07124,   0.02685,  11.50403,   0.79122,   0.20863,
     +    0.00000,   0.50121,   0.79665,
     +    0.07371,   0.02863,  13.67303,   0.86953,   0.29696,
     +    0.00002,   0.58456,   0.87509,
     +    0.07599,   0.03029,  15.77015,   0.94740,   0.38372,
     +    0.01834,   0.66690,   0.95309,
     +    0.07811,   0.03183,  17.78607,   1.02450,   0.46879,
     +    0.09732,   0.74801,   1.03030,
     +    0.08011,   0.03329,  19.72060,   1.10081,   0.55236,
     +    0.18896,   0.82796,   1.10670 /
        data ((f(i,11,k),i=1,5),f(7,11,k),f(6,11,k),f(8,11,k),k=8,14)/
     +    0.08200,   0.03466,  21.57309,   1.17610,   0.63432,
     +    0.27677,   0.90661,   1.18209,
     +    0.08380,   0.03597,  23.34392,   1.25015,   0.71453,
     +    0.36262,   0.98375,   1.25623,
     +    0.08550,   0.03721,  25.03760,   1.32287,   0.79298,
     +    0.44622,   1.05935,   1.32903,
     +    0.08713,   0.03840,  26.66037,   1.39427,   0.86971,
     +    0.52777,   1.13343,   1.40051,
     +    0.08869,   0.03954,  28.21049,   1.46420,   0.94464,
     +    0.60720,   1.20587,   1.47050,
     +    0.09019,   0.04063,  29.67432,   1.53217,   1.01730,
     +    0.68409,   1.27620,   1.53854,
     +    0.09163,   0.04168,  31.08479,   1.59892,   1.08848,
     +    0.75924,   1.34517,   1.60535 /
        data ((f(i,11,k),i=1,5),f(7,11,k),f(6,11,k),f(8,11,k),k=15,19)/
     +    0.09302,   0.04269,  32.43724,   1.66431,   1.15805,
     +    0.83256,   1.41266,   1.67080,
     +    0.09436,   0.04366,  33.70975,   1.72762,   1.22534,
     +    0.90340,   1.47796,   1.73417,
     +    0.09565,   0.04460,  34.94885,   1.79010,   1.29159,
     +    0.97302,   1.54234,   1.79670,
     +    0.09690,   0.04551,  36.10852,   1.85038,   1.35546,
     +    1.04010,   1.60442,   1.85703,
     +    0.09811,   0.04639,  37.23849,   1.90983,   1.41834,
     +    1.10603,   1.66559,   1.91652 /
        data ((f(i,12,k),i=1,5),f(7,12,k),f(6,12,k),f(8,12,k),k=1,7)/
     +    0.08306,   0.03468,   5.62636,   0.51597,   0.02225,
     +    0.00000,   0.27072,   0.52239,
     +    0.08703,   0.03776,   7.30762,   0.57341,   0.09065,
     +    0.00000,   0.33370,   0.58003,
     +    0.09059,   0.04052,   8.91820,   0.63079,   0.15743,
     +    0.00000,   0.39583,   0.63757,
     +    0.09384,   0.04302,  10.42288,   0.68708,   0.22191,
     +    0.00001,   0.45625,   0.69400,
     +    0.09682,   0.04531,  11.84112,   0.74211,   0.28420,
     +    0.01314,   0.51495,   0.74915,
     +    0.09958,   0.04743,  13.17321,   0.79574,   0.34436,
     +    0.06936,   0.57187,   0.80289,
     +    0.10218,   0.04942,  14.42432,   0.84804,   0.40262,
     +    0.13384,   0.62718,   0.85529 /
        data ((f(i,12,k),i=1,5),f(7,12,k),f(6,12,k),f(8,12,k),k=8,14)/
     +    0.10463,   0.05128,  15.59858,   0.89896,   0.45901,
     +    0.19482,   0.68086,   0.90630,
     +    0.10695,   0.05304,  16.70023,   0.94841,   0.51352,
     +    0.25373,   0.73286,   0.95584,
     +    0.10916,   0.05470,  17.73563,   0.99642,   0.56622,
     +    0.31046,   0.78323,   1.00392,
     +    0.11125,   0.05629,  18.71164,   1.04304,   0.61723,
     +    0.36522,   0.83206,   1.05061,
     +    0.11325,   0.05780,  19.62926,   1.08822,   0.66653,
     +    0.41804,   0.87932,   1.09586,
     +    0.11517,   0.05923,  20.48106,   1.13167,   0.71384,
     +    0.46864,   0.92471,   1.13936,
     +    0.11701,   0.06060,  21.29117,   1.17396,   0.75978,
     +    0.51769,   0.96885,   1.18172 /
        data ((f(i,12,k),i=1,5),f(7,12,k),f(6,12,k),f(8,12,k),k=15,19)/
     +    0.11877,   0.06192,  22.05750,   1.21503,   0.80430,
     +    0.56513,   1.01165,   1.22284,
     +    0.12047,   0.06317,  22.76675,   1.25441,   0.84696,
     +    0.61056,   1.05268,   1.26227,
     +    0.12211,   0.06439,  23.45059,   1.29300,   0.88867,
     +    0.65489,   1.09284,   1.30090,
     +    0.12369,   0.06555,  24.07961,   1.32988,   0.92851,
     +    0.69723,   1.13121,   1.33782,
     +    0.12521,   0.06668,  24.68698,   1.36602,   0.96749,
     +    0.73859,   1.16878,   1.37400 /
        data ((f(i,13,k),i=1,5),f(7,13,k),f(6,13,k),f(8,13,k),k=1,7)/
     +    0.10659,   0.05365,   4.52734,   0.41874,   0.01687,
     +    0.00002,   0.21978,   0.42669,
     +    0.11190,   0.05787,   5.73271,   0.46075,   0.06788,
     +    0.00001,   0.26634,   0.46889,
     +    0.11665,   0.06161,   6.84270,   0.50165,   0.11652,
     +    0.00002,   0.31116,   0.50994,
     +    0.12094,   0.06495,   7.84306,   0.54081,   0.16243,
     +    0.00002,   0.35373,   0.54924,
     +    0.12486,   0.06799,   8.75681,   0.57835,   0.20596,
     +    0.00920,   0.39430,   0.58687,
     +    0.12847,   0.07078,   9.59102,   0.61424,   0.24725,
     +    0.04814,   0.43292,   0.62286,
     +    0.13186,   0.07335,  10.35507,   0.64866,   0.28662,
     +    0.09226,   0.46984,   0.65737 /
        data ((f(i,13,k),i=1,5),f(7,13,k),f(6,13,k),f(8,13,k),k=8,14)/
     +    0.13503,   0.07575,  11.05375,   0.68165,   0.32415,
     +    0.13337,   0.50511,   0.69044,
     +    0.13802,   0.07799,  11.69351,   0.71321,   0.35990,
     +    0.17253,   0.53878,   0.72205,
     +    0.14084,   0.08010,  12.28241,   0.74340,   0.39402,
     +    0.20976,   0.57095,   0.75230,
     +    0.14352,   0.08209,  12.82666,   0.77236,   0.42664,
     +    0.24529,   0.60176,   0.78131,
     +    0.14606,   0.08397,  13.32714,   0.80010,   0.45781,
     +    0.27918,   0.63123,   0.80909,
     +    0.14848,   0.08575,  13.78036,   0.82638,   0.48732,
     +    0.31125,   0.65913,   0.83542,
     +    0.15080,   0.08744,  14.20426,   0.85170,   0.51569,
     +    0.34202,   0.68599,   0.86078 /
        data ((f(i,13,k),i=1,5),f(7,13,k),f(6,13,k),f(8,13,k),k=15,19)/
     +    0.15302,   0.08905,  14.59854,   0.87603,   0.54292,
     +    0.37151,   0.71178,   0.88515,
     +    0.15514,   0.09057,  14.95536,   0.89911,   0.56875,
     +    0.39948,   0.73624,   0.90825,
     +    0.15718,   0.09203,  15.29492,   0.92154,   0.59380,
     +    0.42656,   0.75999,   0.93072,
     +    0.15913,   0.09343,  15.59800,   0.94268,   0.61744,
     +    0.45213,   0.78238,   0.95189,
     +    0.16102,   0.09478,  15.88724,   0.96326,   0.64040,
     +    0.47693,   0.80416,   0.97249 /
        data ((f(i,14,k),i=1,5),f(7,14,k),f(6,14,k),f(8,14,k),k=1,7)/
     +    0.12441,   0.06939,   3.98165,   0.37079,   0.01422,
     +    0.00002,   0.19472,   0.37969,
     +    0.13072,   0.07435,   4.95415,   0.40520,   0.05672,
     +    0.00003,   0.23323,   0.41426,
     +    0.13632,   0.07871,   5.82556,   0.43811,   0.09659,
     +    0.00003,   0.26967,   0.44729,
     +    0.14137,   0.08255,   6.59158,   0.46912,   0.13371,
     +    0.00003,   0.30376,   0.47841,
     +    0.14597,   0.08601,   7.27642,   0.49842,   0.16845,
     +    0.00735,   0.33581,   0.50780,
     +    0.15019,   0.08916,   7.89017,   0.52610,   0.20103,
     +    0.03830,   0.36597,   0.53556,
     +    0.15410,   0.09206,   8.44100,   0.55235,   0.23176,
     +    0.07307,   0.39448,   0.56186 /
        data ((f(i,14,k),i=1,5),f(7,14,k),f(6,14,k),f(8,14,k),k=8,14)/
     +    0.15777,   0.09475,   8.93561,   0.57718,   0.26072,
     +    0.10512,   0.42139,   0.58676,
     +    0.16120,   0.09724,   9.38150,   0.60070,   0.28808,
     +    0.13539,   0.44683,   0.61033,
     +    0.16444,   0.09956,   9.78434,   0.62302,   0.31396,
     +    0.16395,   0.47095,   0.63268,
     +    0.16751,   0.10174,  10.15014,   0.64422,   0.33851,
     +    0.19100,   0.49384,   0.65392,
     +    0.17040,   0.10379,  10.48254,   0.66433,   0.36176,
     +    0.21658,   0.51553,   0.67406,
     +    0.17314,   0.10571,  10.77755,   0.68325,   0.38363,
     +    0.24065,   0.53593,   0.69301,
     +    0.17575,   0.10753,  11.04940,   0.70133,   0.40451,
     +    0.26358,   0.55541,   0.71111 /
        data ((f(i,14,k),i=1,5),f(7,14,k),f(6,14,k),f(8,14,k),k=15,19)/
     +    0.17823,   0.10927,  11.29810,   0.71857,   0.42439,
     +    0.28540,   0.57398,   0.72835,
     +    0.18061,   0.11090,  11.51828,   0.73473,   0.44308,
     +    0.30592,   0.59141,   0.74454,
     +    0.18289,   0.11247,  11.72573,   0.75036,   0.46112,
     +    0.32569,   0.60825,   0.76019,
     +    0.18507,   0.11393,  11.90742,   0.76496,   0.47802,
     +    0.34424,   0.62401,   0.77481,
     +    0.18718,   0.11535,  12.07889,   0.77911,   0.49436,
     +    0.36216,   0.63925,   0.78896 /
        data ((f(i,15,k),i=1,5),f(7,15,k),f(6,15,k),f(8,15,k),k=1,7)/
     +    0.13963,   0.08328,   3.63159,   0.34026,   0.01251,
     +    0.00002,   0.17877,   0.34987,
     +    0.14675,   0.08874,   4.45618,   0.36979,   0.04961,
     +    0.00002,   0.21213,   0.37954,
     +    0.15305,   0.09349,   5.17942,   0.39765,   0.08399,
     +    0.00003,   0.24329,   0.40749,
     +    0.15870,   0.09764,   5.80246,   0.42356,   0.11563,
     +    0.00003,   0.27208,   0.43349,
     +    0.16381,   0.10136,   6.34982,   0.44777,   0.14495,
     +    0.00621,   0.29888,   0.45777,
     +    0.16850,   0.10473,   6.83238,   0.47042,   0.17222,
     +    0.03227,   0.32386,   0.48048,
     +    0.17281,   0.10781,   7.25862,   0.49169,   0.19770,
     +    0.06136,   0.34725,   0.50177 /
        data ((f(i,15,k),i=1,5),f(7,15,k),f(6,15,k),f(8,15,k),k=8,14)/
     +    0.17684,   0.11065,   7.63543,   0.51164,   0.22155,
     +    0.08796,   0.36915,   0.52176,
     +    0.18060,   0.11326,   7.96998,   0.53036,   0.24389,
     +    0.11291,   0.38969,   0.54051,
     +    0.18414,   0.11569,   8.26775,   0.54799,   0.26490,
     +    0.13631,   0.40901,   0.55816,
     +    0.18747,   0.11795,   8.53430,   0.56461,   0.28469,
     +    0.15834,   0.42723,   0.57480,
     +    0.19061,   0.12007,   8.77298,   0.58026,   0.30331,
     +    0.17905,   0.44437,   0.59047,
     +    0.19357,   0.12204,   8.98090,   0.59487,   0.32071,
     +    0.19841,   0.46038,   0.60508,
     +    0.19637,   0.12391,   9.17000,   0.60875,   0.33725,
     +    0.21678,   0.47558,   0.61896 /
        data ((f(i,15,k),i=1,5),f(7,15,k),f(6,15,k),f(8,15,k),k=15,19)/
     +    0.19904,   0.12568,   9.34047,   0.62189,   0.35290,
     +    0.23416,   0.48998,   0.63210,
     +    0.20158,   0.12734,   9.48812,   0.63411,   0.36753,
     +    0.25042,   0.50341,   0.64433,
     +    0.20402,   0.12892,   9.62581,   0.64588,   0.38159,
     +    0.26602,   0.51632,   0.65610,
     +    0.20634,   0.13040,   9.74323,   0.65678,   0.39467,
     +    0.28058,   0.52832,   0.66700,
     +    0.20857,   0.13182,   9.85289,   0.66729,   0.40728,
     +    0.29459,   0.53988,   0.67751 /
        data ((f(i,16,k),i=1,5),f(7,16,k),f(6,16,k),f(8,16,k),k=1,7)/
     +    0.15327,   0.09584,   3.37937,   0.31835,   0.01131,
     +    0.00002,   0.16734,   0.32849,
     +    0.16108,   0.10164,   4.09838,   0.34437,   0.04455,
     +    0.00004,   0.19701,   0.35462,
     +    0.16799,   0.10663,   4.71769,   0.36860,   0.07504,
     +    0.00004,   0.22442,   0.37896,
     +    0.17410,   0.11100,   5.24194,   0.39092,   0.10283,
     +    0.00003,   0.24948,   0.40129,
     +    0.17962,   0.11488,   5.69572,   0.41158,   0.12839,
     +    0.00545,   0.27260,   0.42199,
     +    0.18466,   0.11836,   6.08940,   0.43072,   0.15197,
     +    0.02810,   0.29397,   0.44117,
     +    0.18930,   0.12152,   6.43224,   0.44857,   0.17388,
     +    0.05329,   0.31386,   0.45904 /
        data ((f(i,16,k),i=1,5),f(7,16,k),f(6,16,k),f(8,16,k),k=8,14)/
     +    0.19361,   0.12442,   6.73166,   0.46515,   0.19422,
     +    0.07616,   0.33233,   0.47565,
     +    0.19762,   0.12708,   6.99285,   0.48062,   0.21317,
     +    0.09751,   0.34955,   0.49112,
     +    0.20136,   0.12954,   7.22282,   0.49508,   0.23088,
     +    0.11741,   0.36563,   0.50557,
     +    0.20488,   0.13183,   7.42642,   0.50861,   0.24747,
     +    0.13604,   0.38068,   0.51910,
     +    0.20821,   0.13395,   7.60415,   0.52126,   0.26300,
     +    0.15348,   0.39478,   0.53177,
     +    0.21131,   0.13594,   7.75704,   0.53297,   0.27742,
     +    0.16968,   0.40784,   0.54346,
     +    0.21425,   0.13780,   7.89445,   0.54405,   0.29106,
     +    0.18501,   0.42021,   0.55452 /
        data ((f(i,16,k),i=1,5),f(7,16,k),f(6,16,k),f(8,16,k),k=15,19)/
     +    0.21706,   0.13954,   8.01655,   0.55450,   0.30395,
     +    0.19948,   0.43188,   0.56497,
     +    0.21971,   0.14116,   8.11928,   0.56413,   0.31590,
     +    0.21292,   0.44266,   0.57459,
     +    0.22226,   0.14271,   8.21397,   0.57335,   0.32734,
     +    0.22578,   0.45299,   0.58380,
     +    0.22466,   0.14416,   8.29227,   0.58183,   0.33794,
     +    0.23772,   0.46253,   0.59226,
     +    0.22696,   0.14554,   8.36434,   0.58997,   0.34812,
     +    0.24917,   0.47169,   0.60038 /
        data ((f(i,17,k),i=1,5),f(7,17,k),f(6,17,k),f(8,17,k),k=1,7)/
     +    0.21004,   0.14651,   2.68652,   0.25825,   0.00799,
     +    0.00001,   0.13599,   0.26981,
     +    0.22012,   0.15270,   3.12191,   0.27451,   0.03088,
     +    0.00002,   0.15558,   0.28606,
     +    0.22881,   0.15789,   3.47119,   0.28899,   0.05113,
     +    0.00000,   0.17293,   0.30048,
     +    0.23634,   0.16230,   3.74667,   0.30175,   0.06897,
     +    0.00002,   0.18823,   0.31316,
     +    0.24302,   0.16610,   3.96980,   0.31313,   0.08491,
     +    0.00345,   0.20186,   0.32446,
     +    0.24902,   0.16942,   4.15146,   0.32331,   0.09921,
     +    0.01754,   0.21410,   0.33457,
     +    0.25445,   0.17234,   4.29869,   0.33243,   0.11214,
     +    0.03288,   0.22511,   0.34364 /
        data ((f(i,17,k),i=1,5),f(7,17,k),f(6,17,k),f(8,17,k),k=8,14)/
     +    0.25940,   0.17494,   4.41846,   0.34064,   0.12387,
     +    0.04651,   0.23506,   0.35179,
     +    0.26393,   0.17728,   4.51544,   0.34807,   0.13457,
     +    0.05900,   0.24411,   0.35914,
     +    0.26810,   0.17939,   4.59424,   0.35478,   0.14434,
     +    0.07041,   0.25234,   0.36577,
     +    0.27196,   0.18131,   4.65814,   0.36087,   0.15331,
     +    0.08088,   0.25985,   0.37179,
     +    0.27555,   0.18305,   4.70790,   0.36640,   0.16155,
     +    0.09055,   0.26672,   0.37727,
     +    0.27887,   0.18461,   4.74467,   0.37133,   0.16904,
     +    0.09936,   0.27290,   0.38211,
     +    0.28198,   0.18603,   4.77407,   0.37585,   0.17599,
     +    0.10755,   0.27863,   0.38658 /
        data ((f(i,17,k),i=1,5),f(7,17,k),f(6,17,k),f(8,17,k),k=15,19)/
     +    0.28491,   0.18734,   4.79632,   0.37997,   0.18244,
     +    0.11516,   0.28391,   0.39067,
     +    0.28761,   0.18853,   4.80952,   0.38366,   0.18832,
     +    0.12214,   0.28868,   0.39427,
     +    0.29017,   0.18966,   4.81959,   0.38712,   0.19387,
     +    0.12874,   0.29317,   0.39766,
     +    0.29255,   0.19067,   4.82134,   0.39014,   0.19888,
     +    0.13475,   0.29717,   0.40063,
     +    0.29483,   0.19162,   4.82118,   0.39298,   0.20364,
     +    0.14045,   0.30095,   0.40342 /
        data ((f(i,18,k),i=1,5),f(7,18,k),f(6,18,k),f(8,18,k),k=1,7)/
     +    0.30013,   0.21489,   2.09615,   0.20512,   0.00519,
     +   -0.00012,   0.10809,   0.21700,
     +    0.31140,   0.21926,   2.30651,   0.21297,   0.01984,
     +   -0.00014,   0.11928,   0.22464,
     +    0.32072,   0.22263,   2.45504,   0.21947,   0.03220,
     +   -0.00009,   0.12866,   0.23086,
     +    0.32846,   0.22525,   2.55577,   0.22471,   0.04260,
     +   -0.00010,   0.13640,   0.23586,
     +    0.33509,   0.22729,   2.62536,   0.22904,   0.05153,
     +    0.00184,   0.14297,   0.23996,
     +    0.34081,   0.22892,   2.67192,   0.23257,   0.05927,
     +    0.00979,   0.14856,   0.24331,
     +    0.34585,   0.23018,   2.70094,   0.23551,   0.06605,
     +    0.01828,   0.15339,   0.24606 /
        data ((f(i,18,k),i=1,5),f(7,18,k),f(6,18,k),f(8,18,k),k=8,14)/
     +    0.35029,   0.23117,   2.71617,   0.23789,   0.07198,
     +    0.02557,   0.15751,   0.24826,
     +    0.35420,   0.23194,   2.72121,   0.23981,   0.07722,
     +    0.03208,   0.16107,   0.25005,
     +    0.35766,   0.23253,   2.71849,   0.24139,   0.08186,
     +    0.03789,   0.16414,   0.25147,
     +    0.36075,   0.23298,   2.70995,   0.24268,   0.08600,
     +    0.04311,   0.16681,   0.25259,
     +    0.36354,   0.23329,   2.69663,   0.24370,   0.08972,
     +    0.04784,   0.16913,   0.25347,
     +    0.36598,   0.23347,   2.67878,   0.24441,   0.09297,
     +    0.05202,   0.17107,   0.25402,
     +    0.36820,   0.23358,   2.65897,   0.24492,   0.09591,
     +    0.05581,   0.17278,   0.25442 /
        data ((f(i,18,k),i=1,5),f(7,18,k),f(6,18,k),f(8,18,k),k=15,19)/
     +    0.37022,   0.23363,   2.63718,   0.24526,   0.09856,
     +    0.05923,   0.17425,   0.25470,
     +    0.37200,   0.23356,   2.61265,   0.24543,   0.10088,
     +    0.06236,   0.17546,   0.25478,
     +    0.37367,   0.23346,   2.58826,   0.24555,   0.10303,
     +    0.06528,   0.17656,   0.25480,
     +    0.37511,   0.23330,   2.56176,   0.24547,   0.10490,
     +    0.06783,   0.17744,   0.25457,
     +    0.37645,   0.23311,   2.53572,   0.24535,   0.10664,
     +    0.07022,   0.17823,   0.25431 /
        data ((f(i,19,k),i=1,5),f(7,19,k),f(6,19,k),f(8,19,k),k=1,7)/
     +    0.37435,   0.25824,   1.77708,   0.17306,   0.00395,
     +    0.00000,   0.09130,   0.18433,
     +    0.38426,   0.25969,   1.88087,   0.17665,   0.01458,
     +   -0.00001,   0.09834,   0.18753,
     +    0.39209,   0.26040,   1.94096,   0.17928,   0.02324,
     +    0.00005,   0.10392,   0.18977,
     +    0.39827,   0.26051,   1.96970,   0.18099,   0.03031,
     +    0.00003,   0.10822,   0.19124,
     +    0.40327,   0.26026,   1.97969,   0.18217,   0.03619,
     +    0.00132,   0.11167,   0.19206,
     +    0.40739,   0.25974,   1.97654,   0.18287,   0.04114,
     +    0.00664,   0.11444,   0.19248,
     +    0.41076,   0.25907,   1.96431,   0.18316,   0.04541,
     +    0.01216,   0.11669,   0.19263 /
        data ((f(i,19,k),i=1,5),f(7,19,k),f(6,19,k),f(8,19,k),k=8,14)/
     +    0.41354,   0.25821,   1.94539,   0.18320,   0.04903,
     +    0.01685,   0.11844,   0.19245,
     +    0.41575,   0.25730,   1.92203,   0.18305,   0.05210,
     +    0.02097,   0.11985,   0.19206,
     +    0.41757,   0.25630,   1.89570,   0.18271,   0.05476,
     +    0.02456,   0.12095,   0.19149,
     +    0.41912,   0.25520,   1.86760,   0.18219,   0.05709,
     +    0.02772,   0.12181,   0.19079,
     +    0.42040,   0.25403,   1.83844,   0.18150,   0.05913,
     +    0.03057,   0.12249,   0.18997,
     +    0.42129,   0.25288,   1.80774,   0.18075,   0.06086,
     +    0.03303,   0.12291,   0.18904,
     +    0.42201,   0.25171,   1.77732,   0.17993,   0.06240,
     +    0.03524,   0.12322,   0.18807 /
        data ((f(i,19,k),i=1,5),f(7,19,k),f(6,19,k),f(8,19,k),k=15,19)/
     +    0.42260,   0.25049,   1.74710,   0.17905,   0.06377,
     +    0.03722,   0.12341,   0.18706,
     +    0.42298,   0.24922,   1.71651,   0.17808,   0.06489,
     +    0.03895,   0.12347,   0.18597,
     +    0.42328,   0.24799,   1.68711,   0.17712,   0.06590,
     +    0.04053,   0.12348,   0.18490,
     +    0.42338,   0.24674,   1.65738,   0.17610,   0.06673,
     +    0.04189,   0.12335,   0.18372,
     +    0.42342,   0.24553,   1.62891,   0.17508,   0.06750,
     +    0.04314,   0.12320,   0.18258 /
        data ((f(i,20,k),i=1,5),f(7,20,k),f(6,20,k),f(8,20,k),k=1,7)/
     +    0.43672,   0.28564,   1.55169,   0.14808,   0.00311,
     +    0.00008,   0.07818,   0.15829,
     +    0.44382,   0.28390,   1.59176,   0.14907,   0.01129,
     +    0.00006,   0.08264,   0.15881,
     +    0.44892,   0.28182,   1.60210,   0.14940,   0.01774,
     +    0.00005,   0.08594,   0.15868,
     +    0.45254,   0.27937,   1.59256,   0.14913,   0.02284,
     +    0.00009,   0.08827,   0.15816,
     +    0.45506,   0.27697,   1.57257,   0.14859,   0.02697,
     +    0.00103,   0.09001,   0.15730,
     +    0.45677,   0.27446,   1.54594,   0.14781,   0.03037,
     +    0.00483,   0.09126,   0.15625,
     +    0.45785,   0.27196,   1.51525,   0.14685,   0.03319,
     +    0.00876,   0.09212,   0.15502 /
        data ((f(i,20,k),i=1,5),f(7,20,k),f(6,20,k),f(8,20,k),k=8,14)/
     +    0.45843,   0.26944,   1.48214,   0.14579,   0.03556,
     +    0.01200,   0.09270,   0.15363,
     +    0.45864,   0.26691,   1.44787,   0.14457,   0.03754,
     +    0.01480,   0.09304,   0.15222,
     +    0.45855,   0.26441,   1.41340,   0.14326,   0.03920,
     +    0.01719,   0.09317,   0.15077,
     +    0.45820,   0.26198,   1.37933,   0.14192,   0.04062,
     +    0.01926,   0.09315,   0.14928,
     +    0.45763,   0.25960,   1.34583,   0.14053,   0.04185,
     +    0.02106,   0.09305,   0.14774,
     +    0.45678,   0.25725,   1.31263,   0.13917,   0.04282,
     +    0.02266,   0.09281,   0.14618,
     +    0.45583,   0.25498,   1.28073,   0.13783,   0.04365,
     +    0.02406,   0.09251,   0.14464 /
        data ((f(i,20,k),i=1,5),f(7,20,k),f(6,20,k),f(8,20,k),k=15,19)/
     +    0.45480,   0.25276,   1.24994,   0.13650,   0.04436,
     +    0.02527,   0.09215,   0.14310,
     +    0.45364,   0.25053,   1.21982,   0.13508,   0.04494,
     +    0.02634,   0.09170,   0.14164,
     +    0.45246,   0.24839,   1.19131,   0.13371,   0.04545,
     +    0.02732,   0.09125,   0.14024,
     +    0.45108,   0.24633,   1.16340,   0.13238,   0.04583,
     +    0.02811,   0.09073,   0.13868,
     +    0.44972,   0.24433,   1.13695,   0.13109,   0.04617,
     +    0.02883,   0.09021,   0.13720 /
        data ((f(i,21,k),i=1,5),f(7,21,k),f(6,21,k),f(8,21,k),k=1,7)/
     +    0.48847,   0.30165,   1.37264,   0.12695,   0.00253,
     +    0.00008,   0.06700,   0.13583,
     +    0.49191,   0.29675,   1.37121,   0.12624,   0.00888,
     +    0.00007,   0.06973,   0.13474,
     +    0.49363,   0.29197,   1.35088,   0.12521,   0.01381,
     +    0.00007,   0.07156,   0.13324,
     +    0.49399,   0.28740,   1.31920,   0.12382,   0.01760,
     +    0.00011,   0.07267,   0.13153,
     +    0.49353,   0.28301,   1.28289,   0.12237,   0.02064,
     +    0.00078,   0.07336,   0.12966,
     +    0.49259,   0.27868,   1.24451,   0.12077,   0.02308,
     +    0.00362,   0.07369,   0.12781,
     +    0.49116,   0.27455,   1.20543,   0.11907,   0.02504,
     +    0.00644,   0.07377,   0.12595 /
        data ((f(i,21,k),i=1,5),f(7,21,k),f(6,21,k),f(8,21,k),k=8,14)/
     +    0.48944,   0.27055,   1.16658,   0.11735,   0.02666,
     +    0.00878,   0.07369,   0.12405,
     +    0.48741,   0.26671,   1.12864,   0.11568,   0.02797,
     +    0.01076,   0.07344,   0.12213,
     +    0.48517,   0.26305,   1.09201,   0.11402,   0.02902,
     +    0.01242,   0.07310,   0.12027,
     +    0.48279,   0.25956,   1.05697,   0.11238,   0.02987,
     +    0.01384,   0.07270,   0.11845,
     +    0.48024,   0.25626,   1.02352,   0.11080,   0.03059,
     +    0.01513,   0.07222,   0.11657,
     +    0.47763,   0.25296,   0.99127,   0.10920,   0.03118,
     +    0.01618,   0.07165,   0.11477,
     +    0.47505,   0.24978,   0.96083,   0.10761,   0.03167,
     +    0.01708,   0.07106,   0.11309 /
        data ((f(i,21,k),i=1,5),f(7,21,k),f(6,21,k),f(8,21,k),k=15,19)/
     +    0.47250,   0.24668,   0.93196,   0.10602,   0.03206,
     +    0.01784,   0.07044,   0.11154,
     +    0.46978,   0.24376,   0.90429,   0.10453,   0.03235,
     +    0.01852,   0.06979,   0.10986,
     +    0.46711,   0.24099,   0.87835,   0.10313,   0.03259,
     +    0.01915,   0.06916,   0.10823,
     +    0.46439,   0.23822,   0.85343,   0.10169,   0.03276,
     +    0.01967,   0.06850,   0.10671,
     +    0.46175,   0.23558,   0.83000,   0.10032,   0.03290,
     +    0.02013,   0.06785,   0.10524 /
        data ((f(i,22,k),i=1,5),f(7,22,k),f(6,22,k),f(8,22,k),k=1,7)/
     +    0.53953,   0.30942,   1.18740,   0.10410,   0.00193,
     +    0.00004,   0.05492,   0.11164,
     +    0.53742,   0.30119,   1.15255,   0.10223,   0.00679,
     +    0.00006,   0.05631,   0.10926,
     +    0.53429,   0.29360,   1.10937,   0.10028,   0.01038,
     +    0.00008,   0.05698,   0.10680,
     +    0.53041,   0.28651,   1.06249,   0.09814,   0.01312,
     +    0.00010,   0.05722,   0.10436,
     +    0.52610,   0.28000,   1.01611,   0.09603,   0.01517,
     +    0.00061,   0.05716,   0.10209,
     +    0.52157,   0.27391,   0.97144,   0.09401,   0.01682,
     +    0.00259,   0.05692,   0.09977,
     +    0.51681,   0.26826,   0.92871,   0.09206,   0.01812,
     +    0.00457,   0.05649,   0.09746 /
        data ((f(i,22,k),i=1,5),f(7,22,k),f(6,22,k),f(8,22,k),k=8,14)/
     +    0.51212,   0.26280,   0.88821,   0.09009,   0.01910,
     +    0.00617,   0.05597,   0.09534,
     +    0.50735,   0.25767,   0.85014,   0.08816,   0.01993,
     +    0.00751,   0.05538,   0.09328,
     +    0.50252,   0.25288,   0.81454,   0.08634,   0.02059,
     +    0.00865,   0.05474,   0.09127,
     +    0.49772,   0.24836,   0.78133,   0.08462,   0.02109,
     +    0.00960,   0.05407,   0.08933,
     +    0.49297,   0.24408,   0.75025,   0.08296,   0.02148,
     +    0.01036,   0.05342,   0.08747,
     +    0.48833,   0.23990,   0.72100,   0.08134,   0.02177,
     +    0.01102,   0.05271,   0.08567,
     +    0.48378,   0.23595,   0.69387,   0.07980,   0.02198,
     +    0.01161,   0.05200,   0.08398 /
        data ((f(i,22,k),i=1,5),f(7,22,k),f(6,22,k),f(8,22,k),k=15,19)/
     +    0.47926,   0.23221,   0.66857,   0.07833,   0.02212,
     +    0.01212,   0.05129,   0.08240,
     +    0.47483,   0.22860,   0.64465,   0.07686,   0.02222,
     +    0.01250,   0.05062,   0.08082,
     +    0.47057,   0.22518,   0.62242,   0.07546,   0.02231,
     +    0.01282,   0.04997,   0.07931,
     +    0.46632,   0.22186,   0.60137,   0.07411,   0.02235,
     +    0.01311,   0.04926,   0.07788,
     +    0.46222,   0.21870,   0.58173,   0.07283,   0.02237,
     +    0.01336,   0.04859,   0.07650 /
        data ((f(i,23,k),i=1,5),f(7,23,k),f(6,23,k),f(8,23,k),k=1,7)/
     +    0.57633,   0.30733,   1.03032,   0.08465,   0.00158,
     +    0.00012,   0.04476,   0.09074,
     +    0.56850,   0.29621,   0.97512,   0.08219,   0.00528,
     +    0.00016,   0.04521,   0.08790,
     +    0.56023,   0.28646,   0.91944,   0.07979,   0.00791,
     +    0.00014,   0.04524,   0.08519,
     +    0.55194,   0.27751,   0.86550,   0.07745,   0.00985,
     +    0.00013,   0.04497,   0.08247,
     +    0.54388,   0.26931,   0.81547,   0.07519,   0.01136,
     +    0.00051,   0.04452,   0.07992,
     +    0.53587,   0.26193,   0.76943,   0.07311,   0.01248,
     +    0.00195,   0.04393,   0.07753,
     +    0.52802,   0.25510,   0.72705,   0.07105,   0.01329,
     +    0.00335,   0.04330,   0.07532 /
        data ((f(i,23,k),i=1,5),f(7,23,k),f(6,23,k),f(8,23,k),k=8,14)/
     +    0.52045,   0.24865,   0.68803,   0.06913,   0.01395,
     +    0.00447,   0.04263,   0.07316,
     +    0.51307,   0.24271,   0.65228,   0.06728,   0.01446,
     +    0.00539,   0.04186,   0.07118,
     +    0.50591,   0.23714,   0.61950,   0.06551,   0.01483,
     +    0.00616,   0.04116,   0.06927,
     +    0.49900,   0.23186,   0.58943,   0.06385,   0.01509,
     +    0.00680,   0.04055,   0.06742,
     +    0.49233,   0.22685,   0.56185,   0.06230,   0.01529,
     +    0.00732,   0.03984,   0.06573,
     +    0.48575,   0.22220,   0.53623,   0.06080,   0.01544,
     +    0.00773,   0.03909,   0.06412,
     +    0.47945,   0.21780,   0.51271,   0.05940,   0.01555,
     +    0.00809,   0.03836,   0.06259 /
        data ((f(i,23,k),i=1,5),f(7,23,k),f(6,23,k),f(8,23,k),k=15,19)/
     +    0.47341,   0.21360,   0.49099,   0.05809,   0.01563,
     +    0.00841,   0.03764,   0.06110,
     +    0.46753,   0.20956,   0.47080,   0.05677,   0.01565,
     +    0.00869,   0.03698,   0.05972,
     +    0.46190,   0.20575,   0.45219,   0.05551,   0.01564,
     +    0.00893,   0.03635,   0.05842,
     +    0.45634,   0.20213,   0.43474,   0.05435,   0.01558,
     +    0.00906,   0.03572,   0.05715,
     +    0.45103,   0.19867,   0.41857,   0.05323,   0.01552,
     +    0.00918,   0.03512,   0.05596 /
        data ((f(i,24,k),i=1,5),f(7,24,k),f(6,24,k),f(8,24,k),k=1,7)/
     +    0.59996,   0.29826,   0.89347,   0.06814,   0.00117,
     +    0.00004,   0.03597,   0.07309,
     +    0.58637,   0.28495,   0.82658,   0.06549,   0.00403,
     +   -0.00001,   0.03595,   0.07010,
     +    0.57337,   0.27331,   0.76486,   0.06305,   0.00608,
     +    0.00002,   0.03564,   0.06722,
     +    0.56089,   0.26301,   0.70869,   0.06075,   0.00751,
     +    0.00001,   0.03514,   0.06452,
     +    0.54925,   0.25365,   0.65873,   0.05848,   0.00858,
     +    0.00035,   0.03449,   0.06219,
     +    0.53815,   0.24527,   0.61410,   0.05646,   0.00931,
     +    0.00139,   0.03388,   0.05991,
     +    0.52753,   0.23767,   0.57406,   0.05450,   0.00991,
     +    0.00243,   0.03321,   0.05781 /
        data ((f(i,24,k),i=1,5),f(7,24,k),f(6,24,k),f(8,24,k),k=8,14)/
     +    0.51747,   0.23057,   0.53806,   0.05266,   0.01035,
     +    0.00324,   0.03245,   0.05595,
     +    0.50785,   0.22400,   0.50557,   0.05102,   0.01067,
     +    0.00397,   0.03169,   0.05412,
     +    0.49862,   0.21793,   0.47630,   0.04946,   0.01090,
     +    0.00453,   0.03099,   0.05241,
     +    0.48984,   0.21226,   0.44993,   0.04795,   0.01104,
     +    0.00493,   0.03033,   0.05085,
     +    0.48156,   0.20686,   0.42605,   0.04657,   0.01109,
     +    0.00528,   0.02966,   0.04941,
     +    0.47345,   0.20197,   0.40396,   0.04529,   0.01119,
     +    0.00558,   0.02899,   0.04791,
     +    0.46577,   0.19730,   0.38391,   0.04405,   0.01124,
     +    0.00582,   0.02836,   0.04656 /
        data ((f(i,24,k),i=1,5),f(7,24,k),f(6,24,k),f(8,24,k),k=15,19)/
     +    0.45847,   0.19279,   0.36563,   0.04282,   0.01122,
     +    0.00598,   0.02776,   0.04540,
     +    0.45139,   0.18858,   0.34877,   0.04171,   0.01120,
     +    0.00611,   0.02718,   0.04420,
     +    0.44463,   0.18464,   0.33331,   0.04067,   0.01116,
     +    0.00623,   0.02663,   0.04305,
     +    0.43794,   0.18095,   0.31892,   0.03970,   0.01109,
     +    0.00631,   0.02609,   0.04196,
     +    0.43162,   0.17744,   0.30570,   0.03879,   0.01102,
     +    0.00639,   0.02557,   0.04092 /
        data ((f(i,25,k),i=1,5),f(7,25,k),f(6,25,k),f(8,25,k),k=1,7)/
     +    0.61180,   0.28403,   0.77290,   0.05428,   0.00091,
     +    0.00000,   0.02857,   0.05826,
     +    0.59269,   0.26924,   0.70031,   0.05176,   0.00312,
     +    0.00002,   0.02829,   0.05523,
     +    0.57527,   0.25644,   0.63701,   0.04948,   0.00456,
     +    0.00006,   0.02779,   0.05255,
     +    0.55906,   0.24517,   0.58177,   0.04728,   0.00569,
     +    0.00004,   0.02721,   0.05008,
     +    0.54419,   0.23517,   0.53394,   0.04516,   0.00643,
     +    0.00029,   0.02654,   0.04800,
     +    0.53038,   0.22615,   0.49229,   0.04336,   0.00698,
     +    0.00105,   0.02587,   0.04596,
     +    0.51721,   0.21817,   0.45567,   0.04177,   0.00734,
     +    0.00181,   0.02519,   0.04405 /
        data ((f(i,25,k),i=1,5),f(7,25,k),f(6,25,k),f(8,25,k),k=8,14)/
     +    0.50499,   0.21076,   0.42329,   0.04019,   0.00764,
     +    0.00240,   0.02456,   0.04228,
     +    0.49362,   0.20380,   0.39454,   0.03862,   0.00781,
     +    0.00283,   0.02388,   0.04084,
     +    0.48275,   0.19751,   0.36890,   0.03726,   0.00795,
     +    0.00317,   0.02319,   0.03942,
     +    0.47243,   0.19176,   0.34601,   0.03604,   0.00806,
     +    0.00347,   0.02253,   0.03801,
     +    0.46286,   0.18624,   0.32555,   0.03477,   0.00806,
     +    0.00373,   0.02193,   0.03685,
     +    0.45349,   0.18125,   0.30698,   0.03370,   0.00809,
     +    0.00390,   0.02141,   0.03556,
     +    0.44468,   0.17651,   0.29019,   0.03269,   0.00808,
     +    0.00407,   0.02090,   0.03441 /
        data ((f(i,25,k),i=1,5),f(7,25,k),f(6,25,k),f(8,25,k),k=15,19)/
     +    0.43638,   0.17197,   0.27493,   0.03172,   0.00804,
     +    0.00424,   0.02039,   0.03342,
     +    0.42838,   0.16779,   0.26109,   0.03080,   0.00800,
     +    0.00434,   0.01988,   0.03244,
     +    0.42080,   0.16388,   0.24849,   0.02993,   0.00796,
     +    0.00442,   0.01940,   0.03152,
     +    0.41343,   0.16018,   0.23672,   0.02914,   0.00787,
     +    0.00442,   0.01892,   0.03067,
     +    0.40644,   0.15669,   0.22597,   0.02840,   0.00779,
     +    0.00443,   0.01846,   0.02987 /
        data ((f(i,26,k),i=1,5),f(7,26,k),f(6,26,k),f(8,26,k),k=1,7)/
     +    0.61300,   0.26638,   0.66597,   0.04286,   0.00064,
     +    0.00008,   0.02243,   0.04577,
     +    0.58913,   0.25049,   0.59189,   0.04046,   0.00237,
     +    0.00008,   0.02216,   0.04297,
     +    0.56777,   0.23695,   0.52989,   0.03826,   0.00346,
     +    0.00008,   0.02154,   0.04078,
     +    0.54829,   0.22528,   0.47756,   0.03639,   0.00427,
     +    0.00009,   0.02089,   0.03856,
     +    0.53078,   0.21480,   0.43304,   0.03459,   0.00480,
     +    0.00021,   0.02024,   0.03676,
     +    0.51459,   0.20567,   0.39519,   0.03298,   0.00517,
     +    0.00076,   0.01964,   0.03504,
     +    0.49960,   0.19740,   0.36238,   0.03151,   0.00548,
     +    0.00133,   0.01896,   0.03347 /
        data ((f(i,26,k),i=1,5),f(7,26,k),f(6,26,k),f(8,26,k),k=8,14)/
     +    0.48565,   0.18994,   0.33366,   0.03022,   0.00567,
     +    0.00171,   0.01838,   0.03193,
     +    0.47260,   0.18312,   0.30864,   0.02906,   0.00576,
     +    0.00198,   0.01783,   0.03052,
     +    0.46044,   0.17683,   0.28664,   0.02797,   0.00583,
     +    0.00223,   0.01727,   0.02928,
     +    0.44908,   0.17100,   0.26714,   0.02694,   0.00589,
     +    0.00247,   0.01671,   0.02818,
     +    0.43838,   0.16554,   0.24979,   0.02594,   0.00593,
     +    0.00270,   0.01620,   0.02720,
     +    0.42825,   0.16055,   0.23419,   0.02502,   0.00590,
     +    0.00284,   0.01574,   0.02619,
     +    0.41871,   0.15591,   0.22028,   0.02416,   0.00585,
     +    0.00292,   0.01531,   0.02529 /
        data ((f(i,26,k),i=1,5),f(7,26,k),f(6,26,k),f(8,26,k),k=15,19)/
     +    0.40969,   0.15156,   0.20783,   0.02333,   0.00577,
     +    0.00296,   0.01490,   0.02450,
     +    0.40101,   0.14755,   0.19640,   0.02262,   0.00570,
     +    0.00301,   0.01446,   0.02372,
     +    0.39283,   0.14380,   0.18606,   0.02197,   0.00564,
     +    0.00305,   0.01405,   0.02299,
     +    0.38512,   0.14008,   0.17672,   0.02129,   0.00559,
     +    0.00309,   0.01369,   0.02228,
     +    0.37778,   0.13663,   0.16816,   0.02065,   0.00553,
     +    0.00312,   0.01334,   0.02163 /
        data ((f(i,27,k),i=1,5),f(7,27,k),f(6,27,k),f(8,27,k),k=1,7)/
     +    0.60505,   0.24641,   0.57098,   0.03341,   0.00055,
     +    0.00001,   0.01754,   0.03535,
     +    0.57715,   0.22979,   0.49826,   0.03116,   0.00182,
     +    0.00005,   0.01707,   0.03323,
     +    0.55242,   0.21595,   0.43921,   0.02926,   0.00283,
     +    0.00002,   0.01659,   0.03104,
     +    0.53033,   0.20407,   0.39068,   0.02764,   0.00335,
     +    0.00009,   0.01595,   0.02929,
     +    0.51047,   0.19377,   0.35043,   0.02612,   0.00374,
     +    0.00023,   0.01529,   0.02777,
     +    0.49247,   0.18462,   0.31648,   0.02487,   0.00398,
     +    0.00064,   0.01477,   0.02625,
     +    0.47596,   0.17644,   0.28761,   0.02362,   0.00419,
     +    0.00095,   0.01424,   0.02497 /
        data ((f(i,27,k),i=1,5),f(7,27,k),f(6,27,k),f(8,27,k),k=8,14)/
     +    0.46064,   0.16916,   0.26274,   0.02253,   0.00424,
     +    0.00125,   0.01375,   0.02382,
     +    0.44659,   0.16240,   0.24127,   0.02153,   0.00430,
     +    0.00150,   0.01328,   0.02271,
     +    0.43346,   0.15628,   0.22253,   0.02068,   0.00429,
     +    0.00167,   0.01282,   0.02171,
     +    0.42119,   0.15072,   0.20607,   0.01991,   0.00425,
     +    0.00178,   0.01235,   0.02083,
     +    0.40990,   0.14547,   0.19155,   0.01906,   0.00430,
     +    0.00189,   0.01188,   0.02003,
     +    0.39918,   0.14066,   0.17887,   0.01833,   0.00426,
     +    0.00201,   0.01147,   0.01927,
     +    0.38916,   0.13620,   0.16751,   0.01763,   0.00421,
     +    0.00209,   0.01110,   0.01859 /
        data ((f(i,27,k),i=1,5),f(7,27,k),f(6,27,k),f(8,27,k),k=15,19)/
     +    0.37974,   0.13203,   0.15724,   0.01695,   0.00416,
     +    0.00210,   0.01077,   0.01798,
     +    0.37084,   0.12813,   0.14796,   0.01634,   0.00412,
     +    0.00213,   0.01044,   0.01732,
     +    0.36247,   0.12451,   0.13959,   0.01578,   0.00408,
     +    0.00215,   0.01014,   0.01670,
     +    0.35437,   0.12114,   0.13203,   0.01528,   0.00400,
     +    0.00218,   0.00984,   0.01615,
     +    0.34671,   0.11797,   0.12517,   0.01481,   0.00393,
     +    0.00219,   0.00957,   0.01563 /
        data ((f(i,28,k),i=1,5),f(7,28,k),f(6,28,k),f(8,28,k),k=1,7)/
     +    0.58954,   0.22478,   0.48679,   0.02554,   0.00046,
     +    0.00010,   0.01354,   0.02713,
     +    0.55783,   0.20837,   0.41754,   0.02375,   0.00155,
     +    0.00004,   0.01304,   0.02512,
     +    0.53047,   0.19458,   0.36280,   0.02216,   0.00217,
     +    0.00009,   0.01257,   0.02346,
     +    0.50623,   0.18291,   0.31868,   0.02078,   0.00257,
     +    0.00012,   0.01199,   0.02208,
     +    0.48459,   0.17289,   0.28261,   0.01964,   0.00276,
     +    0.00015,   0.01153,   0.02074,
     +    0.46534,   0.16399,   0.25296,   0.01852,   0.00288,
     +    0.00047,   0.01102,   0.01971,
     +    0.44781,   0.15608,   0.22776,   0.01751,   0.00304,
     +    0.00077,   0.01059,   0.01860 /
        data ((f(i,28,k),i=1,5),f(7,28,k),f(6,28,k),f(8,28,k),k=8,14)/
     +    0.43160,   0.14906,   0.20665,   0.01674,   0.00313,
     +    0.00090,   0.01014,   0.01755,
     +    0.41675,   0.14264,   0.18838,   0.01601,   0.00313,
     +    0.00109,   0.00971,   0.01672,
     +    0.40311,   0.13677,   0.17264,   0.01526,   0.00312,
     +    0.00126,   0.00934,   0.01597,
     +    0.39060,   0.13141,   0.15908,   0.01450,   0.00312,
     +    0.00135,   0.00901,   0.01526,
     +    0.37913,   0.12652,   0.14720,   0.01371,   0.00311,
     +    0.00136,   0.00867,   0.01461,
     +    0.36790,   0.12200,   0.13660,   0.01324,   0.00310,
     +    0.00144,   0.00836,   0.01398,
     +    0.35764,   0.11778,   0.12722,   0.01273,   0.00306,
     +    0.00149,   0.00806,   0.01344 /
        data ((f(i,28,k),i=1,5),f(7,28,k),f(6,28,k),f(8,28,k),k=15,19)/
     +    0.34831,   0.11381,   0.11888,   0.01213,   0.00298,
     +    0.00149,   0.00776,   0.01297,
     +    0.33905,   0.11027,   0.11151,   0.01168,   0.00295,
     +    0.00155,   0.00754,   0.01253,
     +    0.33035,   0.10698,   0.10490,   0.01127,   0.00292,
     +    0.00159,   0.00735,   0.01212,
     +    0.32231,   0.10376,   0.09886,   0.01087,   0.00284,
     +    0.00156,   0.00710,   0.01171,
     +    0.31471,   0.10076,   0.09342,   0.01051,   0.00279,
     +    0.00153,   0.00687,   0.01131 /
        data ((f(i,29,k),i=1,5),f(7,29,k),f(6,29,k),f(8,29,k),k=1,7)/
     +    0.56688,   0.20310,   0.41234,   0.01932,   0.00037,
     +    0.00010,   0.01025,   0.02057,
     +    0.53263,   0.18681,   0.34778,   0.01782,   0.00114,
     +    0.00000,   0.00983,   0.01899,
     +    0.50313,   0.17350,   0.29790,   0.01659,   0.00163,
     +    0.00010,   0.00946,   0.01757,
     +    0.47731,   0.16225,   0.25853,   0.01552,   0.00192,
     +    0.00007,   0.00903,   0.01634,
     +    0.45481,   0.15248,   0.22705,   0.01446,   0.00214,
     +    0.00016,   0.00866,   0.01526,
     +    0.43472,   0.14398,   0.20114,   0.01356,   0.00225,
     +    0.00036,   0.00822,   0.01442,
     +    0.41654,   0.13646,   0.17977,   0.01277,   0.00233,
     +    0.00055,   0.00784,   0.01362 /
        data ((f(i,29,k),i=1,5),f(7,29,k),f(6,29,k),f(8,29,k),k=8,14)/
     +    0.39984,   0.12979,   0.16184,   0.01210,   0.00234,
     +    0.00073,   0.00747,   0.01294,
     +    0.38473,   0.12375,   0.14652,   0.01151,   0.00232,
     +    0.00084,   0.00716,   0.01224,
     +    0.37088,   0.11829,   0.13346,   0.01094,   0.00234,
     +    0.00090,   0.00685,   0.01159,
     +    0.35807,   0.11336,   0.12224,   0.01040,   0.00233,
     +    0.00094,   0.00654,   0.01103,
     +    0.34617,   0.10884,   0.11247,   0.00994,   0.00222,
     +    0.00101,   0.00634,   0.01056,
     +    0.33523,   0.10466,   0.10394,   0.00940,   0.00222,
     +    0.00105,   0.00607,   0.01014,
     +    0.32500,   0.10079,   0.09643,   0.00897,   0.00223,
     +    0.00109,   0.00582,   0.00973 /
        data ((f(i,29,k),i=1,5),f(7,29,k),f(6,29,k),f(8,29,k),k=15,19)/
     +    0.31536,   0.09720,   0.08980,   0.00866,   0.00222,
     +    0.00115,   0.00561,   0.00934,
     +    0.30640,   0.09388,   0.08383,   0.00825,   0.00216,
     +    0.00112,   0.00541,   0.00897,
     +    0.29806,   0.09080,   0.07851,   0.00786,   0.00209,
     +    0.00109,   0.00522,   0.00863,
     +    0.29018,   0.08785,   0.07380,   0.00757,   0.00206,
     +    0.00111,   0.00504,   0.00829,
     +    0.28273,   0.08512,   0.06954,   0.00732,   0.00203,
     +    0.00113,   0.00487,   0.00798 /
        data ((f(i,30,k),i=1,5),f(7,30,k),f(6,30,k),f(8,30,k),k=1,7)/
     +    0.53888,   0.18160,   0.34717,   0.01472,   0.00020,
     +   -0.00007,   0.00753,   0.01533,
     +    0.50255,   0.16608,   0.28792,   0.01347,   0.00077,
     +   -0.00004,   0.00723,   0.01407,
     +    0.47171,   0.15342,   0.24361,   0.01246,   0.00112,
     +   -0.00012,   0.00691,   0.01296,
     +    0.44538,   0.14248,   0.20883,   0.01144,   0.00141,
     +   -0.00006,   0.00651,   0.01211,
     +    0.42220,   0.13331,   0.18143,   0.01066,   0.00150,
     +   -0.00005,   0.00622,   0.01131,
     +    0.40163,   0.12535,   0.15956,   0.00999,   0.00158,
     +    0.00014,   0.00593,   0.01058,
     +    0.38321,   0.11843,   0.14159,   0.00930,   0.00167,
     +    0.00033,   0.00564,   0.00997 /
        data ((f(i,30,k),i=1,5),f(7,30,k),f(6,30,k),f(8,30,k),k=8,14)/
     +    0.36668,   0.11216,   0.12646,   0.00877,   0.00168,
     +    0.00042,   0.00530,   0.00939,
     +    0.35141,   0.10663,   0.11384,   0.00842,   0.00169,
     +    0.00054,   0.00508,   0.00877,
     +    0.33765,   0.10162,   0.10312,   0.00791,   0.00168,
     +    0.00062,   0.00490,   0.00832,
     +    0.32516,   0.09702,   0.09393,   0.00738,   0.00166,
     +    0.00065,   0.00470,   0.00799,
     +    0.31329,   0.09287,   0.08602,   0.00716,   0.00163,
     +    0.00068,   0.00450,   0.00758,
     +    0.30256,   0.08908,   0.07912,   0.00683,   0.00157,
     +    0.00072,   0.00429,   0.00713,
     +    0.29258,   0.08558,   0.07306,   0.00647,   0.00153,
     +    0.00072,   0.00410,   0.00678 /
        data ((f(i,30,k),i=1,5),f(7,30,k),f(6,30,k),f(8,30,k),k=15,19)/
     +    0.28323,   0.08234,   0.06772,   0.00611,   0.00150,
     +    0.00068,   0.00393,   0.00654,
     +    0.27434,   0.07935,   0.06308,   0.00586,   0.00147,
     +    0.00071,   0.00380,   0.00632,
     +    0.26608,   0.07659,   0.05895,   0.00565,   0.00145,
     +    0.00074,   0.00368,   0.00611,
     +    0.25850,   0.07395,   0.05519,   0.00543,   0.00142,
     +    0.00077,   0.00352,   0.00583,
     +    0.25135,   0.07150,   0.05182,   0.00524,   0.00139,
     +    0.00078,   0.00339,   0.00558 /
        data ((f(i,31,k),i=1,5),f(7,31,k),f(6,31,k),f(8,31,k),k=1,7)/
     +    0.50649,   0.16023,   0.28913,   0.01065,   0.00028,
     +    0.00012,   0.00570,   0.01125,
     +    0.46912,   0.14548,   0.23659,   0.00964,   0.00070,
     +    0.00008,   0.00541,   0.01033,
     +    0.43759,   0.13365,   0.19727,   0.00892,   0.00101,
     +    0.00003,   0.00514,   0.00936,
     +    0.41091,   0.12356,   0.16745,   0.00810,   0.00108,
     +    0.00010,   0.00485,   0.00878,
     +    0.38758,   0.11508,   0.14411,   0.00752,   0.00119,
     +    0.00014,   0.00456,   0.00818,
     +    0.36703,   0.10781,   0.12554,   0.00712,   0.00124,
     +    0.00024,   0.00430,   0.00758,
     +    0.34878,   0.10139,   0.11047,   0.00657,   0.00124,
     +    0.00033,   0.00407,   0.00716 /
        data ((f(i,31,k),i=1,5),f(7,31,k),f(6,31,k),f(8,31,k),k=8,14)/
     +    0.33220,   0.09579,   0.09801,   0.00629,   0.00125,
     +    0.00040,   0.00390,   0.00662,
     +    0.31754,   0.09068,   0.08765,   0.00582,   0.00122,
     +    0.00043,   0.00371,   0.00632,
     +    0.30415,   0.08610,   0.07896,   0.00536,   0.00122,
     +    0.00048,   0.00352,   0.00604,
     +    0.29173,   0.08204,   0.07158,   0.00510,   0.00124,
     +    0.00053,   0.00335,   0.00568,
     +    0.28030,   0.07837,   0.06527,   0.00496,   0.00119,
     +    0.00050,   0.00320,   0.00533,
     +    0.26988,   0.07494,   0.05978,   0.00468,   0.00117,
     +    0.00050,   0.00305,   0.00505,
     +    0.26026,   0.07179,   0.05497,   0.00443,   0.00114,
     +    0.00051,   0.00291,   0.00483 /
        data ((f(i,31,k),i=1,5),f(7,31,k),f(6,31,k),f(8,31,k),k=15,19)/
     +    0.25131,   0.06888,   0.05074,   0.00423,   0.00110,
     +    0.00055,   0.00276,   0.00468,
     +    0.24278,   0.06623,   0.04703,   0.00415,   0.00108,
     +    0.00056,   0.00266,   0.00438,
     +    0.23485,   0.06379,   0.04375,   0.00409,   0.00106,
     +    0.00056,   0.00258,   0.00409,
     +    0.22745,   0.06148,   0.04090,   0.00395,   0.00098,
     +    0.00055,   0.00247,   0.00399,
     +    0.22064,   0.05933,   0.03833,   0.00379,   0.00093,
     +    0.00055,   0.00237,   0.00388 /
        data ((f(i,32,k),i=1,5),f(7,32,k),f(6,32,k),f(8,32,k),k=1,7)/
     +    0.47097,   0.14042,   0.23951,   0.00786,   0.00020,
     +   -0.00001,   0.00404,   0.00812,
     +    0.43309,   0.12675,   0.19286,   0.00709,   0.00041,
     +    0.00002,   0.00386,   0.00749,
     +    0.40159,   0.11571,   0.15879,   0.00649,   0.00062,
     +   -0.00002,   0.00363,   0.00685,
     +    0.37506,   0.10651,   0.13337,   0.00594,   0.00072,
     +   -0.00003,   0.00340,   0.00630,
     +    0.35218,   0.09870,   0.11373,   0.00546,   0.00082,
     +    0.00006,   0.00323,   0.00581,
     +    0.33191,   0.09203,   0.09825,   0.00508,   0.00087,
     +    0.00013,   0.00306,   0.00550,
     +    0.31426,   0.08624,   0.08583,   0.00463,   0.00089,
     +    0.00019,   0.00289,   0.00513 /
        data ((f(i,32,k),i=1,5),f(7,32,k),f(6,32,k),f(8,32,k),k=8,14)/
     +    0.29831,   0.08117,   0.07577,   0.00444,   0.00086,
     +    0.00020,   0.00268,   0.00472,
     +    0.28389,   0.07665,   0.06732,   0.00426,   0.00084,
     +    0.00023,   0.00250,   0.00437,
     +    0.27094,   0.07256,   0.06023,   0.00395,   0.00086,
     +    0.00026,   0.00239,   0.00415,
     +    0.25923,   0.06885,   0.05431,   0.00365,   0.00087,
     +    0.00029,   0.00230,   0.00398,
     +    0.24850,   0.06547,   0.04930,   0.00349,   0.00080,
     +    0.00033,   0.00218,   0.00371,
     +    0.23850,   0.06249,   0.04492,   0.00336,   0.00075,
     +    0.00033,   0.00206,   0.00354,
     +    0.22935,   0.05975,   0.04116,   0.00318,   0.00073,
     +    0.00033,   0.00195,   0.00340 /
        data ((f(i,32,k),i=1,5),f(7,32,k),f(6,32,k),f(8,32,k),k=15,19)/
     +    0.22097,   0.05720,   0.03791,   0.00296,   0.00071,
     +    0.00035,   0.00186,   0.00326,
     +    0.21303,   0.05487,   0.03502,   0.00292,   0.00069,
     +    0.00034,   0.00178,   0.00302,
     +    0.20567,   0.05272,   0.03247,   0.00290,   0.00068,
     +    0.00032,   0.00171,   0.00279,
     +    0.19876,   0.05069,   0.03025,   0.00280,   0.00065,
     +    0.00033,   0.00164,   0.00269,
     +    0.19238,   0.04883,   0.02826,   0.00270,   0.00064,
     +    0.00034,   0.00158,   0.00258 /
        data ((f(i,33,k),i=1,5),f(7,33,k),f(6,33,k),f(8,33,k),k=1,7)/
     +    0.43318,   0.12133,   0.19629,   0.00548,   0.00015,
     +    0.00002,   0.00293,   0.00593,
     +    0.39569,   0.10878,   0.15587,   0.00491,   0.00041,
     +    0.00002,   0.00278,   0.00534,
     +    0.36483,   0.09879,   0.12669,   0.00450,   0.00049,
     +    0.00007,   0.00265,   0.00484,
     +    0.33855,   0.09050,   0.10534,   0.00408,   0.00060,
     +    0.00004,   0.00243,   0.00455,
     +    0.31639,   0.08359,   0.08908,   0.00378,   0.00063,
     +    0.00006,   0.00227,   0.00414,
     +    0.29715,   0.07760,   0.07629,   0.00348,   0.00064,
     +    0.00011,   0.00214,   0.00385,
     +    0.28011,   0.07244,   0.06611,   0.00331,   0.00061,
     +    0.00012,   0.00198,   0.00348 /
        data ((f(i,33,k),i=1,5),f(7,33,k),f(6,33,k),f(8,33,k),k=8,14)/
     +    0.26472,   0.06791,   0.05790,   0.00317,   0.00055,
     +    0.00015,   0.00189,   0.00328,
     +    0.25119,   0.06381,   0.05116,   0.00296,   0.00055,
     +    0.00018,   0.00177,   0.00306,
     +    0.23907,   0.06021,   0.04558,   0.00269,   0.00055,
     +    0.00021,   0.00166,   0.00287,
     +    0.22799,   0.05704,   0.04090,   0.00247,   0.00055,
     +    0.00021,   0.00159,   0.00273,
     +    0.21775,   0.05416,   0.03692,   0.00242,   0.00053,
     +    0.00022,   0.00150,   0.00253,
     +    0.20849,   0.05150,   0.03349,   0.00231,   0.00050,
     +    0.00025,   0.00143,   0.00236,
     +    0.19999,   0.04908,   0.03055,   0.00222,   0.00050,
     +    0.00026,   0.00137,   0.00221 /
        data ((f(i,33,k),i=1,5),f(7,33,k),f(6,33,k),f(8,33,k),k=15,19)/
     +    0.19212,   0.04688,   0.02803,   0.00214,   0.00052,
     +    0.00025,   0.00132,   0.00207,
     +    0.18484,   0.04488,   0.02583,   0.00207,   0.00051,
     +    0.00028,   0.00126,   0.00197,
     +    0.17813,   0.04305,   0.02390,   0.00198,   0.00049,
     +    0.00029,   0.00120,   0.00188,
     +    0.17185,   0.04129,   0.02220,   0.00184,   0.00047,
     +    0.00023,   0.00112,   0.00182,
     +    0.16602,   0.03968,   0.02069,   0.00172,   0.00044,
     +    0.00019,   0.00105,   0.00176 /
        data ((f(i,34,k),i=1,5),f(7,34,k),f(6,34,k),f(8,34,k),k=1,7)/
     +    0.39419,   0.10390,   0.15953,   0.00373,   0.00014,
     +    0.00008,   0.00208,   0.00419,
     +    0.35773,   0.09262,   0.12489,   0.00340,   0.00028,
     +    0.00007,   0.00195,   0.00377,
     +    0.32770,   0.08368,   0.10038,   0.00308,   0.00034,
     +    0.00000,   0.00180,   0.00350,
     +    0.30285,   0.07628,   0.08252,   0.00276,   0.00037,
     +    0.00002,   0.00164,   0.00320,
     +    0.28160,   0.07004,   0.06909,   0.00261,   0.00038,
     +    0.00006,   0.00151,   0.00291,
     +    0.26312,   0.06490,   0.05877,   0.00248,   0.00039,
     +    0.00007,   0.00144,   0.00259,
     +    0.24706,   0.06033,   0.05059,   0.00236,   0.00041,
     +    0.00012,   0.00133,   0.00239 /
        data ((f(i,34,k),i=1,5),f(7,34,k),f(6,34,k),f(8,34,k),k=8,14)/
     +    0.23299,   0.05629,   0.04403,   0.00212,   0.00038,
     +    0.00013,   0.00123,   0.00226,
     +    0.22019,   0.05279,   0.03870,   0.00203,   0.00037,
     +    0.00012,   0.00117,   0.00212,
     +    0.20870,   0.04969,   0.03428,   0.00196,   0.00038,
     +    0.00011,   0.00109,   0.00194,
     +    0.19844,   0.04691,   0.03058,   0.00186,   0.00039,
     +    0.00011,   0.00101,   0.00179,
     +    0.18911,   0.04438,   0.02748,   0.00176,   0.00040,
     +    0.00015,   0.00105,   0.00170,
     +    0.18057,   0.04211,   0.02488,   0.00167,   0.00036,
     +    0.00011,   0.00095,   0.00151,
     +    0.17269,   0.04006,   0.02264,   0.00163,   0.00032,
     +    0.00010,   0.00088,   0.00141 /
        data ((f(i,34,k),i=1,5),f(7,34,k),f(6,34,k),f(8,34,k),k=15,19)/
     +    0.16538,   0.03820,   0.02070,   0.00162,   0.00029,
     +    0.00014,   0.00084,   0.00140,
     +    0.15882,   0.03648,   0.01901,   0.00151,   0.00030,
     +    0.00014,   0.00081,   0.00128,
     +    0.15280,   0.03490,   0.01752,   0.00140,   0.00031,
     +    0.00013,   0.00078,   0.00117,
     +    0.14700,   0.03344,   0.01623,   0.00138,   0.00029,
     +    0.00010,   0.00072,   0.00113,
     +    0.14169,   0.03209,   0.01508,   0.00133,   0.00028,
     +    0.00009,   0.00067,   0.00110 /
        data ((f(i,35,k),i=1,5),f(7,35,k),f(6,35,k),f(8,35,k),k=1,7)/
     +    0.35487,   0.08795,   0.12828,   0.00254,   0.00002,
     +    0.00001,   0.00147,   0.00297,
     +    0.31956,   0.07794,   0.09880,   0.00231,   0.00013,
     +   -0.00003,   0.00134,   0.00259,
     +    0.29089,   0.06987,   0.07835,   0.00217,   0.00025,
     +   -0.00006,   0.00121,   0.00237,
     +    0.26731,   0.06336,   0.06381,   0.00194,   0.00029,
     +   -0.00001,   0.00118,   0.00211,
     +    0.24752,   0.05796,   0.05285,   0.00173,   0.00029,
     +   -0.00002,   0.00110,   0.00192,
     +    0.23036,   0.05334,   0.04453,   0.00170,   0.00030,
     +    0.00004,   0.00100,   0.00174,
     +    0.21543,   0.04939,   0.03809,   0.00160,   0.00030,
     +    0.00007,   0.00092,   0.00151 /
        data ((f(i,35,k),i=1,5),f(7,35,k),f(6,35,k),f(8,35,k),k=8,14)/
     +    0.20219,   0.04598,   0.03290,   0.00146,   0.00031,
     +    0.00008,   0.00088,   0.00145,
     +    0.19039,   0.04296,   0.02871,   0.00139,   0.00030,
     +    0.00007,   0.00081,   0.00137,
     +    0.17990,   0.04029,   0.02531,   0.00135,   0.00029,
     +    0.00008,   0.00075,   0.00121,
     +    0.17053,   0.03792,   0.02248,   0.00128,   0.00028,
     +    0.00009,   0.00072,   0.00107,
     +    0.16199,   0.03580,   0.02011,   0.00118,   0.00026,
     +    0.00009,   0.00067,   0.00102,
     +    0.15424,   0.03387,   0.01811,   0.00114,   0.00026,
     +    0.00011,   0.00068,   0.00101,
     +    0.14720,   0.03212,   0.01641,   0.00107,   0.00026,
     +    0.00011,   0.00065,   0.00099 /
        data ((f(i,35,k),i=1,5),f(7,35,k),f(6,35,k),f(8,35,k),k=15,19)/
     +    0.14078,   0.03053,   0.01494,   0.00095,   0.00026,
     +    0.00010,   0.00060,   0.00095,
     +    0.13481,   0.02907,   0.01368,   0.00091,   0.00025,
     +    0.00012,   0.00057,   0.00090,
     +    0.12932,   0.02774,   0.01257,   0.00088,   0.00025,
     +    0.00013,   0.00055,   0.00085,
     +    0.12430,   0.02651,   0.01162,   0.00079,   0.00025,
     +    0.00014,   0.00053,   0.00081,
     +    0.11963,   0.02538,   0.01077,   0.00072,   0.00025,
     +    0.00014,   0.00051,   0.00077 /
        data ((f(i,36,k),i=1,5),f(7,36,k),f(6,36,k),f(8,36,k),k=1,7)/
     +    0.31595,   0.07371,   0.10210,   0.00168,   0.00003,
     +   -0.00002,   0.00089,   0.00213,
     +    0.28252,   0.06482,   0.07750,   0.00166,   0.00011,
     +   -0.00002,   0.00085,   0.00185,
     +    0.25592,   0.05781,   0.06078,   0.00142,   0.00013,
     +   -0.00006,   0.00078,   0.00163,
     +    0.23394,   0.05218,   0.04893,   0.00125,   0.00020,
     +    0.00001,   0.00075,   0.00153,
     +    0.21534,   0.04753,   0.04021,   0.00121,   0.00018,
     +   -0.00001,   0.00069,   0.00139,
     +    0.19961,   0.04360,   0.03361,   0.00115,   0.00019,
     +    0.00001,   0.00065,   0.00115,
     +    0.18589,   0.04019,   0.02853,   0.00107,   0.00020,
     +    0.00003,   0.00060,   0.00106 /
        data ((f(i,36,k),i=1,5),f(7,36,k),f(6,36,k),f(8,36,k),k=8,14)/
     +    0.17372,   0.03729,   0.02447,   0.00105,   0.00016,
     +    0.00000,   0.00054,   0.00100,
     +    0.16312,   0.03472,   0.02126,   0.00101,   0.00017,
     +    0.00002,   0.00050,   0.00090,
     +    0.15373,   0.03246,   0.01864,   0.00093,   0.00017,
     +    0.00003,   0.00047,   0.00078,
     +    0.14531,   0.03047,   0.01648,   0.00085,   0.00016,
     +    0.00003,   0.00044,   0.00070,
     +    0.13780,   0.02865,   0.01469,   0.00076,   0.00017,
     +    0.00005,   0.00043,   0.00065,
     +    0.13088,   0.02704,   0.01318,   0.00067,   0.00018,
     +    0.00006,   0.00042,   0.00066,
     +    0.12449,   0.02559,   0.01191,   0.00065,   0.00017,
     +    0.00006,   0.00039,   0.00065 /
        data ((f(i,36,k),i=1,5),f(7,36,k),f(6,36,k),f(8,36,k),k=15,19)/
     +    0.11855,   0.02427,   0.01081,   0.00069,   0.00015,
     +    0.00005,   0.00036,   0.00061,
     +    0.11339,   0.02307,   0.00986,   0.00061,   0.00016,
     +    0.00005,   0.00035,   0.00054,
     +    0.10868,   0.02197,   0.00904,   0.00051,   0.00016,
     +    0.00005,   0.00035,   0.00047,
     +    0.10418,   0.02095,   0.00833,   0.00045,   0.00014,
     +    0.00005,   0.00032,   0.00048,
     +    0.10003,   0.02002,   0.00770,   0.00040,   0.00012,
     +    0.00006,   0.00030,   0.00049 /
        data ((f(i,37,k),i=1,5),f(7,37,k),f(6,37,k),f(8,37,k),k=1,7)/
     +    0.27754,   0.06092,   0.08010,   0.00124,  -0.00001,
     +    0.00000,   0.00060,   0.00138,
     +    0.24661,   0.05323,   0.05995,   0.00114,   0.00008,
     +    0.00001,   0.00058,   0.00126,
     +    0.22199,   0.04722,   0.04647,   0.00105,   0.00006,
     +   -0.00001,   0.00052,   0.00102,
     +    0.20191,   0.04238,   0.03700,   0.00095,   0.00007,
     +   -0.00005,   0.00052,   0.00088,
     +    0.18485,   0.03844,   0.03013,   0.00101,   0.00009,
     +   -0.00005,   0.00044,   0.00074,
     +    0.17067,   0.03511,   0.02500,   0.00088,   0.00011,
     +   -0.00002,   0.00041,   0.00069,
     +    0.15839,   0.03224,   0.02106,   0.00081,   0.00011,
     +   -0.00001,   0.00037,   0.00063 /
        data ((f(i,37,k),i=1,5),f(7,37,k),f(6,37,k),f(8,37,k),k=8,14)/
     +    0.14767,   0.02980,   0.01798,   0.00075,   0.00013,
     +    0.00003,   0.00035,   0.00056,
     +    0.13818,   0.02767,   0.01552,   0.00064,   0.00011,
     +    0.00003,   0.00032,   0.00057,
     +    0.12970,   0.02578,   0.01354,   0.00061,   0.00011,
     +    0.00001,   0.00030,   0.00050,
     +    0.12218,   0.02412,   0.01192,   0.00061,   0.00013,
     +    0.00002,   0.00030,   0.00043,
     +    0.11560,   0.02262,   0.01058,   0.00044,   0.00011,
     +    0.00004,   0.00029,   0.00051,
     +    0.10949,   0.02130,   0.00945,   0.00039,   0.00011,
     +    0.00004,   0.00027,   0.00045,
     +    0.10393,   0.02010,   0.00849,   0.00036,   0.00011,
     +    0.00004,   0.00025,   0.00039 /
        data ((f(i,37,k),i=1,5),f(7,37,k),f(6,37,k),f(8,37,k),k=15,19)/
     +    0.09887,   0.01901,   0.00769,   0.00035,   0.00010,
     +    0.00003,   0.00023,   0.00037,
     +    0.09421,   0.01803,   0.00700,   0.00034,   0.00010,
     +    0.00004,   0.00022,   0.00036,
     +    0.08995,   0.01714,   0.00640,   0.00034,   0.00011,
     +    0.00005,   0.00022,   0.00036,
     +    0.08603,   0.01632,   0.00588,   0.00032,   0.00010,
     +    0.00005,   0.00020,   0.00032,
     +    0.08241,   0.01556,   0.00543,   0.00030,   0.00010,
     +    0.00005,   0.00019,   0.00030 /
        data ((f(i,38,k),i=1,5),f(7,38,k),f(6,38,k),f(8,38,k),k=1,7)/
     +    0.24135,   0.04971,   0.06204,   0.00073,  -0.00002,
     +   -0.00005,   0.00039,   0.00086,
     +    0.21273,   0.04315,   0.04573,   0.00081,   0.00003,
     +   -0.00003,   0.00034,   0.00077,
     +    0.19057,   0.03799,   0.03499,   0.00059,   0.00003,
     +   -0.00001,   0.00031,   0.00068,
     +    0.17220,   0.03393,   0.02754,   0.00059,   0.00003,
     +   -0.00006,   0.00029,   0.00058,
     +    0.15713,   0.03060,   0.02221,   0.00051,   0.00004,
     +   -0.00006,   0.00025,   0.00049,
     +    0.14434,   0.02781,   0.01828,   0.00049,   0.00008,
     +   -0.00002,   0.00026,   0.00045,
     +    0.13327,   0.02546,   0.01528,   0.00046,   0.00007,
     +   -0.00002,   0.00025,   0.00045 /
        data ((f(i,38,k),i=1,5),f(7,38,k),f(6,38,k),f(8,38,k),k=8,14)/
     +    0.12377,   0.02343,   0.01295,   0.00040,   0.00010,
     +   -0.00001,   0.00022,   0.00037,
     +    0.11541,   0.02167,   0.01112,   0.00036,   0.00010,
     +    0.00001,   0.00021,   0.00034,
     +    0.10797,   0.02013,   0.00964,   0.00033,   0.00009,
     +    0.00003,   0.00021,   0.00036,
     +    0.10139,   0.01876,   0.00844,   0.00029,   0.00010,
     +    0.00003,   0.00021,   0.00036,
     +    0.09562,   0.01756,   0.00746,   0.00022,   0.00009,
     +    0.00002,   0.00021,   0.00033,
     +    0.09020,   0.01647,   0.00665,   0.00026,   0.00009,
     +    0.00002,   0.00019,   0.00026,
     +    0.08536,   0.01550,   0.00596,   0.00026,   0.00009,
     +    0.00003,   0.00017,   0.00023 /
        data ((f(i,38,k),i=1,5),f(7,38,k),f(6,38,k),f(8,38,k),k=15,19)/
     +    0.08106,   0.01462,   0.00538,   0.00019,   0.00009,
     +    0.00004,   0.00017,   0.00026,
     +    0.07706,   0.01383,   0.00488,   0.00019,   0.00009,
     +    0.00004,   0.00017,   0.00024,
     +    0.07340,   0.01312,   0.00445,   0.00018,   0.00009,
     +    0.00005,   0.00016,   0.00022,
     +    0.07003,   0.01245,   0.00408,   0.00017,   0.00009,
     +    0.00005,   0.00015,   0.00021,
     +    0.06694,   0.01184,   0.00376,   0.00016,   0.00008,
     +    0.00005,   0.00014,   0.00019 /
        data ((f(i,39,k),i=1,5),f(7,39,k),f(6,39,k),f(8,39,k),k=1,7)/
     +    0.20680,   0.03990,   0.04722,   0.00049,  -0.00001,
     +   -0.00005,   0.00023,   0.00055,
     +    0.18100,   0.03435,   0.03429,   0.00049,   0.00001,
     +   -0.00006,   0.00021,   0.00046,
     +    0.16088,   0.03009,   0.02586,   0.00050,   0.00002,
     +   -0.00004,   0.00020,   0.00039,
     +    0.14469,   0.02671,   0.02013,   0.00037,   0.00008,
     +   -0.00001,   0.00020,   0.00041,
     +    0.13141,   0.02396,   0.01609,   0.00030,   0.00007,
     +   -0.00003,   0.00020,   0.00032,
     +    0.12022,   0.02167,   0.01313,   0.00023,   0.00006,
     +   -0.00002,   0.00017,   0.00028,
     +    0.11054,   0.01975,   0.01090,   0.00023,   0.00007,
     +   -0.00001,   0.00017,   0.00026 /
        data ((f(i,39,k),i=1,5),f(7,39,k),f(6,39,k),f(8,39,k),k=8,14)/
     +    0.10218,   0.01812,   0.00917,   0.00023,   0.00008,
     +    0.00001,   0.00017,   0.00024,
     +    0.09484,   0.01669,   0.00783,   0.00023,   0.00006,
     +   -0.00001,   0.00013,   0.00018,
     +    0.08844,   0.01545,   0.00676,   0.00020,   0.00006,
     +    0.00000,   0.00012,   0.00017,
     +    0.08282,   0.01436,   0.00589,   0.00015,   0.00007,
     +    0.00002,   0.00014,   0.00018,
     +    0.07778,   0.01338,   0.00519,   0.00013,   0.00008,
     +    0.00004,   0.00014,   0.00015,
     +    0.07324,   0.01253,   0.00461,   0.00010,   0.00007,
     +    0.00003,   0.00012,   0.00015,
     +    0.06912,   0.01176,   0.00412,   0.00009,   0.00007,
     +    0.00003,   0.00011,   0.00014 /
        data ((f(i,39,k),i=1,5),f(7,39,k),f(6,39,k),f(8,39,k),k=15,19)/
     +    0.06534,   0.01106,   0.00370,   0.00011,   0.00007,
     +    0.00002,   0.00011,   0.00013,
     +    0.06198,   0.01044,   0.00335,   0.00010,   0.00006,
     +    0.00003,   0.00011,   0.00013,
     +    0.05893,   0.00987,   0.00304,   0.00008,   0.00006,
     +    0.00003,   0.00011,   0.00013,
     +    0.05609,   0.00935,   0.00279,   0.00006,   0.00005,
     +    0.00003,   0.00009,   0.00013,
     +    0.05349,   0.00887,   0.00256,   0.00005,   0.00005,
     +    0.00002,   0.00008,   0.00012 /
        data ((f(i,40,k),i=1,5),f(7,40,k),f(6,40,k),f(8,40,k),k=1,7)/
     +    0.17485,   0.03164,   0.03552,   0.00040,  -0.00006,
     +   -0.00010,   0.00008,   0.00037,
     +    0.15202,   0.02704,   0.02538,   0.00041,  -0.00002,
     +   -0.00006,   0.00011,   0.00030,
     +    0.13424,   0.02357,   0.01893,   0.00037,   0.00001,
     +   -0.00007,   0.00012,   0.00025,
     +    0.12024,   0.02082,   0.01458,   0.00022,  -0.00001,
     +   -0.00007,   0.00010,   0.00018,
     +    0.10850,   0.01858,   0.01155,   0.00022,   0.00002,
     +   -0.00004,   0.00010,   0.00022,
     +    0.09880,   0.01673,   0.00936,   0.00019,   0.00003,
     +   -0.00005,   0.00008,   0.00016,
     +    0.09043,   0.01519,   0.00771,   0.00021,   0.00004,
     +   -0.00002,   0.00010,   0.00014 /
        data ((f(i,40,k),i=1,5),f(7,40,k),f(6,40,k),f(8,40,k),k=8,14)/
     +    0.08328,   0.01387,   0.00646,   0.00018,   0.00004,
     +   -0.00002,   0.00009,   0.00014,
     +    0.07712,   0.01274,   0.00549,   0.00013,   0.00004,
     +    0.00000,   0.00008,   0.00013,
     +    0.07171,   0.01175,   0.00472,   0.00010,   0.00004,
     +    0.00000,   0.00008,   0.00012,
     +    0.06689,   0.01088,   0.00410,   0.00009,   0.00004,
     +    0.00000,   0.00007,   0.00011,
     +    0.06254,   0.01012,   0.00359,   0.00012,   0.00004,
     +    0.00001,   0.00006,   0.00012,
     +    0.05870,   0.00945,   0.00317,   0.00011,   0.00003,
     +    0.00001,   0.00006,   0.00011,
     +    0.05528,   0.00884,   0.00283,   0.00010,   0.00003,
     +    0.00000,   0.00006,   0.00010 /
        data ((f(i,40,k),i=1,5),f(7,40,k),f(6,40,k),f(8,40,k),k=15,19)/
     +    0.05219,   0.00830,   0.00254,   0.00007,   0.00002,
     +    0.00001,   0.00006,   0.00009,
     +    0.04939,   0.00781,   0.00229,   0.00006,   0.00002,
     +    0.00001,   0.00005,   0.00007,
     +    0.04685,   0.00737,   0.00208,   0.00004,   0.00002,
     +    0.00000,   0.00005,   0.00006,
     +    0.04451,   0.00697,   0.00190,   0.00002,   0.00003,
     +    0.00001,   0.00005,   0.00008,
     +    0.04236,   0.00660,   0.00174,   0.00002,   0.00003,
     +    0.00002,   0.00005,   0.00009 /
        data ((f(i,41,k),i=1,5),f(7,41,k),f(6,41,k),f(8,41,k),k=1,7)/
     +    0.14537,   0.02451,   0.02606,   0.00020,  -0.00003,
     +   -0.00004,   0.00006,   0.00020,
     +    0.12554,   0.02085,   0.01841,   0.00024,  -0.00001,
     +   -0.00003,   0.00006,   0.00017,
     +    0.11019,   0.01807,   0.01358,   0.00020,  -0.00003,
     +   -0.00004,   0.00006,   0.00017,
     +    0.09805,   0.01588,   0.01038,   0.00018,   0.00000,
     +   -0.00002,   0.00005,   0.00012,
     +    0.08810,   0.01412,   0.00815,   0.00017,   0.00000,
     +   -0.00001,   0.00005,   0.00009,
     +    0.07983,   0.01267,   0.00655,   0.00013,  -0.00001,
     +   -0.00002,   0.00003,   0.00011,
     +    0.07291,   0.01145,   0.00537,   0.00008,   0.00000,
     +   -0.00002,   0.00002,   0.00009 /
        data ((f(i,41,k),i=1,5),f(7,41,k),f(6,41,k),f(8,41,k),k=8,14)/
     +    0.06691,   0.01043,   0.00447,   0.00007,   0.00001,
     +   -0.00001,   0.00003,   0.00008,
     +    0.06168,   0.00954,   0.00378,   0.00007,   0.00001,
     +    0.00000,   0.00004,   0.00009,
     +    0.05712,   0.00878,   0.00323,   0.00008,   0.00001,
     +    0.00000,   0.00003,   0.00008,
     +    0.05311,   0.00812,   0.00279,   0.00010,   0.00001,
     +   -0.00001,   0.00003,   0.00006,
     +    0.04954,   0.00753,   0.00244,   0.00010,   0.00001,
     +    0.00001,   0.00003,   0.00005,
     +    0.04638,   0.00701,   0.00215,   0.00008,   0.00001,
     +    0.00000,   0.00003,   0.00006,
     +    0.04354,   0.00654,   0.00191,   0.00007,   0.00001,
     +    0.00000,   0.00003,   0.00006 /
        data ((f(i,41,k),i=1,5),f(7,41,k),f(6,41,k),f(8,41,k),k=15,19)/
     +    0.04095,   0.00613,   0.00171,   0.00008,   0.00000,
     +    0.00000,   0.00002,   0.00006,
     +    0.03868,   0.00575,   0.00154,   0.00005,   0.00001,
     +    0.00000,   0.00002,   0.00006,
     +    0.03663,   0.00541,   0.00139,   0.00002,   0.00001,
     +    0.00000,   0.00002,   0.00006,
     +    0.03470,   0.00511,   0.00127,   0.00003,   0.00000,
     +    0.00000,   0.00002,   0.00005,
     +    0.03295,   0.00483,   0.00116,   0.00003,   0.00000,
     +    0.00000,   0.00002,   0.00004 /
        data ((f(i,42,k),i=1,5),f(7,42,k),f(6,42,k),f(8,42,k),k=1,7)/
     +    0.09524,   0.01389,   0.01314,   0.00006,  -0.00002,
     +   -0.00001,   0.00002,   0.00007,
     +    0.08096,   0.01162,   0.00898,   0.00009,   0.00000,
     +    0.00000,   0.00003,   0.00004,
     +    0.07009,   0.00994,   0.00644,   0.00006,  -0.00001,
     +   -0.00002,   0.00002,   0.00004,
     +    0.06161,   0.00862,   0.00480,   0.00003,   0.00001,
     +    0.00000,   0.00002,   0.00005,
     +    0.05476,   0.00758,   0.00370,  -0.00001,   0.00001,
     +    0.00000,   0.00002,   0.00006,
     +    0.04915,   0.00674,   0.00292,   0.00000,   0.00001,
     +    0.00000,   0.00002,   0.00005,
     +    0.04439,   0.00604,   0.00236,   0.00001,   0.00001,
     +    0.00000,   0.00002,   0.00004 /
        data ((f(i,42,k),i=1,5),f(7,42,k),f(6,42,k),f(8,42,k),k=8,14)/
     +    0.04040,   0.00545,   0.00194,   0.00001,   0.00002,
     +    0.00000,   0.00002,   0.00001,
     +    0.03694,   0.00494,   0.00162,   0.00000,   0.00002,
     +    0.00000,   0.00002,   0.00002,
     +    0.03392,   0.00451,   0.00137,   0.00001,   0.00001,
     +    0.00000,   0.00002,   0.00003,
     +    0.03130,   0.00414,   0.00118,   0.00002,   0.00001,
     +    0.00000,   0.00002,   0.00002,
     +    0.02902,   0.00381,   0.00102,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00003,
     +    0.02698,   0.00353,   0.00089,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02517,   0.00327,   0.00079,   0.00002,   0.00001,
     +    0.00000,   0.00001,   0.00001 /
        data ((f(i,42,k),i=1,5),f(7,42,k),f(6,42,k),f(8,42,k),k=15,19)/
     +    0.02355,   0.00305,   0.00070,   0.00002,   0.00001,
     +    0.00001,   0.00002,   0.00001,
     +    0.02210,   0.00285,   0.00063,   0.00002,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02079,   0.00267,   0.00056,   0.00002,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01961,   0.00250,   0.00051,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.01853,   0.00236,   0.00047,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001 /
        data ((f(i,43,k),i=1,5),f(7,43,k),f(6,43,k),f(8,43,k),k=1,7)/
     +    0.05737,   0.00716,   0.00596,   0.00004,  -0.00001,
     +   -0.00001,   0.00000,   0.00001,
     +    0.04794,   0.00589,   0.00393,   0.00005,  -0.00001,
     +   -0.00001,  -0.00001,   0.00001,
     +    0.04088,   0.00496,   0.00274,   0.00002,   0.00000,
     +   -0.00001,   0.00000,   0.00003,
     +    0.03548,   0.00425,   0.00200,   0.00000,   0.00000,
     +    0.00000,   0.00001,   0.00002,
     +    0.03116,   0.00369,   0.00151,  -0.00001,   0.00000,
     +    0.00000,   0.00001,   0.00001,
     +    0.02762,   0.00324,   0.00117,   0.00001,   0.00001,
     +    0.00000,   0.00001,   0.00001,
     +    0.02472,   0.00288,   0.00093,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00001 /
        data ((f(i,43,k),i=1,5),f(7,43,k),f(6,43,k),f(8,43,k),k=8,14)/
     +    0.02227,   0.00257,   0.00076,   0.00000,   0.00000,
     +    0.00000,   0.00001,   0.00000,
     +    0.02018,   0.00232,   0.00062,   0.00000,   0.00001,
     +    0.00000,   0.00001,   0.00000,
     +    0.01840,   0.00210,   0.00052,   0.00000,   0.00001,
     +    0.00000,   0.00000,   0.00000,
     +    0.01686,   0.00191,   0.00044,  -0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.01549,   0.00174,   0.00038,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01430,   0.00160,   0.00033,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.01326,   0.00148,   0.00029,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00001 /
        data ((f(i,43,k),i=1,5),f(7,43,k),f(6,43,k),f(8,43,k),k=15,19)/
     +    0.01233,   0.00137,   0.00026,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01150,   0.00127,   0.00023,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.01075,   0.00118,   0.00021,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.01008,   0.00110,   0.00019,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.00947,   0.00103,   0.00017,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,44,k),i=1,5),f(7,44,k),f(6,44,k),f(8,44,k),k=1,7)/
     +    0.03069,   0.00320,   0.00231,   0.00001,   0.00000,
     +   -0.00001,  -0.00001,   0.00000,
     +    0.02518,   0.00259,   0.00147,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.02117,   0.00215,   0.00100,   0.00000,   0.00001,
     +    0.00000,   0.00000,   0.00000,
     +    0.01808,   0.00181,   0.00071,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01564,   0.00156,   0.00052,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.01370,   0.00135,   0.00040,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01212,   0.00119,   0.00031,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,44,k),i=1,5),f(7,44,k),f(6,44,k),f(8,44,k),k=8,14)/
     +    0.01081,   0.00105,   0.00025,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00970,   0.00094,   0.00021,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00876,   0.00084,   0.00017,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00796,   0.00076,   0.00014,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00727,   0.00069,   0.00012,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00666,   0.00063,   0.00011,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00612,   0.00057,   0.00009,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,44,k),i=1,5),f(7,44,k),f(6,44,k),f(8,44,k),k=15,19)/
     +    0.00565,   0.00053,   0.00008,   0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00524,   0.00049,   0.00007,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00487,   0.00045,   0.00007,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00453,   0.00042,   0.00006,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00423,   0.00039,   0.00005,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,45,k),i=1,5),f(7,45,k),f(6,45,k),f(8,45,k),k=1,7)/
     +    0.01397,   0.00119,   0.00072,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.01120,   0.00094,   0.00044,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00922,   0.00076,   0.00029,  -0.00001,   0.00000,
     +    0.00000,   0.00000,   0.00001,
     +    0.00773,   0.00063,   0.00020,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00659,   0.00053,   0.00014,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00569,   0.00046,   0.00011,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00497,   0.00040,   0.00008,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,45,k),i=1,5),f(7,45,k),f(6,45,k),f(8,45,k),k=8,14)/
     +    0.00437,   0.00035,   0.00007,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00388,   0.00030,   0.00005,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00347,   0.00027,   0.00004,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00312,   0.00024,   0.00004,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00282,   0.00022,   0.00003,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00256,   0.00020,   0.00003,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00233,   0.00018,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,45,k),i=1,5),f(7,45,k),f(6,45,k),f(8,45,k),k=15,19)/
     +    0.00214,   0.00016,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00196,   0.00015,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00181,   0.00014,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00167,   0.00013,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00155,   0.00012,   0.00001,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,46,k),i=1,5),f(7,46,k),f(6,46,k),f(8,46,k),k=1,7)/
     +    0.00108,   0.00005,   0.00002,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00081,   0.00004,   0.00001,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00063,   0.00003,   0.00001,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00051,   0.00002,   0.00001,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00041,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00034,   0.00002,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00029,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,46,k),i=1,5),f(7,46,k),f(6,46,k),f(8,46,k),k=8,14)/
     +    0.00025,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00021,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00018,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00016,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00014,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00012,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00011,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
        data ((f(i,46,k),i=1,5),f(7,46,k),f(6,46,k),f(8,46,k),k=15,19)/
     +    0.00010,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00009,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00008,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00007,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000,
     +    0.00006,   0.00001,   0.00000,   0.00000,   0.00000,
     +    0.00000,   0.00000,   0.00000 /
      data xx/1.d-5,2.d-5,4.d-5,6.d-5,8.d-5,
     .        1.d-4,2.d-4,4.d-4,6.d-4,8.d-4,
     .        1.d-3,2.d-3,4.d-3,6.d-3,8.d-3,
     .        1.d-2,2.d-2,4.d-2,6.d-2,8.d-2,
     .     .1d0,.125d0,.15d0,.175d0,.2d0,.225d0,.25d0,.275d0,
     .     .3d0,.325d0,.35d0,.375d0,.4d0,.425d0,.45d0,.475d0,
     .     .5d0,.525d0,.55d0,.575d0,.6d0,.65d0,.7d0,.75d0,
     .     .8d0,.9d0,1.d0/
      data xmin,xmax,qsqmin,qsqmax/1.d-5,1.d0,5.d0,1310720.d0/
      data n0/2,5,5,9,0,0,9,9/
      data zerod/0.d0/,poned/0.1d0/,oned/1.d0/,onedo/1.1d0/,twod/2.d0/
      data init/0/
c
      xsave=x
c
      if(init.ne.0) goto 10
      init=1
      do 20 n=1,nx-1
      do 20 m=1,19
c     read(92,50)f(1,n,m),f(2,n,m),f(3,n,m),f(4,n,m),f(5,n,m),f(7,n,m),
c    .          f(6,n,m),f(8,n,m)
c 1=uv 2=dv 3=glue 4=ubar 5=cbar 7=bbar 6=sbar 8=dbar
         do 25 i=1,8
  25     f(i,n,m)=f(i,n,m)/(oned-xx(n))**n0(i)
  20  continue
      do 31 j=1,ntenth-1
      xx(j)= log10(xx(j))+onedo
      do 31 i=1,8
      if(i.eq.7) go to 31
      do 30 k=1,19
  30  f(i,j,k)= log(f(i,j,k))*f(i,ntenth,k)/ log(f(i,ntenth,k))
  31  continue
  50  format(8f10.5)
      do 40 i=1,8
      do 40 m=1,19
  40  f(i,nx,m)=zerod
  10  continue
      if(x.lt.xmin) x=xmin
      if(x.gt.xmax) x=xmax
c furler 8.8.94
c   for compability with venus is scale = q2
c     qsq=scale**2
      qsq=scale
      if(qsq.lt.qsqmin) qsq=qsqmin
      if(qsq.gt.qsqmax) qsq=qsqmax
      xxx=x
      if(x.lt.poned) xxx= log10(x)+onedo
      n=0
  70  n=n+1
      if(xxx.gt.xx(n+1)) goto 70
      a=(xxx-xx(n))/(xx(n+1)-xx(n))
      rm= log(qsq/qsqmin)/ log(twod)
      b=rm-aint(rm)
      m=1+  int(rm)
      do 60 i=1,8
      g(i)= (oned-a)*(oned-b)*f(i,n,m)+(oned-a)*b*f(i,n,m+1)
     .    + a*(oned-b)*f(i,n+1,m)  + a*b*f(i,n+1,m+1)
      if(n.ge.ntenth) goto 65
      if(i.eq.7) goto 65
          fac=(oned-b)*f(i,ntenth,m)+b*f(i,ntenth,m+1)
          g(i)=fac**(g(i)/fac)
  65  continue
      g(i)=g(i)*(oned-x)**n0(i)
  60  continue
      upv=g(1)
      dnv=g(2)
      usea=g(4)
      dsea=g(8)
      str=g(6)
      chm=g(5)
      glu=g(3)
      bot=g(7)
c
      x=xsave
c
      return
      end
c end sty2??

      function stzzse(x)
c     sea structure fctn
c     new structure functions included (e. furler, 4.11.94)
      common /cipio/ipio
      common /paroa/iopadi,q2soft
      double precision dx,dscale,dupv,ddnv,dusea,ddsea,dstr,dchm,dbot,
     *                 dgl
      q2=q2soft
      if(ipio.eq.0)then
        if(iopadi.eq.0) then
          stzzse=stospr(x,q2)
        elseif (iopadi.eq.1) then
          call sty1aa(q2,x,z,s)
          z = amin1(1.,z)
          stzzse=(sty1us(z,q2,s)+sty1ss(z,q2,s)) / z
        else
          dx=dble(x)
          dscale=dble(q2)
          if(iopadi.eq.2)
     *      call sty2di(dx,dscale,dupv,ddnv,dusea,ddsea,dstr,
     *                  dchm,dbot,dgl)
          if(iopadi.eq.3)
     *      call sty2ms(dx,dscale,dupv,ddnv,dusea,ddsea,dstr,
     *                  dchm,dbot,dgl)
          usea=real(dusea)
          dsea=real(ddsea)
          if (x.gt.0.) stzzse=(usea + dsea) / x
        endif
      elseif(ipio.eq.1)then
        stzzse=stospi(x,0.)
      else
        call utstop('stzzse: ipio must be 0 or 1!          ')
      endif
      return
      end
 
      function stzzvl(x)
c     valence structure fctn
c     new structure functions included (e. furler, 4.9.94)
      common /cipio/ipio
      common /paroa/iopadi,q2soft
      double precision dx,dscale,dupv,ddnv,dusea,ddsea,dstr,dchm,dbot,
     *                 dgl
      q2=q2soft
      if(ipio.eq.0)then
        if(iopadi.eq.0) then
          stzzvl=stovpr(x,q2)/3
        elseif (iopadi.eq.1) then
          call sty1aa(q2,x,z,s)
          z = amin1(1.,z)
          stzzvl=(sty1vv(z,q2,s)) / z
        else
          dx=dble(x)
          dscale=dble(q2)
          if(iopadi.eq.2)
     *      call sty2di(dx,dscale,dupv,ddnv,dusea,ddsea,dstr,
     *                  dchm,dbot,dgl)
          if(iopadi.eq.3)
     *      call sty2ms(dx,dscale,dupv,ddnv,dusea,ddsea,dstr,
     *                  dchm,dbot,dgl)
          upv=real(dupv)
          dnv=real(ddnv)
          if (x.gt.0.) stzzvl=(upv + dnv) / x
        endif
      elseif(ipio.eq.1)then
        stzzvl=stovpi(x,0.)/3
      else
        call utstop('stzzvl: ipio must be 0 or 1!            ')
      endif
      return
      end
 
c-----------------------------------------------------------------------
      function utacos(x)
c     returns acos(x) for -1 <= x <= 1 , acos(+-1) else
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      argum=x
      if(x.lt.-1.)then
      if(ish.ge.90)then
      call utmsg('utacos')
      write(ifch,*)'*****  argum = ',argum,' set -1'
      call utmsgf
      endif
      argum=-1.
      elseif(x.gt.1.)then
      if(ish.ge.90)then
      call utmsg('utacos')
      write(ifch,*)'*****  argum = ',argum,' set 1'
      call utmsgf
      endif
      argum=1.
      endif
      utacos=acos(argum)
      return
      end
 
c----------------------------------------------------------------------
      function utamnu(keux,kedx,kesx,kecx,modus)
c     returns min mass of droplet with given u,d,s,c content
c     keux: net u quark number
c     kedx: net d quark number
c     kesx: net s quark number
c     kecx: net c quark number
c     modus:0,1,2,3,4,5,6
c----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/csjcga/amnull,nsuha(7),asuha(7),entrpy,amegam,nopha
      common/paro3/uentro,sigppe,sigppd,asuhax(7),asuhay(7),omega
      amnull=0.
 
      do 1 i=1,7
      if(modus.eq.0)asuha(i)=asuhay(i)
      if(modus.eq.1.and.nopha.ge.1)asuha(i)=asuhay(i)
      if(modus.eq.1.and.nopha.eq.0)asuha(i)=asuhax(i)
      if(modus.eq.2)asuha(i)=asuhay(i)
      if(modus.eq.3)asuha(i)=asuhay(i)
      if(modus.eq.4)asuha(i)=asuhax(i)    !two lowest multiplets
      if(modus.eq.5)asuha(i)=asuhay(i)    !lowest multiplet
      if(modus.eq.6)asuha(i)=sqrt(asuhay(i)**2+delmss**2)
1     nsuha(i)=0
      ke=iabs(keux+kedx+kesx+kecx)
 
      if(keux+kedx+kesx+kecx.ge.0)then
      keu=keux
      ked=kedx
      kes=kesx
      kec=kecx
      else
      keu=-keux
      ked=-kedx
      kes=-kesx
      kec=-kecx
      endif
 
      if(kec.ne.0)then
10    continue
      if(kec.lt.0)then
      kec=kec+1
      if(keu.gt.ked)then
      keu=keu-1
      else
      ked=ked-1
      endif
      amnull=amnull+1.87
      goto10
      endif
11    continue
      if(kec.gt.0)then
      kec=kec-1
      if(keu.lt.ked)then
      keu=keu+1
      else
      ked=ked+1
      endif
      amnull=amnull+1.87
      goto11
      endif
      endif
 
5     continue
      if(kes.lt.0)then
      amnull=amnull+asuha(6)
      if(keu.ge.ked)then
      keu=keu-1
      else
      ked=ked-1
      endif
      kes=kes+1
      goto5
      endif
 
6     continue
      if(ked.lt.0)then
      if(keu.ge.kes)then
      amnull=amnull+asuha(5)
      keu=keu-1
      else
      amnull=amnull+asuha(6)
      kes=kes-1
      endif
      ked=ked+1
      goto6
      endif
 
7     continue
      if(keu.lt.0)then
      if(ked.ge.kes)then
      amnull=amnull+asuha(5)
      ked=ked-1
      else
      amnull=amnull+asuha(6)
      kes=kes-1
      endif
      keu=keu+1
      goto7
      endif
 
      if(keu+ked+kes+kec.ne.ke)
     *call utstop('utamnu: sum_kei /= ke                   ')
      keq=keu+ked
      keqx=keq
 
      if(modus.eq.2.and.ke.gt.3)then
      call smasst(keu,ked,kes,kec,a,y,z)
      amnuz=smass(a,y,z)
      endif
 
      amnux=0
 
      i=4
2     i=i-1
3     continue
      if((4-i)*kes.gt.(i-1)*keq)then
      amnux=amnux+asuha(1+i)
      kes=kes-i
      keq=keq-3+i
      if(kes.lt.0)
     *call utstop('utamnu: negative kes                     ')
      if(keq.lt.0)
     *call utstop('utamnu: negative keq                     ')
      goto3
      endif
      if(i.gt.1)goto2
 
      if(keqx.gt.keq)then
      do 8 k=1,keqx-keq
      if(keu.ge.ked)then
      keu=keu-1
      else
      ked=ked-1
      endif
8     continue
      endif
      if(keu+ked.ne.keq)
     *call utstop('utamnu: keu+ked /= keq                   ')
 
9     continue
      if(keu.gt.2*ked)then
      amnux=amnux+asuha(7)
      keu=keu-3
      if(keu.lt.0)
     *call utstop('utamnu: negative keu                     ')
      goto9
      endif
      if(ked.gt.2*keu)then
      amnux=amnux+asuha(7)
      ked=ked-3
      if(ked.lt.0)
     *call utstop('utamnu: negative ked                     ')
      goto9
      endif
 
      keq=keu+ked
      if(mod(keq,3).ne.0)
     *call utstop('utamnu: mod(keq,3) /= 0                  ')
      amnux=amnux+asuha(1)*keq/3
 
      amnu=amnux
      if(modus.eq.2.and.ke.gt.3)amnu=amin1(amnux,amnuz)
      amnull=amnull+amnu
 
      if(amnull.eq.0..and.modus.gt.0)amnull=asuha(5)
 
1000  utamnu=amnull
      return
      end
 
c-----------------------------------------------------------------------
      function utamnx(jcp,jcm)
c-----------------------------------------------------------------------
c returns minimum mass for the decay of jcp---jcm (by calling utamnu).
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer jcp(nflav,2),jcm(nflav,2)
 
      do i=1,nflav
      do j=1,2
      if(jcp(i,j).ne.0)goto1
      enddo
      enddo
      keu=jcm(1,1)-jcm(1,2)
      ked=jcm(2,1)-jcm(2,2)
      kes=jcm(3,1)-jcm(3,2)
      kec=jcm(4,1)-jcm(4,2)
      utamnx=utamnu(keu,ked,kes,kec,5)
      return
1     continue
 
      do i=1,nflav
      do j=1,2
      if(jcm(i,j).ne.0)goto2
      enddo
      enddo
      keu=jcp(1,1)-jcp(1,2)
      ked=jcp(2,1)-jcp(2,2)
      kes=jcp(3,1)-jcp(3,2)
      kec=jcp(4,1)-jcp(4,2)
      utamnx=utamnu(keu,ked,kes,kec,5)
      return
2     continue
 
      keu=jcp(1,1)-jcp(1,2)
      ked=jcp(2,1)-jcp(2,2)
      kes=jcp(3,1)-jcp(3,2)
      kec=jcp(4,1)-jcp(4,2)
      ke=keu+ked+kes+kec
      if(mod(ke+1,3).eq.0)then
      keu=keu+1
      elseif(mod(ke-1,3).eq.0)then
      keu=keu-1
      else
      call utstop('utamnx: no singlet possible (1)         ')
      endif
      amms=utamnu(keu,ked,kes,kec,5)
      keu=jcm(1,1)-jcm(1,2)
      ked=jcm(2,1)-jcm(2,2)
      kes=jcm(3,1)-jcm(3,2)
      kec=jcm(4,1)-jcm(4,2)
      ke=keu+ked+kes+kec
      if(mod(ke+1,3).eq.0)then
      keu=keu+1
      elseif(mod(ke-1,3).eq.0)then
      keu=keu-1
      else
      call utstop('utamnx: no singlet possible (2)         ')
      endif
      utamnx=amms+utamnu(keu,ked,kes,kec,5)
      return
      end
 
c-----------------------------------------------------------------------
      function utamny(jcp,jcm)
c-----------------------------------------------------------------------
c returns minimum mass of jcp+jcm (by calling utamnu).
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer jcp(nflav,2),jcm(nflav,2),jc(nflav,2)
      do 7 nf=1,nflav
      jc(nf,1)=jcp(nf,1)+jcm(nf,1)
7     jc(nf,2)=jcp(nf,2)+jcm(nf,2)
      keu=jc(1,1)-jc(1,2)
      ked=jc(2,1)-jc(2,2)
      kes=jc(3,1)-jc(3,2)
      kec=jc(4,1)-jc(4,2)
      utamny=utamnu(keu,ked,kes,kec,5)
      return
      end

c-----------------------------------------------------------------------
      function utamnz(jc,modus)
c-----------------------------------------------------------------------
c returns minimum mass of jc (by calling utamnu).
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer jc(nflav,2)
      keu=jc(1,1)-jc(1,2)
      ked=jc(2,1)-jc(2,2)
      kes=jc(3,1)-jc(3,2)
      kec=jc(4,1)-jc(4,2)
      utamnz=utamnu(keu,ked,kes,kec,modus)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utamst(sts,am,amin,iret)
c     input:  sts   = string  (single)
c     output: am    = mass
c             amin  = minimum mass
c             iret  = return code (=3 if am**2.lt.amin**2, 0 else)
c-----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      parameter (nsi=6,nflav=6)
      real sts(nsi,2)
      integer ic1(2),ic2(2),jc1(nflav,2),jc2(nflav,2),jc(nflav,2)
      iret=0
      am2= (sts(4,1)+sts(4,2))**2 - (sts(3,1)+sts(3,2))**2 -
     * (sts(2,1)+sts(2,2))**2 - (sts(1,1)+sts(1,2))**2
      am=-am2
      do 1 i=1,2
      ic1(i)=nint(sts(4+i,1))
1     ic2(i)=nint(sts(4+i,2))
      call iddeco(ic1,jc1)
      call iddeco(ic2,jc2)
      do 2 nf=1,nflav
      jc(nf,1)=jc1(nf,1)+jc2(nf,1)
      jc(nf,2)=jc1(nf,2)+jc2(nf,2)
      if(nf.gt.4.and.(jc(nf,1).ne.0.or.jc(nf,2).ne.0))
     *call utstop('utamst: flavour > 4                     ')
2     continue
      keu=jc(1,1)-jc(1,2)
      ked=jc(2,1)-jc(2,2)
      kes=jc(3,1)-jc(3,2)
      kec=jc(4,1)-jc(4,2)
      amin=utamnu(keu,ked,kes,kec,4)
      if(am2.lt.amin**2)then
      iret=3
      return
      else
      am=sqrt(am2)
      endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utar(i1,i2,i3,x0,x1,x2,x3,xx)
c     returns the array xx with xx(1)=x0 <= xx(i) <= xx(i3)=x3
c-----------------------------------------------------------------------
      real xx(i3)
      do 1 i=1,i1-1
  1   xx(i)=x0+(i-1.)/(i1-1.)*(x1-x0)
      do 2 i=i1,i2-1
  2   xx(i)=x1+(i-i1*1.)/(i2-i1*1.)*(x2-x1)
      do 3 i=i2,i3
  3   xx(i)=x2+(i-i2*1.)/(i3-i2*1.)*(x3-x2)
      return
      end

c---------------------------------------------------------------------
      subroutine utaxis(i,j,a1,a2,a3)
c     calculates the axis defined by the ptls i,j in the i,j cm system
c---------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt                                     
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      a1=0
      a2=0
      a3=1
      pi1=pptl(1,i) 
      pi2=pptl(2,i) 
      pi3=pptl(3,i) 
      pi4=pptl(4,i) 
      pj1=pptl(1,j) 
      pj2=pptl(2,j) 
      pj3=pptl(3,j) 
      pj4=pptl(4,j) 
      p1=pi1+pj1
      p2=pi2+pj2
      p3=pi3+pj3
      p4=pi4+pj4
      p5=sqrt(p4**2-p3**2-p2**2-p1**2)
      call utlobo(1,p1,p2,p3,p4,p5,pi1,pi2,pi3,pi4)                                
      call utlobo(1,p1,p2,p3,p4,p5,pj1,pj2,pj3,pj4)                                
           err=(pi1+pj1)**2+(pi2+pj2)**2+(pi3+pj3)**2 
           if(err.gt.1e-3)then
      call utmsg('utaxis')                                                      
      write(ifch,*)'*****  err=',err
      write(ifch,*)'pi:',pi1,pi2,pi3,pi4
      write(ifch,*)'pj:',pj1,pj2,pj3,pj4
      call utmsgf                                                               
           endif
      a=sqrt( (pj1-pi1)**2 + (pj2-pi2)**2 + (pj3-pi3)**2 )
      if(a.eq.0.)return
      a1=(pi1-pj1)/a
      a2=(pi2-pj2)/a
      a3=(pi3-pj3)/a
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utchm(arp,arm,ii)
c     checks whether arp**2=0 and arm**2=0.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      double precision arp(4),arm(4),difp,difm
      difp=arp(4)**2-arp(1)**2-arp(2)**2-arp(3)**2
      difm=arm(4)**2-arm(1)**2-arm(2)**2-arm(3)**2
      if(dabs(difp).gt.1e-3*arp(4)**2
     *.or.dabs(difm).gt.1e-3*arm(4)**2)then
      call utmsg('utchm ')
      write(ifch,*)'*****  mass non zero  -  ',ii
      write(ifch,*)'jet-mass**2`s:    ',difp,difm
      write(ifch,*)'energy**2`s:      ',arp(4)**2,arm(4)**2
      write(ifch,*)(sngl(arp(i)),i=1,4)
      write(ifch,*)(sngl(arm(i)),i=1,4)
      call utmsgf
      endif
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utclea(nptl0)
c     overwrites istptl=2 particles in /cptl/, reduces so nptl.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (myptl=1000,nzeta=60)
      common/c4ptl/iyptlx,iyptl(mxptl),weiptl(myptl,nzeta),iyiptl(myptl)
      integer newptl(mxptl),oldptl(mxptl),ii(mxptl)
 
      ish0=ish
      if(ishsub/100.eq.18)ish=mod(ishsub,100)
 
      if(ish.ge.92)then
      write(ifch,*)('-',l=1,68)
      write(ifch,*)'sr utclea. initial.'
      write(ifch,*)('-',l=1,68)
      do 34 n=1,nptl
      write(ifch,116)iorptl(n),jorptl(n),n,ifrptl(1,n),ifrptl(2,n)
     *,idptl(n),sqrt(pptl(1,n)**2+pptl(2,n)**2),pptl(3,n),pptl(5,n)
     *,istptl(n)
34    continue
116   format(1x,i4,i4,4x,i4,4x,i4,i4,i12,3x,3(e8.2,1x),i2,i2)
      endif

      ish=ish0
      if(iyptlx.gt.0)call utcley(2)
      ish0=ish
      if(ishsub/100.eq.18)ish=mod(ishsub,100)

      i=0
1     i=i+1
      if(i.gt.nptl)goto1000
      if(istptl(i).eq.2)goto2
      newptl(i)=i
      oldptl(i)=i
      goto1
 
2     i=i-1
      j=i
3     i=i+1
4     j=j+1
      if(j.gt.nptl)goto5
      newptl(j)=0
      if(istptl(j).eq.2)goto4
      newptl(j)=i
      oldptl(i)=j
      call utrepl(i,j)
      goto3
 
5     nptl=i-1
      if(nptl.eq.0)nptl0=0  
      if(nptl.eq.0)goto1000

20    n0=newptl(nptl0)
      if(n0.gt.0)then
      nptl0=n0
      else
      nptl0=nptl0-1
      if(nptl0.gt.0)goto20
      endif

      do 11 k=1,nptl
      io=iorptl(k)
      if(io.le.0)ii(k)=io
      if(io.gt.0)ii(k)=newptl(io)
11    continue
      do 12 k=1,nptl
12    iorptl(k)=ii(k)
 
      do 13 k=1,nptl
      jo=jorptl(k)
      if(jo.le.0)ii(k)=jo
      if(jo.gt.0)ii(k)=newptl(jo)
13    continue
      do 14 k=1,nptl
14    jorptl(k)=ii(k)
 
      do 15 k=1,nptl
      if1=ifrptl(1,k)
      if(if1.le.0)ii(k)=if1
      if(if1.gt.0)ii(k)=newptl(if1)
15    continue
      do 16 k=1,nptl
16    ifrptl(1,k)=ii(k)
 
      do 17 k=1,nptl
      if2=ifrptl(2,k)
      if(if2.le.0)ii(k)=if2
      if(if2.gt.0)ii(k)=newptl(if2)
17    continue
      do 18 k=1,nptl
18    ifrptl(2,k)=ii(k)
 
      do 19 k=1,nptl
      if(ifrptl(1,k).eq.0.and.ifrptl(2,k).gt.0)ifrptl(1,k)=ifrptl(2,k)
      if(ifrptl(2,k).eq.0.and.ifrptl(1,k).gt.0)ifrptl(2,k)=ifrptl(1,k)
19    continue
 
1000  continue
 
      if(ish.ge.92)then
      write(ifch,*)'before exiting subr utclea:'
      do 35 n=1,nptl
      write(ifch,116)iorptl(n),jorptl(n),n,ifrptl(1,n),ifrptl(2,n)
     *,idptl(n),sqrt(pptl(1,n)**2+pptl(2,n)**2),pptl(3,n),pptl(5,n)
     *,istptl(n)
35    continue
      endif
 
      ish=ish0
      return
      end

c-----------------------------------------------------------------------
      subroutine utcley(ist)
c     overwrites istptl=ist particles in /c4ptl/,/c5ptl/;reduces so iyptlx.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (myptl=1000,nzeta=60)
      common/c4ptl/iyptlx,iyptl(mxptl),weiptl(myptl,nzeta),iyiptl(myptl)
      parameter (nflav=6)
      common/c5ptl/iflptl(myptl,nzeta,nflav)
      integer irem(myptl)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 

      ish0=ish
      if(ishsub/100.eq.22)ish=mod(ishsub,100)
 
      if(ish.ge.92)write(ifch,*)'long droplets at taus=',sngl(ttaus)
      if(ish.ge.92)write(ifch,*)'before cleanup:'
      do 34 i=1,iyptlx
      n=iyiptl(i)
      zfi=xorptl(3,n)+(tivptl(2,n)-xorptl(4,n))*pptl(3,n)/pptl(4,n)
      call uttaus(zfi,tzfi,szfi)
      delt=tzfi-tivptl(2,n)
      irem(i)=0
      if(ist.eq.1.and.istptl(n).eq.1
     *.and.(ifrptl(1,n).eq.0.or.delt.gt.0.1))irem(i)=1
      if(ist.eq.2.and.istptl(n).eq.2)irem(i)=1
      if(ish.ge.92)
     *write(ifch,116)n,i
     *,(weiptl(i,nn),nn=nzeta/2-3,nzeta/2+4),istptl(n),ifrptl(1,n),delt
     *,irem(i)
34    continue
116   format(i6,i5,8f6.1,i3,i6,1x,e8.2,i3)

      iyptlw=iyptlx

      i=0
1     i=i+1
      if(i.gt.iyptlx)goto1000
      ni=iyiptl(i)
      if(irem(i).eq.1)goto2
      goto1
 
2     i=i-1
      j=i
3     i=i+1
      ni=iyiptl(i)
4     j=j+1
      if(j.gt.iyptlx)goto5
      if(irem(j).eq.1)goto4
      nj=iyiptl(j)
      iyptl(ni)=j
      iyptl(nj)=i
      iyiptl(i)=nj
      iyiptl(j)=ni
      do nn=1,nzeta
      weiptl(i,nn)=weiptl(j,nn)
      weiptl(j,nn)=0
      do nf=1,nflav
      iflptl(i,nn,nf)=iflptl(j,nn,nf)      
      iflptl(j,nn,nf)=0
      enddo
      enddo
      goto3
 
5     iyptlx=i-1

1000  continue

      if(iyptlx+1.le.iyptlw)then
      do i=iyptlx+1,iyptlw
      ni=iyiptl(i)
      iyptl(ni)=0
      iyiptl(i)=0
      enddo
      endif

      if(ish.ge.92)write(ifch,*)'after cleanup:'
      if(ish.ge.92)then
      do 35 i=1,iyptlx
      n=iyiptl(i)
      write(ifch,117)n,i
     *,(weiptl(i,nn),nn=nzeta/2-3,nzeta/2+4),istptl(n)
     *,irem(i)
117   format(i6,i5,8f6.1,i3,i3)
35    continue
      endif

      ish=ish0

      return
      end

c---------------------------------------------------------------------
      subroutine utfit(x,y,ndata,sig,mwt,a,b,siga,sigb,chi2,q)
c---------------------------------------------------------------------
c linear fit to data 
c input:
c    ndata: nr of data points
c    x(),y(),sig(): data
c    mwt: unweighted (0) or weighted (else) data points
c output:
c    a,b: parameters of linear fit a+b*x
c---------------------------------------------------------------------   
      INTEGER mwt,ndata
      REAL a,b,chi2,q,siga,sigb,sig(ndata),x(ndata),y(ndata)
CU    USES utgmq 
      INTEGER i
      REAL sigdat,ss,st2,sx,sxoss,sy,t,wt,utgmq
      sx=0.
      sy=0.
      st2=0.
      b=0.
      if(mwt.ne.0) then
        ss=0.
        do 11 i=1,ndata
          wt=1./(sig(i)**2)
          ss=ss+wt
          sx=sx+x(i)*wt
          sy=sy+y(i)*wt
11      continue
      else
        do 12 i=1,ndata
          sx=sx+x(i)
          sy=sy+y(i)
12      continue
        ss=float(ndata)
      endif
      sxoss=sx/ss
      if(mwt.ne.0) then
        do 13 i=1,ndata
          t=(x(i)-sxoss)/sig(i)
          st2=st2+t*t
          b=b+t*y(i)/sig(i)
13      continue
      else
        do 14 i=1,ndata
          t=x(i)-sxoss
          st2=st2+t*t
          b=b+t*y(i)
14      continue
      endif
      b=b/st2
      a=(sy-sx*b)/ss
      siga=sqrt((1.+sx*sx/(ss*st2))/ss)
      sigb=sqrt(1./st2)
      chi2=0.
      if(mwt.eq.0) then
        do 15 i=1,ndata
          chi2=chi2+(y(i)-a-b*x(i))**2
15      continue
        q=1.
        sigdat=sqrt(chi2/(ndata-2))
        siga=siga*sigdat
        sigb=sigb*sigdat
      else
        do 16 i=1,ndata
          chi2=chi2+((y(i)-a-b*x(i))/sig(i))**2
16      continue
        q=utgmq(0.5*(ndata-2),0.5*chi2)
      endif
      return
      END
 
c----------------------------------------------------------------------
      subroutine utfost(sia,sib,sts)
c     input :  sia = string end
c              sib = string end
c     output : sts = string (single)
c----------------------------------------------------------------------
      parameter (nsi=6)
      real sia(nsi),sib(nsi),sts(nsi,2)
      do 1 n=1,nsi
      sts(n,1)=sia(n)
1     sts(n,2)=sib(n)
      return
      end

c-----------------------------------------------------------------------
      function utgam1(x)
c     gamma fctn
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      dimension c(13)
      data c
     1/ 0.00053 96989 58808, 0.00261 93072 82746, 0.02044 96308 23590,
     2  0.07309 48364 14370, 0.27964 36915 78538, 0.55338 76923 85769,
     3  0.99999 99999 99998,-0.00083 27247 08684, 0.00469 86580 79622,
     4  0.02252 38347 47260,-0.17044 79328 74746,-0.05681 03350 86194,
     5  1.13060 33572 86556/
      utgam1=0
      z=x
      if(x .gt. 0.0) goto1
      if(x .eq. aint(x)) goto5
      z=1.0-z
    1 f=1.0/z
      if(z .le. 1.0) goto4
      f=1.0
    2 continue
      if(z .lt. 2.0) goto3
      z=z-1.0
      f=f*z
      goto2
    3 z=z-1.0
    4 utgam1=
     1 f*((((((c(1)*z+c(2))*z+c(3))*z+c(4))*z+c(5))*z+c(6))*z+c(7))/
     2   ((((((c(8)*z+c(9))*z+c(10))*z+c(11))*z+c(12))*z+c(13))*z+1.0)
      if(x .gt. 0.0) return
      utgam1=3.141592653589793/(sin(3.141592653589793*x)*utgam1)
      return
    5 write(ifch,10)x
   10 format(1x,'argument of gamma fctn = ',e20.5)
      call utstop('utgam1 : negative integer argument        ')
      end

c-----------------------------------------------------------------------
      double precision function utgam2(x)
c-----------------------------------------------------------------------
c  gamma fctn
c  double precision
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      double precision c(13),x,z,f
      data c
     1/ 0.00053 96989 58808, 0.00261 93072 82746, 0.02044 96308 23590,
     2  0.07309 48364 14370, 0.27964 36915 78538, 0.55338 76923 85769,
     3  0.99999 99999 99998,-0.00083 27247 08684, 0.00469 86580 79622,
     4  0.02252 38347 47260,-0.17044 79328 74746,-0.05681 03350 86194,
     5  1.13060 33572 86556/
      utgam2=0
      z=x
      if(x .gt. 0.0d0) goto1
      if(x .eq. int(x)) goto5
      z=1.0d0-z
    1 f=1.0d0/z
      if(z .le. 1.0d0) goto4
      f=1.0d0
    2 continue
      if(z .lt. 2.0d0) goto3
      z=z-1.0d0
      f=f*z
      goto2
    3 z=z-1.0d0
    4 utgam2=
     1 f*((((((c(1)*z+c(2))*z+c(3))*z+c(4))*z+c(5))*z+c(6))*z+c(7))/
     2 ((((((c(8)*z+c(9))*z+c(10))*z+c(11))*z+c(12))*z+c(13))*z+1.0d0)       
      if(x .gt. 0.0d0) return
      utgam2=3.141592653589793d0/(sin(3.141592653589793d0*x)*utgam2)
      return
    5 write(ifch,10)sngl(x)
   10 format(1x,'argument of gamma fctn = ',e20.5)
      call utstop('utgam2 : negative integer argument        ')
      end

c---------------------------------------------------------------------
      subroutine utgcf(gammcf,a,x,gln)
c---------------------------------------------------------------------
      INTEGER ITMAX
      REAL a,gammcf,gln,x,EPS,FPMIN
      PARAMETER (ITMAX=100,EPS=3.e-7,FPMIN=1.e-30)
CU    USES utgmln
      INTEGER i
      REAL an,b,c,d,del,h,utgmln
      gln=utgmln(a)
      b=x+1.-a
      c=1./FPMIN
      d=1./b
      h=d
      do 11 i=1,ITMAX
        an=-i*(i-a)
        b=b+2.
        d=an*d+b
        if(abs(d).lt.FPMIN)d=FPMIN
        c=b+an/c
        if(abs(c).lt.FPMIN)c=FPMIN
        d=1./d
        del=d*c
        h=h*del
        if(abs(del-1.).lt.EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in utgcf'
1     gammcf=exp(-x+a*log(x)-gln)*h
      return
      END

c---------------------------------------------------------------------
      function utgmln(xx)
c---------------------------------------------------------------------
      REAL utgmln,xx
      INTEGER j
      DOUBLE PRECISION ser,stp,tmp,x,y,cof(6)
      SAVE cof,stp
      DATA cof,stp/76.18009172947146d0,-86.50532032941677d0,
     *24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     *-.5395239384953d-5,2.5066282746310005d0/
      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue
      utgmln=tmp+log(stp*ser/x)
      return
      END

c---------------------------------------------------------------------
      function utgmq(a,x)
c---------------------------------------------------------------------
      REAL a,utgmq,x
CU    USES utgcf,utgser
      REAL gammcf,gamser,gln
      if(x.lt.0..or.a.le.0.)pause 'bad arguments in utgmq'
      if(x.lt.a+1.)then
        call utgser(gamser,a,x,gln)
        utgmq=1.-gamser
      else
        call utgcf(gammcf,a,x,gln)
        utgmq=gammcf
      endif
      return
      END

c---------------------------------------------------------------------
      subroutine utgser(gamser,a,x,gln)
c---------------------------------------------------------------------
      INTEGER ITMAX
      REAL a,gamser,gln,x,EPS
      PARAMETER (ITMAX=100,EPS=3.e-7)
CU    USES utgmln
      INTEGER n
      REAL ap,del,sum,utgmln
      gln=utgmln(a)
      if(x.le.0.)then
        if(x.lt.0.)pause 'x < 0 in utgser'
        gamser=0.
        return
      endif
      ap=a
      sum=1./a
      del=sum
      do 11 n=1,ITMAX
        ap=ap+1.
        del=del*x/ap
        sum=sum+del
        if(abs(del).lt.abs(sum)*EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in utgser'
1     gamser=sum*exp(-x+a*log(x)-gln)
      return
      END
 
c----------------------------------------------------------------------
      subroutine uthist(x1,x2,y1,y2,n,x,y,lity,lilo,text1,text2,text3)
c     writes arrays x,y in histo-format
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      real x(n),y(n)
      character*50 text1,text2,text3
      character lity*3,lilo*6
      write(ifhi,100)('-',i=1,69)
      write(ifhi,100)('-',i=1,69)
100   format(1x,1h',69a1,1h')
      write(ifhi,101)x1,x2,0.,y1,y2,0.,'_____',0
     *,'___',lity,'___','_______',lilo,0.,3,n,0.
101   format(1x,2f9.2,4f8.2,2x,1h',a5,1h',i11
     */4x,1h',a3,1h',3x,1h',a3,a3,1h',3x,1h',a7,a6,1h',f8.2,2i6,f10.4)
      write(ifhi,100)('-',i=1,69)
      write(ifhi,102)0.,0.,text1
      write(ifhi,102)0.,0.,text2
      write(ifhi,102)0.,0.,text3
102   format(4x,f4.2,2x,f4.2,2x,1h',a50,1h')
      write(ifhi,100)('-',i=1,69)
      do 36 k=1,n
36    write(ifhi,103)k,x(k),0.,y(k),0.,0.
103   format(4x,i5,2x,5e12.4)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine uthsea
c  creates histogram of sea structure fctn
c-----------------------------------------------------------------------
      parameter (nstru=2049)
      common/stru/xstru(nstru),wstru(nstru)
     *,qvap(nstru),qvat(nstru),qsep(nstru),qset(nstru)
      real xar(1000),yar(1000)
      dx=1
      x1=-1
      do 3 k=1,2
      dx=dx/1000.
      x1=x1-3
      do 2 n=1,1000
      yar(n)=0
2     xar(n)=-dx/2+n*dx
      do 1 i=1,100000
      x=utinvt(nstru,xstru,qsep,rangen()*qsep(nstru))
      n=1+x/dx
      if(n.le.1000)yar(n)=yar(n)+1
1     continue
      call uthist(x1,x1+4,0.,5.,1000,xar,yar,'poc','loglog'
     *,'xaxis x                         $                 '
     *,'yaxis counts                    $                 '
     *,'title sea quark structure fctn                    ')
3     continue
      return
      end
 
c-----------------------------------------------------------------------
      function utinvt(n,x,q,y)
c     returns x with y=q(x)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cutinv/lutinv
      real x(n),q(n)
      if(q(n).eq.0.)
     *call utstop('utinvt: q(n)=0                          ')
           if(y.lt.0.)then
      if(ish.ge.90)then
      call utmsg('utinvt')
      write(ifch,*)'*****  y=',y,' < 0'
      call utmsgf
      endif
      y=0.
           elseif(y.gt.q(n))then
      if(ish.ge.90)then
      call utmsg('utinvt')
      write(ifch,*)'*****  y=',y,' > ',q(n)
      call utmsgf
      endif
      y=q(n)
           endif
      lu=1
      lo=n
1     lz=(lo+lu)/2
      if((q(lu).le.y).and.(y.le.q(lz)))then
      lo=lz
      elseif((q(lz).lt.y).and.(y.le.q(lo)))then
      lu=lz
      else
      write(ifch,*)'q(1),y,q(n):',q(1),y,q(n)
      write(ifch,*)'lu,lz,lo:',lu,lz,lo
      write(ifch,*)'q(lu),q(lz),q(lo):',q(lu),q(lz),q(lo)
      call utstop('utinvt: no interval found               ')
      endif
      if((lo-lu).ge.2) goto1
      if(lo.le.lu)
     *call utstop('utinvt: lo.le.lu                        ')
      utinvt=x(lu)+(y-q(lu))*(x(lo)-x(lu))/(q(lo)-q(lu))
      lutinv=lu
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utkprj(proj,kmax)
c     returns kmax for proj
c-----------------------------------------------------------------------
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      real proj(nsi,nha),sil(nsi),six(nsi,nsix)
      call hdecmp(proj,sil,six)
      call utksix(six,kmax)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utksix(six,kmax)
c     returns kmax for six
c-----------------------------------------------------------------------
      parameter (nsi=6,nsix=100)
      real six(nsi,nsix)
      do 10 k=0,nsix-1
      kmax=k
      sixsqr=0
      do 20 i=1,nsi
20    sixsqr=sixsqr+six(i,k+1)**2
      if(sixsqr.le.1e-5)return
10    continue
      call utstop('utksix: dimension nsix too small        ')
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utkstr(str,kmax)
c     returns kmax for str
c-----------------------------------------------------------------------
      parameter (nsi=6,nsix=100)
      real str(nsi,nsix+1)
      do 10 k=0,nsix
      kmax=k
      strsqr=0
      do 20 i=1,nsi
20    strsqr=strsqr+str(i,k+1)**2
      if(strsqr.le.1e-5)return
10    continue
      call utstop('utkstr: dimension nsix too small        ')
      end
 
c-----------------------------------------------------------------------
      subroutine utlob2(isig,p1,p2,p3,p4,p5,x1,x2,x3,x4)
c-----------------------------------------------------------------------
c  performs a lorentz boost, double prec.
c  isig=+1 is to boost the four vector x1,x2,x3,x4 such as to obtain it
c  in the frame specified by the 5-vector p1...p5 (5-vector=4-vector+mass).
c  isig=-1: the other way round, that means,
c  if the 4-vector x1...x4 is given in some frame characterized by 
c  p1...p5 with respect to to some lab-frame, utlob2 returns the 4-vector 
c  x1...x4  in the lab frame.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      double precision beta(4),z(4),p1,p2,p3,p4,p5,pp,bp,x1,x2,x3,x4
     *,xx0,x10,x20,x30,x40,x4x
           if(ish.ge.90)then
      if(ish.ge.98)then
      write(ifch,101)sngl(x1),sngl(x2),sngl(x3),sngl(x4)
     *,sngl(x4**2-x3**2-x2**2-x1**2)
101   format(' utlob2: x =  ',5e13.5)
      endif
      pp=p4**2-p3**2-p2**2-p1**2
      if(dabs(pp-p5**2).gt.1e-3*p4**2.and.dabs(pp-p5**2).gt.1e-3)then
      call utmsg('utlob2')
      write(ifch,*)'*****  p**2 .ne. p5**2'
      write(ifch,*)'p**2,p5**2: ',pp,p5**2
      write(ifch,*)'p: ',p1,p2,p3,p4,p5
      call utmsgf
      endif
      x10=x1
      x20=x2
      x30=x3
      x40=x4
           endif
      xx0=x4**2-x3**2-x2**2-x1**2
      if(p5.le.0.)then
      call utmsg('utlob2')
      write(ifch,*)'*****  p5 negative.'
      write(ifch,*)'p(5): ',p1,p2,p3,p4,p5
      call utmsgf
      call utstop('utlob2: p5 negative.                    ')
      endif
      z(1)=x1
      z(2)=x2
      z(3)=x3
      z(4)=x4
      beta(1)=-p1/p5
      beta(2)=-p2/p5
      beta(3)=-p3/p5
      beta(4)= p4/p5
      bp=0.
      do 220 k=1,3
220   bp=bp+z(k)*isig*beta(k)
      do 230 k=1,3
230   z(k)=z(k)+isig*beta(k)*z(4)
     *+isig*beta(k)*bp/(beta(4)+1.)
      z(4)=beta(4)*z(4)+bp
      x1=z(1)
      x2=z(2)
      x3=z(3)
      x4=z(4)
      if(ish.ge.98)
     *write(ifch,101)sngl(x1),sngl(x2),sngl(x3),sngl(x4)
     *,sngl(x4**2-x3**2-x2**2-x1**2)
      x4x=x4
      x4=sign( dsqrt(xx0+x1**2+x2**2+x3**2) , x4x )
      if(ish.ge.98)then
      write(ifch,101)sngl(x1),sngl(x2),sngl(x3),sngl(x4)
     *,sngl(x4**2-x3**2-x2**2-x1**2)
      endif
           if(ish.ge.90)then
      if(dabs(x4-x4x).gt.1e-2*dabs(x4).and.dabs(x4-x4x).gt.1e-2)then
      call utmsg('utlob2')
      write(ifch,*)'*****  x**2_ini .ne. x**2_fin.'
      write(ifch,*)'x1 x2 x3 x4 x**2 (initial/final/corrected):'
102   format(5e13.5)
      write(ifch,102)sngl(x10),sngl(x20),sngl(x30),sngl(x40)
     *,sngl(x40**2-x30**2-x20**2-x10**2)
      write(ifch,102)sngl(x1),sngl(x2),sngl(x3),sngl(x4x)
     *,sngl(x4x**2-x3**2-x2**2-x1**2)
      write(ifch,102)sngl(x1),sngl(x2),sngl(x3),sngl(x4)
     *,sngl(x4**2-x3**2-x2**2-x1**2)
      call utmsgf
      endif
           endif
      return
      end

c-----------------------------------------------------------------------
      subroutine utlob3(isig,p1,p2,p3,p4,p5,x1,x2,x3,x4)
c  performs a lorentz boost, double prec.
c  but arguments are single precision
c-----------------------------------------------------------------------
      call utlob2(isig,dble(p1),dble(p2),dble(p3),dble(p4)
     *,dble(p5),dble(x1),dble(x2),dble(x3),dble(x4))
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utlobo(isig,p1,p2,p3,p4,p5,x1,x2,x3,x4)
c     performs a lorentz boost
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      real beta(4),z(4)
      if(p5.le.0.)then
      call utmsg('utlobo')
      write(ifch,*)'*****  mass <= 0.'
      write(ifch,*)'p(5): ',p1,p2,p3,p4,p5
      call utmsgf
      call utstop('utlobo: mass <= 0.                      ')
      endif
      z(1)=x1
      z(2)=x2
      z(3)=x3
      z(4)=x4
      beta(1)=-p1/p5
      beta(2)=-p2/p5
      beta(3)=-p3/p5
      beta(4)= p4/p5
      bp=0.
      do 220 k=1,3
220   bp=bp+z(k)*isig*beta(k)
      do 230 k=1,3
230   z(k)=z(k)+isig*beta(k)*z(4)
     *+isig*beta(k)*bp/(beta(4)+1.)
      z(4)=beta(4)*z(4)+bp
      x1=z(1)
      x2=z(2)
      x3=z(3)
      x4=z(4)
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utloc(ar,n,a,l)
c-----------------------------------------------------------------------
      real ar(n)
      do 1 i=1,n
      l=i-1
      if(a.lt.ar(i))return
1     continue
      l=n
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utlow(cone)
c-----------------------------------------------------------------------
      character*1 cone
      if(cone.eq.'A')cone='a'
      if(cone.eq.'B')cone='b'
      if(cone.eq.'C')cone='c'
      if(cone.eq.'D')cone='d'
      if(cone.eq.'E')cone='e'
      if(cone.eq.'F')cone='f'
      if(cone.eq.'G')cone='g'
      if(cone.eq.'H')cone='h'
      if(cone.eq.'I')cone='i'
      if(cone.eq.'J')cone='j'
      if(cone.eq.'K')cone='k'
      if(cone.eq.'L')cone='l'
      if(cone.eq.'M')cone='m'
      if(cone.eq.'N')cone='n'
      if(cone.eq.'O')cone='o'
      if(cone.eq.'P')cone='p'
      if(cone.eq.'Q')cone='q'
      if(cone.eq.'R')cone='r'
      if(cone.eq.'S')cone='s'
      if(cone.eq.'T')cone='t'
      if(cone.eq.'U')cone='u'
      if(cone.eq.'V')cone='v'
      if(cone.eq.'W')cone='w'
      if(cone.eq.'X')cone='x'
      if(cone.eq.'Y')cone='y'
      if(cone.eq.'Z')cone='z'
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utlow3(cthree)
c-----------------------------------------------------------------------
      character cthree*3
      do 1 i=1,3
1     call utlow(cthree(i:i))
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utlow6(csix)
c-----------------------------------------------------------------------
      character csix*6
      do 1 i=1,6
1     call utlow(csix(i:i))
      return
      end
 
c-----------------------------------------------------------------------
      function utmom(k,n,x,q)
c     calculates kth moment for f(x) with q(i)=int[0,x(i)]f(z)dz
c-----------------------------------------------------------------------
      real x(n),q(n)
      if(n.lt.2)
     *call utstop('utmom : dimension too small             ')
      utmom=0
      do 1 i=2,n
1     utmom=utmom+((x(i)+x(i-1))/2)**k*(q(i)-q(i-1))
      utmom=utmom/q(n)
      return
      end
 
c----------------------------------------------------------------------
      subroutine utovel
c     fills array ovel(1+i,1+j) containing the logarithm (ln) of
c       i_over_j:
c     ovel(1+i,1+j)=alog(i!/j!/(j-i)!)      i>=0 j>=0
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (iovmax=100,jovmax=100)
      common/covel/ovel(1+iovmax,1+jovmax)
 
      do 1 i=0,iovmax
      ovel(1+i,1)=0.
1     ovel(1+i,1+i)=0.
      do 2 j=1,jovmax-1
      do 3 i=j+1,iovmax
3     ovel(1+i,1+j)=ovel(1+i,j)+alog((i-j+1.)/j)
2     continue
 
      if(ish.ge.90)then
      do 5 j=1,49
      do 5 i=j+1,50
      relerr=abs((exp(ovel(1+i,1+j))
     *-exp(ovel(i,j))-exp(ovel(i,1+j)))/exp(ovel(1+i,1+j)))
      if(relerr.gt.1e-4)then
      call utmsg('utovel')
      write(ifch,*)'*****  ovel(,) violates recurrence relation'
      write(ifch,*)exp(ovel(1+i,1+j)),exp(ovel(i,j))+exp(ovel(i,1+j))
      call utmsgf
      endif
5     continue
      endif
 
      if(ish.ge.95)then
      write(ifch,*)'   exp( ovel(1+i,1+j) )'
      do 6 i=0,15
6     write(ifch,*)(exp( ovel(1+i,1+j) ),j=0,min0(4,i))
      endif
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utpage
c-----------------------------------------------------------------------
c     increases ipage by 1, changes ish
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cishi/ishi
      double precision seedi,seedj,seedc
      common /cseed/seedi,seedj,seedc
      ipage=ipage+1
      if(ipagi.le.0)return
      if(ipage.eq.1)ishi=ish
      ish=0
      if(ipage.ge.ipagi/10000.and.ipage.le.mod(ipagi,10000))ish=ishi
      return
      end
 
c----------------------------------------------------------------------
      subroutine utpart
c     fills array partx(k,n) containing the number
c       of partitions of n into at most k integers (k>=1 n>=1)  .
c     fills array part(k,n) containing  the number
c       of partitions of n into k integers (k>=1 n>=1)  .
c     fills array ipart(n,j) containing the partitions of n.
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (nquax=12)
      parameter (kpamax=nquax)
      parameter (jpamax=nquax*nquax*nquax)
      common/cpart/part(kpamax,nquax)
      common/cparta/parta(nquax),ipart(nquax,jpamax)
      common/cpartx/partx(kpamax,nquax)
      integer iswap(jpamax)
      write(ifmt,*)'execute sr utpart ...'
      do 10 n=1,nquax
      do 10 j=1,jpamax
10    ipart(n,j)=0
 
      do 6 n=1,nquax
      partx(1,n)=1
6     part(1,n)=1
      do 1 k=2,kpamax
      do 2 n=1,nquax
      u=0
      do 3 i=0,n/k
      if(n-i*k.eq.0)u=u+1
3     if(n-i*k.ne.0)u=u+partx(k-1,n-i*k)
      partx(k,n)=u
2     continue
1     continue
      do 7 n=1,nquax
      parta(n)=part(1,n)
      do 7 k=2,kpamax
      part(k,n)=partx(k,n)-partx(k-1,n)
7     parta(n)=parta(n)+part(k,n)
 
      if(ish.ge.93)then
121   format(79a1)
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   partitions of n into at most k integers'
      write(ifch,121)('-',ic=1,79)
      write(ifch,101)((k),k=1,10),kpamax
      write(ifch,121)('-',ic=1,79)
      do 8 n=1,nquax
8     write(ifch,100)n,(nint(partx(k,n)),k=1,10),nint(parta(n))
100   format(3x,'n = ',i2,3x,10i5,3x,i5)
101   format(9x,'k =',10i5,3x,i5)
      endif
 
c     n=1
c     ---
      ipart(1,1)=-1
      ipart(1,2)=-1
      ipart(1,3)=1
 
c     n>1
c     ---
      do 11 n=2,nquax
      write(ifmt,*)'sr utpart: n=',n
      if(ish.ge.95)write(ifch,*)'n=',n
      ipart(n,1)=-1
      ipart(n,2)=-1
      ipart(n,3)=n
      ii=3
 
c     n=n1+n2, multiply partitions of n1 and n2
c     -----------------------------------------
      do 12 n1=1,n/2
      n2=n-n1
      if(ish.ge.95)write(ifch,*)'n1,n2=',n1,n2
      k1max=-ipart(n1,1)
      k2max=-ipart(n2,1)
      if(k1max.lt.0.or.k2max.lt.0)
     *call utstop('utpart: kimax negative                  ')
      j1=2
      do 13 k1=1,k1max
      l1=-ipart(n1,j1)
      if(l1.lt.0)
     *call utstop('utpart: l1 negative (1)                 ')
      j2=2
      do 15 k2=1,k2max
      l2=-ipart(n2,j2)
      if(l2.lt.0)
     *call utstop('utpart: l2 negative (1)                 ')
 
      ipart(n,1)=ipart(n,1)-1
      ii=ii+1
      if(ii.gt.jpamax)goto1000
      ipart(n,ii)=-l1-l2
      ii0=ii+1
      do 17 i1=1,l1
      ii=ii+1
      if(ii.gt.jpamax)goto1000
17    ipart(n,ii)=ipart(n1,j1+i1)
      do 18 i2=1,l2
      ii=ii+1
      if(ii.gt.jpamax)goto1000
18    ipart(n,ii)=ipart(n2,j2+i2)
 
      if(ish.ge.95)then
      write(ifch,*)'k1,k2=',k1,k2
      write(ifch,103)n,-ipart(n,1),nint(parta(n))
103   format(3x,'n = ',i2,'   p(n) = ',i4,'   p0(n) = ',i4)
      jy=2
      do 29 ky=1,-ipart(n,1)
      ly=-ipart(n,jy)
      write(ifch,102)ky,ly,(ipart(n,i),i=jy+1,jy+min0(20,ly))
102   format(3x,i2,3x,i2,3x,20i3)
29    jy=jy+ly+1
      endif
 
      ipri=0
      do 19 i=ii0,ii-1
      do 19 j=i+1,ii
      if(ipart(n,i).lt.ipart(n,j))then
      ipri=1
      isti=ipart(n,i)
      ipart(n,i)=ipart(n,j)
      ipart(n,j)=isti
      endif
19    continue
      if(ipri.eq.1.and.ish.ge.95)then
      write(ifch,*)'sequence changed'
      endif
 
      jx=2
      do 28 kx=1,-ipart(n,1)-1
      lx=-ipart(n,jx)
      if(lx.lt.0)
     *call utstop('utpart: lx negative                     ')
      if(lx.eq.l1+l2)then
      do 22 l=1,lx
      if(ipart(n,jx+l).ne.ipart(n,ii0-1+l))goto23
22    continue
      if(ish.ge.95)then
      write(ifch,*)'exists already'
      endif
      ipri=1
      do 33 i=ii0-1,ii
33    ipart(n,i)=0
      ii=ii0-2
      ipart(n,1)=ipart(n,1)+1
      goto30
23    continue
      endif
28    jx=jx+lx+1
30    continue
 
      if(ipri.eq.1.and.ish.ge.95)then
      write(ifch,103)n,-ipart(n,1),nint(parta(n))
      jy=2
      do 32 ky=1,-ipart(n,1)
      ly=-ipart(n,jy)
      write(ifch,102)ky,ly,(ipart(n,i),i=jy+1,jy+min0(20,ly))
32    jy=jy+ly+1
      endif
 
15    j2=j2+l2+1
13    j1=j1+l1+1
12    continue
 
c     ordering
c     --------
      km=-ipart(n,1)
      if(km.lt.0)
     *call utstop('utpart: km negative                     ')
      j1=2
      do 20 k1=1,km-1
      l1=-ipart(n,j1)
      if(l1.lt.0)
     *call utstop('utpart: l1 negative (2)                 ')
      j2=2
      do 21 k2=1,km
      l2=-ipart(n,j2)
      if(l2.lt.0)
     *call utstop('utpart: l2 negative (2)                 ')
      if(k2.le.k1)goto21
      if(l1.gt.l2)then
      do 24 l=1,l2+1
24    iswap(l)=ipart(n,j2-1+l)
      do 25 l=1,j2-j1
      i=j2-l
25    ipart(n,i+l2+1)=ipart(n,i)
      do 26 l=1,l2+1
26    ipart(n,j1-1+l)=iswap(l)
      l1=-ipart(n,j1)
      if(l1.lt.0)
     *call utstop('utpart: l1 negative (2)                 ')
      if(ish.ge.95)then
      write(ifch,*)'order changed.      k1,k2=',k1,k2
      write(ifch,103)n,-ipart(n,1),nint(parta(n))
      jy=2
      do 31 ky=1,-ipart(n,1)
      ly=-ipart(n,jy)
      write(ifch,102)ky,ly,(ipart(n,i),i=jy+1,jy+min0(20,ly))
31    jy=jy+ly+1
      endif
      endif
21    j2=j2+l2+1
20    j1=j1+l1+1
 
      if(ish.ge.93.and.n.le.8)then
      write(ifch,113)('-',ic=1,79),n,-ipart(n,1),('-',ic=1,79)
113   format(79a1/7x,'n = ',i2,'   --->   ',i4,' partitions'/79a1)
      jy=2
      do 27 ky=1,-ipart(n,1)
      ly=-ipart(n,jy)
      write(ifch,112)ky,(ipart(n,i),i=jy+1,jy+min0(20,ly))
112   format(2x,i2,'. partition:',3x,20i3)
27    jy=jy+ly+1
      endif
      if(-ipart(n,1).ne.nint(parta(n)))
     *call utstop('utpart: # of partitions wrong           ')
11    continue
 
      return
 
1000  continue
      write(ifch,*)('*',j=1,79)
      write(ifch,*)'***** n=',n
      write(ifch,*)'***** jpamax=',jpamax
      write(ifch,*)('*',j=1,79)
      call utstop('utpart: dimension jpamax too small.     ')
 
      end
 
c-----------------------------------------------------------------------
      function utpcm(a,b,c)
c     calculates cm momentum for a-->b+c
c-----------------------------------------------------------------------
      val=(a**2-b**2-c**2)**2-(2.*b*c)**2
      if(val.lt.0..and.val.gt.-1e-4)then
      utpcm=0
      return
      endif
      utpcm=sqrt(val)/(2.*a)
      return
      end

c-----------------------------------------------------------------------
      function utquad(m,x,f,k)
c     performs an integration according to simpson
c-----------------------------------------------------------------------
      real x(m),f(m)
      utquad=0
      do 1 i=1,k-1
  1   utquad=utquad+(f(i)+f(i+1))/2*(x(i+1)-x(i))
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utquaf(fu,n,x,q,wo,x0,x1,x2,x3)
c     returns q(i) = integral [x(1)->x(i)] fu(x) dx
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      real x(n),q(n),wo(n)
      parameter (m=10)
      real xa(m),fa(m)
      external fu
      if(x1.lt.x0.or.x2.lt.x1.or.x3.lt.x2)then
      if(ish.ge.90)then
      call utmsg('utquaf')
      write(ifch,*)'   xi=',x0,x1,x2,x3
      call utmsgf
      endif
      endif
      call utar(n/3,n*2/3,n,x0,x1,x2,x3,x)
      q(1)=0
      do 2 i=2,n
      do 3 k=1,m
      z=x(i-1)+(k-1.)/(m-1.)*(x(i)-x(i-1))
      xa(k)=z
3     fa(k)=fu(z)
      q(i)=q(i-1)+utquad(m,xa,fa,m)
2     continue
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utremb(proj,targ,ii)
c     remembers nevt,nstr,nptl,proj,targ
c-----------------------------------------------------------------------
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter(mxstr=20000)
      common/cstr/nstr,pstr(5,mxstr),icstr(4,mxstr),iorstr(mxstr)
     *,irlstr(mxstr),rotstr(3,mxstr),xorstr(4,mxstr)
      common/cevt/phievt,nevt,bimevt,kolevt,colevt,pmxevt,egyevt,npjevt
     *,ntgevt,npnevt,nppevt,ntnevt,ntpevt,jpnevt,jppevt,jtnevt,jtpevt
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      real proj(nsi,nha),targ(nsi,nha)
      common/cremb/nevtre(2),nstrre(2),nptlre(2)
     *,projre(2,nsi,nha),targre(2,nsi,nha)
 
      nevtre(ii)=nevt
      nstrre(ii)=nstr
      nptlre(ii)=nptl
      do 56 m=1,nha
      sm=0
      do 57 n=1,nsi
      sm=sm+proj(n,m)**2
57    projre(ii,n,m)=proj(n,m)
      if(m.ge.3.and.sm.lt.1e-5)goto58
56    continue
58    continue
      do 53 m=1,nha
      sm=0
      do 54 n=1,nsi
      sm=sm+targ(n,m)**2
54    targre(ii,n,m)=targ(n,m)
      if(m.ge.3.and.sm.lt.1e-5)goto55
53    continue
55    continue
      return
 
      entry utrest(proj,targ,ii)
      nevt=nevtre(ii)
      nstr=nstrre(ii)
      nptl=nptlre(ii)
      do 66 m=1,nha
      sm=0
      do 67 n=1,nsi
      sm=sm+projre(ii,n,m)**2
67    proj(n,m)=projre(ii,n,m)
      if(m.ge.3.and.sm.lt.1e-5)goto68
66    continue
68    continue
      do 63 m=1,nha
      sm=0
      do 64 n=1,nsi
      sm=sm+targre(ii,n,m)**2
64    targ(n,m)=targre(ii,n,m)
      if(m.ge.3.and.sm.lt.1e-5)goto65
63    continue
65    continue
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utrepl(i,j)
c     i is replaced by j in /cptl/
c-----------------------------------------------------------------------
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c2ptl/iaaptl(mxptl),radptl(mxptl),amiptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      parameter (myptl=1000,nzeta=60)
      common/c4ptl/iyptlx,iyptl(mxptl),weiptl(myptl,nzeta),iyiptl(myptl)
      do 1 k=1,5
1     pptl(k,i)  =pptl(k,j)
      iorptl(i)  =iorptl(j)
      idptl(i)   =idptl(j)
      istptl(i)  =istptl(j)
      do 2 k=1,2
2     tivptl(k,i)=tivptl(k,j)
      do 3 k=1,2
3     ifrptl(k,i)=ifrptl(k,j)
      jorptl(i)  =jorptl(j)
      do 4 k=1,4
4     xorptl(k,i)=xorptl(k,j)
      do 5 k=1,4
5     ibptl(k,i) =ibptl(k,j)
      iclptl(i)  =iclptl(j)
      iaaptl(i)  =iaaptl(j)
      radptl(i)  =radptl(j)
      amiptl(i)  =amiptl(j)
      desptl(i)  =desptl(j)
      dezptl(i)  =dezptl(j)
      iyptl(i)   =iyptl(j)
      iyiptl(iyptl(i))=i
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utresm(icp1,icp2,icm1,icm2,amp,idpr,iadj,ireten)
c-----------------------------------------------------------------------
      parameter (nflav=6)
      integer icm(2),icp(2),jcm(nflav,2),jcp(nflav,2)
      icm(1)=icm1
      icm(2)=icm2
      icp(1)=icp1
      icp(2)=icp2
      CALL IDDECO(ICM,JCM)
      CALL IDDECO(ICP,JCP)
      do 37 nf=1,nflav
      do 37 k=1,2
37    jcP(nf,k)=jcp(nf,k)+jcm(nf,k)
      CALL IDENCO(JCP,ICP,IRETEN)
      IDP=IDTRA(ICP,0,0,3)
      call idres(idp,amp,idpr,iadj)
      return
      end

c-----------------------------------------------------------------------
      subroutine utroa1(phi,a1,a2,a3,x1,x2,x3)
c-----------------------------------------------------------------------
c  rotates x by angle phi around axis a.
c  normalization of a is irrelevant.
c-----------------------------------------------------------------------
      double precision aaa,aa(3),xxx,xx(3),e1(3),e2(3),e3(3),xp,xt,dphi
      dphi=phi
      xx(1)=x1
      xx(2)=x2
      xx(3)=x3
      aa(1)=a1
      aa(2)=a2
      aa(3)=a3
      aaa=0
      xxx=0
      do i=1,3
      aaa=aaa+aa(i)**2
      xxx=xxx+xx(i)**2
      enddo
      if(xxx.eq.0d0)return
      if(aaa.eq.0d0)
     *call utstop('utroa1: zero rotation axis              ')
      aaa=dsqrt(aaa)
      xxx=dsqrt(xxx)
c e3 = a / !a!
      do i=1,3
      e3(i)=aa(i)/aaa
      enddo
c x_parallel
      xp=0
      do i=1,3
      xp=xp+xx(i)*e3(i)
      enddo
c x_transverse
      if(xxx**2-xp**2.le.0.)return
      xt=dsqrt(xxx**2-xp**2)
c e1 = vector x_transverse / absolute value x_transverse
      do i=1,3
      e1(i)=(xx(i)-e3(i)*xp)/xt
      enddo
c e2 orthogonal e3,e1
      call utvec2(e3,e1,e2)
c rotate x
      do i=1,3
      xx(i)=xp*e3(i)+xt*dcos(dphi)*e1(i)+xt*dsin(dphi)*e2(i)
      enddo
c back to single precision
      x1=xx(1)
      x2=xx(2)
      x3=xx(3)
      return
      end

c-----------------------------------------------------------------------
      subroutine utroa2(phi,a1,a2,a3,x1,x2,x3)
c-----------------------------------------------------------------------
c  rotates x by angle phi around axis a.
c  normalization of a is irrelevant.
c  double precision phi,a1,a2,a3,x1,x2,x3
c-----------------------------------------------------------------------
      double precision phi,a1,a2,a3,x1,x2,x3
      double precision aaa,aa(3),xxx,xx(3),e1(3),e2(3),e3(3),xp,xt,dphi
      dphi=phi
      xx(1)=x1
      xx(2)=x2
      xx(3)=x3
      aa(1)=a1
      aa(2)=a2
      aa(3)=a3
      aaa=0
      xxx=0
      do i=1,3
      aaa=aaa+aa(i)**2
      xxx=xxx+xx(i)**2
      enddo
      if(xxx.eq.0d0)return
      if(aaa.eq.0d0)
     *call utstop('utroa1: zero rotation axis              ')
      aaa=1.0/dsqrt(aaa)
c e3 = a / !a!
      do i=1,3
      e3(i)=aa(i)*aaa
      enddo
c x_parallel
      xp=0
      do i=1,3
      xp=xp+xx(i)*e3(i)
      enddo
c x_transverse
      if(xxx-xp**2.le.0.)return
      xt=dsqrt(xxx-xp**2)
c e1 = vector x_transverse / absolute value x_transverse
      do i=1,3
      e1(i)=(xx(i)-e3(i)*xp)/xt
      enddo
c e2 orthogonal e3,e1
      call utvec2(e3,e1,e2)
c rotate x
      do i=1,3
      xx(i)=xp*e3(i)+xt*dcos(dphi)*e1(i)+xt*dsin(dphi)*e2(i)
      enddo
c back to single precision
      x1=xx(1)
      x2=xx(2)
      x3=xx(3)
      return
      end

c--------------------------------------------------------------------
      function utroot(funcd,x1,x2,xacc)
c--------------------------------------------------------------------
c combination of newton-raphson and bisection method for root finding
c input:
c   funcd: subr returning fctn value and first derivative
c   x1,x2: x-interval
c   xacc:  accuracy
c output: 
c   utroot: root
c--------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      INTEGER MAXIT
      REAL utroot,x1,x2,xacc
      EXTERNAL funcd
      PARAMETER (MAXIT=100)
      INTEGER j
      REAL df,dx,dxold,f,fh,fl,temp,xh,xl
      call funcd(x1,fl,df)
      call funcd(x2,fh,df)
      if((fl.gt.0..and.fh.gt.0.).or.(fl.lt.0..and.fh.lt.0.))
     *call utstop('utroot: root must be bracketed          ')
      if(fl.eq.0.)then
        utroot=x1
        return
      else if(fh.eq.0.)then
        utroot=x2
        return
      else if(fl.lt.0.)then
        xl=x1
        xh=x2
      else
        xh=x1
        xl=x2
      endif
      utroot=.5*(x1+x2)
      dxold=abs(x2-x1)
      dx=dxold
      call funcd(utroot,f,df)
      do 11 j=1,MAXIT
        if(((utroot-xh)*df-f)*((utroot-xl)*df-f).ge.0..or. abs(2.*
     *f).gt.abs(dxold*df) ) then
          dxold=dx
          dx=0.5*(xh-xl)
          utroot=xl+dx
          if(xl.eq.utroot)return
        else
          dxold=dx
          dx=f/df
          temp=utroot
          utroot=utroot-dx
          if(temp.eq.utroot)return
        endif
        if(abs(dx).lt.xacc) return
        call funcd(utroot,f,df)
        if(f.lt.0.) then
          xl=utroot
        else
          xh=utroot
        endif
11    continue
      call utmsg('utroot')
      write(ifch,*)'*****  exceeding maximum iterations'
      write(ifch,*)'dx:',dx
      call utmsgf
      return
      END

c-----------------------------------------------------------------------
      subroutine utrot2(isig,ax,ay,az,x,y,z)
c     performs a rotation, double prec.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      double precision ax,ay,az,x,y,z,rx,ry,rz
     *,alp,bet,cosa,sina,cosb,sinb,xs,ys,zs
         if(ax**2.eq.0.and.ay**2.eq.0.and.az**2.eq.0.)then
      write(ifch,*)'ax**2,ay**2,az**2:',ax**2,ay**2,az**2
      write(ifch,*)'ax,ay,az:',ax,ay,az
      call utstop('utrot2: zero vector.                    ')
         endif
         if(az.ge.0.)then
      rx=ax
      ry=ay
      rz=az
         else
      rx=-ax
      ry=-ay
      rz=-az
         endif
      if(rz**2+ry**2.ne.0.)then
      alp=dabs(dacos(rz/dsqrt(rz**2+ry**2)))*sign(1.,sngl(ry))
      bet=
     *dabs(dacos(dsqrt(rz**2+ry**2)/dsqrt(rz**2+ry**2+rx**2)))*
     *sign(1.,sngl(rx))
      else
      alp=3.1415927/2
      bet=3.1415927/2
      endif
      cosa=dcos(alp)
      sina=dsin(alp)
      cosb=dcos(bet)
      sinb=dsin(bet)
           if(isig.gt.0)then
      xs=x*cosb-y*sina*sinb-z*cosa*sinb
      ys=       y*cosa     -z*sina
      zs=x*sinb+y*sina*cosb+z*cosa*cosb
           elseif(isig.lt.0)then
      xs= x*cosb            +z*sinb
      ys=-x*sinb*sina+y*cosa+z*cosb*sina
      zs=-x*sinb*cosa-y*sina+z*cosb*cosa
           endif
      x=xs
      y=ys
      z=zs
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utrota(isig,ax,ay,az,x,y,z)
c     performs a rotation
c-----------------------------------------------------------------------
         if(az.ge.0.)then
      rx=ax
      ry=ay
      rz=az
         else
      rx=-ax
      ry=-ay
      rz=-az
         endif
      alp=abs(utacos(rz/sqrt(rz**2+ry**2)))*sign(1.,ry)
      bet=
     *abs(utacos(sqrt(rz**2+ry**2)/sqrt(rz**2+ry**2+rx**2)))*sign(1.,rx)
      cosa=cos(alp)
      sina=sin(alp)
      cosb=cos(bet)
      sinb=sin(bet)
           if(isig.gt.0)then
      xs=x*cosb-y*sina*sinb-z*cosa*sinb
      ys=       y*cosa     -z*sina
      zs=x*sinb+y*sina*cosb+z*cosa*cosb
           elseif(isig.lt.0)then
      xs= x*cosb            +z*sinb
      ys=-x*sinb*sina+y*cosa+z*cosb*sina
      zs=-x*sinb*cosa-y*sina+z*cosb*cosa
           endif
      x=xs
      y=ys
      z=zs
      return
      end
 
c-----------------------------------------------------------------------
      function utsqsi(sil)
c-----------------------------------------------------------------------
      parameter (nsi=6)
      real sil(nsi)
      utsqsi=0
      do 1 n=1,nsi
1     utsqsi=utsqsi+sil(n)**2
      return
      end
 
c-----------------------------------------------------------------------
      function utsqst(str)
c-----------------------------------------------------------------------
      parameter (nsi=6,nsix=100)
      parameter (nha=nsix+2)
      real str(nsi,2)
      utsqst=0
      do 1 n=1,nsi
1     utsqst=utsqst+str(n,1)**2+str(n,2)**2
      return
      end
 
c-----------------------------------------------------------------------
      function utsstr(str)
c     calculates the mass of string str
c-----------------------------------------------------------------------
      parameter (nsi=6)
      real str(nsi,2)
      utsstr=(str(4,1)+str(4,2))**2-(str(1,1)+str(1,2))**2
     *-(str(2,1)+str(2,2))**2-(str(3,1)+str(3,2))**2
      return
      end
 
c-----------------------------------------------------------------------
      subroutine utstop(text)
c     returns error message and stops execution
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      double precision seedi,seedj,seedc
      common /cseed/seedi,seedj,seedc
      character text*40,star*1,txt*6
      star='*'
      do 1 i=1,2
      if(i.eq.1)ifi=ifch
      if(i.eq.2)ifi=ifmt
      write(ifi,101)(star,j=1,75),text,nrevt+1,ipage,seedc,(star,j=1,75)
101   format(
     *1x,75a1
     */1x,'***** stop in ',a40
     */1x,'***** current event number: ',i12
     */1x,'***** current page number: ',i12
     */1x,'***** initial seed for current event:',d25.15
     */1x,75a1)
1     continue
      stop
      entry utmsg(txt)
      imsg=imsg+1
      write(ifch,'(77a1)')('-',j=1,77)
      write(ifch,100)txt,nrevt+1,ipage,seedc
100   format(1x,'***** msg from ',a6,'.   eps:',i7,i5,2x,d23.17)
      return
      entry utmsgf
      if(ish.eq.90.and.ishsub.eq.0)return
      write(ifch,'(77a1)')('-',j=1,77)
      end
 
c-----------------------------------------------------------------------
      subroutine uttain(i,x,y,z,t,n,iopt)
c-----------------------------------------------------------------------
c returns intersection (x,y,z,t) of ptl-i-trajectory with taus-line.
c input:    
c   i: particle number
c   iopt: formation time considered (0) or not (1)
c output:
c   x,y,z,t: 4-vector of intersection point
c   n: exit code
c       n=0: ok
c       n=1: ptl lives later
c       n=2: ptl lives earlier
c       n=9: tiv1>tiv2
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat /ctain/mtain
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      double precision vv,zza,zz,tt,xo3,xo4,ti1,ti2,derr,dd
      double precision ttp,zzp,ttt,zzt,vvt,vvp
      common/ctfi/tin,tfi
      
      n=0

      tin=0
      tfi=0

      derr=1e-2
      ttp=tpro*ttaus
      zzp=zpro*ttaus
      ttt=ttar*ttaus
      zzt=ztar*ttaus
      vv=pptl(3,i)/pptl(4,i)
      xo3=xorptl(3,i)
      xo4=xorptl(4,i)
      zza=xo3-xo4*vv
      if(iopt.eq.0)ti1=tivptl(1,i)
      if(iopt.eq.1)ti1=xo4
      ti2=tivptl(2,i)
 
      if(ti1.gt.ti2)then
      n=9
      goto1
      endif
 
      zfi=xo3+(ti2-xo4)*vv
      call uttaus(zfi,tzfi,szfi)
      tfi=tzfi
      if(tfi.ge.sngl(ti2))then
      n=2
      goto1
      endif
 
      zin=xo3+(ti1-xo4)*vv
      call uttaus(zin,tzin,szin)
      tin=tzin
      if(tin.le.sngl(ti1))then
      n=1
      goto1
      endif

    1 continue
 
           if(sngl(ttaus).le.0.)then
      tt=ttaus
      zz=xo3+(tt-xo4)*vv
      if(tt.lt.ti1.and.n.eq.0)n=1
      if(tt.ge.ti2.and.n.eq.0)n=2
      goto1000
           else
      vvt=zzt/ttt
      vvp=zzp/ttp
      tt=(ttt+(zza-zzt)*vvt)/(1-vv*vvt)
      zz=xo3+(tt-xo4)*vv
      if(zz.le.zzt)then
      if(tt.lt.ti1.and.n.eq.0)n=1
      if(tt.ge.ti2.and.n.eq.0)n=2
      goto1000
      endif
      tt=(ttp+(zza-zzp)*vvp)/(1-vv*vvp)
      zz=xo3+(tt-xo4)*vv
      if(zz.ge.zzp)then
      if(tt.lt.ti1.and.n.eq.0)n=1
      if(tt.ge.ti2.and.n.eq.0)n=2
      goto1000
      endif
      dd=1-vv**2
      if(sngl(dd).eq.0..and.vv.gt.0.)then
      tt=-(ttaus**2+zza**2)/2./zza
      elseif(sngl(dd).eq.0..and.vv.lt.0.)then
      tt=(ttaus**2+zza**2)/2./zza
      else
      tt=(zza*vv+dsqrt(zza**2+ttaus**2*dd))/dd
      endif
      zz=xo3+(tt-xo4)*vv
      if(tt.lt.ti1.and.n.eq.0)n=1
      if(tt.ge.ti2.and.n.eq.0)n=2
        if(dabs(ttaus**2-(tt+zz)*(tt-zz)).gt.derr*ttaus**2.and.
     *dabs(ttaus**2-(tt+zz)*(tt-zz)).gt.derr)then
      if(ish.ge.0)then
      call utmsg('uttain')
      write(ifch,*)'*****  ttaus**2 .ne. (tt+zz)*(tt-zz)'
      write(ifch,*)sngl(ttaus**2),sngl((tt+zz)*(tt-zz))
      call utmsgf
      endif
      goto1000
        endif
           endif
 
1000  t=tt
      z=zz
      x=xorptl(1,i)+(t-xorptl(4,i))*pptl(1,i)/pptl(4,i)
      y=xorptl(2,i)+(t-xorptl(4,i))*pptl(2,i)/pptl(4,i)
      return

      end

c-----------------------------------------------------------------------
      subroutine uttaix(i,tau,zor,tor,z,t)
c     returns intersection z,t of ptl-i-trajectory with hyperbola h.
c        h: (t-tor)**2-(z-zor)**2=tau**2 .
c        zor, tor double precision.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      double precision tor,zor,tors,zors,vv,cc,dd,ttau,derr,tt,zz
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      derr=1e-3
      ttau=tau
      zors=xorptl(3,i)-zor
      tors=xorptl(4,i)-tor
      vv=pptl(3,i)/pptl(4,i)
      vv=dmin1(vv,1d0)
      vv=dmax1(vv,-1d0)
      cc=zors-tors*vv
      dd=1d0-vv**2
      dd=dmax1(dd,0d0)
           if(dd.eq.0d0.and.cc.eq.0d0)then
      if(tau.eq.0.)tt=0.
      if(tau.ne.0.)tt=ainfin
      zz=tt
      goto1000
           elseif(dd.eq.0d0)then
      tt=-(ttau**2+cc**2)/2./cc/vv
           elseif(dd.lt.1e-8)then
      tt=-(ttau**2+cc**2)/2./cc/vv
      call utmsg('uttaix')
      write(ifch,*)'*****  dd = ',dd,'    treated as zero'
      call utmsgf
           else
      tt=(cc*vv+dsqrt(cc**2+ttau**2*dd))
      tt=tt/dd
           endif
      zz=cc+tt*vv
      if(dabs(ttau**2-(tt+zz)*(tt-zz)).gt.derr*ttau**2
     *.and.dabs(ttau**2-(tt+zz)*(tt-zz)).gt.derr
     *.and.tors**2-zors**2.lt.1e6)then
      if(ish.ge.90)then
      call utmsg('uttaix')
      write(ifch,*)'*****  ttau**2 .ne. (tt+zz)*(tt-zz)'
      write(ifch,*)sngl(ttau**2),sngl((tt+zz)*(tt-zz))
      write(ifch,*)'tau,t,z:'
      write(ifch,*)tau,tt,zz
      write(ifch,*)'#,id(ptl):',i,idptl(i)
      write(ifch,*)'zor,tor(str):',zor,tor
      write(ifch,*)'zors,tors,p,e(ptl):'
      write(ifch,*)sngl(zors),sngl(tors),pptl(3,i),pptl(4,i)
      call utmsgf
      endif
      endif
1000  z=zz+zor
      t=tt+tor
      return
      end

c-----------------------------------------------------------------------
      subroutine uttaug(su,so,g,y)
c-----------------------------------------------------------------------
c  returns g factor and rapidity y for given su, so
c-----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg

      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 

      double precision ttp,zzp,ttt,zzt,ssp,sst,ssu,sso,ss1,ss2,gg
     *,ssav,yyav,hh

      ssu=su
      sso=so

      if(ssu.ge.sso)then 
      sso=ssu+dezzer*ttaus*0.5
      so=su  +dezzer*ttaus*0.5
      endif
      if(ssu.ge.sso)print*,ssu,sso
      if(ssu.ge.sso)stop'STOP: sr uttaug: ssu.ge.sso'
      
      g=1

      if(ttaus.le.0.)return

      ttp=tpro*ttaus    
      zzp=zpro*ttaus
      ttt=ttar*ttaus
      zzt=ztar*ttaus
      ssp=ttaus*0.5*dlog((ttp+zzp)/(ttp-zzp))
      sst=ttaus*0.5*dlog((ttt+zzt)/(ttt-zzt))

      ssav=(ssu+sso)/2
      yyav=ssav/ttaus
      if(ssav.ge.ssp)yyav=detap 
      if(ssav.le.sst)yyav=detat

      gg=0
      if(ssu.lt.sst)gg=gg + dcosh(detat-yyav) * (dmin1(sst,sso)-ssu)
      if(sso.gt.ssp)gg=gg + dcosh(detap-yyav) * (sso-dmax1(ssp,ssu))
      if(ssu.lt.ssp.and.sso.gt.sst)then
      ss1=dmax1(ssu,sst)
      ss2=dmin1(sso,ssp)
      gg=gg+ttaus*( dsinh(ss2/ttaus-yyav)-dsinh(ss1/ttaus-yyav) )
      endif
      gg=gg/(sso-ssu)

      hh=0
      if(ssu.lt.sst)hh=hh + dsinh(detat-yyav) * (dmin1(sst,sso)-ssu)
      if(sso.gt.ssp)hh=hh + dsinh(detap-yyav) * (sso-dmax1(ssp,ssu))
      if(ssu.lt.ssp.and.sso.gt.sst)then
      ss1=dmax1(ssu,sst)
      ss2=dmin1(sso,ssp)
      hh=hh+ttaus*( dcosh(ss2/ttaus-yyav)-dcosh(ss1/ttaus-yyav) )
      endif
      hh=hh/(sso-ssu)

      yyav=yyav+0.5*dlog((gg+hh)/(gg-hh))

      gg=0
      if(ssu.lt.sst)gg=gg + dcosh(detat-yyav) * (dmin1(sst,sso)-ssu)
      if(sso.gt.ssp)gg=gg + dcosh(detap-yyav) * (sso-dmax1(ssp,ssu))
      if(ssu.lt.ssp.and.sso.gt.sst)then
      ss1=dmax1(ssu,sst)
      ss2=dmin1(sso,ssp)
      gg=gg+ttaus*( dsinh(ss2/ttaus-yyav)-dsinh(ss1/ttaus-yyav) )
      endif
      gg=gg/(sso-ssu)

      g=gg
      y=yyav
      return
      end

c-----------------------------------------------------------------------
      subroutine uttauh(ip,utot,otot,gtot,ytot,wsmax)
c-----------------------------------------------------------------------
c  returns g factor gtot and rapidity ytot and maximal w(s) wsmax
c  for given ip, utot, otot, and /c3ptl/...weiptl()
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (mxptl=65000,myptl=1000,nzeta=60)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/c3ptl/desptl(mxptl),dezptl(mxptl)
      common/c4ptl/iyptlx,iyptl(mxptl),weiptl(myptl,nzeta),iyiptl(myptl)
      real g(nzeta),y(nzeta),u(nzeta),o(nzeta)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 

      if(dezptl(ip).eq.0..or.iyptl(ip).eq.0)then
      call uttaug(utot,otot,gtot,ytot)
      wsmax=1/(otot-utot)
      return
      endif

      iy=iyptl(ip)
      if(iy.lt.1.or.iy.gt.myptl)
     *call utstop('uttauh: iy out of range                 ')
      nzetat=nzeta
      if(mod(nzetat,2).ne.0)
     *call utstop('uttauh: mod(nzeta,2) /= 0               ')

      w=0
      do 1 n=1,nzeta
      if(weiptl(iy,n).le.1e-6)goto1
      zn=(-nzeta/2-0.5+n)*dlzeta 
      u(n)=(zn-0.5*dlzeta)*ttaus
      o(n)=(zn+0.5*dlzeta)*ttaus 
           if(utot.eq.o(n))then
      u(n)=o(n)-1e-5
      call utmsg('uttauh')
      write(ifch,*)'*****  utot .eq. o(n)'
      write(ifch,*)'nptl iy:',ip,iy
      do m=max(1,n-4),n
      write(ifch,100)m,weiptl(iy,m),u(m),o(m),utot,otot
100   format(1x,i6,5x,f11.7,5x,2f10.5,5x,2f10.5)
      enddo
      call utmsgf
           endif
      if(utot.gt.u(n).and.utot.lt.o(n))u(n)=utot
           if(otot.eq.u(n))then
      o(n)=u(n)+1e-5
      call utmsg('uttauh')
      write(ifch,*)'*****  otot .eq. u(n)'
      write(ifch,*)'nptl iy:',ip,iy
      do m=max(1,n-4),n
      write(ifch,100)m,weiptl(iy,m),u(m),o(m),utot,otot
      enddo
      call utmsgf
           endif
      if(otot.gt.u(n).and.otot.lt.o(n))o(n)=otot
      call uttaug(u(n),o(n),g(n),y(n))
      w=w+weiptl(iy,n)
1     continue

      call uttaug(utot,otot,gtot,ytot)

      e=0
      p=0
      do 2 n=1,nzeta
      if(weiptl(iy,n).le.1e-6)goto2   
      e=e+weiptl(iy,n)/w*g(n)*cosh(y(n)-ytot)
      p=p+weiptl(iy,n)/w*g(n)*sinh(y(n)-ytot)
2     continue

      if(e-p.le.0.)then
      if(ish.ge.0)then
      call utmsg('uttauh')
      write(ifch,*)'*****  e-p<=0    --> only appr gtot ytot wsmax!'
      write(ifch,*)'e p w:',e,p,w
      write(ifch,*)'ip dez iy:',ip,dezptl(ip),iy
      write(ifch,*)'id mass:',idptl(ip),pptl(5,ip)
      write(ifch,*)'wei:'
      write(ifch,*)(weiptl(iy,n),n=1,nzeta)
      call utmsgf
      endif
      wsmax=1/(otot-utot)
      return
      else
      ytot=ytot+0.5*alog((e+p)/(e-p))
      endif

      e=0
      p=0
      wsmx=0
      xav=0
      x2av=0
      do 3 n=1,nzeta
      if(weiptl(iy,n).le.1e-6)goto3   
      e=e+weiptl(iy,n)/w*g(n)*cosh(y(n)-ytot)
      p=p+weiptl(iy,n)/w*g(n)*sinh(y(n)-ytot)
      wsmx=amax1(wsmx,weiptl(iy,n)/w/(o(n)-u(n)))
      xav=xav+weiptl(iy,n)/w*(-nzeta/2-0.5+n)
      x2av=x2av+weiptl(iy,n)/w*(-nzeta/2-0.5+n)**2
3     continue

      sgmx2=x2av-xav**2
      sgmx2=amax1(sgmx2,1./12.)
      sgmx=sqrt(sgmx2)
      sgm=sgmx*dlzeta*ttaus
      wsmax=1/(sqrt(2*pi)*sgm)

      if(ish.ge.0.and.abs(p).gt.1e-4)then  
      call utmsg('uttauh')
      write(ifch,*)'*****  p /= 0   ',p
      call utmsgf
      endif

      gtot=e

      return
      end

c-----------------------------------------------------------------------
      subroutine uttaui(s,ts,zs)
c-----------------------------------------------------------------------
c  returns time ts and coord zs corresponding to ttaus and inv. length s
c-----------------------------------------------------------------------

      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 

      double precision ttp,zzp,ttt,zzt,ssp,sst,ss,zeta

      zs=s
      ts=ttaus

      if(ttaus.le.0.)return

      ttp=tpro*ttaus    
      zzp=zpro*ttaus
      ttt=ttar*ttaus
      zzt=ztar*ttaus
      ssp=ttaus*0.5*dlog((ttp+zzp)/(ttp-zzp))
      sst=ttaus*0.5*dlog((ttt+zzt)/(ttt-zzt))
      ss=s

           if(ss.le.sst)then
      zs=zzt+ttar*(ss-sst)
      ts=ttt+(zs-zzt)*zzt/ttt
           elseif(ss.ge.ssp)then
      zs=zzp+tpro*(ss-ssp)
      ts=ttp+(zs-zzp)*zzp/ttp
           else
      zeta=ss/ttaus
      ts=ttaus*dcosh(zeta)
      zs=ttaus*dsinh(zeta)
           endif

      return
      end

c-----------------------------------------------------------------------
      subroutine uttaus(z,tz,sz)
c-----------------------------------------------------------------------
c  returns time tz and inv length sz corresponding to ttaus and z
c-----------------------------------------------------------------------

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cnsta/pi,pii,hquer,prom,piom,ainfin

      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 

      double precision ttp,zzp,ttt,zzt,zz

      tz=ttaus
      sz=z

      if(ttaus.le.0.)return

      ttp=tpro*ttaus
      zzp=zpro*ttaus
      ttt=ttar*ttaus
      zzt=ztar*ttaus
      zz=z

           if(zz.le.zzt)then
      tz=ttt+(zz-zzt)*zzt/ttt
      sz=ttaus*detat+(zz-zzt)/ttar
           elseif(zz.ge.zzp)then
      tz=ttp+(zz-zzp)*zzp/ttp
      sz=ttaus*detap+(zz-zzp)/tpro
           else
      if(sngl(ttaus).ge.ainfin)then
      tz=ttaus
      sz=0
      if(ish.ge.90)then
      call utmsg('uttaus')
      write(ifch,*)'*****  large ttaus; set tz=ttaus, sz=0'
      write(ifch,*)'ttaus=',ttaus,'zz=',zz
      call utmsgf
      endif
      else
      tz=dsqrt(ttaus**2+zz**2)
      sz=ttaus*0.5*dlog((tz+zz)/(tz-zz))
      endif
           endif

      return
      end

c-----------------------------------------------------------------------
      subroutine uttaux(t,z,ttaux)
c-----------------------------------------------------------------------
c  returns ttaux (-> tau-line) for given t and z.
c  ttaux: double precision.
c-----------------------------------------------------------------------

      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat 
      double precision ttaux,tt,zz

      ttaux=t

      if(t.le.0.)return

      tt=t
      zz=z

           if(zz.lt.ztar/ttar*tt)then
      ttaux=(ttar*tt-ztar*zz)/(ttar**2-ztar**2)
           elseif(zz.gt.zpro/tpro*tt)then
      ttaux=(tpro*tt-zpro*zz)/(tpro**2-zpro**2)
           else
      ttaux=dsqrt(tt**2-zz**2)
           endif

      return
      end
 
c-----------------------------------------------------------------------
      subroutine uttima(text)
c     returns time.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      character*15 text
      timaa=0
      tima=0
c-c   call timax(timaa)
c-c   call timad(tima)
      if(text.eq.'               ')return
      write(ifch,100)text,tima/5.,timaa/5.
100   format(1x,a15,5x,f12.5,5x,f12.5)
      return
 
      entry uttimt
c-c   call timast(1e10)
      return
 
      entry uttimx(timaax)
      timaax=0
c-c   call timax(timaax)
      return
      end
 
c-----------------------------------------------------------------
      subroutine uttrap(func,a,b,s)
c-----------------------------------------------------------------
c trapezoidal method for integration.
c input: fctn func and limits a,b
c output: value s of the integral
c-----------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cepsr/epsr,nepsr

      INTEGER JMAX
      REAL a,b,func,s,epsr
      EXTERNAL func
      PARAMETER (JMAX=10)
CU    USES uttras
      INTEGER j
      REAL olds
      olds=-1.e30
      do 11 j=1,JMAX
        if(ish.ge.98)write(ifch,*)'sr uttrap:   j:',j
        call uttras(func,a,b,s,j)
        ds=abs(s-olds)
        if (ds.lt.epsr*abs(olds)) return
        olds=s
11    continue
 
c-c   nepsr=nepsr+1
      if(ish.ge.98)then
      call utmsg('uttrap')
      write(ifch,*)
     *'*****  requested accuracy could not be achieved'
      write(ifch,*)'achieved accuracy: ',ds/abs(olds)
      write(ifch,*)'requested accuracy:',epsr
      call utmsgf
      endif

      END

c-----------------------------------------------------------------
      SUBROUTINE uttraq(func,a,b,s)
c-----------------------------------------------------------------
c trapezoidal method for integration.
c input: function func and limits a,b
c output: value s of the integral
c-----------------------------------------------------------------

      REAL a,b,func,s
      EXTERNAL func
      PARAMETER (eps=1.e-6)
CU    USES uttras
      INTEGER j
      REAL olds
      olds=-1.e30
      j=1
10      call uttras(func,a,b,s,j)
        ds=abs(s-olds)
        if (ds.le.eps*abs(olds)) return
        olds=s
c       if(j.ge.15)goto20
        if(j.ge.15)return
        j=j+1
      goto10
      END

c-----------------------------------------------------------------
      subroutine uttras(func,a,b,s,n)
c-----------------------------------------------------------------
c performs one iteration of the trapezoidal method for integration
c-----------------------------------------------------------------
      INTEGER n
      REAL a,b,s,func
      EXTERNAL func
      INTEGER it,j
      REAL del,sum,tnm,x
      if (n.eq.1) then
        s=0.5*(b-a)*(func(a)+func(b))
      else
        it=2**(n-2)
        tnm=it
        del=(b-a)/tnm
        x=a+0.5*del
        sum=0.
        do 11 j=1,it
          sum=sum+func(x)
          x=x+del
11      continue
        s=0.5*(s+(b-a)*sum/tnm)
      endif
      return
      END

c----------------------------------------------------------------------
      subroutine uttucl
c     fills array tucl(1+k,1+n) containing the logarithm (ln) of
c       the number of k-tupels n_i with sum_i n_i = n:
c       tucl(1+k,1+n)=dlog((n+1)*(n+2)...*(n+k-1)/(k-1)!)  k>=0 n>=0 .
c     double precision tucl.
c----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      double precision tucl,dd
      parameter (ktumax=100,ntumax=100)
      common/ctucl/tucl(1+ktumax,1+ntumax)
 
      do 6 n=0,ntumax
      tucl(1,1+n)=-100
6     tucl(2,1+n)=0
      do 7 k=2,ktumax
7     tucl(1+k,1)=0
      do 2 n=1,ntumax
      do 1 k=2,ktumax
      dd=(n+k-1.)/(k-1.)
      tucl(1+k,1+n)=tucl(k,1+n)+dlog(dd)
1     continue
2     continue
 
      if(ish.ge.93)then
      do 5 k=2,50
      do 5 n=1,50
      relerr=dabs((dexp(tucl(1+k,1+n))
     *-dexp(tucl(1+k,n))-dexp(tucl(k,1+n)))/dexp(tucl(1+k,1+n)))
      if(relerr.gt.1e-4)then
      call utmsg('uttucl')
      write(ifch,*)'*****  tucl(,) violates recurrence relation'
      write(ifch,*)sngl(dexp(tucl(1+k,1+n)))
     *,sngl(dexp(tucl(1+k,n))+dexp(tucl(k,1+n)))
      call utmsgf
      endif
5     continue
      endif
 
      if(ish.ge.93)then
121   format(79a1)
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   degeneracy of energy levels of k-dimensional'
     *,' oscillator'
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   =='
     *,'     number of k-tupels of length n (sum_i n_i = n)'
      write(ifch,121)('-',ic=1,79)
      write(ifch,*)'   k:','   1','   2','   3','   4'
      write(ifch,121)('-',ic=1,79)
      do 8 n=0,50
8     write(ifch,*)n,(sngl(dexp( tucl(1+k,1+n) )),k=1,4)
      endif
 
      return
      end

c-----------------------------------------------------------------------
      subroutine utvec1(a,b,c)
c-----------------------------------------------------------------------
c  returns vector product c = a x b .
c-----------------------------------------------------------------------
      real a(3),b(3),c(3)
      c(1)=a(2)*b(3)-a(3)*b(2)
      c(2)=a(3)*b(1)-a(1)*b(3)
      c(3)=a(1)*b(2)-a(2)*b(1)
      return
      end

c-----------------------------------------------------------------------
      subroutine utvec2(a,b,c)
c-----------------------------------------------------------------------
c  returns vector product c = a x b .
c  a,b,c double precision.
c-----------------------------------------------------------------------
      double precision a(3),b(3),c(3)
      c(1)=a(2)*b(3)-a(3)*b(2)
      c(2)=a(3)*b(1)-a(1)*b(3)
      c(3)=a(1)*b(2)-a(2)*b(1)
      return
      end

c-------------------------------------------------------------------
      subroutine utword(line,i,j,iqu)
c-------------------------------------------------------------------
c  finds the first word of the character string line(j+1:80).
c  the word is line(i:j) (with new i and j).
c  if j<0 or if no word found --> new line read.
c  a text between quotes "..." is considered one word; 
c  stronger: a text between double quotes ""..."" is consid one word
c  stronger: a text between "{ and }" is considered one word
c-------------------------------------------------------------------
c  input: 
c    line: character string (*80)
c    i: integer between 1 and 80
c    iqu: for iqu=1 a "?" is written to output before reading a line,
c         otherwise (iqu/=1) nothing is typed
c  output: 
c    i,j: left and right end of word (word=line(i:j))
c-------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro9/radmes,radbar,rinmes,rinbar,epscri,sigmes,sigbar
      common/parob/istore,iprmpt,iecho
      common/copen/nopen
      parameter(mempty=2)
      character*1 empty(mempty),line*80
      character*2 mrk
      data empty/' ',','/

      if(j.ge.0)then
      i=j
      goto1
      endif

    5 continue
      if(iqu.eq.1.and.iprmpt.gt.0)write(ifmt,'(a)')'?'
      read(ifop,'(a80)',end=9999)line
      if(iecho.eq.1)write(ifmt,'(a80)')line
      i=0

    1 i=i+1
      if(i.gt.80.or.line(i:i).eq.'!')goto5
      do ne=1,mempty
      if(line(i:i).eq.empty(ne))goto1
      enddo

      mrk='  '
      if(line(i:i+1).eq.'"{')mrk='}"'
      if(line(i:i+1).eq.'""')mrk='""'
      if(mrk.ne.'  ')goto10
      if(line(i:i).eq.'"')goto8
      j=i-1
    6 j=j+1
      if(j.gt.80)goto7
      if(line(j:j).eq.'!')goto7
      do ne=1,mempty
      if(line(j:j).eq.empty(ne))goto7
      enddo
      goto6

    8 continue
      if(i.ge.79)stop'STOP: syntax error (utword)'
      i=i+1
      j=i
      if(line(j:j).eq.'"')stop'STOP: syntax error (utword)'
    9 j=j+1
      if(j.gt.80)goto7
      if(line(j:j).eq.'"')then
      line(i-1:i-1)=' '
      line(j:j)=' '
      goto7
      endif
      goto9

   10 continue
      if(i.ge.77)stop'STOP: syntax error (utword)'
      i=i+2
      j=i
      if(line(j:j+1).eq.mrk)stop'STOP: syntax error (utword)'
   11 j=j+1
      if(j+1.gt.80)goto7
      if(line(j:j+1).eq.mrk)then
      line(i-2:i-1)='  '
      line(j:j+1)='  '
      goto7
      endif
      goto11

    7 j=j-1
      return

9999  close(ifop)
      nopen=0
      if(iprmpt.eq.-1)iprmpt=1
      goto5
      end

c--------------------------------------------------------------------
      subroutine utworn(line,j,ne)
c--------------------------------------------------------------------
c  returns number ne of nonempty characters of line(j+1:80)
c--------------------------------------------------------------------
      character line*80
      ne=0
      do l=j+1,80
      if(line(l:l).ne.' ')ne=ne+1
      enddo
      return
      end
 
c-----------------------------------------------------------------------
      subroutine veanly
c-----------------------------------------------------------------------
c  analyses venus results
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter (mxtrig=99,mxidco=99)
      character*6 xvaria,yvaria,trvari
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/p11/nrtrig,trmin(mxtrig),trmax(mxtrig),trvari(mxtrig)
      common/p12/nridco,idcode(mxidco)
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (ndep=129,ndet=129,kollmx=5000)
      common/col/rmproj,rmtarg,bmax,bimp,koll,nproj,ntarg
     *,xdep(ndep),qdep(ndep),wdep(ndep),xdet(ndet),qdet(ndet),wdet(ndet)
     *,nrproj(kollmx),nrtarg(kollmx),distce(kollmx),nord(kollmx)
     *,coord(4,kollmx)
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)
      real yacevt(mxbins),yactot(mxbins)
      common/cyield/yield
      character*6 trg
      integer nacevt(mxbins)
      real zactot(mxbins)
      if(ish.ge.94)write(ifch,*)('-',i=1,10)
     *,' entry sr veanly ',('-',i=1,30)

      write(ifmt,'(a)')'histogram filling ...'

      if(xvaria(4:6).ne.yvaria(4:6))
     *stop'xvaria and yvaria have to be of the same type'
      if(nrbins.gt.mxbins)stop'mxbins too small'
      if(nrbins.le.0)stop'negative or zero number of bins'

      do 10 k=1,mxbins
      zactot(k)=0
10    yactot(k)=0
      do 1 k=1,mxbins
      do 1 l=1,5
1     ar(k,l)=0
      par1=0
      par2=0
      par3=0
      par4=0
      ipbi=0
      parpbi=0
      rin1=0
      rin2=1e+10
      if(nrtrig.gt.0)then
      do 4 nt=1,nrtrig
      trg=trvari(nt)
      if(trg(1:3).eq.'par')then
      if(trg.eq.'parp12')par1=trmin(nt)
      if(trg.eq.'parp12')par2=trmax(nt)
      if(trg.eq.'parp34')par3=trmin(nt)
      if(trg.eq.'parp34')par4=trmax(nt)
      if(trg.eq.'parpbi')ipbi=1
      if(trg.eq.'parpbi')parpbi=trmin(nt)
      if(trg.eq.'parrin')rin1=trmin(nt)
      if(trg.eq.'parrin')rin2=trmax(nt)
      if(trg.eq.'parish')ish=nint(trmin(nt))
      if(trg.eq.'partrj')y1=trmin(nt)
      if(trg.eq.'partrj')y2=trmax(nt)
      endif
4     continue
      endif
      xinc=(xmaxim-xminim)/nrbins
      norm1=mod(normal,10)
      norm2=mod(normal/10,10)
      norm3=mod(normal/100,10)
      if(ipbi.eq.1)then
      if(norm1.eq.1.or.norm1.eq.5)stop'ipbi=1 => norm1 .ne. 1,5 !'
      endif

c     read evt
c     --------
      if(ish.ge.94)write(ifch,'(a)')' read from ifdt'
      rewind ifdt
      nevt=0
      nptl=0
      zevt0=0
      nscat=0
96    nevt=nevt+1
      do 29 k=1,nrbins
      nacevt(k)=0
29    yacevt(k)=0
      read(ifdt,end=98)nevts,nptevt,bimevt,phievt,kolevt,colevt
     *,pmxevt,egyevt,npjevt,ntgevt
      if(ish.ge.91)write(ifch,103)nevts,nptevt,bimevt
     *,kolevt,npjevt,ntgevt
103   format(' evt:',i5,i4,f9.2,3i5)
      if(nevts.ne.nevt)stop'***** evt# mismatch during read evt'

      pxlevt=0
      pylevt=0
      pzlevt=0
      enlevt=0
      ev4evt=0
      ev6evt=0
      eplevt=0
      etlevt=0
      ezlevt=0
      mulevt=0
      mufevt=0
      mubevt=0
      mldevt=0
      malevt=0
      mksevt=0
      mngevt=0
      rlnevt=0
      ranevt=0
      rknevt=0

      nptl=0
      do 97 npts=1,nptevt
      read(ifdt)nptls,idp
     *,pxlptl,pylptl,pzlptl,p0lptl,amsptl,iorptl,jorptl,istptl
     *,xo1ptl,xo2ptl,xo3ptl,xo4ptl,t1ptl,t2ptl
      if(ish.ge.94)write(ifch,'(a,i4,i10,5(1x,e8.2),i2,i5)')
     *' ptl:',nptls,idp,pxlptl,pylptl,pzlptl,p0lptl,amsptl
     *,istptl,iorptl
      numptl=1
      nldptl=0
      if(idp.eq.2130)nldptl=1
      nalptl=0
      if(idp.eq.-2130)nalptl=1
      nksptl=0
      if(idp.eq.20)nksptl=1
      nngptl=0
      if(idp.eq.-120.or.idp.eq.-130.or.idp.eq.-1120.or.idp.eq.-1130
     *.or.idp.eq.2230.or.idp.eq.2330.or.idp.eq.3331)nngptl=1
      chrg=0
      if(iabs(idp).le.9999.and.mod(iabs(idp),10).le.1)
     *call idchrg(idp,chrg)
      if(chrg.eq.0.)then
      npmptl=0
      else
      npmptl=sign(1.,chrg)
      endif
      npaptl=isign(1,idp)
      if(pxlptl.eq.0.)phlptl=pi/2
      if(pxlptl.ne.0.)phlptl=abs(atan(pylptl/pxlptl))
      if(pxlptl.lt.0..and.pylptl.ge.0.)phlptl=pi-phlptl
      if(pxlptl.lt.0..and.pylptl.lt.0.)phlptl=pi+phlptl
      if(pxlptl.ge.0..and.pylptl.lt.0.)phlptl=2*pi-phlptl
      phlptl=phlptl/pi*180
      plbptl=sqrt(pxlptl**2+pylptl**2+pzlptl**2)
      ptrptl=sqrt(pxlptl**2+pylptl**2)
      pt2ptl=ptrptl**2
      if(ptrptl.le.1e-6)then
      glbptl=sign(1.,pzlptl)*1e+6
      else
      glbptl=sign(1.,pzlptl)*
     *alog((sqrt(pzlptl**2+ptrptl**2)+abs(pzlptl))/ptrptl)
      endif
      amlptl=-1
      if(amsptl.gt.0.)amlptl=alog10(amsptl)
      amtptl=sqrt(amsptl**2+ptrptl**2)
      ylbptl=sign(1.,pzlptl)*alog((p0lptl+abs(pzlptl))/amtptl)
      ycmptl=ylbptl-yhaha*labsys
      ycnptl=-ycmptl
      yacptl=abs(ycmptl)
      pc1ptl=pxlptl
      pc2ptl=pylptl
      pc3ptl=amtptl*sinh(ycmptl)
      pc4ptl=amtptl*cosh(ycmptl)
      if(ptrptl.le.1e-6)then
      gcmptl=sign(1.,pc3ptl)*1e+6
      else
      gcmptl=sign(1.,pc3ptl)*
     *alog((sqrt(pc3ptl**2+ptrptl**2)+abs(pc3ptl))/ptrptl)
      endif
      xlbptl=0
      if(pmxevt.ne.0.)xlbptl=pzlptl/pmxevt
      xcmptl=xlbptl
      if(iappl.ne.3)xcmptl=0
      if(iappl.ne.3.and.pnllx.ne.0.)xcmptl=pc3ptl/pnllx
      xenptl=0
      if(engy.ne.0.)xenptl=2*pc4ptl/engy
      xelptl=xenptl
      if(iappl.ne.3)xelptl=0.
      if(iappl.ne.3.and.pnll.ne.0.)xelptl=p0lptl/pnll
      xpcptl=0
      if(engy.ne.0.)xpcptl=2*sqrt(pc3ptl**2+ptrptl**2)/engy
      xpaptl=abs(xpcptl)
      xpe=0
      if(engy.ne.0.)xpeptl=2*pc3ptl/engy
      sobptl=ainfin
      if(sqrt(pc3ptl**2+pc2ptl**2+pc1ptl**2).ne.0)
     *sobptl=engy**2/sqrt(pc3ptl**2+pc2ptl**2+pc1ptl**2)*pc4ptl
      eefptl=p0lptl
      if(idp.ge.1000)eefptl=eefptl-prom
      if(idp.le.-1000)eefptl=eefptl+prom
      if(glbptl.gt.4.)ev4evt=ev4evt+eefptl
      if(glbptl.gt.6.)ev6evt=ev6evt+eefptl
      etlptl=0
      if(ptrptl.ne.0)
     *etlptl=eefptl*ptrptl/sqrt(pzlptl**2+ptrptl**2)
      vel=sqrt(pzlptl**2+ptrptl**2)/p0lptl
      if(idp.eq.10)ezlptl=p0lptl
      if(abs(chrg).gt.0.99.and.vel.gt.0.80)ezlptl=.500
      alsptl=alog10(engy**2)
      nufptl=0
      nubptl=0
      if(idp.eq.120 .or.idp.eq.-120.or.idp.eq.130 .or.idp.eq.-130
     *.or.idp.eq.1120.or.idp.eq.-1120.or.idp.eq.1130.or.idp.eq.-1130
     *.or.idp.eq.2230.or.idp.eq.-2230.or.idp.eq.2330.or.idp.eq.-2330
     *.or.idp.eq.3331.or.idp.eq.-3331)then
      if(ycmptl.ge.rin1.and.ycmptl.le.rin2)nufptl=1
      if(ycmptl.ge.-rin2.and.ycmptl.le.-rin1)nubptl=1
      endif
      mufevt=mufevt+nufptl
      mubevt=mubevt+nubptl
      mbfevt=mubevt*mufevt

           if(nridco.gt.0)then
      if(idcode(1).eq.9990)then
      elseif(idcode(1).eq.9980)then
      if(idp.ne.20 .and.idp.ne.-20.and.iabs(idp).lt.100)goto20
      elseif(idcode(1).eq.9970)then
      if(idp.ne.120 .and.idp.ne.-120.and.idp.ne.130 .and.idp.ne.-130
     *.and.idp.ne.1120.and.idp.ne.-1120.and.idp.ne.1130.and.idp.ne.-1130
     *.and.idp.ne.2230.and.idp.ne.-2230.and.idp.ne.2330.and.idp.ne.-2330
     *.and.idp.ne.3331.and.idp.ne.-3331)goto20
      elseif(idcode(1).eq.9960)then
      if(idp.ne.120 .and.idp.ne.130 .and.idp.ne.1120.and.idp.ne.1130
     *.and.idp.ne.-2230.and.idp.ne.-2330.and.idp.ne.-3331)goto20
      elseif(idcode(1).eq.-9960)then
      if(idp.ne.-120.and.idp.ne.-130.and.idp.ne.-1120.and.idp.ne.-1130
     *.and.idp.ne.2230.and.idp.ne.2330.and.idp.ne.3331)goto20
      elseif(idcode(1).eq.9950)then
      if(istptl.ne.0)goto20
      if(iabs(idp).lt.1000)goto20
      elseif(idcode(1).eq.9940)then
      if(istptl.ne.0)goto20
      if(idp.lt.1000)goto20
      elseif(idcode(1).eq.-9940)then
      if(istptl.ne.0)goto20
      if(idp.gt.-1000)goto20
      elseif(idcode(1).eq.9930)then
      if(istptl.ne.0)goto20
      if(idp.ne.20 .and.idp.ne.-20.and.iabs(idp).lt.100)goto20
      if(iabs(idp).gt.1000)goto20
      else
      do 39 n=1,nridco
      if(idp.eq.idcode(n))goto40
39    continue
      goto20
40    continue
      endif
           endif

           if(nrtrig.gt.0)then
      do 22 nt=1,nrtrig
      trg=trvari(nt)
      if(trg(4:6).ne.'ptl')goto22
      tr=0.5*(trmin(nt)+trmax(nt))
      if(trvari(nt).eq.'numptl')tr=numptl
      if(trvari(nt).eq.'nufptl')tr=nufptl
      if(trvari(nt).eq.'nubptl')tr=nubptl
      if(trvari(nt).eq.'npmptl')tr=npmptl
      if(trvari(nt).eq.'npaptl')tr=npaptl
      if(trvari(nt).eq.'xcmptl')tr=xcmptl
      if(trvari(nt).eq.'xenptl')tr=xenptl
      if(trvari(nt).eq.'xelptl')tr=xelptl
      if(trvari(nt).eq.'xpcptl')tr=xpcptl
      if(trvari(nt).eq.'xpaptl')tr=xpaptl
      if(trvari(nt).eq.'xpeptl')tr=xpeptl
      if(trvari(nt).eq.'sobptl')tr=sobptl
      if(trvari(nt).eq.'xlbptl')tr=xlbptl
      if(trvari(nt).eq.'ycmptl')tr=ycmptl
      if(trvari(nt).eq.'ycnptl')tr=ycnptl
      if(trvari(nt).eq.'yacptl')tr=yacptl
      if(trvari(nt).eq.'ylbptl')tr=ylbptl
      if(trvari(nt).eq.'gcmptl')tr=gcmptl
      if(trvari(nt).eq.'glbptl')tr=glbptl
      if(trvari(nt).eq.'ptrptl')tr=ptrptl
      if(trvari(nt).eq.'plbptl')tr=plbptl
      if(trvari(nt).eq.'etlptl')tr=etlptl
      if(trvari(nt).eq.'pt2ptl')tr=pt2ptl
      if(trvari(nt).eq.'amtptl')tr=amtptl
      if(trvari(nt).eq.'amsptl')tr=amsptl
      if(trvari(nt).eq.'amlptl')tr=amlptl
      if(trvari(nt).eq.'phlptl')tr=phlptl
      if(trvari(nt).eq.'alsptl')tr=alsptl
      if(trvari(nt).eq.'iorptl')tr=iorptl
      if(trvari(nt).eq.'p0lptl')tr=p0lptl
      if(tr.lt.trmin(nt).or.tr.gt.trmax(nt))goto20
22    continue
           endif

      if(xvaria.eq.'numptl')x=numptl
      if(xvaria.eq.'nufptl')x=nufptl
      if(xvaria.eq.'nubptl')x=nubptl
      if(xvaria.eq.'npmptl')x=npmptl
      if(xvaria.eq.'npaptl')x=npaptl
      if(xvaria.eq.'xcmptl')x=xcmptl
      if(xvaria.eq.'xenptl')x=xenptl
      if(xvaria.eq.'xelptl')x=xelptl
      if(xvaria.eq.'xpcptl')x=xpcptl
      if(xvaria.eq.'xpaptl')x=xpaptl
      if(xvaria.eq.'xpeptl')x=xpeptl
      if(xvaria.eq.'sobptl')x=sobptl
      if(xvaria.eq.'xlbptl')x=xlbptl
      if(xvaria.eq.'ycmptl')x=ycmptl
      if(xvaria.eq.'ycnptl')x=ycnptl
      if(xvaria.eq.'yacptl')x=yacptl
      if(xvaria.eq.'ylbptl')x=ylbptl
      if(xvaria.eq.'gcmptl')x=gcmptl
      if(xvaria.eq.'glbptl')x=glbptl
      if(xvaria.eq.'ptrptl')x=ptrptl
      if(xvaria.eq.'plbptl')x=plbptl
      if(xvaria.eq.'etlptl')x=etlptl
      if(xvaria.eq.'pt2ptl')x=pt2ptl
      if(xvaria.eq.'amtptl')x=amtptl
      if(xvaria.eq.'amsptl')x=amsptl
      if(xvaria.eq.'amlptl')x=amlptl
      if(xvaria.eq.'phlptl')x=phlptl
      if(xvaria.eq.'alsptl')x=alsptl
      if(xvaria.eq.'iorptl')x=iorptl
      if(xvaria.eq.'p0lptl')x=p0lptl
      if(yvaria.eq.'numptl')y=numptl
      if(yvaria.eq.'nufptl')y=nufptl
      if(yvaria.eq.'nubptl')y=nubptl
      if(yvaria.eq.'npmptl')y=npmptl
      if(yvaria.eq.'npaptl')y=npaptl
      if(yvaria.eq.'xcmptl')y=xcmptl
      if(yvaria.eq.'xenptl')y=xenptl
      if(yvaria.eq.'xelptl')y=xelptl
      if(yvaria.eq.'xpcptl')y=xpcptl
      if(yvaria.eq.'xpaptl')y=xpaptl
      if(yvaria.eq.'xpeptl')y=xpeptl
      if(yvaria.eq.'sobptl')y=sobptl
      if(yvaria.eq.'xlbptl')y=xlbptl
      if(yvaria.eq.'ycmptl')y=ycmptl
      if(yvaria.eq.'ycnptl')y=ycnptl
      if(yvaria.eq.'yacptl')y=yacptl
      if(yvaria.eq.'ylbptl')y=ylbptl
      if(yvaria.eq.'gcmptl')y=gcmptl
      if(yvaria.eq.'glbptl')y=glbptl
      if(yvaria.eq.'ptrptl')y=ptrptl
      if(yvaria.eq.'plbptl')y=plbptl
      if(yvaria.eq.'etlptl')y=etlptl
      if(yvaria.eq.'pt2ptl')y=pt2ptl
      if(yvaria.eq.'amtptl')y=amtptl
      if(yvaria.eq.'amsptl')y=amsptl
      if(yvaria.eq.'amlptl')y=amlptl
      if(yvaria.eq.'phlptl')y=phlptl
      if(yvaria.eq.'alsptl')y=alsptl
      if(yvaria.eq.'iorptl')y=iorptl
      if(yvaria.eq.'p0lptl')y=p0lptl
      pxlevt=pxlevt+pxlptl
      pylevt=pylevt+pylptl
      pzlevt=pzlevt+pzlptl
      enlevt=enlevt+p0lptl
      eplevt=eplevt+eefptl
      etlevt=etlevt+etlptl
      ezlevt=ezlevt+ezlptl
      mulevt=mulevt+numptl
      mldevt=mldevt+nldptl
      malevt=malevt+nalptl
      mksevt=mksevt+nksptl
      itrptl=1
      if(ish.ge.94)write(ifch,101)
     *nevt,nptlo,idp,ycmptl,amsptl,istptl,iorptl
101   format(1x,'accepted:  ',i5,i5,i11,f7.1,2x,e8.2,i4,i10)
      mscat=min(mxbins,nint(mxbins*hisfac))
      if(xvaria(4:6).eq.'ptl'.and.norm3.eq.4.and.nscat.lt.mscat)then
      if(xvaria.eq.'trjptl')then
      nscat=nscat+1
      t1=amin1(t1ptl,y2)
      t1=amax1(t1,y1)
      ar(nscat,1)=xo3ptl+pc3ptl/pc4ptl*(t1-xo4ptl)
      ar(nscat,3)=t1
      nscat=nscat+1
      t2=amin1(t2ptl,y2)
      t2=amax1(t2,y1)
      ar(nscat,1)=xo3ptl+pc3ptl/pc4ptl*(t2-xo4ptl)
      ar(nscat,3)=t2
      else
      nscat=nscat+1
      ar(nscat,1)=x
      ar(nscat,3)=y
      endif
      endif
      goto21
20    itrptl=0
21    continue
      mngevt=mngevt+nngptl
 
c     last ptl in event
c     -----------------
      lptevt=0
      if(npts.eq.nptevt)then
      lptevt=1
      numevt=1
      if(mngevt.gt.0.)rlnevt=mldevt/real(mngevt)
      if(mngevt.gt.0.)ranevt=malevt/real(mngevt)
      if(mngevt.gt.0.)rknevt=mksevt/real(mngevt)
      enlevt=enlevt/1000.
      eplevt=eplevt/1000.
      ev4evt=ev4evt/1000.
      ev6evt=ev6evt/1000.
      alsevt=alog10(engy**2)
      if(nrtrig.gt.0)then
      do 25 nt=1,nrtrig
      trg=trvari(nt)
      if(trg(4:6).ne.'evt')goto25
      tr=0.5*(trmin(nt)+trmax(nt))
      if(trvari(nt).eq.'numevt')tr=numevt
      if(trvari(nt).eq.'xlbevt')tr=pzlevt/pmxevt
      if(trvari(nt).eq.'enlevt')tr=enlevt
      if(trvari(nt).eq.'eplevt')tr=eplevt
      if(trvari(nt).eq.'ev4evt')tr=ev4evt
      if(trvari(nt).eq.'ev6evt')tr=ev6evt
      if(trvari(nt).eq.'alsevt')tr=alsevt
      if(trvari(nt).eq.'etlevt')tr=etlevt
      if(trvari(nt).eq.'ezlevt')tr=ezlevt
      if(trvari(nt).eq.'mulevt')tr=mulevt
      if(trvari(nt).eq.'mufevt')tr=mufevt
      if(trvari(nt).eq.'mubevt')tr=mubevt
      if(trvari(nt).eq.'mbfevt')tr=mbfevt
      if(trvari(nt).eq.'mldevt')tr=mldevt
      if(trvari(nt).eq.'malevt')tr=malevt
      if(trvari(nt).eq.'mksevt')tr=mksevt
      if(trvari(nt).eq.'mngevt')tr=mngevt
      if(trvari(nt).eq.'rlnevt')tr=rlnevt
      if(trvari(nt).eq.'ranevt')tr=ranevt
      if(trvari(nt).eq.'rknevt')tr=rknevt
      if(trvari(nt).eq.'bimevt')tr=bimevt
      if(trvari(nt).eq.'kolevt')tr=kolevt
      if(trvari(nt).eq.'colevt')tr=colevt
      if(trvari(nt).eq.'egyevt')tr=egyevt
      if(trvari(nt).eq.'npjevt')tr=npjevt
      if(trvari(nt).eq.'ntgevt')tr=ntgevt
      if(tr.lt.trmin(nt).or.tr.gt.trmax(nt))goto26
25    continue
      endif
      if(xvaria.eq.'numevt')x=numevt
      if(xvaria.eq.'xlbevt')x=pzlevt/pmxevt
      if(xvaria.eq.'enlevt')x=enlevt
      if(xvaria.eq.'eplevt')x=eplevt
      if(xvaria.eq.'ev4evt')x=ev4evt
      if(xvaria.eq.'ev6evt')x=ev6evt
      if(xvaria.eq.'alsevt')x=alsevt
      if(xvaria.eq.'etlevt')x=etlevt
      if(xvaria.eq.'ezlevt')x=ezlevt
      if(xvaria.eq.'mulevt')x=mulevt
      if(xvaria.eq.'mufevt')x=mufevt
      if(xvaria.eq.'mubevt')x=mubevt
      if(xvaria.eq.'mbfevt')x=mbfevt
      if(xvaria.eq.'mldevt')x=mldevt
      if(xvaria.eq.'malevt')x=malevt
      if(xvaria.eq.'mksevt')x=mksevt
      if(xvaria.eq.'mngevt')x=mngevt
      if(xvaria.eq.'rlnevt')x=rlnevt
      if(xvaria.eq.'ranevt')x=ranevt
      if(xvaria.eq.'rknevt')x=rknevt
      if(xvaria.eq.'bimevt')x=bimevt
      if(xvaria.eq.'kolevt')x=kolevt
      if(xvaria.eq.'colevt')x=colevt
      if(xvaria.eq.'egyevt')x=egyevt
      if(xvaria.eq.'npjevt')x=npjevt
      if(xvaria.eq.'ntgevt')x=ntgevt
      if(yvaria.eq.'numevt')y=numevt
      if(yvaria.eq.'xlbevt')y=pzlevt/pmxevt
      if(yvaria.eq.'enlevt')y=enlevt
      if(yvaria.eq.'eplevt')y=eplevt
      if(yvaria.eq.'ev4evt')y=ev4evt
      if(yvaria.eq.'ev6evt')y=ev6evt
      if(yvaria.eq.'alsevt')y=alsevt
      if(yvaria.eq.'etlevt')y=etlevt
      if(yvaria.eq.'ezlevt')y=ezlevt
      if(yvaria.eq.'mulevt')y=mulevt
      if(yvaria.eq.'mufevt')y=mufevt
      if(yvaria.eq.'mubevt')y=mubevt
      if(yvaria.eq.'mbfevt')y=mbfevt
      if(yvaria.eq.'mldevt')y=mldevt
      if(yvaria.eq.'malevt')y=malevt
      if(yvaria.eq.'mksevt')y=mksevt
      if(yvaria.eq.'mngevt')y=mngevt
      if(yvaria.eq.'rlnevt')y=rlnevt
      if(yvaria.eq.'ranevt')y=ranevt
      if(yvaria.eq.'rknevt')y=rknevt
      if(yvaria.eq.'bimevt')y=bimevt
      if(yvaria.eq.'kolevt')y=kolevt
      if(yvaria.eq.'colevt')y=colevt
      if(yvaria.eq.'egyevt')y=egyevt
      if(yvaria.eq.'npjevt')y=npjevt
      if(yvaria.eq.'ntgevt')y=ntgevt
      if(xvaria(4:6).eq.'evt'.and.norm3.eq.4.and.nscat.lt.mxbins)then
      nscat=nscat+1
      ar(nscat,1)=x
      ar(nscat,3)=y
      endif
      itrevt=1
      goto27
26    itrevt=0
27    continue
      endif
 
      if((xvaria(4:6).eq.'ptl'.and.itrptl.eq.1
     *.or.xvaria(4:6).eq.'evt'.and.itrevt.eq.1.and.lptevt.eq.1)
     *.and.norm3.ne.4)then
      if(x.ge.xminim.and.x.le.xmaxim)then
      if(norm3.eq.1)y=y*x
      if(norm3.eq.2)y=y/(2*pi*x)
      if(norm3.eq.5)then
      if(x**2.gt.amsptl**2)then
      y=y/(4*pi*x*sqrt(x**2-amsptl**2))
      else
      y=0
      endif
      endif
      endif
      k=1+(x-xminim)/xinc
      if(k.ge.1.and.k.le.nrbins)nacevt(k)=nacevt(k)+1
      if(k.ge.1.and.k.le.nrbins)yacevt(k)=yacevt(k)+y
      endif
      if(lptevt.eq.1.and.itrevt.eq.1)then
      if(ipbi.ne.1)wei=1
      if(ipbi.eq.1)wei=parpbi*kolevt
      zevt0=zevt0+wei
      do 28 k=1,nrbins
      zactot(k)=zactot(k)+nacevt(k)
28    yactot(k)=yactot(k)+yacevt(k)*wei
      endif
 
97    continue
      goto96
98    continue
      nevt=nevt-1

c     normalization
c     ------------- 
      zcount=0
      if(norm3.eq.4)then
      first=0
      varnce=0
      goto1000
      endif
      yacsum=0
      do 30 k=1,nrbins
      ar(k,1)=xminim+(k-.5)*xinc
      if(norm1.ne.4)ar(k,3)=yactot(k)
      if(norm1.eq.4.and.yactot(k).ne.0.)ar(k,3)=yactot(k)/zactot(k)
      ar(k,5)=zactot(k)
      yacsum=yacsum+yactot(k)
30    zcount=zcount+zactot(k)
      yield=zcount/zevt0
      if(bmax.eq.0.)sgm=sigppi*10
      if(bmax.ne.0.)sgm=pi*bmax**2*nevt/ntevt *10
      if(norm1.eq.0)cnorm=1.
      if(norm1.eq.1)cnorm=1./nevt
      if(norm1.eq.2.and.zevt0.ne.0.)cnorm=1/zevt0
      if(norm1.eq.3.and.zcount.ne.0.)cnorm=1/zcount
      if(norm1.eq.4)cnorm=1.
      if(norm1.eq.5.and.naevt.ne.0)cnorm=1./naevt
      if(norm1.eq.5.and.naevt.eq.0)cnorm=0
      if(norm1.eq.6.and.yacsum.ne.0.)cnorm=1./yacsum
      if(norm2.eq.1)cnorm=cnorm/xinc
      if(norm2.eq.2)cnorm=cnorm/xinc*sgm
      first=0
      secnd=0
      do 31 k=1,nrbins
      if(zcount.ne.0.)first=first+ar(k,1)*ar(k,5)/zcount
      if(zcount.ne.0.)secnd=secnd+ar(k,1)**2*ar(k,5)/zcount
31    continue
      varnce=secnd-first**2
      do 32 k=1,nrbins
      ar(k,3)=ar(k,3)*cnorm*hisfac
      if(norm3.eq.3)ar(k,3)=ar(k,3)*first
      if(norm3.eq.3)ar(k,1)=ar(k,1)/first
      if(ar(k,5).ne.0.)ar(k,4)=ar(k,3)/sqrt(ar(k,5))
32    continue

1000  continue
      if(norm3.eq.4)nrbins=nscat
      if(ish.ge.94)write(ifch,*)('-',i=1,30)
     *,' exit sr veanly ',('-',i=1,10)
      return
      end

c-----------------------------------------------------------------------
      subroutine wclu
c     analysis. droplet (resonance) formation and decay.
c     fills arclu(n,k); k:mass bin.
c     n=1,12: droplet formation (m=meson, b=baryon, ab=antibaryon
c       s=strange, as=antistrange, ns=nonstrange).
c       arclu contains the number of droplets for the foll. reactions:
c          n=1:   m+m
c          n=2:   m+b
c          n=3:   b+b
c          n=4:   m+ab
c          n=5:   b+ab
c          n=6:   p+p
c          n=7:   p+lda
c          n=8:   m/s+b/ns
c          n=9:   m/ns+b/s
c          n=10:  m/ns+ab/as
c          n=11:  m/as+ab/ns
c          n=12:  m/ns+ab/ns
c     n=13,60: droplet decay.
c          n=13,16:  # of all, k+, k-, lda   from m
c          n=17,20:  # of all, k+, k-, lda   from b
c          n=21,24:  # of all, k+, k-, lda   from bb (6 quarks)
c          n=25,36:  like 13-24, however nonstrange droplets required
c          n=37,60:  # of droplets corr to n=13,36 (for norm.)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      integer ifuptl(mxptl)
      parameter(nclu=60,kclu=25)
      common/cclu/arclu(nclu,kclu),aclu(kclu)
      real tar(nclu)
      parameter (ncomx=100)
      integer iar(ncomx)
      parameter (nflav=6)
      integer jcdu(nflav,2)
      amin=0
      amax=5
 
      if(nrevt.eq.1)then
      do 44 k=1,kclu
      aclu(k)=amin+(k-.5)*(amax-amin)/kclu
      do 44 n=1,nclu
44    arclu(n,k)=0
      endif
 
      do 1 i=1,nptl
      ifuptl(i)=0
      if(istptl(i).ne.1.or.iorptl(i).eq.-1)goto1
         if(ifrptl(1,i).ne.0.and.istptl(ifrptl(1,i)).ne.2)then
      if(ifrptl(2,i).lt.ifrptl(1,i))
     *call utstop('wclu: ifrptl2<ifrptl1                   ')
      en=pptl(4,i)
      do 2 j=ifrptl(1,i),ifrptl(2,i)
2     en=en-pptl(4,j)
      if(abs(en/pptl(4,i)).gt.1e-3)then
      if(ish.ge.90)then
      call utmsg('jclu  ')
      write(ifch,*)'*****  en_droplet.ne.en_fragments.'
      write(ifch,*)'en_cl, en_cl-en_fr:',pptl(4,i),en
      call utmsgf
      endif
      endif
         else
      ifuptl(i)=2
         endif
1     continue
 
      if(ish.eq.10)then
      write(ifch,101)('-',i=1,16)
101   format(1x,'ptl list in wclu'/1x,16a1)
      do 10 i=1,nptl
10    write(ifch,100)i,idptl(i),ifuptl(i)
     *,iorptl(i),jorptl(i),ifrptl(1,i),ifrptl(2,i),istptl(i)
100   format(1x,i4,i10,i6,7x,2i4,7x,2i4,i8)
      endif
 
      do 3 i=1,nptl
      if(istptl(i).eq.2)goto3
      io=iorptl(i)
      jo=jorptl(i)
      if(jo.eq.0)goto3
      ifuptl(io)=ifuptl(io)-2
      ifuptl(jo)=ifuptl(jo)-2
      if(abs(pptl(4,i)-pptl(4,io)-pptl(4,jo)).gt.1e-3)then
      ifuptl(io)=ifuptl(io)+1
      ifuptl(jo)=ifuptl(jo)+1
      goto3
      endif
      idi=idptl(io)
      idj=idptl(jo)
      call idquac(io,nqi,nsi,ndummy,jcdu)
      call idquac(jo,nqj,nsj,ndummy,jcdu)
      k=1+(pptl(5,i)-amin)/(amax-amin)*kclu
         if(k.ge.1.and.k.le.kclu)then
      if(nqi.eq.0.and.nqj.eq.0)       arclu( 1,k)=arclu( 1,k)+1
      if(nqi.eq.0.and.nqj.eq.3
     *.or.nqj.eq.0.and.nqi.eq.3)      arclu( 2,k)=arclu( 2,k)+1
      if(nqi.eq.3.and.nqj.eq.3)       arclu( 3,k)=arclu( 3,k)+1
      if(nqi.eq.0.and.nqj.eq.-3
     *.or.nqj.eq.0.and.nqi.eq.-3)     arclu( 4,k)=arclu( 4,k)+1
      if(nqi.eq.3.and.nqj.eq.-3
     *.or.nqj.eq.3.and.nqi.eq.-3)     arclu( 5,k)=arclu( 5,k)+1
      if(idi.eq.1120.and.idj.eq.1120) arclu( 6,k)=arclu( 6,k)+1
      if(idi.eq.1120.and.idj.eq.2130
     *.or.idj.eq.1120.and.idi.eq.2130)arclu( 7,k)=arclu( 7,k)+1
      if(nqi.eq.0.and.nsi.eq.1
     *.and.nqj.eq.3.and.nsj.eq.0
     *.or.nqj.eq.0.and.nsj.eq.1
     *.and.nqi.eq.3.and.nsi.eq.0)     arclu( 8,k)=arclu( 8,k)+1
      if(nqi.eq.0.and.nsi.eq.0
     *.and.nqj.eq.3.and.nsj.eq.1
     *.or.nqj.eq.0.and.nsj.eq.0
     *.and.nqi.eq.3.and.nsi.eq.1)     arclu( 9,k)=arclu( 9,k)+1
      if(nqi.eq.0.and.nsi.eq.0
     *.and.nqj.eq.-3.and.nsj.eq.-1
     *.or.nqj.eq.0.and.nsj.eq.0
     *.and.nqi.eq.-3.and.nsi.eq.-1)   arclu(10,k)=arclu(10,k)+1
      if(nqi.eq.0.and.nsi.eq.-1
     *.and.nqj.eq.-3.and.nsj.eq.0
     *.or.nqj.eq.0.and.nsj.eq.-1
     *.and.nqi.eq.-3.and.nsi.eq.0)    arclu(11,k)=arclu(11,k)+1
      if(nqi.eq.0.and.nsi.eq.0
     *.and.nqj.eq.-3.and.nsj.eq.0
     *.or.nqj.eq.0.and.nsj.eq.0
     *.and.nqi.eq.-3.and.nsi.eq.0)    arclu(12,k)=arclu(12,k)+1
         endif
3     continue
 
      do 4 i=1,nptl
      if(ifuptl(i).eq.0)goto4
      if(ish.ge.90
     *.or.ish.eq.10)then
      call utmsg('jclu  ')
      write(ifch,*)'*****  ifuptl .ne. 0.'
      write(ifch,*)'i,ifuptl(i): ',i,ifuptl(i)
      call utmsgf
      endif
4     continue
 
      do 5 i=1,nptl
      if(istptl(i).ne.1.or.iorptl(i).eq.-1)goto5
      if(.not.(ifrptl(1,i).ne.0.and.istptl(ifrptl(1,i)).ne.2))goto5
      k=1+(pptl(5,i)-amin)/(amax-amin)*kclu
      if(k.lt.1.or.k.gt.kclu)goto5
      call idquac(i,nq,ns,ndummy,jcdu)
      do 45 n=1,nclu
45    tar(n)=0
      iar(1)=i
      nco=1
      npo=0
7     npo=npo+1
      ipo=iar(npo)
         if(istptl(ipo).eq.2)goto5
         if(istptl(ipo).eq.0)then
      id=idptl(ipo)
      if(nq.eq.0)               tar(13)=tar(13)+1
      if(nq.eq.0.and.id.eq.130) tar(14)=tar(14)+1
      if(nq.eq.0.and.id.eq.-130)tar(15)=tar(15)+1
      if(nq.eq.0.and.id.eq.2130)tar(16)=tar(16)+1
      if(nq.eq.3)               tar(17)=tar(17)+1
      if(nq.eq.3.and.id.eq.130) tar(18)=tar(18)+1
      if(nq.eq.3.and.id.eq.-130)tar(19)=tar(19)+1
      if(nq.eq.3.and.id.eq.2130)tar(20)=tar(20)+1
      if(nq.eq.6)               tar(21)=tar(21)+1
      if(nq.eq.6.and.id.eq.130) tar(22)=tar(22)+1
      if(nq.eq.6.and.id.eq.-130)tar(23)=tar(23)+1
      if(nq.eq.6.and.id.eq.2130)tar(24)=tar(24)+1
      if(ns.eq.0.and.nq.eq.0)               tar(25)=tar(25)+1
      if(ns.eq.0.and.nq.eq.0.and.id.eq.130) tar(26)=tar(26)+1
      if(ns.eq.0.and.nq.eq.0.and.id.eq.-130)tar(27)=tar(27)+1
      if(ns.eq.0.and.nq.eq.0.and.id.eq.2130)tar(28)=tar(28)+1
      if(ns.eq.0.and.nq.eq.3)               tar(29)=tar(29)+1
      if(ns.eq.0.and.nq.eq.3.and.id.eq.130) tar(30)=tar(30)+1
      if(ns.eq.0.and.nq.eq.3.and.id.eq.-130)tar(31)=tar(31)+1
      if(ns.eq.0.and.nq.eq.3.and.id.eq.2130)tar(32)=tar(32)+1
      if(ns.eq.0.and.nq.eq.6)               tar(33)=tar(33)+1
      if(ns.eq.0.and.nq.eq.6.and.id.eq.130) tar(34)=tar(34)+1
      if(ns.eq.0.and.nq.eq.6.and.id.eq.-130)tar(35)=tar(35)+1
      if(ns.eq.0.and.nq.eq.6.and.id.eq.2130)tar(36)=tar(36)+1
      if(npo.lt.nco)goto7
         elseif(ifrptl(1,ipo).ne.0.and.istptl(ifrptl(1,ipo)).ne.2)then
      do 6 j=ifrptl(1,ipo),ifrptl(2,ipo)
      nco=nco+1
      if(nco.gt.ncomx)
     *call utstop('wclu: ncomx too small. increase dim.    ')
6     iar(nco)=j
      goto7
         else
      goto5
         endif
      if(nq.eq.0)            tar(37)=tar(37)+1
      if(nq.eq.3)            tar(41)=tar(41)+1
      if(nq.eq.6)            tar(45)=tar(45)+1
      if(ns.eq.0.and.nq.eq.0)tar(49)=tar(49)+1
      if(ns.eq.0.and.nq.eq.3)tar(53)=tar(53)+1
      if(ns.eq.0.and.nq.eq.6)tar(57)=tar(57)+1
      do 8 n=13,36
8     arclu(n,k)=arclu(n,k)+tar(n)
      do 9 n=37,60
9     arclu(n,k)=arclu(n,k)+tar(1+(n-1)/4*4)
5     continue
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine wclufi
c     analysis. droplet (resonance) formation and decay. normalization.
c     0 <= n <= 36, meaning of n see initial comment in subr wclu.
c-----------------------------------------------------------------------
      character*6 xvaria,yvaria
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter(nclu=60,kclu=25)
      common/cclu/arclu(nclu,kclu),aclu(kclu)
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)

      do n=0,36

      nrbins=kclu
      da=aclu(2)-aclu(1)
      do 1 k=1,kclu
      ar(k,1)=aclu(k)
      if(n.ge.1.and.n.le.12.and.arclu(n,k).gt.0.)then
      ar(k,3)=arclu(n,k)/nrevt/da
      ar(k,4)=arclu(n,k)/nrevt/da  /   sqrt(arclu(n,k))
      elseif(n.ge.13.and.n.le.36.and.arclu(n,k).gt.0.)then
      ar(k,3)=arclu(n,k)/arclu(n+24,k)
      ar(k,4)=arclu(n,k)/arclu(n+24,k)  /  sqrt(arclu(n,k))
      elseif(arclu(n,k).gt.0.)then
      call utstop('wclufi: only optn 1-36 possible.        ')
      endif
1     continue

      enddo

      return
      end
 
c-----------------------------------------------------------------------
      subroutine wimi
c     analysis. calculates factorial moments.
c-----------------------------------------------------------------------
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      integer nhis(32)
      double precision afacmo
      common /cfacmo/afacmo(5,32)
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      common/cwimiy/ymin,ymax
      integer ifacmo(5)
      ymin=-ymximi
      ymax=ymximi
 
      if(nrevt.eq.1)then
      do 44 m=1,5
      do 44 kmax=1,32
44    afacmo(m,kmax)=0
      endif
 
      do 45 kmax=1,32
      do 40 m=1,5
40    ifacmo(m)=0
      do 47 k=1,32
47    nhis(k)=0
      do 46 i=1,nptl
      if(istptl(i).ne.0)goto46
      amtptl=sqrt(pptl(5,i)**2+pptl(1,i)**2+pptl(2,i)**2)
      ylbptl=sign(1.,pptl(3,i))*alog((pptl(4,i)+abs(pptl(3,i)))/amtptl)
      k=1+(ylbptl-ymin)/(ymax-ymin)*kmax
      if(k.ge.1.and.k.le.kmax)nhis(k)=nhis(k)+1
46    continue
      do 41 k=1,kmax
      n=nhis(k)
      ifacmo(1)=ifacmo(1)+n
      ifacmo(2)=ifacmo(2)+n*(n-1)
      ifacmo(3)=ifacmo(3)+n*(n-1)*(n-2)
      ifacmo(4)=ifacmo(4)+n*(n-1)*(n-2)*(n-3)
      ifacmo(5)=ifacmo(5)+n*(n-1)*(n-2)*(n-3)*(n-4)
41    continue
      do 42 m=1,5
42    afacmo(m,kmax)=afacmo(m,kmax)+1d0*ifacmo(m)/kmax
45    continue
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine wimino
c     analysis. normalizes factorial moments.
c-----------------------------------------------------------------------
      character*6 xvaria,yvaria
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      double precision afacmo,f1,fm
      common /cfacmo/afacmo(5,32)
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)
      common/cwimiy/ymin,ymax

      do mom=1,5

      nrbins=32
      do 45 kmax=1,32
      f1=afacmo(  1,kmax)/nrevt
      fm=afacmo(mom,kmax)/nrevt/f1**mom
      ar(kmax,1)=-alog((ymax-ymin)/kmax)
      if(fm.le.0d0)ar(kmax,3)=-ainfin
      if(fm.gt.0d0)ar(kmax,3)=dlog(fm)
45    continue

      enddo

      return
      end
 
c-----------------------------------------------------------------------
      subroutine wjpsi
c     jpsi analysis.
c-----------------------------------------------------------------------
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (njpsi=50)
      common /cjpsi/xjpsi(njpsi),yjpsi(njpsi),zjpsi(njpsi)
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/tpro,zpro,ttar,ztar,ttaus,detap,detat /ctain/mtain

      detap=(ypjtl-yhaha)*etafac
      detat=-yhaha*etafac
      tpro=dcosh(detap)
      zpro=dsinh(detap)
      ttar=dcosh(detat)
      ztar=dsinh(detat)
 
      radsqr=1
      taumi=0
      del=0.2
      fac=1.041
 
      if(nrevt.eq.1)then
      do 4 nt=1,njpsi
      taus=taumi+del*(fac**(nt-1.)-1)/(fac-1)
      xjpsi(nt)=taus
      yjpsi(nt)=0
4     zjpsi(nt)=0
      endif
 
      do 1 i=1,nptl
      if(idptl(i).eq.441)j=i
1     continue
 
      do 2 nt=1,njpsi
      ttaus=xjpsi(nt)
      call uttain(j,xj,yj,zj,tj,n,1)
      if(n.eq.1.or.n.eq.2.or.n.eq.9)goto2
      zjpsi(nt)=zjpsi(nt)+1
      do 3 i=1,nptl
      if(i.eq.j)goto3
      call uttain(i,x,y,z,t,n,1)
      if(n.eq.1.or.n.eq.2.or.n.eq.9)goto3
      pde=(pptl(3,i)+pptl(3,j))/(pptl(4,i)+pptl(4,j))
      gam2i=1-pde**2
      if(gam2i.eq.0.)goto3
      if((x-xj)**2+(y-yj)**2+1/gam2i*(z-zj-(t-tj)*pde)**2
     *.gt.radsqr)goto3
      yjpsi(nt)=yjpsi(nt)+1
3     continue
2     continue
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine wjpsif
c     jpsi analysis. normalized histogram -> /canar/
c-----------------------------------------------------------------------
      character*6 xvaria,yvaria
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)
      parameter (njpsi=50)
      common /cjpsi/xjpsi(njpsi),yjpsi(njpsi),zjpsi(njpsi)
 
      do 1 i=1,njpsi
      ar(i,1)=xjpsi(i)
      ar(i,2)=0
      if(zjpsi(i).eq.0.)ar(i,3)=0.
      if(zjpsi(i).ne.0.)ar(i,3)=yjpsi(i)/zjpsi(i)
      ar(i,4)=0
      ar(i,5)=zjpsi(i)/nrevt
1     continue
 
      nrbins=njpsi
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine wspa
c     analysis. space-time evolution.
c     fills ar(i,j,k) with
c        ar: dspa:sum{1}, sspa:sum{strangn}, cspa:sum{p3/p4},
c            espa:sum{p4}, pspa:sum{p3}, tspa:sum{p3**2/p4}
c            aspa:sum{p5}, px2spa:sum{p1**2}, py2spa:sum{p2**2}
c        i: z-bin number  (for iwcent=1:baryon#-bin)
c        j: t-bin number
c        k: meson (1) or baryon (2)  (for iwcent=1: only k=1 filled)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      parameter (nspaz=51,nspat=9)
      common /cspati/dspa(nspaz,nspat,2),cspa(nspaz,nspat,2)
     *,espa(nspaz,nspat,2),pspa(nspaz,nspat,2),tspa(nspaz,nspat,2)
     *,aspa(nspaz,nspat,2),px2spa(nspaz,nspat,2),py2spa(nspaz,nspat,2)
     *,zspa(nspaz),sspa(nspaz,nspat,2),atra,dznull
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      integer ibaptl(mxptl),iacptl(mxptl),ntbptl(mxptl),ntsptl(mxptl)
      real amiptl(mxptl)
      parameter (nflav=6)
      integer jcdu(nflav,2)
      parameter (mxifr=mxptl)
      integer ifr(mxifr),ifrx(mxifr)
      mami=min0(maproj,matarg)
      r0=1.19*mami**(1./3.)-1.61*mami**(-1./3.)
      atra=(r0/2)**2*pi
      tmin=wtmini
      tstep=wtstep
      zmin=-6
      zmax=6
      iz0=(nspaz-1)/2+1
 
      if(nrevt.eq.1)then
      do 44 i=1,nspaz
      zspa(i)=zmin+(i-.5)*(zmax-zmin)/nspaz
      if(i.eq.iz0+4)zplus=zspa(i)
      if(i.eq.iz0-5)zminus=zspa(i)
      if(iwcent.eq.1)zspa(i)=i-25
      do 44 j=1,nspat
      do 44 k=1,2
      dspa(i,j,k)=0.
      sspa(i,j,k)=0.
      cspa(i,j,k)=0.
      espa(i,j,k)=0.
      pspa(i,j,k)=0.
      tspa(i,j,k)=0.
      aspa(i,j,k)=0.
      px2spa(i,j,k)=0.
44    py2spa(i,j,k)=0.
      dznull=zplus-zminus
      endif
 
      do 52 i=1,nptl
      call idquac(i,nq,ns,ndummy,jcdu)
      ibaptl(i)=1
      if(nq.ne.0)ibaptl(i)=2
      ntbptl(i)=nq/3
      ntsptl(i)=ns
      amiptl(i)=prom+amsiac
      if(nq.eq.0)amiptl(i)=piom+amsiac
52    continue
 
      nr=0
      do 45 it=1,nspat
      t=tmin+(it-1)*tstep
      do 43 i=1,nptl
43    iacptl(i)=1
      do 46 i=1,nptl
      if(istptl(i).gt.1)goto46
      if(iacptl(i).eq.0)goto46
      if(iabs(idptl(i)).lt.100.and.iabs(idptl(i)).ne.20)goto46
      if(ispall.eq.1)then
      if(t.lt.xorptl(4,i).or.t.ge.tivptl(2,i))goto46
      else
      if(t.lt.tivptl(1,i).or.t.ge.tivptl(2,i))goto46
      if(iclptl(i).eq.0.and.pptl(5,i).gt.amiptl(i))goto46
      endif
      x=xorptl(1,i)+(t-xorptl(4,i))*pptl(1,i)/pptl(4,i)
      y=xorptl(2,i)+(t-xorptl(4,i))*pptl(2,i)/pptl(4,i)
      z=xorptl(3,i)+(t-xorptl(4,i))*pptl(3,i)/pptl(4,i)
      if(x**2+y**2.gt.atra/pi)goto46
      iz=1+(z-zmin)/(zmax-zmin)*nspaz
      if(iz.lt.1.or.iz.gt.nspaz)goto46
      ib=ibaptl(i)
      if(iwcent.eq.1)then
      if(iz.lt.iz0-4.or.iz.gt.iz0+4)goto46
      iz=25+ntbptl(i)
      if(iz.lt.1.or.iz.gt.nspaz)goto46
      ib=1
      endif
      nr=nr+1
      dspa(iz,it,ib)=dspa(iz,it,ib)+1
      sspa(iz,it,ib)=sspa(iz,it,ib)+ntsptl(i)
      cspa(iz,it,ib)=cspa(iz,it,ib)+pptl(3,i)/pptl(4,i)
      espa(iz,it,ib)=espa(iz,it,ib)+pptl(4,i)
      pspa(iz,it,ib)=pspa(iz,it,ib)+pptl(3,i)
      tspa(iz,it,ib)=tspa(iz,it,ib)+pptl(3,i)**2/pptl(4,i)
      aspa(iz,it,ib)=aspa(iz,it,ib)+pptl(5,i)
      px2spa(iz,it,ib)=px2spa(iz,it,ib)+pptl(1,i)**2
      py2spa(iz,it,ib)=py2spa(iz,it,ib)+pptl(2,i)**2
      write(ifdt)nr,iz,it,ib,pptl(3,i),pptl(4,i)
 
c     children: iacptl=0
c     ------------------
      k1=0
      k2=0
      loop=0
31    loop=loop+1
      if(loop.eq.1)n=i
      if(ifrptl(1,n).gt.0)then
      if(ish.ge.92)write(ifch,*)n,' ---> ',ifrptl(1,n),ifrptl(2,n)
      do 30 k=ifrptl(1,n),ifrptl(2,n)
      if(iacptl(k).eq.0)goto30
      iacptl(k)=0
      k2=k2+1
           if(k2.gt.mxifr/10)then
      if(ish.ge.90.and.k2-1.eq.mxifr/10)then
      call utmsg('wspa  ')
      write(ifch,*)'*****  k2= ',k2,' > mxifr/10 = ',mxifr/10
      do 55 kk=1,k2
55    write(ifch,*)kk,'        ',ifrx(kk),'  --->  ',ifr(kk)
      call utmsgf
      endif
      if(k2.gt.mxifr)then
      call utmsg('wspa  ')
      write(ifch,*)'*****  k2= ',k2,' > mxifr = ',mxifr
      call utstop('wspa  : k2 > mxifr                      ')
      endif
           endif
      ifrx(k2)=0
      if(k.eq.ifrptl(1,n))ifrx(k2)=n
      ifr(k2)=k
30    continue
      endif
      k1=k1+1
      if(k1.le.k2)then
      n=ifr(k1)
      if(iacptl(n).ne.0)
     *call utstop('wspa  : should not happen               ')
      goto31
      endif
 
c     parents: iacptl=0
c     -----------------
      k1=0
      k2=0
      loop=0
12    loop=loop+1
      if(loop.eq.1)n=i
      if(iorptl(n).gt.0)then
      if(ish.ge.93)write(ifch,*)n,' <--- ',iorptl(n),jorptl(n)
           if(k2+2.gt.mxifr/10)then
      if(ish.ge.90.and.(k2+1.eq.mxifr/10.or.k2.eq.mxifr/10))then
      call utmsg('wspa  ')
      write(ifch,*)'*****  k2+2= ',k2+2,' > mxifr/10 = ',mxifr/10
      do 56 kk=1,k2
56    write(ifch,*)kk,'        ',ifrx(kk),'  <---  ',ifr(kk)
      call utmsgf
      endif
      if(k2+2.gt.mxifr)then
      call utmsg('wspa  ')
      write(ifch,*)'*****  k2+2= ',k2+2,' > mxifr = ',mxifr
      call utstop('wspa  : k2+2 > mxifr                    ')
      endif
           endif
      if(iacptl(iorptl(n)).ne.0)then
      k2=k2+1
      ifr(k2)=iorptl(n)
      ifrx(k2)=n
      iacptl(iorptl(n))=0
      endif
      if(jorptl(n).gt.0)then
      if(iacptl(jorptl(n)).ne.0)then
      k2=k2+1
      ifr(k2)=jorptl(n)
      ifrx(k2)=0
      iacptl(jorptl(n))=0
      endif
      endif
      endif
      k1=k1+1
      if(k1.le.k2)then
      n=ifr(k1)
      if(iacptl(n).ne.0)
     *call utstop('wspa  : should not happen (2)           ')
      goto12
      endif
 
46    continue
45    continue
 
      return
      end
 
c-----------------------------------------------------------------------
      subroutine wspafi
c     analysis. space-time evolution.
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/parog/isphis,ispall,wtmini,wtstep,iwcent
      character*6 xvaria,yvaria
      common/p10/xvaria,yvaria,normal,xminim,xmaxim,nrbins,hisfac
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter(mxbins=10000)
      common/canar/ar(mxbins,5)
      parameter (nspaz=51,nspat=9)
      common /cspati/dspa(nspaz,nspat,2),cspa(nspaz,nspat,2)
     *,espa(nspaz,nspat,2),pspa(nspaz,nspat,2),tspa(nspaz,nspat,2)
     *,aspa(nspaz,nspat,2),px2spa(nspaz,nspat,2),py2spa(nspaz,nspat,2)
     *,zspa(nspaz),sspa(nspaz,nspat,2),atra,dznull
      real vav(nspaz),rav(nspaz),pz2(nspaz)

      do loop=1,2

      if(loop.eq.1)ise=0321
      if(loop.eq.2)ise=0721

c     ise=dcba: a=1:mesons ,a=2:baryons  (iwcent=1:a set to 1)
c               b=1:velocity, b=2:rapidity, b=3:density, b=4:en.density
c               b=5:mass/ptl, b=6:en*/ptl, b=7:pz2, b=8:px2, b=9:px2/pz2
c               b=0:strangeness/ptl
c               dc:time
 
      nrbins=nspaz
      k=mod(ise,10)
      m=mod(ise/10,10)
      j=mod(ise/100,100)
      if(iwcent.eq.1)k=1
 
      do 44 i=1,nspaz
      pz2(i)=0
      vav(i)=0
      rav(i)=0
      if(dspa(i,j,k).eq.0.)goto44
      vav(i)=pspa(i,j,k)/espa(i,j,k)
      rav(i)=.5*alog((1+vav(i))/(1-vav(i)))
44    continue
 
      rewind ifdt
1     read(ifdt,end=2)nr,i,js,ks,ps,es
      if(js.eq.j.and.ks.eq.k)then
      p=-sinh(rav(i))*es+cosh(rav(i))*ps
      pz2(i)=pz2(i)+p**2
      endif
      goto1
2     continue
 
      do 45 i=1,nspaz
      ar(i,1)=zspa(i)
      if(dspa(i,j,k).eq.0.)goto45
      if(i.eq.1)dz=zspa(2)-zspa(1)
      if(i.gt.1)dz=zspa(i)-zspa(i-1)
      if(iwcent.eq.1)dz=dznull
      ano=1./nrevt/atra/dz
      gma=1./sqrt(1-vav(i)**2)
      den=(dspa(i,j,k)-cspa(i,j,k)*vav(i))*gma*ano
      ede=(espa(i,j,k)-2*pspa(i,j,k)*vav(i)+tspa(i,j,k)*vav(i)**2)*
     *gma**2*ano
      if(m.eq.1)ar(i,3)=vav(i)
      if(m.eq.2)ar(i,3)=rav(i)
      if(m.eq.3)ar(i,3)=den
      if(m.eq.4)ar(i,3)=ede
      if(m.eq.5)ar(i,3)=aspa(i,j,k)/dspa(i,j,k)
      if(m.eq.6)ar(i,3)=ede/den
      if(m.eq.7)ar(i,3)=pz2(i)/dspa(i,j,k)
      if(m.eq.8)ar(i,3)=px2spa(i,j,k)/dspa(i,j,k)
      if(m.eq.9.and.pz2(i).ne.0.)ar(i,3)=px2spa(i,j,k)/pz2(i)
      if(m.eq.0)ar(i,3)=sspa(i,j,k)/dspa(i,j,k)
45    continue

      enddo
 
      return
      end

c------------------------------------------------------------------------------
      subroutine wtime(mode)
c------------------------------------------------------------------------------
c determines (mode=1) and plots (mode=2) formation time distribution
c------------------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/parof/ymximi,imihis,iclhis,iwtime,wtimet,wtimei,wtimea
      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)
      parameter (nids=3,nbns=100)
      dimension xx(nbns)
      common/cwtime/yy(nids,nbns),ww(nids,nbns)
      character cid*7
      common/ccttim/ncttim
      data ncnt/0/

      tmax=wtimet
      amin=wtimei
      amax=wtimea

           if(mode.eq.1)then

      ncttim=ncttim+1

      if(ncttim.eq.1)then
      do j=1,nids
      do nn=1,nbns
      yy(j,nn)=0
      enddo
      enddo
      endif

      do n=1,nptl
      if(istptl(n).ne.0)goto3

      id=idptl(n)

c determine mass am of origin of id
      i=n
    4 continue
      io=iorptl(i)
      jo=jorptl(i)
      if(io.eq.0.and.jo.eq.0)then      !string
      am=0.00001
      elseif(io.ne.0.and.jo.ne.0)then  !elastic or droplet or sec-string
      io1=iorptl(i+1)
      jo1=jorptl(i+1)
      io2=iorptl(i-1)
      jo2=jorptl(i-1)
      if(io1.eq.io.and.jo1.eq.jo .or. io2.eq.io.and.jo2.eq.jo)then
      i=io                             !elastic
      goto4
      else                             !droplet or secondary-string
      am=pptl(5,i)
      endif
      else                             !fragmentation/decay
      i=io                            
      goto4
      endif      

      if(am.lt.amin.or.am.gt.amax)goto3

      t=tivptl(1,n)
      k=1+t/tmax*nbns 
      if(k.gt.nbns)goto3
      if(k.lt.1)goto3
      if(id.eq.120.or.id.eq.-120)yy(1,k)=yy(1,k)+1
      if(id.eq.130.or.id.eq.-130)yy(2,k)=yy(2,k)+1
      if(id.eq.1120)             yy(3,k)=yy(3,k)+1

    3 continue
      enddo

           elseif(mode.eq.2)then

      do mm=1,nbns
      xx(mm)=(mm-0.5)*tmax/nbns
      do j=1,nids
      yy(j,mm)=yy(j,mm)/ncttim/(tmax/nbns)
      enddo
      enddo

      write(ifhi,'(a)')    'newpage zone 1 3 1'

      do j=1,nids
      if(j.eq.1)cid='pions  '
      if(j.eq.2)cid='kaons  '
      if(j.eq.3)cid='protons'
      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a)')      'htyp lin xmod lin ymod lin'
      write(ifhi,'(a,e9.3)') 'xrange 0 ',tmax
      write(ifhi,'(a)')      'text 0 0 "xaxis formation time t"'
      write(ifhi,'(a)')      'text 0 0 "yaxis dN/dt"'
      write(ifhi,'(a)')      'text 0.4 0.91 "'//cid//'"'
      write(ifhi,'(a)')      'array 2'
      do mm=1,nbns
      write(ifhi,'(2e11.3)')xx(mm),yy(j,mm)
      enddo
      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto plot 0'
      enddo

           endif

      return
      end

c-----------------------------------------------------------------------
      subroutine xchamt(amt,iii)
c-----------------------------------------------------------------------
c creates unnormalized histogram for net flavor content of grand 
c canonically generated sample
c xpar1: specifies width of plot, netflavor centered
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter(nbmx=200)
      common/camdat/data(nbmx),datb(nbmx)
      common/cnrclu/nrclu
      parameter(mxclu=10000)
      real am(mxclu)
      character cen*6,cvol*6

      if(iii.eq.0)then
       
      am(nrclu)=amt

      return
 
      elseif(iii.lt.0)then

      nbin=nint(xpar3)
      x1=xpar1
      x2=xpar2
      dam=(x2-x1)/nbin
      write(cen,'(f6.1)')tecm
      write(cvol,'(f6.1)')volu

      do i=1,nbin
      data(i)=x1+(i-1)*dam
      datb(i)=0.0
      enddo

      do i=1,nrclu
      xnb=(am(i)-x1)/dam+1.
      nb=nint(xnb)
      if(nb.le.nbin.and.nb.ge.1)datb(nb)=datb(nb)+1
      enddo

      write(ifhi,'(a)')       'newpage zone 1 2 1'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp his'
      write(ifhi,'(a)')       'xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')    'text 0 0 "xaxis total mass"'
      write(ifhi,'(a)')    'text 0 0 "yaxis N"'
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvol//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,nbin
      write(ifhi,'(2e11.3)')data(j),datb(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto plot 0'


      return

           endif

       end

c-----------------------------------------------------------------------
      subroutine xchfla(u,d,s,iii)
c-----------------------------------------------------------------------
c creates unnormalized histogram for net flavor content of grand 
c canonically generated sample
c-----------------------------------------------------------------------
      integer u,d,s

      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter(nb=200)
      common/cfldat/data(nb),datb(nb),datc(nb),datu(nb)
     *,datd(nb),dats(nb)
      common/cnrclu/nrclu
      parameter(mxclu=10000)
      integer ku(mxclu),kd(mxclu),ks(mxclu)
      character cfl*3,cen*6,cvol*6

      if(iii.eq.0)then
       
      ku(nrclu)=u
      kd(nrclu)=d
      ks(nrclu)=s

      return
 
      elseif(iii.lt.0)then

      kwid=nint(xpar1)
      nbin=2*kwid+1
      x1u=keu-kwid
      x2u=keu+kwid
      x1d=ked-kwid
      x2d=ked+kwid
      x1s=kes-kwid
      x2s=kes+kwid
      write(cen,'(f6.1)')tecm
      write(cvol,'(f6.1)')volu

      do i=1,nbin
      data(i)=x1u+(i-1)
      datb(i)=x1d+(i-1)
      datc(i)=x1s+(i-1)
      datu(i)=0.0
      datd(i)=0.0
      dats(i)=0.0
      enddo

      do i=1,nrclu
      nbu=(ku(i)-x1u+1)
      nbd=(kd(i)-x1d+1)
      nbs=(ks(i)-x1s+1)
      if(nbu.le.nbin.and.nbu.ge.1)datu(nbu)=datu(nbu)+1
      if(nbd.le.nbin.and.nbd.ge.1)datd(nbd)=datd(nbd)+1
      if(nbs.le.nbin.and.nbs.ge.1)dats(nbs)=dats(nbs)+1
      enddo

      write(ifhi,'(a)')       'newpage zone 1 3 1'

      write(cfl,'(i3)')keu
      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp his'
      write(ifhi,'(a)')       'xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1u,x2u
      write(ifhi,'(a)')    'text 0 0 "xaxis net u content"'
      write(ifhi,'(a)')    'text 0 0 "yaxis N"'
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvol//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a,a)')     'text 0.65 0.91 "N?u!='//cfl//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,nbin
      write(ifhi,'(2e11.3)')data(j),datu(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto plot 0'

      write(cfl,'(i3)')ked
      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp his'
      write(ifhi,'(a)')       'xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1d,x2d
      write(ifhi,'(a)')    'text 0 0 "xaxis net d content"'
      write(ifhi,'(a)')    'text 0 0 "yaxis N"'
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvol//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a,a)')     'text 0.65 0.91 "N?d!='//cfl//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,nbin
      write(ifhi,'(2e11.3)')datb(j),datd(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto plot 0'

      write(cfl,'(i3)')kes
      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp his'
      write(ifhi,'(a)')       'xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1s,x2s
      write(ifhi,'(a)')    'text 0 0 "xaxis net s content"'
      write(ifhi,'(a)')    'text 0 0 "yaxis N"'
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvol//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a,a)')     'text 0.65 0.91 "N?s!='//cfl//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,nbin
      write(ifhi,'(2e11.3)')datc(j),dats(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto plot 0'

      return

           endif

       end

c-----------------------------------------------------------------------
      subroutine xchico(chi)
c-----------------------------------------------------------------------
c creates unnormalized histogram for chi-squared test of initial
c configuration (grand-canonical results are used)
c for chi>0: chi-squared for each droplet configuration is written
c            to /cchi/
c for chi<0: creates histogram
c            xpar1 specifies lower limit
c            xpar2 specifies upper limit
c            xpar3 specifies bin width
c  newpage, zone and plot commands not included !!!
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter(nbin=200)
      common/chidat/data(nbin),datb(nbin),datc(nbin),datd(nbin)
      common/cnrclu/nrclu
      parameter(mxclu=10000)
      common/cchi/chi2(mxclu)
      character cnu*2,cinco*1,cen*6,cvol*6

         if(chi.ge.0.0)then

      nrclu=nrclu+1
      chi2(nrclu)=chi

      return

         elseif(chi.lt.0.0)then

      x1=nint(xpar1)
      x2=nint(xpar2)
      da=xpar3
      write(cnu,'(i2)')nspecs
      write(cinco,'(i1)')ioinco
      write(cen,'(f6.1)')tecm
      write(cvol,'(f6.1)')volu

      if(x2.eq.0)x2=50.0
      da=max(0.1,da)
      a0=x1

      do i=1,nbin
      data(i)=a0+(i-1)*da
      datb(i)=0.0
      enddo

      do i=1,nrclu
      nb=(chi2(i)+da/2.-a0)/da
      if(nb.le.nbin.and.nb.ge.1)datb(nb)=datb(nb)+1
      enddo

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp his'
      write(ifhi,'(a)')       'xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')    'text 0 0 "xaxis [V]^2"'
      write(ifhi,'(a)')    'text 0 0 "yaxis f([V]^2,n?eff!)"'
      if(iappl.eq.4)write(ifhi,'(a,a)')'text 0.4 0.91 "V='//cvol//'"'
      if(iappl.eq.4)write(ifhi,'(a,a)')'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,nbin
      dat=datb(j)/nevent/da
      write(ifhi,'(2e11.3)')data(j),dat
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'

      return

           endif

       end

c-----------------------------------------------------------------------
      subroutine xdense(ii,itau,ip,x,y,rad,o,u)
c-----------------------------------------------------------------------
c partly copied from sr jintcl.
c ii=1: determining dense regions in space of individual events
c       ip: particle number
c       x,y,rad: tranverse coordinates and radius of particle i
c       o,u: specifies long range: u < s < o (s: long coordinate)
c ii=2: plot of individual event
c       xpar1: event nr
c       xpar2: itau (tau=2**(itau-1)); valid: 1,2,3
c       xpar3: s-range: -xpar3 < s < xpar3 
c       xpar4, x-range: -xpar4 < x < xpar4 
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro5/bag4rt,corlen,dezzer,amuseg
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8

      parameter (mxptl=65000)
      common/cptl/nptl,pptl(5,mxptl),iorptl(mxptl),idptl(mxptl)
     *,istptl(mxptl),tivptl(2,mxptl),ifrptl(2,mxptl),jorptl(mxptl)
     *,xorptl(4,mxptl),ibptl(4,mxptl),iclptl(mxptl)

      double precision tpro,zpro,ttar,ztar,ttaus,detap,detat
      common/cttaus/   tpro,zpro,ttar,ztar,ttaus,detap,detat 
      parameter (mxnev=20,matau=6,nzeta=60,mxcoox=20)
      common/cdensi/kdensi(mxnev,matau,nzeta,mxcoox),tauv(matau)
      common/ctaumx/itaumx    

      if(idensi.ne.1)stop'STOP in xdense: idensi must be set 1'
    
      dlcoox=0.5

           if(ii.eq.1)then

      if(nrevt+1.gt.mxnev)return
      if(itau.lt.1.or.itau.gt.matau)return

      tau=sngl(ttaus)
      zu=u/tau
      zo=o/tau

      if(tauv(itau).eq.0.)then
      itaumx=itau
      tauv(itau)=tau
      do i=1,nzeta
      do j=1,mxcoox
      kdensi(nrevt+1,itau,i,j)=0
      enddo
      enddo
      endif

            do 1 i=1,nzeta
      zi=-nzeta*dlzeta/2-dlzeta/2+i*dlzeta
      if(zu.gt.zi.or.zo.lt.zi)goto1
      do 2 j=1,mxcoox
      xj=-mxcoox*dlcoox/2-dlcoox/2+j*dlcoox
      if((x-xj)**2+y**2.gt.rad**2)goto2
      kdensi(nrevt+1,itau,i,j)=1
    2 continue
    1 continue

           elseif(ii.eq.2)then

      nrevtx=nint(xpar1)
      itaux=nint(xpar2)
      if(itaux.gt.itaumx)stop'STOP in xdense: itaux too large'

      iz=nint(xpar3/dlzeta)
      ix=nint(xpar4/dlcoox)
      if(iz.gt.nzeta/2)stop'STOP in xdense: zeta-range too large'
      if(ix.gt.mxcoox/2)stop'STOP in xdense: x-range too large'

      write(ifhi,'(a,f7.2)')      '! tau: ',tauv(itaux)
      write(ifhi,'(a)')      'openhisto'
      write(ifhi,'(a,2f7.2)')'xrange',-iz*dlzeta,iz*dlzeta
      write(ifhi,'(a,2f7.2)')'yrange',-ix*dlcoox,ix*dlcoox
      write(ifhi,'(a)')      'set ityp2d 3'
      write(ifhi,'(a)') 'text 0 0 "xaxis space-time rapidity [z]"'
      write(ifhi,'(a)') 'text 0 0 "yaxis transverse coordinate x (fm)"'
      write(ifhi,'(a,i4)')   'array2d',2*iz
      do j=mxcoox/2+1-ix,mxcoox/2+ix
      write(ifhi,'(40i2)')    (kdensi(nrevtx,itaux,i,j),
     *                        i=nzeta/2+1-iz,nzeta/2+iz) 
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot2d'

           else
      
      stop'STOP in xdense: wrong option'

           endif

      return
      end

c-----------------------------------------------------------------------
      subroutine xqgcen
c-----------------------------------------------------------------------
c  creates energy spectrum plot for decayed QM-droplet
c  using grand canonical results
c input:
c  xpar1 specifies particle species by paige id, 0 for all
c  xpar2 and xpar3 specify xrange of plot
c  xpar4 specifies line type : dashed (0), dotted (1), full (2) dado (3)
c  xpar5 specifies statistics to be used ,(0) same as iostat
c                                         (1) boltzmann
c output:
c  histo-file 
c  newpage, zone and plot commands not included !!!
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nbin=200)
      real datx(nbin),daty(nbin)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cbol/rmsbol(mspecs),ptlbol(mspecs),chebol(mspecs),tembol
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      character ctem*5,cit*5,cen*6,cvo*6

      idpa=nint(xpar1)
      x1=xpar2
      x2=xpar3
      ltyp=nint(xpar4)
      ist=nint(xpar5)
      if(ist.eq.0.and.iostat.eq.1)ist=1

      id=0
      jx=100
      do i=1,nspecs
      if(ispecs(i).eq.idpa)id=i
      enddo

      dx=(x2-x1)/2./jx
      x0=x1+dx

         do j=1,jx
         datx(j)=x0+(j-1)*dx*2.
	 daty(j)=0.0

       if(id.eq.0)then

      do 10 i=1,nspecs
      dnde=0.0
        if(datx(j).ge.aspecs(i))then
      x=100.
      if(tem.ne.0.0.and.ist.eq.0)x=(datx(j)-chemgc(i))/tem
      if(tem.ne.0.0.and.ist.eq.1)x=(datx(j)-chebol(i))/tembol
      igsp=gspecs(i)
       if(x.ge.60)goto10
       if(mod(igsp,2).eq.0.and.ist.eq.0)then
      dnde=1./(exp(x)+1.)
       elseif(x.le.1.e-7.and.ist.eq.0)then
      dnde=1.e7
       elseif(ist.eq.0)then
      dnde=1./(exp(x)-1.)
       elseif(ist.eq.1)then
      dnde=exp(-x)
       endif
        endif
      daty(j)=daty(j)+dnde*gspecs(i)*volu/hquer**3/8./pi**3
10    continue
    
       else
 
      dnde=0.0
        if(datx(j).ge.aspecs(id))then
      x=100.
      if(tem.ne.0.0.and.ist.eq.0)x=(datx(j)-chemgc(id))/tem
      if(tem.ne.0.0.and.ist.eq.1)x=(datx(j)-chebol(id))/tembol
      igsp=gspecs(id)
       if(x.ge.60)goto11
       if(mod(igsp,2).eq.0.and.ist.eq.0)then
      dnde=1./(exp(x)+1.)
       elseif(x.le.1.e-7.and.ist.eq.0)then
      dnde=1.e7
       elseif(ist.eq.0)then
      dnde=1./(exp(x)-1.)
       elseif(ist.eq.1)then
      dnde=exp(-x)
       endif
        endif
11    daty(j)=dnde*gspecs(id)*volu/hquer**3/8./pi**3

       endif

         enddo

      ctem='     '
      if(tem.gt.0.)write(ctem,'(f5.3)')tem
      write(cen,'(f6.1)')tecm
      write(cvo,'(f6.1)')volu
      write(cit,'(i5)')itermx
      write(ifhi,'(a)')       'openhisto'
      if(ltyp.eq.0)then
      write(ifhi,'(a)')       'htyp lda'
      elseif(ltyp.eq.1)then
      write(ifhi,'(a)')       'htyp ldo'
      elseif(ltyp.eq.2)then
      write(ifhi,'(a)')       'htyp lfu'
      elseif(ltyp.eq.3)then
      write(ifhi,'(a)')       'htyp ldd'
      endif
      write(ifhi,'(a)')       'xmod lin ymod log'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')    'text 0 0 "xaxis E?[n]! (GeV)"'
      write(ifhi,'(a)')    'text 0 0 "yaxis dN?[n]!/d^3!p"'
      write(ifhi,'(a,a)')     'text 0.3 0.10 "T='//ctem//'"'
      write(ifhi,'(a,a)')     'text 0.3 0.20 "i?max!='//cit//'"'
      if(iocite.ne.1)then
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvo//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      endif
      write(ifhi,'(a)')       'array 2'

         do j=1,jx
      write(ifhi,'(2e12.4)')datx(j),daty(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'

      return
      end

c-----------------------------------------------------------------------
      subroutine xqgcmt
c-----------------------------------------------------------------------
c creates transverse mass spectrum for QM-droplet decay
c according to grand canonical results
c input:
c  xpar1 specifies particle species by paige id, 0 for all
c  xpar2 and xpar3 specify xrange of plot
c  xpar4 specifies line type : dashed (0), dotted (1), full (2)
c output:
c  histo-file 
c  newpage, zone and plot commands not included !!!
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nbin=200)
      real datx(nbin),daty(nbin)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cbol/rmsbol(mspecs),ptlbol(mspecs),chebol(mspecs),tembol
      character cen*6,cvo*6,cit*5,ctem*5

      idpa=nint(xpar1)
      x1=xpar2
      x2=xpar3
      ltyp=nint(xpar4)

      id=0
      jx=100
      do i=1,nspecs
      if(ispecs(i).eq.idpa)id=i
      enddo

      dx=(x2-x1)/2./jx
      x0=x1+dx

         do j=1,jx
         datx(j)=x0+(j-1)*dx*2.
         daty(j)=0.0

       if(id.eq.0)then

      do 10 i=1,nspecs
      dndmt=0.0
      if(datx(j).ge.aspecs(i))then
      x=100.
      xx=100.
      if(tem.ne.0.)x=datx(j)/tem
      if(tem.ne.0.)xx=chemgc(i)/tem
      if(abs(xx).le.60)dndmt=gspecs(i)*volu/hquer**3*exp(xx)*datx(j)
     */4./pi**3*qgcbk1(x)
      endif
      daty(j)=daty(j)+dndmt
10    continue

       else

      dndmt=0.0
      if(datx(j).ge.aspecs(id))then
      x=100.
      xx=100.
      if(tem.ne.0.)x=datx(j)/tem
      if(tem.ne.0.)xx=chemgc(id)/tem
      if(abs(xx).le.60)dndmt=gspecs(id)*volu/hquer**3*exp(xx)*datx(j)
     */4./pi**3*qgcbk1(x)
      endif
      daty(j)=dndmt

       endif

         enddo

      write(cit,'(i5)')itermx
      write(cen,'(f6.1)')tecm
      write(cvo,'(f6.1)')volu
      write(ctem,'(f5.3)')tem
      write(ifhi,'(a)')       'openhisto'
      if(ltyp.eq.0)then
      write(ifhi,'(a)')       'htyp lda'
      elseif(ltyp.eq.1)then
      write(ifhi,'(a)')       'htyp ldo'
      elseif(ltyp.eq.2)then
      write(ifhi,'(a)')       'htyp lfu'
      endif
      write(ifhi,'(a)')       'xmod lin ymod log'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')    'text 0 0 "xaxis m?t! (GeV)"'
      write(ifhi,'(a)')    'text 0 0 "yaxis dN?[n]!/d^2!m?t! "'
      write(ifhi,'(a,a)')     'text 0.3 0.10 "T='//ctem//'"'
      write(ifhi,'(a,a)')     'text 0.3 0.20 "i?max!='//cit//'"'
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvo//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,jx
      write(ifhi,'(2e12.4)')datx(j),daty(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'

      return
      end

c-----------------------------------------------------------------------
      subroutine xqgcmu
c-----------------------------------------------------------------------
c creates multiplicity plot for decayed QM-droplet
c according to grand canonical results
c input:
c  xpar1 specifies species by paige id, 0 for total multiplicity
c  xpar2 specifies xrange to be set automatically (0) or by hand (1)
c  xpar3 and xpar4 xrange if xpar2 ne 0
c  xpar5 xrange = average+-sigma*xpar5
c  xpar6 specifies line type : dashed (0), dotted (1), full (2)
c  xpar7 specifies statistics : same as iostat (0)
c                               boltzmann (1)
c output:
c  histo-file 
c  newpage, zone and plot commands not included !!!
c-----------------------------------------------------------------------  
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nbin=200)
      real datx(nbin),daty(nbin)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cbol/rmsbol(mspecs),ptlbol(mspecs),chebol(mspecs),tembol
      common/cgctot/rmstot,ptltot
      character cyield*8,cen*6,cvo*6,cinco*1
      

      idpa=nint(xpar1)
      ixra=nint(xpar2)
      iwid=nint(xpar5)
      ltyp=nint(xpar6)
      ist=nint(xpar7)
      if(ist.eq.0.and.iostat.eq.1)ist=1

 
      id=0
      jx=100
      ymin=1./nevent/10.
      if(nevent.le.10)ymin=ymin/10.
      do i=1,nspecs
      if(ispecs(i).eq.idpa)id=i
      enddo
   
       if(ixra.eq.1)then
      x1=anint(xpar3)
      x2=anint(xpar4)
       else
      if(id.eq.0)then
      x1=anint(ptltot-iwid*rmstot)
      x2=anint(ptltot+iwid*rmstot)
      else 
      x1=anint(ptlngc(id)-iwid*rmsngc(id))
      x2=anint(ptlngc(id)+iwid*rmsngc(id))
      endif
      x2=max(x2,3.0)
       endif

      x1=max(x1,0.0)
      dx=(x2-x1)/2./jx
      x0=x1+dx

      do j=1,jx
      datx(j)=x0+(j-1)*dx*2.
      if(id.eq.0)then

c     total multiplicity
c     ------------------
      x=100.
      if(rmstot.ge.1.e-10)x=(datx(j)-ptltot)**2/rmstot**2/2.
 
       if(x.ge.60)then
      pn=0.0
       else
      pn=exp(-x)/rmstot/sqrt(2.*pi)
       endif

      daty(j)=pn

         else
 
c     one species (specified by id)
c     ------------------------------    
      x=100.
      if(rmsngc(id).ge.1.e-10.and.ist.eq.0)
     *x=(datx(j)-ptlngc(id))**2/rmsngc(id)**2/2.
      if(rmsbol(id).ge.1.e-10.and.ist.eq.1)
     *x=(datx(j)-ptlbol(id))**2/rmsbol(id)**2/2.

       if(x.ge.60)then
      pn=0.0
       else
      if(ist.eq.0)pn=exp(-x)/rmsngc(id)/sqrt(2*pi)
      if(ist.eq.1)pn=exp(-x)/rmsbol(id)/sqrt(2*pi)
       endif

      daty(j)=pn

         endif
         enddo

      if(id.eq.0)then
      write(cyield,'(f8.3)')ptltot
      else
      write(cyield,'(f8.3)')ptlngc(id)
      endif
      write(cinco,'(i1)')ioinco
      write(cen,'(f6.1)')tecm
      write(cvo,'(f6.1)')volu
      write(ifhi,'(a)')       'openhisto'
      if(ltyp.eq.0)then
      write(ifhi,'(a)')       'htyp lda'
      elseif(ltyp.eq.1)then
      write(ifhi,'(a)')       'htyp ldo'
      elseif(ltyp.eq.2)then
      write(ifhi,'(a)')       'htyp lfu'
      elseif(ltyp.eq.3)then
      write(ifhi,'(a)')       'htyp ldd'
      endif
      write(ifhi,'(a)')       'xmod lin ymod log'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a,e11.3,a)')'yrange',ymin,'  auto'
      write(ifhi,'(a)')    'text 0 0 "xaxis N?[n]!"'
      write(ifhi,'(a)')    'text 0 0 "yaxis p(N?[n]!)"'
      write(ifhi,'(a,a)')'text 0.3 0.10 "" "L#N?[n]!"G#='//cyield//'""'
      write(ifhi,'(a,a)')     'text 0.3 0.2 "conf?in!='//cinco//'"'
      if(iocite.ne.1)then
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvo//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      endif
      write(ifhi,'(a)')       'array 2'

         do j=1,jx
      write(ifhi,'(2e12.4)')datx(j),daty(j)
         enddo 

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'


      return 
      end


c-----------------------------------------------------------------------
      subroutine xqgcmx
c-----------------------------------------------------------------------
c creates multiplicity plot for decayed QM-droplet
c according to grand canonical results POISSON DISTRIB.!!!!
c input:
c  xpar1 specifies species by paige id, 0 for total multiplicity
c  xpar2 specifies xrange to be set automatically (0) or by hand (1)
c  xpar3 and xpar4 xrange if xpar2 ne 0
c  xpar5 xrange = average+-sigma*xpar5
c  xpar6 specifies line type : dashed (0), dotted (1), full (2) dado (3)
c  xpar7 specifies statistics : same as iostat (0)
c                               boltzmann (1)
c output:
c  histo-file 
c  newpage, zone and plot commands not included !!!
c-----------------------------------------------------------------------  
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nbin=200)
      real datx(nbin),daty(nbin)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cbol/rmsbol(mspecs),ptlbol(mspecs),chebol(mspecs),tembol
      common/cgctot/rmstot,ptltot
      character cyield*8,cen*6,cvo*6,cinco*1
      

      idpa=nint(xpar1)
      ixra=nint(xpar2)
      iwid=nint(xpar5)
      ltyp=nint(xpar6)
      ist=nint(xpar7)
      if(ist.eq.0.and.iostat.eq.1)ist=1

 
      id=0
      ymin=1./nevent/10.
      if(nevent.le.10)ymin=ymin/10.
      do i=1,nspecs
      if(ispecs(i).eq.idpa)id=i
      enddo
   
       if(ixra.eq.1)then
      n1=nint(xpar3)
      n2=nint(xpar4)
       else
      if(id.eq.0)then
      n1=nint(ptltot-iwid*rmstot)
      n2=nint(ptltot+iwid*rmstot)
      else 
      n1=nint(ptlngc(id)-iwid*rmsngc(id))
      n2=nint(ptlngc(id)+iwid*rmsngc(id))
      endif
      n2=max(n2,3)
       endif

      n1=max(n1,0)
      jx=n2+1

      do j=1,jx
      datx(j)=j-1
      jf=1
      if(j.gt.1)then
      do i=1,j-1
      jf=jf*i
      enddo
      endif
      if(id.eq.0)then

c     total multiplicity
c     ------------------

      daty(j)=1./jf*ptltot**(j-1)*exp(-ptltot)

         else
 
c     one species (specified by id)
c     ------------------------------    

      if(ist.eq.0)pn=1./jf*ptlngc(id)**(j-1)*exp(-ptlngc(id))
      if(ist.eq.1)pn=1./jf*ptlbol(id)**(j-1)*exp(-ptlbol(id))

      daty(j)=pn

         endif
         enddo

      if(id.eq.0)then
      write(cyield,'(f8.3)')ptltot
      else
      write(cyield,'(f8.3)')ptlngc(id)
      endif
      write(cinco,'(i1)')ioinco
      write(cen,'(f6.1)')tecm
      write(cvo,'(f6.1)')volu
      write(ifhi,'(a)')       'openhisto'
      if(ltyp.eq.0)then
      write(ifhi,'(a)')       'htyp lda'
      elseif(ltyp.eq.1)then
      write(ifhi,'(a)')       'htyp ldo'
      elseif(ltyp.eq.2)then
      write(ifhi,'(a)')       'htyp lfu'
      elseif(ltyp.eq.3)then
      write(ifhi,'(a)')       'htyp ldd'
      endif
      write(ifhi,'(a)')       'xmod lin ymod log'
      write(ifhi,'(a,2i3)')'xrange',n1,n2
      write(ifhi,'(a,e11.3,a)')'yrange',ymin,'  auto'
      write(ifhi,'(a)')    'text 0 0 "xaxis N?[n]!"'
      write(ifhi,'(a)')    'text 0 0 "yaxis p(N?[n]!)"'
      write(ifhi,'(a,a)')'text 0.3 0.10 "" "L#N?[n]!"G#='//cyield//'""'
      write(ifhi,'(a,a)')     'text 0.3 0.2 "conf?in!='//cinco//'"'
      if(iocite.ne.1)then
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvo//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      endif
      write(ifhi,'(a)')       'array 2'

         do j=1,jx
      write(ifhi,'(2e12.4)')datx(j),daty(j)
         enddo 

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'


      return 
      end

c-----------------------------------------------------------------------
      subroutine xqgcpt
c-----------------------------------------------------------------------
c creates transverse momentum spectrum for decayed QM-droplet
c according to grand canonical results
c input:
c  xpar1 specifies particle species by paige id, 0 for all
c  xpar2 rapidity window
c  xpar3 and xpar4 specify xrange of plot
c  xpar5 specifies line type : dashed (0), dotted (1), full (2)
c output:
c  histo-file 
c  newpage, zone and plot commands not included !!!
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nbin=200)
      real datx(nbin),daty(nbin)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cbol/rmsbol(mspecs),ptlbol(mspecs),chebol(mspecs),tembol
      character crap*5,cen*6,cvo*6,cit*5

      idpa=nint(xpar1)
      y=xpar2
      x1=xpar3
      x2=xpar4
      ltyp=xpar5

      write(crap,'(f5.1)')y
      id=0
      jx=100
      do i=1,nspecs
      if(ispecs(i).eq.idpa)id=i
      enddo

      dx=(x2-x1)/2./jx
      x0=x1+dx

         do j=1,jx
         datx(j)=x0+(j-1)*dx*2.
         daty(j)=0.0

       if(id.eq.0)then

      do 10 i=1,nspecs
      x=100.
      if(tem.ne.0.)
     *x=(sqrt(aspecs(i)**2+datx(j)**2)*cosh(y)-chemgc(i))/tem
       if(x.ge.60)then
      dndpt=0.0
       else
      dndpt=exp(-x)
       endif
      dndpt=dndpt*gspecs(i)*volu/hquer**3*cosh(y)
     **sqrt(aspecs(i)**2+datx(j)**2)/8./pi**3
      daty(j)=daty(j)+dndpt
10    continue

       else

      x=100.
      if(tem.ne.0.)
     *x=(sqrt(aspecs(id)**2+datx(j)**2)*cosh(y)-chemgc(id))/tem
       if(x.ge.60)then
      dndpt=0.0
       else
      dndpt=exp(-x)
       endif
      dndpt=dndpt*gspecs(id)*volu/hquer**3*cosh(y)
     **sqrt(aspecs(id)**2+datx(j)**2)/8./pi**3
      daty(j)=dndpt

       endif

         enddo

      write(cit,'(i5)')itermx
      write(cen,'(f6.1)')tecm
      write(cvo,'(f6.1)')volu
      write(ifhi,'(a)')       'openhisto'
      if(ltyp.eq.0)then
      write(ifhi,'(a)')       'htyp lda'
      elseif(ltyp.eq.1)then
      write(ifhi,'(a)')       'htyp ldo'
      elseif(ltyp.eq.2)then
      write(ifhi,'(a)')       'htyp lfu'
      endif
      write(ifhi,'(a)')       'xmod lin ymod log'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')    'text 0 0 "xaxis p?t! (GeV/c)"'
      write(ifhi,'(a)')    'text 0 0 "yaxis dN?[n]!/dyd^2!p?t!"'
      write(ifhi,'(a)')    'text 0.10 0.10 "y = '//crap//'"'
      write(ifhi,'(a)')    'text 0.10 0.30 "i?max! = '//cit//'"'
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvo//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,jx
      write(ifhi,'(2e12.4)')datx(j),daty(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'

      return
      end

c-----------------------------------------------------------------------
      subroutine xqgcra
c-----------------------------------------------------------------------
c creates rapidity distribution for decayed QM-droplet
c according to grand canonical results
c input:
c  xpar1 specifies particle species by paige id, 0 for all
c  xpar2 and xpar3 specify xrange of plot
c  xpar4 specifies line type : dashed (0), dotted (1), full (2)
c output:
c  histo-file 
c  newpage, zone and plot commands not included !!!
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter (nbin=200)
      real datx(nbin),daty(nbin)
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cgctot/rmstot,ptltot
      common/cbol/rmsbol(mspecs),ptlbol(mspecs),chebol(mspecs),tembol
      character cen*6,cvo*6,cng*8

      idpa=nint(xpar1)
      x1=nint(xpar2)
      x2=nint(xpar3)
      ltyp=nint(xpar4)

      id=0
      jx=100
      ymin=1./nevent/10.
      if(nevent.le.10)ymin=ymin/10.
      do i=1,nspecs
      if(ispecs(i).eq.idpa)id=i
      enddo

      dx=(x2-x1)/2./jx
      x0=x1+dx

         do j=1,jx

         datx(j)=x0+(j-1)*dx*2.
	 daty(j)=0.0
         y=datx(j)
         if(ish.ge.98)write(ifch,*)'cosh y:',cosh(y)

       if(id.eq.0)then

      do 10 i=1,nspecs
      dndy=0.0
      sum=aspecs(i)**2*tem+2.*aspecs(i)*tem**2/cosh(y)
     *+2.*tem**3/cosh(y)**2
      x=100.
      if(tem.ne.0.0)
     *x=(aspecs(i)*cosh(y)-chemgc(i))/tem

       if(x.ge.60.)then
      pro=0.0
       else
      pro=exp(-x)
      endif

      pro=pro*gspecs(i)*volu/hquer**3/4./pi**2

      if(pro.ge.(1.e-30).and.sum.ge.(1.e-30))then
      che=alog(pro)+alog(sum)
      else
      che=-61.0
      endif
      if(che.le.60.0.and.che.ge.(-60.0))dndy=pro*sum
c     if(che.le.60.0.and.che.ge.(-60.0))dndy=exp(che)

      daty(j)=daty(j)+dndy

10    continue       

       else

      dndy=0.0
      sum=aspecs(id)**2*tem+2.*aspecs(id)*tem**2/cosh(y)
     *+2.*tem**3/cosh(y)**2
      x=100.
      if(tem.ne.0.0)
     *x=(aspecs(id)*cosh(y)-chemgc(id))/tem

       if(x.ge.60.)then
      pro=0.0
       else
      pro=exp(-x)
      endif

      pro=pro*gspecs(id)*volu/hquer**3/4./pi**2

      if(pro.ge.(1.e-30).and.sum.ge.(1.e-30))then
      che=alog(pro)+alog(sum)
      else
      che=-61.0
      endif
      if(che.le.60..and.che.ge.-60.)dndy=pro*sum

      daty(j)=dndy

       endif

	 enddo
      
      write(cen,'(f6.1)')tecm
      write(cvo,'(f6.1)')volu
      if(id.eq.0)then
      write(cng,'(f8.3)')ptltot
      else
      write(cng,'(f8.3)')ptlngc(id)
      endif
      write(ifhi,'(a)')       'openhisto'
      if(ltyp.eq.0)then
      write(ifhi,'(a)')       'htyp lda'
      elseif(ltyp.eq.1)then
      write(ifhi,'(a)')       'htyp ldo'
      elseif(ltyp.eq.2)then
      write(ifhi,'(a)')       'htyp lfu'
      endif

      write(ifhi,'(a)')       'xmod lin ymod log'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a,e11.3,a)')'yrange',ymin,'  auto'
      write(ifhi,'(a)')    'text 0 0 "xaxis y"'
      write(ifhi,'(a)')    'text 0 0 "yaxis dN?[n]!/dy"'
      write(ifhi,'(a,a)')     'text 0.4 0.91 "V='//cvo//'"'
      write(ifhi,'(a,a)')     'text 0.15 0.91 "E='//cen//'"'
      write(ifhi,'(a,a)')     'text 0.3 0.10 "N?[n]!='//cng//'"'
      write(ifhi,'(a)')       'array 2'

         do j=1,jx
      write(ifhi,'(2e12.4)')datx(j),daty(j)
         enddo

      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'

      return
      end

c-----------------------------------------------------------------------
      subroutine xqnben
c-----------------------------------------------------------------------
c produces histogram of energy spectrum (after metropolis run)
c complete histogram: openhisto ... closehisto
c iocite=1 required
c-----------------------------------------------------------------------
c xpar1: particle species (venus id-code)
c xpar2: 1: actual spectrum 2: fit
c xpar3: 1: de/d3p 2: ede/d3e
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (nhise=100)
      common/chise/hise(mspecs,nhise)
      parameter (literm=500)
      common/cmet/kspecs(mspecs),liter,lspecs(literm,mspecs)
     *,iterl(literm),iterc(literm)
      real datx(nhise),daty(nhise),dats(nhise)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      character ch*1,chid*5,cyield*9,ctem*5
      parameter (mxtext=10)
      character*50 ctext(mxtext)
      common/chisto/ntext,xtext(mxtext),ytext(mxtext),ctext
      common/ckhist/khisto
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      de=2./nhise/2.

      if(iocite.ne.1)stop'STOP: xqnben: iocite=1 required'

      idcode=nint(xpar1)
      mode=nint(xpar2)
      kind=nint(xpar3)

           do j=1,nspecs
           if(idcode.eq.ispecs(j))then

      id=idcode
      am=aspecs(j)
      yield=1.*kspecs(j)/(itermx-iternc)
      if(kind.eq.1)ch=' '
      if(kind.eq.2)ch='e'
      ll=kind-1
      e0=am+de
      nebins=0
        do i=1,nhise
      e=e0+(i-1)*2*de
      p1=sqrt((e-de)**2-am**2)
      p2=sqrt((e+de)**2-am**2)
      d3p=4*pi*(p2**3-p1**3)/3
      datx(i)=e
      y=(1-ll+ll*e)*hise(j,i)/(itermx-iternc)/d3p
      if(y.gt.0.)then
      nebins=nebins+1
      daty(i)=alog(y)
      d=y/sqrt(hise(j,i))
      dats(i)=1e10
      if(y-d.gt.0.)dats(i)=alog(y+d)-alog(y-d)
      else
      daty(i)=-100
      dats(i)=1e10
      endif
c-c   if(e.lt.0.2)dats(i)=1e10
        enddo
      a=0.
      b=0.
        if(nebins.ge.3)then
      call utfit(datx,daty,nhise,dats,1,a,b,siga,sigb,chi2,q)
      tem=-1./b
      if(tem.lt.0.050.or.tem.gt.10.)then
      tem=0.
      a=0.
      b=0.
      endif
        endif
      do i=1,nhise
      daty(i)=exp(daty(i))
      enddo
      write(chid,'(i5)')id
      write(cyield,'(f9.4)')yield
      ctem='     '
      if(tem.gt.0.)write(ctem,'(f5.3)')tem
      write(ifhi,'(a)')    'openhisto xrange 0 3'
      write(ifhi,'(a)')    'htyp lin xmod lin ymod log'
      write(ifhi,'(a,a)')  'text 0 0 "title id='//chid
     *                           ,'   N='//cyield//'   T='//ctem//'"'
      write(ifhi,'(a)')    'text 0 0 "xaxis energy (GeV)"'
      write(ifhi,'(a)')    'text 0 0 "yaxis '//ch//' dn/d3p (GeV-3)"' 
      write(ifhi,'(a)')    'array 2'
      do i=1,nhise
      if(mode.eq.1)write(ifhi,'(2e12.4)')datx(i),daty(i)
      if(mode.eq.2)write(ifhi,'(2e12.4)')datx(i),exp(a+b*datx(i))
      enddo
      write(ifhi,'(a)')    '  endarray'
      write(ifhi,'(a)')    'closehisto'

           endif
           enddo

      return
      end

c-----------------------------------------------------------------------
      subroutine xqnbit
c-----------------------------------------------------------------------
c produces histogram of multiplicity versus iterations (after metropolis run)
c complete histogram: openhisto ... closehisto
c iocite=1 required
c-----------------------------------------------------------------------
c xpar1: particle species (0=all, else venus id-code)
c xpar2: 1:actual multiplicity 2:average multiplicity 3:grand canonical
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/ctst/psulog,wtulog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (nhismu=500)
      parameter (nflav=6)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/chismu/hismu(mspecs,0:nhismu),hismus(nhismu)
      parameter (literm=500)
      common/cmet/kspecs(mspecs),liter,lspecs(literm,mspecs)
     *,iterl(literm),iterc(literm)
      real datlx(literm),datly(literm)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common /clatt/nlattc,npmax
      character chid*5,ctecm*5,cvolu*6
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter (mxtext=10)
      character*50 ctext(mxtext)
      common/chisto/ntext,xtext(mxtext),ytext(mxtext),ctext
      common/ckhist/khisto
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      common/cgctot/rmstot,ptltot

      if(iocite.ne.1)stop'STOP: xqnbit: iocite=1 required'

      idcode=nint(xpar1)
      mode=nint(xpar2)

           if(idcode.eq.0)then

      yield=0
      do j=1,nspecs
      yield=yield+1.*kspecs(j)/(itermx-iternc)
      enddo
      datlx(1)=(iterl(1)+1)/2.
      do li=2,liter-1
      datlx(li)=(iterl(li)+iterl(li-1)+1)/2.
      enddo
      x1=0
      x2=iterl(liter-1)
      do li=1,liter-1
      y=0
      do j=1,nspecs
      y=y+lspecs(li,j)
      enddo
      if(mode.eq.1)datly(li)=y/iterc(li)
      if(mode.eq.2)datly(li)=yield
      if(mode.eq.3)datly(li)=ptltot
      enddo
      write(ctecm,'(f5.1)')tecm
      write(cvolu,'(f6.1)')volu
      write(ifhi,'(a,2e11.3)')'openhisto xrange',x1,x2
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a,a)')     'text 0 0 "title E = '//ctecm//'   V = '
     *                                 ,cvolu//'"'
      write(ifhi,'(a)')       'text 0 0 "xaxis iterations"'
      write(ifhi,'(a)')       'text 0 0 "yaxis multiplicity"'
      write(ifhi,'(a)')       'array 2'
      do i=1,liter-1
      write(ifhi,'(2e12.4)')   datlx(i),datly(i)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

           else

           do j=1,nspecs
           if(idcode.eq.ispecs(j))then

      yield=1.*kspecs(j)/(itermx-iternc)
      write(chid,'(i5)')id
      do li=1,liter-1
      datlx(li)=iterl(li)
      enddo
      x1=0
      x2=datlx(liter-1)
      do li=1,liter-1
      if(mode.eq.1)datly(li)=lspecs(li,j)*1./iterc(li)
      if(mode.eq.2)datly(li)=yield
      if(mode.eq.3)datly(li)=ptlngc(j)
      enddo
      write(ifhi,'(a,2e11.3)')'openhisto xrange',x1,x2
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "title id='//chid
      write(ifhi,'(a)')       'text 0 0 "xaxis iterations "'
      write(ifhi,'(a)')       'text 0 0 "yaxis multiplicity"'
      write(ifhi,'(a)')       'array 2'
      do i=1,liter-1
      write(ifhi,'(2e12.4)')   datlx(i),datly(i)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

           endif
           enddo

           endif

      return
      end

c-----------------------------------------------------------------------
      subroutine xqnbmu
c-----------------------------------------------------------------------
c produces histogram of multiplicity distribution (after metropolis run)
c complete histogram: openhisto ... closehisto
c iocite=1 required
c-----------------------------------------------------------------------
c xpar1: particle species (0=all, else venus id-code)
c xpar2: xrange automatic (0) or given via xpar3,4 (else)
c xpar3,4: xrange
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/ctst/psulog,wtulog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (nhismu=500)
      parameter (nflav=6)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/chismu/hismu(mspecs,0:nhismu),hismus(nhismu)
      parameter (literm=500)
      common/cmet/kspecs(mspecs),liter,lspecs(literm,mspecs)
     *,iterl(literm),iterc(literm)
      real datx(nhismu),daty(nhismu)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common /clatt/nlattc,npmax
      character chid*5,cyield*9,ctecm*5,cvolu*6
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter (mxtext=10)
      character*50 ctext(mxtext)
      common/chisto/ntext,xtext(mxtext),ytext(mxtext),ctext
      common/ckhist/khisto
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8

      if(iocite.ne.1)stop'STOP: xqnbmu: iocite=1 required'

      idcode=nint(xpar1)
      ixr=nint(xpar2)
      xx1=xpar3
      xx2=xpar4

      write(ctecm,'(f5.1)')tecm
      write(cvolu,'(f6.1)')volu

           if(idcode.eq.0)then

      yield=0
      do j=1,nspecs
      yield=yield+1.*kspecs(j)/(itermx-iternc)
      enddo
      write(cyield,'(f9.4)')yield
      i1=0
      i2=nlattc
      mus=0
      do i=1,nhismu
      if(i1.eq.0.and.nint(hismus(i)).gt.0)i1=i
      if(nint(hismus(i)).gt.0)i2=i
      mus=mus+hismus(i)
      enddo
      ij=0.5*(i1+i2)*0.20
      if(itermx.le.1000)ij=0.5*(i1+i2)*0.40
      if(itermx.le.100)ij=0.5*(i1+i2)*0.80
      i1=i1-ij
      i1=max(i1,2)
      i2=i2+ij
      ii=10
      if(i1.le.50)ii=5
      if(i1.le.20)ii=2
      i1=i1/ii*ii
      i2=i2/ii*ii+ii
           do i=i1,i2
      l=1+i-i1
      datx(l)=i
      daty(l)=hismus(i)/mus
           enddo
      jx=1+i2-i1
      if(ixr.eq.0)then
      x1=i1
      x2=i2
      else
      x1=xx1
      x2=xx2
      endif
      write(ifhi,'(a,2e11.3)')'openhisto xrange',x1,x2
      write(ifhi,'(a)')       'htyp lin xmod lin ymod log'
      write(ifhi,'(a,a)')     'text 0 0 "title E = '//ctecm//'   V = '
     *                              ,cvolu//'"'
      write(ifhi,'(a)')       'text 0 0 "xaxis multiplicity n  "'
      write(ifhi,'(a)')       'text 0 0 "yaxis dN/dn"'
      write(ifhi,'(a)')       'text 0.30 0.25 "N?MC!='//cyield//'"'   
      write(ifhi,'(a)')       'array 2'
      do i=1,jx
      write(ifhi,'(2e12.4)')   datx(i),daty(i)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

           else

           do j=1,nspecs
           if(idcode.eq.ispecs(j))then

      yield=1.*kspecs(j)/(itermx-iternc)
      write(cyield,'(f9.4)')yield
      write(chid,'(i5)')idcode
      i1=0
      i2=nlattc
      mus=0
      do i=0,nhismu
      if(i1.eq.0.and.nint(hismu(j,i)).gt.0)i1=i
      if(nint(hismu(j,i)).gt.0)i2=i
      mus=mus+hismu(j,i)
      enddo
      ij=0.5*(i1+i2)*0.30
      if(itermx.le.1000)ij=0.5*(i1+i2)*0.60
      if(itermx.le.100)ij=0.5*(i1+i2)*1.20
      i1=i1-ij
      i1=max(i1,0)
      i2=i2+ij
      ii=10
      if(i1.le.50)ii=5
      if(i1.le.20)ii=2
      i1=i1/ii*ii
      i2=i2/ii*ii+ii
           do i=i1,i2
      l=1+i-i1
      datx(l)=i
      daty(l)=hismu(j,i)/mus
           enddo
      jx=1+i2-i1
      if(ixr.eq.0)then
      x1=i1
      x2=i2
      else
      x1=xx1
      x2=xx2
      endif
      write(ifhi,'(a,2e11.3)')'openhisto xrange',x1,x2
      write(ifhi,'(a)')       'htyp lin xmod lin ymod log'
      write(ifhi,'(a)')       'text 0 0 "title id='//chid//'"'
      write(ifhi,'(a)')       'text 0 0 "xaxis multiplicity n  "'
      write(ifhi,'(a)')       'text 0 0 "yaxis dN/dn"'
      write(ifhi,'(a)')       'text 0.30 0.25 "N?MC!='//cyield//'"'   
      write(ifhi,'(a)')       'array 2'
      do i=1,jx
      write(ifhi,'(2e12.4)')   datx(i),daty(i)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

           endif
           enddo

           endif

      return
      end

c-----------------------------------------------------------------------
      subroutine xqnbmz
c-----------------------------------------------------------------------
c produces histogram of multiplicity distribution from droplet decay 
c or average multiplicity versus iterations
c for massless hadrons
c complete histogram: openhisto ... closehisto
c-----------------------------------------------------------------------
c xpar1: particle species (0=all, else venus id-code)
c xpar2: lower limit multiplicity
c xpar3: upper limit multiplicity
c xpar4: lower limit total multiplicity   (also necc for xpar1.ne.0)
c xpar5: upper limit  "      "            (also necc for xpar1.ne.0)
c xpar6: sets htyp: 1->lfu, 2->ldo, 3->lda, 4->ldd 
c xpar7: 0: multiplicity distribution
c        >0: av multiplicity vs iterations (itermx=xpar7)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt 
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/ctst/psulog,wtulog
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      parameter (nhismu=500)
      parameter (nflav=6)
      common/cflac/ifok(nflav,mspecs),ifoa(nflav)
      common/chismu/hismu(mspecs,0:nhismu),hismus(nhismu)
      parameter (literm=500)
      common/cmet/kspecs(mspecs),liter,lspecs(literm,mspecs)
     *,iterl(literm),iterc(literm)
      real datx(nhismu),datyu(nhismu)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      common /clatt/nlattc,npmax
      character cyieur*9
      real pzlog(nhismu)
      common/cinfla/keu,ked,kes,kec,keb,ket
      parameter (mxtext=10)
      character*50 ctext(mxtext)
      common/chisto/ntext,xtext(mxtext),ytext(mxtext),ctext
      double precision spelog,cc,bb,dsu
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8
      common/cyield/yield
      character*3 htyp

      idcode=nint(xpar1)
      x1=xpar2
      x2=xpar3
      i1=nint(xpar2)
      i2=nint(xpar3)
      ii1=nint(xpar4)
      ii2=nint(xpar5)
      ih=nint(xpar6)
      htyp='lin'
      if(ih.eq.1)htyp='lfu'
      if(ih.eq.2)htyp='ldo'
      if(ih.eq.3)htyp='lda'
      if(ih.eq.4)htyp='ldd'
      itmax=nint(xpar7)

      wtrlog=-1e30
           do i=ii1,ii2
      if(i.ge.2)then
      np=i
      do k=1,np
      ident(k)=110
      enddo
      call qnbtst(0)
      wtzlog=wtulog
      if(ioflac.eq.0)call qnbspg(keu,ked,kes,kec,keb,ket,0,np,spelog)
      if(ioflac.ne.0)call qnbspf(keu,ked,kes,kec,keb,ket,0,np,spelog)       
      wtulog=wtulog+spelog
      else
      wtzlog=-1000
      wtulog=-1000
      endif
      pzlog(1+i-ii1)=wtzlog
      datyu(1+i-ii1)=wtulog
      wtrlog=max(wtrlog,wtulog)
           enddo
      yield=0
      su=0
           do i=ii1,ii2
      l=1+i-ii1
      pzlog(l)=pzlog(l)-wtrlog
      datyu(l)=datyu(l)-wtrlog
      if(datyu(l).gt.-50.)then
      datyu(l)=exp(datyu(l))
      else
      datyu(l)=exp(-50.)
      endif  
      yield=yield+i*datyu(l)
      su=su+datyu(l)
           enddo
      yield=yield/su
           do i=ii1,ii2
      l=1+i-ii1
      datx(l)=i
      datyu(l)=datyu(l)/su
           enddo
      jx=1+ii2-ii1
      write(cyieur,'(f9.4)')yield
c     ---
        if(idcode.eq.0.and.itmax.eq.0)then
      write(ifhi,'(a,2e11.3)')'openhisto xrange',x1,x2
      write(ifhi,'(a)')       'htyp '//htyp//' xmod lin ymod log'
      write(ifhi,'(a)')       'text 0.30 0.15 "N?ana!='//cyieur//'"'   
      write(ifhi,'(a)')       'array 2'
      do i=1,jx
      write(ifhi,'(2e12.4)')   datx(i),datyu(i)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'
        elseif(idcode.eq.0)then
      write(ifhi,'(a,2e11.3)')'openhisto xrange',0.,itmax*1.
      write(ifhi,'(a)')       'htyp '//htyp//' xmod lin ymod lin'
      write(ifhi,'(a)')       'array 2'
      itm=20
      do i=1,itm
      write(ifhi,'(2e12.4)')   (i-1.)*itmax/(itm-1.),yield
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'
        endif
c     ---
      if(idcode.eq.0)return

           do j=1,nspecs
           if(idcode.eq.ispecs(j))then

      wtrlog=-1e30
           do i=i1,i2
      l=1+i-i1
      datx(l)=i
           enddo
      yield=0
      suj=0
      dsu=su
           do i=i1,i2
      l=1+i-i1
      bb=0
      nfi=0
      do ntot=max(i+1,ii1),min(i2*nspecs,ii2)
      nfi=nfi+1
      cc=1d0
      do kc=1,i
      cc=cc*(1.+ntot-kc)/kc*gspecs(j)
      enddo
      ku=keu-i*ifok(1,j)
      kd=ked-i*ifok(2,j)
      ks=kes-i*ifok(3,j)
      kc=kec-i*ifok(4,j)
      kb=keb-i*ifok(5,j)
      kt=ket-i*ifok(6,j)
      if(ioflac.eq.0)call qnbspg(ku,kd,ks,kc,kb,kt,j,ntot-i,spelog)
      if(ioflac.ne.0)call qnbspf(ku,kd,ks,kc,kb,kt,j,ntot-i,spelog)
      cc=cc*dexp(spelog)
      bb=bb+cc*dexp(1.d0*pzlog(1+ntot-ii1))/dsu
      enddo
      datyu(l)=bb
      yield=yield+i*datyu(l)
      suj=suj+datyu(l)
           enddo
      yield=yield/suj
      jx=1+i2-i1
      write(cyieur,'(f9.4)')yield
c     ---
        if(itmax.eq.0)then
      write(ifhi,'(a,2e11.3)')'openhisto xrange',x1,x2
      write(ifhi,'(a)')       'htyp '//htyp//' xmod lin ymod log'
      write(ifhi,'(a)')       'text 0.30 0.15 "N?ana!='//cyieur//'"'   
      write(ifhi,'(a)')       'array 2'
      do i=1,jx
      write(ifhi,'(2e12.4)')   datx(i),datyu(i)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'
        else
      write(ifhi,'(a,2e11.3)')'openhisto xrange',0.,itmax*1.
      write(ifhi,'(a)')       'htyp '//htyp//' xmod lin ymod lin'
      write(ifhi,'(a)')       'array 2'
      itm=20
      do i=1,itm
      write(ifhi,'(2e12.4)')   (i-1.)*itmax/(itm-1.),yield
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'
        endif
c     ---
      return

           endif
           enddo

      end

c-----------------------------------------------------------------------
      subroutine xtauex(iii)
c-----------------------------------------------------------------------
c fills histograms (iii>=0) or writes histogram to histo-file (iii<0)
c regarding exponential autocorrelation time and acceptance rate
c
c input:
c   requires complete run with application hadron (iappl=1) 
c   or application metropolis (iappl=4)
c   ioceau=1 necessary
c
c  output:
c   for iii=0 (only valid for iappl=4):
c     data(nrevt): nrevt  (event number)               /cdat/
c     datb(nrevt): taui   (calculated corr time)       /cdat/
c     datc(nrevt): accrat (acceptance rate)            /cdat/      
c     datd(nrevt): taue   (parametrized corr time)     /cdat/
c   for iii>0 (only valid for iappl=1):
c     nrclu=nrclu+1                                    /cnrclu/
c     data(nrclu): nrclu  (droplet number)             /cdat/
c     datb(nrclu): taui-taue (calc - param corr time)  /cdat/
c     datc(nrclu): accrat (acceptance rate)            /cdat/   
c     datd(nrclu): avnp (average particle number)      /cdat/
c   for iii<0:
c     writes complete histogram (openhisto ... closehisto) to histofile
c       for iappl=4:                for iappl=1:
c         xpar1=1: (data,datb,datd) xpar1=1: (data,datb)
c         xpar1=2: (data,datc)      xpar1=2: (data,datd)
c                                   xpar1=3: (data,datc)
c-----------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxit=50000)
      common/count/nacc,nrej,naccit(maxit),nptot,npit(maxit)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common /clatt/nlattc,npmax
      common/cgctot/rmstot,ptltot
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter (nbin=500)
      common/cdat/ data(nbin),datb(nbin),datc(nbin),datd(nbin)
      real dev(maxit)
      character cobs*5,cnc*5,cdz*5,czer*5
     *,cmom*5,cnp*7,cen*7,cvol*7,clatt*5,cit*5
      common/ctaue/taue
      common/cnrclu/nrclu
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8

      if(ioceau.ne.1)stop'STOP: ioceau=1 required'
      if(iii.eq.0.and.iappl.ne.4)stop'STOP: iappl=4 required'
      if(iii.gt.0.and.iappl.ne.1)stop'STOP: iappl=1 required'

      if(iii.lt.0)jjj=nint(xpar1)

      ish0=ish
c     ish=98

c          ----------------
           if(iii.ge.0)then
c          ----------------

      if(iii.gt.0)nrclu=nrclu+1
      if(nrclu.gt.500)return

c     mean
c     ----
      xnptot=nptot
      avnp=xnptot/(itermx-iternc)
      if(ish.ge.98)write(ifch,*)'event:',nrevt,'   droplet:',nrclu
     *,'   avnp:',avnp

c     calculate corfct_0
c     ------------------
      corzer=0.0
      do i=iternc+1,itermx
      dev(i)=npit(i)-avnp
      corzer=corzer+dev(i)**2
      enddo
      corzer=corzer/(itermx-iternc)
      if(ish.ge.98)write(ifch,*)'c_0:',corzer

c     calculate corfct_1
c     ------------------
      corone=0.0
      do i=iternc+1,itermx-1
      corone=corone+dev(i)*dev(i+1)
      enddo
      corone=corone/(itermx-iternc-1)

c     calculate initial autocorrelation time
c     -----------------------------------------
      if(corone.gt.1.e-30.and.corzer.gt.1.e-30)then
      r=alog(corone)-alog(corzer)
      if(ish.ge.98)write(ifch,*)'log rho_1:',r
      taui=(-1.)/r
      else
      taui=0.
      endif
      if(ish.ge.98)write(ifch,*)'tau_init:',taui

c     calculate parametrized autocorrelation time (if necessary)
c     ----------------------------------------------------------
      if(taue.eq.0.0)then
      e=tecm/volu
      b=1.1*(e+0.33)**0.66
      a=13.*(e+0.13)**(-0.65)
      tm=34.*(e+0.65)**(-0.61)
      t=a+b*volu
      taue=max(t,tm)
      endif

c     calculate acceptance rate
c     -------------------------
      xa=nacc
      ya=itermx
      accrat=xa/ya

c     write to data/b/c/d
c     -------------------
       if(iii.eq.0)then
      if(iozevt.gt.0)then
      data(nrevt)=iozero
      else
      data(nrevt)=nrevt
      endif
      datb(nrevt)=taui
      datc(nrevt)=accrat
      datd(nrevt)=taue
       else
      data(nrclu)=nrclu
      datb(nrclu)=taui-taue
      datc(nrclu)=accrat
      datd(nrclu)=avnp
       endif
          
c          -----------------------------------  
           elseif(iii.lt.0.and.iappl.eq.4)then
c          -----------------------------------

      write(cmom,'(i3)')iomom
      write(cen,'(f7.3)')tecm
       if(ioobsv.eq.0)then
      write(cnp,'(f7.3)')ptltot
       else
       do i=1,nspecs
       if(ioobsv.eq.ispecs(i))id=i
       enddo
      write(cnp,'(f7.3)')ptlngc(id)
       endif
      write(cvol,'(f7.3)')volu
      write(clatt,'(i3)')nlattc
      write(cit,'(i5)')itermx     
      if(ioobsv.eq.0)then
      write(cobs,'(a)')'all'
      else
      write(cobs,'(i5)')ioobsv
      endif
      write(cnc,'(i5)')iternc
      if(iozevt.eq.0)write(czer,'(i5)')iozero
      if(iozevt.gt.0)write(cdz,'(i5)')iozinc
                
      x1=1
      x2=nevent

      if(jjj.eq.1)then

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      if(iozevt.gt.0)then
      write(ifhi,'(a)')       'text 0 0 "xaxis iozero"'
      else
      write(ifhi,'(a)')       'text 0 0 "xaxis event"'
      endif
      write(ifhi,'(a)')       'text 0 0 "yaxis [t]?exp!"'
      write(ifhi,'(a)')       'text 0.05 0.95 "E='//cen//'"'
      write(ifhi,'(a)')       'text 0.2  0.95 "V='//cvol//'"'
      write(ifhi,'(a)')       'text 0.35 0.95 "N?g!='//cnp//'"'
      write(ifhi,'(a)')       'text 0.55 0.95 "observable  '//cobs//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 2'
      do j=1,nevent
      write(ifhi,'(2e12.4)')data(j),datb(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0-'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 2'
      do j=1,nevent
      write(ifhi,'(2e12.4)')data(j),datd(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

      elseif(jjj.eq.2)then

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      if(iozevt.gt.0)then
      write(ifhi,'(a)')       'text 0 0 "xaxis iozero"'
      else
      write(ifhi,'(a)')       'text 0 0 "xaxis event"'
      endif
      write(ifhi,'(a)')       'text 0 0 "yaxis acceptence rate"'
      write(ifhi,'(a)')       'text 0.05 0.95 "iomom= '//cmom//'"'
      write(ifhi,'(a)')       'text 0.2  0.95 "nlattc= '//clatt//'"'
      if(iozevt.eq.0)
     *write(ifhi,'(a)')       'text 0.35 0.95 "iozero= '//czer//'"'
      write(ifhi,'(a)')       'text 0.55 0.95 "itermx= '//cit//'"'
      write(ifhi,'(a)')       'text 0.75 0.95 "iternc= '//cnc//'"'
      if(iozevt.gt.0)
     *write(ifhi,'(a)')       'text 0.35  0.95 "dzero= '//cdz//'"'
      if(iorejz.eq.1)
     *write(ifhi,'(a)')    'text 0.25 0.05 "zeros rejected !"'
      if(ioinco.eq.1)then
      write(ifhi,'(a)')    'text 0.05 0.05 "hot start"'
      else
      write(ifhi,'(a)')    'text 0.05 0.05 "cold start"'
      endif
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 2'
      do j=1,nevent
      write(ifhi,'(2e12.4)')data(j),datc(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

      endif

c          -----------------------------------  
           elseif(iii.lt.0.and.iappl.eq.1)then
c          -----------------------------------
      
      if(ioobsv.eq.0)then
      write(cobs,'(a)')'all'
      else
      write(cobs,'(i5)')ioobsv
      endif

      x1=1
      x2=nrclu

      if(jjj.eq.1)then

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis droplet"'
      write(ifhi,'(a)')       'text 0 0 "yaxis [D][t]?exp!"'
      write(ifhi,'(a)')       'text 0.05 0.91 "[D][t]?exp!=[t]?measured!
     *-[t]?parametrized"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a,a,a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,nrclu
      write(ifhi,'(2e12.4)')data(j),datb(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

      elseif(jjj.eq.2)then

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis droplet"'
      write(ifhi,'(a)')       'text 0 0 "yaxis N?obs!"'
      write(ifhi,'(a)')       'text 0.05 0.95 "observable  '//cobs//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 2'
      do j=1,nrclu
      write(ifhi,'(2e12.4)')data(j),datd(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

      elseif(jjj.eq.3)then

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis droplet"'
      write(ifhi,'(a)')       'text 0 0 "yaxis accep. rate"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 2'
      do j=1,nrclu
      write(ifhi,'(2e12.4)')data(j),datc(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

      endif

c          -----
           endif
c          -----

      ish=ish0
      return
      end

c-------------------------------------------------------------------------
      subroutine xtauin(iii)
c-------------------------------------------------------------------------
c fills histograms (iii=0) or writes histogram to histo-file (iii<0)
c regarding integrated autocorrelation time and corresponding multiplicity
c and variance
c
c input:
c   requires complete run with application metropolis (iappl=4)
c   iociau=1 necessary
c   iompar (parameter for windowing algorithm by  a.d.sokal) must
c   be set to 3 < c_M < 11
c
c  output:
c   for iii=0 (only valid for iappl=4):
c     data(nrevt): nrevt (event number)              /cdat/
c     datb(nrevt): tau   (calculated int corr time)  /cdat/
c     datc(nrevt): stau  (variance tau)              /cdat/        
c     datd(nrevt): avnp  (multiplicity)              /cdat/
c     date(nrevt): sobs  (variance multiplicity)     /cdat/
c     datf(nrevt):       (gc multiplicity)           /cdat/
c   for iii=0 and iosngl>0:
c     writes complete set of histograms (newpage zone 1 3 1
c     openhisto ... closehisto plot0 ... openhisto ... closehisto plot 0)
c     concerning acceptance rate, rejection rate, correlation function 
c     for specific event, specified by value of iosngl (=nrevt+1)
c   for iii<0:
c     writes complete histogram (openhisto ... closehisto) to histofile
c       xpar1=1: (data,datb,datc)   
c       xpar1=2: (data,datd,date,datf)   
c------------------------------------------------------------------------
      common/files/ifop,ifmt,ifch,ifhi,ifdt
      common/paroh/delvol,deleps,dlzeta,etafac
      common/paro1/ptmx,gaumx,sigppi,core,fctrmx,neqmn,iaqu,cutmsq
     *,taunll,maxres,ptf,ptq,xcut,ioptq,irescl,ko1ko2,kentro,labsys
     *,ntrymx,delmss,pud,pspinl,pspinh,pispn,ncolmx,tensn 
     *,cutmss,qvapc,qvatc,qsepc,qsetc,rstras,neqmx,taumx,nsttau,sigj
     *,pdiqua,parea,delrem,taumin,deltau,factau,numtau,iopenu,iopent
     *,themas,amsiac,wproj,wtarg,iopbrk,nclean
     *,ifrade,amprif,iojint,pth,pvalen,phard,ioptf,delrex
      common/paro2/nevent,modsho,engy,pnll,pnllx,yhaha,ish,iappl,prosea
     *,laproj,maproj,latarg,matarg,irandm,irewch
     *,istmax,ipagi,jpsi,jpsifi,elepti,elepto,angmue,ishsub
     *,idproj,idtarg,amproj,amtarg,ypjtl,rhophi
     *,ijphis,ientro,kutdiq,ishevt,idpm,taurea
      common/paro6/iospec,iocova,iopair,iozero,ioflac,iomom
      common/paro7/idensi,ioclud,nadd,icinpu,iograc,iocite,ioceau,iociau
      common/cnsta/pi,pii,hquer,prom,piom,ainfin
      parameter(maxit=50000)
      common/count/nacc,nrej,naccit(maxit),nptot,npit(maxit)
      common/citer/iter,itermx,iterma,iternc,iterpr,iterpl,iozinc,iozevt
      parameter(maxp=1000)
      common/config/np,tecm,amass(maxp),ident(maxp),pcm(5,maxp),volu
     *,wtxlog,wtlog
      common /clatt/nlattc,npmax
      common/cgctot/rmstot,ptltot
      common/paroc/iostat,ioinco,ionlat,ioobsv,iosngl,iorejz,iompar
      common/accum/imsg,jerr,ntevt,nrevt,naevt,nrstr,nrptl,amsac,ipage
     *,inoiac,ilamas
      parameter (mspecs=54)
      common/cspecs/nspecs,ispecs(mspecs),aspecs(mspecs),gspecs(mspecs)
      common/cgchg/rmsngc(mspecs),ptlngc(mspecs),chemgc(mspecs),tem
      parameter (nbin=500)
      common/cdat2/data(nbin),datb(nbin),datc(nbin),datd(nbin)
     *,date(nbin),datf(nbin),datg(nbin),dath(nbin)
      common/cdat3/datx(nbin),daty(nbin),datz(nbin),datr(nbin)
     *,dats(nbin)
      real corfct(maxit),dev(maxit)
      character cobs*5,cdz*5,ccuev*5,cmpar*3,ctau*7
      character cmom*5,cnp*7,cen*7,cvol*7,clatt*5,cit*5,cavnp*7
      character cnacc*10,cnrej*10,caver*10,cioz*5,ciom*3,cnlat*5
      common/cxpar/xpar1,xpar2,xpar3,xpar4,xpar5,xpar6,xpar7,xpar8

      if(iociau.ne.1)stop'STOP: iociau=1 required'
      if(iii.eq.0.and.iappl.ne.4)stop'STOP: iappl=4 required'
      if(iii.gt.0)stop'STOP: iii>0 not supported'

      jjj=nint(xpar1)

c          ----------------
           if(iii.eq.0)then
c          ----------------
      
c     mean
c     ----
      xnptot=nptot
      avnp=xnptot/(itermx-iternc)
      if(ish.ge.98)write(ifch,*)'event:',nrevt,'   avnp:',avnp

c     normalization of corfct_i
c     -------------------------
      corzer=0.0
      do i=iternc+1,itermx
      dev(i)=npit(i)-avnp
      if(ish.ge.99)write(ifch,*)'i:',i,'  dev_i:',dev(i)
      corzer=corzer+dev(i)**2
      enddo
      corzer=corzer/(itermx-iternc)
      if(ish.ge.98)write(ifch,*)'c_0:',corzer

c     calculate corfct_i
c     ------------------
      nt=itermx-iternc-1
      do it=1,nt
      corfct(it)=0.0
      do i=iternc+1,itermx-it
      corfct(it)=corfct(it)+dev(i)*dev(i+it)
      enddo
      corfct(it)=corfct(it)/(itermx-iternc-it)
      if(it.le.10.and.ish.ge.99)
     *write(ifch,*)'t:',it,'  c_t:',corfct(it)
      enddo

c     calculate initial autocorrelation time
c     -----------------------------------------
      if(corfct(1).gt.1.e-30.and.corzer.gt.1.e-30)then
      r=alog(corfct(1))-alog(corzer)
      if(ish.ge.98)write(ifch,*)'log rho_1:',r
      taui=(-1.)/r
      else
      taui=0.
      endif
      if(ish.ge.98)write(ifch,*)'tau_init:',taui

c     calculate integrated autocorrelation time
c     -----------------------------------------
      k=1
      mpar=iompar
      tau=taui
      taux=taui
      taum=0.0
      if(ish.ge.98)write(ifch,*)'initial tau:',tau,'   c_M:',mpar

        if(corzer.gt.1.e-30)then

5     mcut=mpar*abs(taux)
      tauo=tau
      tau=.5
      do it=1,mcut
      tau=tau+corfct(it)/corzer
      enddo
      taum=taum+tau
      taux=taum/k
      if(ish.ge.98)write(ifch,*)'iteration:',k,'   M:',mcut,'  tau:',tau
      if(mcut.lt.(mpar*tau).or.mcut.gt.(10.*tau))then
      dt=abs(tau-tauo)
      if(k.lt.20.and.dt.gt.0.2)then
      k=k+1
      goto5
      endif
      endif
      mcut=mpar*abs(taux)
      if(ish.ge.98)write(ifch,*)'tau_mean:',taux,'   M:',mcut
      tau=0.5
      do it=1,mcut
      tau=tau+corfct(it)/corzer
      enddo

       endif

      vtau=(2.*mcut+1.)*2./(itermx-iternc)*tau**2
      stau=0.0
      if(vtau.ge.0.0)stau=sqrt(vtau)
      if(ish.ge.98)
     *write(ifch,*)'tau_int:',tau,'   var:',vtau,'   sig:',stau

c     calculate variance of observable
c     --------------------------------
      vobs=2.*tau*corzer/(itermx-iternc)
      sobs=0.0
      if(vobs.ge.0.0)sobs=sqrt(vobs)

c     write to data-f
c     ---------------
       if(ioobsv.eq.0)then
      datf(nrevt)=ptltot
       else
      do j=1,np
      if(ioobsv.eq.ispecs(j))id=j
      enddo
      datf(nrevt)=ptlngc(id)
       endif
      datb(nrevt)=tau
      datc(nrevt)=stau
      date(nrevt)=sobs
      datd(nrevt)=avnp
      if(iozevt.gt.0)then
      data(nrevt)=iozero
      else
      data(nrevt)=nrevt
      endif

c          -------------------------
           if(iosngl.eq.nrevt+1)then
c          -------------------------

      nb=itermx/iterpl
      if(nb.gt.nbin)nb=nbin

      datx(1)=iterpl/2
      daty(1)=naccit(1)
      datz(1)=1-naccit(1)
      if(iterpl.ge.2)then
      do j=1,iterpl-1
      daty(1)=daty(1)+naccit(1+j)
      datz(1)=datz(1)+1-naccit(1+j)
      enddo
      endif
      datr(1)=daty(1)/iterpl
      dats(1)=datz(1)/iterpl
      do i=2,nb
      datx(i)=datx(i-1)+iterpl
      daty(i)=daty(i-1)
      datz(i)=datz(i-1)
      do j=1,iterpl
      daty(i)=daty(i)+naccit((i-1)*iterpl+j)
      datz(i)=datz(i)+1-naccit((i-1)*iterpl+j)
      enddo
      datr(i)=daty(i)/i/iterpl
      dats(i)=datz(i)/i/iterpl
      enddo
      b=nacc
      c=itermx
      avrate=b/c
      write(cnacc,'(i6)')nacc
      write(cnrej,'(i6)')nrej
      write(caver,'(f5.3)')avrate
      write(cioz,'(i5)')iozero
      write(ciom,'(i3)')iomom
      write(cnlat,'(i5)')nlattc
      x1=datx(1)
      x2=datx(nb)

      write(ifhi,'(a)')       'newpage zone 1 3 1 openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis iterations"'
      write(ifhi,'(a)')       'text 0 0 "yaxis acceptence rate"'
      write(ifhi,'(a)')       'text 0.6 0.5 "accepted '//cnacc//'"'
      write(ifhi,'(a)')       'text 0.6 0.4 "rejected  '//cnrej//'"'
      write(ifhi,'(a)')       'text 0.6 0.3 "aver. rate  '//caver//'"'
      write(ifhi,'(a)')       'text 0.4 0.5 "nlattc='//cnlat//'"'
      write(ifhi,'(a)')       'text 0.4 0.4 "iozero='//cioz//'"'
      write(ifhi,'(a)')       'text 0.4 0.3 "iomom='//ciom//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 2'
      do j=1,nb
      write(ifhi,'(2e12.4)')datx(j),datr(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0-'

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 2'
      do j=1,nb
      write(ifhi,'(2e12.4)')datx(j),dats(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0'

      m=min(mcut,500)
      do i=1,m
      datg(i)=i
      dath(i)=1000.
      if(corzer.gt.1.e-30)dath(i)=corfct(i)/corzer
      enddo
      write(ccuev,'(i5)')nrevt+1
      x1=1.
      x2=m

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lin xmod lin ymod lin'
      write(ifhi,'(a)')       'text 0 0 "xaxis t"'
      write(ifhi,'(a)')       'text 0 0 "yaxis correl. func."'
      write(ifhi,'(a)')       'text 0.8 0.95 "event '//ccuev//'"'
      write(ifhi,'(a)')'text 0.05 0.95  "window parameter= '//cmpar//'"'
      write(ifhi,'(a)')       'text 0.35 0.95  "tau= '//ctau//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a,a,a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 2'
      do j=1,m
      write(ifhi,'(2e12.4)')datg(j),dath(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto plot 0'

c          -----
           endif
c          -----

c          --------------------
           elseif(iii.lt.0)then
c          --------------------

      write(cmom,'(i3)')iomom
       if(ioobsv.eq.0)then
      write(cnp,'(f7.3)')ptltot
       else
      write(cnp,'(f7.3)')ptlngc(id)
       endif
      write(cen,'(f7.3)')tecm
      write(cvol,'(f7.3)')volu
      write(clatt,'(i3)')nlattc
      write(cit,'(i5)')itermx
      write(cavnp,'(f7.3)')avnp
      if(iozevt.gt.0)
     *write(cdz,'(i5)')iozinc
      write(cmpar,'(i3)')mpar
      if(ioobsv.eq.0)then
      write(cobs,'(a)')'all'
      else
      write(cobs,'(i5)')ioobsv
      endif

      x1=data(1)
      x2=data(nevent)

      if(jjj.eq.1)then

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp pnt xmod lin ymod lin'
      if(iozevt.gt.0)then
      write(ifhi,'(a)')       'text 0 0 "xaxis iozero"'
      else
      write(ifhi,'(a)')       'text 0 0 "xaxis event"'
      endif
      write(ifhi,'(a)')       'text 0 0 "yaxis [t]?int!"'
      write(ifhi,'(a)')'text 0.05 0.95  "window parameter '//cmpar//'"'
      if(iozevt.gt.0)
     *write(ifhi,'(a)')       'text 0.8  0.95 "dzero= '//cdz//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a)')       'array 3'
      do j=1,nevent
      write(ifhi,'(3e12.4)')data(j),datb(j),datc(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

      elseif(jjj.eq.2)then

      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp pnt xmod lin ymod lin'
      if(iozevt.gt.0)then
      write(ifhi,'(a)')       'text 0 0 "xaxis iozero"'
      else
      write(ifhi,'(a)')       'text 0 0 "xaxis event"'
      endif
      write(ifhi,'(a)')       'text 0 0 "yaxis multiplicity"'
      write(ifhi,'(a)')       'text 0.05 0.95 "E='//cen//'"'
      write(ifhi,'(a)')       'text 0.2 0.95 "V='//cvol//'"'
      write(ifhi,'(a)')       'text 0.35 0.95 "N?g!='//cnp//'"'
      write(ifhi,'(a)')       'text 0.55 0.95 "observable  '//cobs//'"'
      write(ifhi,'(a,2e11.3)')'xrange',x1,x2
      write(ifhi,'(a,a,a)')'yrange',' auto',' auto'
      write(ifhi,'(a)')       'array 3'
      do j=1,nevent
      write(ifhi,'(3e12.4)')data(j),datd(j),date(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto   plot 0-'


      write(ifhi,'(a)')       'openhisto'
      write(ifhi,'(a)')       'htyp lda xmod lin ymod lin'
      write(ifhi,'(a)')       'array 2'
      do j=1,nevent
      write(ifhi,'(2e12.4)')data(j),datf(j)
      enddo
      write(ifhi,'(a)')       '  endarray'
      write(ifhi,'(a)')       'closehisto'

      endif

c          -----
           endif
c          -----     

      return 
      end
